@@ -456,8 +456,8 @@ void KateSpellCheckConfigTab::showWhatsThis(const QString& text)
 
 void KateSpellCheckConfigTab::apply()
 {
-// nothing changed, no need to apply stuff
   if (!hasChanged()) {
+    // nothing changed, no need to apply stuff
     return;
   }
   m_changed = false;
@@ -465,10 +465,8 @@ void KateSpellCheckConfigTab::apply()
   KateDocumentConfig::global()->configStart();
   m_sonnetConfigWidget->save();
   KateDocumentConfig::global()->configEnd();
-  const QList<KateDocument*>& kateDocumentList = KateGlobal::self()->kateDocuments();
-  for (QList<KateDocument*>::const_iterator it = kateDocumentList.begin();
-       it != kateDocumentList.end(); ++it) {
-    (*it)->refreshOnTheFlyCheck();
+  foreach (KateDocument *doc, KateGlobal::self()->kateDocuments()) {
+    doc->refreshOnTheFlyCheck();
   }
 }
 
@@ -746,7 +744,7 @@ KateViewDefaultsConfig::KateViewDefaultsConfig(QWidget *parent)
   // hide power user mode if activated anyway
   if (!KateGlobal::self()->simpleMode ())
     ui->chkDeveloperMode->hide ();
-    
+
   // What's This? help is in the ui-file
 
   reload();
@@ -866,6 +864,7 @@ KateSaveConfigTab::KateSaveConfigTab( QWidget *parent )
 
   connect( ui->cmbEncoding, SIGNAL(activated(int)), this, SLOT(slotChanged()));
   connect( ui->cmbEncodingDetection, SIGNAL(activated(int)), this, SLOT(slotChanged()));
+  connect( ui->cmbEncodingFallback, SIGNAL(activated(int)), this, SLOT(slotChanged()));
   connect( ui->cmbEOL, SIGNAL(activated(int)), this, SLOT(slotChanged()));
   connect( ui->chkDetectEOL, SIGNAL( toggled(bool) ), this, SLOT( slotChanged() ) );
   connect( ui->chkEnableBOM, SIGNAL( toggled(bool) ), this, SLOT( slotChanged() ) );
@@ -906,6 +905,7 @@ void KateSaveConfigTab::apply()
     return;
   m_changed = false;
 
+  KateGlobalConfig::global()->configStart ();
   KateDocumentConfig::global()->configStart ();
 
   if ( uiadv->edtBackupSuffix->text().isEmpty() && uiadv->edtBackupPrefix->text().isEmpty() ) {
@@ -936,25 +936,29 @@ void KateSaveConfigTab::apply()
 
   KateDocumentConfig::global()->setConfigFlags(configFlags);
 
+  // set both standard and fallback encoding
   KateDocumentConfig::global()->setEncoding((ui->cmbEncoding->currentIndex() == 0) ? "" : KGlobal::charsets()->encodingForName(ui->cmbEncoding->currentText()));
-  KateDocumentConfig::global()->setEncodingProberType(
-      (KEncodingProber::ProberType)ui->cmbEncodingDetection->itemData(ui->cmbEncodingDetection->currentIndex()).toUInt());
+
+  KateGlobalConfig::global()->setProberType((KEncodingProber::ProberType)ui->cmbEncodingDetection->currentIndex());
+  KateGlobalConfig::global()->setFallbackEncoding(KGlobal::charsets()->encodingForName(ui->cmbEncodingFallback->currentText()));
 
   KateDocumentConfig::global()->setEol(ui->cmbEOL->currentIndex());
   KateDocumentConfig::global()->setAllowEolDetection(ui->chkDetectEOL->isChecked());
   KateDocumentConfig::global()->setBom(ui->chkEnableBOM->isChecked());
 
   KateDocumentConfig::global()->configEnd ();
+  KateGlobalConfig::global()->configEnd ();
 }
 
 void KateSaveConfigTab::reload()
 {
   modeConfigPage->reload();
 
-  // encoding
+  // encodings
   ui->cmbEncoding->clear ();
   ui->cmbEncoding->addItem (i18n("KDE Default"));
   ui->cmbEncoding->setCurrentIndex(0);
+  ui->cmbEncodingFallback->clear ();
   QStringList encodings (KGlobal::charsets()->descriptiveEncodingNames());
   int insert = 1;
   for (int i=0; i < encodings.count(); i++)
@@ -965,40 +969,42 @@ void KateSaveConfigTab::reload()
     if (found)
     {
       ui->cmbEncoding->addItem (encodings[i]);
+      ui->cmbEncodingFallback->addItem (encodings[i]);
 
       if ( codecForEnc->name() == KateDocumentConfig::global()->encoding() )
       {
         ui->cmbEncoding->setCurrentIndex(insert);
       }
 
+      if ( codecForEnc == KateGlobalConfig::global()->fallbackCodec() )
+      {
+        // adjust index for fallback config, has no default!
+        ui->cmbEncodingFallback->setCurrentIndex(insert-1);
+      }
+
       insert++;
     }
   }
 
   // encoding detection
   ui->cmbEncodingDetection->clear ();
-
-  ui->cmbEncodingDetection->addItem (i18n("Disabled"), QVariant((uint)KEncodingProber::None));
-  ui->cmbEncodingDetection->setCurrentIndex(0);
-
-  ui->cmbEncodingDetection->addItem (i18n("Universal"), QVariant((uint)KEncodingProber::Universal));
-
-  QStringList items;
-  foreach (const QStringList &encodingsForScript, KGlobal::charsets()->encodingsByScript())
-    items << encodingsForScript.at(0);
-  items.sort();
-  foreach (const QString &item, items) {
-    KEncodingProber::ProberType scri=KEncodingProber::proberTypeForName(item);
-    ui->cmbEncodingDetection->addItem (item, QVariant((uint)scri));
-    if (scri==KateDocumentConfig::global()->encodingProberType())
+  bool found = false;
+  for (int i = 0; !KEncodingProber::nameForProberType ((KEncodingProber::ProberType) i).isEmpty(); ++i) {
+    ui->cmbEncodingDetection->addItem (KEncodingProber::nameForProberType ((KEncodingProber::ProberType) i));
+    if (i == KateGlobalConfig::global()->proberType()) {
       ui->cmbEncodingDetection->setCurrentIndex(ui->cmbEncodingDetection->count()-1);
+      found = true;
+    }
   }
+  if (!found)
+      ui->cmbEncodingDetection->setCurrentIndex(KEncodingProber::Universal);
+
 
   // eol
   ui->cmbEOL->setCurrentIndex(KateDocumentConfig::global()->eol());
   ui->chkDetectEOL->setChecked(KateDocumentConfig::global()->allowEolDetection());
   ui->chkEnableBOM->setChecked(KateDocumentConfig::global()->bom());
-  
+
   const uint configFlags = KateDocumentConfig::global()->configFlags();
   ui->chkRemoveTrailingSpaces->setChecked(configFlags & KateDocumentConfig::cfRemoveSpaces);
   uiadv->sbConfigFileSearchDepth->setValue(KateDocumentConfig::global()->searchDirConfigDepth());
@@ -1071,7 +1077,7 @@ KatePartPluginConfigPage::KatePartPluginConfigPage (QWidget *parent)
 
   // add all tabs
   tabWidget->insertTab(0, tmpWidget, i18n("Plugins"));
-//  tabWidget->insertTab(1, scriptConfigPage, i18n("Scripts"));
+  tabWidget->insertTab(1, scriptConfigPage, i18n("Scripts"));
 
   generalLayout->addWidget(tabWidget);
   setLayout(generalLayout);
@@ -1169,7 +1175,6 @@ KateHlDownloadDialog::KateHlDownloadDialog(QWidget *parent, const char *name, bo
   setDefaultButton( User1 );
   setObjectName( name );
   setModal( modal );
-  showButtonSeparator( true );
 
   KVBox* vbox = new KVBox(this);
   setMainWidget(vbox);
@@ -1287,9 +1292,12 @@ void KateHlDownloadDialog::slotUser1()
 //END KateHlDownloadDialog
 
 //BEGIN KateGotoBar
-KateGotoBar::KateGotoBar(KateView* view, QWidget *parent)
-  : KateViewBarWidget( true, view, parent )
+KateGotoBar::KateGotoBar(KTextEditor::View *view, QWidget *parent)
+  : KateViewBarWidget( true, parent )
+  , m_view( view )
 {
+  Q_ASSERT( m_view != 0 );  // this bar widget is pointless w/o a view
+
   QHBoxLayout *topLayout = new QHBoxLayout( centralWidget() );
   topLayout->setMargin(0);
   gotoRange = new KIntSpinBox(centralWidget());
@@ -1297,7 +1305,7 @@ KateGotoBar::KateGotoBar(KateView* view, QWidget *parent)
   QLabel *label = new QLabel(i18n("&Go to line:"), centralWidget() );
   label->setBuddy(gotoRange);
 
-  btnOK = new QToolButton();
+  QToolButton *btnOK = new QToolButton(centralWidget());
   btnOK->setAutoRaise(true);
   btnOK->setIcon(QIcon(SmallIcon("go-jump")));
   btnOK->setText(i18n("Go"));
@@ -1313,13 +1321,10 @@ KateGotoBar::KateGotoBar(KateView* view, QWidget *parent)
 
 void KateGotoBar::updateData()
 {
-  if (!view())
-    return;
-
-  gotoRange->setMaximum(view()->doc()->lines());
+  gotoRange->setMaximum(m_view->document()->lines());
   if (!isVisible())
   {
-    gotoRange->setValue(view()->cursorPosition().line() + 1);
+    gotoRange->setValue(m_view->cursorPosition().line() + 1);
     gotoRange->adjustSize(); // ### does not respect the range :-(
   }
   gotoRange->setFocus(Qt::OtherFocusReason);
@@ -1338,16 +1343,18 @@ void KateGotoBar::keyPressEvent(QKeyEvent* event)
 
 void KateGotoBar::gotoLine()
 {
-  view()->setCursorPosition( KTextEditor::Cursor(gotoRange->value() - 1, 0) );
-  view()->setFocus();
+  m_view->setCursorPosition( KTextEditor::Cursor(gotoRange->value() - 1, 0) );
+  m_view->setFocus();
   emit hideMe();
 }
 //END KateGotoBar
 
 //BEGIN KateDictionaryBar
 KateDictionaryBar::KateDictionaryBar(KateView* view, QWidget *parent)
-  : KateViewBarWidget( true, view, parent )
+  : KateViewBarWidget( true, parent )
+  , m_view( view )
 {
+  Q_ASSERT(m_view != 0); // this bar widget is pointless w/o a view
 
   QHBoxLayout *topLayout = new QHBoxLayout(centralWidget());
   topLayout->setMargin(0);
@@ -1372,11 +1379,7 @@ KateDictionaryBar::~KateDictionaryBar()
 
 void KateDictionaryBar::updateData()
 {
-  if (!view()) {
-    return;
-  }
-
-  KateDocument *document = view()->doc();
+  KateDocument *document = m_view->doc();
   QString dictionary = document->defaultDictionary();
   if(dictionary.isEmpty()) {
     dictionary = Sonnet::Speller().defaultLanguage();
@@ -1386,17 +1389,12 @@ void KateDictionaryBar::updateData()
 
 void KateDictionaryBar::dictionaryChanged(const QString& dictionary)
 {
-  KateView *kateView = view();
-  if (!kateView) {
-    return;
-  }
-
-  KTextEditor::Range selection = kateView->selectionRange();
+  KTextEditor::Range selection = m_view->selectionRange();
   if(selection.isValid() && !selection.isEmpty()) {
-    kateView->doc()->setDictionary(dictionary, selection);
+    m_view->doc()->setDictionary(dictionary, selection);
   }
   else {
-    kateView->doc()->setDefaultDictionary(dictionary);
+    m_view->doc()->setDefaultDictionary(dictionary);
   }
 }
 
@@ -1437,7 +1435,6 @@ KateModOnHdPrompt::KateModOnHdPrompt( KateDocument *doc,
   setButtonWhatsThis( Cancel, i18n("Do nothing. Next time you focus the file, "
       "or try to save it or close it, you will be prompted again.") );
 
-  showButtonSeparator( true );
   setCaption( title );
 
   QWidget *w = new QWidget(this);
