@@ -115,17 +115,23 @@ bool KateEditInfo::isRemoval() const
 
 KateEditHistory::KateEditHistory( KateDocument * doc )
   : QObject(doc)
+  , m_mutex (QMutex::Recursive)
   , m_revision(0)
 {
 }
 
 KateEditHistory::~KateEditHistory()
 {
+  // just delete all edits, referenced or not
+  qDeleteAll (m_edits);
 }
 
 int KateEditHistory::revision()
 {
-  QMutexLocker lock(&m_mutex);
+  // protect the edit history
+  QMutexLocker locker (&m_mutex);
+
+  // just take the last edit
   if (!m_edits.isEmpty()) {
     KateEditInfo* edit = m_edits.last();
     if (!edit->isReferenced())
@@ -135,41 +141,75 @@ int KateEditHistory::revision()
     return m_revision;
   }
 
+  // no edits, no revision
   return 0;
 }
 
 void KateEditHistory::releaseRevision(int revision)
 {
-  QMutexLocker lock(&m_mutex);
-  if (m_revisions.contains(revision)) {
-    KateEditInfo* edit = m_revisions[revision];
-    edit->dereferenceRevision();
-    if (!edit->isReferenced())
-      m_revisions.remove(revision);
-    return;
-  }
+  // protect the edit history
+  QMutexLocker locker (&m_mutex);
 
-  kWarning() << "Unknown revision token " << revision;
+  // search for revision, if not found, back out
+  QMap<int, KateEditInfo*>::iterator it = m_revisions.find (revision);
+  if (it == m_revisions.end())
+    return;
+
+  // dereference the revision
+  KateEditInfo *edit = *it;
+  edit->dereferenceRevision();
+  if (!edit->isReferenced()) {
+    // no longer refed
+    m_revisions.remove(revision);
+
+    // clean up memory, try to delete any unrefed revisions
+
+    // delete unrefed
+    int deletedEdits = 0;
+    for (int i = 0; i < m_edits.size(); ++i) {
+      if (m_edits[i]->isReferenced())
+	break;
+
+      // already delete edit
+      delete m_edits[i];
+
+      // remember deleted count
+      ++deletedEdits;
+    }
+
+    // remove unrefed from the list now, already deleted
+    if (deletedEdits > 0)
+      m_edits.erase (m_edits.begin(), m_edits.begin() + deletedEdits);
+  }
 }
 
-void KateEditHistory::doEdit(KateEditInfo* edit) {
-  
-  m_mutex.lock();
-  m_edits.append(edit);
-  m_mutex.unlock();
-  
+void KateEditHistory::doEdit (KateEditInfo* edit) {
+  {
+    QMutexLocker locker (&m_mutex);
+
+    // here we go, don't be a memory pirate
+    // if we atm only have one edit and it is not referenced, just delete it
+    // in other cases, the releaseRevision will clean up
+    if (m_edits.size() == 1 && !m_edits.last()->isReferenced()) {
+	delete m_edits.takeLast ();
+    }
+
+    // append the new edit in any case
+    m_edits.append(edit);
+  }
+
   emit editDone(edit);
 }
 
 QList<KateEditInfo*> KateEditHistory::editsBetweenRevisions(int from, int to) const
 {
-  QMutexLocker lock(&m_mutex);
-
   QList<KateEditInfo*> ret;
 
   if (from == -1)
     return ret;
 
+  QMutexLocker locker (&m_mutex);
+
   if (m_edits.isEmpty())
     return ret;
 
