@@ -1,20 +1,23 @@
-/* This file is part of the KDE libraries
-   Copyright (C) 2003,2004,2005 Hamish Rodda <rodda@kde.org>
-
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License version 2 as published by the Free Software Foundation.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
-*/
+/*  This file is part of the KDE libraries and the Kate part.
+ *
+ *  Copyright (C) 2003,2004,2005 Hamish Rodda <rodda@kde.org>
+ *  Copyright (C) 2010 Christoph Cullmann <cullmann@kde.org>
+ *
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Library General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2 of the License, or (at your option) any later version.
+ *
+ *  This library is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Library General Public License for more details.
+ *
+ *  You should have received a copy of the GNU Library General Public License
+ *  along with this library; see the file COPYING.LIB.  If not, write to
+ *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ *  Boston, MA 02110-1301, USA.
+ */
 
 #include "katesmartrange.h"
 
@@ -23,36 +26,21 @@
 #include "kateview.h"
 #include <ktexteditor/attribute.h>
 #include "katesmartmanager.h"
-#include "katedynamicanimation.h"
-
-#include <kdebug.h>
 
 KateSmartRange::KateSmartRange(const KTextEditor::Range& range, KateDocument* doc, KTextEditor::SmartRange* parent, KTextEditor::SmartRange::InsertBehaviors insertBehavior)
   : KTextEditor::SmartRange(new KateSmartCursor(range.start(), doc), new KateSmartCursor(range.end(), doc), parent, insertBehavior)
-//  , m_feedbackLevel(NoFeedback)
-  , m_dynamicDoc(0L)
-  , m_mouseOver(false)
-  , m_caretOver(false)
   , m_isInternal(false)
 {
 }
 
 KateSmartRange::KateSmartRange(KateDocument* doc, KTextEditor::SmartRange* parent)
   : KTextEditor::SmartRange(new KateSmartCursor(doc), new KateSmartCursor(doc), parent)
-//  , m_feedbackLevel(NoFeedback)
-  , m_dynamicDoc(0L)
-  , m_mouseOver(false)
-  , m_caretOver(false)
   , m_isInternal(false)
 {
 }
 
 KateSmartRange::KateSmartRange( KateSmartCursor * start, KateSmartCursor * end, KTextEditor::SmartRange * parent, KTextEditor::SmartRange::InsertBehaviors insertBehavior )
   : KTextEditor::SmartRange(start, end, parent, insertBehavior)
-//  , m_feedbackLevel(NoFeedback)
-  , m_dynamicDoc(0L)
-  , m_mouseOver(false)
-  , m_caretOver(false)
   , m_isInternal(false)
 {
 }
@@ -69,9 +57,6 @@ KateSmartRange::~KateSmartRange()
 
   if (m_start)
     kateDocument()->smartManager()->rangeDeleted(this);
-
-  foreach (KateSmartRangePtr* ptr, m_pointers)
-    ptr->deleted();
 }
 
 KateDocument * KateSmartRange::kateDocument( ) const
@@ -230,40 +215,6 @@ void KateSmartRange::setInternal( )
   kEnd().setInternal();
 }
 
-/*KateDynamicAnimation * KateSmartRange::dynamicForView( const KateView * const view) const
-{
-  foreach (KateDynamicAnimation* anim, m_dynamic)
-    if (anim->view() == view)
-      return anim;
-
-  return 0L;
-}*/
-
-void KateSmartRange::addDynamic( KateDynamicAnimation * anim )
-{
-  m_dynamic.append(anim);
-}
-
-void KateSmartRange::removeDynamic( KateDynamicAnimation * anim )
-{
-  m_dynamic.removeAll(anim);
-}
-
-const QList<KateDynamicAnimation*> & KateSmartRange::dynamicAnimations( ) const
-{
-  return m_dynamic;
-}
-
-void KateSmartRange::registerPointer( KateSmartRangePtr * ptr )
-{
-  m_pointers.append(ptr);
-}
-
-void KateSmartRange::deregisterPointer( KateSmartRangePtr * ptr )
-{
-  m_pointers.removeAll(ptr);
-}
-
 void KateSmartRange::checkFeedback()
 {
   kStart().checkFeedback();
