@@ -1,5 +1,6 @@
 /* This file is part of the KDE libraries
    Copyright (C) 2007 Mirko Stocker <me@misto.ch>
+   Copyright (C) 2007 Matthew Woehlke <mw_triad@users.sourceforge.net>
    Copyright (C) 2003, 2004 Anders Lund <anders@alweb.dk>
    Copyright (C) 2003 Hamish Rodda <rodda@kde.org>
    Copyright (C) 2001,2002 Joseph Wenninger <jowenn@kde.org>
@@ -46,11 +47,14 @@
 #include <kmessagebox.h>
 #include <kapplication.h>
 
+#include <ktexteditor/highlightinterface.h>
+
 #include <QtCore/QSet>
 #include <QtGui/QAction>
 #include <QtGui/QApplication>
 #include <QtCore/QStringList>
 #include <QtCore/QTextStream>
+#include <QVarLengthArray>
 //END
 
 //BEGIN defines
@@ -82,6 +86,7 @@ KateHighlighting::KateHighlighting(const KateSyntaxModeListItem *def) : refCount
     m_additionalData["none"]->deliminator = stdDeliminator;
     m_additionalData["none"]->wordWrapDeliminator = stdDeliminator;
     m_hlIndex[0] = "none";
+    m_ctxIndex[0]="";
   }
   else
   {
@@ -112,6 +117,9 @@ void KateHighlighting::cleanup ()
   qDeleteAll (m_contexts);
   m_contexts.clear ();
 
+  qDeleteAll (m_hlItemCleanupList);
+  m_hlItemCleanupList.clear ();
+
   m_attributeArrays.clear ();
 
   internalIDList.clear();
@@ -204,7 +212,9 @@ int KateHighlighting::makeDynamicContext(KateHlContext *model, const QStringList
     value = dynamicCtxs[key];
   else
   {
+#ifdef HIGHLIGHTING_DEBUG
     kDebug(13010) << "new stuff: " << startctx;
+#endif
 
     KateHlContext *newctx = model->clone(args);
 
@@ -243,8 +253,8 @@ void KateHighlighting::dropDynamicContexts()
  * @param foldingList will be filled
  * @param ctxChanged will be set to reflect if the context changed
  */
-void KateHighlighting::doHighlight ( KateTextLine *prevLine,
-                                     KateTextLine *textLine,
+void KateHighlighting::doHighlight ( Kate::TextLineData *prevLine,
+                                     Kate::TextLineData *textLine,
                                      QVector<int> &foldingList,
                                      bool &ctxChanged )
 {
@@ -256,7 +266,7 @@ void KateHighlighting::doHighlight ( KateTextLine *prevLine,
 
   // no hl set, nothing to do more than the above cleaning ;)
   if (noHl) {
-    textLine->addAttribute (0, textLine->length(), KateExtendedAttribute::dsNormal);
+    textLine->addAttribute (0, textLine->length(), KTextEditor::HighlightInterface::dsNormal);
     return;
   }
 
@@ -311,15 +321,21 @@ void KateHighlighting::doHighlight ( KateTextLine *prevLine,
 
   // loop over the line, offset gives current offset
   int offset = 0;
+
+  KateHighlighting::HighlightPropertyBag* additionalData = m_additionalData[context->hlId];
+  KateHlContext* oldContext = context;
+
+  // optimization: list of highlighting items that need their cache reset
+  static QVarLengthArray<KateHlItem*> cachingItems;
+
+
+  QChar lastDelimChar = 0;
   while (offset < len)
   {
     bool anItemMatched = false;
-    bool standardStartEnableDetermined = false;
     bool customStartEnableDetermined = false;
 
-    int index = 0;
-    
-    for (item = context->items.value(0, 0); item; item = context->items.value(++index, 0))
+    foreach (item, context->items)
     {
       // does we only match if we are firstNonSpace?
       if (item->firstNonSpace && (offset > startNonSpace))
@@ -333,21 +349,31 @@ void KateHighlighting::doHighlight ( KateTextLine *prevLine,
       {
         if (item->customStartEnable)
         {
-            if (customStartEnableDetermined || m_additionalData[context->hlId]->deliminator.contains(lastChar))
-            customStartEnableDetermined = true;
-          else
-            continue;
+            if ( oldContext != context ) {
+              oldContext = context;
+              additionalData = m_additionalData[oldContext->hlId];
+            }
+            if (customStartEnableDetermined || additionalData->deliminator.contains(lastChar))
+              customStartEnableDetermined = true;
+            else
+              continue;
         }
         else
         {
-          if (standardStartEnableDetermined || stdDeliminator.contains(lastChar))
-            standardStartEnableDetermined = true;
-          else
+          if (lastDelimChar == lastChar) {
+          } else if ( stdDeliminator.contains(lastChar) ) {
+            lastDelimChar = lastChar;
+          } else {
             continue;
+          }
         }
       }
 
       int offset2 = item->checkHgl(text, offset, len-offset);
+      if ( item->haveCache && !item->cachingHandled ) {
+        cachingItems.append(item);
+        item->cachingHandled = true;
+      }
 
       if (offset2 <= offset)
         continue;
@@ -401,17 +427,18 @@ void KateHighlighting::doHighlight ( KateTextLine *prevLine,
       // dynamic context: substitute the model with an 'instance'
       if (context->dynamic)
       {
-        QStringList *lst = item->capturedTexts();
-        if (lst != 0)
+        // try to retrieve captures from regexp
+        QStringList captures;
+        item->capturedTexts (captures);
+        if (!captures.empty())
         {
           // Replace the top of the stack and the current context
-          int newctx = makeDynamicContext(context, lst);
+          int newctx = makeDynamicContext(context, &captures);
           if (ctx.size() > 0)
             ctx[ctx.size() - 1] = newctx;
 
           context = contextNum(newctx);
         }
-        delete lst;
       }
 
       // dominik: look ahead w/o changing offset?
@@ -437,6 +464,8 @@ void KateHighlighting::doHighlight ( KateTextLine *prevLine,
     if (anItemMatched)
       continue;
 
+    item = 0;
+
     // nothing found: set attribute of one char
     // anders: unless this context does not want that!
     if ( context->fallthrough )
@@ -494,8 +523,15 @@ void KateHighlighting::doHighlight ( KateTextLine *prevLine,
 
   //set the dsNormal attribute if we haven't found anything else
   if(textLine->attributesList().empty()) {
-    textLine->addAttribute (0, textLine->length(), KateExtendedAttribute::dsNormal);
+    textLine->addAttribute (0, textLine->length(), KTextEditor::HighlightInterface::dsNormal);
   }
+
+  // invalidate caches
+  for ( int i = 0; i < cachingItems.size(); ++i) {
+    cachingItems[i]->cachingHandled = false;
+    cachingItems[i]->haveCache = false;
+  }
+  cachingItems.clear();
 }
 
 void KateHighlighting::getKateExtendedAttributeList (const QString &schema, QList<KateExtendedAttribute::Ptr> &list)
@@ -516,7 +552,7 @@ void KateHighlighting::getKateExtendedAttributeList (const QString &schema, QLis
     if (s.count()>0)
     {
 
-      while(s.count()<9) s<<"";
+      while(s.count()<10) s<<"";
       p->clear();
 
       QString tmp=s[0]; if (!tmp.isEmpty()) p->setDefaultStyleIndex(tmp.toInt());
@@ -543,6 +579,8 @@ void KateHighlighting::getKateExtendedAttributeList (const QString &schema, QLis
       tmp=s[8]; if (!tmp.isEmpty()) {
          col=tmp.toUInt(0,16); p->setSelectedBackground(QColor(col)); }
 
+      tmp=s[9]; if (!tmp.isEmpty() && tmp!=QLatin1String("---")) p->setFontFamily(tmp);
+
     }
   }
 }
@@ -587,11 +625,39 @@ void KateHighlighting::setKateExtendedAttributeList(uint schema, QList<KateExten
     settings<<(p->hasProperty(QTextFormat::FontUnderline)?(p->fontUnderline()?"1":"0"):"");
     settings<<(p->hasProperty(QTextFormat::BackgroundBrush)?QString::number(p->background().color().rgb(),16):"");
     settings<<(p->hasProperty(KTextEditor::Attribute::SelectedBackground)?QString::number(p->selectedBackground().color().rgb(),16):"");
+    settings<<(p->hasProperty(QTextFormat::FontFamily)?(p->fontFamily()):QString());
     settings<<"---";
     config.writeEntry(p->name(),settings);
   }
 }
 
+const QHash<QString, QChar>& KateHighlighting::getCharacterEncodings( int attrib ) const
+{
+  return m_additionalData[ hlKeyForAttrib( attrib ) ]->characterEncodings;
+}
+
+const KatePrefixStore& KateHighlighting::getCharacterEncodingsPrefixStore( int attrib ) const
+{
+  return m_additionalData[ hlKeyForAttrib( attrib ) ]->characterEncodingsPrefixStore;
+}
+
+const QHash<QChar, QString>& KateHighlighting::getReverseCharacterEncodings( int attrib ) const
+{
+  return m_additionalData[ hlKeyForAttrib( attrib ) ]->reverseCharacterEncodings;
+}
+
+int KateHighlighting::getEncodedCharactersInsertionPolicy( int attrib ) const
+{
+  return m_additionalData[ hlKeyForAttrib( attrib ) ]->encodedCharactersInsertionPolicy;
+}
+
+void KateHighlighting::addCharacterEncoding( const QString& key, const QString& encoding, const QChar& c )
+{
+  m_additionalData[ key ]->characterEncodingsPrefixStore.addPrefix(encoding);
+  m_additionalData[ key ]->characterEncodings[ encoding ] = c;
+  m_additionalData[ key ]->reverseCharacterEncodings[ c ] = encoding;
+}
+
 /**
  * Increase the usage count, and trigger initialization if needed.
  */
@@ -656,7 +722,7 @@ void KateHighlighting::createKateExtendedAttribute(QList<KateExtendedAttribute::
   // If no highlighting is selected we need only one default.
   if (noHl)
   {
-    list.append(KateExtendedAttribute::Ptr(new KateExtendedAttribute(i18n("Normal Text"), KateExtendedAttribute::dsNormal)));
+    list.append(KateExtendedAttribute::Ptr(new KateExtendedAttribute(i18n("Normal Text"), KTextEditor::HighlightInterface::dsNormal)));
     return;
   }
 
@@ -688,6 +754,8 @@ void KateHighlighting::addToKateExtendedAttributeList()
     QString strikeOut = KateHlManager::self()->syntax->groupData(data,QString("strikeOut"));
     QString bgColor = KateHlManager::self()->syntax->groupData(data,QString("backgroundColor"));
     QString selBgColor = KateHlManager::self()->syntax->groupData(data,QString("selBackgroundColor"));
+    QString spellChecking = KateHlManager::self()->syntax->groupData(data,QString("spellChecking"));
+    QString fontFamily = KateHlManager::self()->syntax->groupData(data,QString("fontFamily"));
 
     KateExtendedAttribute::Ptr newData(new KateExtendedAttribute(
             buildPrefix+KateHlManager::self()->syntax->groupData(data,QString("name")).simplified(),
@@ -703,6 +771,9 @@ void KateHighlighting::addToKateExtendedAttributeList()
     if (!strikeOut.isEmpty()) newData->setFontStrikeOut( IS_TRUE(strikeOut) );
     if (!bgColor.isEmpty()) newData->setBackground(QColor(bgColor));
     if (!selBgColor.isEmpty()) newData->setSelectedBackground(QColor(selBgColor));
+    // is spellchecking desired?
+    if (!spellChecking.isEmpty()) newData->setPerformSpellchecking( IS_TRUE(spellChecking) );
+    if (!fontFamily.isEmpty()) newData->setFontFamily(fontFamily);
 
     internalIDList.append(newData);
   }
@@ -728,7 +799,10 @@ int  KateHighlighting::lookupAttrName(const QString& name, QList<KateExtendedAtt
     if (iDl.at(i)->name() == buildPrefix+name)
       return i;
 
+#ifdef HIGHLIGHTING_DEBUG
   kDebug(13010)<<"Couldn't resolve itemDataName:"<<name;
+#endif
+
   return 0;
 }
 
@@ -776,7 +850,9 @@ KateHlItem *KateHighlighting::createKateHlItem(KateSyntaxContextData *data,
 
     regionId++;
 
+#ifdef HIGHLIGHTING_DEBUG
     kDebug(13010) << "########### BEG REG: "  << beginRegionStr << " NUM: " << regionId;
+#endif
   }
 
   if (!endRegionStr.isEmpty())
@@ -791,7 +867,9 @@ KateHlItem *KateHighlighting::createKateHlItem(KateSyntaxContextData *data,
 
     regionId2 = -regionId2 - 1;
 
+#ifdef HIGHLIGHTING_DEBUG
     kDebug(13010) << "########### END REG: "  << endRegionStr << " NUM: " << regionId2;
+#endif
   }
 
   int attr = 0;
@@ -820,23 +898,21 @@ KateHlItem *KateHighlighting::createKateHlItem(KateSyntaxContextData *data,
     context=getContextModificationFromString(ContextNameList, tmpcontext,unresolvedContext);
 
   // Get the char parameter (eg DetectChar)
-  char chr;
-  if (! KateHlManager::self()->syntax->groupItemData(data,QString("char")).isEmpty())
-    chr= qPrintable((KateHlManager::self()->syntax->groupItemData(data,QString("char"))))[0];
-  else
-    chr=0;
+  QChar chr;
+  if (! KateHlManager::self()->syntax->groupItemData(data,QString("char")).isEmpty()) {
+    chr= (KateHlManager::self()->syntax->groupItemData(data,QString("char")))[0];
+  }
 
   // Get the String parameter (eg. StringDetect)
   QString stringdata=KateHlManager::self()->syntax->groupItemData(data,QString("String"));
 
   // Get a second char parameter (char1) (eg Detect2Chars)
-  char chr1;
-  if (! KateHlManager::self()->syntax->groupItemData(data,QString("char1")).isEmpty())
-    chr1= (KateHlManager::self()->syntax->groupItemData(data,QString("char1")).toLatin1())[0];
-  else
-    chr1=0;
+  QChar chr1;
+  if (! KateHlManager::self()->syntax->groupItemData(data,QString("char1")).isEmpty()) {
+    chr1= (KateHlManager::self()->syntax->groupItemData(data,QString("char1")))[0];
+  }
 
-  // Will be removed eventually. Atm used for StringDetect, keyword and RegExp
+  // Will be removed eventually. Atm used for StringDetect, WordDetect, keyword and RegExp
   const QString & insensitive_str = KateHlManager::self()->syntax->groupItemData(data,QString("insensitive"));
   bool insensitive = IS_TRUE( insensitive_str );
 
@@ -875,6 +951,7 @@ KateHlItem *KateHighlighting::createKateHlItem(KateSyntaxContextData *data,
   else if (dataname=="RangeDetect") tmpItem=(new KateHlRangeDetect(attr,context,regionId,regionId2, chr, chr1));
   else if (dataname=="LineContinue") tmpItem=(new KateHlLineContinue(attr,context,regionId,regionId2));
   else if (dataname=="StringDetect") tmpItem=(new KateHlStringDetect(attr,context,regionId,regionId2,stringdata,insensitive));
+  else if (dataname=="WordDetect") tmpItem=(new KateHlWordDetect(attr,context,regionId,regionId2,stringdata,insensitive));
   else if (dataname=="AnyChar") tmpItem=(new KateHlAnyChar(attr,context,regionId,regionId2,stringdata));
   else if (dataname=="RegExpr") tmpItem=(new KateHlRegExpr(attr,context,regionId,regionId2,stringdata, insensitive, minimal));
   else if (dataname=="HlCChar") tmpItem= ( new KateHlCChar(attr,context,regionId,regionId2));
@@ -902,9 +979,40 @@ KateHlItem *KateHighlighting::createKateHlItem(KateSyntaxContextData *data,
     unresolvedContextReferences.insert(&(tmpItem->ctx),unresolvedContext);
   }
 
+  // remember all to delete them
+  m_hlItemCleanupList.append (tmpItem);
+
   return tmpItem;
 }
 
+int KateHighlighting::attribute(int ctx) const
+{
+  return m_contexts[ctx]->attr;
+}
+
+bool KateHighlighting::attributeRequiresSpellchecking( int attr )
+{
+  QList<KTextEditor::Attribute::Ptr> attributeList = attributes("");
+  if(attr < attributeList.length() && attributeList[attr]->hasProperty(KateExtendedAttribute::Spellchecking)) {
+    return attributeList[attr]->boolProperty(KateExtendedAttribute::Spellchecking);
+  }
+  return true;
+}
+
+QString KateHighlighting::hlKeyForContext(int i) const
+{
+  int k = 0;
+  QMap<int,QString>::const_iterator it = m_ctxIndex.constEnd();
+  while ( it != m_hlIndex.constBegin() )
+  {
+    --it;
+    k = it.key();
+    if ( i >= k )
+      break;
+  }
+  return it.value();
+}
+
 QString KateHighlighting::hlKeyForAttrib( int i ) const
 {
   // find entry. This is faster than QMap::find. m_hlIndex always has an entry
@@ -936,7 +1044,10 @@ bool KateHighlighting::canBreakAt( QChar c, int attrib ) const
 
 QLinkedList<QRegExp> KateHighlighting::emptyLines(int attrib) const
 {
+#ifdef HIGHLIGHTING_DEBUG
   kDebug(13010)<<"hlKeyForAttrib: "<<hlKeyForAttrib(attrib);
+#endif
+
   return m_additionalData[hlKeyForAttrib(attrib)]->emptyLines;
 }
 
@@ -973,6 +1084,10 @@ KateHighlighting::CSLPos KateHighlighting::getCommentSingleLinePosition( int att
   return m_additionalData[ hlKeyForAttrib( attrib) ]->singleLineCommentPosition;
 }
 
+const QHash<QString, QChar>& KateHighlighting::characterEncodings( int attrib ) const
+{
+  return m_additionalData[ hlKeyForAttrib( attrib) ]->characterEncodings;
+}
 
 /**
  * Helper for makeContextList. It parses the xml file for
@@ -1031,7 +1146,10 @@ void KateHighlighting::readEmptyLineConfig()
   {
     while  (KateHlManager::self()->syntax->nextGroup(data))
     {
+#ifdef HIGHLIGHTING_DEBUG
       kDebug(13010)<<"creating an empty line regular expression";
+#endif
+
       QString regexprline=KateHlManager::self()->syntax->groupData(data,"regexpr");
       bool regexprcase=(KateHlManager::self()->syntax->groupData(data,"casesensitive").toUpper().compare("TRUE")==0);
       exprList.append(QRegExp(regexprline,regexprcase?Qt::CaseSensitive:Qt::CaseInsensitive));
@@ -1055,15 +1173,20 @@ void KateHighlighting::readEmptyLineConfig()
 void KateHighlighting::readGlobalKeywordConfig()
 {
   deliminator = stdDeliminator;
-  // Tell the syntax document class which file we want to parse
-  kDebug(13010)<<"readGlobalKeywordConfig:BEGIN";
 
+#ifdef HIGHLIGHTING_DEBUG
+  kDebug(13010)<<"readGlobalKeywordConfig:BEGIN";
+#endif
+
+  // Tell the syntax document class which file we want to parse
   KateHlManager::self()->syntax->setIdentifier(buildIdentifier);
   KateSyntaxContextData *data = KateHlManager::self()->syntax->getConfig("general","keywords");
 
   if (data)
   {
+#ifdef HIGHLIGHTING_DEBUG
     kDebug(13010)<<"Found global keyword config";
+#endif
 
     if ( IS_TRUE( KateHlManager::self()->syntax->groupItemData(data,QString("casesensitive")) ) )
       casesensitive=true;
@@ -1073,7 +1196,9 @@ void KateHighlighting::readGlobalKeywordConfig()
     //get the weak deliminators
     weakDeliminator=(KateHlManager::self()->syntax->groupItemData(data,QString("weakDeliminator")));
 
+#ifdef HIGHLIGHTING_DEBUG
     kDebug(13010)<<"weak delimiters are: "<<weakDeliminator;
+#endif
 
     // remove any weakDelimitars (if any) from the default list and store this list.
     for (int s=0; s < weakDeliminator.length(); s++)
@@ -1098,9 +1223,10 @@ void KateHighlighting::readGlobalKeywordConfig()
     weakDeliminator=QString("");
   }
 
+#ifdef HIGHLIGHTING_DEBUG
   kDebug(13010)<<"readGlobalKeywordConfig:END";
-
   kDebug(13010)<<"delimiterCharacters are: "<<deliminator;
+#endif
 
   m_additionalData[buildIdentifier]->deliminator = deliminator;
 }
@@ -1117,27 +1243,35 @@ void KateHighlighting::readGlobalKeywordConfig()
  */
 void KateHighlighting::readWordWrapConfig()
 {
-  // Tell the syntax document class which file we want to parse
+#ifdef HIGHLIGHTING_DEBUG
   kDebug(13010)<<"readWordWrapConfig:BEGIN";
+#endif
 
+  // Tell the syntax document class which file we want to parse
   KateHlManager::self()->syntax->setIdentifier(buildIdentifier);
   KateSyntaxContextData *data = KateHlManager::self()->syntax->getConfig("general","keywords");
 
   QString wordWrapDeliminator = stdDeliminator;
   if (data)
   {
+#ifdef HIGHLIGHTING_DEBUG
     kDebug(13010)<<"Found global keyword config";
+#endif
 
     wordWrapDeliminator = (KateHlManager::self()->syntax->groupItemData(data,QString("wordWrapDeliminator")));
     //when no wordWrapDeliminator is defined use the deliminator list
     if ( wordWrapDeliminator.length() == 0 ) wordWrapDeliminator = deliminator;
 
+#ifdef HIGHLIGHTING_DEBUG
     kDebug(13010) << "word wrap deliminators are " << wordWrapDeliminator;
+#endif
 
     KateHlManager::self()->syntax->freeGroupInfo(data);
   }
 
+#ifdef HIGHLIGHTING_DEBUG
   kDebug(13010)<<"readWordWrapConfig:END";
+#endif
 
   m_additionalData[buildIdentifier]->wordWrapDeliminator = wordWrapDeliminator;
 }
@@ -1159,15 +1293,19 @@ void KateHighlighting::readIndentationConfig()
 
 void KateHighlighting::readFoldingConfig()
 {
-  // Tell the syntax document class which file we want to parse
+#ifdef HIGHLIGHTING_DEBUG
   kDebug(13010)<<"readfoldignConfig:BEGIN";
+#endif
 
+  // Tell the syntax document class which file we want to parse
   KateHlManager::self()->syntax->setIdentifier(buildIdentifier);
   KateSyntaxContextData *data = KateHlManager::self()->syntax->getConfig("general","folding");
 
   if (data)
   {
+#ifdef HIGHLIGHTING_DEBUG
     kDebug(13010)<<"Found global keyword config";
+#endif
 
     if ( IS_TRUE( KateHlManager::self()->syntax->groupItemData(data,QString("indentationsensitive")) ) )
       m_foldingIndentationSensitive=true;
@@ -1182,14 +1320,73 @@ void KateHighlighting::readFoldingConfig()
     m_foldingIndentationSensitive = false;
   }
 
+#ifdef HIGHLIGHTING_DEBUG
   kDebug(13010)<<"readfoldingConfig:END";
-
   kDebug(13010)<<"############################ use indent for fold are: "<<m_foldingIndentationSensitive;
+#endif
+}
+
+void KateHighlighting::readSpellCheckingConfig()
+{
+  KateHlManager::self()->syntax->setIdentifier(buildIdentifier);
+  KateSyntaxContextData *data=KateHlManager::self()->syntax->getGroupInfo("spellchecking","encoding");
+
+  if (data)
+  {
+    while  (KateHlManager::self()->syntax->nextGroup(data))
+    {
+        QString encoding = KateHlManager::self()->syntax->groupData(data,"string");
+        QString character = KateHlManager::self()->syntax->groupData(data,"char");
+        QString ignored = KateHlManager::self()->syntax->groupData(data,"ignored");
+
+        const bool ignoredIsTrue = IS_TRUE(ignored);
+        if(encoding.isEmpty() || (character.isEmpty() && !ignoredIsTrue))
+        {
+          continue;
+        }
+        QRegExp newLineRegExp();
+        if(encoding.indexOf(QRegExp("\\r|\\n")) >= 0)
+        {
+          encoding.replace(QRegExp("\\r|\\n"), "<\\n|\\r>");
+
+#ifdef HIGHLIGHTING_DEBUG
+          kDebug() << "Encoding" << encoding
+                                 << "contains new-line characters. Ignored.";
+#endif
+        }
+        QChar c = (character.isEmpty() || ignoredIsTrue) ? QChar() : character[0];
+        addCharacterEncoding(buildIdentifier, encoding, c);
+    }
+
+    KateHlManager::self()->syntax->freeGroupInfo(data);
+  }
+
+  data=KateHlManager::self()->syntax->getConfig("spellchecking","configuration");
+  if (data)
+  {
+    QString policy = KateHlManager::self()->syntax->groupItemData(data,"encodingReplacementPolicy");
+    QString policyLowerCase = policy.toLower();
+    int p;
+
+    if(policyLowerCase == "encodewhenpresent") {
+      p = KateDocument::EncodeWhenPresent;
+    }
+    else if(policyLowerCase == "encodealways") {
+      p = KateDocument::EncodeAlways;
+    }
+    else {
+      p = KateDocument::EncodeNever;
+    }
+
+    m_additionalData[buildIdentifier]->encodedCharactersInsertionPolicy = p;
+  }
 }
 
 void  KateHighlighting::createContextNameList(QStringList *ContextNameList,int ctx0)
 {
+#ifdef HIGHLIGHTING_DEBUG
   kDebug(13010)<<"creatingContextNameList:BEGIN";
+#endif
 
   if (ctx0 == 0)
       ContextNameList->clear();
@@ -1216,8 +1413,10 @@ void  KateHighlighting::createContextNameList(QStringList *ContextNameList,int c
      }
      KateHlManager::self()->syntax->freeGroupInfo(data);
   }
-  kDebug(13010)<<"creatingContextNameList:END";
 
+#ifdef HIGHLIGHTING_DEBUG
+  kDebug(13010)<<"creatingContextNameList:END";
+#endif
 }
 
 KateHlContextModification KateHighlighting::getContextModificationFromString(QStringList *ContextNameList, QString tmpLineEndContext, /*NO CONST*/ QString &unres)
@@ -1233,10 +1432,11 @@ KateHlContextModification KateHighlighting::getContextModificationFromString(QSt
 
   // we allow arbitrary #stay and #pop at the start
   bool anyFound = false;
-  while (tmpLineEndContext.startsWith("#stay") || tmpLineEndContext.startsWith("#pop"))
+  while (tmpLineEndContext.startsWith(QLatin1String("#stay")) ||
+         tmpLineEndContext.startsWith(QLatin1String("#pop")))
   {
     // ignore stay
-    if (tmpLineEndContext.startsWith("#stay"))
+    if (tmpLineEndContext.startsWith(QLatin1String("#stay")))
     {
       tmpLineEndContext.remove (0, 5);
     }
@@ -1276,7 +1476,11 @@ KateHlContextModification KateHighlighting::getContextModificationFromString(QSt
     QString tmp=tmpLineEndContext.mid(o+2);
     if (!embeddedHls.contains(tmp))  embeddedHls.insert(tmp,KateEmbeddedHlInfo());
     unres=tmp+':'+tmpLineEndContext.left(o);
+
+#ifdef HIGHLIGHTING_DEBUG
     kDebug(13010) << "unres = " << unres;
+#endif
+
     context=0;
   }
 
@@ -1308,6 +1512,7 @@ void KateHighlighting::makeContextList()
     return;
 
   embeddedHls.clear();
+  embeddedHighlightingModes.clear();
   unresolvedContextReferences.clear();
   RegionList.clear();
   ContextNameList.clear();
@@ -1324,22 +1529,34 @@ void KateHighlighting::makeContextList()
 
   do
   {
+#ifdef HIGHLIGHTING_DEBUG
     kDebug(13010)<<"**************** Outer loop in make ContextList";
     kDebug(13010)<<"**************** Hl List count:"<<embeddedHls.count();
+#endif
+
     something_changed=false; //assume all "embedded" hls have already been loaded
     for (KateEmbeddedHlInfos::iterator it=embeddedHls.begin(); it!=embeddedHls.end();++it)
     {
       if (!it.value().loaded)  // we found one, we still have to load
       {
+#ifdef HIGHLIGHTING_DEBUG
         kDebug(13010)<<"**************** Inner loop in make ContextList";
+#endif
+
         QString identifierToUse;
+
+#ifdef HIGHLIGHTING_DEBUG
         kDebug(13010)<<"Trying to open highlighting definition file: "<< it.key();
+#endif
+
         if (iName==it.key()) // the own identifier is known
           identifierToUse=identifier;
         else                 // all others have to be looked up
           identifierToUse=KateHlManager::self()->identifierForName(it.key());
 
+#ifdef HIGHLIGHTING_DEBUG
         kDebug(13010)<<"Location is:"<< identifierToUse;
+#endif
 
         buildPrefix=it.key()+':';  // attribute names get prefixed by the names
                                    // of the highlighting definitions they belong to
@@ -1347,7 +1564,9 @@ void KateHighlighting::makeContextList()
         if (identifierToUse.isEmpty() )
           kDebug(13010)<<"OHOH, unknown highlighting description referenced";
 
+#ifdef HIGHLIGHTING_DEBUG
         kDebug(13010)<<"setting ("<<it.key()<<") to loaded";
+#endif
 
         //mark hl as loaded
         it=embeddedHls.insert(it.key(),KateEmbeddedHlInfo(true,startctx));
@@ -1366,9 +1585,11 @@ void KateHighlighting::makeContextList()
                   // repeat everything, there could be newly added embedded hls.
 
 
+#ifdef HIGHLIGHTING_DEBUG
   // at this point all needed highlighing (sub)definitions are loaded. It's time
-  // to resolve cross file  references (if there are any)
+  // to resolve cross file  references (if there are any)#
   kDebug(13010)<<"Unresolved contexts, which need attention: "<<unresolvedContextReferences.count();
+#endif
 
   //optimize this a littlebit
   for (KateHlUnresolvedCtxRefs::iterator unresIt=unresolvedContextReferences.begin();
@@ -1376,12 +1597,18 @@ void KateHighlighting::makeContextList()
        ++unresIt)
   {
     QString incCtx = unresIt.value();
+
+#ifdef HIGHLIGHTING_DEBUG
     kDebug(13010) << "Context " <<incCtx << " is unresolved";
+#endif
 
     // only resolve '##Name' contexts here; handleKateHlIncludeRules() can figure
     // out 'Name##Name'-style inclusions, but we screw it up
     if (incCtx.endsWith(':')) {
+#ifdef HIGHLIGHTING_DEBUG
       kDebug(13010)<<"Looking up context0 for ruleset "<<incCtx;
+#endif
+
       incCtx = incCtx.left(incCtx.length()-1);
       //try to find the context0 id for a given unresolvedReference
       KateEmbeddedHlInfos::const_iterator hlIt=embeddedHls.constFind(incCtx);
@@ -1396,6 +1623,9 @@ void KateHighlighting::makeContextList()
   // belongs to
   handleKateHlIncludeRules();
 
+  embeddedHighlightingModes = embeddedHls.keys();
+  embeddedHighlightingModes.removeOne(iName);
+
   embeddedHls.clear(); //save some memory.
   unresolvedContextReferences.clear(); //save some memory
   RegionList.clear();  // I think you get the idea ;)
@@ -1416,7 +1646,10 @@ void KateHighlighting::makeContextList()
 void KateHighlighting::handleKateHlIncludeRules()
 {
   // if there are noe include rules to take care of, just return
+#ifdef HIGHLIGHTING_DEBUG
   kDebug(13010)<<"KateHlIncludeRules, which need attention: " <<includeRules.size();
+#endif
+
   if (includeRules.isEmpty()) return;
 
   buildPrefix="";
@@ -1447,11 +1680,15 @@ void KateHighlighting::handleKateHlIncludeRules()
       {
         // resolve name to id
         (*it)->incCtx=getContextModificationFromString(&ContextNameList,(*it)->incCtxN,dummy).newContext;
+
+#ifdef HIGHLIGHTING_DEBUG
         kDebug(13010)<<"Resolved "<<(*it)->incCtxN<< " to "<<(*it)->incCtx.newContext<<" for include rule";
+#endif
+
         // It would be good to look here somehow, if the result is valid
       }
     }
-    else ++it; //nothing to do, already resolved (by the cross defintion reference resolver)
+    else ++it; //nothing to do, already resolved (by the cross definition reference resolver)
   }
 
   // now that all KateHlIncludeRule items should be valid and completely resolved,
@@ -1544,7 +1781,7 @@ void KateHighlighting::handleKateHlIncludeRulesRecursive(int index, KateHlInclud
  */
 int KateHighlighting::addToContextList(const QString &ident, int ctx0)
 {
-  kDebug(13010)<<"=== Adding hl with ident '"<<ident<<"'";
+  //kDebug(13010)<<"=== Adding hl with ident '"<<ident<<"' ctx0="<<ctx0;
 
   buildIdentifier=ident;
   KateSyntaxContextData *data, *datasub;
@@ -1571,7 +1808,13 @@ int KateHighlighting::addToContextList(const QString &ident, int ctx0)
   RegionList<<"!KateInternal_TopLevel!";
 
   m_hlIndex[internalIDList.count()] = ident;
-  m_additionalData.insert( ident, new HighlightPropertyBag );
+  m_ctxIndex[ctx0]=ident;
+
+  // clear and reuse or create new
+  if (m_additionalData[ident])
+    *m_additionalData[ident] = HighlightPropertyBag ();
+  else
+    m_additionalData.insert( ident, new HighlightPropertyBag );
 
   // fill out the propertybag
   readCommentConfig();
@@ -1581,6 +1824,8 @@ int KateHighlighting::addToContextList(const QString &ident, int ctx0)
 
   readFoldingConfig ();
 
+  readSpellCheckingConfig();
+
   QString ctxName;
 
   // This list is needed for the translation of the attribute parameter,
@@ -1590,8 +1835,10 @@ int KateHighlighting::addToContextList(const QString &ident, int ctx0)
 
   createContextNameList(&ContextNameList,ctx0);
 
-
+#ifdef HIGHLIGHTING_DEBUG
   kDebug(13010)<<"Parsing Context structure";
+#endif
+
   //start the real work
   data=KateHlManager::self()->syntax->getGroupInfo("highlighting","context");
   uint i=buildContext0Offset;
@@ -1599,7 +1846,10 @@ int KateHighlighting::addToContextList(const QString &ident, int ctx0)
   {
     while (KateHlManager::self()->syntax->nextGroup(data))
     {
+#ifdef HIGHLIGHTING_DEBUG
       kDebug(13010)<<"Found a context in file, building structure now";
+#endif
+
       //BEGIN - Translation of the attribute parameter
       QString tmpAttr=KateHlManager::self()->syntax->groupData(data,QString("attribute")).simplified();
       int attr;
@@ -1636,7 +1886,9 @@ int KateHighlighting::addToContextList(const QString &ident, int ctx0)
           // stay is not allowed, we need to #pop or push some context...
           if (ftc.type == KateHlContextModification::doNothing) ftc = 0;
 
+#ifdef HIGHLIGHTING_DEBUG
           kDebug(13010)<<"Setting fall through context (context "<<i<<"): "<<ftc.newContext;
+#endif
         }
       }
       //END falltrhough props
@@ -1656,7 +1908,9 @@ int KateHighlighting::addToContextList(const QString &ident, int ctx0)
 
       m_contexts.push_back (ctxNew);
 
+#ifdef HIGHLIGHTING_DEBUG
       kDebug(13010) << "INDEX: " << i << " LENGTH " << m_contexts.size()-1;
+#endif
 
       //Let's create all items for the context
       while (KateHlManager::self()->syntax->nextItem(data))
@@ -1683,14 +1937,19 @@ int KateHighlighting::addToContextList(const QString &ident, int ctx0)
              QString incCtxN = incSet + ':' + incCtx.left(incCtxi);
 
              //a cross highlighting reference
+#ifdef HIGHLIGHTING_DEBUG
              kDebug(13010)<<"Cross highlight reference <IncludeRules>, context "<<incCtxN;
+#endif
+
              KateHlIncludeRule *ir=new KateHlIncludeRule(i,m_contexts[i]->items.count(),incCtxN,includeAttrib);
 
              //use the same way to determine cross hl file references as other items do
              if (!embeddedHls.contains(incSet))
                embeddedHls.insert(incSet,KateEmbeddedHlInfo());
+#ifdef HIGHLIGHTING_DEBUG
              else
                kDebug(13010)<<"Skipping embeddedHls.insert for "<<incCtxN;
+#endif
 
             unresolvedContextReferences.insert(&(ir->incCtx), incCtxN);
 
@@ -1764,11 +2023,16 @@ int KateHighlighting::addToContextList(const QString &ident, int ctx0)
           "<b>%1</b>: Specified multiline comment region (%2) could not be resolved<br />"
                              , buildIdentifier,  m_additionalData[ ident ]->multiLineRegion );
       m_additionalData[ ident ]->multiLineRegion.clear();
-      kDebug(13010)<<"ERROR comment region attribute could not be resolved";
 
+#ifdef HIGHLIGHTING_DEBUG
+      kDebug(13010)<<"ERROR comment region attribute could not be resolved";
+#endif
     } else {
       m_additionalData[ ident ]->multiLineRegion=QString::number(commentregionid+1);
+
+#ifdef HIGHLIGHTING_DEBUG
       kDebug(13010)<<"comment region resolved to:"<<m_additionalData[ ident ]->multiLineRegion;
+#endif
     }
   }
   //END Resolve multiline region if possible
@@ -1839,6 +2103,11 @@ QList<KTextEditor::Attribute::Ptr> KateHighlighting::attributes (const QString &
   return array;
 }
 
+QStringList KateHighlighting::getEmbeddedHighlightingModes() const
+{
+  return embeddedHighlightingModes;
+}
+
 //END
 
 // kate: space-indent on; indent-width 2; replace-tabs on;
