@@ -132,8 +132,8 @@ class KateHighlighting
     void cleanup ();
 
   public:
-    void doHighlight ( KateTextLine *prevLine,
-                       KateTextLine *textLine,
+    void doHighlight ( Kate::TextLineData *prevLine,
+                       Kate::TextLineData *textLine,
                        QVector<int> &foldingList,
                        bool &ctxChanged );
 
@@ -249,7 +249,7 @@ class KateHighlighting
     /**
      * Returns a list of names of embedded modes.
      */
-    QStringList getEmbeddedModes() const;
+    QStringList getEmbeddedHighlightingModes() const;
 
   private:
     /**
@@ -301,7 +301,7 @@ class KateHighlighting
     // NOTE: gets cleaned once makeContextList() finishes
     KateEmbeddedHlInfos embeddedHls;
     // valid once makeContextList() finishes
-    QStringList embeddedModes;
+    QStringList embeddedHighlightingModes;
     KateHlUnresolvedCtxRefs unresolvedContextReferences;
     QStringList RegionList;
     QStringList ContextNameList;
@@ -337,6 +337,8 @@ class KateHighlighting
     // map schema name to attributes...
     QHash< QString, QList<KTextEditor::Attribute::Ptr> > m_attributeArrays;
 
+    // list of all created items to delete them later
+    QList<KateHlItem *> m_hlItemCleanupList;
 
     /**
      * This class holds the additional properties for one highlight
