@@ -25,6 +25,7 @@
 
 //BEGIN Includes
 #include "test_regression.h"
+#include "testutils.h"
 
 #include "kateview.h"
 #include "katedocument.h"
@@ -32,8 +33,7 @@
 #include "kateconfig.h"
 #include "katecmd.h"
 #include "kateglobal.h"
-#include "interfaces/ktexteditor/commandinterface.h"
-#include "completiontest.h"
+#include <ktexteditor/commandinterface.h>
 
 #include <kapplication.h>
 #include <kglobal.h>
@@ -72,6 +72,7 @@
 #include <QtCore/QFileInfo>
 #include <QtCore/Q_PID>
 #include <QtCore/QEvent>
+#include <QtCore/QTimer>
 
 #include <QtScript/QScriptEngine>
 #include <QTest>
@@ -83,265 +84,6 @@
 
 static KMainWindow* toplevel;
 
-//BEGIN TestScriptEnv
-
-
-QScriptValue runNativeTestCase(QScriptContext *context, QScriptEngine *engine)
-{
-  QObject* test = 0;
-  if (context->argument(0).toString() == "completion") {
-    test = new CompletionTest;
-  } else {
-    return context->throwError("test does not exist");
-  }
-  KateDocumentObject* d = qobject_cast<KateDocumentObject*>(engine->globalObject().property("document").toQObject());
-  QStringList arguments;
-  arguments << context->argument(0).toString();
-  arguments << context->argument(1).toString();
-  if (QTest::qExec(test, arguments) == 0) {
-    d->document()->setText("success");
-    return engine->newVariant(true);
-  } else {
-    d->document()->setText("failure");
-    return engine->newVariant(false);
-  }
-}
-
-TestScriptEnv::TestScriptEnv(KateDocument *part, bool &cflag)
-  : m_engine(0), m_viewObj(0), m_docObj(0), m_output(0)
-{
-  m_engine = new QScriptEngine(this);
-
-  KateView *view = qobject_cast<KateView *>(part->widget());
-
-  m_viewObj = new KateViewObject(view);
-  QScriptValue sv = m_engine->newQObject(m_viewObj);
-
-  m_engine->globalObject().setProperty("view", sv);
-  m_engine->globalObject().setProperty("v", sv);
-
-  m_docObj = new KateDocumentObject(view->doc());
-  QScriptValue sd = m_engine->newQObject(m_docObj);
-
-  m_engine->globalObject().setProperty("document", sd);
-  m_engine->globalObject().setProperty("d", sd);
-
-  m_output = new OutputObject(view, cflag);
-  QScriptValue so = m_engine->newQObject(m_output);
-
-  m_engine->globalObject().setProperty("output", so);
-  m_engine->globalObject().setProperty("out", so);
-  m_engine->globalObject().setProperty("o", so);
-
-  m_engine->globalObject().setProperty("runNativeTestCase", m_engine->newFunction(runNativeTestCase));
-}
-//END TestScriptEnv
-
-//BEGIN KateViewObject
-
-KateViewObject::KateViewObject(KateView *view)
-  : KateScriptView()
-{
-  setView(view);
-}
-
-bool KateViewObject::setCursorPosition(int row, int col)
-{
-  return view()->setCursorPosition(KTextEditor::Cursor(row, col));
-}
-
-// Implements a function that calls an edit function repeatedly as specified by
-// its first parameter (once if not specified).
-#define REP_CALL(func) \
-void KateViewObject::func(int cnt) {  \
-  while (cnt--) { view()->func(); }   \
-}
-REP_CALL(keyReturn)
-REP_CALL(backspace)
-REP_CALL(deleteWordLeft)
-REP_CALL(keyDelete)
-REP_CALL(deleteWordRight)
-REP_CALL(transpose)
-REP_CALL(cursorLeft)
-REP_CALL(shiftCursorLeft)
-REP_CALL(cursorRight)
-REP_CALL(shiftCursorRight)
-REP_CALL(wordLeft)
-REP_CALL(shiftWordLeft)
-REP_CALL(wordRight)
-REP_CALL(shiftWordRight)
-REP_CALL(home)
-REP_CALL(shiftHome)
-REP_CALL(end)
-REP_CALL(shiftEnd)
-REP_CALL(up)
-REP_CALL(shiftUp)
-REP_CALL(down)
-REP_CALL(shiftDown)
-REP_CALL(scrollUp)
-REP_CALL(scrollDown)
-REP_CALL(topOfView)
-REP_CALL(shiftTopOfView)
-REP_CALL(bottomOfView)
-REP_CALL(shiftBottomOfView)
-REP_CALL(pageUp)
-REP_CALL(shiftPageUp)
-REP_CALL(pageDown)
-REP_CALL(shiftPageDown)
-REP_CALL(top)
-REP_CALL(shiftTop)
-REP_CALL(bottom)
-REP_CALL(shiftBottom)
-REP_CALL(toMatchingBracket)
-REP_CALL(shiftToMatchingBracket)
-#undef REP_CALL
-
-bool KateViewObject::type(const QString& str) {
-  return view()->doc()->typeChars(view(), str);
-}
-
-#define ALIAS(alias, func) \
-void KateViewObject::alias(int cnt) { \
-  func(cnt);                          \
-}
-ALIAS(enter, keyReturn)
-ALIAS(cursorPrev, cursorLeft)
-ALIAS(left, cursorLeft)
-ALIAS(prev, cursorLeft)
-ALIAS(shiftCursorPrev, shiftCursorLeft)
-ALIAS(shiftLeft, shiftCursorLeft)
-ALIAS(shiftPrev, shiftCursorLeft)
-ALIAS(cursorNext, cursorRight)
-ALIAS(right, cursorRight)
-ALIAS(next, cursorRight)
-ALIAS(shiftCursorNext, shiftCursorRight)
-ALIAS(shiftRight, shiftCursorRight)
-ALIAS(shiftNext, shiftCursorRight)
-ALIAS(wordPrev, wordLeft)
-ALIAS(shiftWordPrev, shiftWordLeft)
-ALIAS(wordNext, wordRight)
-ALIAS(shiftWordNext, shiftWordRight)
-#undef ALIAS
-
-//END KateViewObject
-
-//BEGIN KateDocumentObject
-
-KateDocumentObject::KateDocumentObject(KateDocument *doc)
-  : KateScriptDocument()
-{
-  setDocument(doc);
-}
-
-//END KateDocumentObject
-
-//BEGIN OutputObject
-
-OutputObject::OutputObject(KateView *v, bool &cflag)
-  : view(v), cflag(cflag)
-{
-}
-
-void OutputObject::output(bool cp, bool ln)
-{
-  RegressionTest::createMissingDirs(filename);
-  QFile out(filename);
-
-  QString str;
-  for (int i = 0; i < context()->argumentCount(); ++i) {
-    QScriptValue arg = context()->argument(i);
-    str += arg.toString();
-  }
-
-  if (cp) {
-    KTextEditor::Cursor c = view->cursorPosition();
-    str += '(' + QString::number(c.line()) + ',' + QString::number(c.column()) + ')';
-  }
-
-  if (ln) {
-    str += '\n';
-  }
-
-  QFile::OpenMode mode = QFile::WriteOnly | (cflag ? QFile::Append : QFile::Truncate);
-  if (!out.open(mode)) {
-    fprintf(stderr, "ERROR: Could not append to %s\n", filename.toLatin1().constData());
-  }
-  out.write(str.toUtf8());
-  cflag = true;
-}
-
-void OutputObject::write()
-{
-  output(false, false);
-}
-
-void OutputObject::writeln()
-{
-  output(false, true);
-}
-
-void OutputObject::writeLn()
-{
-  output(false, true);
-}
-
-void OutputObject::print()
-{
-  output(false, false);
-}
-
-void OutputObject::println()
-{
-  output(false, true);
-}
-
-void OutputObject::printLn()
-{
-  output(false, true);
-}
-
-void OutputObject::writeCursorPosition()
-{
-  output(true, false);
-}
-
-void OutputObject::writeCursorPositionln()
-{
-  output(true, true);
-}
-
-void OutputObject::cursorPosition()
-{
-  output(true, false);
-}
-
-void OutputObject::cursorPositionln()
-{
-  output(true, true);
-}
-
-void OutputObject::cursorPositionLn()
-{
-  output(true, true);
-}
-
-void OutputObject::pos()
-{
-  output(true, false);
-}
-
-void OutputObject::posln()
-{
-  output(true, true);
-}
-
-void OutputObject::posLn()
-{
-  output(true, true);
-}
-
-//END OutputObject
-
 // -------------------------------------------------------------------------
 
 const char failureSnapshotPrefix[] = "testkateregressionrc-FS.";
@@ -378,15 +120,15 @@ int main(int argc, char *argv[])
   options.add("+[testcases]", ki18n("Relative path to testcase, or directory of testcases to be run (equivalent to -t)."));
 
   // forget about any settings
-  passwd* pw = getpwuid( getuid() );
-  if (!pw) {
+  passwd* password = getpwuid( getuid() );
+  if (!password) {
     fprintf(stderr, "dang, I don't even know who I am.\n");
     exit(1);
   }
 
-  QString kh(UNIQUE_HOME_DIR);
-  kh = kh.arg( pw->pw_name );
-  setenv( "KDEHOME", kh.toLatin1().constData(), 1 );
+  QString kdeHome(UNIQUE_HOME_DIR);
+  kdeHome = kdeHome.arg( password->pw_name );
+  setenv( "KDEHOME", kdeHome.toLatin1().constData(), 1 );
   setenv( "LC_ALL", "C", 1 );
   setenv( "LANG", "C", 1 );
 
@@ -433,166 +175,58 @@ int main(int argc, char *argv[])
   if (baseDir.isEmpty()) baseDir = args->arg(testcase_index++);
 
   QFileInfo bdInfo(baseDir);
-  baseDir = QFile::encodeName(bdInfo.absoluteFilePath());
+  baseDir = bdInfo.absoluteFilePath();
 
   const char *subdirs[] = {"tests", "baseline", "output", "resources"};
   for ( int i = 0; i < 2; i++ ) {
-    QFileInfo sourceDir(QFile::encodeName( baseDir ) + '/' + subdirs[i]);
+    QFileInfo sourceDir(baseDir + '/' + subdirs[i]);
     if ( !sourceDir.exists() || !sourceDir.isDir() ) {
       fprintf(stderr,"ERROR: Source directory \"%s/%s\": no such directory.\n", baseDir.toLatin1().constData(), subdirs[i]);
       exit(1);
     }
   }
 
-  KApplication a;
+  KateTestApp a(args, baseDir, testcase_index);
 
-  // FIXME: Any analogous call for dbus?
-//   a.disableAutoDcopRegistration();
-  a.setStyle("windows");
-  KConfig cfg( "testkateregressionrc", KConfig::SimpleConfig );
-  KConfigGroup group = cfg.group("Kate Document Defaults");
-  group.writeEntry("Basic Config Flags",
-                   KateDocumentConfig::cfBackspaceIndents
-//                  | KateDocumentConfig::cfWordWrap
-//                  | KateDocumentConfig::cfRemoveSpaces
-                 | KateDocumentConfig::cfWrapCursor
-//                  | KateDocumentConfig::cfAutoBrackets
-//                  | KateDocumentConfig::cfTabIndentsMode
-//                  | KateDocumentConfig::cfOvr
-                 | KateDocumentConfig::cfKeepExtraSpaces
-                 | KateDocumentConfig::cfTabIndents
-                 | KateDocumentConfig::cfShowTabs
-//                  | KateDocumentConfig::cfSpaceIndent TODO lookup replacement
-                 | KateDocumentConfig::cfSmartHome
-                 | KateDocumentConfig::cfTabInsertsTab
-//                  | KateDocumentConfig::cfReplaceTabsDyn
-//                  | KateDocumentConfig::cfRemoveTrailingDyn
-//                  | KateDocumentConfig::cfDoxygenAutoTyping // TODO lookup replacement
-//                  | KateDocumentConfig::cfMixedIndent
-                 | KateDocumentConfig::cfIndentPastedText
-                  );
-  cfg.sync();
+  // queue quit action
+  QTimer::singleShot(0, &a, SLOT(quit()));
 
-  {
-    KConfig dc( "kdebugrc", KConfig::SimpleConfig );
-    // FIXME adapt to kate
-    static int areas[] = { 1000, 13000, 13001, 13002, 13010,
-        13020, 13025, 13030, 13033, 13035,
-        13040, 13050, 13051, 13070, 7000, 7006, 170,
-        171, 7101, 7002, 7019, 7027, 7014,
-        7001, 7011, 6070, 6080, 6090, 0};
-    int channel = args->isSet( "debug" ) ? 2 : 4;
-    for ( int i = 0; areas[i]; ++i ) {
-      KConfigGroup group = dc.group( QString::number( areas[i] ) );
-      group.writeEntry( "InfoOutput", channel );
-    }
-    dc.sync();
+  a.exec();
 
-    kClearDebugConfig();
-  }
-
-  // create widgets
-  std::auto_ptr<KMainWindow> mw(toplevel = new KMainWindow());
-  std::auto_ptr<KateDocument> part(new KateDocument(true, false, false, mw.get()));
-  part->setObjectName("testkate");
-
-  mw->setCentralWidget( part->widget() );
-
-  if (args->isSet("show"))
-    mw->show();
-
-  // we're not interested
-  mw->statusBar()->hide();
-
-  if (qgetenv("KDE_DEBUG").isEmpty()) {
-    // set ulimits
-    rlimit vmem_limit = { 256*1024*1024, RLIM_INFINITY };	// 256Mb Memory should suffice
-#if defined(RLIMIT_AS)
-    setrlimit(RLIMIT_AS, &vmem_limit);
-#endif
-#if defined(RLIMIT_DATA)
-    setrlimit(RLIMIT_DATA, &vmem_limit);
-#endif
-    rlimit stack_limit = { 8*1024*1024, RLIM_INFINITY };	// 8Mb Memory should suffice
-    setrlimit(RLIMIT_STACK, &stack_limit);
-  }
-
-  // run the tests
-  std::auto_ptr<RegressionTest> regressionTest(new RegressionTest(part.get(), &cfg, baseDir, args));
-
-  {
-    QString failureSnapshot = args->getOption("cmp-failures");
-    if (failureSnapshot.isEmpty())
-      failureSnapshot = findMostRecentFailureSnapshot();
-    if (!failureSnapshot.isEmpty())
-      regressionTest->setFailureSnapshotConfig(new KConfig(failureSnapshotPrefix + failureSnapshot,
-                                                           KConfig::SimpleConfig),
-                                               failureSnapshot);
-  }
-
-  if (args->isSet("save-failures")) {
-    QString failureSaver = args->getOption("save-failures");
-    regressionTest->setFailureSnapshotSaver(new KConfig(failureSnapshotPrefix + failureSaver,
-                                                        KConfig::SimpleConfig),
-                                            failureSaver);
-  }
-
-  bool result = false;
-  QStringList tests = args->getOptionList("test");
-  // merge testcases specified on command line
-  for (; testcase_index < args->count(); testcase_index++)
-    tests << args->arg(testcase_index);
-  if (tests.count() > 0) {
-    for (QStringList::ConstIterator it = tests.constBegin(); it != tests.constEnd(); ++it) {
-      result = regressionTest->runTests(*it,true);
-      if (!result) break;
-    }
-  } else {
-    result = regressionTest->runTests();
-  }
-
-  if (result) {
-    if (args->isSet("genoutput")) {
-      printf("\nOutput generation completed.\n");
-    } else {
-      regressionTest->printSummary();
-    }
-  }
-
-  // Only return a 0 exit code if all tests were successful
-  return regressionTest->allTestsSucceeded() ? 0 : 1;
+  return a.allTestsSucceeded() ? 0 : 1;
 }
 
 // -------------------------------------------------------------------------
 
-RegressionTest::RegressionTest(KateDocument *part, KConfig *baseConfig,
-                               const QString &baseDir, KCmdLineArgs *args)
+RegressionTest::RegressionTest(KateDocument *part, const KConfig *baseConfig,
+                               const QString &baseDir, const KCmdLineArgs *args)
   : QObject(part)
+  , m_part(part)
+  , m_view( static_cast<KateView *>(m_part->widget()) )
+  , m_baseConfig(baseConfig)
+  , m_baseDir(baseDir)
+  , m_outputDir(args->getOption("output"))
+  , m_genOutput(args->isSet("genoutput"))
+  , m_fork(args->isSet("fork"))
+  , m_failureComp(0)
+  , m_failureSave(0)
+  , m_keepOutput(args->isSet("keep-output"))
+  , m_passes_work(0)
+  , m_passes_fail(0)
+  , m_passes_new(0)
+  , m_failures_work(0)
+  , m_failures_fail(0)
+  , m_failures_new(0)
+  , m_errors(0)
 {
-  m_part = part;
-  m_view = static_cast<KateView *>(m_part->widget());
-  m_baseConfig = baseConfig;
-  m_baseDir = baseDir;
   m_baseDir = m_baseDir.replace( "//", "/" );
   if ( m_baseDir.endsWith( "/" ) )
     m_baseDir = m_baseDir.left( m_baseDir.length() - 1 );
 
-  QString outputDir = args->getOption("output");
-  if (outputDir.isEmpty())
+  if (m_outputDir.isEmpty())
     m_outputDir = m_baseDir + "/output";
-  else
-    m_outputDir = outputDir;
-  createMissingDirs(m_outputDir + '/');
-  m_keepOutput = args->isSet("keep-output");
-  m_genOutput = args->isSet("genoutput");
-  m_fork = args->isSet("fork");
-  m_failureComp = 0;
-  m_failureSave = 0;
-  m_passes_work = m_passes_fail = m_passes_new = 0;
-  m_failures_work = m_failures_fail = m_failures_new = 0;
-  m_errors = 0;
 
-  ::unlink( QFile::encodeName( m_outputDir + "/links.html" ) );
+  QFile::remove( m_outputDir + "/links.html" );
   QFile f( m_outputDir + "/empty.html" );
   QString s;
   f.open( QFile::WriteOnly | QFile::Truncate );
@@ -746,7 +380,7 @@ bool RegressionTest::allTestsSucceeded() const
 
 void RegressionTest::createLink( const QString& test, int failures )
 {
-  createMissingDirs( m_outputDir + '/' + test + "-compare.html" );
+  OutputObject::createMissingDirs( m_outputDir + '/' + test + "-compare.html" );
 
   QFile list( m_outputDir + "/links.html" );
   list.open( QFile::WriteOnly|QFile::Append );
@@ -817,16 +451,6 @@ static QString makeRelativePath(const QString &base, const QString &path)
   return rel;
 }
 
-/** processes events for at least \c msec milliseconds */
-static void pause(int msec)
-{
-  QTime t;
-  t.start();
-  do {
-    kapp->processEvents();
-  } while (t.elapsed() < msec);
-}
-
 /**
  * returns a unique file name
  * (Cannot have QTemporaryFile as it won't return a file name without actually
@@ -867,7 +491,7 @@ static QString readVariable(const QString &varName)
 void RegressionTest::doFailureReport( const QString& test, int failures )
 {
   if ( failures == NoFailure ) {
-    ::unlink( QFile::encodeName( m_outputDir + '/' + test + "-compare.html" ) );
+    QFile::remove( m_outputDir + '/' + test + "-compare.html" );
     return;
   }
 
@@ -880,9 +504,8 @@ void RegressionTest::doFailureReport( const QString& test, int failures )
   QString renderDiff;
   QString domDiff;
 
-  char pwd[PATH_MAX];
-  (void) getcwd( pwd, PATH_MAX );
-  chdir( QFile::encodeName( m_baseDir ) );
+  QString pwd = QDir::currentPath();
+  QDir::setCurrent( m_baseDir );
   QString resolvedBaseDir = QDir::currentPath();
 
   QString relOutputDir = makeRelativePath(resolvedBaseDir/*m_baseDir*/, m_outputDir);
@@ -910,13 +533,13 @@ void RegressionTest::doFailureReport( const QString& test, int failures )
     delete is;
     domDiff += "</pre>";
     if (!err.isEmpty()) {
-      qWarning() << "cwd: " << QFile::encodeName(resolvedBaseDir) << ", basedir " << QFile::encodeName( m_baseDir);
+      qWarning() << "cwd: " << resolvedBaseDir << ", basedir " << m_baseDir;
       qWarning() << "diff " << args.join(" ");
       qWarning() << "Errors: " << err;
     }
   }
 
-  chdir( pwd );
+  QDir::setCurrent( pwd );
 
     // create a relative path so that it works via web as well. ugly
   QString relpath = makeRelativePath(m_outputDir + '/'
@@ -1027,7 +650,7 @@ void RegressionTest::testStaticFile(const QString & filename, const QStringList
   if (pid == 0) {
     // Execute script
     TestScriptEnv jsenv(m_part, m_outputCustomised);
-    jsenv.output()->setOutputFile( ( m_genOutput ? m_baseDir + "/baseline/" : m_outputDir + '/' ) + filename + "-result" );
+    jsenv.output()->setOutputFile( ( m_genOutput ? QString(m_baseDir + "/baseline/") : QString(m_outputDir + '/') ) + filename + "-result" );
     script_error = evalJS(jsenv.engine(), m_baseDir + "/tests/"+QFileInfo(filename).dir().path()+"/.kateconfig-script", true)
         && evalJS(jsenv.engine(), m_baseDir + "/tests/"+filename+"-script");
 
@@ -1155,7 +778,7 @@ RegressionTest::CheckResult RegressionTest::checkOutput(const QString &againstFi
 
     result = ( fileData == data ) ? Success : Failure;
     if ( !m_genOutput && result == Success && !m_keepOutput ) {
-        ::unlink( QFile::encodeName( outputFilename ) );
+        QFile::remove( outputFilename );
         return Success;
     }
   } else if (!m_genOutput) {
@@ -1164,7 +787,7 @@ RegressionTest::CheckResult RegressionTest::checkOutput(const QString &againstFi
   }
 
   // generate result file
-  createMissingDirs( outputFilename );
+  OutputObject::createMissingDirs( outputFilename );
   QFile file2(outputFilename);
   if (!file2.open(QFile::WriteOnly)) {
     fprintf(stderr,"Error writing to file %s\n",outputFilename.toLatin1().constData());
@@ -1267,30 +890,33 @@ void RegressionTest::printDescription(const QString& description)
 
 void RegressionTest::printSummary()
 {
-  printf("\nTests completed.\n");
-  printf("Total:    %d\n", m_passes_work + m_passes_fail +
-      m_failures_work + m_failures_fail + m_errors);
+  QTextStream out(stdout, QIODevice::WriteOnly);
+
+  out << endl;
+  out << "Tests completed." << endl;
+  out << "Total:    " << m_passes_work + m_passes_fail +
+      m_failures_work + m_failures_fail + m_errors << endl;
 
   //BEGIN Passes
-  printf("Passes:   %d", m_passes_work);
+  out << "Passes:   " << m_passes_work;
   if (m_passes_fail > 0)
-    printf(" (%d unexpected passes)", m_passes_fail);
+    out << QString(" (%1 unexpected passes)").arg(m_passes_fail);
   if (m_passes_new > 0)
-    printf(" (%d new since %s)", m_passes_new, m_failureComp->name().toLatin1().constData());
-  printf("\n");
+    out << QString(" (%1 new since %2)").arg(m_passes_new).arg(m_failureComp->name());
+  out << endl;
   //END Passes
 
   //BEGIN Failures
-  printf("Failures: %d", m_failures_work);
+  out << "Failures: " << m_failures_work;
   if (m_failures_fail > 0)
-    printf(" (%d expected failures)", m_failures_fail);
+    out << QString(" (%1 expected failures)").arg(m_failures_fail);
   if (m_failures_new > 0)
-    printf(" (%d new since %s)", m_failures_new, m_failureComp->name().toLatin1().constData());
-  printf("\n");
+    out << QString(" (%d new since %s)").arg(m_failures_new).arg(m_failureComp->name());
+  out << endl;
   //END Failures
 
   if (m_errors > 0)
-    printf("Errors:   %d\n", m_errors);
+    out << "Errors:   " << m_errors << endl;
 
   //BEGIN html
   QFile list(m_outputDir + "/links.html");
@@ -1314,34 +940,6 @@ void RegressionTest::printSummary()
   //END html
 }
 
-void RegressionTest::createMissingDirs(const QString & filename)
-{
-  QFileInfo dif(filename);
-  QFileInfo dirInfo( dif.dir().path() );
-  if (dirInfo.exists())
-    return;
-
-  QStringList pathComponents;
-  QFileInfo parentDir = dirInfo;
-  pathComponents.prepend(parentDir.absoluteFilePath());
-  while (!parentDir.exists()) {
-    QString parentPath = parentDir.absoluteFilePath();
-    int slashPos = parentPath.lastIndexOf('/');
-    if (slashPos < 0)
-      break;
-    parentPath = parentPath.left(slashPos);
-    pathComponents.prepend(parentPath);
-    parentDir = QFileInfo(parentPath);
-  }
-  for (int pathno = 1; pathno < pathComponents.count(); pathno++) {
-    if (!QFileInfo(pathComponents[pathno]).exists() &&
-        !QDir(pathComponents[pathno-1]).mkdir(pathComponents[pathno])) {
-      fprintf(stderr,"Error creating directory %s\n",pathComponents[pathno].toLatin1().constData());
-      exit(1);
-    }
-  }
-}
-
 void RegressionTest::slotOpenURL(const KUrl &url, const KParts::OpenUrlArguments & args, const KParts::BrowserArguments&)
 {
   m_part->setArguments(args);
@@ -1373,5 +971,145 @@ void RegressionTest::resizeTopLevelWidget( int w, int h )
   QApplication::sendPostedEvents( 0, QEvent::Resize );
 }
 
+
+
+KateTestApp::KateTestApp(KCmdLineArgs *args, const QString& baseDir, int testcaseIndex)
+  : KApplication()
+  , m_args(args)
+  , m_cfg("testkateregressionrc", KConfig::SimpleConfig)
+  , m_baseDir(baseDir)
+  , m_testcaseIndex(testcaseIndex)
+{
+  // FIXME: Any analogous call for dbus?
+  //   a.disableAutoDcopRegistration();
+  setStyle("windows");
+  KConfigGroup group = m_cfg.group("Kate Document Defaults");
+  group.writeEntry("Basic Config Flags",
+                   KateDocumentConfig::cfBackspaceIndents
+//                  | KateDocumentConfig::cfWordWrap
+//                  | KateDocumentConfig::cfRemoveSpaces
+                   | KateDocumentConfig::cfWrapCursor
+//                  | KateDocumentConfig::cfAutoBrackets
+//                  | KateDocumentConfig::cfTabIndentsMode
+//                  | KateDocumentConfig::cfOvr
+                   | KateDocumentConfig::cfKeepExtraSpaces
+                   | KateDocumentConfig::cfTabIndents
+                   | KateDocumentConfig::cfShowTabs
+//                  | KateDocumentConfig::cfSpaceIndent TODO lookup replacement
+                   | KateDocumentConfig::cfSmartHome
+                   | KateDocumentConfig::cfTabInsertsTab
+//                  | KateDocumentConfig::cfReplaceTabsDyn
+//                  | KateDocumentConfig::cfRemoveTrailingDyn
+//                  | KateDocumentConfig::cfDoxygenAutoTyping // TODO lookup replacement
+//                  | KateDocumentConfig::cfMixedIndent
+                   | KateDocumentConfig::cfIndentPastedText
+                   );
+  m_cfg.sync();
+
+//   {
+//     KConfig dc( "kdebugrc", KConfig::SimpleConfig );
+//     // FIXME adapt to kate
+//     static int areas[] = { 1000, 13000, 13001, 13002, 13010,
+//     13020, 13025, 13030, 13033, 13035,
+//     13040, 13050, 13051, 13070, 7000, 7006, 170,
+//     171, 7101, 7002, 7019, 7027, 7014,
+//     7001, 7011, 6070, 6080, 6090, 0};
+//     int channel = args->isSet( "debug" ) ? 2 : 4;
+//     for ( int i = 0; areas[i]; ++i ) {
+//       KConfigGroup group = dc.group( QString::number( areas[i] ) );
+//       group.writeEntry( "InfoOutput", channel );
+//     }
+//     dc.sync();
+//
+//     kClearDebugConfig();
+//   }
+
+  // create widgets
+  toplevel = new KMainWindow();
+  m_document = new KateDocument(true, false, false, toplevel);
+  m_document->setObjectName("testkate");
+
+  toplevel->setCentralWidget( m_document->widget() );
+
+  if (args->isSet("show"))
+    toplevel->show();
+
+  // we're not interested
+  toplevel->statusBar()->hide();
+
+  if (false && qgetenv("KDE_DEBUG").isEmpty()) {
+    // set ulimits
+    rlimit vmem_limit = { 256*1024*1024, RLIM_INFINITY };   // 256Mb Memory should suffice
+#if defined(RLIMIT_AS)
+    setrlimit(RLIMIT_AS, &vmem_limit);
+#endif
+#if defined(RLIMIT_DATA)
+    setrlimit(RLIMIT_DATA, &vmem_limit);
+#endif
+    rlimit stack_limit = { 8*1024*1024, RLIM_INFINITY };    // 8Mb Memory should suffice
+    setrlimit(RLIMIT_STACK, &stack_limit);
+  }
+
+  // run the tests
+  m_regressionTest = new RegressionTest(m_document, &m_cfg, baseDir, args);
+
+  {
+    QString failureSnapshot = args->getOption("cmp-failures");
+    if (failureSnapshot.isEmpty())
+      failureSnapshot = findMostRecentFailureSnapshot();
+    if (!failureSnapshot.isEmpty())
+      m_regressionTest->setFailureSnapshotConfig(new KConfig(failureSnapshotPrefix + failureSnapshot,
+                                                             KConfig::SimpleConfig),
+                                                 failureSnapshot);
+  }
+
+  if (args->isSet("save-failures")) {
+    QString failureSaver = args->getOption("save-failures");
+    m_regressionTest->setFailureSnapshotSaver(new KConfig(failureSnapshotPrefix + failureSaver,
+                                                          KConfig::SimpleConfig),
+                                              failureSaver);
+  }
+
+  QTimer::singleShot(0, this, SLOT(runTests()));
+}
+
+KateTestApp::~KateTestApp()
+{
+  delete m_document;
+  m_document = 0;
+
+  Q_ASSERT(m_regressionTest == 0);
+}
+
+bool KateTestApp::allTestsSucceeded()
+{
+  return m_regressionTest->allTestsSucceeded();
+}
+
+void KateTestApp::runTests()
+{
+  bool result = false;
+  QStringList tests = m_args->getOptionList("test");
+  // merge testcases specified on command line
+  for (; m_testcaseIndex < m_args->count(); m_testcaseIndex++)
+    tests << m_args->arg(m_testcaseIndex);
+  if (tests.count() > 0) {
+    foreach (const QString &test, tests) {
+      result = m_regressionTest->runTests(test, true);
+      if (!result) break;
+    }
+  } else {
+    result = m_regressionTest->runTests();
+  }
+
+  if (result) {
+    if (m_args->isSet("genoutput")) {
+      printf("\nOutput generation completed.\n");
+    } else {
+      m_regressionTest->printSummary();
+    }
+  }
+}
+
 #include "test_regression.moc"
 // kate: space-indent on; indent-width 2; replace-tabs on;
