@@ -1,4 +1,5 @@
 /* This file is part of the KDE libraries
+   Copyright (C) 2007, 2008 Matthew Woehlke <mw_triad@users.sourceforge.net>
    Copyright (C) 2003 Christoph Cullmann <cullmann@kde.org>
 
    This library is free software; you can redistribute it and/or
@@ -23,7 +24,6 @@
 #include "kateview.h"
 #include "katedocument.h"
 #include "kateschema.h"
-#include "katehistorymodel.h"
 
 #include <math.h>
 
@@ -33,10 +33,7 @@
 #include <kcolorutils.h>
 #include <kcharsets.h>
 #include <klocale.h>
-#include <kfinddialog.h>
-#include <kreplacedialog.h>
 #include <kcomponentdata.h>
-#include <kfind.h>
 #include <kdebug.h>
 
 #include <QtCore/QTextCodec>
@@ -79,17 +76,90 @@ void KateConfig::configEnd ()
 //END
 
 //BEGIN KateDocumentConfig
+KateGlobalConfig *KateGlobalConfig::s_global = 0;
 KateDocumentConfig *KateDocumentConfig::s_global = 0;
 KateViewConfig *KateViewConfig::s_global = 0;
 KateRendererConfig *KateRendererConfig::s_global = 0;
 
+KateGlobalConfig::KateGlobalConfig ()
+{
+  s_global = this;
+
+  // init with defaults from config or really hardcoded ones
+  KConfigGroup cg( KGlobal::config(), "Kate Part Defaults");
+  readConfig (cg);
+}
+
+KateGlobalConfig::~KateGlobalConfig ()
+{
+}
+
+void KateGlobalConfig::readConfig (const KConfigGroup &config)
+{
+  configStart ();
+
+  setProberType ((KEncodingProber::ProberType)config.readEntry("Encoding Prober Type", (int)KEncodingProber::Universal));
+  setFallbackEncoding (config.readEntry("Fallback Encoding", ""));
+
+  configEnd ();
+}
+
+void KateGlobalConfig::writeConfig (KConfigGroup &config)
+{
+  config.writeEntry("Encoding Prober Type", (int)proberType());
+  config.writeEntry("Fallback Encoding", fallbackEncoding());
+}
+
+void KateGlobalConfig::updateConfig ()
+{
+}
+
+void KateGlobalConfig::setProberType (KEncodingProber::ProberType proberType)
+{
+  configStart ();
+  m_proberType = proberType;
+  configEnd ();
+}
+
+const QString &KateGlobalConfig::fallbackEncoding () const
+{
+  return m_fallbackEncoding;
+}
+
+QTextCodec *KateGlobalConfig::fallbackCodec () const
+{
+  if (m_fallbackEncoding.isEmpty())
+      return QTextCodec::codecForName("ISO 8859-15");
+
+  return KGlobal::charsets()->codecForName (m_fallbackEncoding);
+}
+
+bool KateGlobalConfig::setFallbackEncoding (const QString &encoding)
+{
+  QTextCodec *codec;
+  bool found = false;
+  if (encoding.isEmpty())
+  {
+    codec = s_global->fallbackCodec();
+    found = true;
+  }
+  else
+    codec = KGlobal::charsets()->codecForName (encoding, found);
+
+  if (!found || !codec)
+    return false;
+
+  configStart ();
+  m_fallbackEncoding = codec->name();
+  configEnd ();
+  return true;
+}
 KateDocumentConfig::KateDocumentConfig ()
  : m_indentationWidth (2),
    m_tabWidth (8),
    m_tabHandling (tabSmart),
    m_configFlags (0),
    m_wordWrapAt (80),
-   m_proberTypeForEncodingAutoDetection(KEncodingProber::Universal),
    m_tabWidthSet (true),
    m_indentationWidthSet (true),
    m_indentationModeSet (true),
@@ -99,12 +169,14 @@ KateDocumentConfig::KateDocumentConfig ()
    m_configFlagsSet (0xFFFF),
    m_encodingSet (true),
    m_eolSet (true),
+   m_bomSet (true),
    m_allowEolDetectionSet (false),
    m_allowSimpleModeSet (false),
    m_backupFlagsSet (true),
    m_searchDirConfigDepthSet (true),
    m_backupPrefixSet (true),
    m_backupSuffixSet (true),
+   m_onTheFlySpellCheckSet (true),
    m_doc (0)
 {
   s_global = this;
@@ -126,15 +198,16 @@ KateDocumentConfig::KateDocumentConfig (KateDocument *doc)
    m_configFlagsSet (0),
    m_encodingSet (false),
    m_eolSet (false),
+   m_bomSet (false),
    m_allowEolDetectionSet (false),
    m_allowSimpleModeSet (false),
    m_backupFlagsSet (false),
    m_searchDirConfigDepthSet (false),
    m_backupPrefixSet (false),
    m_backupSuffixSet (false),
+   m_onTheFlySpellCheckSet (false),
    m_doc (doc)
 {
-  m_proberTypeForEncodingAutoDetection=s_global->encodingProberType();
 }
 
 KateDocumentConfig::~KateDocumentConfig ()
@@ -163,11 +236,12 @@ void KateDocumentConfig::readConfig (const KConfigGroup &config)
     | KateDocumentConfig::cfSmartHome));
 
   setEncoding (config.readEntry("Encoding", ""));
-  setEncodingProberType ((KEncodingProber::ProberType)config.readEntry("ProberType for Encoding Autodetection", 0));
 
   setEol (config.readEntry("End of Line", 0));
   setAllowEolDetection (config.readEntry("Allow End of Line Detection", true));
 
+  setBom (config.readEntry("BOM",false));
+
   setAllowSimpleMode (config.readEntry("Allow Simple Mode", true));
 
   setBackupFlags (config.readEntry("Backup Config Flags", 1));
@@ -178,6 +252,8 @@ void KateDocumentConfig::readConfig (const KConfigGroup &config)
 
   setBackupSuffix (config.readEntry("Backup Suffix", QString ("~")));
 
+  setOnTheFlySpellCheck(config.readEntry("On-The-Fly Spellcheck", false));
+
   configEnd ();
 }
 
@@ -198,11 +274,12 @@ void KateDocumentConfig::writeConfig (KConfigGroup &config)
   config.writeEntry("Basic Config Flags", configFlags());
 
   config.writeEntry("Encoding", encoding());
-  config.writeEntry("ProberType for Encoding Autodetection", (int)encodingProberType());
 
   config.writeEntry("End of Line", eol());
   config.writeEntry("Allow End of Line Detection", allowEolDetection());
 
+  config.writeEntry("BOM",bom());
+
   config.writeEntry("Allow Simple Mode", allowSimpleMode());
 
   config.writeEntry("Backup Config Flags", backupFlags());
@@ -212,6 +289,8 @@ void KateDocumentConfig::writeConfig (KConfigGroup &config)
   config.writeEntry("Backup Prefix", backupPrefix());
 
   config.writeEntry("Backup Suffix", backupSuffix());
+
+  config.writeEntry("On-The-Fly Spellcheck", onTheFlySpellCheck());
 }
 
 void KateDocumentConfig::updateConfig ()
@@ -420,16 +499,13 @@ QTextCodec *KateDocumentConfig::codec () const
   return s_global->codec ();
 }
 
-bool KateDocumentConfig::setEncoding (const QString &encoding, bool resetDetection)
+bool KateDocumentConfig::setEncoding (const QString &encoding)
 {
   QTextCodec *codec;
   bool found = false;
   if (encoding.isEmpty())
   {
     codec = s_global->codec();
-#ifdef DECODE_DEBUG
-    kWarning()<<"defaulting to " << codec->name();
-#endif
     found=true;
   }
   else
@@ -439,21 +515,13 @@ bool KateDocumentConfig::setEncoding (const QString &encoding, bool resetDetecti
     return false;
 
   configStart ();
-
-  if (resetDetection)
-  {
-    if (!m_encodingSet || encoding.isEmpty())
-      m_proberTypeForEncodingAutoDetection=s_global->encodingProberType();
-    else
-      m_proberTypeForEncodingAutoDetection = KEncodingProber::None;
-  }
   m_encodingSet = true;
   m_encoding = codec->name();
 
+  if (isGlobal())
+    KateGlobal::self()->setDefaultEncoding (m_encoding);
+
   configEnd ();
-#ifdef DECODE_DEBUG
-  kWarning()<<"set to " << codec->name();
-#endif
   return true;
 }
 
@@ -462,22 +530,6 @@ bool KateDocumentConfig::isSetEncoding () const
   return m_encodingSet;
 }
 
-KEncodingProber::ProberType KateDocumentConfig::encodingProberType() const
-{
-  return m_proberTypeForEncodingAutoDetection;
-}
-
-void KateDocumentConfig::setEncodingProberType(KEncodingProber::ProberType proberType)
-{
-  configStart ();
-
-  m_proberTypeForEncodingAutoDetection=proberType;
-
-  configEnd ();
-
-}
-
-
 int KateDocumentConfig::eol () const
 {
   if (m_eolSet || isGlobal())
@@ -508,6 +560,24 @@ void KateDocumentConfig::setEol (int mode)
   configEnd ();
 }
 
+void KateDocumentConfig::setBom (bool bom)
+{
+  configStart ();
+
+  m_bomSet = true;
+  m_bom = bom;
+
+  configEnd ();
+}
+
+bool KateDocumentConfig::bom() const
+{
+    if (m_bomSet || isGlobal())
+      return m_bom;
+    return s_global->bom();
+}
+
+
 bool KateDocumentConfig::allowEolDetection () const
 {
   if (m_allowEolDetectionSet || isGlobal())
@@ -617,6 +687,34 @@ void KateDocumentConfig::setSearchDirConfigDepth (int depth)
   configEnd ();
 }
 
+bool KateDocumentConfig::onTheFlySpellCheck() const
+{
+  if(isGlobal()) {
+    // WARNING: this is slightly hackish, but it's currently the only way to
+    //          do it, see also the KTextEdit class
+    KConfigGroup configGroup(KGlobal::config(), "Spelling");
+    return configGroup.readEntry("checkerEnabledByDefault", false);
+  }
+  if (m_onTheFlySpellCheckSet) {
+    return m_onTheFlySpellCheck;
+  }
+
+  return s_global->onTheFlySpellCheck();
+}
+
+void KateDocumentConfig::setOnTheFlySpellCheck(bool on)
+{
+  configStart ();
+
+  m_onTheFlySpellCheckSet = true;
+  m_onTheFlySpellCheck = on;
+
+  configEnd ();
+}
+
+
+
+
 //END
 
 //BEGIN KateViewConfig
@@ -639,6 +737,8 @@ KateViewConfig::KateViewConfig ()
    m_automaticCompletionInvocationSet (true),
    m_wordCompletionSet (true),
    m_wordCompletionMinimalWordLengthSet (true),
+   m_smartCopyCutSet (true),
+   m_scrollPastEndSet (true),
    m_view (0)
 {
   s_global = this;
@@ -650,6 +750,7 @@ KateViewConfig::KateViewConfig ()
 
 KateViewConfig::KateViewConfig (KateView *view)
  :
+   m_searchFlags (PowerModePlainText),
    m_dynWordWrapSet (false),
    m_dynWordWrapIndicatorsSet (false),
    m_dynWordWrapAlignIndentSet (false),
@@ -667,6 +768,8 @@ KateViewConfig::KateViewConfig (KateView *view)
    m_automaticCompletionInvocationSet (false),
    m_wordCompletionSet (false),
    m_wordCompletionMinimalWordLengthSet (false),
+   m_smartCopyCutSet (false),
+   m_scrollPastEndSet (false),
    m_view (view)
 {
 }
@@ -706,6 +809,8 @@ void KateViewConfig::readConfig ( const KConfigGroup &config)
   setSearchFlags(config.readEntry(KEY_SEARCH_REPLACE_FLAGS,
       IncFromCursor|PowerMatchCase|PowerModePlainText));
 
+  m_maxHistorySize = config.readEntry("Maximum Search History Size", 100);
+
   setDefaultMarkType (config.readEntry( "Default Mark Type", int(KTextEditor::MarkInterface::markType01) ));
 
   setPersistentSelection (config.readEntry( "Persistent Selection", false ));
@@ -717,19 +822,17 @@ void KateViewConfig::readConfig ( const KConfigGroup &config)
   setAutomaticCompletionInvocation (config.readEntry( "Auto Completion", true ));
   setWordCompletion (config.readEntry( "Word Completion", true ));
   setWordCompletionMinimalWordLength (config.readEntry( "Word Completion Minimal Word Length", 3 ));
+  setSmartCopyCut (config.readEntry( "Smart Copy Cut", false ));
+  setScrollPastEnd (config.readEntry( "Scroll Past End", false ));
 
   if (isGlobal()) {
-    QStringList empty;
-
     // Read search pattern history
-    QStringListModel * const patternHistoryModel = KateHistoryModel::getPatternHistoryModel();
-    QStringList patternHistory = config.readEntry(KEY_PATTERN_HISTORY, empty);
-    patternHistoryModel->setStringList(patternHistory);
+    QStringList patternHistory = config.readEntry(KEY_PATTERN_HISTORY, QStringList());
+    m_patternHistoryModel.setStringList(patternHistory);
 
     // Read replacement text history
-    QStringListModel * const replacementHistoryModel = KateHistoryModel::getReplacementHistoryModel();
-    QStringList replacementHistory = config.readEntry(KEY_REPLACEMENT_HISTORY, empty);
-    replacementHistoryModel->setStringList(replacementHistory);
+    QStringList replacementHistory = config.readEntry(KEY_REPLACEMENT_HISTORY, QStringList());
+    m_replacementHistoryModel.setStringList(replacementHistory);
   }
 
   configEnd ();
@@ -755,6 +858,8 @@ void KateViewConfig::writeConfig (KConfigGroup &config)
 
   config.writeEntry(KEY_SEARCH_REPLACE_FLAGS, int(searchFlags()));
 
+  config.writeEntry("Maximum Search History Size", m_maxHistorySize);
+
   config.writeEntry("Default Mark Type", defaultMarkType());
 
   config.writeEntry("Persistent Selection", persistentSelection());
@@ -763,20 +868,22 @@ void KateViewConfig::writeConfig (KConfigGroup &config)
   config.writeEntry( "Word Completion", wordCompletion());
   config.writeEntry( "Word Completion Minimal Word Length", wordCompletionMinimalWordLength());
 
+  config.writeEntry( "Smart Copy Cut", smartCopyCut() );
+  config.writeEntry( "Scroll Past End" , scrollPastEnd() );
+
   config.writeEntry( "Vi Input Mode", viInputMode());
 
   config.writeEntry( "Vi Input Mode Steal Keys", viInputModeStealKeys());
 
   config.writeEntry( "Vi Input Mode Hide Status Bar", viInputModeHideStatusBar());
 
+
   if (isGlobal()) {
     // Write search pattern history
-    QStringListModel * const patternHistoryModel = KateHistoryModel::getPatternHistoryModel();
-    config.writeEntry(KEY_PATTERN_HISTORY, patternHistoryModel->stringList());
+    config.writeEntry(KEY_PATTERN_HISTORY, m_patternHistoryModel.stringList());
 
     // Write replacement text history
-    QStringListModel * const replacementHistoryModel = KateHistoryModel::getReplacementHistoryModel();
-    config.writeEntry(KEY_REPLACEMENT_HISTORY, replacementHistoryModel->stringList());
+    config.writeEntry(KEY_REPLACEMENT_HISTORY, m_replacementHistoryModel.stringList());
   }
 }
 
@@ -978,6 +1085,21 @@ void KateViewConfig::setSearchFlags (long flags)
   configEnd ();
 }
 
+QStringListModel *KateViewConfig::patternHistoryModel()
+{
+  return &m_patternHistoryModel;
+}
+
+int KateViewConfig::maxHistorySize() const
+{
+  return m_maxHistorySize;
+}
+
+QStringListModel *KateViewConfig::replacementHistoryModel()
+{
+  return &m_replacementHistoryModel;
+}
+
 uint KateViewConfig::defaultMarkType () const
 {
   if (m_defaultMarkTypeSet || isGlobal())
@@ -1040,7 +1162,7 @@ void KateViewConfig::setViInputMode (bool on)
 
   // make sure to turn off edits mergin when leaving vi input mode
   if (!on && m_view) {
-    m_view->doc()->setMergeAllEdits(false);
+    m_view->doc()->setUndoMergeAllEdits(false);
   }
 
   configEnd ();
@@ -1146,6 +1268,42 @@ void KateViewConfig::setWordCompletionMinimalWordLength (int length)
   configEnd ();
 }
 
+bool KateViewConfig::smartCopyCut () const
+{
+  if (m_smartCopyCutSet || isGlobal())
+     return m_smartCopyCut;
+
+  return s_global->smartCopyCut();
+}
+
+void KateViewConfig::setSmartCopyCut (bool on)
+{
+  configStart ();
+
+  m_smartCopyCutSet = true;
+  m_smartCopyCut = on;
+
+  configEnd ();
+}
+
+bool KateViewConfig::scrollPastEnd () const
+{
+  if (m_scrollPastEndSet || isGlobal())
+    return m_scrollPastEnd;
+
+  return s_global->scrollPastEnd();
+}
+
+void KateViewConfig::setScrollPastEnd (bool on)
+{
+  configStart ();
+
+  m_scrollPastEndSet = true;
+  m_scrollPastEnd = on;
+
+  configEnd ();
+}
+
 //END
 
 //BEGIN KateRendererConfig
@@ -1165,6 +1323,7 @@ KateRendererConfig::KateRendererConfig ()
    m_tabMarkerColorSet(true),
    m_iconBarColorSet (true),
    m_lineNumberColorSet (true),
+   m_spellingMistakeLineColorSet (true),
    m_templateColorsSet(true),
    m_lineMarkerColorSet (m_lineMarkerColor.size()),
    m_renderer (0)
@@ -1195,6 +1354,7 @@ KateRendererConfig::KateRendererConfig (KateRenderer *renderer)
    m_tabMarkerColorSet(false),
    m_iconBarColorSet (false),
    m_lineNumberColorSet (false),
+   m_spellingMistakeLineColorSet (false),
    m_templateColorsSet(false),
    m_lineMarkerColorSet (m_lineMarkerColor.size()),
    m_renderer (renderer)
@@ -1286,6 +1446,7 @@ void KateRendererConfig::setSchemaInternal( const QString &schema )
   KColorScheme schemeView(QPalette::Active, KColorScheme::View);
   KColorScheme schemeWindow(QPalette::Active, KColorScheme::Window);
   KColorScheme schemeSelection(QPalette::Active, KColorScheme::Selection);
+  KColorScheme schemeTooltip(QPalette::Active, KColorScheme::Tooltip);
   QColor tmp0( schemeView.background().color() );
   QColor tmp1( schemeSelection.background().color() );
   QColor tmp2( schemeView.background(KColorScheme::AlternateBackground).color() );
@@ -1296,6 +1457,7 @@ void KateRendererConfig::setSchemaInternal( const QString &schema )
   QColor tmp5( KColorUtils::shade( tmp0, bgLuma > 0.7 ? -0.35 : 0.3 ) );
   QColor tmp6( schemeWindow.background().color() );
   QColor tmp7( schemeWindow.foreground().color() );
+  QColor tmp8( schemeView.foreground(KColorScheme::NegativeText).color() );
 
   m_backgroundColor = config.readEntry("Color Background", tmp0);
   m_backgroundColorSet = true;
@@ -1313,6 +1475,8 @@ void KateRendererConfig::setSchemaInternal( const QString &schema )
   m_iconBarColorSet = true;
   m_lineNumberColor = config.readEntry("Color Line Number", tmp7);
   m_lineNumberColorSet = true;
+  m_spellingMistakeLineColor = config.readEntry("Color Spelling Mistake Line", tmp8);
+  m_spellingMistakeLineColorSet = true;
 
     // same std colors like in KateDocument::markColor
   QColor mark[7];
@@ -1337,10 +1501,17 @@ void KateRendererConfig::setSchemaInternal( const QString &schema )
   m_fontMetrics = QFontMetrics(m_font);
   m_fontSet = true;
 
-  m_templateBackgroundColor=config.readEntry(QString("Color Template Background"),QColor(0xcc,0xcc,0xcc));
-  m_templateEditablePlaceholderColor = config.readEntry(QString("Color Template Editable Placeholder"),QColor(0xcc,0xff,0xcc));
-  m_templateFocusedEditablePlaceholderColor=config.readEntry(QString("Color Template Focused Editable Placeholder"),QColor(0x66,0xff,0x66));
-  m_templateNotEditablePlaceholderColor=config.readEntry(QString("Color Template Not Editable Placeholder"),QColor(0xff,0xcc,0xcc));
+  m_templateBackgroundColor=config.readEntry(QString("Color Template Background"),
+                                             schemeTooltip.background(KColorScheme::NormalBackground).color());
+
+  m_templateEditablePlaceholderColor = config.readEntry(QString("Color Template Editable Placeholder"),
+                                                        schemeTooltip.background(KColorScheme::NeutralBackground).color());
+
+  m_templateFocusedEditablePlaceholderColor=config.readEntry(QString("Color Template Focused Editable Placeholder"),
+                                                             schemeTooltip.background(KColorScheme::PositiveBackground).color());
+
+  m_templateNotEditablePlaceholderColor=config.readEntry(QString("Color Template Not Editable Placeholder"),
+                                                         schemeTooltip.background(KColorScheme::NegativeBackground).color());
 
   m_templateColorsSet=true;
 }
@@ -1591,6 +1762,24 @@ void KateRendererConfig::setLineNumberColor (const QColor &col)
   configEnd ();
 }
 
+const QColor& KateRendererConfig::spellingMistakeLineColor() const
+{
+  if (m_spellingMistakeLineColorSet || isGlobal())
+    return m_spellingMistakeLineColor;
+
+  return s_global->spellingMistakeLineColor();
+}
+
+void KateRendererConfig::setSpellingMistakeKineColor (const QColor &col)
+{
+  configStart ();
+
+  m_spellingMistakeLineColorSet = true;
+  m_spellingMistakeLineColor = col;
+
+  configEnd ();
+}
+
 bool KateRendererConfig::showIndentationLines () const
 {
   if (m_showIndentationLinesSet || isGlobal())
