@@ -1,21 +1,22 @@
-/* This file is part of the KDE libraries
- * Copyright (C) 2008 Erlend Hamberg <ehamberg@gmail.com>
- * Copyright (C) 2008 Evgeniy Ivanov <powerfox@kde.ru>
+/*  This file is part of the KDE libraries and the Kate part.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) version 3.
+ *  Copyright (C) 2008 Erlend Hamberg <ehamberg@gmail.com>
+ *  Copyright (C) 2008 Evgeniy Ivanov <powerfox@kde.ru>
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Library General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2 of the License, or (at your option) any later version.
  *
- * You should have received a copy of the GNU Library General Public License
- * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ *  This library is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Library General Public License for more details.
+ *
+ *  You should have received a copy of the GNU Library General Public License
+ *  along with this library; see the file COPYING.LIB.  If not, write to
+ *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ *  Boston, MA 02110-1301, USA.
  */
 
 #include "katevikeyparser.h"
@@ -226,8 +227,9 @@ void KateViKeyParser::initKeyTables()
   m_qt2katevi->insert( Qt::Key_Zoom, QString( "zoom" ) );
   m_qt2katevi->insert( Qt::Key_Cancel, QString( "cancel" ) );
 
-  foreach( int code, m_qt2katevi->keys() ) {
-      m_katevi2qt->insert( m_qt2katevi->value( code ), code );
+  for (QHash<int, QString>::const_iterator i = m_qt2katevi->constBegin();
+      i != m_qt2katevi->constEnd(); ++i) {
+    m_katevi2qt->insert( i.value(), i.key() );
   }
 
   m_nameToKeyCode->insert( QString( "invalid" ), -1 );
@@ -473,8 +475,9 @@ void KateViKeyParser::initKeyTables()
   m_nameToKeyCode->insert( QString( " " ), 339 );
   //m_nameToKeyCode->insert( QString( "<" ), 341 );
 
-  foreach( const QString& str, m_nameToKeyCode->keys() ) {
-      m_keyCodeToName->insert( m_nameToKeyCode->value( str ), str );
+  for (QHash<QString, int>::const_iterator i = m_nameToKeyCode->constBegin();
+      i != m_nameToKeyCode->constEnd(); ++i) {
+    m_keyCodeToName->insert( i.value(), i.key() );
   }
 }
 
