@@ -571,12 +571,13 @@ void K3DictSpellingHighlighter::slotAutoDetection()
 	    d->active = true;
     }
     if ( d->active != savedActive ) {
-	if ( d->wordCount > 1 )
+	if ( d->wordCount > 1 ) {
 	    if ( d->active )
 		emit activeChanged( i18n("As-you-type spell checking enabled.") );
 	    else
 		emit activeChanged( i18n( "Too many misspelled words. "
 					  "As-you-type spell checking disabled." ) );
+	}
 	d->completeRehighlightRequired = true;
 	d->rehighlightRequest->start( 100, true );
     }
