@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2009 John Layt <john@layt.net>
+    Copyright 2009, 2010 John Layt <john@layt.net>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -18,6 +18,8 @@
 */
 
 #include "kcalendarsystemcoptic_p.h"
+#include "kcalendarsystemprivate_p.h"
+#include "kcalendarsystemcopticprivate_p.h"
 
 #include "kdebug.h"
 #include "klocale.h"
@@ -25,30 +27,130 @@
 #include <QtCore/QDate>
 #include <QtCore/QCharRef>
 
-class KCalendarSystemCopticPrivate
+// Shared d pointer implementations
+
+KCalendarSystemCopticPrivate::KCalendarSystemCopticPrivate( KCalendarSystemCoptic *q )
+                             :KCalendarSystemPrivate( q )
 {
-public:
-    KCalendarSystemCopticPrivate( KCalendarSystem *q ): q( q )
-    {
+}
+
+KCalendarSystemCopticPrivate::~KCalendarSystemCopticPrivate()
+{
+}
+
+void KCalendarSystemCopticPrivate::initDefaultEraList()
+{
+    QString name, shortName, format;
+    // AM for Anno Martyrum or "Year of the Martyrs"
+    name = i18nc( "Calendar Era: Coptic Era of Martyrs, years > 0, LongFormat", "Anno Martyrum" );
+    shortName = i18nc( "Calendar Era: Coptic Era of Martyrs, years > 0, ShortFormat", "AM" );
+    format = i18nc( "(kdedt-format) Coptic, AM, full era year format used for %EY, e.g. 2000 AM", "%Ey %EC" );
+    addEra( '+', 1, q->epoch(), 1, q->latestValidDate(), name, shortName, format );
+}
+
+int KCalendarSystemCopticPrivate::monthsInYear( int year ) const
+{
+    Q_UNUSED( year )
+    return 13;
+}
+
+int KCalendarSystemCopticPrivate::daysInMonth( int year, int month ) const
+{
+    if ( month == 13 ) {
+        if ( isLeapYear( year ) ) {
+            return 6;
+        } else {
+            return 5;
+        }
     }
 
-    ~KCalendarSystemCopticPrivate()
-    {
+    return 30;
+}
+
+int KCalendarSystemCopticPrivate::daysInYear( int year ) const
+{
+    if ( isLeapYear( year ) ) {
+        return 366;
+    } else {
+        return 365;
+    }
+}
+
+int KCalendarSystemCopticPrivate::daysInWeek() const
+{
+    return 7;
+}
+
+bool KCalendarSystemCopticPrivate::isLeapYear( int year ) const
+{
+    //Uses same rule as Julian but offset by 1 year with year 3 being first leap year
+    if ( year < 1 ) {
+        year = year + 2;
+    } else {
+        year = year + 1;
     }
 
-    KCalendarSystem *q;
-};
+    if ( year % 4 == 0 ) {
+        return true;
+    }
+    return false;
+}
 
-KCalendarSystemCoptic::KCalendarSystemCoptic( const KLocale * locale )
-                     : KCalendarSystem( locale ), d( 0 )
+bool KCalendarSystemCopticPrivate::hasLeapMonths() const
 {
-    setHasYear0(false);
-    setMaxMonthsInYear(13);
+    return false;
+}
+
+bool KCalendarSystemCopticPrivate::hasYearZero() const
+{
+    return false;
+}
+
+int KCalendarSystemCopticPrivate::maxDaysInWeek() const
+{
+    return 7;
+}
+
+int KCalendarSystemCopticPrivate::maxMonthsInYear() const
+{
+    return 13;
+}
+
+int KCalendarSystemCopticPrivate::earliestValidYear() const
+{
+    return 1;
+}
+
+int KCalendarSystemCopticPrivate::latestValidYear() const
+{
+    return 9999;
+}
+
+KCalendarSystemCoptic::KCalendarSystemCoptic( const KLocale *locale )
+                     : KCalendarSystem( *new KCalendarSystemCopticPrivate( this ), KSharedConfig::Ptr(), locale ),
+                       dont_use( 0 )
+{
+    d_ptr->initialiseEraList( calendarType() );
+}
+
+KCalendarSystemCoptic::KCalendarSystemCoptic( const KSharedConfig::Ptr config, const KLocale *locale )
+                     : KCalendarSystem( *new KCalendarSystemCopticPrivate( this ), config, locale ),
+                       dont_use( 0 )
+{
+    d_ptr->initialiseEraList( calendarType() );
+}
+
+KCalendarSystemCoptic::KCalendarSystemCoptic( KCalendarSystemCopticPrivate &dd,
+                                              const KSharedConfig::Ptr config, const KLocale *locale )
+                     : KCalendarSystem( dd, config, locale ),
+                       dont_use( 0 )
+{
+    d_ptr->initialiseEraList( calendarType() );
 }
 
 KCalendarSystemCoptic::~KCalendarSystemCoptic()
 {
-    delete d;
+    delete dont_use;
 }
 
 QString KCalendarSystemCoptic::calendarType() const
@@ -74,29 +176,13 @@ QDate KCalendarSystemCoptic::latestValidDate() const
 {
     // Set to last day of year 9999 until confirm date formats & widgets support > 9999
     //9999-12-30
-    //100283-08-29 AD Julian
+    //10283-08-29 AD Julian
     return QDate::fromJulianDay( 5477164 );
 }
 
 bool KCalendarSystemCoptic::isValid( int year, int month, int day ) const
 {
-    if ( year < 0 || year > 9999 ) {
-        return false;
-    }
-
-    if ( month < 1 || month > 13 ) {
-        return false;
-    }
-
-    if ( month == 13 ) {
-        if ( isLeapYear( year ) ) {
-            return ( day >= 1 && day <= 6 );
-        } else {
-            return ( day >= 1 && day <= 5 );
-        }
-    }
-
-    return ( day >= 1 && day <= 30 );
+    return KCalendarSystem::isValid( year, month, day );
 }
 
 bool KCalendarSystemCoptic::isValid( const QDate &date ) const
@@ -147,8 +233,7 @@ QDate KCalendarSystemCoptic::addDays( const QDate &date, int ndays ) const
 
 int KCalendarSystemCoptic::monthsInYear( const QDate &date ) const
 {
-    Q_UNUSED( date )
-    return 13;
+    return KCalendarSystem::monthsInYear( date );
 }
 
 int KCalendarSystemCoptic::weeksInYear( const QDate &date ) const
@@ -163,40 +248,17 @@ int KCalendarSystemCoptic::weeksInYear( int year ) const
 
 int KCalendarSystemCoptic::daysInYear( const QDate &date ) const
 {
-    if ( !isValid( date ) ) {
-        return -1;
-    }
-
-    if ( isLeapYear( date ) ) {
-        return 366;
-    } else {
-        return 365;
-    }
+    return KCalendarSystem::daysInYear( date );
 }
 
 int KCalendarSystemCoptic::daysInMonth( const QDate &date ) const
 {
-    if ( !isValid( date ) ) {
-        return -1;
-    }
-
-    int m = month( date );
-
-    if ( m == 13 ) {
-        if ( isLeapYear( year( date ) ) ) {
-            return 6;
-        } else {
-            return 5;
-        }
-    }
-
-    return 30;
+    return KCalendarSystem::daysInMonth( date );
 }
 
 int KCalendarSystemCoptic::daysInWeek( const QDate &date ) const
 {
-    Q_UNUSED( date );
-    return 7;
+    return KCalendarSystem::daysInWeek( date );
 }
 
 int KCalendarSystemCoptic::dayOfYear( const QDate &date ) const
@@ -216,19 +278,7 @@ int KCalendarSystemCoptic::weekNumber( const QDate &date, int * yearNum ) const
 
 bool KCalendarSystemCoptic::isLeapYear( int year ) const
 {
-    //Uses same rule as Julian but offset by 1 year with year 3 being first leap year
-    int y;
-
-    if ( year < 1 ) {
-        y = year + 2;
-    } else {
-        y = year + 1;
-    }
-
-    if ( y % 4 == 0 ) {
-        return true;
-    }
-    return false;
+    return KCalendarSystem::isLeapYear( year );
 }
 
 bool KCalendarSystemCoptic::isLeapYear( const QDate &date ) const
