@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2009 John Layt <john@layt.net>
+    Copyright 2009, 2010 John Layt <john@layt.net>
  
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -20,20 +20,170 @@
 // Derived gregorian kde calendar class
 
 #include "kcalendarsystemgregorianproleptic_p.h"
+#include "kcalendarsystemgregorianprolepticprivate_p.h"
+#include "kcalendarera_p.h"
 
 #include "kdebug.h"
 #include "klocale.h"
+#include "kglobal.h"
+#include "kconfiggroup.h"
 
 #include <QtCore/QDate>
 #include <QtCore/QCharRef>
 
-KCalendarSystemGregorianProleptic::KCalendarSystemGregorianProleptic( const KLocale * locale )
-                         : KCalendarSystem( locale ), d( 0 )
+// Shared d pointer base class definitions
+
+KCalendarSystemGregorianProlepticPrivate::KCalendarSystemGregorianProlepticPrivate( KCalendarSystemGregorianProleptic *q )
+                                         :KCalendarSystemPrivate( q )
 {
 }
 
+KCalendarSystemGregorianProlepticPrivate::~KCalendarSystemGregorianProlepticPrivate()
+{
+}
+
+// Dummy version using Gregorian as an example
+// This method MUST be re-implemented in any new Calendar System
+void KCalendarSystemGregorianProlepticPrivate::initDefaultEraList()
+{
+    QString name, shortName, format;
+
+    KConfigGroup cg( config(), QString( "KCalendarSystem %1" ).arg( q->calendarType() ) );
+    m_useCommonEra = cg.readEntry( "UseCommonEra", false );
+
+    if ( m_useCommonEra ) {
+        name = i18nc( "Calendar Era: Gregorian Common Era, years < 0, LongFormat", "Before Common Era" );
+        shortName = i18nc( "Calendar Era: Gregorian Common Era, years < 0, ShortFormat", "BCE" );
+    } else {
+        name = i18nc( "Calendar Era: Gregorian Christian Era, years < 0, LongFormat", "Before Christ" );
+        shortName = i18nc( "Calendar Era: Gregorian Christian Era, years < 0, ShortFormat", "BC" );
+    }
+    format = i18nc( "(kdedt-format) Gregorian, BC, full era year format used for %EY, e.g. 2000 BC", "%Ey %EC" );
+    addEra( '-', 1, q->epoch().addDays( -1 ), -1, q->earliestValidDate(), name, shortName, format );
+
+    if ( m_useCommonEra ) {
+        name = i18nc( "Calendar Era: Gregorian Common Era, years > 0, LongFormat", "Common Era" );
+        shortName = i18nc( "Calendar Era: Gregorian Common Era, years > 0, ShortFormat", "CE" );
+    } else {
+        name = i18nc( "Calendar Era: Gregorian Christian Era, years > 0, LongFormat", "Anno Domini" );
+        shortName = i18nc( "Calendar Era: Gregorian Christian Era, years > 0, ShortFormat", "AD" );
+    }
+    format = i18nc( "(kdedt-format) Gregorian, AD, full era year format used for %EY, e.g. 2000 AD", "%Ey %EC" );
+    addEra( '+', 1, q->epoch(), 1, q->latestValidDate(), name, shortName, format );
+}
+
+int KCalendarSystemGregorianProlepticPrivate::monthsInYear( int year ) const
+{
+    Q_UNUSED( year )
+    return 12;
+}
+
+int KCalendarSystemGregorianProlepticPrivate::daysInMonth( int year, int month ) const
+{
+    if ( month == 2 ) {
+        if ( isLeapYear( year ) ) {
+            return 29;
+        } else {
+            return 28;
+        }
+    }
+
+    if ( month == 4 || month == 6 || month == 9 || month == 11 ) {
+        return 30;
+    }
+
+    return 31;
+}
+
+int KCalendarSystemGregorianProlepticPrivate::daysInYear( int year ) const
+{
+    if ( isLeapYear( year ) ) {
+        return 366;
+    } else {
+        return 365;
+    }
+}
+
+int KCalendarSystemGregorianProlepticPrivate::daysInWeek() const
+{
+    return 7;
+}
+
+bool KCalendarSystemGregorianProlepticPrivate::isLeapYear( int year ) const
+{
+    if ( !hasYearZero() && year < 1 ) {
+        year = year + 1;
+    }
+
+    if ( year % 4 == 0 ) {
+        if ( year % 100 != 0 ) {
+            return true;
+        } else if ( year % 400 == 0 ) {
+            return true;
+        }
+    }
+
+    return false;
+}
+
+bool KCalendarSystemGregorianProlepticPrivate::hasLeapMonths() const
+{
+    return false;
+}
+
+bool KCalendarSystemGregorianProlepticPrivate::hasYearZero() const
+{
+    return false;
+}
+
+int KCalendarSystemGregorianProlepticPrivate::maxDaysInWeek() const
+{
+    return 7;
+}
+
+int KCalendarSystemGregorianProlepticPrivate::maxMonthsInYear() const
+{
+    return 12;
+}
+
+int KCalendarSystemGregorianProlepticPrivate::earliestValidYear() const
+{
+    return -4713;
+}
+
+int KCalendarSystemGregorianProlepticPrivate::latestValidYear() const
+{
+    return 9999;
+}
+
+
+KCalendarSystemGregorianProleptic::KCalendarSystemGregorianProleptic( const KLocale *locale )
+                                 : KCalendarSystem( *new KCalendarSystemGregorianProlepticPrivate( this ), KSharedConfig::Ptr(), locale ),
+                                 dont_use( 0 )
+{
+    d_ptr->initialiseEraList( calendarType() );
+}
+
+KCalendarSystemGregorianProleptic::KCalendarSystemGregorianProleptic( const KSharedConfig::Ptr config,
+                                                                      const KLocale *locale )
+                                 : KCalendarSystem( *new KCalendarSystemGregorianProlepticPrivate( this ), config, locale ),
+                                   dont_use( 0 )
+{
+    d_ptr->initialiseEraList( calendarType() );
+}
+
+KCalendarSystemGregorianProleptic::KCalendarSystemGregorianProleptic( KCalendarSystemGregorianProlepticPrivate &dd,
+                                                                      const KSharedConfig::Ptr config,
+                                                                      const KLocale *locale )
+                                 : KCalendarSystem( dd, config, locale ),
+                                   dont_use( 0 )
+{
+    d_ptr->initialiseEraList( calendarType() );
+}
+
 KCalendarSystemGregorianProleptic::~KCalendarSystemGregorianProleptic()
 {
+    delete dont_use;
 }
 
 QString KCalendarSystemGregorianProleptic::calendarType() const
@@ -48,7 +198,7 @@ QDate KCalendarSystemGregorianProleptic::epoch() const
 
 QDate KCalendarSystemGregorianProleptic::earliestValidDate() const
 {
-    // 1 Jan 4713 BC, no year zero
+    // Gregorian 1 Jan 4713 BC, no year zero
     return QDate::fromJulianDay( 38 );
 }
 
@@ -62,27 +212,7 @@ QDate KCalendarSystemGregorianProleptic::latestValidDate() const
 
 bool KCalendarSystemGregorianProleptic::isValid( int year, int month, int day ) const
 {
-    if ( year < -4713 || year > 9999 || year == 0 ) {
-        return false;
-    }
-
-    if ( month < 1 || month > 12 ) {
-        return false;
-    }
-
-    if ( month == 2 ) {
-        if ( isLeapYear( year ) ) {
-            return ( day >= 1 && day <= 29 );
-        } else {
-            return ( day >= 1 && day <= 28 );
-        }
-    }
-
-    if ( month == 4 || month == 6 || month == 9 || month == 11  ) {
-        return ( day >= 1 && day <= 30 );
-    }
-
-    return ( day >= 1 && day <= 31 );
+    return KCalendarSystem::isValid( year, month, day );
 }
 
 bool KCalendarSystemGregorianProleptic::isValid( const QDate &date ) const
@@ -96,9 +226,9 @@ bool KCalendarSystemGregorianProleptic::setDate( QDate &date, int year, int mont
 }
 
 // Deprecated
-bool KCalendarSystemGregorianProleptic::setYMD( QDate &date, int y, int m, int d ) const
+bool KCalendarSystemGregorianProleptic::setYMD( QDate &date, int year, int month, int day ) const
 {
-    return KCalendarSystem::setDate( date, y, m, d );
+    return KCalendarSystem::setYMD( date, year, month, day );
 }
 
 int KCalendarSystemGregorianProleptic::year( const QDate &date ) const
@@ -133,8 +263,7 @@ QDate KCalendarSystemGregorianProleptic::addDays( const QDate &date, int ndays )
 
 int KCalendarSystemGregorianProleptic::monthsInYear( const QDate &date ) const
 {
-    Q_UNUSED( date )
-    return 12;
+    return KCalendarSystem::monthsInYear( date );
 }
 
 int KCalendarSystemGregorianProleptic::weeksInYear( const QDate &date ) const
@@ -149,44 +278,17 @@ int KCalendarSystemGregorianProleptic::weeksInYear( int year ) const
 
 int KCalendarSystemGregorianProleptic::daysInYear( const QDate &date ) const
 {
-    if ( !isValid( date ) ) {
-        return -1;
-    }
-
-    if ( isLeapYear( date ) ) {
-        return 366;
-    } else {
-        return 365;
-    }
+    return KCalendarSystem::daysInYear( date );
 }
 
 int KCalendarSystemGregorianProleptic::daysInMonth( const QDate &date ) const
 {
-    if ( !isValid( date ) ) {
-        return -1;
-    }
-
-    int m = month( date );
-
-    if ( m == 2 ) {
-        if ( isLeapYear( year( date ) ) ) {
-            return 29;
-        } else {
-            return 28;
-        }
-    }
-
-    if ( m == 4 || m == 6 || m == 9 || m == 11 ) {
-        return 30;
-    }
-
-    return 31;
+    return KCalendarSystem::daysInMonth( date );
 }
 
 int KCalendarSystemGregorianProleptic::daysInWeek( const QDate &date ) const
 {
-    Q_UNUSED( date );
-    return 7;
+    return KCalendarSystem::daysInWeek( date );
 }
 
 int KCalendarSystemGregorianProleptic::dayOfYear( const QDate &date ) const
@@ -206,22 +308,7 @@ int KCalendarSystemGregorianProleptic::weekNumber( const QDate &date, int * year
 
 bool KCalendarSystemGregorianProleptic::isLeapYear( int year ) const
 {
-    int y;
-
-    if ( year < 1 ) {
-        y = year + 1;
-    } else {
-        y = year;
-    }
-
-    if ( y % 4 == 0 ) {
-        if ( y % 100 != 0 ) {
-            return true;
-        } else if ( y % 400 == 0 ) {
-            return true;
-        }
-    }
-    return false;
+    return KCalendarSystem::isLeapYear( year );
 }
 
 bool KCalendarSystemGregorianProleptic::isLeapYear( const QDate &date ) const
@@ -477,6 +564,8 @@ bool KCalendarSystemGregorianProleptic::isProleptic() const
 
 bool KCalendarSystemGregorianProleptic::julianDayToDate( int jd, int &year, int &month, int &day ) const
 {
+    Q_D( const KCalendarSystemGregorianProleptic );
+
     // Formula from The Calendar FAQ by Claus Tondering
     // http://www.tondering.dk/claus/cal/node3.html#SECTION003161000000000000000
     // NOTE: Coded from scratch from mathematical formulas, not copied from
@@ -485,24 +574,26 @@ bool KCalendarSystemGregorianProleptic::julianDayToDate( int jd, int &year, int
     int a = jd + 32044;
     int b = ( ( 4 * a ) + 3 ) / 146097;
     int c = a - ( ( 146097 * b ) / 4 );
-    int d = ( ( 4 * c ) + 3 ) / 1461;
-    int e = c - ( ( 1461 * d ) / 4 );
+    int dd = ( ( 4 * c ) + 3 ) / 1461;
+    int e = c - ( ( 1461 * dd ) / 4 );
     int m = ( ( 5 * e ) + 2 ) / 153;
     day = e - ( ( (153 * m ) + 2 ) / 5 ) + 1;
     month = m + 3 - ( 12 * ( m / 10 ) );
-    year = ( 100 * b ) + d - 4800 + ( m / 10 );
+    year = ( 100 * b ) + dd - 4800 + ( m / 10 );
 
     // If year is -ve then is BC.  In Gregorian there is no year 0, but the maths
     // is easier if we pretend there is, so internally year of 0 = 1BC = -1 outside
-    if ( year < 1 ) {
+    // Check for Year 0 support as some Gregorian based calendars do have it, e.g. Thai and ISO
+    if ( !d->hasYearZero() && year < 1 ) {
         year = year - 1;
     }
-
     return true;
 }
 
 bool KCalendarSystemGregorianProleptic::dateToJulianDay( int year, int month, int day, int &jd ) const
 {
+    Q_D( const KCalendarSystemGregorianProleptic );
+
     // Formula from The Calendar FAQ by Claus Tondering
     // http://www.tondering.dk/claus/cal/node3.html#SECTION003161000000000000000
     // NOTE: Coded from scratch from mathematical formulas, not copied from
@@ -510,8 +601,9 @@ bool KCalendarSystemGregorianProleptic::dateToJulianDay( int year, int month, in
 
     // If year is -ve then is BC.  In Gregorian there is no year 0, but the maths
     // is easier if we pretend there is, so internally year of -1 = 1BC = 0 internally
+    // Check for Year 0 support as some Gregorian based calendars do have it, e.g. Thai and ISO
     int y;
-    if ( year < 1 ) {
+    if ( !d->hasYearZero() && year < 1 ) {
         y = year + 1;
     } else {
         y = year;
