@@ -83,7 +83,7 @@
 #include "kcomponentdata.h"
 
 #ifdef Q_OS_SOLARIS
-// For the purposes of KDebug Solaris has a GNU-libc-compatible 
+// For the purposes of KDebug Solaris has a GNU-libc-compatible
 // backtrace() function. This isn't detected by the CMake checks
 // normally (check_function_exists fails), but we know it's there.
 // For better results, we would use walk_context(), but that's
@@ -93,6 +93,10 @@
 
 #ifdef HAVE_BACKTRACE
 #include <execinfo.h>
+#ifdef __GNUC__
+#define HAVE_BACKTRACE_DEMANGLE
+#include <cxxabi.h>
+#endif
 #endif
 
 #include "kdebugdbusiface_p.h"
@@ -216,10 +220,10 @@ struct KDebugPrivate
         Q_ASSERT(int(QtDebugMsg) == 0);
         Q_ASSERT(int(QtFatalMsg) == 3);
 
-        // Create the dbus interface if it has not been created yet
-        // But only register to DBus if we are in a process with a dbus event loop,
+        // Create the D-Bus interface if it has not been created yet
+        // But only register to D-Bus if we are in a process with a D-Bus event loop,
         // otherwise introspection will just hang.
-        // Examples of processes without a dbus event loop: kioslaves and the main kdeinit process.
+        // Examples of processes without a D-Bus event loop: kioslaves and the main kdeinit process.
         //
         // How to know that we have a real event loop? That's tricky.
         // We could delay registration in kDebugDBusIface with a QTimer, but
@@ -273,7 +277,7 @@ struct KDebugPrivate
         uint lineNumber=0;
 
         while (!file.atEnd()) {
-            QByteArray line = file.readLine().trimmed();
+            const QByteArray line = file.readLine().trimmed();
             ++lineNumber;
             if (line.isEmpty())
                 continue;
@@ -397,19 +401,20 @@ struct KDebugPrivate
             return cache.find(0);
         }
 
-        if (cache.count() <= 1) { // empty or containing only entry "0"
+        if (!cache.contains(0)) {
             loadAreaNames(); // fills 'cache'
         }
 
         Cache::Iterator it = cache.find(num);
         if (it == cache.end()) {
             // unknown area
+            Q_ASSERT(cache.contains(0));
             return cache.find(0);
         }
 
-        if (num == 0) { // area 0 is special, it becomes the named area "appname"
+        if (num == 0 && type == QtDebugMsg) { // area 0 is special, it becomes the named area "appname"
             static bool s_firstDebugFromApplication = true;
-            if (s_firstDebugFromApplication) {
+            if (s_firstDebugFromApplication && !m_disableAll) {
                 s_firstDebugFromApplication = false;
                 writeGroupForNamedArea(it->name, enableByDefault);
             }
@@ -509,8 +514,13 @@ struct KDebugPrivate
         static bool printProcessInfo = (qgetenv("KDE_DEBUG_NOPROCESSINFO").isEmpty());
         static bool printAreaName = (qgetenv("KDE_DEBUG_NOAREANAME").isEmpty());
         static bool printMethodName = (qgetenv("KDE_DEBUG_NOMETHODNAME").isEmpty());
+        static bool printTimeStamp = !(qgetenv("KDE_DEBUG_TIMESTAMP").isEmpty());
         QByteArray programName;
         s = s.nospace();
+        if (printTimeStamp) {
+            s << qPrintable(QDateTime::currentDateTime().time().toString());
+            s << ' ';
+        }
         if (printProcessInfo) {
             programName = cache.value(0).name;
             if (programName.isEmpty()) {
@@ -615,6 +625,7 @@ struct KDebugPrivate
         case SyslogOutput:
             s = setupSyslogWriter(type);
             break;
+        case Unknown: // don't want kdelibs debug output in Qt-only programs with no componentdata (-> no kdebugrc)
         case NoOutput:
             s = QDebug(&devnull);
             return s; //no need to take the time to "print header" if we don't want to output anyway
@@ -641,6 +652,7 @@ struct KDebugPrivate
             const QString key = QString::fromLatin1("InfoOutput");
             if (!cg.hasKey(key)) {
                 cg.writeEntry(key, int(enabled ? KDebugPrivate::QtOutput : KDebugPrivate::NoOutput));
+                cg.sync();
             }
         }
     }
@@ -661,6 +673,36 @@ struct KDebugPrivate
 
 K_GLOBAL_STATIC(KDebugPrivate, kDebug_data)
 
+#ifdef HAVE_BACKTRACE
+static QString maybeDemangledName(char *name)
+{
+#ifdef HAVE_BACKTRACE_DEMANGLE
+    const int len = strlen(name);
+    QByteArray in = QByteArray::fromRawData(name, len);
+    const int mangledNameStart = in.indexOf("(_");
+    if (mangledNameStart >= 0) {
+        const int mangledNameEnd = in.indexOf('+', mangledNameStart + 2);
+        if (mangledNameEnd >= 0) {
+            int status;
+            // if we forget about this line and the one that undoes its effect we don't change the
+            // internal data of the QByteArray::fromRawData() ;)
+            name[mangledNameEnd] = 0;
+            char *demangled = abi::__cxa_demangle(name + mangledNameStart + 1, 0, 0, &status);
+            name[mangledNameEnd] = '+';
+            if (demangled) {
+                QString ret = QString::fromLatin1(name, mangledNameStart + 1) +
+                              QString::fromLatin1(demangled) +
+                              QString::fromLatin1(name + mangledNameEnd, len - mangledNameEnd);
+                free(demangled);
+                return ret;
+            }
+        }
+    }
+#endif
+    return QString::fromLatin1(name);
+}
+#endif
+
 QString kRealBacktrace(int levels)
 {
     QString s;
@@ -676,9 +718,8 @@ QString kRealBacktrace(int levels)
     s = QLatin1String("[\n");
 
     for (int i = 0; i < n; ++i)
-        s += QString::number(i) +
-             QLatin1String(": ") +
-             QLatin1String(strings[i]) + QLatin1String("\n");
+        s += QString::number(i) + QLatin1String(": ") +
+             maybeDemangledName(strings[i]) + QLatin1Char('\n');
     s += QLatin1String("]\n");
     if (strings)
         free (strings);
