@@ -234,7 +234,7 @@ static QByteArray uriListData(const KUrl::List& urls)
     return uriListData;
 }
 
-static const char* s_kdeUriListMime = "application/x-kde4-urilist";
+static const char s_kdeUriListMime[] = "application/x-kde4-urilist";
 
 void KUrl::List::populateMimeData( QMimeData* mimeData,
                                    const KUrl::MetaDataMap& metaData,
@@ -523,10 +523,10 @@ KUrl::KUrl( const KUrl& _u, const QString& _rel_url )
   else if ( rUrl[0] == '#' )
   {
     *this = _u;
-    QString strRef_encoded = rUrl.mid(1);
+    QByteArray strRef_encoded = rUrl.mid(1).toLatin1();
     if ( strRef_encoded.isNull() )
         strRef_encoded = ""; // we know there was an (empty) html ref, we saw the '#'
-    setFragment( strRef_encoded );
+    setEncodedFragment( strRef_encoded );
   }
   else if ( isRelativeUrl( rUrl ) )
   {
@@ -616,6 +616,14 @@ bool KUrl::equals( const KUrl &_u, const EqualsOptions& options ) const
   {
     QString path1 = path((options & CompareWithoutTrailingSlash) ? RemoveTrailingSlash : LeaveTrailingSlash);
     QString path2 = _u.path((options & CompareWithoutTrailingSlash) ? RemoveTrailingSlash : LeaveTrailingSlash);
+
+    if (options & AllowEmptyPath) {
+        if (path1 == QLatin1String("/"))
+            path1.clear();
+        if (path2 == QLatin1String("/"))
+            path2.clear();
+    }
+
 #ifdef Q_WS_WIN
     const bool bLocal1 = isLocalFile();
     const bool bLocal2 = _u.isLocalFile();
@@ -836,7 +844,7 @@ void KUrl::setEncodedPathAndQuery( const QString& _txt )
   }
   else
   {
-    setPath( QUrl::fromPercentEncoding( _txt.toLatin1() ).left( pos ) );
+    setPath( QUrl::fromPercentEncoding(_txt.toLatin1().left(pos)) );
     _setQuery( _txt.right( _txt.length() - pos - 1 ) );
   }
 }
@@ -1231,7 +1239,7 @@ QString KUrl::fileName( const DirectoryOptions& options ) const
   Q_ASSERT( options != 0 ); //Disallow options == false
   QString fname;
   if (hasSubUrl()) { // If we have a suburl, then return the filename from there
-    KUrl::List list = KUrl::split(*this);
+    const KUrl::List list = KUrl::split(*this);
     return list.last().fileName(options);
   }
   const QString path = this->path();
@@ -1466,7 +1474,7 @@ QString KUrl::htmlRef() const
       return QUrl::fromPercentEncoding( ref().toLatin1() );
   }
 
-  List lst = split( *this );
+  const List lst = split( *this );
   return QUrl::fromPercentEncoding( (*lst.begin()).ref().toLatin1() );
 }
 
@@ -1477,7 +1485,7 @@ QString KUrl::encodedHtmlRef() const
     return ref();
   }
 
-  List lst = split( *this );
+  const List lst = split( *this );
   return (*lst.begin()).ref();
 }
 
@@ -1503,7 +1511,7 @@ bool KUrl::hasHTMLRef() const
     return hasRef();
   }
 
-  List lst = split( *this );
+  const List lst = split( *this );
   return (*lst.begin()).hasRef();
 }
 
@@ -1573,26 +1581,18 @@ bool urlcmp( const QString& _url1, const QString& _url2 )
 
 bool urlcmp( const QString& _url1, const QString& _url2, const KUrl::EqualsOptions& _options )
 {
-    QUrl u1( _url1 );
-    QUrl u2( _url2 );
-    QUrl::FormattingOptions options = QUrl::None;
-    if ( _options & KUrl::CompareWithoutTrailingSlash )
-        options |= QUrl::StripTrailingSlash;
-    if ( _options & KUrl::CompareWithoutFragment )
-        options |= QUrl::RemoveFragment;
-#ifdef Q_WS_WIN
-    if ( ::isLocalFile( u1 ) && ::isLocalFile( u2 ) )
-      return 0 == QString::compare( u1.toString( options ), u2.toString( options ), Qt::CaseInsensitive );
-#endif
-    return u1.toString( options ) == u2.toString( options );
+    // Both empty ?
+    if (_url1.isEmpty() && _url2.isEmpty())
+        return true;
+    // Only one empty ?
+    if (_url1.isEmpty() || _url2.isEmpty())
+        return false;
 
-#if 0
-  // Both empty ?
-  if ( _url1.isEmpty() && _url2.isEmpty() )
-    return true;
-  // Only one empty ?
-  if ( _url1.isEmpty() || _url2.isEmpty() )
-    return false;
+    KUrl u1(_url1);
+    KUrl u2(_url2);
+    return u1.equals(u2, _options);
+
+#if 0 // kde3 code that supported nested urls
 
   KUrl::List list1 = KUrl::split( _url1 );
   KUrl::List list2 = KUrl::split( _url2 );
@@ -1726,6 +1726,8 @@ void KUrl::setPath( const QString& _path )
     if ( scheme().isEmpty() )
         setScheme( QLatin1String( "file" ) );
     QString path = KShell::tildeExpand( _path );
+    if (path.isEmpty())
+        path = _path;
 #ifdef Q_WS_WIN
     const int len = path.length();
     if( len == 2 && IS_LETTER(path[0]) && path[1] == QLatin1Char(':') )
