@@ -31,9 +31,6 @@
 class QStringList;
 class QMimeData;
 
-// Temporary 4.2.x hack to avoid compilation problems in other modules
-#define KURL_HAS_DECODEOPTIONS
-
 class KUrlPrivate;
 
 // maybe we should encapsulate QUrl instead of inheriting from it.
@@ -397,7 +394,7 @@ public:
   enum AdjustPathOption
   {
     /**
-     * strips any trailing '/'
+     * strips a trailing '/', except when the path is already just "/".
      */
     RemoveTrailingSlash,
 
@@ -544,20 +541,21 @@ public:
   QString query() const;
 
   /**
+   * Returns the reference (or "fragment") of the URL.
    * The reference is @em never decoded automatically.
    * @return the undecoded reference, or QString() if there is none
    */
   QString ref() const;
 
   /**
-   * Sets the reference part (everything after '#').
+   * Sets the reference/fragment part (everything after '#').
    * If you have an encoded fragment already (as a QByteArray), you can call setFragment directly.
-   * @param fragment the encoded reference (or QString() to remove it).
+   * @param fragment the unencoded reference (or QString() to remove it).
    */
   void setRef( const QString& fragment );
 
   /**
-   * Checks whether the URL has a reference part.
+   * Checks whether the URL has a reference/fragment part.
    * @return true if the URL has a reference part. In a URL like
    *         http://www.kde.org/kdebase.tar#tar:/README it would
    *         return true, too.
@@ -821,7 +819,7 @@ public:
    * @param trailing use to add or remove a trailing slash to/from the path. see adjustPath.
    * @since 4.2
    */
-  QString pathOrUrl(AdjustPathOption trailing) const; // KDE5: merge with above
+  QString pathOrUrl(AdjustPathOption trailing) const; // KDE5: merge with above. Rename to toUrlOrLocalFile?
 
   /**
    * Returns the URL as a string, using the standard conventions for mime data
@@ -873,18 +871,35 @@ public:
 
 
   /**
-   * flags to be used in url compartators function like equal, or urlcmp
-   **/
+   * Flags to be used in URL comparison functions like equals, or urlcmp
+   */
   enum EqualsOption
   {
     /**
-     * ignore trailing '/' characters
+     * ignore trailing '/' characters. The paths "dir" and "dir/" are treated the same.
+     * Note however, that by default, the paths "" and "/" are not the same
+     * (For instance ftp://user@host redirects to ftp://user@host/home/user (on a linux server),
+     * while ftp://user@host/ is the root dir).
+     * This is also why path(RemoveTrailingSlash) for "/" returns "/" and not "".
+     *
+     * When dealing with web pages however, you should also set AllowEmptyPath so that
+     * no path and "/" are considered equal.
      */
     CompareWithoutTrailingSlash = 0x01,
     /**
      * disables comparison of HTML-style references.
      */
-    CompareWithoutFragment = 0x02
+    CompareWithoutFragment = 0x02,
+    /**
+     * Treat a URL with no path as equal to a URL with a path of "/",
+     * when CompareWithoutTrailingSlash is set.
+     * Example:
+     * KUrl::urlcmp("http://www.kde.org", "http://www.kde.org/", KUrl::CompareWithoutTrailingSlash | KUrl::AllowEmptyPath)
+     * returns true.
+     * This option is ignored if CompareWithoutTrailingSlash isn't set.
+     * @since 4.5
+     */
+    AllowEmptyPath = 0x04
   };
   Q_DECLARE_FLAGS(EqualsOptions,EqualsOption)
 
@@ -895,7 +910,7 @@ public:
    * @return True if both urls are the same. If at least one of the urls is invalid,
    * false is returned.
    * @see operator==. This function should be used if you want to
-   * ignore trailing '/' characters.
+   * set additional options, like ignoring trailing '/' characters.
    */
   bool equals( const KUrl &u, const EqualsOptions& options=0 ) const;
 
@@ -1007,7 +1022,7 @@ public:
    * everything is 7 bit (ascii) after being percent-encoded.
    */
   static KDE_DEPRECATED QString encode_string(const QString &str) {
-      return QLatin1String( QUrl::toPercentEncoding( str ) ); //krazy:exclude=qclasses
+    return QString::fromLatin1( QUrl::toPercentEncoding( str ).constData() ); //krazy:exclude=qclasses
   }
 
   /**
@@ -1022,7 +1037,7 @@ public:
    *
    */
   static KDE_DEPRECATED QString encode_string_no_slash(const QString &str) {
-      return QString::fromLatin1( QUrl::toPercentEncoding( str, "/" ) ); //krazy:exclude=qclasses
+      return QString::fromLatin1( QUrl::toPercentEncoding( str, "/" ).constData() ); //krazy:exclude=qclasses
   }
 
   /**
@@ -1045,6 +1060,17 @@ public:
    *
    * Returns whether '_url' is likely to be a "relative" URL instead of
    * an "absolute" URL.
+   *
+   * This is mostly meant for KUrl(url, relativeUrl).
+   *
+   * If you are looking for the notion of "relative path" (foo) vs "absolute path" (/foo),
+   * use QUrl::isRelative() instead.
+   * Indeed, isRelativeUrl() returns true for the string "/foo" since it doesn't contain a protocol,
+   * while KUrl("/foo").isRelative() is false since the KUrl constructor turns it into file:///foo.
+   * The two methods basically test the same thing, but this one takes a string (which is faster)
+   * while the class method requires a QUrl/KUrl which gives a more expected result, given
+   * the "magic" in the KUrl constructor.
+   *
    * @param _url URL to examine
    * @return true when the URL is likely to be "relative", false otherwise.
    */
@@ -1083,7 +1109,7 @@ private:
   QString toString() const; // forbidden, use url(), prettyUrl(), or pathOrUrl() instead.
   operator QString() const; // forbidden, use url(), prettyUrl(), or pathOrUrl() instead.
 private:
-  KUrlPrivate* const d;
+  KUrlPrivate* const d; // Don't ever use this, it would break clear() (which is in QUrl)
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(KUrl::EncodedPathAndQueryOptions)
@@ -1101,8 +1127,9 @@ Q_DECLARE_METATYPE(KUrl::List)
  * Two malformed URLs with the same string representation
  * are nevertheless considered to be unequal.
  * That means no malformed URL equals anything else.
+ * @deprecated use KUrl(_url1).equals(KUrl(_url2)) instead.
  */
-KDECORE_EXPORT bool urlcmp( const QString& _url1, const QString& _url2 );
+KDECORE_EXPORT_DEPRECATED bool urlcmp( const QString& _url1, const QString& _url2 ); // KDE5: remove, KUrl::equals is better API
 
 /**
  * \relates KUrl
@@ -1114,8 +1141,9 @@ KDECORE_EXPORT bool urlcmp( const QString& _url1, const QString& _url2 );
  * @param _url1 A reference URL
  * @param _url2 A URL that will be compared with the reference URL
  * @param options a set of KUrl::EqualsOption flags
+ * @deprecated use KUrl(_url1).equals(KUrl(_url2), options) instead.
  */
-KDECORE_EXPORT bool urlcmp( const QString& _url1, const QString& _url2, const KUrl::EqualsOptions& options );
+KDECORE_EXPORT_DEPRECATED bool urlcmp( const QString& _url1, const QString& _url2, const KUrl::EqualsOptions& options ); // KDE5: remove, KUrl::equals is better API
 
 KDECORE_EXPORT uint qHash(const KUrl& kurl);
 
