@@ -57,7 +57,6 @@
 #include "kcalendarsystem.h"
 #include "kcurrencycode.h"
 #include "klocalizedstring.h"
-#include "ktranslit_p.h"
 #include "kconfiggroup.h"
 #include "kcatalogname_p.h"
 #include "common_helpers_p.h"
@@ -83,8 +82,18 @@ KLocaleStaticData::KLocaleStaticData()
     // Languages using non-Western Arabic digit sets.
     // FIXME: Temporary until full language-sensitivity implemented.
     languagesUsingDigitSet.insert(KLocale::ArabicIndicDigits, QStringList() << "ar" << "ps");
-    languagesUsingDigitSet.insert(KLocale::EasternArabicIndicDigits, QStringList() << "fa" << "ur");
+    languagesUsingDigitSet.insert(KLocale::BengaliDigits, QStringList() << "bn" << "as" );
     languagesUsingDigitSet.insert(KLocale::DevenagariDigits, QStringList() << "hi" << "ne");
+    languagesUsingDigitSet.insert(KLocale::EasternArabicIndicDigits, QStringList() << "fa" << "ur");
+    languagesUsingDigitSet.insert(KLocale::GujaratiDigits, QStringList() << "gu" );
+    languagesUsingDigitSet.insert(KLocale::GurmukhiDigits, QStringList() << "pa" );
+    languagesUsingDigitSet.insert(KLocale::KannadaDigits, QStringList() << "kn" );
+    languagesUsingDigitSet.insert(KLocale::KhmerDigits, QStringList() << "km" );
+    languagesUsingDigitSet.insert(KLocale::MalayalamDigits, QStringList() << "ml" );
+    languagesUsingDigitSet.insert(KLocale::OriyaDigits, QStringList() << "or" );
+    languagesUsingDigitSet.insert(KLocale::TamilDigits, QStringList() << "ta" );
+    languagesUsingDigitSet.insert(KLocale::TeluguDigits, QStringList() << "te" );
+    languagesUsingDigitSet.insert(KLocale::ThaiDigits, QStringList() << "th");
 }
 
 K_GLOBAL_STATIC(KLocaleStaticData, staticData)
@@ -169,7 +178,7 @@ public:
     /**
      * @internal function used by the translate versions
      */
-    void translate_priv(const char *msgctxt, const char *msgid, const char *msgid_plural = 0,
+    void translate_priv(const char *catname, const char *msgctxt, const char *msgid, const char *msgid_plural = 0,
                         unsigned long n = 0, QString *language = 0, QString *translation = 0) const;
 
     /**
@@ -216,6 +225,11 @@ public:
      */
     static QString formatSingleDuration(DurationType durationType, int n);
 
+    /**
+     * @internal Returns a list of all available countries
+     */
+    QStringList allCountriesList() const;
+
     // Numbers and money
     QString decimalSymbol;
     QString thousandsSeparator;
@@ -344,13 +358,14 @@ void KLocalePrivate::initMainCatalogs()
         catalogNames.append(KCatalogName(catalogName));   // application catalog
 
         // catalogs from which each application can draw translations
-        numberOfSysCatalogs = 6;
+        const int numberOfCatalogs = catalogNames.size();
         catalogNames.append(KCatalogName("libphonon"));
         catalogNames.append(KCatalogName("kio4"));
         catalogNames.append(KCatalogName("kdelibs4"));
         catalogNames.append(KCatalogName("kdeqt"));
         catalogNames.append(KCatalogName("solid_qt"));
         catalogNames.append(KCatalogName("kdecalendarsystems"));
+        numberOfSysCatalogs = catalogNames.size() - numberOfCatalogs;
 
         updateCatalogs(); // evaluate this for all languages
     }
@@ -391,10 +406,23 @@ void KLocalePrivate::initLanguageList(KConfig *config, bool useEnv)
     KConfigGroup cg(config, "Locale");
 
     // Set the country as specified by the KDE config or use default,
-    // do not consider environment variables.
+    // If the user has set a default, then always use that.
+    // If not, then use QLocale magic to determine a good default.
+    // QLocale attempts to find the system country, in Windows and Mac
+    // this uses the provided system api, under *nix platforms it uses
+    // the various locale environemnt variables and CLDR to try determine a
+    // sensible default.
+    // If QLocale fails, then fallback to C
     if (country.isEmpty()) {
         country = cg.readEntry("Country");
     }
+    if (country.isEmpty()) {
+        QString systemCountry, s1, s2, s3;
+        KLocale::splitLocale( QLocale::system().name(), s1, systemCountry, s2, s3 );
+        if ( !systemCountry.isEmpty() && allCountriesList().contains( systemCountry, Qt::CaseInsensitive ) ) {
+            country =  systemCountry.toLower();
+        }
+    }
     if (country.isEmpty()) {
         country = KLocale::defaultCountry();
     }
@@ -675,14 +703,8 @@ bool KLocalePrivate::isApplicationTranslatedInto(const QString & lang)
         return false;
     }
 
-    // Check for this language and possible transliteration fallbacks.
-    QStringList possibles;
-    possibles += lang;
-    possibles += KTranslit::fallbackList(lang);
-    foreach(const QString &lang, possibles) {
-        if (! KCatalog::catalogLocaleDir(catalogName, lang).isEmpty()) {
-            return true;
-        }
+    if (!KCatalog::catalogLocaleDir(catalogName, lang).isEmpty()) {
+        return true;
     }
     return false;
 }
@@ -771,19 +793,18 @@ void KLocalePrivate::updateCatalogs()
 
     QList<KCatalog> newCatalogs;
 
-    // Insert possible transliteration fallbacks after each set language.
-    QStringList languageListFB;
-    foreach(const QString &lang, languageList) {
-        languageListFB += lang;
-        languageListFB += KTranslit::fallbackList(lang);
-    }
-
     // now iterate over all languages and all wanted catalog names and append or create them in the
     // right order the sequence must be e.g. nds/appname nds/kdelibs nds/kio de/appname de/kdelibs
     // de/kio etc. and not nds/appname de/appname nds/kdelibs de/kdelibs etc. Otherwise we would be
     // in trouble with a language sequende nds,<default>,de. In this case <default> must hide
     // everything after itself in the language list.
-    foreach(const QString &lang, languageListFB) {
+    foreach(const QString &lang, languageList) {
+        if (lang == KLocale::defaultLanguage()) {
+            // Default language has no catalogs (messages from the code),
+            // so loading catalogs for languages below the default
+            // would later confuse the fallback resolution.
+            break;
+        }
         foreach(const KCatalogName &name, catalogNames) {
             // create and add catalog for this name and language if it exists
             if (! KCatalog::catalogLocaleDir(name.name, lang).isEmpty()) {
@@ -795,7 +816,7 @@ void KLocalePrivate::updateCatalogs()
 
     // notify KLocalizedString of catalog update.
     catalogs = newCatalogs;
-    KLocalizedString::notifyCatalogsUpdated(languageListFB, catalogNames);
+    KLocalizedString::notifyCatalogsUpdated(languageList, catalogNames);
 }
 
 void KLocale::removeCatalog(const QString &catalog)
@@ -835,7 +856,7 @@ KLocale::~KLocale()
     delete d;
 }
 
-void KLocalePrivate::translate_priv(const char *msgctxt, const char *msgid,
+void KLocalePrivate::translate_priv(const char *catname, const char *msgctxt, const char *msgid,
                                     const char *msgid_plural, unsigned long n, QString *language,
                                     QString *translation) const
 {
@@ -879,7 +900,11 @@ void KLocalePrivate::translate_priv(const char *msgctxt, const char *msgid,
         return;
     }
 
-    QList<KCatalog> catalogList = catalogs;
+    const QList<KCatalog> catalogList = catalogs;
+    QString catNameDecoded;
+    if (catname != NULL) {
+        catNameDecoded = QString::fromUtf8(catname);
+    }
     for (QList<KCatalog>::ConstIterator it = catalogList.constBegin(); it != catalogList.constEnd();
          ++it) {
         // shortcut evaluation: once we have arrived at default language, we cannot consult
@@ -889,50 +914,74 @@ void KLocalePrivate::translate_priv(const char *msgctxt, const char *msgid,
             return;
         }
 
-        QString text;
-        if (msgctxt != NULL && msgid_plural != NULL) {
-            text = (*it).translateStrict(msgctxt, msgid, msgid_plural, n);
-        } else if (msgid_plural != NULL) {
-            text = (*it).translateStrict(msgid, msgid_plural, n);
-        } else if (msgctxt != NULL) {
-            text = (*it).translateStrict(msgctxt, msgid);
-        } else {
-            text = (*it).translateStrict(msgid);
-        }
+        if (catNameDecoded.isEmpty() || catNameDecoded == (*it).name()) {
+            QString text;
+            if (msgctxt != NULL && msgid_plural != NULL) {
+                text = (*it).translateStrict(msgctxt, msgid, msgid_plural, n);
+            } else if (msgid_plural != NULL) {
+                text = (*it).translateStrict(msgid, msgid_plural, n);
+            } else if (msgctxt != NULL) {
+                text = (*it).translateStrict(msgctxt, msgid);
+            } else {
+                text = (*it).translateStrict(msgid);
+            }
 
-        if (!text.isEmpty()) {
-            // we found it
-            if (language) {
-                *language = (*it).language();
+            if (!text.isEmpty()) {
+                // we found it
+                if (language) {
+                    *language = (*it).language();
+                }
+                if (translation) {
+                    *translation = text;
+                }
+                return;
             }
-            if (translation) {
-                *translation = text;
-            }
-            return;
         }
     }
 }
 
+void KLocale::translateRawFrom(const char* catname, const char* msg, QString *lang, QString *trans) const
+{
+    d->translate_priv(catname, 0, msg, 0, 0, lang, trans);
+}
+
 void KLocale::translateRaw(const char* msg, QString *lang, QString *trans) const
 {
-    d->translate_priv(0, msg, 0, 0, lang, trans);
+    d->translate_priv(0, 0, msg, 0, 0, lang, trans);
+}
+
+void KLocale::translateRawFrom(const char* catname, const char *ctxt, const char *msg, QString *lang, QString *trans) const
+{
+    d->translate_priv(catname, ctxt, msg, 0, 0, lang, trans);
 }
 
 void KLocale::translateRaw(const char *ctxt, const char *msg, QString *lang, QString *trans) const
 {
-    d->translate_priv(ctxt, msg, 0, 0, lang, trans);
+    d->translate_priv(0, ctxt, msg, 0, 0, lang, trans);
+}
+
+void KLocale::translateRawFrom(const char* catname, const char *singular, const char *plural, unsigned long n, QString *lang,
+                               QString *trans) const
+{
+    d->translate_priv(catname, 0, singular, plural, n, lang, trans);
 }
 
 void KLocale::translateRaw(const char *singular, const char *plural, unsigned long n, QString *lang,
                            QString *trans) const
 {
-    d->translate_priv(0, singular, plural, n, lang, trans);
+    d->translate_priv(0, 0, singular, plural, n, lang, trans);
+}
+
+void KLocale::translateRawFrom(const char* catname, const char *ctxt, const char *singular, const char *plural,
+                               unsigned long n, QString *lang, QString *trans) const
+{
+    d->translate_priv(catname, ctxt, singular, plural, n, lang, trans);
 }
 
 void KLocale::translateRaw(const char *ctxt, const char *singular, const char *plural,
                            unsigned long n, QString *lang, QString *trans) const
 {
-    d->translate_priv(ctxt, singular, plural, n, lang, trans);
+    d->translate_priv(0, ctxt, singular, plural, n, lang, trans);
 }
 
 QString KLocale::translateQt(const char *context, const char *sourceText, const char *comment) const
@@ -977,14 +1026,14 @@ QString KLocale::translateQt(const char *context, const char *sourceText, const
 
     if (comment && comment[0]) {
         // Comment given, go for context call.
-        d->translate_priv(comment, sourceText, 0, 0, &language, &translation);
+        d->translate_priv(0, comment, sourceText, 0, 0, &language, &translation);
     } else {
         // Comment not given, go for try-fallback with context.
         if (context && context[0]) {
-            d->translate_priv(context, sourceText, 0, 0, &language, &translation);
+            d->translate_priv(0, context, sourceText, 0, 0, &language, &translation);
         }
         if (language.isEmpty() || language == defaultLanguage()) {
-            d->translate_priv(0, sourceText, 0, 0, &language, &translation);
+            d->translate_priv(0, 0, sourceText, 0, 0, &language, &translation);
         }
     }
 
@@ -1001,8 +1050,18 @@ QList<KLocale::DigitSet> KLocale::allDigitSetsList() const
     QList<DigitSet> digitSets;
     digitSets.append(ArabicDigits);
     digitSets.append(ArabicIndicDigits);
-    digitSets.append(EasternArabicIndicDigits);
+    digitSets.append(BengaliDigits);
     digitSets.append(DevenagariDigits);
+    digitSets.append(EasternArabicIndicDigits);
+    digitSets.append(GujaratiDigits);
+    digitSets.append(GurmukhiDigits);
+    digitSets.append(KannadaDigits);
+    digitSets.append(KhmerDigits);
+    digitSets.append(MalayalamDigits);
+    digitSets.append(OriyaDigits);
+    digitSets.append(TamilDigits);
+    digitSets.append(TeluguDigits);
+    digitSets.append(ThaiDigits);
     return digitSets;
 }
 
@@ -1011,10 +1070,30 @@ static QString digitSetString(KLocale::DigitSet digitSet)
     switch (digitSet) {
     case KLocale::ArabicIndicDigits:
         return QString::fromUtf8("٠١٢٣٤٥٦٧٨٩");
-    case KLocale::EasternArabicIndicDigits:
-        return QString::fromUtf8("۰۱۲۳۴۵۶۷۸۹");
+    case KLocale::BengaliDigits:
+        return QString::fromUtf8("০১২৩৪৫৬৭৮৯");
     case KLocale::DevenagariDigits:
         return QString::fromUtf8("०१२३४५६७८९");
+    case KLocale::EasternArabicIndicDigits:
+        return QString::fromUtf8("۰۱۲۳۴۵۶۷۸۹");
+    case KLocale::GujaratiDigits:
+        return QString::fromUtf8("૦૧૨૩૪૫૬૭૮૯");
+    case KLocale::GurmukhiDigits:
+        return QString::fromUtf8("੦੧੨੩੪੫੬੭੮੯");
+    case KLocale::KannadaDigits:
+        return QString::fromUtf8("೦೧೨೩೪೫೬೭೮೯");
+    case KLocale::KhmerDigits:
+        return QString::fromUtf8("០១២៣៤៥៦៧៨៩");
+    case KLocale::MalayalamDigits:
+        return QString::fromUtf8("൦൧൨൩൪൫൬൭൮൯");
+    case KLocale::OriyaDigits:
+        return QString::fromUtf8("୦୧୨୩୪୫୬୭୮୯");
+    case KLocale::TamilDigits:
+        return QString::fromUtf8("௦௧௨௩௪௫௬௭௮");
+    case KLocale::TeluguDigits:
+        return QString::fromUtf8("౦౧౨౩౪౫౬౭౯");
+    case KLocale::ThaiDigits:
+        return QString::fromUtf8("๐๑๒๓๔๕๖๗๘๙");
     default:
         return QString::fromUtf8("0123456789");
     }
@@ -1027,11 +1106,41 @@ QString KLocale::digitSetToName(DigitSet digitSet, bool withDigits) const
     case KLocale::ArabicIndicDigits:
         name = i18nc("digit set", "Arabic-Indic");
         break;
+    case KLocale::BengaliDigits:
+        name = i18nc("digit set", "Bengali");
+        break;
+    case KLocale::DevenagariDigits:
+        name = i18nc("digit set", "Devanagari");
+        break;
     case KLocale::EasternArabicIndicDigits:
         name = i18nc("digit set", "Eastern Arabic-Indic");
         break;
-    case KLocale::DevenagariDigits:
-        name = i18nc("digit set", "Devenagari");
+    case KLocale::GujaratiDigits:
+        name = i18nc("digit set", "Gujarati");
+        break;
+    case KLocale::GurmukhiDigits:
+        name = i18nc("digit set", "Gurmukhi");
+        break;
+    case KLocale::KannadaDigits:
+        name = i18nc("digit set", "Kannada");
+        break;
+    case KLocale::KhmerDigits:
+        name = i18nc("digit set", "Khmer");
+        break;
+    case KLocale::MalayalamDigits:
+        name = i18nc("digit set", "Malayalam");
+        break;
+    case KLocale::OriyaDigits:
+        name = i18nc("digit set", "Oriya");
+        break;
+    case KLocale::TamilDigits:
+        name = i18nc("digit set", "Tamil");
+        break;
+    case KLocale::TeluguDigits:
+        name = i18nc("digit set", "Telugu");
+        break;
+    case KLocale::ThaiDigits:
+        name = i18nc("digit set", "Thai");
         break;
     default:
         name = i18nc("digit set", "Arabic");
@@ -1443,8 +1552,8 @@ QList<QString> KLocalePrivate::dialectUnitsList(KLocale::BinaryUnitDialect diale
     QString s; // Used in CACHE_BYTE_FMT macro defined shortly
 
     // Adds a given translation to the binaryUnits list.
-#define CACHE_BYTE_FMT(ctxt, x) \
-        translate_priv(ctxt, x, 0, 0, 0, &s); \
+#define CACHE_BYTE_FMT(ctxt_text) \
+        translate_priv(0, ctxt_text, 0, 0, 0, &s); \
         binaryUnits.append(s);
 
     // Do not remove i18n: comments below, they are used by the
@@ -1452,65 +1561,65 @@ QList<QString> KLocalePrivate::dialectUnitsList(KLocale::BinaryUnitDialect diale
 
     // This prefix is shared by all current dialects.
     // i18n: Dumb message, avoid any markup or scripting.
-    CACHE_BYTE_FMT("size in bytes", I18N_NOOP2("size in bytes", "%1 B"));
+    CACHE_BYTE_FMT(I18N_NOOP2_NOSTRIP("size in bytes", "%1 B"));
 
     switch (dialect) {
     case KLocale::MetricBinaryDialect:
         // i18n: Dumb message, avoid any markup or scripting.
-        CACHE_BYTE_FMT("size in 1000 bytes", I18N_NOOP2("size in 1000 bytes", "%1 kB"));
+        CACHE_BYTE_FMT(I18N_NOOP2_NOSTRIP("size in 1000 bytes", "%1 kB"));
         // i18n: Dumb message, avoid any markup or scripting.
-        CACHE_BYTE_FMT("size in 10^6 bytes", I18N_NOOP2("size in 10^6 bytes", "%1 MB"));
+        CACHE_BYTE_FMT(I18N_NOOP2_NOSTRIP("size in 10^6 bytes", "%1 MB"));
         // i18n: Dumb message, avoid any markup or scripting.
-        CACHE_BYTE_FMT("size in 10^9 bytes", I18N_NOOP2("size in 10^9 bytes", "%1 GB"));
+        CACHE_BYTE_FMT(I18N_NOOP2_NOSTRIP("size in 10^9 bytes", "%1 GB"));
         // i18n: Dumb message, avoid any markup or scripting.
-        CACHE_BYTE_FMT("size in 10^12 bytes", I18N_NOOP2("size in 10^12 bytes", "%1 TB"));
+        CACHE_BYTE_FMT(I18N_NOOP2_NOSTRIP("size in 10^12 bytes", "%1 TB"));
         // i18n: Dumb message, avoid any markup or scripting.
-        CACHE_BYTE_FMT("size in 10^15 bytes", I18N_NOOP2("size in 10^15 bytes", "%1 PB"));
+        CACHE_BYTE_FMT(I18N_NOOP2_NOSTRIP("size in 10^15 bytes", "%1 PB"));
         // i18n: Dumb message, avoid any markup or scripting.
-        CACHE_BYTE_FMT("size in 10^18 bytes", I18N_NOOP2("size in 10^18 bytes", "%1 EB"));
+        CACHE_BYTE_FMT(I18N_NOOP2_NOSTRIP("size in 10^18 bytes", "%1 EB"));
         // i18n: Dumb message, avoid any markup or scripting.
-        CACHE_BYTE_FMT("size in 10^21 bytes", I18N_NOOP2("size in 10^21 bytes", "%1 ZB"));
+        CACHE_BYTE_FMT(I18N_NOOP2_NOSTRIP("size in 10^21 bytes", "%1 ZB"));
         // i18n: Dumb message, avoid any markup or scripting.
-        CACHE_BYTE_FMT("size in 10^24 bytes", I18N_NOOP2("size in 10^24 bytes", "%1 YB"));
+        CACHE_BYTE_FMT(I18N_NOOP2_NOSTRIP("size in 10^24 bytes", "%1 YB"));
         break;
 
     case KLocale::JEDECBinaryDialect:
         // i18n: Dumb message, avoid any markup or scripting.
-        CACHE_BYTE_FMT("memory size in 1024 bytes", I18N_NOOP2("memory size in 1024 bytes", "%1 KB"));
+        CACHE_BYTE_FMT(I18N_NOOP2_NOSTRIP("memory size in 1024 bytes", "%1 KB"));
         // i18n: Dumb message, avoid any markup or scripting.
-        CACHE_BYTE_FMT("memory size in 2^20 bytes", I18N_NOOP2("memory size in 2^20 bytes", "%1 MB"));
+        CACHE_BYTE_FMT(I18N_NOOP2_NOSTRIP("memory size in 2^20 bytes", "%1 MB"));
         // i18n: Dumb message, avoid any markup or scripting.
-        CACHE_BYTE_FMT("memory size in 2^30 bytes", I18N_NOOP2("memory size in 2^30 bytes", "%1 GB"));
+        CACHE_BYTE_FMT(I18N_NOOP2_NOSTRIP("memory size in 2^30 bytes", "%1 GB"));
         // i18n: Dumb message, avoid any markup or scripting.
-        CACHE_BYTE_FMT("memory size in 2^40 bytes", I18N_NOOP2("memory size in 2^40 bytes", "%1 TB"));
+        CACHE_BYTE_FMT(I18N_NOOP2_NOSTRIP("memory size in 2^40 bytes", "%1 TB"));
         // i18n: Dumb message, avoid any markup or scripting.
-        CACHE_BYTE_FMT("memory size in 2^40 bytes", I18N_NOOP2("memory size in 2^40 bytes", "%1 PB"));
+        CACHE_BYTE_FMT(I18N_NOOP2_NOSTRIP("memory size in 2^50 bytes", "%1 PB"));
         // i18n: Dumb message, avoid any markup or scripting.
-        CACHE_BYTE_FMT("memory size in 2^50 bytes", I18N_NOOP2("memory size in 2^50 bytes", "%1 EB"));
+        CACHE_BYTE_FMT(I18N_NOOP2_NOSTRIP("memory size in 2^60 bytes", "%1 EB"));
         // i18n: Dumb message, avoid any markup or scripting.
-        CACHE_BYTE_FMT("memory size in 2^60 bytes", I18N_NOOP2("memory size in 2^60 bytes", "%1 ZB"));
+        CACHE_BYTE_FMT(I18N_NOOP2_NOSTRIP("memory size in 2^70 bytes", "%1 ZB"));
         // i18n: Dumb message, avoid any markup or scripting.
-        CACHE_BYTE_FMT("memory size in 2^70 bytes", I18N_NOOP2("memory size in 2^70 bytes", "%1 YB"));
+        CACHE_BYTE_FMT(I18N_NOOP2_NOSTRIP("memory size in 2^80 bytes", "%1 YB"));
         break;
 
     case KLocale::IECBinaryDialect:
     default:
         // i18n: Dumb message, avoid any markup or scripting.
-        CACHE_BYTE_FMT("size in 1024 bytes", I18N_NOOP2("size in 1024 bytes", "%1 KiB"));
+        CACHE_BYTE_FMT(I18N_NOOP2_NOSTRIP("size in 1024 bytes", "%1 KiB"));
         // i18n: Dumb message, avoid any markup or scripting.
-        CACHE_BYTE_FMT("size in 2^20 bytes", I18N_NOOP2("size in 2^20 bytes", "%1 MiB"));
+        CACHE_BYTE_FMT(I18N_NOOP2_NOSTRIP("size in 2^20 bytes", "%1 MiB"));
         // i18n: Dumb message, avoid any markup or scripting.
-        CACHE_BYTE_FMT("size in 2^30 bytes", I18N_NOOP2("size in 2^30 bytes", "%1 GiB"));
+        CACHE_BYTE_FMT(I18N_NOOP2_NOSTRIP("size in 2^30 bytes", "%1 GiB"));
         // i18n: Dumb message, avoid any markup or scripting.
-        CACHE_BYTE_FMT("size in 2^40 bytes", I18N_NOOP2("size in 2^40 bytes", "%1 TiB"));
+        CACHE_BYTE_FMT(I18N_NOOP2_NOSTRIP("size in 2^40 bytes", "%1 TiB"));
         // i18n: Dumb message, avoid any markup or scripting.
-        CACHE_BYTE_FMT("size in 2^40 bytes", I18N_NOOP2("size in 2^40 bytes", "%1 PiB"));
+        CACHE_BYTE_FMT(I18N_NOOP2_NOSTRIP("size in 2^50 bytes", "%1 PiB"));
         // i18n: Dumb message, avoid any markup or scripting.
-        CACHE_BYTE_FMT("size in 2^50 bytes", I18N_NOOP2("size in 2^50 bytes", "%1 EiB"));
+        CACHE_BYTE_FMT(I18N_NOOP2_NOSTRIP("size in 2^60 bytes", "%1 EiB"));
         // i18n: Dumb message, avoid any markup or scripting.
-        CACHE_BYTE_FMT("size in 2^60 bytes", I18N_NOOP2("size in 2^60 bytes", "%1 ZiB"));
+        CACHE_BYTE_FMT(I18N_NOOP2_NOSTRIP("size in 2^70 bytes", "%1 ZiB"));
         // i18n: Dumb message, avoid any markup or scripting.
-        CACHE_BYTE_FMT("size in 2^70 bytes", I18N_NOOP2("size in 2^70 bytes", "%1 YiB"));
+        CACHE_BYTE_FMT(I18N_NOOP2_NOSTRIP("size in 2^80 bytes", "%1 YiB"));
         break;
     }
 
@@ -2346,7 +2455,7 @@ void KLocalePrivate::initEncoding()
     systemLocale = "UTF-8";
 #endif //Q_OS_MAC
 
-    if (systemLocale == "ANSI_X3.4-1968") {
+    if ((systemLocale == "ANSI_X3.4-1968") || (systemLocale == "US-ASCII")) {
         // means ascii, "C"; QTextCodec doesn't know, so avoid warning
         systemLocale = "ISO-8859-1";
     }
@@ -2636,7 +2745,7 @@ QString KLocale::languageCodeToName(const QString &language) const
     return cg.readEntry("Name");
 }
 
-QStringList KLocale::allCountriesList() const
+QStringList KLocalePrivate::allCountriesList() const
 {
     QStringList countries;
     const QStringList paths = KGlobal::dirs()->findAllResources("locale", "l10n/*/entry.desktop");
@@ -2649,6 +2758,11 @@ QStringList KLocale::allCountriesList() const
     return countries;
 }
 
+QStringList KLocale::allCountriesList() const
+{
+    return d->allCountriesList();
+}
+
 QString KLocale::countryCodeToName(const QString &country) const
 {
     QString countryName;
