@@ -34,13 +34,15 @@
 #include <kglobal.h>
 #include <kcatalog_p.h>
 #include <kuitformats_p.h>
-#include <ktranslit_p.h>
+#include <klocale.h>
 #include <kconfiggroup.h>
 
 // Truncates string, for output of long messages.
+// (But don't truncate too much otherwise it's impossible to determine
+// which message is faulty if many messages start with the same beginning).
 static QString shorten (const QString &str)
 {
-    const int maxlen = 50;
+    const int maxlen = 80;
     if (str.length() <= maxlen)
         return str;
     else
@@ -65,7 +67,8 @@ namespace Kuit {
     namespace Att { // tag attribute names
         typedef enum {
             None,
-            Ctx, Url, Address, Section, Label
+            Ctx, Url, Address, Section, Label, Strong,
+            Width, Fill // internal helpers for numbers, not part of DTD
         } Var;
     }
 
@@ -80,7 +83,7 @@ namespace Kuit {
         typedef enum {
             None,
             Button, Inmenu, Intoolbar,
-            Window, Menu, Tab, Group, Column,
+            Window, Menu, Tab, Group, Column, Row,
             Slider, Spinbox, Listbox, Textbox, Chooser,
             Check, Radio,
             Inlistbox, Intable, Inrange, Intext,
@@ -177,16 +180,16 @@ KuitSemanticsStaticData::KuitSemanticsStaticData ()
     SETUP_TAG(Bcode, "bcode", None, None);
     SETUP_TAG(Shortcut, "shortcut", None, None);
     SETUP_TAG(Interface, "interface", None, None);
-    SETUP_TAG(Emphasis, "emphasis", None, None);
+    SETUP_TAG(Emphasis, "emphasis", Strong, None);
     SETUP_TAG(Placeholder, "placeholder", None, None);
     SETUP_TAG(Email, "email", Address, None);
     SETUP_TAG(Envar, "envar", None, None);
     SETUP_TAG(Message, "message", None, None);
     SETUP_TAG(Numid, "numid", None, None);
     SETUP_TAG(Nl, "nl", None, None);
-
-    SETUP_TAG(NumIntg, KUIT_NUMINTG, None, None); // internal, not part of DTD
-    SETUP_TAG(NumReal, KUIT_NUMREAL, None, None); // internal, not part of DTD
+    // Internal, not part of DTD.
+    SETUP_TAG(NumIntg, KUIT_NUMINTG, Width << Fill, None);
+    SETUP_TAG(NumReal, KUIT_NUMREAL, Width << Fill, None);
 
     // Setup known attribute names.
     #undef SETUP_ATT
@@ -198,6 +201,10 @@ KuitSemanticsStaticData::KuitSemanticsStaticData ()
     SETUP_ATT(Address, "address");
     SETUP_ATT(Section, "section");
     SETUP_ATT(Label, "label");
+    SETUP_ATT(Strong, "strong");
+    // Internal, not part of DTD.
+    SETUP_ATT(Width, "width");
+    SETUP_ATT(Fill, "fill");
 
     // Setup known format names.
     #undef SETUP_FMT
@@ -221,7 +228,7 @@ KuitSemanticsStaticData::KuitSemanticsStaticData ()
     SETUP_ROL(Action, "action", Plain,
               Button << Inmenu << Intoolbar);
     SETUP_ROL(Title, "title", Plain,
-              Window << Menu << Tab << Group << Column);
+              Window << Menu << Tab << Group << Column << Row);
     SETUP_ROL(Label, "label", Plain,
               Slider << Spinbox << Listbox << Textbox << Chooser);
     SETUP_ROL(Option, "option", Plain,
@@ -255,6 +262,7 @@ KuitSemanticsStaticData::KuitSemanticsStaticData ()
     SETUP_CUE(Tab, "tab");
     SETUP_CUE(Group, "group");
     SETUP_CUE(Column, "column");
+    SETUP_CUE(Row, "row");
     SETUP_CUE(Slider, "slider");
     SETUP_CUE(Spinbox, "spinbox");
     SETUP_CUE(Listbox, "listbox");
@@ -311,7 +319,7 @@ KuitSemanticsStaticData::KuitSemanticsStaticData ()
     xmlEntitiesInverse[QString('"')] = "quot";
 }
 
-K_GLOBAL_STATIC(KuitSemanticsStaticData, staticData)
+K_GLOBAL_STATIC(KuitSemanticsStaticData, semanticsStaticData)
 
 
 // -----------------------------------------------------------------------------
@@ -393,7 +401,8 @@ class KuitSemanticsPrivate
                                        int &numle, int &numtr);
 
     // Modifies text for some tags.
-    QString modifyTagText (Kuit::TagVar tag, const QString &text,
+    QString modifyTagText (const QString &text, Kuit::TagVar tag,
+                           const QHash<Kuit::AttVar, QString> &avals,
                            int numctx, Kuit::FmtVar fmt) const;
 
     private:
@@ -411,12 +420,10 @@ class KuitSemanticsPrivate
 
     // For fetching metatranslations.
     KCatalog *m_metaCat;
-    KTranslit *m_metaTranslit;
-    QString m_metaScript;
 };
 
 KuitSemanticsPrivate::KuitSemanticsPrivate (const QString &lang)
-: m_metaCat(NULL), m_metaTranslit(NULL)
+: m_metaCat(NULL)
 {
     m_lang = lang;
 
@@ -426,27 +433,7 @@ KuitSemanticsPrivate::KuitSemanticsPrivate (const QString &lang)
     // Also, pattern/transformation strings are "metastrings", not
     // fully proper i18n strings on their own.
 
-    // If this language may be made by transliteration from another,
-    // look for the catalog in transliteration fallbacks too.
-    QStringList possibleLangs = KTranslit::fallbackList(lang);
-    possibleLangs.prepend(lang);
-    QString realLang = lang;
-    foreach (const QString& clang, possibleLangs) {
-        if (!KCatalog::catalogLocaleDir("kdelibs4", clang).isEmpty()) {
-            realLang = clang;
-            break;
-        }
-    }
-    m_metaCat = new KCatalog("kdelibs4", realLang);
-
-    // Create transliterator and script to transliterate into.
-    m_metaTranslit = KTranslit::create(realLang); // may be NULL
-    int pos = lang.indexOf('@');
-    if (pos >= 0) {
-        m_metaScript = lang.mid(pos + 1);
-    }
-
-    // Fetching of metatranslations prepared, assemble all metadata.
+    m_metaCat = new KCatalog("kdelibs4", lang);
 
     // Get formatting patterns for all tag/att/fmt combinations.
     setFormattingPatterns();
@@ -454,11 +441,9 @@ KuitSemanticsPrivate::KuitSemanticsPrivate (const QString &lang)
     // Get data for tag text transformations.
     setTextTransformData();
 
-    // Catalog and transliterator not needed any more.
+    // Catalog not needed any more.
     delete m_metaCat;
     m_metaCat = NULL;
-    delete m_metaTranslit;
-    m_metaTranslit = NULL;
 }
 
 QString KuitSemanticsPrivate::metaTr (const char *ctxt, const char *id) const
@@ -466,11 +451,7 @@ QString KuitSemanticsPrivate::metaTr (const char *ctxt, const char *id) const
     if (m_metaCat == NULL) {
         return QString(id);
     }
-    QString meta = m_metaCat->translate(ctxt, id);
-    if (m_metaTranslit != NULL) {
-        meta = m_metaTranslit->transliterate(meta, m_metaScript);
-    }
-    return meta;
+    return m_metaCat->translate(ctxt, id);
 }
 
 void KuitSemanticsPrivate::setFormattingPatterns ()
@@ -721,6 +702,14 @@ void KuitSemanticsPrivate::setFormattingPatterns ()
                 I18N_NOOP2("@emphasis/rich",
     // i18n: KUIT pattern, see the comment to the first of these entries above.
                            "<i>%1</i>"));
+    SET_PATTERN(Tag::Emphasis, Att::Strong, Fmt::Plain,
+                I18N_NOOP2("@emphasis-strong/plain",
+    // i18n: KUIT pattern, see the comment to the first of these entries above.
+                           "**%1**"));
+    SET_PATTERN(Tag::Emphasis, Att::Strong, Fmt::Rich,
+                I18N_NOOP2("@emphasis-strong/rich",
+    // i18n: KUIT pattern, see the comment to the first of these entries above.
+                           "<b>%1</b>"));
 
     // -------> Placeholder
     SET_PATTERN(Tag::Placeholder, Att::None, Fmt::Plain,
@@ -914,7 +903,7 @@ Kuit::FmtVar KuitSemanticsPrivate::formatFromContextMarker (
     Q_UNUSED(text);
     #endif
 
-    KuitSemanticsStaticData *s = staticData;
+    KuitSemanticsStaticData *s = semanticsStaticData;
 
     // Semantic context marker is in the form @rolname:cuename/fmtname,
     // and must start just after any leading whitespace in the context string.
@@ -934,7 +923,7 @@ Kuit::FmtVar KuitSemanticsPrivate::formatFromContextMarker (
         }
 
         // Possible interface subcue.
-        int pcue = ctxmark.indexOf(':'); 
+        int pcue = ctxmark.indexOf(':');
         if (pcue >= 0) {
             cuename = ctxmark.mid(pcue + 1);
             ctxmark = ctxmark.left(pcue);
@@ -1011,7 +1000,7 @@ Kuit::FmtVar KuitSemanticsPrivate::formatFromContextMarker (
 
 Kuit::FmtVar KuitSemanticsPrivate::formatFromTags (const QString &text)
 {
-    KuitSemanticsStaticData *s = staticData;
+    KuitSemanticsStaticData *s = semanticsStaticData;
     static QRegExp staticTagRx("<\\s*(\\w+)[^>]*>");
 
     QRegExp tagRx = staticTagRx; // for thread-safety
@@ -1029,7 +1018,7 @@ Kuit::FmtVar KuitSemanticsPrivate::formatFromTags (const QString &text)
 QString KuitSemanticsPrivate::equipTopTag (const QString &text_,
                                            Kuit::TagVar &toptag)
 {
-    KuitSemanticsStaticData *s = staticData;
+    KuitSemanticsStaticData *s = semanticsStaticData;
 
     // Unless the text opens either with TopLong or TopShort tags,
     // make a guess: if it opens with one of Title, Subtitle, Para,
@@ -1096,7 +1085,7 @@ QString KuitSemanticsPrivate::semanticToVisualText (const QString &text_,
                                                     Kuit::FmtVar fmtExp_,
                                                     Kuit::FmtVar fmtImp_) const
 {
-    KuitSemanticsStaticData *s = staticData;
+    KuitSemanticsStaticData *s = semanticsStaticData;
 
     // Replace &-shortcut marker with "&amp;", not to confuse the parser;
     // but do not touch & which forms an XML entity as it is.
@@ -1121,11 +1110,14 @@ QString KuitSemanticsPrivate::semanticToVisualText (const QString &text_,
     bool hadAnyHtmlTag = false;
     QStack<OpenEl> openEls;
     QXmlStreamReader xml(text);
+    QStringRef lastElementName;
 
     while (!xml.atEnd()) {
         xml.readNext();
 
         if (xml.isStartElement()) {
+            lastElementName = xml.name();
+
             // Find first proper enclosing element tag.
             Kuit::TagVar etag = Kuit::Tag::None;
             for (int i = openEls.size() - 1; i >= 0; --i) {
@@ -1199,8 +1191,8 @@ QString KuitSemanticsPrivate::semanticToVisualText (const QString &text_,
     }
 
     if (xml.hasError()) {
-        kDebug(173) << QString("Markup error in message {%1}: %2")
-                              .arg(shorten(text), xml.errorString());
+        kDebug(173) << QString("Markup error in message {%1}: %2. Last tag parsed: %3")
+                              .arg(shorten(text), xml.errorString(), lastElementName.toString());
         return QString();
     }
 
@@ -1217,7 +1209,7 @@ KuitSemanticsPrivate::parseOpenEl (const QXmlStreamReader &xml,
     Q_UNUSED(text);
     #endif
 
-    KuitSemanticsStaticData *s = staticData;
+    KuitSemanticsStaticData *s = semanticsStaticData;
 
     OpenEl oel;
     oel.name = xml.name().toString().toLower();
@@ -1309,14 +1301,15 @@ QString KuitSemanticsPrivate::formatSubText (const QString &ptext,
                                              Kuit::FmtVar fmt,
                                              int numctx) const
 {
-    KuitSemanticsStaticData *s = staticData;
+    KuitSemanticsStaticData *s = semanticsStaticData;
 
     if (oel.handling == OpenEl::Proper) {
         // Select formatting pattern.
         QString pattern = visualPattern(oel.tag, oel.akey, fmt);
 
         // Some tags modify their text.
-        QString mtext = modifyTagText(oel.tag, oel.formattedText, numctx, fmt);
+        QString mtext = modifyTagText(oel.formattedText, oel.tag, oel.avals,
+                                      numctx, fmt);
 
         using namespace Kuit;
 
@@ -1392,8 +1385,9 @@ void KuitSemanticsPrivate::countWrappingNewlines (const QString &text,
     }
 }
 
-QString KuitSemanticsPrivate::modifyTagText (Kuit::TagVar tag,
-                                             const QString &text,
+QString KuitSemanticsPrivate::modifyTagText (const QString &text,
+                                             Kuit::TagVar tag,
+                                             const QHash<Kuit::AttVar, QString> &avals,
                                              int numctx,
                                              Kuit::FmtVar fmt) const
 {
@@ -1401,7 +1395,11 @@ QString KuitSemanticsPrivate::modifyTagText (Kuit::TagVar tag,
     if (   (tag == Kuit::Tag::NumIntg || tag == Kuit::Tag::NumReal) \
         && numctx < 1)
     {
-        return KGlobal::locale()->formatNumber(text, false);
+        int fieldWidth = avals.value(Kuit::Att::Width, QString('0')).toInt();
+        QString fillStr = avals.value(Kuit::Att::Fill, QString(' '));
+        QChar fillChar = !fillStr.isEmpty() ? fillStr[0] : QChar(' ');
+        return QString("%1").arg(KGlobal::locale()->formatNumber(text, false),
+                                 fieldWidth, fillChar);
     }
     else if (tag == Kuit::Tag::Filename) {
         return QDir::toNativeSeparators(text);
@@ -1422,7 +1420,7 @@ QString KuitSemanticsPrivate::finalizeVisualText (const QString &final,
                                                   bool hadQtTag,
                                                   bool hadAnyHtmlTag) const
 {
-    KuitSemanticsStaticData *s = staticData;
+    KuitSemanticsStaticData *s = semanticsStaticData;
 
     QString text = final;
 
@@ -1476,7 +1474,7 @@ QString KuitSemanticsPrivate::finalizeVisualText (const QString &final,
 QString KuitSemanticsPrivate::salvageMarkup (const QString &text_,
                                              Kuit::FmtVar fmt) const
 {
-    KuitSemanticsStaticData *s = staticData;
+    KuitSemanticsStaticData *s = semanticsStaticData;
     QString text = text_;
     QString ntext;
     int pos;
@@ -1503,7 +1501,6 @@ QString KuitSemanticsPrivate::salvageMarkup (const QString &text_,
         if (s->knownTags.contains(tagname)) {
             // Select formatting pattern.
             // TODO: Do not ignore attributes (in capts[3]).
-            // TODO: Locale-format numbers (in num* tags).
             QString pattern = visualPattern(s->knownTags[tagname], 0, fmt);
             ntext += pattern.arg(content);
         } else {
@@ -1562,7 +1559,7 @@ QString KuitSemantics::format (const QString &text, const QString &ctxt) const
 
 bool KuitSemantics::mightBeRichText (const QString &text)
 {
-    KuitSemanticsStaticData *s = staticData;
+    KuitSemanticsStaticData *s = semanticsStaticData;
 
     // Check by appearance of a valid XML entity at first ampersand.
     int p1 = text.indexOf('&');
@@ -1617,6 +1614,10 @@ QString KuitSemantics::escape (const QString &text)
             ntext += "&lt;";
         } else if (c == '>') {
             ntext += "&gt;";
+        } else if (c == '\'') {
+            ntext += "&apos;";
+        } else if (c == '"') {
+            ntext += "&quot;";
         } else {
             ntext += c;
         }
@@ -1624,4 +1625,3 @@ QString KuitSemantics::escape (const QString &text)
 
     return ntext;
 }
-
