@@ -42,6 +42,11 @@
 #include "kservicefactory.h"
 #include "kservicetypefactory.h"
 
+int servicesDebugArea() {
+    static int s_area = KDebug::registerArea("kdecore (services)");
+    return s_area;
+}
+
 QDataStream &operator<<(QDataStream &s, const KService::ServiceTypeAndPreference &st)
 {
     s << st.preference << st.serviceType;
@@ -89,7 +94,7 @@ void KServicePrivate::init( const KDesktopFile *config, KService* q )
     entryMap.remove("Type");
     if ( m_strType.isEmpty() )
     {
-        /*kWarning(7012) << "The desktop entry file " << entryPath
+        /*kWarning(servicesDebugArea()) << "The desktop entry file " << entryPath
           << " has no Type=... entry."
           << " It should be \"Application\" or \"Service\"" << endl;
           m_bValid = false;
@@ -97,7 +102,7 @@ void KServicePrivate::init( const KDesktopFile *config, KService* q )
         m_strType = "Application";
     } else if ( m_strType != "Application" && m_strType != "Service" )
     {
-        kWarning(7012) << "The desktop entry file " << entryPath
+        kWarning(servicesDebugArea()) << "The desktop entry file " << entryPath
                        << " has Type=" << m_strType
                        << " instead of \"Application\" or \"Service\"" << endl;
         m_bValid = false;
@@ -112,7 +117,7 @@ void KServicePrivate::init( const KDesktopFile *config, KService* q )
     if ( m_strType == "Application" ) {
         // It's an application? Should have an Exec line then, otherwise we can't run it
         if (m_strExec.isEmpty()) {
-            kWarning(7012) << "The desktop entry file " << entryPath
+            kWarning(servicesDebugArea()) << "The desktop entry file " << entryPath
                            << " has Type=" << m_strType
                            << " but no Exec line" << endl;
             m_bValid = false;
@@ -134,7 +139,7 @@ void KServicePrivate::init( const KDesktopFile *config, KService* q )
          (resource != "apps") &&
          !absPath)
     {
-        kWarning(7012) << "The desktop entry file " << entryPath
+        kWarning(servicesDebugArea()) << "The desktop entry file " << entryPath
                        << " has Type=" << m_strType << " but is located under \"" << resource
                        << "\" instead of \"apps\"" << endl;
         m_bValid = false;
@@ -146,7 +151,7 @@ void KServicePrivate::init( const KDesktopFile *config, KService* q )
          (resource != "services") &&
          !absPath)
     {
-        kWarning(7012) << "The desktop entry file " << entryPath
+        kWarning(servicesDebugArea()) << "The desktop entry file " << entryPath
                        << " has Type=" << m_strType << " but is located under \"" << resource
                        << "\" instead of \"services\"";
         m_bValid = false;
@@ -188,6 +193,13 @@ void KServicePrivate::init( const KDesktopFile *config, KService* q )
     // which makes user-preference handling difficult.
     m_strLibrary = desktopGroup.readEntry( "X-KDE-Library" );
     entryMap.remove("X-KDE-Library");
+    if (!m_strLibrary.isEmpty() && m_strType == "Application") {
+        kWarning(servicesDebugArea()) << "The desktop entry file" << entryPath
+                       << "has Type=" << m_strType
+                       << "but also has a X-KDE-Library key. This works for now,"
+                          " but makes user-preference handling difficult, so support for this might"
+                          " be removed at some point. Consider splitting it into two desktop files.";
+    }
 
     QStringList lstServiceTypes = desktopGroup.readEntry( "ServiceTypes", QStringList() );
     entryMap.remove("ServiceTypes");
@@ -210,7 +222,7 @@ void KServicePrivate::init( const KDesktopFile *config, KService* q )
     while ( st_it.hasNext() ) {
         const QString st = st_it.next();
         if (st.isEmpty()) {
-            kWarning(7012) << "The desktop entry file" << entryPath
+            kWarning(servicesDebugArea()) << "The desktop entry file" << entryPath
                            << "has an empty mimetype!";
             continue;
         }
@@ -265,7 +277,7 @@ void KServicePrivate::init( const KDesktopFile *config, KService* q )
         const QString key = it.key();
         // do not store other translations like Name[fr]; kbuildsycoca will rerun if we change languages anyway
         if (!key.contains('[')) {
-            //kDebug(7012) << "  Key =" << key << " Data =" << *it;
+            //kDebug(servicesDebugArea()) << "  Key =" << key << " Data =" << *it;
             m_mapProps.insert(key, QVariant(*it));
         }
     }
@@ -289,7 +301,7 @@ void KServicePrivate::parseActions(const KDesktopFile *config, KService* q)
         if (config->hasActionGroup(group)) {
             const KConfigGroup cg = config->actionGroup(group);
             if ( !cg.hasKey( "Name" ) || !cg.hasKey( "Exec" ) ) {
-                kWarning(7012) << "The action" << group << "in the desktop file" << q->entryPath()
+                kWarning(servicesDebugArea()) << "The action" << group << "in the desktop file" << q->entryPath()
                                << "has no Name or no Exec key";
             } else {
                 m_actions.append(KServiceAction(group,
@@ -299,7 +311,7 @@ void KServicePrivate::parseActions(const KDesktopFile *config, KService* q)
                                                 cg.readEntry("NoDisplay", false)));
             }
         } else {
-            kWarning(7012) << "The desktop file" << q->entryPath()
+            kWarning(servicesDebugArea()) << "The desktop file" << q->entryPath()
                            << "references the action" << group << "but doesn't define it";
         }
     }
@@ -425,7 +437,7 @@ bool KService::hasMimeType( const KServiceType* ptr ) const
     QVector<ServiceTypeAndPreference>::ConstIterator it = d->m_serviceTypes.begin();
     for( ; it != d->m_serviceTypes.end(); ++it ) {
         const QString& st = (*it).serviceType;
-        //kDebug(7012) << "    has " << (*it);
+        //kDebug(servicesDebugArea()) << "    has " << (*it);
         if ( st == ptr->name() )
             return true;
         // also the case of parent servicetypes
@@ -487,7 +499,7 @@ QVariant KServicePrivate::property( const QString& _name, QVariant::Type t ) con
         return QVariant( m_bAllowAsDefault );
     else if ( _name == "InitialPreference" )
         return QVariant( m_initialPreference );
-    else if ( _name == "Library" || _name == "X-KDE-Library" )
+    else if ( _name == "Library" )
         return makeStringVariant( m_strLibrary );
     else if ( _name == "DesktopEntryPath" ) // can't be null
         return QVariant( path );
@@ -507,7 +519,7 @@ QVariant KServicePrivate::property( const QString& _name, QVariant::Type t ) con
         t = KServiceTypeFactory::self()->findPropertyTypeByName(_name);
         if (t == QVariant::Invalid)
         {
-            kDebug(7012) << "Request for unknown property '" << _name << "'\n";
+            kDebug(servicesDebugArea()) << "Request for unknown property '" << _name << "'\n";
             return QVariant(); // Unknown property: Invalid variant.
         }
     }
@@ -515,7 +527,7 @@ QVariant KServicePrivate::property( const QString& _name, QVariant::Type t ) con
     QMap<QString,QVariant>::ConstIterator it = m_mapProps.find( _name );
     if ( (it == m_mapProps.end()) || (!it->isValid()))
     {
-        //kDebug(7012) << "Property not found " << _name;
+        //kDebug(servicesDebugArea()) << "Property not found " << _name;
         return QVariant(); // No property set.
     }
 
@@ -580,7 +592,7 @@ KService::Ptr KService::serviceByDesktopName( const QString& _name )
     // Prefer kde4-konsole over kde-konsole, if both are available
     QString name = _name.toLower();
     KService::Ptr s;
-    if (!_name.startsWith("kde4-"))
+    if (!_name.startsWith(QLatin1String("kde4-")))
         s = KServiceFactory::self()->findServiceByDesktopName( "kde4-" + name );
     if (!s)
         s = KServiceFactory::self()->findServiceByDesktopName( name );
@@ -609,10 +621,10 @@ KService::Ptr KService::serviceByStorageId( const QString& _storageId )
     QString tmp = _storageId;
     tmp = tmp.mid(tmp.lastIndexOf('/')+1); // Strip dir
 
-    if (tmp.endsWith(".desktop"))
+    if (tmp.endsWith(QLatin1String(".desktop")))
         tmp.truncate(tmp.length()-8);
 
-    if (tmp.endsWith(".kdelnk"))
+    if (tmp.endsWith(QLatin1String(".kdelnk")))
         tmp.truncate(tmp.length()-7);
 
     service = KService::serviceByDesktopName(tmp);
@@ -637,29 +649,34 @@ QString KService::username() const {
     return user;
 }
 
-bool KService::noDisplay() const {
-    if ( qvariant_cast<bool>(property("NoDisplay", QVariant::Bool)) )
-        return true;
-
+bool KService::showInKDE() const
+{
     Q_D(const KService);
 
     QMap<QString,QVariant>::ConstIterator it = d->m_mapProps.find( "OnlyShowIn" );
     if ( (it != d->m_mapProps.end()) && (it->isValid()))
     {
-        QString aValue = it->toString();
-        QStringList aList = aValue.split(';');
+        const QStringList aList = it->toString().split(';');
         if (!aList.contains("KDE"))
-            return true;
+            return false;
     }
 
     it = d->m_mapProps.find( "NotShowIn" );
     if ( (it != d->m_mapProps.end()) && (it->isValid()))
     {
-        QString aValue = it->toString();
-        QStringList aList = aValue.split(';');
+        const QStringList aList = it->toString().split(';');
         if (aList.contains("KDE"))
-            return true;
+            return false;
     }
+    return true;
+}
+
+bool KService::noDisplay() const {
+    if ( qvariant_cast<bool>(property("NoDisplay", QVariant::Bool)) )
+        return true;
+
+    if (!showInKDE())
+        return true;
 
     if (!KAuthorized::authorizeControlModule( storageId() ) )
         return true;
@@ -742,7 +759,7 @@ QString KService::storageId() const
 QString KService::locateLocal() const
 {
     Q_D(const KService);
-    if (d->menuId.isEmpty() || entryPath().startsWith(".hidden") ||
+    if (d->menuId.isEmpty() || entryPath().startsWith(QLatin1String(".hidden")) ||
         (QDir::isRelativePath(entryPath()) && d->categories.isEmpty()))
         return KDesktopFile::locateLocal(entryPath());
 
@@ -799,7 +816,7 @@ QString KService::exec() const
     Q_D(const KService);
     if (d->m_strType == "Application" && d->m_strExec.isEmpty())
     {
-        kWarning(7012) << "The desktop entry file " << entryPath()
+        kWarning(servicesDebugArea()) << "The desktop entry file " << entryPath()
                        << " has Type=" << d->m_strType << " but has no Exec field." << endl;
     }
     return d->m_strExec;
@@ -829,7 +846,7 @@ bool KService::terminal() const
     return d->m_bTerminal;
 }
 
-// KDE4: remove and port code to entryPath?
+// KDE5: remove and port code to entryPath?
 QString KService::desktopEntryPath() const
 {
     return entryPath();
