@@ -47,8 +47,7 @@ class KServicePrivate;
  * where the 'ServiceTypes=' field is used.
  *
  * For a tutorial on how to build a plugin-loading mechanism and how to write plugins
- * in general, see http://developer.kde.org/documentation/tutorials/developing-a-plugin-structure/index.html#developing_plugins
- * (todo: port to kde4, create techbase article)
+ * in general, see http://techbase.kde.org/Development/Tutorials#Services:_Applications_and_Plugins
  *
  * @see KServiceType
  * @see KServiceGroup
@@ -149,7 +148,7 @@ public:
      * different user id.
      * @return the username under which the service has to be run,
      *         or QString() if not set
-     * @see substututeUid()a
+     * @see substituteUid()
      */
     QString username() const;
 
@@ -325,11 +324,26 @@ public:
     int initialPreference() const;
 
     /**
-     * Whether the entry should be suppressed in menus.
+     * Whether the entry should be suppressed in the K menu.
      * @return true to suppress this service
+     *
+     * Such services still appear in trader queries, i.e. in
+     * "Open With" popup menus for instance.
      */
     bool noDisplay() const;
 
+    /**
+     * Whether the service should be shown in KDE at all
+     * (including in context menus).
+     * @return true if the service should be shown.
+     * 
+     * KMimeTypeTrader honours this and removes such services
+     * from its results.
+     *
+     * @since 4.5
+     */
+    bool showInKDE() const;
+
     /**
      * Name of the application this service belongs to.
      * (Useful for e.g. plugins)
@@ -387,16 +401,16 @@ public:
     void setTerminalOptions(const QString &options);
 
     /**
-     * Find a service by name, i.e. the translated Name field. You should
-     * never use this method with a literal name as argument, since the name
-     * is translated Name field of the desktop file. See serviceByStorageId instead.
+     * Find a service by name, i.e. the translated Name field. Don't use this.
+     * Use serviceByDesktopPath or serviceByStorageId instead.
      *
      * @param _name the name to search
      * @return a pointer to the requested service or 0 if the service is
      *         unknown.
      * @em Very @em important: Don't store the result in a KService* !
+     * @deprecated there is never a good reason to use this method.
      */
-    static Ptr serviceByName( const QString& _name );
+    static Ptr serviceByName( const QString& _name ); // KDE5: remove
 
     /**
      * Find a service based on its path as returned by entryPath().
@@ -528,6 +542,9 @@ public:
         return 0;
     }
 
+    /**
+     * @deprecated Use the non-static service->createInstance<T>(parent, args, &error)
+     */
     template <class T>
     static KDE_DEPRECATED T *createInstance(const KService::Ptr &service, QObject *parent = 0,
             const QVariantList &args = QVariantList(), QString *error = 0)
@@ -535,6 +552,10 @@ public:
         return service->createInstance<T>(parent, args, error);
     }
 
+    /**
+     * @deprecated Use the non-static service->createInstance<T>(parent, args, &error)
+     * where args is a QVariantList rather than a QStringList
+     */
     template <class T>
     static KDE_DEPRECATED T *createInstance( const KService::Ptr &service,
                               QObject *parent,
