@@ -26,7 +26,6 @@
 #include <kconfig.h>
 #include <kdebug.h>
 #include <kconfiggroup.h>
-#include <kstandarddirs.h>
 
 #include <QtNetwork/QHostInfo>
 
@@ -354,6 +353,26 @@ void KConfigTest::testLocale()
     QCOMPARE(group.readEntry("stringEntry1", QString()), Untranslated);
 }
 
+void KConfigTest::testEncoding()
+{
+    QString groupstr = QString::fromUtf8("UTF-8:\xc3\xb6l");
+
+    KConfig c( "kconfigtestencodings" );
+    KConfigGroup cg(&c, groupstr);
+    cg.writeEntry("key", "value");
+    c.sync();
+
+    QList<QByteArray> lines = readLines("kconfigtestencodings");
+    QCOMPARE(lines.count(), 2);
+    QCOMPARE(lines.first(), QByteArray("[UTF-8:\xc3\xb6l]\n"));
+
+    KConfig c2( "kconfigtestencodings" );
+    KConfigGroup cg2(&c2, groupstr);
+    QVERIFY(cg2.readEntry("key") == QByteArray("value"));
+
+    QVERIFY(c2.groupList().contains(groupstr));
+}
+
 void KConfigTest::testLists()
 {
   KConfig sc2( "kconfigtest" );
@@ -580,6 +599,12 @@ void KConfigTest::testChangeGroup()
     newGroup.changeGroup("FooBar"); // deprecated!
     QCOMPARE(newGroup.name(), QString("FooBar"));
     QCOMPARE(sc3.name(), QString("Hello")); // unchanged
+
+    // Write into the "changed group" and check that it works
+    newGroup.writeEntry("InFooBar", "FB");
+    QCOMPARE(KConfigGroup(&sc, "FooBar").entryMap().value("InFooBar"), QString("FB"));
+    QCOMPARE(KConfigGroup(&sc, "Hello").entryMap().value("InFooBar"), QString());
+
     KConfigGroup rootGroup(sc.group(""));
     QCOMPARE(rootGroup.name(), QString("<default>"));
     KConfigGroup sc32(rootGroup.group("Hello"));
@@ -625,7 +650,7 @@ void KConfigTest::testDelete()
   foreach(const QByteArray& item, readLines())
       if (item.startsWith("devices|"))
           count++;
-  QVERIFY(count == 2);
+  QCOMPARE(count, 2);
   cg.deleteEntry("devices|manual|/mnt/ipod");
   cf.sync();
   foreach(const QByteArray& item, readLines())
@@ -662,10 +687,15 @@ void KConfigTest::testDefaultGroup()
 
 #ifdef Q_OS_UNIX
     QList<QByteArray> lines = readLines();
-    QVERIFY(!lines.contains("[]"));
+    QVERIFY(!lines.contains("[]\n"));
     QCOMPARE(lines.first(), QByteArray("TestKey=defaultGroup\n"));
 #endif
 
+    // Now that the group exists make sure it isn't returned from groupList()
+    foreach(const QString& group, sc.groupList()) {
+        QVERIFY(!group.isEmpty() && group != "<default>");
+    }
+
     defaultGroup.deleteGroup();
     sc.sync();
 
@@ -706,10 +736,14 @@ void KConfigTest::testEmptyGroup()
 
 #ifdef Q_OS_UNIX
     QList<QByteArray> lines = readLines();
-    QVERIFY(!lines.contains("[]")); // there's no support for the [] group, in fact.
+    QVERIFY(!lines.contains("[]\n")); // there's no support for the [] group, in fact.
     QCOMPARE(lines.first(), QByteArray("TestKey=emptyGroup\n"));
 #endif
 
+    // Now that the group exists make sure it isn't returned from groupList()
+    foreach(const QString& group, sc.groupList()) {
+        QVERIFY(!group.isEmpty() && group != "<default>");
+    }
     emptyGroup.deleteGroup();
     sc.sync();
 
@@ -815,7 +849,7 @@ void KConfigTest::testOptionOrder()
     {
         QList<QByteArray> lines;
         // this is what the file should look like
-        lines << "[group3]\n" 
+        lines << "[group3]\n"
               << "entry2=modified\n"
               << "entry2[de_DE][$i]=t2\n";
 
@@ -860,6 +894,38 @@ void KConfigTest::testSubGroup()
     QCOMPARE(QStringList(cg.entryMap().keys()), PARENTGROUPKEYS);
     QCOMPARE(QStringList(subcg3.entryMap().keys()), SUBGROUP3KEYS);
 
+    // Create A group containing only other groups. We want to make sure it
+    // shows up in groupList of sc
+    KConfigGroup neg(&sc, "NoEntryGroup");
+    KConfigGroup negsub1(&neg, "NEG Child1");
+                 negsub1.writeEntry( "entry", "somevalue" );
+    KConfigGroup negsub2(&neg, "NEG Child2");
+    KConfigGroup negsub3(&neg, "NEG Child3");
+    KConfigGroup negsub31(&negsub3, "NEG Child3-1");
+    KConfigGroup negsub4(&neg, "NEG Child4");
+    KConfigGroup negsub41(&negsub4, "NEG Child4-1");
+                 negsub41.writeEntry( "entry", "somevalue" );
+
+    // A group exists if it has content
+    QVERIFY(negsub1.exists());
+
+    // But it doesn't exist if it has no content
+    QEXPECT_FAIL("", "Currently groups without content do not exist!", Continue);
+    QVERIFY(negsub2.exists());
+
+    // A subgroup does not qualify as content if it is also empty!
+    QEXPECT_FAIL("", "Currently groups without content do not exist!", Continue);
+    QVERIFY(negsub3.exists());
+
+    // A subgroup with content is ok
+    QVERIFY(negsub4.exists());
+
+    // Only subgroups with content show up in groupList()
+    QEXPECT_FAIL("", "Empty subgroups do not show up in groupList()", Continue);
+    QCOMPARE(neg.groupList(), QStringList() << "NEG Child1" << "NEG Child2" << "NEG Child3" << "NEG Child4");
+    // This is what happens
+    QCOMPARE(neg.groupList(), QStringList() << "NEG Child1" << "NEG Child4");
+
     // make sure groupList() isn't returning something it shouldn't
     foreach(const QString& group, sc.groupList()) {
       QVERIFY(!group.isEmpty() && group != "<default>");
@@ -869,22 +935,6 @@ void KConfigTest::testSubGroup()
     }
 }
 
-void KConfigTest::testKdeglobals()
-{
-    const QString kdeglobals = KStandardDirs::locateLocal("config", "kdeglobals");
-    KConfig sc("kdeglobals");
-    QVERIFY(sc.forceGlobal());
-    QVERIFY(sc.name() == QLatin1String("kdeglobals"));
-
-    KConfig sc2("", KConfig::IncludeGlobals);
-    QVERIFY(sc2.forceGlobal());
-    QVERIFY(sc2.name() == QLatin1String("kdeglobals"));
-
-    KConfig sc3("kdeglobals", KConfig::SimpleConfig);
-    QVERIFY(sc3.forceGlobal());
-    QVERIFY(sc3.name() == QLatin1String("kdeglobals"));
-}
-
 void KConfigTest::testAddConfigSources()
 {
     KConfig cf("specificrc");
@@ -1117,12 +1167,14 @@ void KConfigTest::testDirtyOnEqualOverdo()
     QCOMPARE(cgLocal.readEntry("someKey", defvalr), val2);
 }
 
-QList<QByteArray> KConfigTest::readLines()
+QList<QByteArray> KConfigTest::readLines(const char* fileName)
 {
-    const QString path = KStandardDirs::locateLocal("config", "kconfigtest");
+    const QString path = KStandardDirs::locateLocal("config", fileName);
     Q_ASSERT(!path.isEmpty());
     QFile file(path);
-    Q_ASSERT(file.open(QIODevice::ReadOnly));
+    const bool opened = file.open(QIODevice::ReadOnly);
+    Q_ASSERT(opened);
+    Q_UNUSED(opened);
     QList<QByteArray> lines;
     QByteArray line;
     do {
@@ -1146,6 +1198,10 @@ void KConfigTest::testCreateDir()
     desktopFile.desktopGroup().writeEntry("key", "value");
     desktopFile.sync();
     QVERIFY(QFile::exists(file));
+
+    // Cleanup
+    QFile::remove(file);
+    QDir().rmdir(subdir);
 }
 
 void KConfigTest::testSyncOnExit()
@@ -1170,6 +1226,7 @@ void KConfigTest::testSharedConfig()
 
 void KConfigTest::testLocaleConfig()
 {
+    // Initialize the testdata
     QDir dir;
     QString subdir = QDir::home().canonicalPath() + "/.kde-unit-test/";
     dir.mkpath(subdir);
@@ -1178,19 +1235,226 @@ void KConfigTest::testLocaleConfig()
     QFile f(file);
     QVERIFY(f.open(QIODevice::WriteOnly));
     QTextStream ts(&f);
-    ts << "[Test]\n";
+    ts << "[Test_Wrong]\n";
     ts << "foo[ca]=5\n";
     ts << "foostring[ca]=nice\n";
     ts << "foobool[ca]=true\n";
+    ts << "[Test_Right]\n";
+    ts << "foo=5\n";
+    ts << "foo[ca]=5\n";
+    ts << "foostring=primary\n";
+    ts << "foostring[ca]=nice\n";
+    ts << "foobool=primary\n";
+    ts << "foobool[ca]=true\n";
     f.close();
+
+    // Load the testdata
     QVERIFY(QFile::exists(file));
     KConfig config(file);
     config.setLocale("ca");
-    KConfigGroup cg(&config, "Test");
-    QCOMPARE(cg.readEntry("foo"), QString("5"));
-    QCOMPARE(cg.readEntry("foo", 3), 5);
-    QCOMPARE(cg.readEntry("foostring"), QString("nice"));
-    QCOMPARE(cg.readEntry("foostring", "ugly"), QString("nice"));
-    QCOMPARE(cg.readEntry("foobool"), QString("true"));
+
+    // This group has only localized values. That is not supported. The values
+    // should be dropped on loading.
+    KConfigGroup cg(&config, "Test_Wrong");
+    QEXPECT_FAIL("", "The localized values are not dropped", Continue);
+    QVERIFY(!cg.hasKey("foo"));
+    QEXPECT_FAIL("", "The localized values are not dropped", Continue);
+    QVERIFY(!cg.hasKey("foostring"));
+    QEXPECT_FAIL("", "The localized values are not dropped", Continue);
+    QVERIFY(!cg.hasKey("foobool"));
+
+    // Now check the correct config group
+    KConfigGroup cg2(&config, "Test_Right");
+    QCOMPARE(cg2.readEntry("foo"), QString("5"));
+    QCOMPARE(cg2.readEntry("foo", 3), 5);
+    QCOMPARE(cg2.readEntry("foostring"), QString("nice"));
+    QCOMPARE(cg2.readEntry("foostring", "ugly"), QString("nice"));
+    QCOMPARE(cg2.readEntry("foobool"), QString("true"));
+    QCOMPARE(cg2.readEntry("foobool", false), true);
+
+    // Clean up after the testcase
+    QFile::remove(file);
+}
+
+
+void KConfigTest::testDeleteWhenLocalized()
+{
+    // Initialize the testdata
+    QDir dir;
+    QString subdir = QDir::home().canonicalPath() + "/.kde-unit-test/";
+    dir.mkpath(subdir);
+    QString file = subdir + "/localized_delete.test";
+    QFile::remove(file);
+    QFile f(file);
+    QVERIFY(f.open(QIODevice::WriteOnly));
+    QTextStream ts(&f);
+    ts << "[Test4711]\n";
+    ts << "foo=3\n";
+    ts << "foo[ca]=5\n";
+    ts << "foo[de]=7\n";
+    ts << "foostring=ugly\n";
+    ts << "foostring[ca]=nice\n";
+    ts << "foostring[de]=schoen\n";
+    ts << "foobool=false\n";
+    ts << "foobool[ca]=true\n";
+    ts << "foobool[de]=true\n";
+    f.close();
+
+    // Load the testdata. We start in locale "ca".
+    QVERIFY(QFile::exists(file));
+    KConfig config(file);
+    config.setLocale("ca");
+    KConfigGroup cg(&config, "Test4711");
+
+    // Delete a value. Once with localized, once with Normal
+    cg.deleteEntry("foostring", KConfigBase::Persistent | KConfigBase::Localized);
+    cg.deleteEntry("foobool");
+    config.sync();
+
+    // The value is now gone. The others are still there. Everything correct
+    // here.
+    QVERIFY(!cg.hasKey("foostring"));
+    QVERIFY(!cg.hasKey("foobool"));
+    QVERIFY(cg.hasKey("foo"));
+
+    // The current state is: (Just return before this comment.)
+    // [...]
+    // foobool[ca]=true
+    // foobool[$d]
+    // foobool[de]=wahr
+    // foostring[ca][$d]
+    // foostring=ugly
+    // foostring[de]=schoen
+    // [...]
+
+    // Now switch the locale to "de" and repeat the checks. Results should be
+    // the same. But they currently are not. The localized value are
+    // independent of each other. All values are still there in "de".
+    config.setLocale("de");
+    QEXPECT_FAIL("", "Currently localized values are not deleted correctly", Continue);
+    QVERIFY(!cg.hasKey("foostring"));
+    QEXPECT_FAIL("", "Currently localized values are not deleted correctly", Continue);
+    QVERIFY(!cg.hasKey("foobool"));
+    QVERIFY(cg.hasKey("foo"));
+    // Check where the wrong values come from.
+    // We get the "de" value.
+    QCOMPARE(cg.readEntry("foostring", "nothing"), QString("schoen"));
+    // We get the "de" value.
     QCOMPARE(cg.readEntry("foobool", false), true);
+
+    // Now switch the locale back "ca" and repeat the checks. Results are
+    // again different.
+    config.setLocale("ca");
+    // This line worked above. But now it fails.
+    QEXPECT_FAIL("", "Currently localized values are not deleted correctly", Continue);
+    QVERIFY(!cg.hasKey("foostring"));
+    // This line worked above too.
+    QEXPECT_FAIL("", "Currently localized values are not deleted correctly", Continue);
+    QVERIFY(!cg.hasKey("foobool"));
+    QVERIFY(cg.hasKey("foo"));
+    // Check where the wrong values come from.
+    // We get the primary value because the "ca" value was deleted.
+    QCOMPARE(cg.readEntry("foostring", "nothing"), QString("ugly"));
+    // We get the "ca" value.
+    QCOMPARE(cg.readEntry("foobool", false), true);
+
+    // Now test the deletion of a group.
+    cg.deleteGroup();
+    config.sync();
+
+    // Current state: The same as above [$d] for all values.
+
+    // The group still exists but the values are all gone.
+    QVERIFY(cg.exists());
+    QVERIFY(!cg.hasKey("foo"));
+    QVERIFY(!cg.hasKey("foostring"));
+    QVERIFY(!cg.hasKey("foobool"));
+
+    // Now switch the locale to "de" and repeat the checks. All values
+    // still here because only the primary values are deleted.
+    config.setLocale("de");
+    QEXPECT_FAIL("", "Currently localized values are not deleted correctly", Continue);
+    QVERIFY(!cg.hasKey("foo"));
+    QEXPECT_FAIL("", "Currently localized values are not deleted correctly", Continue);
+    QVERIFY(!cg.hasKey("foostring"));
+    QEXPECT_FAIL("", "Currently localized values are not deleted correctly", Continue);
+    QVERIFY(!cg.hasKey("foobool"));
+    // Check where the wrong values come from.
+    // We get the "de" value.
+    QCOMPARE(cg.readEntry("foostring", "nothing"), QString("schoen"));
+    // We get the "de" value.
+    QCOMPARE(cg.readEntry("foobool", false), true);
+    // We get the "de" value.
+    QCOMPARE(cg.readEntry("foo", 0), 7);
+
+    // Now switch the locale to "ca" and repeat the checks
+    // "foostring" is now really gone because both the primary value and the
+    // "ca" value are deleted.
+    config.setLocale("ca");
+    QEXPECT_FAIL("", "Currently localized values are not deleted correctly", Continue);
+    QVERIFY(!cg.hasKey("foo"));
+    QVERIFY(!cg.hasKey("foostring"));
+    QEXPECT_FAIL("", "Currently localized values are not deleted correctly", Continue);
+    QVERIFY(!cg.hasKey("foobool"));
+    // Check where the wrong values come from.
+    // We get the "ca" value.
+    QCOMPARE(cg.readEntry("foobool", false), true);
+    // We get the "ca" value.
+    QCOMPARE(cg.readEntry("foo", 0), 5);
+
+    // Cleanup
+    QFile::remove(file);
+}
+
+
+void KConfigTest::testKdeGlobals()
+{
+    {
+        KConfig glob("kdeglobals");
+        KConfigGroup general(&glob, "General");
+        general.writeEntry("testKG", "1");
+        glob.sync();
+    }
+
+    KConfig globRead("kdeglobals");
+    const KConfigGroup general(&globRead, "General");
+    QCOMPARE(general.readEntry("testKG"), QString("1"));
+
+    // Check we wrote into kdeglobals
+    const QList<QByteArray> lines = readLines("kdeglobals");
+#ifdef Q_WS_WIN
+#define LINEENDING "\r\n"
+#else
+#define LINEENDING "\n"
+#endif
+    QVERIFY(lines.contains("[General]"LINEENDING));
+    QVERIFY(lines.contains("testKG=1"LINEENDING));
+
+    // Writing using NoGlobals
+    {
+        KConfig glob("kdeglobals", KConfig::NoGlobals);
+        KConfigGroup general(&glob, "General");
+        general.writeEntry("testKG", "2");
+        glob.sync();
+    }
+    globRead.reparseConfiguration();
+    QCOMPARE(general.readEntry("testKG"), QString("2"));
+
+    // Reading using NoGlobals
+    {
+        KConfig globReadNoGlob("kdeglobals", KConfig::NoGlobals);
+        const KConfigGroup generalNoGlob(&globReadNoGlob, "General");
+        QCOMPARE(generalNoGlob.readEntry("testKG"), QString("2"));
+    }
+
+    // TODO now use kconfigtest and writeEntry(,Global) -> should go into kdeglobals
+}
+
+void KConfigTest::testAnonymousConfig()
+{
+    KConfig anonConfig(QString(), KConfig::SimpleConfig);
+    KConfigGroup general(&anonConfig, "General");
+    QCOMPARE(general.readEntry("testKG"), QString()); // no kdeglobals merging
+    general.writeEntry("Foo", "Bar");
+    QCOMPARE(general.readEntry("Foo"), QString("Bar"));
 }
