@@ -31,6 +31,7 @@ private Q_SLOTS:
     void testAppData();
     void testFindResource();
     void testFindAllResources();
+    void testFindAllResourcesNewDir();    
     void testFindDirs();
     void testFindResourceDir();
     void testFindExe();
@@ -39,6 +40,8 @@ private Q_SLOTS:
     void testAddResourceType();
     void testAddResourceDir();
     void testSetXdgDataDirs();
+    void testRestrictedResources();
+    void testThreads();
 
 private:
     QString m_kdehome;
