@@ -38,13 +38,6 @@ QTEST_KDEMAIN_CORE( KUrlTest )
 #include <stdio.h>
 #include <stdlib.h>
 
-// needed to build a valid path on windows
-#ifdef Q_OS_WIN
-#define LOCAL_FILE_PREFIX "C:"
-#else
-#define LOCAL_FILE_PREFIX
-#endif
-
 void KUrlTest::testEmptyURL()
 {
   KUrl emptyURL;
@@ -101,8 +94,8 @@ void KUrlTest::testSetQuery()
   QCOMPARE( url1.query(), QString("?foo=bar") );
   url1.setQuery( "toto=titi&kde=rocks" );
   QCOMPARE( url1.query(), QString("?toto=titi&kde=rocks") );
-  url1.setQuery( "?kde=rocks&a=b" );
-  QCOMPARE( url1.query(), QString("?kde=rocks&a=b") );
+  url1.setQuery( "?kde%20rocks&a=b" ); // must be encoded already, as documented
+  QCOMPARE( url1.query(), QString("?kde%20rocks&a=b") ); // encoded, as documented
   url1.setQuery( "?" );
   QCOMPARE( url1.query(), QString("?") );
   url1.setQuery( "" );
@@ -234,8 +227,12 @@ void KUrlTest::testSimpleMethods() // to test parsing, mostly
 
   u1 = "file:///home/dfaure/my#%2f";
   url1 = u1;
-  // KDE3: was %2f, but this is OK too
+  // KDE3: was %2f, Qt-4.0 to 4.4: #/, bad. 4.5: %2f again, good
+#if QT_VERSION < 0x040500
   QCOMPARE( url1.url(), QString("file:///home/dfaure/my#/") );
+#else
+  QCOMPARE( url1.url(), QString("file:///home/dfaure/my#%2f") );
+#endif
   QVERIFY( url1.hasRef() );
   QVERIFY( url1.hasHTMLRef() );
   QVERIFY( !url1.hasSubUrl() );
@@ -325,7 +322,7 @@ void KUrlTest::testSimpleMethods() // to test parsing, mostly
 
   //NOTE this test should be ran in UTF8 locale
   KUrl percentEncodedQuery( "http://mail.yandex.ru/message_part/%D0%9A%D1%80%D0%B8%D1%82%D0%B5%D1%80%D0%B8%D0%B8%20%D0%BE%D1%86%D0%B5%D0%BD%D0%B8%D0%B2%D0%B0%D0%BD%D0%B8%D1%8F%20%D0%BE%D1%80%D0%BB%D0%BE%D0%B2%D0%BE%D0%B9.rar?hid=1.1&mid=391.56424458.99241672611486679803334485488&name=%D0%9A%D1%80%D0%B8%D1%82%D0%B5%D1%80%D0%B8%D0%B8%20%D0%BE%D1%86%D0%B5%D0%BD%D0%B8%D0%B2%D0%B0%D0%BD%D0%B8%D1%8F%20%D0%BE%D1%80%D0%BB%D0%BE%D0%B2%D0%BE%D0%B9.rar" );
-  QCOMPARE( percentEncodedQuery.prettyUrl(), QString::fromUtf8("http://mail.yandex.ru/message_part/Критерии оценивания орловой.rar?hid=1.1&mid=391.56424458.99241672611486679803334485488&name=Критерии оценивания орловой.rar"));
+  QCOMPARE( percentEncodedQuery.prettyUrl(), QString::fromUtf8("http://mail.yandex.ru/message_part/Критерии оценивания орловой.rar?hid=1.1&mid=391.56424458.99241672611486679803334485488&name=%D0%9A%D1%80%D0%B8%D1%82%D0%B5%D1%80%D0%B8%D0%B8%20%D0%BE%D1%86%D0%B5%D0%BD%D0%B8%D0%B2%D0%B0%D0%BD%D0%B8%D1%8F%20%D0%BE%D1%80%D0%BB%D0%BE%D0%B2%D0%BE%D0%B9.rar"));
 
 #ifdef Q_WS_WIN
 #ifdef Q_CC_MSVC
@@ -341,6 +338,9 @@ void KUrlTest::testSimpleMethods() // to test parsing, mostly
   tilde.setPath( QString::fromUtf8( "~%1/Matériel" ).arg( userName ) );
   QString homeDir = currentUser.homeDir();
   QCOMPARE( tilde.url(), QString("file://%1/Mat%C3%A9riel").arg(homeDir));
+  tilde = KUrl("http://foo.bar/index.html");
+  tilde.setPath( "~slajsjdlsjd/test.html" );
+  QCOMPARE( tilde.url(), QString("http://foo.bar/~slajsjdlsjd/test.html"));
 #endif
 }
 
@@ -527,6 +527,12 @@ void KUrlTest::testPathAndQuery()
   QCOMPARE( tobi1.query(), QString("?another&query") );
   QCOMPARE( tobi1.path(), QString("another/path") ); // without trailing slash
   QCOMPARE( tobi1.encodedPathAndQuery(), QString( "another/path?another&query" ) );
+  tobi1.setEncodedPathAndQuery("and%20another%2Bpath%2E?bug=%2B258301&query%2Bword");
+  QCOMPARE( tobi1.query(), QString("?bug=%2B258301&query%2Bword") ); // encoded
+  QCOMPARE( tobi1.queryItem("bug"), QString("+258301") ); // decoded
+  QCOMPARE( tobi1.path(), QString("and another+path.") ); // decoded
+  QCOMPARE( tobi1.encodedPath().constData(), "and%20another+path." ); // from QUrl. It only encodes the space.
+  QCOMPARE( tobi1.encodedPathAndQuery(), QString( "and%20another+path.?bug=%2B258301&query%2Bword" ) );
 
   tobi1 = "http://host.net/path/#no-query";
   QCOMPARE( tobi1.encodedPathAndQuery(), QString( "/path/" ) );
@@ -549,6 +555,9 @@ void KUrlTest::testPathAndQuery()
 
   KUrl maelcum(QString::fromUtf8("http://a.b.c/äöu"));
   QCOMPARE(maelcum.encodedPathAndQuery(), QString("/%C3%A4%C3%B6u"));
+
+  KUrl gof("file:%2Ftmp%2Fkde-ogoffart%2Fkmail"); // weird URL, but well ;)
+  QCOMPARE(gof.path(), QString("/tmp/kde-ogoffart/kmail"));
 }
 
 void KUrlTest::testUpUrl()
@@ -628,6 +637,20 @@ void KUrlTest::testSetFileName() // and addPath
   KUrl lastUrl = lst.last();
   QString dir = lastUrl.directory();
   QCOMPARE(  dir, QString("/dir1/dir2") );
+
+  // files without directories
+  KUrl singleFile( "foo.txt" );
+  QCOMPARE( singleFile.path(), QString("foo.txt") );
+  QCOMPARE( singleFile.pathOrUrl(), QString("foo.txt") );
+
+  QString pre;
+#ifdef Q_OS_WIN
+  // On Windows we explicitly prepend a slash, see KUrl::setPath
+  pre = "/";
+#endif
+  singleFile.setFileName( "bar.bin" );
+  QCOMPARE( singleFile.path(), pre + QString("bar.bin") );
+  QCOMPARE( singleFile.pathOrUrl(), pre + QString("bar.bin") );
 }
 
 void KUrlTest::testDirectory()
@@ -676,17 +699,31 @@ void KUrlTest::testDirectory()
 
 void KUrlTest::testPrettyURL()
 {
-  KUrl notPretty("http://ferret.lmh.ox.ac.uk/%7Ekdecvs/");
-  QCOMPARE( notPretty.prettyUrl(), QString("http://ferret.lmh.ox.ac.uk/~kdecvs/") );
-  KUrl notPretty2("file:/home/test/directory%20with%20spaces");
-  QCOMPARE( notPretty2.prettyUrl(), QString("file:///home/test/directory with spaces") );
+  KUrl tildeInPath("http://ferret.lmh.ox.ac.uk/%7Ekdecvs/");
+  QCOMPARE(tildeInPath.prettyUrl(), QString("http://ferret.lmh.ox.ac.uk/~kdecvs/"));
+  // Tilde should not be re-encoded, see end of 2.3 in rfc3986
+  QCOMPARE(KUrl(tildeInPath.prettyUrl()).url(), QString("http://ferret.lmh.ox.ac.uk/~kdecvs/"));
+
+  KUrl spaceInPath("file:/home/test/directory%20with%20spaces");
+  QCOMPARE(spaceInPath.prettyUrl(), QString("file:///home/test/directory with spaces"));
+  QCOMPARE(KUrl(spaceInPath.prettyUrl()).url(), QString("file:///home/test/directory%20with%20spaces"));
+
+  KUrl plusInPath("http://slashdot.org/~RAMMS%2BEIN/"); // #232008
+  QCOMPARE(plusInPath.prettyUrl(), QString::fromLatin1("http://slashdot.org/~RAMMS+EIN/"));
+  QCOMPARE(KUrl(plusInPath.prettyUrl()).url(), QString::fromLatin1("http://slashdot.org/~RAMMS+EIN/"));
 
   KUrl notPretty3("fish://foo/%23README%23");
   QCOMPARE( notPretty3.prettyUrl(), QString("fish://foo/%23README%23") );
+
   KUrl url15581("http://alain.knaff.linux.lu/bug-reports/kde/spaces in url.html");
   QCOMPARE( url15581.prettyUrl(), QString("http://alain.knaff.linux.lu/bug-reports/kde/spaces in url.html") );
   QCOMPARE( url15581.url(), QString("http://alain.knaff.linux.lu/bug-reports/kde/spaces%20in%20url.html") );
 
+  KUrl newLineInQuery("http://localhost/?a=foo%0A%0Abar%20baz&b=foo%0Abar%21%3F");
+  QCOMPARE( newLineInQuery.prettyUrl(), QString("http://localhost/?a=foo%0A%0Abar%20baz&b=foo%0Abar%21%3F") );
+  KUrl nonUtf8Query("http://kde.org/?a=test%C2%A0foo%A0%A0%A0%A0bar");
+  QCOMPARE( nonUtf8Query.prettyUrl(), QString("http://kde.org/?a=test%C2%A0foo%A0%A0%A0%A0bar") );
+
   // KDE3 test was for parsing "percentage%in%url.html", but this is not supported; too broken.
   KUrl url15581bis("http://alain.knaff.linux.lu/bug-reports/kde/percentage%25in%25url.html");
   QCOMPARE( url15581bis.prettyUrl(), QString("http://alain.knaff.linux.lu/bug-reports/kde/percentage%25in%25url.html") );
@@ -696,6 +733,10 @@ void KUrlTest::testPrettyURL()
   QCOMPARE( urlWithPass.pass(), QString::fromLatin1( "password" ) );
   QCOMPARE( urlWithPass.prettyUrl(), QString::fromLatin1( "ftp://user@ftp.kde.org/path" ) );
 
+  KUrl urlWithPassAndNoUser("ftp://:password@ftp.kde.org/path");
+  QCOMPARE( urlWithPassAndNoUser.pass(), QString::fromLatin1( "password" ) );
+  QCOMPARE( urlWithPassAndNoUser.prettyUrl(), QString::fromLatin1( "ftp://ftp.kde.org/path" ) );
+
   KUrl xmppUri("xmpp:ogoffart@kde.org");
   QCOMPARE( xmppUri.prettyUrl(), QString::fromLatin1( "xmpp:ogoffart@kde.org" ) );
 
@@ -717,6 +758,9 @@ void KUrlTest::testPrettyURL()
   QCOMPARE(openWithUrl.prettyUrl(), QString::fromLatin1("kate --use %25U"));
   QCOMPARE(openWithUrl.path(), QString::fromLatin1("kate --use %U"));
   QCOMPARE(openWithUrl.pathOrUrl(), QString::fromLatin1("kate --use %25U")); // caused #153894; better not use KUrl for this.
+
+  KUrl ipv6Address( "http://[::ffff:129.144.52.38]:81/index.html" );
+  QCOMPARE( ipv6Address.prettyUrl(), QString::fromLatin1( "http://[::ffff:129.144.52.38]:81/index.html" ) );
 }
 
 void KUrlTest::testIsRelative()
@@ -732,6 +776,13 @@ void KUrlTest::testIsRelative()
   QCOMPARE(something.url(), QString("something"));
   QCOMPARE(something.protocol(), QString());
   QVERIFY(!something.isLocalFile());
+
+  // Now let's test QUrl::isRelative.
+  QVERIFY(!KUrl("file:///blah").isRelative());
+  QVERIFY(!KUrl("/blah").isRelative());
+  QVERIFY(KUrl("blah").isRelative());
+  QVERIFY(!KUrl("http://www.kde.org").isRelative());
+  QVERIFY(KUrl("foo/bar").isRelative());
 }
 
 void KUrlTest::testRelativePath()
@@ -988,6 +1039,10 @@ void KUrlTest::testBaseURL() // those are tests for the KUrl(base,relative) cons
      KUrl waba2( waba1, "");
      QCOMPARE( waba2.url(), QString("http://www.website.com/directory/?hello#ref") );
   }
+  {
+      KUrl waba2( waba1, "#%72%22method"); // #243217
+     QCOMPARE( waba2.url(), QString("http://www.website.com/directory/?hello#%72%22method") );
+  }
   {
      KUrl base( "http://faure@www.kde.org" ); // no path
      KUrl waba2( base, "filename.html");
@@ -1071,6 +1126,25 @@ void KUrlTest::testBaseURL() // those are tests for the KUrl(base,relative) cons
   QVERIFY(dxOffEagle.isValid());
   //QEXPECT_FAIL("","Issue N183630, task ID 183874", Continue); // Fixed by _setEncodedUrl
   QCOMPARE(dxOffEagle.url(), QString("http://something/newpage.html?%5B%7B%22foo:%20bar%22%7D%5D") );
+  QCOMPARE(dxOffEagle.prettyUrl(), QString("http://something/newpage.html?%5B%7B%22foo:%20bar%22%7D%5D") );
+
+  // QtSw issue 243557
+  QByteArray tsdgeos("http://google.com/c?c=Translation+%C2%BB+trunk|");
+  QUrl tsdgeosQUrl;
+  tsdgeosQUrl.setEncodedUrl(tsdgeos, QUrl::TolerantMode);
+  QVERIFY(tsdgeosQUrl.isValid()); // failed in Qt-4.4, works in Qt-4.5
+  QByteArray tsdgeosExpected("http://google.com/c?c=Translation+%C2%BB+trunk%7C");
+  //QCOMPARE(tsdgeosQUrl.toEncoded(), tsdgeosExpected); // unusable output from qtestlib...
+  QCOMPARE(QString(tsdgeosQUrl.toEncoded()), QString(tsdgeosExpected));
+
+  KUrl tsdgeosUrl(tsdgeos);
+  QCOMPARE(tsdgeosUrl.url(), QString(tsdgeosExpected));
+
+  QByteArray pipesAgain("http://translate.google.com/translate_t#en|uk|demo");
+  QUrl pipesUrl;
+  pipesUrl.setEncodedUrl(pipesAgain, QUrl::TolerantMode);
+  QVERIFY(pipesUrl.isValid());
+  QCOMPARE(QString(pipesUrl.toEncoded()), QString("http://translate.google.com/translate_t#en%7Cuk%7Cdemo"));
 
   // Shows up in nspluginviewer/flash
   QString flashRel = "javascript:window.location+\"__flashplugin_unique__\"";
@@ -1081,11 +1155,47 @@ void KUrlTest::testBaseURL() // those are tests for the KUrl(base,relative) cons
   QCOMPARE(flashComposed.url(), QString("javascript:window.location+%22__flashplugin_unique__%22"));
 }
 
+void KUrlTest::testSetEncodedFragment_data()
+{
+    QTest::addColumn<QByteArray>("base");
+    QTest::addColumn<QByteArray>("fragment");
+    QTest::addColumn<QByteArray>("expected");
+    typedef QByteArray BA;
+    QTest::newRow("basic test") << BA("http://www.kde.org") << BA("abc") << BA("http://www.kde.org#abc");
+    QTest::newRow("initial url has fragment") << BA("http://www.kde.org#old") << BA("new") << BA("http://www.kde.org#new");
+    QTest::newRow("encoded fragment") << BA("http://www.kde.org") << BA("a%20c") << BA("http://www.kde.org#a%20c");
+    QTest::newRow("with #") << BA("http://www.kde.org") << BA("a#b") << BA("http://www.kde.org#a#b");
+    QTest::newRow("empty") << BA("http://www.kde.org") << BA("") << BA("http://www.kde.org#");
+}
+
+void KUrlTest::testSetEncodedFragment()
+{
+    // Bug fixed in 4.5.1 by Thiago
+#if QT_VERSION < 0x040501
+    QSKIP("Bug in Qt-4.4/4.5-rc1: setEncodedFragment doesn't work if the initial url has no fragment", SkipAll);
+#endif
+
+    QFETCH(QByteArray, base);
+    QFETCH(QByteArray, fragment);
+    QFETCH(QByteArray, expected);
+    QUrl u;
+    u.setEncodedUrl(base, QUrl::TolerantMode);
+    QVERIFY(u.isValid());
+    u.setEncodedFragment(fragment);
+    QVERIFY(u.isValid());
+    QCOMPARE(QString::fromLatin1(u.toEncoded()), QString::fromLatin1(expected));
+}
+
 void KUrlTest::testSubURL()
 {
   QString u1 = "file:/home/dfaure/my%20tar%20file.tgz#gzip:/#tar:/#myref";
   KUrl url1(u1);
-  QCOMPARE( url1.url(), QString("file:///home/dfaure/my%20tar%20file.tgz#gzip:/%23tar:/%23myref") ); // KDE3 would say # 3 times
+  // KDE3: was #,#,#, Qt-4.0 to 4.4: #,%23,%23 . 4.5: #,#,#, good
+#if QT_VERSION < 0x040500
+  QCOMPARE( url1.url(), QString("file:///home/dfaure/my%20tar%20file.tgz#gzip:/%23tar:/%23myref") );
+#else
+  QCOMPARE( url1.url(), QString("file:///home/dfaure/my%20tar%20file.tgz#gzip:/#tar:/#myref") );
+#endif
   QVERIFY( url1.hasRef() );
   QVERIFY( !url1.isLocalFile() );  // Not strictly local!
   QVERIFY( url1.hasSubUrl() );
@@ -1099,12 +1209,11 @@ void KUrlTest::testSubURL()
   QCOMPARE( splitted[1].url(), QString("gzip:/#myref") );
   QCOMPARE( splitted[2].url(), QString("tar:/#myref") );
 
-  QSKIP( "Multiple sub urls not supported at the moment, due to #-escaping", SkipSingle );
-
-  // FAILS, due to the #->%23 escaping, see the comment in KUrl::join.
-  // But do we really want to keep the suburl stuff?
-  KUrl rejoined = KUrl::join( splitted );
-  QCOMPARE( rejoined.url(), url1.url() );
+#if QT_VERSION < 0x040500
+  QSKIP( "Multiple sub urls not supported with Qt < 4.5", SkipSingle );
+#endif
+  KUrl rejoined = KUrl::join(splitted);
+  QCOMPARE(rejoined.url(), url1.url());
 
   u1 = "error:/?error=14&errText=Unknown%20host%20asdfu.adgi.sdfgoi#http://asdfu.adgi.sdfgoi";
   url1 = u1;
@@ -1116,7 +1225,7 @@ void KUrlTest::testSubURL()
 
   u1 = "file:/home/dfaure/my%20tar%20file.tgz#gzip:/#tar:/";
   url1 = u1;
-  QCOMPARE( url1.url(), QString("file:///home/dfaure/my%20tar%20file.tgz#gzip:/%23tar:/") ); // KDE3: #tar
+  QCOMPARE( url1.url(), QString("file:///home/dfaure/my%20tar%20file.tgz#gzip:/#tar:/") );
   QVERIFY( url1.hasRef() );
   QVERIFY( !url1.hasHTMLRef() );
   QVERIFY( url1.hasSubUrl() );
@@ -1125,7 +1234,7 @@ void KUrlTest::testSubURL()
 
   u1 = "file:///home/dfaure/my%20tar%20file.tgz#gzip:/#tar:/";
   url1 = u1;
-  QCOMPARE( url1.url(), QString("file:///home/dfaure/my%20tar%20file.tgz#gzip:/%23tar:/") ); // KDE3: #tar
+  QCOMPARE( url1.url(), QString("file:///home/dfaure/my%20tar%20file.tgz#gzip:/#tar:/") );
   QVERIFY( url1.hasRef() );
   QVERIFY( !url1.hasHTMLRef() );
   QVERIFY( url1.hasSubUrl() );
@@ -1146,12 +1255,25 @@ void KUrlTest::testSubURL()
 
   u1 = "file:/home/dfaure/my%20tar%20file.tgz#gzip:/#tar:/README";
   url1 = u1;
-  QCOMPARE( url1.url(), QString("file:///home/dfaure/my%20tar%20file.tgz#gzip:/%23tar:/README") );
+  QCOMPARE( url1.url(), QString("file:///home/dfaure/my%20tar%20file.tgz#gzip:/#tar:/README") );
   QVERIFY( url1.hasRef() );
   QVERIFY( !url1.hasHTMLRef() );
   QVERIFY( url1.hasSubUrl() );
   QCOMPARE( url1.htmlRef(), QString("") );
-  QCOMPARE( url1.upUrl().url(), QString("file:///home/dfaure/my%20tar%20file.tgz#gzip:/%23tar:/") );
+  const KUrl::List url1Splitted = KUrl::split( url1 );
+  QCOMPARE( url1Splitted.count(), 3 );
+  //kDebug() << url1Splitted.toStringList();
+  QCOMPARE(url1Splitted[0].url(), QString("file:///home/dfaure/my%20tar%20file.tgz"));
+  QCOMPARE(url1Splitted[1].url(), QString("gzip:/"));
+  QCOMPARE(url1Splitted[2].url(), QString("tar:/README"));
+  const KUrl url1Rejoined = KUrl::join(url1Splitted);
+  // Bug fixed in 4.5.1 by Thiago
+#if QT_VERSION < 0x040501
+  QSKIP("Bug in Qt-4.4/4.5-rc1: setEncodedFragment doesn't work if the initial url has no fragment", SkipAll);
+#endif
+  QCOMPARE(url1Rejoined.url(), url1.url());
+  QCOMPARE(url1.upUrl().url(), QString("file:///home/dfaure/my%20tar%20file.tgz#gzip:/#tar:/"));
+
 }
 
 void KUrlTest::testSetUser()
@@ -1183,12 +1305,7 @@ void KUrlTest::testSetUser()
 
 void KUrlTest::testComparisons()
 {
-  /// Comparisons
-  QString ucmp1 = "ftp://ftp.de.kde.org/dir";
-  QString ucmp2 = "ftp://ftp.de.kde.org/dir/";
-  QVERIFY( !urlcmp(ucmp1,ucmp2) );
-
-  /* QUrl version of it */
+  /* QUrl version of urlcmp */
   QUrl u1( "ftp://ftp.de.kde.org/dir" );
   QUrl u2( "ftp://ftp.de.kde.org/dir/" );
   QUrl::FormattingOptions options = QUrl::None;
@@ -1200,7 +1317,32 @@ void KUrlTest::testComparisons()
   bool same = str1 == str2;
   QVERIFY( same );
 
-  QVERIFY( urlcmp(ucmp1,ucmp2,KUrl::CompareWithoutTrailingSlash) ); //only slash difference, ignore_trailing
+  QString ucmp1 = "ftp://ftp.de.kde.org/dir";
+  QString ucmp2 = "ftp://ftp.de.kde.org/dir/";
+  QVERIFY(!urlcmp(ucmp1, ucmp2));
+  QVERIFY(urlcmp(ucmp1, ucmp2, KUrl::CompareWithoutTrailingSlash)); //only slash difference, ignore_trailing
+
+  {
+      KUrl u1(ucmp1);
+      KUrl u2(ucmp2);
+      QVERIFY(!u1.equals(u2));
+      QVERIFY(u1.equals(u2, KUrl::CompareWithoutTrailingSlash));
+      QVERIFY(u1.equals(u2, KUrl::CompareWithoutTrailingSlash|KUrl::AllowEmptyPath));
+  }
+
+  // Special case: no path vs '/'.
+  {
+      QString str1 = QString::fromLatin1( "ftp://ftp.de.kde.org" );
+      QString str2 = QString::fromLatin1( "ftp://ftp.de.kde.org/" );
+      QVERIFY(!urlcmp(str1, str2));
+      QVERIFY(!urlcmp(str1, str2, KUrl::CompareWithoutTrailingSlash)); // empty path != '/'
+      QVERIFY(urlcmp(str1, str2, KUrl::CompareWithoutTrailingSlash|KUrl::AllowEmptyPath));
+      KUrl u1(str1);
+      KUrl u2(str2);
+      QVERIFY(!u1.equals(u2));
+      QVERIFY(!u1.equals(u2, KUrl::CompareWithoutTrailingSlash));
+      QVERIFY(u1.equals(u2, KUrl::CompareWithoutTrailingSlash|KUrl::AllowEmptyPath));
+  }
 
   QString ucmp3 = "ftp://ftp.de.kde.org/dir/#";
   QVERIFY( !urlcmp(ucmp2,ucmp3) ); // (only hash difference)
@@ -1213,7 +1355,7 @@ void KUrlTest::testComparisons()
   QVERIFY( !urlcmp("file",ucmp1) ); // (malformed, not empty)
   QVERIFY( !urlcmp("file",ucmp1,KUrl::CompareWithoutFragment) ); // (malformed, not empty)
 
-  KUrl ftpUrl ( "ftp://ftp.de.kde.org" );
+  KUrl ftpUrl( "ftp://ftp.de.kde.org" );
   QCOMPARE( ftpUrl.path(), QString());
   ftpUrl = "ftp://ftp.de.kde.org/";
   QVERIFY( ftpUrl.isParentOf( KUrl("ftp://ftp.de.kde.org/host/subdir/") ) );
@@ -1231,6 +1373,8 @@ void KUrlTest::testStreaming()
 {
   // Streaming operators
   KUrl origURL( "http://www.website.com/directory/?#ref" );
+  KUrl urlWithPassAndNoUser("ftp://:password@ftp.kde.org/path");
+  KUrl accentuated(QString::fromUtf8("trash:/été"));
   KUrl empty( "" );
   KUrl invalid( "ptal://mlc:usb" );
   QVERIFY( !invalid.isValid() );
@@ -1239,6 +1383,8 @@ void KUrlTest::testStreaming()
   {
       QDataStream stream( &buffer, QIODevice::WriteOnly );
       stream << origURL
+             << urlWithPassAndNoUser
+             << accentuated
              << empty
              << invalid
              << waba1; // the IPv6 one
@@ -1248,6 +1394,10 @@ void KUrlTest::testStreaming()
       KUrl restoredURL;
       stream >> restoredURL; // streaming valid url
       QCOMPARE( restoredURL.url(), origURL.url() );
+      stream >> restoredURL; // streaming url with pass an no user
+      QCOMPARE( restoredURL.url(), urlWithPassAndNoUser.url() );
+      stream >> restoredURL; // streaming valid url with accents
+      QCOMPARE( restoredURL.url(), accentuated.url() );
       stream >> restoredURL; // streaming empty url
       QVERIFY( !restoredURL.isValid() );
       QVERIFY( restoredURL.isEmpty() );
@@ -1369,26 +1519,31 @@ void KUrlTest::testMoreBrokenStuff()
   weird = "http://strange<username>@strange<hostname>/";
   QVERIFY( !weird.isValid() );
 
-  weird = "http://strange<username>@ok_hostname/";
+  {
+      QUrl url;
+      url.setUrl("http://strange<username>@hostname/", QUrl::TolerantMode);
+      QVERIFY(url.isValid());
+  }
+  weird = "http://strange<username>@hostname/";
   QVERIFY( weird.isValid() ); // KDE3: was valid. Fixed by _setEncodedUrl.
-  QCOMPARE( weird.host(), QString("ok_hostname") );
+  QCOMPARE( weird.host(), QString("hostname") );
 
   weird = "http://strange;hostname/";
-  QVERIFY( weird.isValid() ); // KDE3: was invalid. bah*2.
+  QVERIFY( !weird.isValid() );
 
   weird = "http://strange;username@strange;hostname/";
-  QVERIFY( weird.isValid() ); // KDE3: was invalid. bah*3.
+  QVERIFY( !weird.isValid() );
 
-  weird = "http://strange;username@ok_hostname/";
+  weird = "http://strange;username@hostname/";
   QVERIFY( weird.isValid() );
-  QCOMPARE( weird.host(), QString("ok_hostname") );
+  QCOMPARE( weird.host(), QString("hostname") );
 
   weird = "http://strange;username:password@strange;hostname/";
-  QVERIFY( weird.isValid() ); // KDE3: was invalid
+  QVERIFY( !weird.isValid() );
 
-  weird = "http://strange;username:password@ok_hostname/";
+  weird = "http://strange;username:password@hostname/";
   QVERIFY( weird.isValid() );
-  QCOMPARE( weird.host(), QString("ok_hostname") );
+  QCOMPARE( weird.host(), QString("hostname") );
 
   weird = "http://[strange;hostname]/";
   QVERIFY( !weird.isValid() );
@@ -1416,15 +1571,10 @@ void KUrlTest::testMoreBrokenStuff()
   KUrl broken;
   broken = "ptal://mlc:usb:PC_970";
   QVERIFY( !broken.isValid() );
-  QEXPECT_FAIL( "", "QUrl doesn't provide the initial string if it's an invalid url...", Continue );
-  QCOMPARE( broken.url(), QString("ptal://mlc:usb:PC_970") ); // FAILS - but we need it...
-
+  QVERIFY (!broken.errorString().isEmpty());
+  //QCOMPARE( broken.url(), QString("ptal://mlc:usb:PC_970") ); // QUrl doesn't provide the initial string if it's an invalid url
   QUrl brokenUrl( "ptal://mlc:usb:PC_970" );
   QVERIFY( !brokenUrl.isValid() );
-  QEXPECT_FAIL( "", "QUrl doesn't provide the initial string if it's an invalid url...", Continue );
-  QCOMPARE( brokenUrl.toString(), QString("ptal://mlc:usb:PC_970") );
-  QEXPECT_FAIL( "", "QUrl doesn't provide the initial string if it's an invalid url...", Continue );
-  QCOMPARE( brokenUrl.toEncoded(), QByteArray("ptal://mlc:usb:PC_970") );
 
   QUrl dxOffEagle( "http://something/newpage.html?[{\"foo: bar\"}]", QUrl::TolerantMode);
   QVERIFY(dxOffEagle.isValid());
@@ -1436,16 +1586,19 @@ void KUrlTest::testMoreBrokenStuff()
 
   QUrl dxOffEagle3;
   dxOffEagle3.setEncodedUrl( "http://something/newpage.html?[{\"foo: bar\"}]", QUrl::TolerantMode);
+#if QT_VERSION < 0x040500
   QEXPECT_FAIL("","Issue N183630, task ID 183874; works with setUrl so we do that in _setEncodedUrl now", Continue);
+#endif
   QVERIFY(dxOffEagle3.isValid());
-  //QEXPECT_FAIL("","Issue N183630, task ID 183874", Continue);
-  //QCOMPARE(dxOffEagle.toEncoded(), dxOffEagle3.toEncoded());
+  QCOMPARE(dxOffEagle.toEncoded(), dxOffEagle3.toEncoded());
 
   QUrl javascript;
   javascript.setUrl("javascript:window.location+\"__flashplugin_unique__\"", QUrl::TolerantMode);
   QVERIFY(javascript.isValid());
   javascript.setEncodedUrl("javascript:window.location+\"__flashplugin_unique__\"", QUrl::TolerantMode);
+#if QT_VERSION < 0x040500
   QEXPECT_FAIL("","Issue N183630, task ID 183874", Continue);
+#endif
   QVERIFY(javascript.isValid());
 }
 
@@ -1492,6 +1645,10 @@ void KUrlTest::testMailto()
   QCOMPARE( umail2.path(), QString("Faure David <faure@kde.org>") );
   QVERIFY( !KUrl::isRelativeUrl("mailto:faure@kde.org") );
 #endif
+
+  KUrl url183433("mailto:test[at]gmail[dot]com");
+  QCOMPARE(url183433.prettyUrl(), QString("mailto:test[at]gmail[dot]com"));
+  QCOMPARE(url183433.url(), QString("mailto:test[at]gmail[dot]com"));
 }
 
 void KUrlTest::testSmb()
@@ -1534,10 +1691,22 @@ void KUrlTest::testOtherProtocols()
   QCOMPARE( leo.path(), QString("text/html,http://www.invalid/" ) );
 
   KUrl ptal( "ptal://mlc:usb@PC_970" ); // User=mlc, password=usb, host=PC_970
+#if QT_VERSION >= 0x040600 && QT_VERSION <= 0x040602 // Hostnames with underscores were invalid in 4.6.0 to 4.6.2, then allowed again in e301c82693c33c0f96c6a756d15fe35a9d877443
+  QCOMPARE(ptal.url(), QString("ptal://mlc:usb@")); // The host "PC_970" is invalid according to STD3 validation
+  KUrl ptalSimpler("ptal://mlc:usb@pc123");
+  QCOMPARE(ptalSimpler.url(), QString("ptal://mlc:usb@pc123"));
+#else
+  QUrl ptal_qurl;
+  ptal_qurl.setUrl("ptal://mlc:usb@PC_970", QUrl::TolerantMode);
+  QVERIFY(ptal_qurl.isValid());
+  QCOMPARE(QString::fromLatin1(ptal_qurl.toEncoded()), QString::fromLatin1("ptal://mlc:usb@pc_970"));
+  QCOMPARE( ptal_qurl.host(), QString("pc_970") );
+
   QVERIFY( ptal.isValid() );
   QCOMPARE( ptal.host(), QString("pc_970") );
   QCOMPARE( ptal.user(), QString("mlc") );
   QCOMPARE( ptal.pass(), QString("usb") );
+#endif
 }
 
 void KUrlTest::testUtf8()
@@ -1581,6 +1750,12 @@ void KUrlTest::testUtf8()
   KUrl wkai(QString::fromUtf8("/tmp/魔"));
   QCOMPARE(wkai.url(), QString("file:///tmp/%E9%AD%94"));
   QCOMPARE(wkai.prettyUrl(), QString::fromUtf8("file:///tmp/魔"));
+
+  // Show that the character "fraction slash" (U+2044) cannot appear in url(),
+  // so it's ok to use that to encode urls as filenames (e.g. in kio_http cache)
+  KUrl fractionSlash(QString("http://kde.org/a")+QChar(0x2044)+"b");
+  QCOMPARE(fractionSlash.url(), QString("http://kde.org/a%E2%81%84b"));
+  QCOMPARE(fractionSlash.prettyUrl(), QString("http://kde.org/a")+QChar(0x2044)+"b");
 }
 
 void KUrlTest::testOtherEncodings()
@@ -1605,8 +1780,8 @@ void KUrlTest::testOtherEncodings()
 void KUrlTest::testPathOrURL()
 {
   // passing path or url to the constructor: both work
-  KUrl uloc( LOCAL_FILE_PREFIX"/home/dfaure/konqtests/Mat%C3%A9riel" );
-  QCOMPARE( uloc.path(), QString(LOCAL_FILE_PREFIX"/home/dfaure/konqtests/Mat%C3%A9riel") );
+  KUrl uloc( "/home/dfaure/konqtests/Mat%C3%A9riel" );
+  QCOMPARE( uloc.path(), QString("/home/dfaure/konqtests/Mat%C3%A9riel") );
   uloc = KUrl( "http://www.kde.org" );
   QCOMPARE( uloc.pathOrUrl(), uloc.url() );
   QCOMPARE( uloc.pathOrUrl(KUrl::AddTrailingSlash), QString("http://www.kde.org/") );
@@ -1631,7 +1806,7 @@ void KUrlTest::testPathOrURL()
 #endif
 
   // pathOrUrl tests
-  uloc = KUrl( LOCAL_FILE_PREFIX"/home/dfaure/konqtests/Mat%C3%A9riel" );
+  uloc = KUrl( "/home/dfaure/konqtests/Mat%C3%A9riel" );
   QCOMPARE( uloc.pathOrUrl(), uloc.path() );
   uloc = "http://www.kde.org";
   QCOMPARE( uloc.url(), QString("http://www.kde.org") );
@@ -1639,8 +1814,8 @@ void KUrlTest::testPathOrURL()
   QCOMPARE( uloc.pathOrUrl(), QString::fromUtf8("file:///home/dfaure/konq tests/Matériel#ref" ) );
   uloc = "file:///home/dfaure/konq%20tests/Mat%C3%A9riel?query";
   QCOMPARE( uloc.pathOrUrl(), QString::fromUtf8("file:///home/dfaure/konq tests/Matériel?query" ) );
-  uloc = KUrl( LOCAL_FILE_PREFIX"/home/dfaure/file#with#hash" );
-  QCOMPARE( uloc.pathOrUrl(), QString(LOCAL_FILE_PREFIX"/home/dfaure/file#with#hash" ) );
+  uloc = KUrl( "/home/dfaure/file#with#hash" );
+  QCOMPARE( uloc.pathOrUrl(), QString("/home/dfaure/file#with#hash" ) );
 
   // test creation of url from pathOrUrl
   uloc = KUrl( QString::fromUtf8("http://www.kde.org/home/andreas/täst") );
@@ -1661,11 +1836,11 @@ void KUrlTest::testAssignment()
 {
   // passing path or url to the constructor: both work
   KUrl uloc;
-  uloc = LOCAL_FILE_PREFIX"/home/dfaure/konqtests/Mat%C3%A9riel";
-  QCOMPARE( uloc.path(), QString(LOCAL_FILE_PREFIX"/home/dfaure/konqtests/Mat%C3%A9riel") );
+  uloc = "/home/dfaure/konqtests/Mat%C3%A9riel";
+  QCOMPARE( uloc.path(), QString("/home/dfaure/konqtests/Mat%C3%A9riel") );
   KUrl u2;
   u2 = uloc;
-  QCOMPARE( u2.path(), QString(LOCAL_FILE_PREFIX"/home/dfaure/konqtests/Mat%C3%A9riel") );
+  QCOMPARE( u2.path(), QString("/home/dfaure/konqtests/Mat%C3%A9riel") );
   uloc = "http://www.kde.org";
   QCOMPARE( uloc.pathOrUrl(), uloc.url() );
   uloc = QString("www.kde.org" );
@@ -1770,14 +1945,6 @@ void KUrlTest::testIdn()
   KUrl uwp( "http://%C3%A4.de" );
   QVERIFY( uwp.isValid() );
   QCOMPARE( thiago.url(), QString("http://xn--4ca.de") ); // as above
-
-#if QT_VERSION >= 0x040500  // was broken in 4.4
-  // #183720
-  const QByteArray init = ".kde.org";
-  const QString fromAce = QUrl::fromAce(init);
-  QCOMPARE( fromAce, QString(".kde.org"));
-  QCOMPARE( QUrl::toAce(fromAce), init );
-#endif
 }
 
 void KUrlTest::testUriMode()
@@ -1797,3 +1964,14 @@ void KUrlTest::testUriMode()
   url1 = "mailto:User@Host.COM?subject=Hello";
   QCOMPARE( url1.path(), QString("User@Host.COM") ); // KDE3: "User@host.com". Does it matter?
 }
+
+void KUrlTest::testToLocalFile()
+{
+  const QString localFile( "/tmp/print.pdf" );
+
+  const KUrl urlWithHost( "file://localhost/tmp/print.pdf" );
+  const KUrl urlWithoutHost( "file:///tmp/print.pdf" );
+
+  QCOMPARE( urlWithHost.toLocalFile(), localFile );
+  QCOMPARE( urlWithoutHost.toLocalFile(), localFile );
+}
