@@ -19,6 +19,7 @@
  **/
 
 #include "kbuildsycoca.h"
+#include "ksycoca_p.h"
 #include "kresourcelist.h"
 #include "vfolder_menu.h"
 
@@ -52,17 +53,7 @@
 #include <kaboutdata.h>
 #include <kcmdlineargs.h>
 #include <kcrash.h>
-#ifdef Q_OS_WIN
 #include <kmemfile.h>
-#endif
-
-#ifdef KBUILDSYCOCA_GUI // KBUILDSYCOCA_GUI is used on win32 to build
-                        // GUI version of kbuildsycoca, so-called "kbuildsycocaw".
-# include <qlabel.h>
-# include <kmessagebox.h>
-  bool silent;
-  bool showprogress;
-#endif
 
 #include <stdlib.h>
 #include <unistd.h>
@@ -173,11 +164,10 @@ KSycocaEntry::Ptr KBuildSycoca::createEntry(const QString &file, bool addToFacto
    return KSycocaEntry::Ptr();
 }
 
-// Callback for VFolderMenu
-void KBuildSycoca::slotCreateEntry(const QString &file, KService::Ptr *service)
+KService::Ptr KBuildSycoca::createService(const QString &path)
 {
-   KSycocaEntry::Ptr entry = createEntry(file, false);
-   *service = KService::Ptr::staticCast( entry );
+   KSycocaEntry::Ptr entry = createEntry(path, false);
+   return KService::Ptr::staticCast(entry);
 }
 
 // returns false if the database is up to date
@@ -300,13 +290,10 @@ bool KBuildSycoca::build()
      g_entryDict = serviceEntryDict;
      g_changed = false;
 
-     g_vfolder = new VFolderMenu(g_bsf);
+     g_vfolder = new VFolderMenu(g_bsf, this);
      if (!m_trackId.isEmpty())
         g_vfolder->setTrackId(m_trackId);
 
-     connect(g_vfolder, SIGNAL(newService(const QString &, KService::Ptr *)),
-             this, SLOT(slotCreateEntry(const QString &, KService::Ptr *)));
-
      VFolderMenu::SubMenu *kdeMenu = g_vfolder->parseMenu("applications.menu", true);
 
      KServiceGroup::Ptr entry = g_bsgf->addNew("/", kdeMenu->directoryFile, KServiceGroup::Ptr(), false);
@@ -316,9 +303,6 @@ bool KBuildSycoca::build()
      (void) existingResourceDirs();
      *g_allResourceDirs += g_vfolder->allDirectories();
 
-     disconnect(g_vfolder, SIGNAL(newService(const QString &, KService::Ptr *)),
-                this, SLOT(slotCreateEntry(const QString &, KService::Ptr *)));
-
      if (g_changed || !g_allEntries)
      {
         uptodate = false;
@@ -345,8 +329,7 @@ void KBuildSycoca::createMenu(const QString &caption_, const QString &name_, VFo
      if (directoryFile.isEmpty())
         directoryFile = subName+".directory";
      quint32 timeStamp = g_ctimeInfo->ctime(directoryFile);
-     if (!timeStamp)
-     {
+     if (!timeStamp) {
         timeStamp = KGlobal::dirs()->calcResourceHash( g_resource, directoryFile,
                                                        KStandardDirs::Recursive );
      }
@@ -409,16 +392,11 @@ bool KBuildSycoca::recreate()
   {
     fprintf(stderr, "kbuildsycoca4: ERROR creating database '%s'! %s\n",
       path.toLocal8Bit().data(), database.errorString().toLocal8Bit().data());
-#ifdef KBUILDSYCOCA_GUI // KBUILDSYCOCA_GUI is used on win32 to build
-                        // GUI version of kbuildsycoca, so-called "kbuildsycocaw".
-    if (!silent)
-      KMessageBox::error(0, i18n("Error creating database '%1'.\nCheck that the permissions are correct on the directory and the disk is not full.\n", path.toLocal8Bit().data()), i18n("KBuildSycoca"));
-#endif
     return false;
   }
 
-  m_str = new QDataStream ( &database );
-  m_str->setVersion(QDataStream::Qt_3_1);
+  QDataStream* str = new QDataStream(&database);
+  str->setVersion(QDataStream::Qt_3_1);
 
   kDebug(7021).nospace() << "Recreating ksycoca file (" << path << ", version " << KSycoca::version() << ")";
 
@@ -432,26 +410,22 @@ bool KBuildSycoca::recreate()
 
   if( build()) // Parse dirs
   {
-    save(); // Save database
-    if (m_str->status() != QDataStream::Ok) // ######## TODO: does this detect write errors, e.g. disk full?
+    save(str); // Save database
+    if (str->status() != QDataStream::Ok) // ######## TODO: does this detect write errors, e.g. disk full?
       database.abort(); // Error
-    delete m_str;
-    m_str = 0L;
+    delete str;
+    str = 0;
     if (!database.finalize())
     {
       fprintf(stderr, "kbuildsycoca4: ERROR writing database '%s'!\n", database.fileName().toLocal8Bit().data());
       fprintf(stderr, "kbuildsycoca4: Disk full?\n");
-#ifdef KBUILDSYCOCA_GUI
-      if (!silent)
-        KMessageBox::error(0, i18n("Error writing database '%1'.\nCheck that the permissions are correct on the directory and the disk is not full.\n", path.toLocal8Bit().data()), i18n("KBuildSycoca"));
-#endif
       return false;
     }
   }
   else
   {
-    delete m_str;
-    m_str = 0L;
+    delete str;
+    str = 0;
     database.abort();
     if (bMenuTest)
        return true;
@@ -471,18 +445,18 @@ bool KBuildSycoca::recreate()
     if (g_vfolder)
         str << g_vfolder->allDirectories(); // Extra resource dirs
   }
-#ifdef Q_OS_WIN
-  KMemFile::fileContentsChanged(path);
-#endif
+  if (d->m_sycocaStrategy == KSycocaPrivate::StrategyMemFile)
+     KMemFile::fileContentsChanged(path);
+
   return true;
 }
 
-void KBuildSycoca::save()
+void KBuildSycoca::save(QDataStream* str)
 {
    // Write header (#pass 1)
-   m_str->device()->seek(0);
+   str->device()->seek(0);
 
-   (*m_str) << (qint32) KSycoca::version();
+   (*str) << (qint32) KSycoca::version();
    KSycocaFactory * servicetypeFactory = 0;
    KBuildMimeTypeFactory * mimeTypeFactory = 0;
    KBuildServiceFactory * serviceFactory = 0;
@@ -499,39 +473,42 @@ void KBuildSycoca::save()
          mimeTypeFactory = static_cast<KBuildMimeTypeFactory *>( *factory );
       else if ( aId == KST_KServiceFactory )
          serviceFactory = static_cast<KBuildServiceFactory *>( *factory );
-      aOffset = (*factory)->offset();
-      (*m_str) << aId;
-      (*m_str) << aOffset;
+      aOffset = (*factory)->offset(); // not set yet, so always 0
+      (*str) << aId;
+      (*str) << aOffset;
    }
-   (*m_str) << (qint32) 0; // No more factories.
+   (*str) << (qint32) 0; // No more factories.
    // Write KDEDIRS
-   (*m_str) << KGlobal::dirs()->kfsstnd_prefixes();
-   (*m_str) << newTimestamp;
-   (*m_str) << KGlobal::locale()->language();
-   (*m_str) << KGlobal::dirs()->calcResourceHash("services", "update_ksycoca",
+   (*str) << KGlobal::dirs()->kfsstnd_prefixes();
+   (*str) << newTimestamp;
+   (*str) << KGlobal::locale()->language();
+   (*str) << KGlobal::dirs()->calcResourceHash("services", "update_ksycoca",
                                                  KStandardDirs::Recursive );
-   (*m_str) << (*g_allResourceDirs);
+   (*str) << (*g_allResourceDirs);
 
    // Calculate per-servicetype/mimetype data
    mimeTypeFactory->parseSubclasses();
    serviceFactory->postProcessServices();
 
+   // Here so that it's the last debug message
+   kDebug(7021) << "Saving";
+
    // Write factory data....
    for(KSycocaFactoryList::Iterator factory = factories()->begin();
        factory != factories()->end();
        ++factory)
    {
-      (*factory)->save(*m_str);
-      if (m_str->status() != QDataStream::Ok) // ######## TODO: does this detect write errors, e.g. disk full?
+      (*factory)->save(*str);
+      if (str->status() != QDataStream::Ok) // ######## TODO: does this detect write errors, e.g. disk full?
          return; // error
    }
 
-   int endOfData = m_str->device()->pos();
+   int endOfData = str->device()->pos();
 
    // Write header (#pass 2)
-   m_str->device()->seek(0);
+   str->device()->seek(0);
 
-   (*m_str) << (qint32) KSycoca::version();
+   (*str) << (qint32) KSycoca::version();
    for(KSycocaFactoryList::Iterator factory = factories()->begin();
        factory != factories()->end(); ++factory)
    {
@@ -539,13 +516,13 @@ void KBuildSycoca::save()
       qint32 aOffset;
       aId = (*factory)->factoryId();
       aOffset = (*factory)->offset();
-      (*m_str) << aId;
-      (*m_str) << aOffset;
+      (*str) << aId;
+      (*str) << aOffset;
    }
-   (*m_str) << (qint32) 0; // No more factories.
+   (*str) << (qint32) 0; // No more factories.
 
    // Jump to end of database
-   m_str->device()->seek(endOfData);
+   str->device()->seek(endOfData);
 }
 
 bool KBuildSycoca::checkDirTimestamps( const QString& dirname, const QDateTime& stamp, bool top )
@@ -652,10 +629,6 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
    options.add("global", ki18n("Create global database"));
    options.add("menutest", ki18n("Perform menu generation test run only"));
    options.add("track <menu-id>", ki18n("Track menu id for debug purposes"));
-#ifdef KBUILDSYCOCA_GUI
-   options.add("silent", ki18n("Silent - work without windows and stderr"));
-   options.add("showprogress", ki18n("Show progress information (even if 'silent' mode is on)"));
-#endif
 
    KCmdLineArgs::init(argc, argv, &d);
    KCmdLineArgs::addCmdLineOptions(options);
@@ -669,27 +642,8 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
      setenv("KDEROOTHOME", "-", 1);
    }
 
-#ifdef KBUILDSYCOCA_GUI
-   KApplication k;
-#else
-   KApplication k(false);
-#endif
-   k.disableSessionManagement();
-
-#ifdef KBUILDSYCOCA_GUI
-   silent = args->isSet("silent");
-   showprogress = args->isSet("showprogress");
-   QLabel progress( QString("<p><br><nobr>    %1    </nobr><br>").arg( i18n("Reloading KDE configuration, please wait...") ), 0, "", Qt::WType_Dialog | Qt::WStyle_DialogBorder  | Qt::WStyle_Customize| Qt::WStyle_Title );
-   QString capt = i18n("KDE Configuration Manager");
-   if (!silent) {
-     if (KMessageBox::No == KMessageBox::questionYesNo(0, i18n("Do you want to reload KDE configuration?"), capt, i18nc("Reload KDE configuration messagebox", "Reload"), i18n("Do Not Reload")))
-       return 0;
-   }
-   if (!silent || showprogress) {
-     progress.setCaption( capt );
-     progress.show();
-   }
-#endif
+   QCoreApplication k(argc, argv);
+   KComponentData mainComponent(d);
 
    KCrash::setCrashHandler(KCrash::defaultCrashHandler);
    KCrash::setEmergencySaveFunction(crashHandler);
@@ -723,7 +677,7 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
    bool incremental = !bGlobalDatabase && args->isSet("incremental") && checkfiles;
    if (incremental || !checkfiles)
    {
-     KSycoca::self()->disableAutoRebuild(); // Prevent deadlock
+     KSycoca::disableAutoRebuild(); // Prevent deadlock
      QString current_language = KGlobal::locale()->language();
      QString ksycoca_language = KSycoca::self()->language();
      quint32 current_update_sig = KGlobal::dirs()->calcResourceHash("services", "update_ksycoca",
@@ -739,7 +693,7 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
      {
         incremental = false;
         checkfiles = true;
-        delete KSycoca::self();
+        KBuildSycoca::clearCaches();
      }
    }
 
@@ -797,7 +751,7 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
       if (incremental)
       {
          kDebug(7021) << "Reusing existing ksycoca";
-         KSycoca *oldSycoca = KSycoca::self();
+         KSycoca::self();
          KSycocaFactoryList *factories = new KSycocaFactoryList;
          g_allEntries = new KSycocaEntryListList;
          g_ctimeDict = new QHash<QString, quint32>;
@@ -819,18 +773,13 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
          delete factories; factories = 0;
          KCTimeInfo *ctimeInfo = new KCTimeInfo;
          ctimeInfo->fillCTimeDict(*g_ctimeDict);
-         delete oldSycoca;
       }
       cSycocaPath = 0;
 
-      KBuildSycoca *sycoca= new KBuildSycoca; // Build data base
+      KBuildSycoca *sycoca = new KBuildSycoca; // Build data base (deletes oldSycoca)
       if (args->isSet("track"))
          sycoca->setTrackId(args->getOption("track"));
       if (!sycoca->recreate()) {
-#ifdef KBUILDSYCOCA_GUI
-        if (!silent || showprogress)
-          progress.close();
-#endif
         return -1;
       }
 
@@ -851,16 +800,13 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
      QDBusMessage signal = QDBusMessage::createSignal("/", "org.kde.KSycoca", "notifyDatabaseChanged" );
      signal << *g_changeList;
 
-     if (QDBusConnection::sessionBus().isConnected())
+     if (QDBusConnection::sessionBus().isConnected()) {
+        kDebug() << "Emitting notifyDatabaseChanged" << *g_changeList;
        QDBusConnection::sessionBus().send(signal);
+       qApp->processEvents(); // make sure the dbus signal is sent before we quit.
+     }
    }
 
-#ifdef KBUILDSYCOCA_GUI
-   if (!silent) {
-     progress.close();
-     KMessageBox::information(0, i18n("Configuration information reloaded successfully."), capt);
-   }
-#endif
    return 0;
 }
 
