@@ -71,7 +71,7 @@ static bool bCheckSycoca;
 static bool bCheckUpdates;
 static bool bCheckHostname;
 
-extern QDBUS_EXPORT void qDBusAddSpyHook(void (*)(const QDBusMessage&));
+extern void qDBusAddSpyHook(void (*)(const QDBusMessage&));
 
 static void runBuildSycoca(QObject *callBackObj=0, const char *callBackSlot=0)
 {
@@ -95,10 +95,6 @@ static void runBuildSycoca(QObject *callBackObj=0, const char *callBackSlot=0)
    else
    {
       KToolInvocation::kdeinitExecWait( exe, args );
-
-      if (callBackObj && callBackSlot) {
-          QTimer::singleShot(0, callBackObj, callBackSlot);
-      }
    }
 }
 
@@ -116,10 +112,16 @@ static void runDontChangeHostname(const QByteArray &oldName, const QByteArray &n
 }
 
 Kded::Kded()
-  : m_needDelayedCheck(false)
+    : m_needDelayedCheck(false)
 {
   _self = this;
 
+  m_serviceWatcher = new QDBusServiceWatcher(this);
+  m_serviceWatcher->setConnection(QDBusConnection::sessionBus());
+  m_serviceWatcher->setWatchMode(QDBusServiceWatcher::WatchForUnregistration);
+  QObject::connect(m_serviceWatcher, SIGNAL(serviceUnregistered(QString)),
+                   this, SLOT(slotApplicationRemoved(QString)));
+
   new KBuildsycocaAdaptor(this);
   new KdedAdaptor(this);
 
@@ -417,8 +419,7 @@ void Kded::slotKDEDModuleRemoved(KDEDModule *module)
   //   lib->unload();
 }
 
-void Kded::slotApplicationRemoved(const QString &name, const QString &oldOwner,
-                                  const QString &newOwner)
+void Kded::slotApplicationRemoved(const QString &name)
 {
 #if 0 // see kdedmodule.cpp (KDED_OBJECTS)
   foreach( KDEDModule* module, m_modules )
@@ -426,9 +427,7 @@ void Kded::slotApplicationRemoved(const QString &name, const QString &oldOwner,
      module->removeAll(appId);
   }
 #endif
-  if (oldOwner.isEmpty() || !newOwner.isEmpty())
-     return;
-
+  m_serviceWatcher->removeWatchedService(name);
   const QList<qlonglong> windowIds = m_windowIdList.value(name);
   for( QList<qlonglong>::ConstIterator it = windowIds.begin();
        it != windowIds.end(); ++it)
@@ -468,7 +467,7 @@ void Kded::updateDirWatch()
 
 void Kded::updateResourceList()
 {
-  delete KSycoca::self();
+  KSycoca::clearCaches();
 
   if (!bCheckUpdates) return;
 
@@ -515,7 +514,8 @@ void Kded::recreate(bool initial)
    {
       if(!delayedCheck)
          updateDirWatch(); // this would search all the directories
-      runBuildSycoca();
+      if (bCheckSycoca)
+         runBuildSycoca();
       recreateDone();
       if(delayedCheck)
       {
@@ -635,6 +635,10 @@ bool Kded::isWindowRegistered(long windowId) const
 
 void Kded::registerWindowId(qlonglong windowId, const QString &sender)
 {
+  if (!m_windowIdList.contains(sender)) {
+      m_serviceWatcher->addWatchedService(sender);
+  }
+
   m_globalWindowIdList.insert(windowId);
   QList<qlonglong> windowIds = m_windowIdList.value(sender);
   windowIds.append(windowId);
@@ -654,10 +658,12 @@ void Kded::unregisterWindowId(qlonglong windowId, const QString &sender)
   if (!windowIds.isEmpty())
   {
      windowIds.removeAll(windowId);
-     if (windowIds.isEmpty())
+     if (windowIds.isEmpty()) {
+        m_serviceWatcher->removeWatchedService(sender);
         m_windowIdList.remove(sender);
-     else
+     } else {
         m_windowIdList.insert(sender, windowIds);
+     }
   }
 
   foreach( KDEDModule* module, m_modules )
@@ -748,37 +754,6 @@ void KHostnameD::checkHostname()
 }
 
 
-#if 0
-// Thiago: I have no idea what the following class is here for
-// David: the commit log was:
-//       Disable DCOP-Qt bridge but make sure that "dcopquit kded" continues to work.
-//       (see the setQtBridgeEnabled below)
-class KDEDQtDCOPObject : public DCOPObject
-{
-public:
-  KDEDQtDCOPObject() : DCOPObject("qt/kded") { }
-
-  virtual bool process(const DCOPCString &fun, const QByteArray &data,
-                       DCOPCString& replyType, QByteArray &replyData)
-    {
-      if (qApp && (fun == "quit()"))
-      {
-        qApp->quit();
-        replyType = "void";
-        return true;
-      }
-      return DCOPObject::process(fun, data, replyType, replyData);
-    }
-
-  DCOPCStringList functions()
-    {
-       DCOPCStringList res = DCOPObject::functions();
-       res += "void quit()";
-       return res;
-    }
-};
-#endif
-
 KBuildsycocaAdaptor::KBuildsycocaAdaptor(QObject *parent)
    : QDBusAbstractAdaptor(parent)
 {
@@ -809,9 +784,6 @@ public:
 
           Kded *kded = Kded::self();
 
-          if (bCheckSycoca)
-              runBuildSycoca();
-
           kded->recreate(true); // initial
 
           if (bCheckUpdates)
@@ -843,21 +815,6 @@ public:
           if (bCheckHostname)
             (void) new KHostnameD(HostnamePollInterval); // Watch for hostname changes
 
-          QObject::connect(QDBusConnection::sessionBus().interface(),
-                          SIGNAL(serviceOwnerChanged(QString,QString,QString)),
-                          kded, SLOT(slotApplicationRemoved(QString,QString,QString)));
-
-          // During startup kdesktop waits for KDED to finish.
-          // Send a notifyDatabaseChanged signal even if the database hasn't
-          // changed.
-          // If the database changed, kbuildsycoca's signal didn't go anywhere
-          // anyway, because it was too early, so let's send this signal
-          // unconditionnally (David)
-
-          QDBusMessage msg = QDBusMessage::createSignal("/kbuildsycoca", "org.kde.KSycoca", "notifyDatabaseChanged" );
-          msg << QStringList();
-          QDBusConnection::sessionBus().send(msg);
-
           kded->initModules();
        } else
           runBuildSycoca();
@@ -872,7 +829,7 @@ extern "C" KDE_EXPORT int kdemain(int argc, char *argv[])
 {
      KAboutData aboutData( "kded" /*don't change this one to kded4! dbus registration should be org.kde.kded etc.*/,
         "kdelibs4", ki18n("KDE Daemon"),
-        "$Id: kded.cpp 944896 2009-03-26 13:00:10Z dfaure $",
+        "$Id: kded.cpp 1156841 2010-07-29 19:59:05Z zander $",
         ki18n("KDE Daemon - triggers Sycoca database updates when needed"));
 
      KCmdLineOptions options;
