@@ -36,6 +36,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 
+#include <QtGui/QButtonGroup>
 #include <QtGui/QCheckBox>
 #include <QtGui/QDesktopWidget>
 #include <QtGui/QRadioButton>
@@ -44,42 +45,43 @@
 #include <QtCore/QFile>
 #include <QtGui/QHeaderView>
 #include <QtGui/QImage>
-#include <QtGui/QItemDelegate>
+#include <QtGui/QStyledItemDelegate>
 #include <QtGui/QLabel>
 #include <QtGui/QLayout>
 #include <QtGui/QPainter>
 #include <QtGui/QPushButton>
 #include <QtGui/QScrollBar>
-#include <QtGui/QDoubleSpinBox>
 #include <QtCore/QTimer>
-#include <QtGui/QDoubleValidator>
 
 #include <kapplication.h>
 #include <kcombobox.h>
 #include <kconfig.h>
 #include <kglobal.h>
 #include <kglobalsettings.h>
+#include <khbox.h>
 #include <kiconloader.h>
 #include <klineedit.h>
 #include <klistwidget.h>
 #include <klocale.h>
 #include <kmessagebox.h>
+#include <knuminput.h>
 #include <kseparator.h>
 #include <kstandarddirs.h>
 #include <kcolorcollection.h>
+#include <kcolorutils.h>
 
 #include "kcolormimedata.h"
 #include <config.h>
 #include <kdebug.h>
 
+#include "kcolorchoosermode_p.h"
+#include "kcolorhelpers_p.h"
 #include "kselector.h"
 #include "kcolorvalueselector.h"
 #include "khuesaturationselect.h"
 #include "kxyselector.h"
 #include <kconfiggroup.h>
 
-#include <iostream>
-
 #ifdef Q_WS_X11
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -87,14 +89,16 @@
 #include <fixx11h.h>
 #endif
 
+using namespace KDEPrivate;
+
 using KDEPrivate::KColorTable;
 
 struct ColorCollectionNameType {
-    const char* m_fileName;
-    const char* m_displayName;
+    const char* const m_fileName;
+    const char* const m_displayName;
 };
 
-const ColorCollectionNameType colorCollectionName[] = {
+static const ColorCollectionNameType colorCollectionName[] = {
     { "Recent_Colors", I18N_NOOP2("palette name", "* Recent Colors *") },
     { "Custom_Colors", I18N_NOOP2("palette name", "* Custom Colors *") },
     { "40.colors",     I18N_NOOP2("palette name", "Forty Colors") },
@@ -105,29 +109,11 @@ const ColorCollectionNameType colorCollectionName[] = {
     { 0, 0 } // end of data
 };
 
-static const int recentColorIndex = 0;
-static const int customColorIndex = 1;
-static const int fortyColorIndex = 2;
-
-class KColorSpinBox : public QSpinBox
+enum ColorCollectionIndices
 {
-public:
-    KColorSpinBox(int minValue, int maxValue, int step, QWidget* parent)
-            : QSpinBox(parent) {
-        setRange(minValue, maxValue); setSingleStep(step);
-    }
-
-
-    // Override Qt's braindead auto-selection.
-    //XXX KDE4 : check this is no more necessary , was disabled to port to Qt4 //mikmak
-    /*
-    virtual void valueChange()
-    {
-        updateDisplay();
-        emit valueChanged( value() );
-        emit valueChanged( currentValueText() );
-    }*/
-
+    recentColorIndex,
+    customColorIndex,
+    fortyColorIndex
 };
 
 //-----------------------------------------------------------------------------
@@ -148,10 +134,55 @@ public:
     bool inMouse;
 };
 
+class KColorCellsItemDelegate: public QStyledItemDelegate
+{
+public:
+    KColorCellsItemDelegate(KColorCells *parent): QStyledItemDelegate(parent) {}
+    virtual void paint(QPainter * painter, const QStyleOptionViewItem & option, const QModelIndex & index) const
+    {
+        QStyleOptionViewItemV4 opt(option);
+        initStyleOption(&opt,index);
+
+        //Get the current cell color
+        QColor backgroundColor = index.data(Qt::BackgroundRole).value<QColor>();
+        if (backgroundColor.isValid()) {
+            //Paint the general background
+            painter->fillRect(opt.rect, backgroundColor);
+            //Paint the selection mark (circle)
+            if (opt.state & QStyle::State_Selected) {
+                //Use black or white, depending on the contrast
+                QColor color = QColor(0, 0, 0, 220);
+                if (KColorUtils::contrastRatio(color, backgroundColor) < 5) {
+                    color = QColor(255, 255, 255, 220);
+                }
+                //Draw the selection (radiobutton-like) circle
+                painter->save();
+                painter->setRenderHint(QPainter::Antialiasing, true);
+                painter->setRenderHint(QPainter::HighQualityAntialiasing, true);
+                painter->setPen(QPen(color, 1.2, Qt::SolidLine));
+                painter->setBrush(QBrush());
+                painter->drawEllipse(opt.rect.adjusted(2,2,-2,-2));
+                painter->restore();
+            }
+        } else {
+            //Paint the "X" (missing) cross on empty background color
+            backgroundColor = opt.palette.color(QPalette::Window);
+            painter->fillRect(opt.rect, backgroundColor);
+            painter->save();
+            QColor crossColor = qGray(backgroundColor.rgb()) > 192 ? backgroundColor.darker(106) :
+                                                                     backgroundColor.lighter(106);
+            painter->setPen(QPen(crossColor, 1.5));
+            painter->drawLine(opt.rect.topLeft(), opt.rect.bottomRight());
+            painter->drawLine(opt.rect.topRight(), opt.rect.bottomLeft());
+            painter->restore();
+        }
+    }
+};
+
 KColorCells::KColorCells(QWidget *parent, int rows, int cols)
         : QTableWidget(parent), d(new KColorCellsPrivate(this))
 {
-    setItemDelegate(new QItemDelegate(this));
+    setItemDelegate(new KColorCellsItemDelegate(this));
 
     setFrameShape(QFrame::NoFrame);
     d->shade = true;
@@ -453,7 +484,7 @@ QColor KColorPatch::color() const
 
 void KColorPatch::setColor(const QColor &col)
 {
-    d->color.setRgb(col.rgb());
+    d->color = col.toRgb();
 
     update();
 }
@@ -462,9 +493,8 @@ void KColorPatch::paintEvent(QPaintEvent* pe)
 {
     QFrame::paintEvent(pe);
     QPainter painter(this);
-    painter.setPen(d->color);
-    painter.setBrush(QBrush(d->color));
-    painter.drawRect(contentsRect());
+
+    fillOpaqueRect(&painter, contentsRect(), d->color);
 }
 
 void KColorPatch::mouseMoveEvent(QMouseEvent *e)
@@ -890,19 +920,13 @@ public:
     void showColor(const QColor &color, const QString &name);
 
     void slotRGBChanged(void);
+    void slotAlphaChanged(void);
     void slotHSVChanged(void);
     void slotHtmlChanged(void);
     void slotHSChanged(int, int);
     void slotVChanged(int);
-
-    void setHMode();
-    void setSMode();
-    void setVMode();
-    void setRMode();
-    void setGMode();
-    void setBMode();
-
-    void updateModeButtons();
+    void slotAChanged(int);
+    void slotModeChanged(int);
 
     void slotColorSelected(const QColor &col);
     void slotColorSelected(const QColor &col, const QString &name);
@@ -933,29 +957,28 @@ public:
     bool bEditHsv;
     bool bEditHtml;
     bool bColorPicking;
+    bool bAlphaEnabled;
     QLabel *colorName;
     KLineEdit *htmlName;
-    KColorSpinBox *hedit;
-    KColorSpinBox *sedit;
-    KColorSpinBox *vedit;
-    KColorSpinBox *redit;
-    KColorSpinBox *gedit;
-    KColorSpinBox *bedit;
-    QRadioButton *hmode;
-    QRadioButton *smode;
-    QRadioButton *vmode;
-    QRadioButton *rmode;
-    QRadioButton *gmode;
-    QRadioButton *bmode;
+    KIntSpinBox *hedit;
+    KIntSpinBox *sedit;
+    KIntSpinBox *vedit;
+    KIntSpinBox *redit;
+    KIntSpinBox *gedit;
+    KIntSpinBox *bedit;
+    QWidget *alphaLabel;
+    KIntSpinBox *aedit;
 
     KColorPatch *patch;
     KColorPatch *comparePatch;
 
     KColorChooserMode _mode;
+    QButtonGroup *modeGroup;
 
     KHueSaturationSelector *hsSelector;
     KColorCollection *palette;
     KColorValueSelector *valuePal;
+    KGradientSelector *alphaSelector;
     QVBoxLayout* l_right;
     QGridLayout* tl_layout;
     QCheckBox *cbDefaultColor;
@@ -990,10 +1013,10 @@ KColorDialog::KColorDialog(QWidget *parent, bool modal)
 {
     setCaption(i18n("Select Color"));
     setButtons(modal ? Ok | Cancel : Close);
-    showButtonSeparator(true);
     setModal(modal);
     d->bRecursion = true;
     d->bColorPicking = false;
+    d->bAlphaEnabled = false;
 #ifdef Q_WS_X11
     d->filter = 0;
 #endif
@@ -1029,12 +1052,6 @@ KColorDialog::KColorDialog(QWidget *parent, bool modal)
     QHBoxLayout *l_ltop = new QHBoxLayout();
     l_left->addLayout(l_ltop);
 
-    // a little space between
-    l_left->addSpacing(10); // FIXME: remove hardcoded values
-
-    QGridLayout *l_lbot = new QGridLayout();
-    l_left->addLayout(l_lbot);
-
     //
     // the palette and value selector go into the H-box
     //
@@ -1052,73 +1069,116 @@ KColorDialog::KColorDialog(QWidget *parent, bool modal)
     connect(d->valuePal, SIGNAL(valueChanged(int)),
             SLOT(slotVChanged(int)));
 
+    d->alphaSelector = new KGradientSelector(Qt::Horizontal, page);
+    d->alphaSelector->setFixedSize(256, 26);
+    d->alphaSelector->setIndent(false);
+    d->alphaSelector->setArrowDirection(Qt::DownArrow);
+    d->alphaSelector->setRange(0, 255);
+    l_left->addWidget(d->alphaSelector, 1);
+    connect(d->alphaSelector, SIGNAL(valueChanged(int)),
+            SLOT(slotAChanged(int)));
+
+    // a little space between
+    l_left->addSpacing(10); // FIXME: remove hardcoded values
+
+    QGridLayout *l_lbot = new QGridLayout();
+    l_left->addLayout(l_lbot);
+
+    // button group that manages the radio buttons
+    QRadioButton *modeButton;
+    d->modeGroup = new QButtonGroup(page);
+    connect(d->modeGroup, SIGNAL(buttonClicked(int)), SLOT(slotModeChanged(int)));
+
     //
     // add the HSV fields
     //
     l_lbot->setColumnStretch(2, 10);
 
-    d->hmode = new QRadioButton(i18n("Hue:"), page);
-    l_lbot->addWidget(d->hmode, 0, 0);
+    modeButton = new QRadioButton(i18n("Hue:"), page);
+    l_lbot->addWidget(modeButton, 0, 0);
+    d->modeGroup->addButton(modeButton, ChooserHue);
 
-    d->hedit = new KColorSpinBox(0, 359, 1, page);
+    d->hedit = new KIntSpinBox(page);
+    d->hedit->setMaximum(359);
+    d->hedit->setSuffix(i18nc("The angular degree unit (for hue)", "\302\260")); //  U+00B0 DEGREE SIGN
     l_lbot->addWidget(d->hedit, 0, 1);
     connect(d->hedit, SIGNAL(valueChanged(int)),
             SLOT(slotHSVChanged()));
-    connect(d->hmode, SIGNAL(clicked()),
-            SLOT(setHMode()));
 
-    d->smode = new QRadioButton(i18n("Saturation:"), page);
-    l_lbot->addWidget(d->smode, 1, 0);
+    modeButton = new QRadioButton(i18n("Saturation:"), page);
+    l_lbot->addWidget(modeButton, 1, 0);
+    d->modeGroup->addButton(modeButton, ChooserSaturation);
 
-    d->sedit = new KColorSpinBox(0, 255, 1, page);
+    d->sedit = new KIntSpinBox(page);
+    d->sedit->setMaximum(255);
     l_lbot->addWidget(d->sedit, 1, 1);
     connect(d->sedit, SIGNAL(valueChanged(int)),
             SLOT(slotHSVChanged()));
-    connect(d->smode, SIGNAL(clicked()),
-            SLOT(setSMode()));
 
-    d->vmode = new QRadioButton(i18nc("This is the V of HSV", "Value:"), page);
-    l_lbot->addWidget(d->vmode, 2, 0);
+    modeButton = new QRadioButton(i18nc("This is the V of HSV", "Value:"), page);
+    l_lbot->addWidget(modeButton, 2, 0);
+    d->modeGroup->addButton(modeButton, ChooserValue);
 
-    d->vedit = new KColorSpinBox(0, 255, 1, page);
+    d->vedit = new KIntSpinBox(page);
+    d->vedit->setMaximum(255);
     l_lbot->addWidget(d->vedit, 2, 1);
     connect(d->vedit, SIGNAL(valueChanged(int)),
             SLOT(slotHSVChanged()));
-    connect(d->vmode, SIGNAL(clicked()),
-            SLOT(setVMode()));
 
 
     //
     // add the RGB fields
     //
-    d->rmode = new QRadioButton(i18n("Red:"), page);
-    l_lbot->addWidget(d->rmode, 0, 3);
-    d->redit = new KColorSpinBox(0, 255, 1, page);
+    modeButton = new QRadioButton(i18n("Red:"), page);
+    l_lbot->addWidget(modeButton, 0, 3);
+    d->modeGroup->addButton(modeButton, ChooserRed);
+
+    d->redit = new KIntSpinBox(page);
+    d->redit->setMaximum(255);
     l_lbot->addWidget(d->redit, 0, 4);
     connect(d->redit, SIGNAL(valueChanged(int)),
             SLOT(slotRGBChanged()));
-    connect(d->rmode, SIGNAL(clicked()),
-            SLOT(setRMode()));
 
-    d->gmode = new QRadioButton(i18n("Green:"), page);
-    l_lbot->addWidget(d->gmode, 1, 3);
+    modeButton = new QRadioButton(i18n("Green:"), page);
+    l_lbot->addWidget(modeButton, 1, 3);
+    d->modeGroup->addButton(modeButton, ChooserGreen);
 
-    d->gedit = new KColorSpinBox(0, 255, 1, page);
+    d->gedit = new KIntSpinBox(page);
+    d->gedit->setMaximum(255);
     l_lbot->addWidget(d->gedit, 1, 4);
     connect(d->gedit, SIGNAL(valueChanged(int)),
             SLOT(slotRGBChanged()));
-    connect(d->gmode, SIGNAL(clicked()),
-            SLOT(setGMode()));
 
-    d->bmode = new QRadioButton(i18n("Blue:"), page);
-    l_lbot->addWidget(d->bmode, 2, 3);
+    modeButton = new QRadioButton(i18n("Blue:"), page);
+    l_lbot->addWidget(modeButton, 2, 3);
+    d->modeGroup->addButton(modeButton, ChooserBlue);
 
-    d->bedit = new KColorSpinBox(0, 255, 1, page);
+    d->bedit = new KIntSpinBox(page);
+    d->bedit->setMaximum(255);
     l_lbot->addWidget(d->bedit, 2, 4);
     connect(d->bedit, SIGNAL(valueChanged(int)),
             SLOT(slotRGBChanged()));
-    connect(d->bmode, SIGNAL(clicked()),
-            SLOT(setBMode()));
+
+    d->alphaLabel = new KHBox(page);
+    QWidget *spacer = new QWidget(d->alphaLabel);
+    label = new QLabel(i18n("Alpha:"), d->alphaLabel);
+    QStyleOptionButton option;
+    option.initFrom(modeButton);
+    QRect labelRect = modeButton->style()->subElementRect(QStyle::SE_RadioButtonContents, &option, modeButton);
+    int indent = layoutDirection() == Qt::LeftToRight ? labelRect.left() : modeButton->geometry().right() - labelRect.right();
+    spacer->setFixedWidth(indent);
+    l_lbot->addWidget(d->alphaLabel, 3, 3);
+
+    d->aedit = new KIntSpinBox(page);
+    d->aedit->setMaximum(255);
+    label->setBuddy(d->aedit);
+    l_lbot->addWidget(d->aedit, 3, 4);
+    connect(d->aedit, SIGNAL(valueChanged(int)),
+            SLOT(slotAlphaChanged()));
+
+    d->aedit->setVisible(false);
+    d->alphaLabel->setVisible(false);
+    d->alphaSelector->setVisible(false);
 
     //
     // add a layout for the right side
@@ -1217,6 +1277,7 @@ KColorDialog::KColorDialog(QWidget *parent, bool modal)
     setTabOrder(d->vedit, d->redit);
     setTabOrder(d->redit, d->gedit);
     setTabOrder(d->gedit, d->bedit);
+    setTabOrder(d->bedit, d->aedit);
 
     tl_layout->activate();
     page->setMinimumSize(page->sizeHint());
@@ -1238,7 +1299,7 @@ KColorDialog::KColorDialog(QWidget *parent, bool modal)
 //   d->hsSelector->installEventFilter(this);
     d->hsSelector->setAcceptDrops(true);
 
-    d->setVMode();
+    d->setChooserMode(ChooserValue);
 }
 
 KColorDialog::~KColorDialog()
@@ -1302,6 +1363,26 @@ QColor KColorDialog::defaultColor() const
     return d->defaultColor;
 }
 
+void KColorDialog::setAlphaChannelEnabled(bool alpha)
+{
+    if (d->bAlphaEnabled != alpha) {
+        d->bAlphaEnabled = alpha;
+        d->aedit->setVisible(d->bAlphaEnabled);
+        d->alphaLabel->setVisible(d->bAlphaEnabled);
+        d->alphaSelector->setVisible(d->bAlphaEnabled);
+
+        mainWidget()->setMaximumSize(QWIDGETSIZE_MAX, QWIDGETSIZE_MAX);   // cancel setFixedSize()
+        d->tl_layout->activate();
+        mainWidget()->setMinimumSize(mainWidget()->sizeHint());
+        setFixedSize(sizeHint());
+    }
+}
+
+bool KColorDialog::isAlphaChannelEnabled() const
+{
+    return d->bAlphaEnabled;
+}
+
 
 void KColorDialog::KColorDialogPrivate::setChooserMode(KColorChooserMode c)
 {
@@ -1309,49 +1390,15 @@ void KColorDialog::KColorDialogPrivate::setChooserMode(KColorChooserMode c)
     hsSelector->setChooserMode(c);
     valuePal->setChooserMode(c);
 
-    updateModeButtons();
+    modeGroup->button(valuePal->chooserMode())->setChecked(true);
     valuePal->updateContents();
     hsSelector->updateContents();
-    valuePal->repaint();
-    hsSelector->repaint();
+    valuePal->update();
+    hsSelector->update();
     slotHSVChanged();
 }
 
 
-
-void
-KColorDialog::KColorDialogPrivate::updateModeButtons()
-{
-    hmode->setChecked(false);
-    smode->setChecked(false);
-    vmode->setChecked(false);
-    rmode->setChecked(false);
-    gmode->setChecked(false);
-    bmode->setChecked(false);
-
-    switch (valuePal->chooserMode()) {
-    case ChooserHue:
-        hmode->setChecked(true);
-        break;
-    case ChooserSaturation:
-        smode->setChecked(true);
-        break;
-    case ChooserRed:
-        rmode->setChecked(true);
-        break;
-    case ChooserGreen:
-        gmode->setChecked(true);
-        break;
-    case ChooserBlue:
-        bmode->setChecked(true);
-        break;
-    case ChooserValue:
-    default:
-        vmode->setChecked(true);
-        break;
-    }
-}
-
 KColorChooserMode KColorDialog::KColorDialogPrivate::chooserMode()
 {
     return _mode;
@@ -1369,40 +1416,9 @@ void KColorDialog::KColorDialogPrivate::slotDefaultColorClicked()
 }
 
 void
-KColorDialog::KColorDialogPrivate::setHMode()
+KColorDialog::KColorDialogPrivate::slotModeChanged(int id)
 {
-    setChooserMode(ChooserHue);
-}
-
-void
-KColorDialog::KColorDialogPrivate::setSMode()
-{
-    setChooserMode(ChooserSaturation);
-}
-
-void
-KColorDialog::KColorDialogPrivate::setVMode()
-{
-    setChooserMode(ChooserValue);
-}
-
-void
-KColorDialog::KColorDialogPrivate::setRMode()
-{
-    setChooserMode(ChooserRed);
-}
-
-void
-KColorDialog::KColorDialogPrivate::setGMode()
-{
-    setChooserMode(ChooserGreen);
-
-}
-
-void
-KColorDialog::KColorDialogPrivate::setBMode()
-{
-    setChooserMode(ChooserBlue);
+    setChooserMode(KColorChooserMode(id));
 }
 
 void
@@ -1507,12 +1523,24 @@ void KColorDialog::KColorDialogPrivate::slotRGBChanged(void)
     if (blu > 255 || blu < 0) return;
 
     QColor col;
-    col.setRgb(red, grn, blu);
+    col.setRgb(red, grn, blu, aedit->value());
     bEditRgb = true;
     _setColor(col);
     bEditRgb = false;
 }
 
+void KColorDialog::KColorDialogPrivate::slotAlphaChanged(void)
+{
+    if (bRecursion) return;
+    int alpha = aedit->value();
+
+    if (alpha > 255 || alpha < 0) return;
+
+    QColor col = selColor;
+    col.setAlpha(alpha);
+    _setColor(col);
+}
+
 void KColorDialog::KColorDialogPrivate::slotHtmlChanged(void)
 {
     if (bRecursion || htmlName->text().isEmpty()) return;
@@ -1549,7 +1577,7 @@ void KColorDialog::KColorDialogPrivate::slotHSVChanged(void)
     if (val > 255 || val < 0) return;
 
     QColor col;
-    col.setHsv(hue, sat, val);
+    col.setHsv(hue, sat, val, aedit->value());
     bEditHsv = true;
     _setColor(col);
     bEditHsv = false;
@@ -1557,77 +1585,25 @@ void KColorDialog::KColorDialogPrivate::slotHSVChanged(void)
 
 void KColorDialog::KColorDialogPrivate::slotHSChanged(int x, int y)
 {
-    int _h, _s, _v, _r, _g, _b;
-
-    _h = selColor.hue();
-    _s = selColor.saturation();
-    _v = selColor.value();
-    _r = selColor.red();
-    _g = selColor.green();
-    _b = selColor.blue();
-
-    QColor col;
-
-    switch (chooserMode()) {
-    case ChooserRed:
-        col.setRgb(_r, x, y);
-        break;
-    case ChooserGreen:
-        col.setRgb(x, _g, y);
-        break;
-    case ChooserBlue:
-        col.setRgb(y, x, _b);
-        break;
-    case ChooserHue:
-        col.setHsv(_h, x, y);
-        break;
-    case ChooserSaturation:
-        col.setHsv(x, _s, y);
-        break;
-    case ChooserValue:
-    default:
-        col.setHsv(x, y, _v);
-        break;
-    }
+    QColor col = selColor;
+    KColorChooserMode xMode = chooserXMode(chooserMode());
+    KColorChooserMode yMode = chooserYMode(chooserMode());
+    setComponentValue(col, xMode, x / (xMode == ChooserHue ? 360.0 : 255.0));
+    setComponentValue(col, yMode, y / (yMode == ChooserHue ? 360.0 : 255.0));
     _setColor(col);
 }
 
 void KColorDialog::KColorDialogPrivate::slotVChanged(int v)
 {
-    int _h, _s, _v, _r, _g, _b;
-
-    _h = selColor.hue();
-    _s = selColor.saturation();
-    _v = selColor.value();
-    _r = selColor.red();
-    _g = selColor.green();
-    _b = selColor.blue();
-
-
-    QColor col;
-
-    switch (chooserMode()) {
-    case ChooserHue:
-        col.setHsv(v, _s, _v);
-        break;
-    case ChooserSaturation:
-        col.setHsv(_h, v, _v);
-        break;
-    case ChooserRed:
-        col.setRgb(v, _g, _b);
-        break;
-    case ChooserGreen:
-        col.setRgb(_r, v, _b);
-        break;
-    case ChooserBlue:
-        col.setRgb(_r, _g, v);
-        break;
-    case ChooserValue:
-    default:
-        col.setHsv(_h, _s, v);
-        break;
-    }
+    QColor col = selColor;
+    setComponentValue(col, chooserMode(), v / (chooserMode() == ChooserHue ? 360.0 : 255.0));
+    _setColor(col);
+}
 
+void KColorDialog::KColorDialogPrivate::slotAChanged(int value)
+{
+    QColor col = selColor;
+    col.setAlpha(value);
     _setColor(col);
 }
 
@@ -1688,36 +1664,23 @@ void KColorDialog::KColorDialogPrivate::showColor(const QColor &color, const QSt
     setRgbEdit(color);
     setHsvEdit(color);
     setHtmlEdit(color);
+    aedit->setValue(color.alpha());
 
+    QColor rgbColor = color.toRgb();
+    bool ltr = q->layoutDirection() == Qt::LeftToRight;
+    rgbColor.setAlpha(ltr ? 0 : 255);
+    alphaSelector->setFirstColor(rgbColor);
+    rgbColor.setAlpha(ltr ? 255 : 0);
+    alphaSelector->setSecondColor(rgbColor);
+    alphaSelector->setValue(color.alpha());
 
-    switch (chooserMode()) {
-    case ChooserSaturation:
-        hsSelector->setValues(color.hue(), color.value());
-        valuePal->setValue(color.saturation());
-        break;
-    case ChooserValue:
-        hsSelector->setValues(color.hue(), color.saturation());
-        valuePal->setValue(color.value());
-        break;
-    case ChooserRed:
-        hsSelector->setValues(color.green(), color.blue());
-        valuePal->setValue(color.red());
-        break;
-    case ChooserGreen:
-        hsSelector->setValues(color.red(), color.blue());
-        valuePal->setValue(color.green());
-        break;
-    case ChooserBlue:
-        hsSelector->setValues(color.green(), color.red());
-        valuePal->setValue(color.blue());
-        break;
-    case ChooserHue:
-    default:
-        hsSelector->setValues(color.saturation(), color.value());
-        valuePal->setValue(color.hue());
-        break;
-
-    }
+    KColorChooserMode xMode = chooserXMode(chooserMode());
+    KColorChooserMode yMode = chooserYMode(chooserMode());
+    int xValue = qRound(getComponentValue(color, xMode) * (xMode == ChooserHue ? 360.0 : 255.0));
+    int yValue = qRound(getComponentValue(color, yMode) * (yMode == ChooserHue ? 360.0 : 255.0));
+    int value = qRound(getComponentValue(color, chooserMode()) * (chooserMode() == ChooserHue ? 360.0 : 255.0));
+    hsSelector->setValues(xValue, yValue);
+    valuePal->setValue(value);
 
     bool blocked = valuePal->blockSignals(true);
 
@@ -1726,7 +1689,7 @@ void KColorDialog::KColorDialogPrivate::showColor(const QColor &color, const QSt
     valuePal->setColorValue(color.value());
     valuePal->updateContents();
     valuePal->blockSignals(blocked);
-    valuePal->repaint();
+    valuePal->update();
 
     blocked = hsSelector->blockSignals(true);
 
@@ -1735,7 +1698,7 @@ void KColorDialog::KColorDialogPrivate::showColor(const QColor &color, const QSt
     hsSelector->setColorValue(color.value());
     hsSelector->updateContents();
     hsSelector->blockSignals(blocked);
-    hsSelector->repaint();
+    hsSelector->update();
 
     bRecursion = false;
 }
