@@ -22,6 +22,7 @@
 
 #include "kdialog.h"
 #include "kdialog_p.h"
+#include <kdebug.h>
 #include "kdialogqueue_p.h"
 
 #include <config.h>
@@ -43,7 +44,6 @@
 #include <kstandardguiitem.h>
 #include <ktoolinvocation.h>
 #include <kurllabel.h>
-#include <kwhatsthismanager_p.h>
 
 #ifdef Q_WS_X11
 #include <qx11info_x11.h>
@@ -68,6 +68,15 @@ void KDialogPrivate::queuedLayoutUpdate()
 
   Q_Q(KDialog);
 
+    // Don't lose the focus widget when re-creating the layout.
+    // Testcase: KOrganizer's "Select Categories" dialog
+    QPointer<QWidget> focusWidget = mMainWidget ? mMainWidget->focusWidget() : 0;
+
+  if (q->layout() && q->layout() != mTopLayout) {
+      kWarning(240) << q->metaObject()->className() << "created with a layout; don't do that, KDialog takes care of it, use mainWidget or setMainWidget instead";
+      delete q->layout();
+  }
+
   delete mTopLayout;
 
   if ( mButtonOrientation == Qt::Horizontal )
@@ -91,13 +100,10 @@ void KDialogPrivate::queuedLayoutUpdate()
     mButtonBox->setOrientation( mButtonOrientation );
     mTopLayout->addWidget( mButtonBox );
   }
-}
 
-void KDialogPrivate::setButtonFocus(QPushButton *button, bool isDefault, bool isFocus)
-{
-  button->setDefault( isDefault );
-  if ( isFocus )
-      button->setFocus();
+    if (focusWidget) {
+        focusWidget->setFocus();
+    }
 }
 
 void KDialogPrivate::appendButton(KDialog::ButtonCode key, const KGuiItem &item)
@@ -134,7 +140,6 @@ void KDialogPrivate::appendButton(KDialog::ButtonCode key, const KGuiItem &item)
     case KDialog::User3:
       role = QDialogButtonBox::ActionRole;
       break;
-    case KDialog::NoDefault:
     default:
       role = QDialogButtonBox::InvalidRole;
       break;
@@ -156,7 +161,6 @@ void KDialogPrivate::appendButton(KDialog::ButtonCode key, const KGuiItem &item)
 void KDialogPrivate::init(KDialog *q)
 {
     q_ptr = q;
-    KWhatsThisManager::init();
 
     dirty = false;
 
@@ -244,7 +248,7 @@ void KDialog::setButtons( ButtonCodes buttonMask )
   if ( buttonMask & No )
     d->appendButton( No, KStandardGuiItem::no() );
   if ( buttonMask & Details ) {
-    d->appendButton( Details, KGuiItem(QString()) );
+    d->appendButton( Details, KGuiItem(QString(), "help-about") );
     setDetailsWidgetVisible( false );
   }
 
@@ -273,19 +277,36 @@ void KDialog::setEscapeButton( ButtonCode id )
 
 void KDialog::setDefaultButton( ButtonCode newDefaultButton )
 {
-    Q_D(KDialog);
-    bool makeDefault = true;
-    if (newDefaultButton == NoDefault) {
-        // if we already have a default, let's be sure to reset
-        // it to not be the default!
-        newDefaultButton = defaultButton();
-        makeDefault = false;
+    if (newDefaultButton == None)
+        newDefaultButton = NoDefault; // #148969
+
+    const KDialog::ButtonCode oldDefault = defaultButton();
+
+    bool oldDefaultHadFocus = false;
+
+    if (oldDefault != NoDefault) {
+        KPushButton *old = button(oldDefault);
+        if (old) {
+            oldDefaultHadFocus = (focusWidget() == old);
+            old->setDefault(false);
+        }
     }
 
-    KPushButton *b = button(newDefaultButton);
-    if (b) {
-        d->setButtonFocus(b, makeDefault, false);
+    if (newDefaultButton != NoDefault) {
+        KPushButton *b = button(newDefaultButton);
+        if (b) {
+            b->setDefault(true);
+            if (focusWidget() == 0 || oldDefaultHadFocus) {
+                // No widget had focus yet, or the old default button had
+                // -> ok to give focus to the new default button, so that it's
+                // really default (Enter triggers it).
+                // But we don't do this if the kdialog user gave focus to a
+                // specific widget in the dialog.
+                b->setFocus();
+            }
+        }
     }
+    Q_ASSERT(defaultButton() == newDefaultButton);
 }
 
 KDialog::ButtonCode KDialog::defaultButton() const
@@ -294,8 +315,9 @@ KDialog::ButtonCode KDialog::defaultButton() const
   QHashIterator<int, KPushButton*> it( d->mButtonList );
   while ( it.hasNext() ) {
     it.next();
-    if ( it.value()->isDefault() )
+    if (it.value()->isDefault()) {
       return (ButtonCode)it.key();
+    }
   }
 
   return NoDefault;
@@ -304,15 +326,14 @@ KDialog::ButtonCode KDialog::defaultButton() const
 void KDialog::setMainWidget( QWidget *widget )
 {
     Q_D(KDialog);
-  if ( d->mMainWidget == widget )
-    return;
-  d->mMainWidget = widget;
-  QLayout* layout = d->mMainWidget->layout();
-  if (layout) {
-    // Avoid double-margin problem
-    layout->setMargin(0);
-  }
-  d->setupLayout();
+    if ( d->mMainWidget == widget )
+        return;
+    d->mMainWidget = widget;
+    if (d->mMainWidget && d->mMainWidget->layout()) {
+        // Avoid double-margin problem
+        d->mMainWidget->layout()->setMargin(0);
+    }
+    d->setupLayout();
 }
 
 QWidget *KDialog::mainWidget()
@@ -426,7 +447,7 @@ QString KDialog::makeStandardCaption( const QString &userCaption,
            !userCaption.endsWith(caption)  ) {
            // TODO: check to see if this is a transient/secondary window before trying to add the app name
            //       on platforms that need this
-          captionString += QString::fromUtf8(" - ") + caption;
+          captionString += i18nc("Document/application separator in titlebar", " – ") + caption;
       }
   }
 
@@ -454,12 +475,13 @@ void KDialog::setCaption( const QString &caption, bool modified )
 
 void KDialog::setPlainCaption( const QString &caption )
 {
-  QDialog::setWindowTitle( caption );
-
+    if (QWidget *win = window()) {
+        win->setWindowTitle( caption );
 #ifdef Q_WS_X11
-  NETWinInfo info( QX11Info::display(), winId(), QX11Info::appRootWindow(), 0 );
-  info.setName( caption.toUtf8().constData() );
+        NETWinInfo info( QX11Info::display(), win->winId(), QX11Info::appRootWindow(), 0 );
+        info.setName( caption.toUtf8().constData() );
 #endif
+    }
 }
 
 void KDialog::resizeLayout( QWidget *widget, int margin, int spacing ) //static
@@ -764,8 +786,9 @@ QString KDialog::buttonWhatsThis( ButtonCode id ) const
 void KDialog::setButtonFocus( ButtonCode id )
 {
   KPushButton *button = this->button( id );
-  if ( button )
-        d_func()->setButtonFocus(button, button->isDefault(), true);
+  if ( button ) {
+      button->setFocus();
+  }
 }
 
 void KDialog::setDetailsWidget( QWidget *detailsWidget )
@@ -877,7 +900,7 @@ void KDialog::slotButtonClicked( int button )
       break;
     case Close:
       emit closeClicked();
-      close();
+      done(Close); // KDE5: call reject() instead; more QDialog-like.
       break;
     case Help:
       emit helpClicked();
@@ -894,6 +917,12 @@ void KDialog::slotButtonClicked( int button )
       setDetailsWidgetVisible( !d->mDetailsVisible );
       break;
   }
+
+    // If we're here from the closeEvent, and auto-delete is on, well, auto-delete now.
+    if (d->mDeferredDelete) {
+        d->mDeferredDelete = false;
+        delayedDestruct();
+    }
 }
 
 void KDialog::enableLinkedHelp( bool state )
@@ -959,11 +988,18 @@ void KDialog::hideEvent( QHideEvent *event )
 void KDialog::closeEvent( QCloseEvent *event )
 {
     Q_D(KDialog);
-  KPushButton *button = this->button( d->mEscapeButton );
-  if ( button && !isHidden() )
-    button->animateClick();
-  else
-    QDialog::closeEvent( event );
+    KPushButton *button = this->button(d->mEscapeButton);
+    if (button && !isHidden()) {
+        button->animateClick();
+
+        if (testAttribute(Qt::WA_DeleteOnClose)) {
+            // Don't let QWidget::close do a deferred delete just yet, wait for the click first
+            d->mDeferredDelete = true;
+            setAttribute(Qt::WA_DeleteOnClose, false);
+        }
+    } else {
+        QDialog::closeEvent(event);
+    }
 }
 
 void KDialog::restoreDialogSize( const KConfigGroup& cfg )
