@@ -30,9 +30,12 @@
 #include <QtGui/QToolButton>
 #include <QtGui/QLabel>
 #include <QtGui/QApplication>
+#include <QtGui/QGridLayout>
+#include <QtGui/QCheckBox>
 #include <QMimeData>
 
 #include <kstandarddirs.h>
+#include <klistwidgetsearchline.h>
 #include <klocale.h>
 #include <kicon.h>
 #include <kiconloader.h>
@@ -47,6 +50,7 @@
 #include <ktoolbar.h>
 #include <kdeversion.h>
 #include <kcombobox.h>
+#include <klineedit.h>
 
 #include "kaction.h"
 #include "kactioncollection.h"
@@ -173,7 +177,8 @@ public:
           m_internalTag(tag),
           m_internalName(name),
           m_statusText(statusText),
-          m_isSeparator(false)
+          m_isSeparator(false),
+          m_isTextAlongsideIconHidden(false)
     {
         // Drop between items, not onto items
         setFlags((flags() | Qt::ItemIsDragEnabled) & ~Qt::ItemIsDropEnabled);
@@ -183,10 +188,12 @@ public:
     void setInternalName(const QString &name) { m_internalName = name; }
     void setStatusText(const QString &text) { m_statusText = text; }
     void setSeparator(bool sep) { m_isSeparator = sep; }
+    void setTextAlongsideIconHidden(bool hidden) { m_isTextAlongsideIconHidden = hidden; }
     QString internalTag() const { return m_internalTag; }
     QString internalName() const { return m_internalName; }
     QString statusText() const { return m_statusText; }
     bool isSeparator() const { return m_isSeparator; }
+    bool isTextAlongsideIconHidden() const { return m_isTextAlongsideIconHidden; }
 
     int index() const { return listWidget()->row(const_cast<ToolBarItem*>(this)); }
 
@@ -195,6 +202,7 @@ private:
     QString m_internalName;
     QString m_statusText;
     bool m_isSeparator;
+    bool m_isTextAlongsideIconHidden;
 };
 
 static QDataStream & operator<< ( QDataStream & s, const ToolBarItem & item ) {
@@ -202,6 +210,7 @@ static QDataStream & operator<< ( QDataStream & s, const ToolBarItem & item ) {
     s << item.internalName();
     s << item.statusText();
     s << item.isSeparator();
+    s << item.isTextAlongsideIconHidden();
     return s;
 }
 static QDataStream & operator>> ( QDataStream & s, ToolBarItem & item ) {
@@ -217,6 +226,9 @@ static QDataStream & operator>> ( QDataStream & s, ToolBarItem & item ) {
     bool sep;
     s >> sep;
     item.setSeparator(sep);
+    bool hidden;
+    s >> hidden;
+    item.setTextAlongsideIconHidden(hidden);
     return s;
 }
 
@@ -268,6 +280,63 @@ ToolBarItem* ToolBarListWidget::currentItem() const
     return static_cast<ToolBarItem*>(QListWidget::currentItem());
 }
 
+
+IconTextEditDialog::IconTextEditDialog(QWidget *parent)
+    : KDialog(parent)
+{
+    setCaption(i18n("Change Text"));
+    setButtons(Ok | Cancel);
+    setDefaultButton(Ok);
+    setModal(true);
+
+    QWidget *mainWidget = new QWidget(this);
+    QGridLayout *layout = new QGridLayout(mainWidget);
+    layout->setMargin(0);
+
+    m_lineEdit = new KLineEdit(mainWidget);
+    m_lineEdit->setClearButtonShown(true);
+    QLabel *label = new QLabel(i18n("Icon te&xt:"), this);
+    label->setBuddy(m_lineEdit);
+    layout->addWidget(label, 0, 0);
+    layout->addWidget(m_lineEdit, 0, 1);
+
+    m_cbHidden = new QCheckBox(i18n("&Hide text when toolbar shows text alongside icons"), mainWidget);
+    layout->addWidget(m_cbHidden, 1, 1);
+
+    connect(m_lineEdit, SIGNAL(textChanged(const QString &)), SLOT(slotTextChanged(const QString &)));
+
+    m_lineEdit->setFocus();
+    setMainWidget(mainWidget);
+    setFixedHeight(sizeHint().height());
+}
+
+void IconTextEditDialog::setIconText(const QString &text)
+{
+    m_lineEdit->setText(text);
+}
+
+QString IconTextEditDialog::iconText() const
+{
+    return m_lineEdit->text().trimmed();
+}
+
+void IconTextEditDialog::setTextAlongsideIconHidden(bool hidden)
+{
+    m_cbHidden->setChecked(hidden);
+}
+
+bool IconTextEditDialog::textAlongsideIconHidden() const
+{
+    return m_cbHidden->isChecked();
+}
+
+void IconTextEditDialog::slotTextChanged(const QString &text)
+{
+    // Do not allow empty icon text
+    enableButton(Ok, !text.trimmed().isEmpty());
+}
+
+
 class KEditToolBarWidgetPrivate
 {
 public:
@@ -309,9 +378,10 @@ public:
     void slotUpButton();
     void slotDownButton();
 
-    void selectActiveItem(QString);
-    
+    void selectActiveItem(const QString&);
+
     void slotChangeIcon();
+    void slotChangeIconText();
 
     void slotProcessExited();
 
@@ -413,6 +483,7 @@ public:
     KSeparator *m_comboSeparator;
     QLabel * m_helpArea;
     KPushButton* m_changeIcon;
+    KPushButton* m_changeIconText;
     KProcess* m_kdialogProcess;
     bool m_isPart : 1;
     bool m_hasKDialog : 1;
@@ -529,7 +600,7 @@ void KEditToolBarPrivate::_k_slotDefault()
     if ( KMessageBox::warningContinueCancel(q, i18n("Do you really want to reset all toolbars of this application to their default? The changes will be applied immediately."), i18n("Reset Toolbars"),KGuiItem(i18n("Reset")))!=KMessageBox::Continue )
         return;
 
-    delete m_widget;
+    KEditToolBarWidget * oldWidget = m_widget;
     m_widget = 0;
     m_accept = false;
 
@@ -578,8 +649,10 @@ void KEditToolBarPrivate::_k_slotDefault()
         q->setResourceFile( m_file, m_global );
     }
 
+    // Copy the geometry to minimize UI flicker
+    m_widget->setGeometry( oldWidget->geometry() );
     q->setMainWidget(m_widget);
-    m_widget->show();
+    delete oldWidget;
 
     q->connect(m_widget, SIGNAL(enableOk(bool)), SLOT(_k_acceptOK(bool)));
     q->connect(m_widget, SIGNAL(enableOk(bool)), SLOT(enableButtonApply(bool)));
@@ -720,8 +793,7 @@ void KEditToolBarWidgetPrivate::initKPart( KXMLGUIFactory* factory,
     }
 
     XmlData data(type, client->localXMLFile(), client->actionCollection());
-    QDomDocument domDoc;
-    domDoc.setContent( KXMLGUIFactory::readConfigFile( client->xmlFile(), client->componentData() ) );
+    QDomDocument domDoc = client->domDocument();
     data.setDomDocument(domDoc);
     m_xmlFiles.append(data);
 
@@ -852,6 +924,9 @@ void KEditToolBarWidgetPrivate::setupLayout()
   QObject::connect(m_inactiveList, SIGNAL(dropped(ToolBarListWidget*, int, ToolBarItem*, bool)),
                    m_widget,       SLOT(slotDropped(ToolBarListWidget*, int, ToolBarItem*, bool)));
 
+  KListWidgetSearchLine *inactiveListSearchLine = new KListWidgetSearchLine(m_widget, m_inactiveList);
+  inactiveListSearchLine->setClickMessage(i18n("Filter"));
+
   // our list of active actions
   QLabel *active_label = new QLabel(i18n("Curr&ent actions:"), m_widget);
   m_activeList = new ToolBarListWidget(m_widget);
@@ -868,8 +943,12 @@ void KEditToolBarWidgetPrivate::setupLayout()
   QObject::connect(m_activeList, SIGNAL(dropped(ToolBarListWidget*, int, ToolBarItem*, bool)),
                    m_widget,     SLOT(slotDropped(ToolBarListWidget*, int, ToolBarItem*, bool)));
 
+  KListWidgetSearchLine *activeListSearchLine = new KListWidgetSearchLine(m_widget, m_activeList);
+  activeListSearchLine->setClickMessage(i18n("Filter"));
+
   // "change icon" button
   m_changeIcon = new KPushButton(i18n( "Change &Icon..." ), m_widget);
+  m_changeIcon->setIcon(KIcon("preferences-desktop-icons"));
   QString kdialogExe = KStandardDirs::findExe(QLatin1String("kdialog"));
   m_hasKDialog = !kdialogExe.isEmpty();
   m_changeIcon->setEnabled(m_hasKDialog && m_activeList->currentItem());
@@ -877,6 +956,14 @@ void KEditToolBarWidgetPrivate::setupLayout()
   QObject::connect( m_changeIcon, SIGNAL( clicked() ),
                     m_widget, SLOT( slotChangeIcon() ) );
 
+  // "change icon text" button
+  m_changeIconText = new KPushButton(i18n( "Change Te&xt..." ), m_widget);
+  m_changeIconText->setIcon(KIcon("edit-rename"));
+  m_changeIconText->setEnabled(m_activeList->currentItem() != 0);
+
+  QObject::connect( m_changeIconText, SIGNAL( clicked() ),
+                    m_widget, SLOT( slotChangeIconText() ) );
+
   // The buttons in the middle
 
   m_upAction     = new QToolButton(m_widget);
@@ -907,19 +994,13 @@ void KEditToolBarWidgetPrivate::setupLayout()
   // now start with our layouts
   QVBoxLayout *top_layout = new QVBoxLayout(m_widget);
   top_layout->setMargin(0);
-  top_layout->setSpacing(KDialog::spacingHint());
 
   QVBoxLayout *name_layout = new QVBoxLayout();
-  name_layout->setSpacing(KDialog::spacingHint());
   QHBoxLayout *list_layout = new QHBoxLayout();
-  list_layout->setSpacing(KDialog::spacingHint());
 
   QVBoxLayout *inactive_layout = new QVBoxLayout();
-  inactive_layout->setSpacing(KDialog::spacingHint());
   QVBoxLayout *active_layout = new QVBoxLayout();
-  active_layout->setSpacing(KDialog::spacingHint());
   QHBoxLayout *changeIcon_layout = new QHBoxLayout();
-  changeIcon_layout->setSpacing(KDialog::spacingHint());
 
   QGridLayout *button_layout = new QGridLayout();
 
@@ -937,15 +1018,17 @@ void KEditToolBarWidgetPrivate::setupLayout()
   button_layout->setRowStretch( 4, 10 );
 
   inactive_layout->addWidget(inactive_label);
+  inactive_layout->addWidget(inactiveListSearchLine);
   inactive_layout->addWidget(m_inactiveList, 1);
 
   active_layout->addWidget(active_label);
+  active_layout->addWidget(activeListSearchLine);
   active_layout->addWidget(m_activeList, 1);
   active_layout->addLayout(changeIcon_layout);
 
-  changeIcon_layout->addStretch( 1 );
   changeIcon_layout->addWidget(m_changeIcon);
   changeIcon_layout->addStretch( 1 );
+  changeIcon_layout->addWidget(m_changeIconText);
 
   list_layout->addLayout(inactive_layout);
   list_layout->addLayout(button_layout);
@@ -1071,8 +1154,9 @@ void KEditToolBarWidgetPrivate::loadActions(const QDomElement& elem)
       {
         // we have a match!
         ToolBarItem *act = new ToolBarItem(m_activeList, it.tagName(), action->objectName(), action->toolTip());
-        act->setText(nameFilter.subs(KGlobal::locale()->removeAcceleratorMarker(action->text())).toString());
+        act->setText(nameFilter.subs(KGlobal::locale()->removeAcceleratorMarker(action->iconText())).toString());
         act->setIcon(!action->icon().isNull() ? action->icon() : m_emptyIcon);
+        act->setTextAlongsideIconHidden(action->priority() < QAction::NormalPriority);
 
         active_list.insert(action->objectName());
         break;
@@ -1178,6 +1262,9 @@ void KEditToolBarWidgetPrivate::slotActiveSelectionChanged()
                             m_hasKDialog &&
                             toolitem->internalTag() == "Action" );
 
+  m_changeIconText->setEnabled( toolitem &&
+                                toolitem->internalTag() == "Action" );
+
   if (toolitem)
   {
     m_upAction->setEnabled(toolitem->index() != 0);
@@ -1197,7 +1284,7 @@ void KEditToolBarWidgetPrivate::slotActiveSelectionChanged()
 void KEditToolBarWidgetPrivate::slotInsertButton()
 {
   QString internalName = static_cast<ToolBarItem *>(m_inactiveList->currentItem())->internalName();
-  
+
   insertActive(m_inactiveList->currentItem(), m_activeList->currentItem(), false);
   // we're modified, so let this change
   emit m_widget->enableOk(true);
@@ -1205,11 +1292,11 @@ void KEditToolBarWidgetPrivate::slotInsertButton()
   // TODO: #### this causes #97572.
   // It would be better to just "delete item; loadActions( ... , ActiveListOnly );" or something.
   slotToolBarSelected( m_toolbarCombo->currentIndex() );
-  
+
   selectActiveItem( internalName );
 }
 
-void KEditToolBarWidgetPrivate::selectActiveItem(QString internalName) 
+void KEditToolBarWidgetPrivate::selectActiveItem(const QString& internalName)
 {
   int activeItemCount = m_activeList->count();
   for(int i = 0; i < activeItemCount; i++)
@@ -1441,7 +1528,7 @@ void KEditToolBarWidgetPrivate::slotChangeIcon()
   (*m_kdialogProcess) << "--caption";
   (*m_kdialogProcess) << i18n( "Change Icon" );
   (*m_kdialogProcess) << "--embed";
-  (*m_kdialogProcess) << QString::number( (ulong)m_widget->window()->winId() );
+  (*m_kdialogProcess) << QString::number( (quintptr)m_widget->window()->winId() );
   (*m_kdialogProcess) << "--geticon";
   (*m_kdialogProcess) << "Toolbar";
   (*m_kdialogProcess) << "Actions";
@@ -1462,6 +1549,51 @@ void KEditToolBarWidgetPrivate::slotChangeIcon()
                     m_widget, SLOT( slotProcessExited() ) );
 }
 
+void KEditToolBarWidgetPrivate::slotChangeIconText()
+{
+  m_currentXmlData->dump();
+  ToolBarItem *item = m_activeList->currentItem();
+
+  if(item){
+    QString iconText = item->text();
+    bool hidden = item->isTextAlongsideIconHidden();
+
+    IconTextEditDialog dialog(m_widget);
+    dialog.setIconText(iconText);
+    dialog.setTextAlongsideIconHidden(hidden);
+
+    bool ok = dialog.exec() == KDialog::Accepted;
+    iconText = dialog.iconText();
+    hidden = dialog.textAlongsideIconHidden();
+
+    bool hiddenChanged = hidden != item->isTextAlongsideIconHidden();
+    bool iconTextChanged = iconText != item->text();
+
+    if (!ok || (!hiddenChanged && !iconTextChanged))
+      return;
+
+    item->setText(iconText);
+    item->setTextAlongsideIconHidden(hidden);
+
+    Q_ASSERT( m_currentXmlData->type() != XmlData::Merged );
+
+    m_currentXmlData->m_isModified = true;
+
+    // Get hold of ActionProperties tag
+    QDomElement elem = KXMLGUIFactory::actionPropertiesElement( m_currentXmlData->domDocument() );
+    // Find or create an element for this action
+    QDomElement act_elem = KXMLGUIFactory::findActionByName( elem, item->internalName(), true /*create*/ );
+    Q_ASSERT( !act_elem.isNull() );
+    if (iconTextChanged)
+        act_elem.setAttribute( "iconText", iconText );
+    if (hiddenChanged)
+        act_elem.setAttribute( "priority", hidden ? QAction::LowPriority : QAction::NormalPriority );
+
+    // we're modified, so let this change
+    emit m_widget->enableOk(true);
+  }
+}
+
 void KEditToolBarWidgetPrivate::slotProcessExited()
 {
   m_activeList->setEnabled( true );
