@@ -305,8 +305,13 @@ struct KIEImgEdit
     {
 	if (img.depth() > 8)
         {
-            if(img.format() == QImage::Format_ARGB32_Premultiplied)
+            //Code using data and pixels assumes that the pixels are stored
+            //in 32bit values and that the image is not premultiplied
+            if ((img.format() != QImage::Format_ARGB32) &&
+                (img.format() != QImage::Format_RGB32))
+            {
                 img = img.convertToFormat(QImage::Format_ARGB32);
+            }
             data   = (unsigned int*)img.bits();
 	    pixels = img.width()*img.height();
 	}
@@ -325,6 +330,18 @@ struct KIEImgEdit
     }
 };
 
+static bool painterSupportsAntialiasing()
+{
+#ifdef Q_WS_WIN
+   // apparently QApplication::desktop()->paintEngine() is null on windows
+   // but we can assume the paint engine supports antialiasing there, right?
+   return true;
+#else
+   QPaintEngine* const pe = QApplication::desktop()->paintEngine();
+   return pe && pe->hasFeature(QPaintEngine::Antialiasing);
+#endif
+}
+
 // Taken from KImageEffect. We don't want to link kdecore to kdeui! As long
 // as this code is not too big, it doesn't seem much of a problem to me.
 
@@ -505,8 +522,7 @@ void KIconEffect::semiTransparent(QImage &img)
         int width  = img.width();
 	int height = img.height();
 
-        QPaintEngine* pe = QApplication::desktop()->paintEngine();
-        if(pe && pe->hasFeature(QPaintEngine::Antialiasing)){
+        if(painterSupportsAntialiasing()){
             unsigned char *line;
             for(y=0; y<height; y++){
                 if(QSysInfo::ByteOrder == QSysInfo::BigEndian)
@@ -529,8 +545,7 @@ void KIconEffect::semiTransparent(QImage &img)
     }
     else{
         if (img.depth() == 8) {
-            QPaintEngine *pe = QApplication::desktop()->paintEngine();
-            if (pe && pe->hasFeature(QPaintEngine::Antialiasing)) {
+            if (painterSupportsAntialiasing()) {
                 // not running on 8 bit, we can safely install a new colorTable
                 QVector<QRgb> colorTable = img.colorTable();
                 for (int i = 0; i < colorTable.size(); ++i) {
@@ -595,8 +610,7 @@ void KIconEffect::semiTransparent(QImage &img)
 
 void KIconEffect::semiTransparent(QPixmap &pix)
 {
-    if (QApplication::desktop()->paintEngine() && QApplication::desktop()->paintEngine()->hasFeature(QPaintEngine::Antialiasing))
-    {
+    if (painterSupportsAntialiasing()) {
         QImage img=pix.toImage();
         semiTransparent(img);
         pix = QPixmap::fromImage(img);
@@ -686,7 +700,7 @@ void KIconEffect::overlay(QImage &src, QImage &overlay)
 	return;
     }
     if (src.format() == QImage::Format_ARGB32_Premultiplied)
-        src.convertToFormat(QImage::Format_ARGB32);
+        src = src.convertToFormat(QImage::Format_ARGB32);
 
     if (overlay.format() == QImage::Format_RGB32)
     {
@@ -694,7 +708,7 @@ void KIconEffect::overlay(QImage &src, QImage &overlay)
 	return;
     }
     else if (overlay.format() == QImage::Format_ARGB32_Premultiplied)
-        overlay.convertToFormat(QImage::Format_ARGB32);
+        overlay = overlay.convertToFormat(QImage::Format_ARGB32);
 
     int i, j;
 
