@@ -2,9 +2,10 @@
  *
  * kiconloader.cpp: An icon loader for KDE with theming functionality.
  *
- * This file is part of the KDE project, module kdecore.
+ * This file is part of the KDE project, module kdeui.
  * Copyright (C) 2000 Geert Jansen <jansen@kde.org>
  *                    Antonio Larrosa <larrosa@kde.org>
+ *               2010 Michael Pyne <mpyne@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -27,17 +28,17 @@
 #include "kiconcache.h"
 #include "k3icon_p.h"
 
-#include <QtCore/QCharRef>
-#include <QtCore/QMutableStringListIterator>
+#include <QtCore/QCache>
 #include <QtGui/QPixmap>
 #include <QtGui/QPixmapCache>
 #include <QtGui/QImage>
 #include <QtCore/QFileInfo>
 #include <QtCore/QDir>
+#include <QtCore/QBuffer>
+#include <QtCore/QDataStream>
+#include <QtCore/QByteArray>
 #include <QtGui/QIcon>
-#include <QtGui/QBitmap>
-#include <QHash>
-#include <QPainter>
+#include <QtGui/QPainter>
 #include <QMovie>
 
 #include <kapplication.h>
@@ -49,6 +50,7 @@
 #include <kcomponentdata.h>
 #include <ksvgrenderer.h>
 #include <kde_file.h>
+#include <kshareddatacache.h>
 
 #include <sys/types.h>
 #include <stdlib.h>     //for abs
@@ -66,6 +68,26 @@
 
 //#define NO_LAZYLOAD_ICONTHEME
 
+/**
+ * Checks for relative paths quickly on UNIX-alikes, slowly on everything else.
+ */
+static bool pathIsRelative(const QString &path)
+{
+#ifdef Q_OS_UNIX
+    return (!path.isEmpty() && path[0] != QChar('/'));
+#else
+    return QDir::isRelativePath(path);
+#endif
+}
+
+/**
+ * Holds a QPixmap for this process, along with its associated path on disk.
+ */
+struct PixmapWithPath
+{
+    QPixmap pixmap;
+    QString path;
+};
 
 /*** KIconThemeNode: A node in the icon theme dependancy tree. ***/
 
@@ -133,7 +155,6 @@ struct KIconGroup
 };
 
 
-static const int MAX_SVG_RENDERERS = 100;
 /*** d pointer for KIconLoader. ***/
 class KIconLoaderPrivate
 {
@@ -141,6 +162,7 @@ public:
     KIconLoaderPrivate(KIconLoader *q)
         : q(q)
         , mpGroups(0)
+        , mIconThemeCache(0)
         , mIconCache(0)
     {
     }
@@ -149,10 +171,9 @@ public:
     {
         /* antlarr: There's no need to delete d->mpThemeRoot as it's already
         deleted when the elements of d->links are deleted */
-        qDeleteAll(imgDict);
         qDeleteAll(links);
-        qDeleteAll(svgRenderers);
         delete[] mpGroups;
+        delete mIconThemeCache;
         delete mIconCache;
     }
 
@@ -221,26 +242,63 @@ public:
      */
     QString removeIconExtension(const QString &name) const;
 
-    KIconLoader *q;
+    /**
+     * @internal
+     * Used with KIconLoader::loadIcon to convert the given name, size, group,
+     * and icon state information to valid states. All parameters except the
+     * name can be modified as well to be valid.
+     */
+    void normalizeIconMetadata(KIconLoader::Group &group, int &size, int &state) const;
+
+    /**
+     * @internal
+     * Used with KIconLoader::loadIcon to get a base key name from the given
+     * icon metadata. Ensure the metadata is normalized first.
+     */
+    QString makeCacheKey(const QString &name, KIconLoader::Group group, const QStringList &overlays,
+                         int size, int state) const;
+
+    /**
+     * @internal
+     * Creates the QImage for @p path, using SVG rendering as appropriate.
+     * @p size is only used for scalable images, but if non-zero non-scalable
+     * images will be resized anyways.
+     */
+    QImage createIconImage(const QString &path, int size = 0);
+
+    /**
+     * @internal
+     * Adds an QPixmap with its associated path to the shared icon cache.
+     */
+    void insertCachedPixmapWithPath(const QString &key, const QPixmap &data, const QString &path);
+
+    /**
+     * @internal
+     * Retrieves the path and pixmap of the given key from the shared
+     * icon cache.
+     */
+    bool findCachedPixmapWithPath(const QString &key, QPixmap &data, QString &path);
+
+    KIconLoader *const q;
 
     QStringList mThemesInTree;
     KIconGroup *mpGroups;
     KIconThemeNode *mpThemeRoot;
     KStandardDirs *mpDirs;
     KIconEffect mpEffect;
-    QHash<QString, QImage*> imgDict;
-    QImage lastImage; // last loaded image without effect applied
-    QString lastImageKey; // key for icon without effect
-    int lastIconType; // see KIconLoader::type
-    int lastIconThreshold; // see KIconLoader::threshold
     QList<KIconThemeNode *> links;
-    QHash<QString, KSvgRenderer*> svgRenderers;
-    KIconCache* mIconCache;
+    KIconCache* mIconThemeCache;
+
+    // This shares the icons across all processes
+    KSharedDataCache* mIconCache;
+
+    // This caches rendered QPixmaps in just this process.
+    QCache<QString, PixmapWithPath> mPixmapCache;
+
     bool extraDesktopIconsLoaded :1;
     // lazy loading: initIconThemes() is only needed when the "links" list is needed
     // mIconThemeInited is used inside initIconThemes() to init only once
     bool mIconThemeInited :1;
-    bool lastWasUnknown :1; // last loaded image was the unknown image
     QString appname;
 
     void drawOverlays(const KIconLoader *loader, KIconLoader::Group group, int state, QPixmap& pix, const QStringList& overlays);
@@ -291,6 +349,7 @@ void KIconLoaderGlobalData::parseGenericIconsFiles(const QString& fileName)
         }
     }
 }
+
 K_GLOBAL_STATIC(KIconLoaderGlobalData, s_globalData)
 
 void KIconLoaderPrivate::drawOverlays(const KIconLoader *iconLoader, KIconLoader::Group group, int state, QPixmap& pix, const QStringList& overlays)
@@ -410,14 +469,20 @@ void KIconLoaderPrivate::init( const QString& _appname, KStandardDirs *_dirs )
         appname = KGlobal::mainComponent().componentName();
 
     // Initialize icon cache
-    mIconCache = new KIconCache;
-    if (!mIconCache->isValid()) {
+    mIconCache = new KSharedDataCache("icon-cache", 10 * 1024 * 1024);
+    // Cost here is number of pixels, not size. So this is actually a bit
+    // smaller.
+    mPixmapCache.setMaxCost(10 * 1024 * 1024);
+
+    // Initialize icon theme cache
+    mIconThemeCache = new KIconCache;
+    if (!mIconThemeCache->isValid()) {
         initIconThemes();
         QList<KIconTheme*> allThemes;
         foreach (KIconThemeNode* node, links) {
             allThemes.append(node->theme);
         }
-        mIconCache->setThemeInfo(allThemes);
+        mIconThemeCache->setThemeInfo(allThemes);
     }
 
     // These have to match the order in kicontheme.h
@@ -439,7 +504,7 @@ void KIconLoaderPrivate::init( const QString& _appname, KStandardDirs *_dirs )
             mpGroups[i].alphaBlending = false;
 
         if (!mpGroups[i].size)
-            mpGroups[i].size = mIconCache->defaultIconSize(i);
+            mpGroups[i].size = mIconThemeCache->defaultIconSize(i);
     }
 
 #ifdef NO_LAZYLOAD_ICONTHEME
@@ -657,6 +722,183 @@ QString KIconLoaderPrivate::removeIconExtension(const QString &name) const
     return name;
 }
 
+void KIconLoaderPrivate::normalizeIconMetadata(KIconLoader::Group &group, int &size, int &state) const
+{
+    if ((state < 0) || (state >= KIconLoader::LastState))
+    {
+        kWarning(264) << "Illegal icon state: " << state;
+        state = KIconLoader::DefaultState;
+    }
+
+    if (size < 0) {
+        size = 0;
+    }
+
+    // For "User" icons, bail early since the size should be based on the size on disk,
+    // which we've already checked.
+    if (group == KIconLoader::User) {
+        return;
+    }
+
+    if ((group < -1) || (group >= KIconLoader::LastGroup))
+    {
+        kWarning(264) << "Illegal icon group: " << group;
+        group = KIconLoader::Desktop;
+    }
+
+    // If size == 0, use default size for the specified group.
+    if (size == 0)
+    {
+        if (group < 0)
+        {
+            kWarning(264) << "Neither size nor group specified!";
+            group = KIconLoader::Desktop;
+        }
+        size = mpGroups[group].size;
+    }
+}
+
+QString KIconLoaderPrivate::makeCacheKey(const QString &name, KIconLoader::Group group,
+                                         const QStringList &overlays, int size, int state) const
+{
+    // The KSharedDataCache is shared so add some namespacing.
+    QString key;
+
+    if (group == KIconLoader::User) {
+        key = QLatin1String("$kicou_");
+    }
+    else {
+        key = QLatin1String("$kico_");
+    }
+
+    key.append(QString("%1_%2_%3").arg(name).arg(size).arg(overlays.join("_")));
+
+    if (group >= 0) {
+        key.append(mpEffect.fingerprint(group, state));
+    }
+    else {
+        key.append(QLatin1String("noeffect"));
+    }
+
+    return key;
+}
+
+QImage KIconLoaderPrivate::createIconImage(const QString &path, int size)
+{
+    // Use the extension as the format. Works for XPM and PNG, but not for SVG. The
+    // "VGZ" is the last 3 characters of "SVGZ"
+    QString ext = path.right(3).toUpper();
+    QImage img;
+
+    if (ext != "SVG" && ext != "VGZ")
+    {
+        // Not a SVG or SVGZ
+        img = QImage(path, ext.toLatin1());
+
+        if (size != 0 && !img.isNull()) {
+            img = img.scaled(size, size, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
+        }
+    }
+    else
+    {
+        KSvgRenderer renderer(path, q);
+
+        if (renderer.isValid()) {
+            img = QImage(size, size, QImage::Format_ARGB32_Premultiplied);
+            img.fill(0);
+            QPainter p(&img);
+            renderer.render(&p);
+        }
+    }
+
+    return img;
+}
+
+void KIconLoaderPrivate::insertCachedPixmapWithPath(
+    const QString &key,
+    const QPixmap &data,
+    const QString &path = QString())
+{
+    // Even if the pixmap is null, we add it to the caches so that we record
+    // the fact that whatever icon led to us getting a null pixmap doesn't
+    // exist.
+
+    QBuffer output;
+    output.open(QIODevice::WriteOnly);
+
+    QDataStream outputStream(&output);
+    outputStream.setVersion(QDataStream::Qt_4_6);
+
+    outputStream << path;
+
+    // Convert the QPixmap to PNG. This is actually done by Qt's own operator.
+    outputStream << data;
+
+    output.close();
+
+    // The byte array contained in the QBuffer is what we want in the cache.
+    mIconCache->insert(key, output.buffer());
+
+    // Also insert the object into our process-local cache for even more
+    // speed.
+    PixmapWithPath *pixmapPath = new PixmapWithPath;
+    pixmapPath->pixmap = data;
+    pixmapPath->path = path;
+
+    mPixmapCache.insert(key, pixmapPath, data.width() * data.height() + 1);
+}
+
+bool KIconLoaderPrivate::findCachedPixmapWithPath(const QString &key, QPixmap &data, QString &path)
+{
+    // If the pixmap is present in our local process cache, use that since we
+    // don't need to decompress and upload it to the X server/graphics card.
+    const PixmapWithPath *pixmapPath = mPixmapCache.object(key);
+    if (pixmapPath) {
+        path = pixmapPath->path;
+        data = pixmapPath->pixmap;
+
+        return true;
+    }
+
+    // Otherwise try to find it in our shared memory cache since that will
+    // be quicker than the disk, especially for SVGs.
+    QByteArray result;
+
+    if (!mIconCache->find(key, &result) || result.isEmpty()) {
+        return false;
+    }
+
+    QBuffer buffer;
+    buffer.setBuffer(&result);
+    buffer.open(QIODevice::ReadOnly);
+
+    QDataStream inputStream(&buffer);
+    inputStream.setVersion(QDataStream::Qt_4_6);
+
+    QString tempPath;
+    inputStream >> tempPath;
+
+    if (inputStream.status() == QDataStream::Ok) {
+        QPixmap tempPixmap;
+        inputStream >> tempPixmap;
+
+        if (inputStream.status() == QDataStream::Ok) {
+            data = tempPixmap;
+            path = tempPath;
+
+            // Since we're here we didn't have a QPixmap cache entry, add one now.
+            PixmapWithPath *newPixmapWithPath = new PixmapWithPath;
+            newPixmapWithPath->pixmap = data;
+            newPixmapWithPath->path = path;
+
+            mPixmapCache.insert(key, newPixmapWithPath, data.width() * data.height() + 1);
+
+            return true;
+        }
+    }
+
+    return false;
+}
 
 K3Icon KIconLoaderPrivate::findMatchingIconWithGenericFallbacks(const QString& name, int size) const
 {
@@ -667,29 +909,7 @@ K3Icon KIconLoaderPrivate::findMatchingIconWithGenericFallbacks(const QString& n
     const QString genericIcon = s_globalData->genericIconFor(name);
     if (!genericIcon.isEmpty()) {
         icon = findMatchingIcon(genericIcon, size);
-        if (icon.isValid())
-            return icon;
     }
-
-    // From update-mime-database.c
-    static const char* const media_types[] = {
-        "text", "application", "image", "audio",
-        "inode", "video", "message", "model", "multipart",
-        "x-content", "x-epoc"
-    };
-    // Shared-mime-info spec says:
-    // "If [generic-icon] is not specified then the mimetype is used to generate the
-    // generic icon by using the top-level media type (e.g. "video" in "video/ogg")
-    // and appending "-x-generic" (i.e. "video-x-generic" in the previous example)."
-    for (uint i = 0 ; i < sizeof(media_types)/sizeof(*media_types) ; i++) {
-        if (name.startsWith(QLatin1String(media_types[i]))) {
-            icon = findMatchingIcon(QString::fromLatin1(media_types[i]) + "-x-generic", size);
-            if (icon.isValid())
-                return icon;
-            break;
-        }
-    }
-    // not found
     return icon;
 }
 
@@ -780,7 +1000,7 @@ K3Icon KIconLoaderPrivate::findMatchingIcon(const QString& name, int size) const
     const char * const ext[4] = { ".png", ".svgz", ".svg", ".xpm" };
 #endif
 
-    const bool genericFallback = name.endsWith(QLatin1String("-x-generic"));
+    bool genericFallback = name.endsWith(QLatin1String("-x-generic"));
 
     foreach(KIconThemeNode *themeNode, links)
     {
@@ -825,13 +1045,28 @@ K3Icon KIconLoaderPrivate::findMatchingIcon(const QString& name, int size) const
                 break;
 
             int rindex = currentName.lastIndexOf('-');
-            if (rindex < 0)
-                break;
+            if (rindex > 1) { // > 1 so that we don't split x-content or x-epoc
+                currentName.truncate(rindex);
 
-            currentName.truncate(rindex);
-
-            if (currentName.endsWith(QLatin1String("-x")))
-                currentName.chop(2);
+                if (currentName.endsWith(QLatin1String("-x")))
+                    currentName.chop(2);
+            } else {
+                // From update-mime-database.c
+                static const QSet<QString> mediaTypes = QSet<QString>()
+                    << "text" << "application" << "image" << "audio"
+                    << "inode" << "video" << "message" << "model" << "multipart"
+                    << "x-content" << "x-epoc";
+                // Shared-mime-info spec says:
+                // "If [generic-icon] is not specified then the mimetype is used to generate the
+                // generic icon by using the top-level media type (e.g. "video" in "video/ogg")
+                // and appending "-x-generic" (i.e. "video-x-generic" in the previous example)."
+                if (mediaTypes.contains(currentName)) {
+                    currentName += QLatin1String("-x-generic");
+                    genericFallback = true;
+                } else {
+                    break;
+                }
+            }
         }
     }
     return icon;
@@ -860,12 +1095,7 @@ QString KIconLoader::iconPath(const QString& _name, int group_or_size,
         return QString();
     }
 
-    if (_name.isEmpty()
-#ifdef Q_OS_WIN
-        || !QDir::isRelativePath(_name))
-#else
-        || _name.at(0) == '/')
-#endif
+    if (_name.isEmpty() || !pathIsRelative(_name))
     {
         // we have either an absolute path or nothing to work with
         return _name;
@@ -924,9 +1154,15 @@ QString KIconLoader::iconPath(const QString& _name, int group_or_size,
     return icon.path;
 }
 
-QPixmap KIconLoader::loadMimeTypeIcon( const QString& iconName, KIconLoader::Group group, int size,
+QPixmap KIconLoader::loadMimeTypeIcon( const QString& _iconName, KIconLoader::Group group, int size,
                                        int state, const QStringList& overlays, QString *path_store ) const
 {
+    QString iconName = _iconName;
+    const int slashindex = iconName.indexOf(QLatin1Char('/'));
+    if (slashindex != -1) {
+        iconName[slashindex] = QLatin1Char('-');
+    }
+
     if ( !d->extraDesktopIconsLoaded )
     {
         const QPixmap pixmap = loadIcon( iconName, group, size, state, overlays, path_store, true );
@@ -948,14 +1184,21 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIconLoader::Group group, in
                               QString *path_store, bool canReturnNull) const
 {
     QString name = _name;
-    QString path;
-    QPixmap pix;
     bool unknownIcon = false;
-    bool absolutePath = false;
     bool favIconOverlay = false;
 
-    if (size < 0)
-        return pix;
+    if (size < 0 || _name.isEmpty())
+        return QPixmap();
+
+    /*
+     * This method works in a kind of pipeline, with the following steps:
+     * 1. Sanity checks.
+     * 2. Convert _name, group, size, etc. to a key name.
+     * 3. Check if the key is already cached.
+     * 4. If not, initialize the theme and find/load the icon.
+     * 4a Apply overlays
+     * 4b Re-add to cache.
+     */
 
     // Special case for absolute path icons.
     if (name.startsWith(QLatin1String("favicons/")))
@@ -964,273 +1207,82 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIconLoader::Group group, in
        name = KStandardDirs::locateLocal("cache", name+".png");
     }
 
-    if (!name.isEmpty()
-#ifdef Q_OS_WIN
-        && !QDir::isRelativePath(name))
-#else
-        && name.at(0) == '/')
-#endif
-    {
-        absolutePath = true;
+    bool absolutePath = !pathIsRelative(name);
+    if (!absolutePath) {
+        name = d->removeIconExtension(name);
     }
 
-    static const QString &str_unknown = KGlobal::staticQString("unknown");
+    // Don't bother looking for an icon with no name.
+    if (name.isEmpty()) {
+        return QPixmap();
+    }
 
-    // Special case for "User" icons.
-    if (group == KIconLoader::User)
-    {
-        QString key;
-        key.reserve(200);
-        key.append(QLatin1String("$kicou_"));
-        key.append(name).append('_').append(QString::number(size));
-        key.append(overlays.join("_")); // krazy:exclude=doublequote_chars
+    // May modify group, size, or state. This function puts them into sane
+    // states.
+    d->normalizeIconMetadata(group, size, state);
 
-        if (d->mIconCache->find(key, pix, path_store)) {
-            //kDebug(264) << "KIL: " << "found the icon from KIC";
-            if (!pix.isNull() || canReturnNull) {
-                return pix;
-            } else if (_name != str_unknown) {
-                return loadIcon(str_unknown, group, size, state,
-                            overlays, path_store, canReturnNull);
-            }
-        }
-        if (!d->initIconThemes()) {
-            return pix;  // null pixmap
+    // See if the image is already cached.
+    QString key = d->makeCacheKey(name, group, overlays, size, state);
+    QPixmap pix;
+    bool iconWasUnknown = false;
+    K3Icon icon;
+
+    if (d->findCachedPixmapWithPath(key, pix, icon.path)) {
+        if (path_store) {
+            *path_store = icon.path;
         }
 
-        path = (absolutePath) ? name :
-                        iconPath(name, KIconLoader::User, canReturnNull);
-        if (path.isEmpty())
-        {
-            d->mIconCache->insert(key, pix); //insert the null icon so we know it's null
-            if (!canReturnNull) {
-#ifndef NDEBUG
-                kWarning(264) << "No such icon" << _name;
-#endif
-                unknownIcon = true;
-            } else {
-                return pix; // null pixmap
-            }
-            // We don't know the desired size: use small
-            path = iconPath(str_unknown, KIconLoader::Small, true);
-            if (path.isEmpty())
-            {
-                kWarning(264) << "Warning: Cannot find \"unknown\" icon.";
-                return pix;
-            }
+        // We cache the pixmap for the event of trying to find an unknown icon
+        // with canReturnNull set to false, but if we *can* return null then
+        // we should do so when necessary.
+        if (canReturnNull && icon.path.isEmpty()) {
+            return QPixmap();
         }
-
-        if (path_store != 0L)
-            *path_store = path;
-        //if (inCache)
-        //    return pix;
-        QImage img(path);
-        if (size != 0)
-            img=img.scaled(size,size, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
-
-        pix = QPixmap::fromImage(img);
-        d->drawOverlays(this, KIconLoader::Desktop, state, pix, overlays);
-        if (!unknownIcon)
-            d->mIconCache->insert(key, pix, path);
         return pix;
     }
 
-    // Regular case: Check parameters
-
-    if ((group < -1) || (group >= KIconLoader::LastGroup))
-    {
-        kDebug(264) << "Illegal icon group: " << group;
-        group = KIconLoader::Desktop;
+    // Image is not cached... go find it and apply effects.
+    if (!d->initIconThemes()) {
+        return QPixmap();
     }
 
-    if ((state < 0) || (state >= KIconLoader::LastState))
-    {
-        kDebug(264) << "Illegal icon state: " << state;
-        state = KIconLoader::DefaultState;
-    }
-
-    if (size == 0 && group < 0)
-    {
-        kDebug(264) << "Neither size nor group specified!";
-        group = KIconLoader::Desktop;
-    }
-
-    if (!absolutePath)
-    {
-        if (!canReturnNull && name.isEmpty())
-            name = str_unknown;
-        else
-            name = d->removeIconExtension(name);
-    }
-
-    // If size == 0, use default size for the specified group.
-    if (size == 0)
-    {
-        size = d->mpGroups[group].size;
-    }
     favIconOverlay = favIconOverlay && size > 22;
 
-    // Generate a unique cache key for the icon.
+    // First we look for non-User icons. If we don't find one we'd search in
+    // the User space anyways...
+    if (group != KIconLoader::User) {
+        // K3Icon seems to hold some needed information.
 
-    QString key;
-    key.reserve(100);
-    key.append(QLatin1String("$kico_"));
-    key.append(name).append('_').append(QString::number(size));
-
-    QString overlayKey = overlays.join("_"); // krazy:exclude=doublequote_chars
-    QString noEffectKey = key + overlayKey;
-
-    if (group >= 0)
-    {
-        key.append(d->mpEffect.fingerprint(group, state));
-    } else {
-        key.append(QLatin1String("noeffect"));
-    }
-    key.append(overlayKey);
-
-    // Is the icon in the cache?
-    if (d->mIconCache->find(key, pix, path_store)) {
-        //kDebug() << "KIL: " << "found icon from KIC";
-        if (!pix.isNull() || canReturnNull) {
-            return pix;
-        } else if (_name != str_unknown) {
-            return loadIcon(str_unknown, group, size, state,
-                            overlays, path_store, canReturnNull);
-        }
-    }
-    if (!d->initIconThemes()) {
-        return pix; // null pixmap
-    }
-
-    QImage *img = 0;
-    int iconType;
-    int iconThreshold;
-
-    if ( ( path_store != 0 ) ||
-         ( noEffectKey != d->lastImageKey ) ||
-         ( d->lastWasUnknown && canReturnNull ) )
-    {
-        // Not in cache and not the same as the last requested icon -> load it.
-        K3Icon icon;
         if (absolutePath && !favIconOverlay)
         {
-            icon.context=KIconLoader::Any;
-            icon.type=KIconLoader::Scalable;
-            icon.path=name;
+            icon.context = KIconLoader::Any;
+            icon.type = KIconLoader::Scalable;
+            icon.path = name;
         }
         else
         {
-            if (!name.isEmpty())
-                icon = d->findMatchingIconWithGenericFallbacks(favIconOverlay ? QString("text-html") : name, size);
-
-            if (!icon.isValid())
-            {
-                // Try "User" icon too. Some apps expect this.
-                if (!name.isEmpty()) {
-                    pix = loadIcon(name, KIconLoader::User, size, state, overlays, path_store, true);
-                }
-                d->mIconCache->insert(key, pix, path);
-                if (!pix.isNull() || canReturnNull) {
-                    return pix;
-                }
-#ifndef NDEBUG
-                kWarning(264) << "No such icon" << _name;
-#endif
-                unknownIcon = true;
-                icon = d->findMatchingIcon(str_unknown, size);
-                if (!icon.isValid())
-                {
-                    kDebug(264)
-                        << "Warning: could not find \"Unknown\" icon for size = "
-                        << size << endl;
-                    return pix;
-                }
-            }
+            icon = d->findMatchingIconWithGenericFallbacks(favIconOverlay ? QString("text-html") : name, size);
         }
-
-        if (path_store != 0)
-            *path_store = icon.path;
-
-        // Use the extension as the format. Works for XPM and PNG, but not for SVG
-        QString ext = icon.path.right(3).toUpper();
-        if(ext != "SVG" && ext != "VGZ")
-        {
-            img = new QImage(icon.path, ext.toLatin1());
-            if (img->isNull()) {
-                if (!unknownIcon)
-                    d->mIconCache->insert(key, pix, path);
-                delete img;
-                return pix;
-            }
-        }
-        else
-        {
-            KSvgRenderer *renderer = d->svgRenderers[icon.path];
-            if (!renderer) {
-                renderer = new KSvgRenderer(icon.path);
-                if (renderer->isValid()) {
-                    if (d->svgRenderers.count() >= MAX_SVG_RENDERERS) {
-                        QList<QString> keys = d->svgRenderers.keys();
-                        for (int i = 0; i < MAX_SVG_RENDERERS/2; ++i) {
-                            KSvgRenderer *oldRenderer = d->svgRenderers.take(keys[i]);
-                            delete oldRenderer;
-                        }
-                    }
-                    d->svgRenderers.insert(icon.path, renderer);
-                }
-            }
-            // Special stuff for SVG icons
-
-            if (renderer && renderer->isValid()) {
-                img = new QImage(size, size, QImage::Format_ARGB32_Premultiplied);
-                img->fill(0);
-                QPainter p(img);
-                renderer->render(&p);
-            } else {
-                delete renderer;
-                if (!unknownIcon)
-                    d->mIconCache->insert(key, pix, path);
-                return pix;
-            }
-        }
-
-        iconType = icon.type;
-        iconThreshold = icon.threshold;
-        path = icon.path;
-
-        d->lastImage = img->copy();
-        d->lastImageKey = noEffectKey;
-        d->lastIconType = iconType;
-        d->lastIconThreshold = iconThreshold;
-        d->lastWasUnknown = unknownIcon;
-    }
-    else
-    {
-        img = new QImage( d->lastImage.copy() );
-        iconType = d->lastIconType;
-        iconThreshold = d->lastIconThreshold;
-        unknownIcon = d->lastWasUnknown;
     }
 
-    // Scale the icon and apply effects if necessary
-#ifndef KDE_QT_SVG_RENDERER_FIXED
-    // The following code needs to be removed after the SVG rendering has been
-    // fixed (please take a look at the comment above). Please do not remove the
-    // #if condition as it marks what needs to be removed (ereslibre)
-    if (iconType == KIconLoader::Scalable && size != img->width())
-    {
-        *img = img->scaled(size, size, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
+    if (icon.path.isEmpty()) {
+        // We do have a "User" icon, or we couldn't find the non-User one.
+        icon.path = (absolutePath) ? name :
+                        iconPath(name, KIconLoader::User, canReturnNull);
     }
-#endif
-    if (iconType == KIconLoader::Threshold && size != img->width())
-    {
-        const int sizeDiff = size - img->width();
-        if (sizeDiff < 0 || sizeDiff > iconThreshold)
-            *img = img->scaled(size, size, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
+
+    // Still can't find it? Use "unknown" if we can't return null.
+    // We keep going in the function so we can ensure this result gets cached.
+    if (icon.path.isEmpty() && !canReturnNull) {
+        icon.path = d->unknownIconPath(size);
+        iconWasUnknown = true;
     }
+
+    QImage img = d->createIconImage(icon.path, size);
+
     if (group >= 0)
     {
-        *img = d->mpEffect.apply(*img, group, state);
+        img = d->mpEffect.apply(img, group, state);
     }
 
     if (favIconOverlay)
@@ -1238,40 +1290,37 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIconLoader::Group group, in
         QImage favIcon(name, "PNG");
         if (!favIcon.isNull()) // if favIcon not there yet, don't try to blend it
         {
+            QPainter p(&img);
+
+            // Align the favicon overlay
+            QRect r(favIcon.rect());
+            r.moveBottomRight(img.rect().bottomRight());
+            r.adjust(-1, -1, -1, -1); // Move off edge
+
             // Blend favIcon over img.
-            // FIXME: This code should be updated to use modern QPainter
-            // features.
-            int x = img->width() - favIcon.width() - 1,
-                y = img->height() - favIcon.height() - 1;
-            favIcon = favIcon.convertToFormat(QImage::Format_ARGB32);
-            *img = img->convertToFormat(QImage::Format_ARGB32);
-            for( int line = 0;
-                 line < favIcon.height();
-                 ++line )
-            {
-                QRgb* fpos = reinterpret_cast< QRgb* >( favIcon.scanLine( line ));
-                QRgb* ipos = reinterpret_cast< QRgb* >( img->scanLine( line + y )) + x;
-                for( int i = 0;
-                     i < favIcon.width();
-                     ++i, ++fpos, ++ipos )
-                    *ipos = qRgba( ( qRed( *ipos ) * ( 255 - qAlpha( *fpos )) + qRed( *fpos ) * qAlpha( *fpos )) / 255,
-                                   ( qGreen( *ipos ) * ( 255 - qAlpha( *fpos )) + qGreen( *fpos ) * qAlpha( *fpos )) / 255,
-                                   ( qBlue( *ipos ) * ( 255 - qAlpha( *fpos )) + qBlue( *fpos ) * qAlpha( *fpos )) / 255,
-                                   ( qAlpha( *ipos ) * ( 255 - qAlpha( *fpos )) + qAlpha( *fpos ) * qAlpha( *fpos )) / 255 );
-            }
+            p.drawImage(r, favIcon);
         }
     }
 
-    pix = QPixmap::fromImage(*img);
+    pix = QPixmap::fromImage(img);
 
+    // TODO: If we make a loadIcon that returns the image we can convert
+    // drawOverlays to use the image instead of pixmaps as well so we don't
+    // have to transfer so much to the graphics card.
     d->drawOverlays(this, group, state, pix, overlays);
 
-    delete img;
-
-    if (!unknownIcon)
-    {
-        d->mIconCache->insert(key, pix, path);
+    // Don't add the path to our unknown icon to the cache, only cache the
+    // actual image.
+    if (iconWasUnknown) {
+        icon.path = QString();
     }
+
+    d->insertCachedPixmapWithPath(key, pix, icon.path);
+
+    if (path_store) {
+        *path_store = icon.path;
+    }
+
     return pix;
 }
 
