@@ -87,7 +87,7 @@ public:
         Action,        ///< An action icon (e.g. 'save', 'print').
         Application,   ///< An icon that represents an application.
         Device,        ///< An icon that represents a device.
-        FileSystem,    ///< An icon that represents a file system.
+        FileSystem,    ///< An icon that represents a file system. @deprecated Use Place instead.
         MimeType,      ///< An icon that represents a mime type (or file type).
         Animation,     ///< An icon that is animated.
         Category,      ///< An icon that represents a category.
@@ -131,7 +131,7 @@ public:
         MainToolbar,
         /// Small icons, e.g. for buttons
         Small,
-        /// Panel (Kicker) icons
+        /// Panel (Plasma Taskbar) icons
         Panel,
         /// Icons for use in dialog titles, page lists, etc
         Dialog,
@@ -230,9 +230,14 @@ public:
      * @param overlays a list of emblem icons to overlay, by name. the emblem
      *                 prefix is applied automatically to each name, e.g.
      *                 "zip" becomes "emblem-zip"
-     * @param path_store If not null, the path of the icon is stored here.
+     * @param path_store If not null, the path of the icon is stored here,
+     *        if the icon was found. If the icon was not found @p path_store
+     *        is unaltered even if the "unknown" pixmap was returned.
      * @param canReturnNull Can return a null pixmap? If false, the
-     * "unknown" pixmap is returned when no appropriate icon has been found.
+     *        "unknown" pixmap is returned when no appropriate icon has been
+     *        found. <em>Note:</em> a null pixmap can still be returned in the
+     *        event of invalid parameters, such as empty names, negative sizes,
+     *        and etc.
      * @return the QPixmap. Can be null when not found, depending on
      *         @p canReturnNull.
      */
@@ -386,7 +391,7 @@ public:
      * Returns a pointer to the current theme. Can be used to query
      * available and default sizes for groups.
      * @note The KIconTheme will change if reconfigure() is called and
-     * therefore it's not recommed to store the pointer somewhere.
+     * therefore it's not recommended to store the pointer anywhere.
      * @return a pointer to the current theme. 0 if no theme set.
      */
     KIconTheme *theme() const;
@@ -439,6 +444,12 @@ public:
     */
     void newIconLoader();
 
+Q_SIGNALS:
+    /**
+     * Emitted by newIconLoader once the new settings have been loaded
+     */
+    void iconLoaderSettingsChanged();
+
  private:
     // @internal the data object
     KIconLoaderPrivate *d;
