@@ -32,6 +32,7 @@
 #include <QtCore/QCharRef>
 #include <QtCore/QMutableStringListIterator>
 #include <QtCore/QMap>
+#include <QtCore/QSet>
 #include <QtGui/QPixmap>
 #include <QtGui/QPixmapCache>
 #include <QtGui/QImage>
@@ -132,7 +133,7 @@ KIconTheme::KIconTheme(const QString& name, const QString& appName)
     QStringList icnlibs;
     QStringList::ConstIterator it, itDir;
     QStringList themeDirs;
-    QString cDir;
+    QSet<QString> addedDirs; // Used for avoiding duplicates.
 
     // Applications can have local additions to the global "locolor" and
     // "hicolor" icon themes. For these, the _global_ theme description
@@ -142,7 +143,7 @@ KIconTheme::KIconTheme(const QString& name, const QString& appName)
        ( name == defaultThemeName() || name== "hicolor" || name == "locolor" ) ) {
         icnlibs = KGlobal::dirs()->resourceDirs("data");
         for (it=icnlibs.constBegin(); it!=icnlibs.constEnd(); ++it) {
-            cDir = *it + appName + "/icons/" + name;
+            const QString cDir = *it + appName + "/icons/" + name;
             if (QFile::exists( cDir )) {
                 themeDirs += cDir + '/';
             }
@@ -152,33 +153,35 @@ KIconTheme::KIconTheme(const QString& name, const QString& appName)
 
     icnlibs = KGlobal::dirs()->resourceDirs("icon")
         << KGlobal::dirs()->resourceDirs("xdgdata-icon")
-        << "/usr/share/pixmaps"
+        << "/usr/share/pixmaps/"
         // These are not in the icon spec, but e.g. GNOME puts some icons there anyway.
         << KGlobal::dirs()->resourceDirs("xdgdata-pixmap");
+    icnlibs.removeDuplicates();
+
+    QString fileName, mainSection;
     for (it=icnlibs.constBegin(); it!=icnlibs.constEnd(); ++it) {
-        cDir = *it + name + '/';
+        const QString cDir = *it + name + '/';
         if (KStandardDirs::exists(cDir)) {
             themeDirs += cDir;
-            if (d->mDir.isEmpty() &&
-                (KStandardDirs::exists( cDir + "index.desktop") || KStandardDirs::exists( cDir + "index.theme"))) {
-                d->mDir = cDir;
+            if (d->mDir.isEmpty()) {
+                if (KStandardDirs::exists(cDir + "index.theme")) {
+                    d->mDir = cDir;
+                    fileName = d->mDir + "index.theme";
+                    mainSection = "Icon Theme";
+                } else if (KStandardDirs::exists(cDir + "index.desktop")) {
+                    d->mDir = cDir;
+                    fileName = d->mDir + "index.desktop";
+                    mainSection = "KDE Icon Theme";
+                }
             }
         }
     }
 
     if (d->mDir.isEmpty()) {
-        kDebug(264) << "Icon theme " << name << " not found.\n";
+        kDebug(264) << "Icon theme" << name << "not found.";
         return;
     }
 
-    QString fileName, mainSection;
-    if (QFile::exists(d->mDir + "index.desktop")) {
-        fileName = d->mDir + "index.desktop";
-        mainSection="KDE Icon Theme";
-    } else {
-        fileName = d->mDir + "index.theme";
-        mainSection="Icon Theme";
-    }
     // Use KSharedConfig to avoid parsing the file many times, from each kinstance.
     // Need to keep a ref to it to make this useful
     d->sharedConfig = KSharedConfig::openConfig(fileName);
@@ -204,7 +207,9 @@ KIconTheme::KIconTheme(const QString& name, const QString& appName)
     for (it=dirs.begin(); it!=dirs.end(); ++it) {
         KConfigGroup cg(d->sharedConfig, *it);
         for (itDir=themeDirs.constBegin(); itDir!=themeDirs.constEnd(); ++itDir) {
-            if (KStandardDirs::exists(*itDir + *it + '/')) {
+            const QString currentDir(*itDir + *it + '/');
+            if (!addedDirs.contains(currentDir) && KStandardDirs::exists(currentDir)) {
+                addedDirs.insert(currentDir);
                 KIconThemeDir *dir = new KIconThemeDir(*itDir, *it, cg);
                 if (!dir->isValid()) {
                     delete dir;
@@ -424,9 +429,9 @@ K3Icon KIconTheme::iconPath(const QString& name, int size, KIconLoader::MatchTyp
 {
     K3Icon icon;
     QString path;
-    int delta = -1000;  // current icon size delta of 'icon'
-    int dw = 1000;      // icon size delta of current directory
-    int dirSize = 1000; // directory size of 'icon'
+    int delta = -INT_MAX;  // current icon size delta of 'icon'
+    int dw = INT_MAX;      // icon size delta of current directory
+    int dirSize = INT_MAX; // directory size of 'icon'
     KIconThemeDir *dir;
 
     const int dirCount = d->mDirs.size();
@@ -632,7 +637,7 @@ KIconThemeDir::KIconThemeDir(const QString& basedir, const QString &themedir, co
 {
     mbValid = false;
     mBaseDirThemeDir = basedir + themedir;
-    
+
     mSize = config.readEntry("Size", 0);
     mMinSize = 1;    // just set the variables to something
     mMaxSize = 50;   // meaningful in case someone calls minSize or maxSize
