@@ -40,11 +40,14 @@
 #include "kaction_p.h"
 #include "kmessagebox.h"
 #include "kshortcut.h"
-#include "kglobalaccel_component_interface.h"
-#include "kglobalaccel_interface.h"
 
-static org::kde::kglobalaccel::Component *getComponent(const QString &componentUnique)
-    {
+org::kde::kglobalaccel::Component *KGlobalAccelPrivate::getComponent(const QString &componentUnique, bool remember = false)
+{
+    // Check if we already have this component
+    if (components.contains(componentUnique)) {
+        return components[componentUnique];
+    }
+
     // Connect to the kglobalaccel daemon
     org::kde::KGlobalAccel kglobalaccel(
             "org.kde.kglobalaccel",
@@ -74,7 +77,8 @@ static org::kde::kglobalaccel::Component *getComponent(const QString &componentU
     org::kde::kglobalaccel::Component *component = new org::kde::kglobalaccel::Component(
         "org.kde.kglobalaccel",
         reply.value().path(),
-        QDBusConnection::sessionBus());
+        QDBusConnection::sessionBus(),
+        q);
 
     // No component no cleaning
     if (!component->isValid()) {
@@ -82,6 +86,15 @@ static org::kde::kglobalaccel::Component *getComponent(const QString &componentU
         return NULL;
     }
 
+    if (remember)
+        {
+        // Connect to the signals we are interested in.
+        q->connect(component, SIGNAL(globalShortcutPressed(const QString &, const QString &, qlonglong)),
+                SLOT(_k_invokeAction(const QString &, const QString &, qlonglong)));
+
+        components[componentUnique] = component;
+        }
+
     return component;
 }
 
@@ -90,7 +103,8 @@ static org::kde::kglobalaccel::Component *getComponent(const QString &componentU
 KGlobalAccelPrivate::KGlobalAccelPrivate(KGlobalAccel *q)
      :  isUsingForeignComponentName(false),
         enabled(true),
-        iface("org.kde.kglobalaccel", "/kglobalaccel", QDBusConnection::sessionBus())
+        iface("org.kde.kglobalaccel", "/kglobalaccel", QDBusConnection::sessionBus()),
+        q(q)
 {
     // Make sure kded is running. The iface declaration above somehow
     // works anyway.
@@ -106,8 +120,12 @@ KGlobalAccelPrivate::KGlobalAccelPrivate(KGlobalAccel *q)
             kError() << "Couldn't start kglobalaccel from kglobalaccel.desktop: " << error << endl;
         }
     }
-    QObject::connect(bus, SIGNAL(serviceOwnerChanged(QString, QString, QString)),
-                     q, SLOT(_k_serviceOwnerChanged(QString, QString, QString)));
+    QDBusServiceWatcher *watcher = new QDBusServiceWatcher(iface.service(),
+                                                           QDBusConnection::sessionBus(),
+                                                           QDBusServiceWatcher::WatchForOwnerChange,
+                                                           q);
+    q->connect(watcher, SIGNAL(serviceOwnerChanged(QString,QString,QString)),
+                     q, SLOT(_k_serviceOwnerChanged(QString,QString,QString)));
 }
 
 
@@ -130,8 +148,6 @@ KGlobalAccel::KGlobalAccel()
     qDBusRegisterMetaType<KGlobalShortcutInfo>();
     qDBusRegisterMetaType<QList<KGlobalShortcutInfo> >();
 
-    connect(&d->iface, SIGNAL(invokeAction(const QStringList &, qlonglong)),
-            SLOT(_k_invokeAction(const QStringList &, qlonglong)));
     connect(&d->iface, SIGNAL(yourShortcutGotChanged(const QStringList &, const QList<int> &)),
             SLOT(_k_shortcutGotChanged(const QStringList &, const QList<int> &)));
 
@@ -162,8 +178,8 @@ void KGlobalAccel::activateGlobalShortcutContext(
 // static
 bool KGlobalAccel::cleanComponent(const QString &componentUnique)
 {
-    std::auto_ptr<org::kde::kglobalaccel::Component> component(getComponent(componentUnique));
-    if (!component.get()) return false;
+    org::kde::kglobalaccel::Component* component = self()->getComponent(componentUnique);
+    if (!component) return false;
 
     return component->cleanUp();
 }
@@ -172,8 +188,8 @@ bool KGlobalAccel::cleanComponent(const QString &componentUnique)
 // static
 bool KGlobalAccel::isComponentActive(const QString &componentUnique)
 {
-    std::auto_ptr<org::kde::kglobalaccel::Component> component(getComponent(componentUnique));
-    if (!component.get()) return false;
+    org::kde::kglobalaccel::Component* component = self()->getComponent(componentUnique);
+    if (!component) return false;
 
     return component->isActive();
 }
@@ -185,6 +201,12 @@ bool KGlobalAccel::isEnabled() const
 }
 
 
+org::kde::kglobalaccel::Component *KGlobalAccel::getComponent(const QString &componentUnique)
+{
+    return d->getComponent(componentUnique);
+}
+
+
 void KGlobalAccel::setEnabled(bool enabled)
 {
     d->enabled = enabled;
@@ -293,6 +315,10 @@ void KGlobalAccelPrivate::updateGlobalShortcut(KAction *action, uint flags)
                 actionId,
                 intListFromShortcut(activeShortcut),
                 activeSetterFlags);
+
+        // Make sure we get informed about changes in the component by kglobalaccel
+        getComponent(componentUniqueForAction(action), true);
+
         // Create a shortcut from the result
         const KShortcut scResult(shortcutFromIntList(result));
 
@@ -371,7 +397,10 @@ QString KGlobalAccelPrivate::componentFriendlyForAction(const KAction *action)
 }
 
 
-void KGlobalAccelPrivate::_k_invokeAction(const QStringList &actionId, qlonglong timestamp)
+void KGlobalAccelPrivate::_k_invokeAction(
+        const QString &componentUnique,
+        const QString &actionUnique,
+        qlonglong timestamp)
 {
     // If overrideMainComponentData() is active the app can only have
     // configuration actions.
@@ -380,9 +409,9 @@ void KGlobalAccelPrivate::_k_invokeAction(const QStringList &actionId, qlonglong
     }
 
     KAction *action = 0;
-    QList<KAction *> candidates = nameToAction.values(actionId.at(KGlobalAccel::ActionUnique));
+    QList<KAction *> candidates = nameToAction.values(actionUnique);
     foreach (KAction *const a, candidates) {
-        if (componentUniqueForAction(a) == actionId.at(KGlobalAccel::ComponentUnique)) {
+        if (componentUniqueForAction(a) == componentUnique) {
             action = a;
         }
     }
@@ -391,8 +420,7 @@ void KGlobalAccelPrivate::_k_invokeAction(const QStringList &actionId, qlonglong
     // - there is no action
     // - the action is not enabled
     // - the action is an configuration action
-    if (!action || !action->isEnabled()
-            || action->property("isConfigurationAction").toBool()) {
+    if (!action || !action->isEnabled() || action->property("isConfigurationAction").toBool()) {
         return;
     }
 
@@ -412,6 +440,7 @@ void KGlobalAccelPrivate::_k_invokeAction(const QStringList &actionId, qlonglong
     action->trigger();
 }
 
+
 void KGlobalAccelPrivate::_k_shortcutGotChanged(const QStringList &actionId,
                                                 const QList<int> &keys)
 {
@@ -427,7 +456,8 @@ void KGlobalAccelPrivate::_k_serviceOwnerChanged(const QString &name, const QStr
 {
     Q_UNUSED(oldOwner);
     if (name == QLatin1String("org.kde.kglobalaccel") && !newOwner.isEmpty()) {
-        // kded was restarted (what? you mean it crashes sometimes?)
+        // kglobalaccel was restarted
+        kDebug(123) << "detected kglobalaccel restarting, re-registering all shortcut keys";
         reRegisterAll();
     }
 }
