@@ -70,6 +70,7 @@ public:
 Highlighter::Private::~Private()
 {
   qDeleteAll(dictCache);
+  delete filter;
 }
 
 Highlighter::Highlighter(QTextEdit *textEdit,
@@ -94,7 +95,7 @@ Highlighter::Highlighter(QTextEdit *textEdit,
     textEdit->viewport()->installEventFilter( this );
 
     d->loader = Loader::openLoader();
-    
+
     //Do not load an empty settings file as it will cause the spellchecker to fail
     //if the KGlobal::locale()->language() (default value) spellchecker is not installed,
     //and we have a global sonnetrc file with a spellcheck lang installed which could be used.
@@ -105,7 +106,7 @@ Highlighter::Highlighter(QTextEdit *textEdit,
             d->filter->setSettings(d->loader->settings());
         }
     }
-    
+
     d->dict   = new Sonnet::Speller();
     if(!d->dict->isValid()) {
 	d->spellCheckerFound = false;
@@ -143,7 +144,7 @@ bool Highlighter::spellCheckerFound() const
 }
 
 // Since figuring out spell correction suggestions is extremely costly,
-// we keep track of whether the user actually wants some, and only offer them 
+// we keep track of whether the user actually wants some, and only offer them
 // in that case
 void Highlighter::connectNotify(const char* signal)
 {
@@ -275,26 +276,19 @@ void Highlighter::highlightBlock(const QString &text)
 {
     if (text.isEmpty() || !d->active || !d->spellCheckerFound)
         return;
-    QTextCursor cursor = d->edit->textCursor();
-    int index = cursor.position();
 
-    const int lengthPosition = text.length() - 1;
-
-    if ( index != lengthPosition ||
-         ( lengthPosition > 0 && !text[lengthPosition-1].isLetter() ) ) {
-        d->filter->setBuffer( text );
-        Word w = d->filter->nextWord();
-        while ( !w.end ) {
-            ++d->wordCount;
-            if (d->dict->isMisspelled(w.word)) {
-                ++d->errorCount;
-                setMisspelled(w.start, w.word.length());
-                if (d->suggestionListeners)
-                    emit newSuggestions(w.word, d->dict->suggest(w.word));
-            } else
-                unsetMisspelled(w.start, w.word.length());
-            w = d->filter->nextWord();
-        }
+    d->filter->setBuffer( text );
+    Word w = d->filter->nextWord();
+    while ( !w.end ) {
+        ++d->wordCount;
+        if (d->dict->isMisspelled(w.word)) {
+            ++d->errorCount;
+            setMisspelled(w.start, w.word.length());
+            if (d->suggestionListeners)
+                emit newSuggestions(w.word, d->dict->suggest(w.word));
+        } else
+            unsetMisspelled(w.start, w.word.length());
+        w = d->filter->nextWord();
     }
     //QTimer::singleShot( 0, this, SLOT(checkWords()) );
     setCurrentBlockState(0);
@@ -446,4 +440,10 @@ void Highlighter::setMisspelledColor(const QColor &color)
     d->spellColor = color;
 }
 
+bool Highlighter::checkerEnabledByDefault() const
+{
+    return d->loader->settings()->checkerEnabledByDefault();
+}
+
+
 }
