@@ -4,6 +4,7 @@
  *                    Tom Braun <braunt@fh-konstanz.de>
  * Copyright 2009 KDE e.V.
  *   By Adriaan de Groot <groot@kde.org>
+ * Copyright (C) 2010 George Kiagiadakis <kiagiadakis.george@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -21,12 +22,6 @@
  * Boston, MA 02110-1301, USA.
  */
 
-/*
- * This file is used to catch signals which would normally
- * crash the application (like segmentation fault, floating
- * point exception and such).
- */
-
 #include "kcrash.h"
 #include <kcmdlineargs.h>
 #include <kstandarddirs.h>
@@ -46,6 +41,9 @@
 #include <sys/wait.h>
 #include <sys/un.h>
 #include <sys/socket.h>
+#ifdef Q_OS_LINUX
+#include <sys/prctl.h>
+#endif
 #include <errno.h>
 
 #include <qwindowdefs.h>
@@ -57,6 +55,9 @@
 
 #include <../kinit/klauncher_cmds.h>
 
+#include <QtCore/QFileInfo>
+#include <QtCore/QDir>
+
 #if defined Q_WS_X11
 #include <qx11info_x11.h>
 #include <X11/Xlib.h>
@@ -79,18 +80,13 @@ static char *s_autoRestartCommand = 0;
 static char *s_appPath = 0;
 static char *s_drkonqiPath = 0;
 static KCrash::CrashFlags s_flags = 0;
-static bool s_launchDrKonqi = true;
+static bool s_launchDrKonqi = false;
 
 namespace KCrash
 {
-  void startDrKonqi( const char* argv[], int argc );
-  void startDirectly( const char* argv[], int argc );
+    void startProcess(int argc, const char *argv[], bool waitAndExit);
 }
 
-
-// This function sets the function which should be called when the
-// application crashes and the
-// application is asked to try to save its data.
 void
 KCrash::setEmergencySaveFunction (HandlerType saveFunction)
 {
@@ -101,7 +97,6 @@ KCrash::setEmergencySaveFunction (HandlerType saveFunction)
    * emergencySaveFunction to be called
    */
   if (s_emergencySaveFunction && !s_crashHandler) {
-      s_launchDrKonqi = false; // --nocrashhandler means "no drkonqi please", let's still honor that
       setCrashHandler(defaultCrashHandler);
   }
 }
@@ -109,7 +104,7 @@ KCrash::setEmergencySaveFunction (HandlerType saveFunction)
 KCrash::HandlerType
 KCrash::emergencySaveFunction()
 {
-	return s_emergencySaveFunction;
+    return s_emergencySaveFunction;
 }
 
 // Set the default crash handler in 10 seconds
@@ -142,7 +137,6 @@ KCrash::setFlags(KCrash::CrashFlags flags)
     if (s_flags & AutoRestart) {
         // We need at least the default crash handler for autorestart to work.
         if (!s_crashHandler) {
-            s_launchDrKonqi = false; // KDE_DEBUG=1 or --nocrashhandler means "no drkonqi please", let's still honor that
             KCmdLineArgs *args = KCmdLineArgs::parsedArgs("kde");
             if (!args->isSet("crashhandler")) // --nocrashhandler was passed, probably due to a crash, delay restart handler
                 new KCrashDelaySetHandler;
@@ -152,21 +146,61 @@ KCrash::setFlags(KCrash::CrashFlags flags)
     }
 }
 
+//### KDE5:Consider merging the setApplicationPath and setApplicationName methods into one.
 void
 KCrash::setApplicationPath(const QString& path)
 {
-	s_appPath = qstrdup(path.toLatin1().constData());
+    s_appPath = qstrdup(QFile::encodeName(path).constData());
+
+    //if the appName has also been specified, update s_autoRestartCommand to be in the form "absolutePath/appName"
+    if (s_appName) {
+        delete[] s_autoRestartCommand;
+        QFileInfo appExecutable(QDir(path), QFile::decodeName(s_appName));
+        QByteArray cmd = QFile::encodeName(appExecutable.absoluteFilePath());
+        s_autoRestartCommand = qstrdup(cmd.constData());
+    }
 }
 
 void
 KCrash::setApplicationName(const QString& name)
 {
-	s_appName = qstrdup(name.toLatin1().constData());
-        s_autoRestartCommand = qstrdup(QString(name + " --nocrashhandler &").toLatin1().constData());
+    s_appName = qstrdup(QFile::encodeName(name).constData());
+
+    //update the autoRestartCommand
+    delete[] s_autoRestartCommand;
+    if (s_appPath) {
+        //if we have appPath, make autoRestartCommand be in the form "absolutePath/appName"...
+        QFileInfo appExecutable(QDir(QFile::decodeName(s_appPath)), name);
+        QByteArray cmd = QFile::encodeName(appExecutable.absoluteFilePath());
+        s_autoRestartCommand = qstrdup(cmd.constData());
+    } else {
+        //...else just use the appName for the autoRestartCommand
+        s_autoRestartCommand = qstrdup(s_appName);
+    }
+}
+
+void KCrash::setDrKonqiEnabled(bool enabled)
+{
+    s_launchDrKonqi = enabled;
+    if (s_launchDrKonqi && !s_drkonqiPath) {
+        s_drkonqiPath = qstrdup(QFile::encodeName(KStandardDirs::findExe("drkonqi")).constData());
+        if (!s_drkonqiPath) {
+            kError() << "Could not find drkonqi";
+            s_launchDrKonqi = false;
+        }
+    }
+
+    //we need at least the default crash handler to launch drkonqi
+    if (s_launchDrKonqi && !s_crashHandler) {
+        setCrashHandler(defaultCrashHandler);
+    }
+}
+
+bool KCrash::isDrKonqiEnabled()
+{
+    return s_launchDrKonqi;
 }
 
-// This function sets the function which should be responsible for
-// the application crash handling.
 void
 KCrash::setCrashHandler (HandlerType handler)
 {
@@ -202,15 +236,12 @@ KCrash::setCrashHandler (HandlerType handler)
 #endif //Q_OS_UNIX
 
   s_crashHandler = handler;
-
-  if (!s_drkonqiPath && handler == defaultCrashHandler)
-    s_drkonqiPath = qstrdup(KStandardDirs::findExe("drkonqi").toLatin1().constData());
 }
 
 KCrash::HandlerType
 KCrash::crashHandler()
 {
-	return s_crashHandler;
+    return s_crashHandler;
 }
 
 static void
@@ -227,50 +258,48 @@ void
 KCrash::defaultCrashHandler (int sig)
 {
 #ifdef Q_OS_UNIX
-  // WABA: Do NOT use kDebug() in this function because it is much too risky!
-  // Handle possible recursions
-  static int crashRecursionCounter = 0;
-  crashRecursionCounter++; // Nothing before this, please !
+    // WABA: Do NOT use kDebug() in this function because it is much too risky!
+    // Handle possible recursions
+    static int crashRecursionCounter = 0;
+    crashRecursionCounter++; // Nothing before this, please !
 
-  signal(SIGALRM, SIG_DFL);
-  alarm(3); // Kill me... (in case we deadlock in malloc)
+    signal(SIGALRM, SIG_DFL);
+    alarm(3); // Kill me... (in case we deadlock in malloc)
 
 #ifdef Q_OS_SOLARIS
-  (void) printstack(2 /* stderr, assuming it's still open. */);
+    (void) printstack(2 /* stderr, assuming it's still open. */);
 #endif
 
-  if (crashRecursionCounter < 2) {
-    if (s_emergencySaveFunction) {
-      s_emergencySaveFunction (sig);
+    if (crashRecursionCounter < 2) {
+        if (s_emergencySaveFunction) {
+            s_emergencySaveFunction (sig);
+        }
+        if ((s_flags & AutoRestart) && s_autoRestartCommand) {
+            sleep(1);
+            const char *restartArgv[3] = { s_autoRestartCommand, "--nocrashhandler", NULL };
+            startProcess(2, restartArgv, false);
+        }
+        crashRecursionCounter++;
     }
-    if ((s_flags & AutoRestart) && s_autoRestartCommand) {
-        sleep(1);
-        setCrashHandler(0); // make sure the new process doesn't inherit SIG_IGN from us
-        system(s_autoRestartCommand);
-    }
-    crashRecursionCounter++; //
-  }
 
-  if (!(s_flags & KeepFDs))
-    closeAllFDs();
+    if (!(s_flags & KeepFDs))
+        closeAllFDs();
 #if defined(Q_WS_X11)
-  else if (QX11Info::display())
-    close(ConnectionNumber(QX11Info::display()));
+    else if (QX11Info::display())
+        close(ConnectionNumber(QX11Info::display()));
 #endif
 
-  // this code is leaking, but this should not hurt cause we will do a
-  // exec() afterwards. exec() is supposed to clean up.
     if (crashRecursionCounter < 3)
     {
 #ifndef NDEBUG
         fprintf(stderr, "KCrash: crashing... crashRecursionCounter = %d\n",
-		crashRecursionCounter);
+                crashRecursionCounter);
         fprintf(stderr, "KCrash: Application Name = %s path = %s pid = %d\n",
-		s_appName ? s_appName : "<unknown>" ,
-		s_appPath ? s_appPath : "<unknown>", getpid());
+                s_appName ? s_appName : "<unknown>" ,
+                s_appPath ? s_appPath : "<unknown>", getpid());
 #else
         fprintf(stderr, "KCrash: Application '%s' crashing...\n",
-		s_appName ? s_appName : "<unknown>");
+                s_appName ? s_appName : "<unknown>");
 #endif
 
         if (!s_launchDrKonqi) {
@@ -279,88 +308,84 @@ KCrash::defaultCrashHandler (int sig)
             return;
         }
 
-          const char * argv[25]; // don't forget to update this
-          int i = 0;
+        const char * argv[25]; // don't forget to update this
+        int i = 0;
 
-          // argument 0 has to be drkonqi
-          argv[i++] = s_drkonqiPath;
+        // argument 0 has to be drkonqi
+        argv[i++] = s_drkonqiPath;
 
 #if defined Q_WS_X11
-          // start up on the correct display
-          argv[i++] = "-display";
-          if ( QX11Info::display() )
+        // start up on the correct display
+        argv[i++] = "-display";
+        if ( QX11Info::display() )
             argv[i++] = XDisplayString(QX11Info::display());
-          else
+        else
             argv[i++] = getenv("DISPLAY");
 #elif defined(Q_WS_QWS)
-          // start up on the correct display
-          argv[i++] = "-display";
-          argv[i++] = getenv("QWS_DISPLAY");
+        // start up on the correct display
+        argv[i++] = "-display";
+        argv[i++] = getenv("QWS_DISPLAY");
 #endif
 
-          argv[i++] = "--appname";
-          argv[i++] = s_appName ? s_appName : "<unknown>";
+        argv[i++] = "--appname";
+        argv[i++] = s_appName ? s_appName : "<unknown>";
 
-          if (KApplication::loadedByKdeinit)
+        if (KApplication::loadedByKdeinit)
             argv[i++] = "--kdeinit";
 
-          // only add apppath if it's not NULL
-          if (s_appPath && *s_appPath) {
+        // only add apppath if it's not NULL
+        if (s_appPath && *s_appPath) {
             argv[i++] = "--apppath";
             argv[i++] = s_appPath;
-          }
+        }
 
-          // signal number -- will never be NULL
-          char sigtxt[ 10 ];
-          sprintf( sigtxt, "%d", sig );
-          argv[i++] = "--signal";
-          argv[i++] = sigtxt;
+        // signal number -- will never be NULL
+        char sigtxt[ 10 ];
+        sprintf( sigtxt, "%d", sig );
+        argv[i++] = "--signal";
+        argv[i++] = sigtxt;
 
-          char pidtxt[ 10 ];
-          sprintf( pidtxt, "%d", getpid());
-          argv[i++] = "--pid";
-          argv[i++] = pidtxt;
+        char pidtxt[ 10 ];
+        sprintf( pidtxt, "%d", getpid());
+        argv[i++] = "--pid";
+        argv[i++] = pidtxt;
 
-          const KComponentData componentData = KGlobal::mainComponent();
-          const KAboutData *about = componentData.isValid() ? componentData.aboutData() : 0;
-          if (about) {
+        const KComponentData componentData = KGlobal::mainComponent();
+        const KAboutData *about = componentData.isValid() ? componentData.aboutData() : 0;
+        if (about) {
             if (about->internalVersion()) {
-              argv[i++] = "--appversion";
-              argv[i++] = about->internalVersion();
+                argv[i++] = "--appversion";
+                argv[i++] = about->internalVersion();
             }
 
             if (about->internalProgramName()) {
-              argv[i++] = "--programname";
-              argv[i++] = about->internalProgramName();
+                argv[i++] = "--programname";
+                argv[i++] = about->internalProgramName();
             }
 
             if (about->internalBugAddress()) {
-              argv[i++] = "--bugaddress";
-              argv[i++] = about->internalBugAddress();
+                argv[i++] = "--bugaddress";
+                argv[i++] = about->internalBugAddress();
             }
-          }
+        }
 
-          char sidtxt[256];
-          if ( kapp && !kapp->startupId().isNull()) {
+        char sidtxt[256];
+        if ( kapp && !kapp->startupId().isNull()) {
             argv[i++] = "--startupid";
             strlcpy(sidtxt, kapp->startupId().constData(), sizeof(sidtxt));
             argv[i++] = sidtxt;
-          }
+        }
 
-          if ( s_flags & SaferDialog )
+        if ( s_flags & SaferDialog )
             argv[i++] = "--safer";
 
-          if ((s_flags & AutoRestart) && s_autoRestartCommand)
+        if ((s_flags & AutoRestart) && s_autoRestartCommand)
             argv[i++] = "--restarted"; //tell drkonqi if the app has been restarted
 
-          // NULL terminated list
-          argv[i] = NULL;
+        // NULL terminated list
+        argv[i] = NULL;
 
-          if (!(s_flags & AlwaysDirectly)) {
-            startDrKonqi( argv, i );
-            fprintf( stderr, "KCrash cannot reach kdeinit, launching directly.\n" );
-          }
-          startDirectly( argv, i );
+        startProcess(i, argv, true);
     }
 
     if (crashRecursionCounter < 4)
@@ -369,24 +394,75 @@ KCrash::defaultCrashHandler (int sig)
     }
 #endif //Q_OS_UNIX
 
-  _exit(255);
+    _exit(255);
 }
 
 #ifdef Q_OS_UNIX
 
-// Since we can't fork() in the crashhandler, we cannot execute any external code
-// (there can be functions registered to be performed before fork(), for example
-// handling of malloc locking, which doesn't work when malloc crashes because of heap corruption).
-
+static bool startProcessInternal(int argc, const char *argv[], bool waitAndExit, bool directly);
+static pid_t startFromKdeinit(int argc, const char *argv[]);
+static pid_t startDirectly(const char *argv[]);
 static int write_socket(int sock, char *buffer, int len);
 static int read_socket(int sock, char *buffer, int len);
 static int openSocket();
 
-void KCrash::startDrKonqi( const char* argv[], int argc )
+void KCrash::startProcess(int argc, const char *argv[], bool waitAndExit)
+{
+    bool startDirectly = true;
+
+    // First try to start the app via kdeinit, if the AlwaysDirectly flag hasn't been specified.
+    // This is done because it is dangerous to use fork() in the crash handler
+    // (there can be functions registered to be performed before fork(), for example handling
+    // of malloc locking, which doesn't work when malloc crashes because of heap corruption).
+    if (!(s_flags & AlwaysDirectly)) {
+        startDirectly = !startProcessInternal(argc, argv, waitAndExit, false);
+    }
+
+    // If we can't reach kdeinit, we can still at least try to fork()
+    if (startDirectly) {
+        startProcessInternal(argc, argv, waitAndExit, true);
+    }
+}
+
+static bool startProcessInternal(int argc, const char *argv[], bool waitAndExit, bool directly)
+{
+    fprintf(stderr, "KCrash: Attempting to start %s %s\n", argv[0], directly ? "directly" : "from kdeinit");
+
+    pid_t pid = directly ? startDirectly(argv) : startFromKdeinit(argc, argv);
+
+    if (pid > 0 && waitAndExit) {
+        // Seems we made it....
+        alarm(0); //stop the pending alarm that was set at the top of the defaultCrashHandler
+
+        // Wait forever until the started process exits. This code path is executed
+        // when launching drkonqi. Note that drkonqi will stop this process in the meantime.
+        if (directly) {
+            //if the process was started directly, use waitpid(), as it's a child...
+            while(waitpid(-1, NULL, 0) != pid) {}
+        } else {
+#ifdef Q_OS_LINUX
+            // Declare the process that will be debugging the crashed KDE app (#245529)
+#ifndef PR_SET_PTRACER
+# define PR_SET_PTRACER 0x59616d61
+#endif
+            prctl(PR_SET_PTRACER, pid, 0, 0, 0);
+#endif
+            //...else poll its status using kill()
+            while(kill(pid, 0) >= 0) {
+                sleep(1);
+            }
+        }
+        _exit(253);
+    }
+
+    return (pid > 0); //return true on success
+}
+
+static pid_t startFromKdeinit(int argc, const char *argv[])
 {
   int socket = openSocket();
   if( socket < -1 )
-    return;
+    return 0;
   klauncher_header header;
   header.cmd = LAUNCHER_EXEC_NEW;
   const int BUFSIZE = 8192; // make sure this is big enough
@@ -403,7 +479,7 @@ void KCrash::startDrKonqi( const char* argv[], int argc )
     if( pos + len >= BUFSIZE )
     {
       fprintf( stderr, "BUFSIZE in KCrash not big enough!\n" );
-      return;
+      return 0;
     }
     memcpy( buffer + pos, argv[ i ], len );
     pos += len;
@@ -420,44 +496,31 @@ void KCrash::startDrKonqi( const char* argv[], int argc )
   if( read_socket( socket, (char *) &header, sizeof(header)) < 0
       || header.cmd != LAUNCHER_OK )
   {
-    return;
+    return 0;
   }
   long pid;
   read_socket(socket, buffer, header.arg_length);
   pid = *((long *) buffer);
-
-  alarm(0); // Seems we made it....
-
-  for(;;)
-  {
-    if( kill( pid, 0 ) < 0 )
-      _exit(253);
-    sleep( 1 ); // the debugger should stop this process anyway
-  }
+  return static_cast<pid_t>(pid);
 }
 
-// If we can't reach kdeinit we can still at least try to fork()
-void KCrash::startDirectly( const char* argv[], int )
+static pid_t startDirectly(const char *argv[])
 {
   pid_t pid = fork();
   switch (pid)
   {
   case -1:
     fprintf( stderr, "KCrash failed to fork(), errno = %d\n", errno );
-    _exit(253);
+    return 0;
   case 0:
     if (setgid(getgid()) < 0 || setuid(getuid()) < 0)
       _exit(253); // This cannot happen. Theoretically.
-    if (s_flags & KeepFDs)
-      closeAllFDs();
-    execvp(s_drkonqiPath, const_cast< char** >( argv ));
+    closeAllFDs(); // We are in the child now. Close FDs unconditionally.
+    execvp(argv[0], const_cast< char** >(argv));
     fprintf( stderr, "KCrash failed to exec(), errno = %d\n", errno );
     _exit(253);
   default:
-    alarm(0); // Seems we made it....
-    // wait for child to exit
-    while(waitpid(-1, NULL, 0) != pid) {}
-    _exit(253);
+    return pid;
   }
 }
 
