@@ -30,7 +30,6 @@
 #include <QtCore/QtGlobal>
 #include <QtGui/QPainter>
 #include <QtCore/QQueue>
-#include <QtCore/QThread>
 #include <QtCore/QTimer>
 #include <QtCore/QMutex>
 #include <QtCore/QMutexLocker>
@@ -40,6 +39,7 @@
 #include <kstandarddirs.h>
 #include <kdebug.h>
 #include <klockfile.h>
+#include <ksavefile.h>
 #include <ksvgrenderer.h>
 #include <kdefakes.h>
 
@@ -50,7 +50,7 @@
 #include <sys/types.h>
 #include <string.h>
 
-#if defined(HAVE_SYS_MMAN_H) && defined(HAVE_MADVISE)
+#if defined(HAVE_MADVISE)
 #include <sys/mman.h>
 #endif
 
@@ -170,7 +170,7 @@ KPCMemoryDevice::KPCMemoryDevice(char* start, quint32* size, quint32 available)
     mAvailable = available;
     mPos = 0;
 
-    open(QIODevice::ReadWrite); // krazy:exclude=syscalls
+    this->open(QIODevice::ReadWrite);
 
     // Load up-to-date size from the memory
     *mSize = mHeader->size;
@@ -243,6 +243,8 @@ public:
     // List of all KPixmapCache::Private instances in this process.
     static QList<KPixmapCache::Private *> mCaches;
 
+    static unsigned kpcNumber; // Used to setup for qpcKey
+
     int findOffset(const QString& key);
     int binarySearchKey(QDataStream& stream, const QString& key, int start);
     void writeIndexEntry(QDataStream& stream, const QString& key, int dataoffset);
@@ -264,9 +266,13 @@ public:
     //  lookups faster as the beginnings of the keys are more random
     QString indexKey(const QString& key);
 
+    // Returns a QString suitable for use in the static QPixmapCache, which
+    // differentiates each KPC object in the process.
+    QString qpcKey(const QString& key) const;
 
     KPixmapCache* q;
 
+    QString mThisString; // Used by qpcKey
     quint32 mHeaderSize;  // full size of the index header, including custom (subclass') header data
     quint32 mIndexRootOffset;  // offset of the first entry in index file
 
@@ -342,55 +348,18 @@ public:
     {
         return a.lastused > b.lastused;
     }
-
-    // Helper class to run the possibly expensive removeEntries() operation in
-    //  the background thread
-    class RemovalThread : public QThread
-    {
-    public:
-        RemovalThread(KPixmapCache::Private* _d) : QThread()
-        {
-            d = _d;
-            mRemovalScheduled = false;
-        }
-        ~RemovalThread()
-        {
-        }
-
-        void scheduleRemoval(int newsize)
-        {
-            mNewSize = newsize;
-            if (!mRemovalScheduled) {
-                QTimer::singleShot(5000, this, SLOT(start()));
-                mRemovalScheduled = true;
-            }
-        }
-
-    protected:
-        virtual void run()
-        {
-            mRemovalScheduled = false;
-            kDebug(264) << "starting";
-            d->removeEntries(mNewSize);
-            kDebug(264) << "done";
-        }
-
-    private:
-        bool mRemovalScheduled;
-        int mNewSize;
-        KPixmapCache::Private* d;
-    };
-    RemovalThread* mRemovalThread;
 };
 
 // List of KPixmapCache::Private instances.
 QList<KPixmapCache::Private *> KPixmapCache::Private::mCaches;
 
+unsigned KPixmapCache::Private::kpcNumber = 0;
+
 KPixmapCache::Private::Private(KPixmapCache* _q)
 {
     q = _q;
     mCaches.append(this);
-    mRemovalThread = 0;
+    mThisString = QString("%1").arg(kpcNumber++);
 }
 
 KPixmapCache::Private::~Private()
@@ -455,7 +424,7 @@ bool KPixmapCache::Private::mmapFile(const QString& filename, MmapInfo* info, in
 
     // Only resize if greater than current file size, otherwise we may cause SIGBUS
     // errors from mmap().
-    if (info->file->size() < info->available && ftruncate(info->file->handle(), info->available) < 0) {
+    if (info->file->size() < info->available && !info->file->resize(info->available)) {
         kError(264) << "Couldn't resize" << filename << "to" << newsize;
         delete info->file;
         info->file = 0;
@@ -472,7 +441,7 @@ bool KPixmapCache::Private::mmapFile(const QString& filename, MmapInfo* info, in
     }
     info->indexHeader = reinterpret_cast<KPixmapCacheIndexHeader *>(indexMem);
 #ifdef HAVE_MADVISE
-    madvise(indexMem, info->size, MADV_WILLNEED);
+    posix_madvise(indexMem, info->size, POSIX_MADV_WILLNEED);
 #endif
 
     info->file->close();
@@ -624,6 +593,10 @@ int KPixmapCache::Private::binarySearchKey(QDataStream& stream, const QString& k
     qint32 leftchild, rightchild;
     stream >> fkey >> foffset >> timesused >> lastused >> leftchild >> rightchild;
 
+    if (fkey.isEmpty()) {
+        return start;
+    }
+
     if (key < fkey) {
         if (leftchild) {
             return binarySearchKey(stream, key, leftchild);
@@ -651,12 +624,23 @@ int KPixmapCache::Private::findOffset(const QString& key)
     //  exist yet and there are no entries. Otherwise, do a binary search
     //  starting from the root node.
     if (!stream.atEnd()) {
+        // One exception is that the root node may exist but be invalid,
+        // which can happen when the cache data is discarded. This is
+        // represented by an empty fkey
+        QString fkey;
+        stream >> fkey;
+
+        if (fkey.isEmpty()) {
+            delete device;
+            return -1;
+        }
+
         int nodeoffset = binarySearchKey(stream, key, mIndexRootOffset);
 
         // Load the found entry and check if it's the one we're looking for.
         device->seek(nodeoffset);
-        QString fkey;
         stream >> fkey;
+
         if (fkey == key) {
             // Read offset and statistics
             qint32 foffset;
@@ -788,6 +772,12 @@ QString KPixmapCache::Private::indexKey(const QString& key)
     return QString("%1%2").arg((ushort)qChecksum(latin1.data(), latin1.size()), 4, 16, QLatin1Char('0')).arg(key);
 }
 
+
+QString KPixmapCache::Private::qpcKey(const QString& key) const
+{
+    return mThisString + key;
+}
+
 void KPixmapCache::Private::writeIndexEntry(QDataStream& stream, const QString& key, int dataoffset)
 {
     // New entry will be written to the end of the file
@@ -799,7 +789,9 @@ void KPixmapCache::Private::writeIndexEntry(QDataStream& stream, const QString&
         QString fkey;
         stream.device()->seek(parentoffset);
         stream >> fkey;
-        if (key == fkey) {
+
+        // The key would be empty if the cache had been discarded.
+        if (key == fkey || fkey.isEmpty()) {
             // We're overwriting an existing entry
             offset = parentoffset;
         }
@@ -833,15 +825,6 @@ void KPixmapCache::Private::writeIndexEntry(QDataStream& stream, const QString&
     }
 }
 
-bool KPixmapCache::Private::scheduleRemoveEntries(int newsize)
-{
-    if (!mRemovalThread) {
-        mRemovalThread = new RemovalThread(this);
-    }
-    mRemovalThread->scheduleRemoval(newsize);
-    return true;
-}
-
 bool KPixmapCache::Private::removeEntries(int newsize)
 {
     KPCLockFile lock(mLockFileName);
@@ -1011,10 +994,6 @@ KPixmapCache::KPixmapCache(const QString& name)
 KPixmapCache::~KPixmapCache()
 {
     d->unmmapFiles();
-    if (d->mRemovalThread) {
-        d->mRemovalThread->wait();
-        delete d->mRemovalThread;
-    }
     delete d;
 }
 
@@ -1155,11 +1134,9 @@ void KPixmapCache::setCacheLimit(int kbytes)
     // if we are initialized, let's make sure that we are actually within
     // our limits.
     if (d->mInited && d->mCacheLimit && size() > d->mCacheLimit) {
-        if (size() > (int)(d->mCacheLimit * 1.2)) {
+        if (size() > (int)(d->mCacheLimit)) {
             // Can't wait any longer, do it immediately
-            d->removeEntries(d->mCacheLimit * 0.75);
-        } else {
-            d->scheduleRemoveEntries(int(d->mCacheLimit * 0.75));
+            d->removeEntries(d->mCacheLimit * 0.65);
         }
     }
 }
@@ -1180,47 +1157,38 @@ bool KPixmapCache::recreateCacheFiles()
         return false;
     }
 
-    d->invalidateMmapFiles();
-    d->unmmapFiles();
-
-    d->mEnabled = false;
-
     KPCLockFile lock(d->mLockFileName);
     // Hope we got the lock...
 
+    d->invalidateMmapFiles();
+    d->mEnabled = false;
+
     // Create index file
-    QFile indexfile(d->mIndexFile);
+    KSaveFile indexfile(d->mIndexFile);
     if (!indexfile.open(QIODevice::WriteOnly)) {
         kError() << "Couldn't create index file" << d->mIndexFile;
         return false;
     }
 
-    KPixmapCacheIndexHeader indexHeader;
     d->mCacheId = ::time(0);
     d->mTimestamp = ::time(0);
 
-    memcpy(indexHeader.magic, KPC_MAGIC, sizeof(indexHeader.magic));
-    indexHeader.cacheVersion = KPIXMAPCACHE_VERSION;
-    indexHeader.timestamp = d->mTimestamp;
-    indexHeader.cacheId = d->mCacheId;
-
     // We can't know the full size until custom headers written.
     // mmapFiles() will take care of correcting the size.
-    indexHeader.size = 0;
+    KPixmapCacheIndexHeader indexHeader = { {0}, KPIXMAPCACHE_VERSION, 0, d->mCacheId, d->mTimestamp };
+    memcpy(indexHeader.magic, KPC_MAGIC, sizeof(indexHeader.magic));
 
     indexfile.write(reinterpret_cast<char*>(&indexHeader), sizeof indexHeader);
 
     // Create data file
-    QFile datafile(d->mDataFile);
+    KSaveFile datafile(d->mDataFile);
     if (!datafile.open(QIODevice::WriteOnly)) {
         kError() << "Couldn't create data file" << d->mDataFile;
         return false;
     }
 
-    KPixmapCacheDataHeader dataHeader;
+    KPixmapCacheDataHeader dataHeader = { {0}, KPIXMAPCACHE_VERSION, sizeof dataHeader };
     memcpy(dataHeader.magic, KPC_MAGIC, sizeof(dataHeader.magic));
-    dataHeader.cacheVersion = KPIXMAPCACHE_VERSION;
-    dataHeader.size = sizeof dataHeader;
 
     datafile.write(reinterpret_cast<char*>(&dataHeader), sizeof dataHeader);
 
@@ -1235,38 +1203,54 @@ bool KPixmapCache::recreateCacheFiles()
     // Close the files and mmap them (if mmapping is used)
     indexfile.close();
     datafile.close();
+    indexfile.finalize();
+    datafile.finalize();
+
     d->mEnabled = true;
     d->mmapFiles();
+
     return true;
 }
 
 void KPixmapCache::deleteCache(const QString& name)
 {
-    foreach (KPixmapCache::Private *d, Private::mCaches) {
-        if (d->mName == name && d->mInited) {
-            d->q->discard();
-        }
-    }
+    QString indexFile = KGlobal::dirs()->locateLocal("cache", "kpc/" + name + ".index");
+    QString dataFile  = KGlobal::dirs()->locateLocal("cache", "kpc/" + name + ".data");
+
+    QFile::remove(indexFile);
+    QFile::remove(dataFile);
 }
 
 void KPixmapCache::discard()
 {
-    d->invalidateMmapFiles();
-    d->unmmapFiles();
-    d->mInited = false;
+    // To "discard" the cache we simply have to make sure that every that
+    // was in there before is no longer present when we search for them.
+    // Easiest way to do *that* is to simply delete the index.
 
-    if (d->mUseQPixmapCache) {
-        QPixmapCache::clear();
+    KPCLockFile lock(d->mLockFileName);
+    if(!lock.isValid()) {
+        kError(264) << "Unable to lock pixmap cache when trying to discard it";
+        return;
     }
 
-    QString indexFile = KGlobal::dirs()->locateLocal("cache", "kpc/" + d->mName + ".index");
-    QString dataFile  = KGlobal::dirs()->locateLocal("cache", "kpc/" + d->mName + ".data");
+    QIODevice *device = d->indexDevice();
+    if (!device) {
+        kError(264) << "Unable to access index when trying to discard cache";
+        return;
+    }
 
-    QFile::remove(indexFile);
-    QFile::remove(dataFile);
+    device->seek(d->mIndexRootOffset);
+    QDataStream stream(device);
 
-    // No need to remove the lockfile
-    d->init();
+    // Stream an empty QString as the hash key to signify that the cache
+    // has been discarded.
+    stream << QString();
+
+    if (d->mUseQPixmapCache) {
+        // TODO: This is broken, it removes every cached QPixmap in the whole
+        // process, not just this KPixmapCache.
+        QPixmapCache::clear();
+    }
 }
 
 void KPixmapCache::removeEntries(int newsize)
@@ -1292,7 +1276,7 @@ bool KPixmapCache::find(const QString& key, QPixmap& pix)
 
     //kDebug(264) << "key:" << key << ", use QPC:" << d->mUseQPixmapCache;
     // First try the QPixmapCache
-    if (d->mUseQPixmapCache && QPixmapCache::find(key, pix)) {
+    if (d->mUseQPixmapCache && QPixmapCache::find(d->qpcKey(key), &pix)) {
         //kDebug(264) << "Found from QPC";
         return true;
     }
@@ -1314,7 +1298,7 @@ bool KPixmapCache::find(const QString& key, QPixmap& pix)
     bool ret = d->loadData(offset, pix);
     if (ret && d->mUseQPixmapCache) {
         // This pixmap wasn't in QPC, put it there
-        QPixmapCache::insert(key, pix);
+        QPixmapCache::insert(d->qpcKey(key), pix);
     }
     return ret;
 }
@@ -1387,7 +1371,7 @@ void KPixmapCache::insert(const QString& key, const QPixmap& pix)
     //kDebug(264) << "key:" << key << ", size:" << pix.width() << "x" << pix.height();
     // Insert to QPixmapCache as well
     if (d->mUseQPixmapCache) {
-        QPixmapCache::insert(key, pix);
+        QPixmapCache::insert(d->qpcKey(key), pix);
     }
 
     KPCLockFile lock(d->mLockFileName);
@@ -1408,11 +1392,9 @@ void KPixmapCache::insert(const QString& key, const QPixmap& pix)
     // Make sure the cache size stays within limits
     if (d->mCacheLimit && size() > d->mCacheLimit) {
         lock.unlock();
-        if (size() > (int)(d->mCacheLimit * 1.2)) {
+        if (size() > (int)(d->mCacheLimit)) {
             // Can't wait any longer, do it immediately
-            d->removeEntries(d->mCacheLimit * 0.75);
-        } else {
-            d->scheduleRemoveEntries(int(d->mCacheLimit * 0.75));
+            d->removeEntries(d->mCacheLimit * 0.65);
         }
     }
 }
