@@ -27,7 +27,6 @@
 
 #include "klineedit.h"
 #include "klineedit_p.h"
-#include "kdeuiwidgetsproxystyle_p.h"
 
 #include <kaction.h>
 #include <kapplication.h>
@@ -49,6 +48,8 @@
 #include <QtGui/QStyleOption>
 #include <QtGui/QToolTip>
 
+class KLineEditStyle;
+
 class KLineEditPrivate
 {
 public:
@@ -68,18 +69,15 @@ public:
         enableClickMsg = false;
         threeStars = false;
         completionRunning = false;
-        if ( !initialized )
-        {
+        if (!s_initialized) {
             KConfigGroup config( KGlobal::config(), "General" );
-            backspacePerformsCompletion = config.readEntry("Backspace performs completion", false);
-
-            initialized = true;
+            s_backspacePerformsCompletion = config.readEntry("Backspace performs completion", false);
+            s_initialized = true;
         }
 
         clearButton = 0;
         clickInClear = false;
         wideEnoughForClear = true;
-        overlap = 0;
 
         // i18n: Placeholder text in line edit widgets is the text appearing
         // before any user input, briefly explaining to the user what to type
@@ -104,16 +102,42 @@ public:
             clearButton->setAnimationsEnabled(KGlobalSettings::graphicEffectsLevel() & KGlobalSettings::SimpleAnimationEffects);
         }
     }
-    
-    void _k_updateUserText( const QString &txt)
-    {  
-        if ((!completionRunning) && (txt != userText))
-	{
-	    userText = txt;
-	    emit q->userTextChanged(txt);
-	}
+
+    void _k_textChanged(const QString &txt)
+    {
+        // COMPAT (as documented): emit userTextChanged whenever textChanged is emitted
+        // KDE5: remove userTextChanged signal, textEdited does the same...
+        if (!completionRunning && (txt != userText)) {
+            userText = txt;
+            emit q->userTextChanged(txt);
+        }
     }
 
+    // Call this when a completion operation changes the lineedit text
+    // "as if it had been edited by the user".
+    void _k_updateUserText(const QString &txt)
+    {
+        if (!completionRunning && (txt != userText)) {
+            userText = txt;
+            q->setModified(true);
+            emit q->userTextChanged(txt);
+            emit q->textEdited(txt);
+            emit q->textChanged(txt);
+        }
+    }
+
+    // This is called when the lineedit is readonly.
+    // Either from setReadOnly() itself, or when we realize that
+    // we became readonly and setReadOnly() wasn't called (because it's not virtual)
+    // Typical case: comboBox->lineEdit()->setReadOnly(true)
+    void adjustForReadOnly()
+    {
+        if (style && style.data()->m_overlap) {
+            style.data()->m_overlap = 0;
+        }
+    }
+
+
     /**
      * Checks whether we should/should not consume a key used as a shortcut.
      * This makes it possible to handle shortcuts in the focused widget before any
@@ -121,8 +145,8 @@ public:
      */
     bool overrideShortcut(const QKeyEvent* e);
 
-    static bool initialized;
-    static bool backspacePerformsCompletion; // Configuration option
+    static bool s_initialized;
+    static bool s_backspacePerformsCompletion; // Configuration option
 
     QColor previousHighlightColor;
     QColor previousHighlightedTextColor;
@@ -133,7 +157,7 @@ public:
     bool handleURLDrops:1;
     bool grabReturnKeyEvents:1;
     bool enableSqueezedText:1;
-    bool completionRunning:1; 
+    bool completionRunning:1;
 
     int squeezedEnd;
     int squeezedStart;
@@ -151,11 +175,11 @@ public:
     bool clickInClear:1;
     bool wideEnoughForClear:1;
     KLineEditButton *clearButton;
+    QWeakPointer<KLineEditStyle> style;
+    QString lastStyleClass;
 
     KCompletionBox *completionBox;
 
-    int overlap;
-
     bool italicizePlaceholder:1;
 
     QAction *noCompletionAction, *shellCompletionAction, *autoCompletionAction, *popupCompletionAction, *shortAutoCompletionAction, *popupAutoCompletionAction, *defaultAction;
@@ -164,36 +188,42 @@ public:
     KLineEdit* q;
 };
 
-// FIXME: Go back to using StyleSheets instead of a proxy style
-// once Qt has been fixed not to mess with widget font when
-// using StyleSheets
-class KLineEditStyle : public KdeUiProxyStyle
+QStyle *KLineEditStyle::style() const
 {
-public:
-  KLineEditStyle(KLineEdit *parent, KLineEditPrivate *lineEditPrivate)
-    : KdeUiProxyStyle(parent), lineEditPrivate(lineEditPrivate) {}
+    if (m_subStyle) {
+        return m_subStyle.data();
+    }
 
-  QRect subElementRect(SubElement element, const QStyleOption *option, const QWidget *widget) const;
-
-  KLineEditPrivate* lineEditPrivate;
-};
+    return KdeUiProxyStyle::style();
+}
 
 QRect KLineEditStyle::subElementRect(SubElement element, const QStyleOption *option, const QWidget *widget) const
 {
-  if (element == SE_LineEditContents)
-  {
-    QRect rect = style()->subElementRect(SE_LineEditContents, option, widget);
+  if (element == SE_LineEditContents) {
+      KLineEditStyle *unconstThis = const_cast<KLineEditStyle *>(this);
 
-    const int overlap = lineEditPrivate->overlap;
-    if (option->direction == Qt::LeftToRight) return rect.adjusted(0, 0, -overlap, 0);
-    else return rect.adjusted(overlap, 0, 0, 0);
+    if (m_sentinel) {
+        // we are recursing: we're wrapping a style that wraps us!
+        unconstThis->m_subStyle.clear();
+    }
+
+    unconstThis->m_sentinel = true;
+    QStyle *s = m_subStyle ? m_subStyle.data() : style();
+    QRect rect = s->subElementRect(SE_LineEditContents, option, widget);
+    unconstThis->m_sentinel = false;
+
+    if (option->direction == Qt::LeftToRight) {
+        return rect.adjusted(0, 0, -m_overlap, 0);
+    } else {
+        return rect.adjusted(m_overlap, 0, 0, 0);
+    }
   }
 
   return KdeUiProxyStyle::subElementRect(element, option, widget);
 }
 
-bool KLineEditPrivate::backspacePerformsCompletion = false;
-bool KLineEditPrivate::initialized = false;
+bool KLineEditPrivate::s_backspacePerformsCompletion = false;
+bool KLineEditPrivate::s_initialized = false;
 
 
 KLineEdit::KLineEdit( const QString &string, QWidget *parent )
@@ -237,11 +267,10 @@ void KLineEdit::init()
     if ( !d->previousHighlightColor.isValid() )
       d->previousHighlightColor=p.color(QPalette::Normal,QPalette::Highlight);
 
-    QStyle *lineEditStyle = new KLineEditStyle(this, d);
-    lineEditStyle->setParent(this);
-    setStyle(lineEditStyle);
-  
-    connect( this, SIGNAL(textChanged( const QString&)), this, SLOT(_k_updateUserText( const QString&)));
+    d->style = new KLineEditStyle(this);
+    setStyle(d->style.data());
+
+    connect(this, SIGNAL(textChanged(QString)), this, SLOT(_k_textChanged(QString)));
 
 }
 
@@ -269,7 +298,9 @@ void KLineEdit::setClearButtonShown(bool show)
         delete d->clearButton;
         d->clearButton = 0;
         d->clickInClear = false;
-        d->overlap = 0;
+        if (d->style) {
+            d->style.data()->m_overlap = 0;
+        }
     }
 }
 
@@ -289,9 +320,14 @@ QSize KLineEdit::clearButtonUsedSize() const
     return s;
 }
 
+// Decides whether to show or hide the icon; called when the text changes
 void KLineEdit::updateClearButtonIcon(const QString& text)
 {
-    if (!d->clearButton || isReadOnly()) {
+    if (!d->clearButton) {
+        return;
+    }
+    if (isReadOnly()) {
+        d->adjustForReadOnly();
         return;
     }
 
@@ -313,12 +349,17 @@ void KLineEdit::updateClearButtonIcon(const QString& text)
         d->clearButton->setPixmap(SmallIcon("edit-clear-locationbar-ltr", 0, clearButtonState));
     }
 
-    d->clearButton->setVisible(text.length());
+    d->clearButton->setVisible(text.length() > 0);
 }
 
+// Determine geometry of clear button. Called initially, and on resizeEvent.
 void KLineEdit::updateClearButton()
 {
-    if (!d->clearButton || isReadOnly()) {
+    if (!d->clearButton) {
+        return;
+    }
+    if (isReadOnly()) {
+        d->adjustForReadOnly();
         return;
     }
 
@@ -335,7 +376,10 @@ void KLineEdit::updateClearButton()
 
     if (newButtonSize != d->clearButton->size()) {
         d->clearButton->resize(newButtonSize);
-        d->overlap = wideEnough ? buttonWidth + frameWidth : 0;
+    }
+
+    if (d->style) {
+        d->style.data()->m_overlap = wideEnough ? buttonWidth + frameWidth : 0;
     }
 
     if (layoutDirection() == Qt::LeftToRight ) {
@@ -394,9 +438,9 @@ void KLineEdit::setCompletedText( const QString& t, bool marked )
 
     if ( t != txt )
     {
-        const int start = marked ? txt.length() : t.length();
         setText(t);
-        setSelection(start, t.length());
+        if ( marked )
+            setSelection(t.length(), txt.length()-t.length());
         setUserSelection(false);
     }
     else
@@ -493,7 +537,7 @@ void KLineEdit::setReadOnly(bool readOnly)
 
         if (d->clearButton) {
             d->clearButton->animateVisible(false);
-            d->overlap = 0;
+            d->adjustForReadOnly();
         }
     } else {
         if (!d->squeezedText.isEmpty()) {
@@ -771,15 +815,14 @@ void KLineEdit::keyPressEvent( QKeyEvent *e )
                 QLineEdit::keyPressEvent ( e );
                 const int cPosition=cursorPosition();
                 setText(old_txt);
-                setCursorPosition(cPosition);
-                if (e->key() ==Qt::Key_Right && cPosition > start )
-		{
-                    setSelection(cPosition, old_txt.length());
-		    //the user explicitly accepted the autocompletion
-		    d->_k_updateUserText(text());
-		}
-                else
-                    setSelection(start, old_txt.length());
+
+                // keep cursor at cPosition
+                setSelection(old_txt.length(), cPosition - old_txt.length());
+                if (e->key() == Qt::Key_Right && cPosition > start )
+                {
+                    //the user explicitly accepted the autocompletion
+                    d->_k_updateUserText(text());
+                }
 
                 d->disableRestoreSelection = false;
                 return;
@@ -844,8 +887,9 @@ void KLineEdit::keyPressEvent( QKeyEvent *e )
                             len = txt.length();
                         }
 
-                        if ( !d->backspacePerformsCompletion || !len )
+                        if (!d->s_backspacePerformsCompletion || !len) {
                             d->autoSuggest = false;
+                        }
                     }
 
                     if (e->key() == Qt::Key_Delete )
@@ -914,8 +958,9 @@ void KLineEdit::keyPressEvent( QKeyEvent *e )
                         len = txt.length();
                     }
 
-                    if ( !d->backspacePerformsCompletion )
+                    if (!d->s_backspacePerformsCompletion) {
                         d->autoSuggest = false;
+                    }
                 }
 
                 if (e->key() == Qt::Key_Delete )
@@ -1100,6 +1145,9 @@ void KLineEdit::tripleClickTimeout()
 QMenu* KLineEdit::createStandardContextMenu()
 {
     QMenu *popup = QLineEdit::createStandardContextMenu();
+    if ( !popup) { // happens on some embedded platforms
+        return 0;
+    }
 
     if( !isReadOnly() )
     {
@@ -1245,7 +1293,17 @@ void KLineEdit::dropEvent(QDropEvent *e)
         const KUrl::List urlList = KUrl::List::fromMimeData( e->mimeData() );
         if ( !urlList.isEmpty() )
         {
-            QString dropText = text();
+            // Let's replace the current text with the dropped URL(s), rather than appending.
+            // Makes more sense in general (#188129), e.g. konq location bar and kurlrequester
+            // can only hold one url anyway. OK this code supports multiple urls being dropped,
+            // but that's not the common case [and it breaks if they contain spaces... this is why
+            // kfiledialog uses double quotes around filenames in multiple-selection mode]...
+            //
+            // Anyway, if some apps prefer "append" then we should have a
+            // setUrlDropsSupport( {NoUrlDrops, SingleUrlDrops, MultipleUrlDrops} )
+            // where Single replaces and Multiple appends.
+            QString dropText;
+            //QString dropText = text();
             KUrl::List::ConstIterator it;
             for( it = urlList.begin() ; it != urlList.end() ; ++it )
             {
@@ -1274,15 +1332,12 @@ bool KLineEdit::event( QEvent* ev )
         if (d->overrideShortcut(e)) {
             ev->accept();
         }
-    }
-    else if( ev->type() == QEvent::KeyPress )
-    {
+    } else if( ev->type() == QEvent::KeyPress ) {
         // Hmm -- all this could be done in keyPressEvent too...
 
         QKeyEvent *e = static_cast<QKeyEvent *>( ev );
 
-        if( e->key() == Qt::Key_Return || e->key() == Qt::Key_Enter )
-        {
+        if (e->key() == Qt::Key_Return || e->key() == Qt::Key_Enter) {
             const bool trap = d->completionBox && d->completionBox->isVisible();
 
             const bool stopEvent = trap || (d->grabReturnKeyEvents &&
@@ -1290,26 +1345,55 @@ bool KLineEdit::event( QEvent* ev )
                                        e->modifiers() == Qt::KeypadModifier));
 
             // Qt will emit returnPressed() itself if we return false
-            if ( stopEvent )
-            {
+            if (stopEvent) {
                 emit QLineEdit::returnPressed();
                 e->accept();
             }
 
             emit returnPressed( displayText() );
 
-            if ( trap )
-            {
+            if (trap) {
                 d->completionBox->hide();
                 deselect();
                 setCursorPosition(text().length());
             }
 
             // Eat the event if the user asked for it, or if a completionbox was visible
-            if (stopEvent)
+            if (stopEvent) {
                 return true;
+            }
+        }
+    } else if (ev->type() == QEvent::ApplicationPaletteChange
+               || ev->type() == QEvent::PaletteChange) {
+        // Assume the widget uses the application's palette
+        QPalette p = QApplication::palette();
+        d->previousHighlightedTextColor=p.color(QPalette::Normal,QPalette::HighlightedText);
+        d->previousHighlightColor=p.color(QPalette::Normal,QPalette::Highlight);
+        setUserSelection(d->userSelection);
+    } else if (ev->type() == QEvent::StyleChange) {
+        // since we have our own style and it relies on this style to Get Things Right,
+        // if a style is set specifically on the widget (which would replace our own style!)
+        // hang on to this special style and re-instate our own style.
+        //FIXME: Qt currently has a grave bug where already deleted QStyleSheetStyle objects
+        // will get passed back in if we set a new style on it here. remove the qstrmcp test
+        // when this is fixed in Qt (or a better approach is found)
+        if (!qobject_cast<KLineEditStyle *>(style()) &&
+            qstrcmp(style()->metaObject()->className(), "QStyleSheetStyle") != 0 &&
+            QLatin1String(style()->metaObject()->className()) != d->lastStyleClass) {
+            KLineEditStyle *kleStyle = d->style.data();
+            if (!kleStyle) {
+                d->style = kleStyle = new KLineEditStyle(this);
+            }
+
+            kleStyle->m_subStyle = style();
+            // this guards against "wrap around" where another style, e.g. QStyleSheetStyle,
+            // is setting the style on QEvent::StyleChange
+            d->lastStyleClass = QLatin1String(style()->metaObject()->className());
+            setStyle(kleStyle);
+            d->lastStyleClass.clear();
         }
     }
+
     return QLineEdit::event( ev );
 }
 
@@ -1348,13 +1432,11 @@ void KLineEdit::setCompletionBox( KCompletionBox *box )
     if ( handleSignals() )
     {
         connect( d->completionBox, SIGNAL(currentTextChanged( const QString& )),
-                 SLOT(setTextWorkaround( const QString& )) );
+                 SLOT(_k_slotCompletionBoxTextChanged( const QString& )) );
         connect( d->completionBox, SIGNAL(userCancelled( const QString& )),
                  SLOT(userCancelled( const QString& )) );
-
-        // TODO: we need our own slot, and to call setModified(true) if Qt4 has that.
-        connect( d->completionBox, SIGNAL( activated( const QString& )),
-                 SIGNAL(completionBoxActivated( const QString& )) );
+        connect( d->completionBox, SIGNAL(activated(QString)),
+                 SIGNAL(completionBoxActivated(QString)) );
     }
 }
 
@@ -1482,33 +1564,22 @@ void KLineEdit::setCompletedItems( const QStringList& items, bool autoSuggest )
         {
             QListWidgetItem* currentItem = d->completionBox->currentItem();
 
-            bool wasSelected = false;
             QString currentSelection;
-
             if ( currentItem != 0 ) {
-                wasSelected = currentItem->isSelected();
                 currentSelection = currentItem->text();
             }
 
             d->completionBox->setItems( items );
 
-            const QList<QListWidgetItem*> matchedItems = d->completionBox->findItems( currentSelection , Qt::MatchExactly);
+            const QList<QListWidgetItem*> matchedItems = d->completionBox->findItems(currentSelection, Qt::MatchExactly);
             QListWidgetItem* matchedItem = matchedItems.isEmpty() ? 0 : matchedItems.first();
 
-            // If no item is selected, that means the listbox hasn't been manipulated by the user yet,
-            // because it's not possible otherwise to have no selected item. In such case make
-            // always the first item current and unselected, so that the current item doesn't jump.
-            if( !matchedItem || !wasSelected )
-            {
-                wasSelected = false;
-                matchedItem = d->completionBox->item( 0 );
-            }
-            if ( matchedItem )
-            {
+            if (matchedItem) {
                 const bool blocked = d->completionBox->blockSignals( true );
                 d->completionBox->setCurrentItem( matchedItem );
-                matchedItem->setSelected(wasSelected);
                 d->completionBox->blockSignals( blocked );
+            } else {
+                d->completionBox->setCurrentRow(-1);
             }
         }
         else // completion box not visible yet -> show it
@@ -1523,7 +1594,7 @@ void KLineEdit::setCompletedItems( const QStringList& items, bool autoSuggest )
         {
             const int index = items.first().indexOf( txt );
             const QString newText = items.first().mid( index );
-            setUserSelection(false);
+            setUserSelection(false); // can be removed? setCompletedText sets it anyway
             setCompletedText(newText,true);
         }
     }
@@ -1568,14 +1639,14 @@ void KLineEdit::create( WId id, bool initializeWindow, bool destroyOldWindow )
 
 void KLineEdit::setUserSelection(bool userSelection)
 {
-    //if !d->userSelection && userSelection we are accepting a completion, 
-    //so trigger an update 
-  
-    if (!d->userSelection && userSelection) 
-    { 
+    //if !d->userSelection && userSelection we are accepting a completion,
+    //so trigger an update
+
+    if (!d->userSelection && userSelection)
+    {
 	d->_k_updateUserText(text());
     }
-  
+
     QPalette p = palette();
 
     if (userSelection)
@@ -1608,11 +1679,13 @@ void KLineEdit::clear()
     setText( QString() );
 }
 
-void KLineEdit::setTextWorkaround( const QString& text )
+void KLineEdit::_k_slotCompletionBoxTextChanged( const QString& text )
 {
     if (!text.isEmpty())
     {
         setText( text );
+        setModified(true);
+        emit textEdited(text);
         end( false ); // force cursor at end
     }
 }
@@ -1638,6 +1711,15 @@ bool KLineEdit::autoSuggest() const
 void KLineEdit::paintEvent( QPaintEvent *ev )
 {
     if (echoMode() == Password && d->threeStars) {
+        // ### hack alert!
+        // QLineEdit has currently no hooks to modify the displayed string.
+        // When we call setText(), an update() is triggered and we get
+        // into an infinite recursion.
+        // Qt offers the setUpdatesEnabled() method, but when we re-enable
+        // them, update() is triggered, and we get into the same recursion.
+        // To work around this problem, we set/clear the internal Qt flag which
+        // marks the updatesDisabled state manually.
+        setAttribute(Qt::WA_UpdatesDisabled, true);
         blockSignals(true);
         const QString oldText = text();
         const bool isModifiedState = isModified(); // save modified state because setText resets it
@@ -1646,31 +1728,57 @@ void KLineEdit::paintEvent( QPaintEvent *ev )
         setText(oldText);
         setModified(isModifiedState);
         blockSignals(false);
-        return;
+        setAttribute(Qt::WA_UpdatesDisabled, false);
+    } else {
+        QLineEdit::paintEvent( ev );
     }
 
-    QLineEdit::paintEvent( ev );
-
     if (d->enableClickMsg && d->drawClickMsg && !hasFocus() && text().isEmpty()) {
         QPainter p(this);
         QFont f = font();
         f.setItalic(d->italicizePlaceholder);
         p.setFont(f);
 
-        p.setPen(palette().color(QPalette::Disabled, QPalette::Text));
+        QColor color(palette().color(foregroundRole()));
+        color.setAlphaF(0.5);
+        p.setPen(color);
 
-        //FIXME: fugly alert!
-        // qlineedit uses an internal qstyleoption set to figure this out
-        // and then adds a hardcoded 2 pixel interior to that.
-        // probably requires fixes to Qt itself to do this cleanly
-        // see define horizontalMargin 2 in qlineedit.cpp
         QStyleOptionFrame opt;
         initStyleOption(&opt);
         QRect cr = style()->subElementRect(QStyle::SE_LineEditContents, &opt, this);
-        cr.setLeft(cr.left() + 2);
-        cr.setRight(cr.right() - 2);
 
-        p.drawText(cr, Qt::AlignLeft|Qt::AlignVCenter, d->clickMessage);
+        // this is copied/adapted from QLineEdit::paintEvent
+        const int verticalMargin(1);
+        const int horizontalMargin(2);
+
+        int left, top, right, bottom;
+        getTextMargins( &left, &top, &right, &bottom );
+        cr.adjust( left, top, -right, -bottom );
+
+        p.setClipRect(cr);
+
+        QFontMetrics fm = fontMetrics();
+        Qt::Alignment va = alignment() & Qt::AlignVertical_Mask;
+        int vscroll;
+        switch (va & Qt::AlignVertical_Mask)
+        {
+            case Qt::AlignBottom:
+            vscroll = cr.y() + cr.height() - fm.height() - verticalMargin;
+            break;
+
+            case Qt::AlignTop:
+            vscroll = cr.y() + verticalMargin;
+            break;
+
+            default:
+            vscroll = cr.y() + (cr.height() - fm.height() + 1) / 2;
+            break;
+
+        }
+
+        QRect lineRect(cr.x() + horizontalMargin, vscroll, cr.width() - 2*horizontalMargin, fm.height());
+        p.drawText(lineRect, Qt::AlignLeft|Qt::AlignVCenter, d->clickMessage);
+
     }
 }
 
@@ -1696,7 +1804,7 @@ void KLineEdit::focusOutEvent( QFocusEvent *ev )
 
 void KLineEdit::setClickMessage( const QString &msg )
 {
-    d->enableClickMsg = true;
+    d->enableClickMsg = !msg.isEmpty();
     d->clickMessage = msg;
     d->drawClickMsg = text().isEmpty();
     update();
