@@ -146,12 +146,11 @@ KMultiTabBarButton::KMultiTabBarButton(const QPixmap& pic, const QString& text,
 	: QPushButton(QIcon(pic), text, parent), m_id(id), d(0)
 {
 	connect(this,SIGNAL(clicked()),this,SLOT(slotClicked()));
-	setFlat(true);
 
-        // we can't see the focus, so don't take focus. #45557
-        // If keyboard navigation is wanted, then only the bar should take focus,
-        // and arrows could change the focused button; but generally, tabbars don't take focus anyway.
-        setFocusPolicy(Qt::NoFocus);
+	// we can't see the focus, so don't take focus. #45557
+	// If keyboard navigation is wanted, then only the bar should take focus,
+	// and arrows could change the focused button; but generally, tabbars don't take focus anyway.
+	setFocusPolicy(Qt::NoFocus);
 }
 
 KMultiTabBarButton::~KMultiTabBarButton()
@@ -186,6 +185,17 @@ void KMultiTabBarButton::showEvent( QShowEvent* he) {
 	if (tb) tb->updateSeparator();
 }
 
+void KMultiTabBarButton::paintEvent(QPaintEvent *) {
+        QStyleOptionButton opt;
+	opt.initFrom(this);
+	opt.icon = icon();
+	opt.iconSize = iconSize();
+	// removes the QStyleOptionButton::HasMenu ButtonFeature
+	opt.features = QStyleOptionButton::Flat;
+	QPainter painter(this);
+	style()->drawControl(QStyle::CE_PushButton, &opt, &painter, this);
+}
+
 // KMultiTabBarTab
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -489,6 +499,8 @@ KMultiTabBar::~KMultiTabBar()
 int KMultiTabBar::appendButton(const QPixmap &pic, int id, QMenu *popup, const QString&)
 {
 	KMultiTabBarButton *btn = new KMultiTabBarButton(pic, QString(), id, this);
+	// a button with a QMenu can have another size. Make sure the button has always the same size.
+	btn->setFixedWidth(btn->height());
 	btn->setMenu(popup);
 	d->m_buttons.append(btn);
 	d->m_l->insertWidget(0,btn);
