@@ -27,6 +27,8 @@
 #include <limits.h>
 #endif
 
+#include <cmath>
+
 #include <QtGui/QApplication>
 #include <QtGui/QLabel>
 #include <QtGui/QLineEdit>
@@ -35,6 +37,7 @@
 
 #include <kdebug.h>
 #include <kdialog.h>
+#include <klocalizedstring.h>
 
 static inline int calcDiffByTen(int x, int y)
 {
@@ -89,12 +92,14 @@ KNumInput::KNumInput(QWidget* parent)
     : QWidget(parent), d(new KNumInputPrivate(this))
 {
     setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed));
+    setFocusPolicy(Qt::StrongFocus);
 }
 
 KNumInput::KNumInput(QWidget* parent, KNumInput* below)
     : QWidget(parent), d(new KNumInputPrivate(this, below))
 {
     setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed));
+    setFocusPolicy(Qt::StrongFocus);
 }
 
 KNumInput::~KNumInput()
@@ -173,7 +178,9 @@ void KNumInput::layout(bool deep)
         return;
     }
 
-    KNumInput* p = this;
+    w2 = d->column2Width;
+
+    KNumInput* p = d->previousNumInput;
     while (p) {
         p->doLayout();
         w1 = qMax(w1, p->d->column1Width);
@@ -225,16 +232,27 @@ void KNumInput::setSteps(int minor, int major)
 class KIntSpinBox::KIntSpinBoxPrivate
 {
 public:
-    KIntSpinBoxPrivate(KIntSpinBox *q, int val_base = 10): q(q), val_base(val_base) {}
+    KIntSpinBoxPrivate(KIntSpinBox *q, int val_base = 10): q(q), val_base(val_base)
+    {
+        connect(q, SIGNAL(valueChanged(int)), q, SLOT(updateSuffix(int)));
+    }
+
+    void updateSuffix(int value)
+    {
+        if (!pluralSuffix.isEmpty()) {
+            KLocalizedString s = pluralSuffix;
+            q->setSuffix(s.subs(value).toString());
+        }
+    }
 
     KIntSpinBox *q;
     int val_base;
+    KLocalizedString pluralSuffix;
 };
 
 KIntSpinBox::KIntSpinBox(QWidget *parent)
     : QSpinBox(parent), d(new KIntSpinBoxPrivate(this))
 {
-    lineEdit()->setAlignment(Qt::AlignRight);
     setValue(0);
 }
 
@@ -243,12 +261,11 @@ KIntSpinBox::~KIntSpinBox()
     delete d;
 }
 
-KIntSpinBox::KIntSpinBox(int lower, int upper, int step, int value, QWidget *parent, int base)
+KIntSpinBox::KIntSpinBox(int lower, int upper, int singleStep, int value, QWidget *parent, int base)
     : QSpinBox(parent), d(new KIntSpinBoxPrivate(this, base))
 {
     setRange(lower, upper);
-    setSingleStep(step);
-    lineEdit()->setAlignment(Qt::AlignRight);
+    setSingleStep(singleStep);
     setValue(value);
 }
 
@@ -286,6 +303,14 @@ void KIntSpinBox::setEditFocus(bool mark)
     }
 }
 
+void KIntSpinBox::setSuffix(const KLocalizedString& suffix)
+{
+    d->pluralSuffix = suffix;
+    if (suffix.isEmpty())
+        setSuffix(QString());
+    else
+        d->updateSuffix(value());
+}
 
 // ----------------------------------------------------------------------------
 
@@ -392,6 +417,7 @@ void KIntNumInput::setSliderEnabled(bool slider)
             connect(priv->slider, SIGNAL(valueChanged(int)),
                     d->intSpinBox, SLOT(setValue(int)));
             priv->slider->setTickPosition(QSlider::TicksBelow);
+            layout(true);
         }
 
         const int value = d->intSpinBox->value();
@@ -403,23 +429,29 @@ void KIntNumInput::setSliderEnabled(bool slider)
         const int major = calcDiffByTen(d->intSpinBox->maximum(), d->intSpinBox->minimum());
 
         priv->slider->setSingleStep(d->intSpinBox->singleStep());
-        priv->slider->setPageStep(major);
+        priv->slider->setPageStep(qMax(1, major));
         priv->slider->setTickInterval(major);
     } else {
+        if (priv->slider) {
+            layout(true);
+        }
         delete priv->slider;
         priv->slider = 0;
     }
 }
 
-void KIntNumInput::setRange(int lower, int upper, int step)
+void KIntNumInput::setRange(int lower, int upper, int singleStep)
 {
-    upper = qMax(upper, lower);
-    lower = qMin(upper, lower);
+    if (upper < lower || singleStep <= 0) {
+        kWarning() << "WARNING: KIntNumInput::setRange() called with bad arguments. Ignoring call...";
+        return;
+    }
+
     d->intSpinBox->setMinimum(lower);
     d->intSpinBox->setMaximum(upper);
-    d->intSpinBox->setSingleStep(step);
+    d->intSpinBox->setSingleStep(singleStep);
 
-    step = d->intSpinBox->singleStep(); // maybe QRangeControl didn't like out lineStep?
+    singleStep = d->intSpinBox->singleStep(); // maybe QRangeControl didn't like our lineStep?
 
     // check that reference point is still inside valid range:
     setReferencePoint(referencePoint());
@@ -431,9 +463,9 @@ void KIntNumInput::setRange(int lower, int upper, int step)
     setSliderEnabled(priv->slider);
 }
 
-void KIntNumInput::setRange(int lower, int upper, int step, bool slider)
+void KIntNumInput::setRange(int lower, int upper, int singleStep, bool slider)
 {
-    setRange(lower, upper, step);
+    setRange(lower, upper, singleStep);
     setSliderEnabled(slider);
 }
 
@@ -457,6 +489,16 @@ int KIntNumInput::maximum() const
     return d->intSpinBox->maximum();
 }
 
+int KIntNumInput::singleStep() const
+{
+    return d->intSpinBox->singleStep();
+}
+
+void KIntNumInput::setSingleStep(int singleStep)
+{
+    d->intSpinBox->setSingleStep(singleStep);
+}
+
 void KIntNumInput::setSuffix(const QString &suffix)
 {
     d->intSpinBox->setSuffix(suffix);
@@ -464,6 +506,12 @@ void KIntNumInput::setSuffix(const QString &suffix)
     layout(true);
 }
 
+void KIntNumInput::setSuffix(const KLocalizedString& suffix)
+{
+    d->intSpinBox->setSuffix(suffix);
+    layout(true);
+}
+
 QString KIntNumInput::suffix() const
 {
     return d->intSpinBox->suffix();
@@ -494,7 +542,7 @@ QSize KIntNumInput::minimumSizeHint() const
     int w;
     int h;
 
-    h = 2 + qMax(d->intSpinBoxSize.height(), priv->sliderSize.height());
+    h = qMax(d->intSpinBoxSize.height(), priv->sliderSize.height());
 
     // if in extra row, then count it here
     if (priv->label && (priv->labelAlignment & (Qt::AlignBottom | Qt::AlignTop))) {
@@ -627,12 +675,14 @@ public:
     KDoubleNumInputPrivate(double r)
             : spin(0),
             referencePoint(r),
-            blockRelative(0) {}
+            blockRelative(0),
+            exponentRatio(1.0) {}
     QDoubleSpinBox * spin;
     double referencePoint;
     short blockRelative;
     QSize editSize;
     QString specialValue;
+    double exponentRatio;
 };
 
 KDoubleNumInput::KDoubleNumInput(QWidget *parent)
@@ -644,20 +694,20 @@ KDoubleNumInput::KDoubleNumInput(QWidget *parent)
 }
 
 KDoubleNumInput::KDoubleNumInput(double lower, double upper, double value, QWidget *parent,
-                                 double step, int precision)
+                                 double singleStep, int precision)
     : KNumInput(parent)
     , d(new KDoubleNumInputPrivate(value))
 {
-    init(value, lower, upper, step, precision);
+    init(value, lower, upper, singleStep, precision);
 }
 
 KDoubleNumInput::KDoubleNumInput(KNumInput *below,
                                  double lower, double upper, double value, QWidget *parent,
-                                 double step, int precision)
+                                 double singleStep, int precision)
     : KNumInput(parent, below)
     , d(new KDoubleNumInputPrivate(value))
 {
-    init(value, lower, upper, step, precision);
+    init(value, lower, upper, singleStep, precision);
 }
 
 KDoubleNumInput::~KDoubleNumInput()
@@ -672,14 +722,13 @@ QString KDoubleNumInput::specialValueText() const
 
 
 void KDoubleNumInput::init(double value, double lower, double upper,
-                           double step, int precision)
+                           double singleStep, int precision)
 {
     d->spin = new QDoubleSpinBox(this);
     d->spin->setRange(lower, upper);
-    d->spin->setSingleStep(step);
+    d->spin->setSingleStep(singleStep);
     d->spin->setValue(value);
     d->spin->setDecimals(precision);
-    d->spin->setAlignment(Qt::AlignRight);
 
     d->spin->setObjectName("KDoubleNumInput::QDoubleSpinBox");
     setFocusProxy(d->spin);
@@ -708,7 +757,8 @@ double KDoubleNumInput::mapSliderToSpin(int val) const
     const double slidemin = priv->slider->minimum(); // cast int to double to avoid
     const double slidemax = priv->slider->maximum(); // overflow in rel denominator
     const double rel = (double(val) - slidemin) / (slidemax - slidemin);
-    return spinmin + rel * (spinmax - spinmin);
+    Q_ASSERT(d->exponentRatio > 0.0);
+    return spinmin + pow(rel, d->exponentRatio ) * (spinmax - spinmin);
 }
 
 void KDoubleNumInput::sliderMoved(int val)
@@ -725,7 +775,8 @@ void KDoubleNumInput::spinBoxChanged(double val)
     const double slidemin = priv->slider->minimum(); // cast int to double to avoid
     const double slidemax = priv->slider->maximum(); // overflow in rel denominator
 
-    const double rel = (val - spinmin) / (spinmax - spinmin);
+    Q_ASSERT(d->exponentRatio > 0.0);
+    const double rel = pow((val - spinmin) / (spinmax - spinmin) , 1.0 / d->exponentRatio);
 
     if (priv->slider) {
         priv->slider->blockSignals(true);
@@ -751,7 +802,7 @@ QSize KDoubleNumInput::minimumSizeHint() const
     int w;
     int h;
 
-    h = 2 + qMax(d->editSize.height(), priv->sliderSize.height());
+    h = qMax(d->editSize.height(), priv->sliderSize.height());
 
     // if in extra row, then count it here
     if (priv->label && (priv->labelAlignment & (Qt::AlignBottom | Qt::AlignTop))) {
@@ -841,7 +892,7 @@ void KDoubleNumInput::setReferencePoint(double ref)
     d->referencePoint = ref;
 }
 
-void KDoubleNumInput::setRange(double lower, double upper, double step,
+void KDoubleNumInput::setRange(double lower, double upper, double singleStep,
                                bool slider)
 {
     K_USING_KNUMINPUT_P(priv);
@@ -853,40 +904,9 @@ void KDoubleNumInput::setRange(double lower, double upper, double step,
                    priv->slider, SLOT(setValue(int)));
     }
     d->spin->setRange(lower, upper);
-    d->spin->setSingleStep(step);
+    d->spin->setSingleStep(singleStep);
 
-    if (slider) {
-        // upcast to base type to get the minimum/maximum in int form:
-        QDoubleSpinBox * spin = d->spin;
-        double multiplicator = 1.0;
-        if (spin->maximum() - spin->minimum() < 10.0) { // if the range is to small, the slider would not be usable (see #168022)
-            multiplicator = 10 / (spin->maximum() - spin->minimum());
-        }
-        const int slmax = qRound(spin->maximum() * multiplicator);
-        const int slmin = qRound(spin->minimum() * multiplicator);
-        const int slvalue = qRound(spin->value() * multiplicator);
-        const int slstep = qRound(spin->singleStep() * multiplicator);
-        if (!priv->slider) {
-            priv->slider = new QSlider(Qt::Horizontal, this);
-            priv->slider->setTickPosition(QSlider::TicksBelow);
-            // feedback line: when one moves, the other moves, too:
-            connect(priv->slider, SIGNAL(valueChanged(int)),
-                    SLOT(sliderMoved(int)));
-        }
-        priv->slider->setRange(slmin, slmax);
-        priv->slider->setSingleStep(slstep);
-        priv->slider->setValue(slvalue);
-        connect(spin, SIGNAL(valueChanged(double)), SLOT(spinBoxChanged(double)));
-        // calculate ( slmax - slmin ) / 10 without overflowing ints:
-        int major = calcDiffByTen(slmax, slmin);
-        if (!major) {
-            major = slstep;   // ### needed?
-        }
-        priv->slider->setTickInterval(major);
-    } else {
-        delete priv->slider;
-        priv->slider = 0;
-    }
+    setSliderEnabled(slider);
 
     setReferencePoint(referencePoint());
 
@@ -894,6 +914,44 @@ void KDoubleNumInput::setRange(double lower, double upper, double step,
     updateLegacyMembers();
 }
 
+void KDoubleNumInput::setSliderEnabled(bool enabled)
+{
+    K_USING_KNUMINPUT_P(priv);
+    if (enabled) {
+        QDoubleSpinBox * spin = d->spin;
+        const double range = spin->maximum() - spin->minimum();
+        const double steps = range * pow(10.0, spin->decimals());
+        if (!priv->slider) {
+            priv->slider = new QSlider(Qt::Horizontal, this);
+            priv->slider->setTickPosition(QSlider::TicksBelow);
+            // feedback line: when one moves, the other moves, too:
+            connect(priv->slider, SIGNAL(valueChanged(int)),
+                    SLOT(sliderMoved(int)));
+            layout(true);
+        }
+        if (steps > 1000 || d->exponentRatio != 1.0) {
+            priv->slider->setRange(0, 1000);
+            priv->slider->setSingleStep(1);
+            priv->slider->setPageStep(50);
+        } else {
+            const int singleSteps = qRound(steps);
+            priv->slider->setRange(0, singleSteps);
+            priv->slider->setSingleStep(1);
+            const int pageSteps = qBound(1, singleSteps / 20, 10);
+            priv->slider->setPageStep(pageSteps);
+        }
+        spinBoxChanged(spin->value());
+        connect(spin, SIGNAL(valueChanged(double)), SLOT(spinBoxChanged(double)));
+    } else {
+        if (priv->slider) {
+            layout(true);
+        }
+        delete priv->slider;
+        priv->slider = 0;
+    }
+}
+
+
 void KDoubleNumInput::setMinimum(double min)
 {
     K_USING_KNUMINPUT_P(priv);
@@ -916,6 +974,16 @@ double KDoubleNumInput::maximum() const
     return d->spin->maximum();
 }
 
+double KDoubleNumInput::singleStep() const
+{
+  return d->spin->singleStep();
+}
+
+void KDoubleNumInput::setSingleStep(double singleStep)
+{
+  d->spin->setSingleStep(singleStep);
+}
+
 double KDoubleNumInput::value() const
 {
     return d->spin->value();
@@ -989,4 +1057,21 @@ void KDoubleNumInput::setLabel(const QString & label, Qt::Alignment a)
     }
 }
 
+double KDoubleNumInput::exponentRatio() const
+{
+    return d->exponentRatio;
+}
+
+void KDoubleNumInput::setExponentRatio(double dbl)
+{
+    Q_ASSERT(dbl > 0.0);
+    if(dbl > 0.0) {
+        d->exponentRatio = dbl;
+        spinBoxChanged( d->spin->value() ); // used to reset the value of the slider
+    } else {
+        kError() << "ExponentRatio need to be strictly positive.";
+    }
+}
+
+
 #include "knuminput.moc"
