@@ -1,6 +1,7 @@
 /* This file is part of the KDE libraries
   Copyright (C) 2003 Stephan Binner <binner@kde.org>
   Copyright (C) 2003 Zack Rusin <zack@kde.org>
+  Copyright (C) 2009 Urs Wolfer <uwolfer @ kde.org>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
@@ -33,6 +34,7 @@
 #include <ksharedconfig.h>
 #include <kiconloader.h>
 #include <kstringhandler.h>
+#include <kdebug.h>
 
 #include <ktabbar.h>
 
@@ -41,29 +43,36 @@
 class KTabWidget::Private
 {
   public:
+    enum {
+        ResizeEnabled = 0,
+        ResizeDisabled,
+        ResizeLater
+    } m_resizeSuspend;
+
     Private( KTabWidget *parent )
-      : m_parent( parent ),
-        m_automaticResizeTabs( false )
+      : m_resizeSuspend(ResizeEnabled),
+        m_parent( parent ),
+        m_automaticResizeTabs( false ),
+        m_tabBarHidden( false )
     {
 
       KConfigGroup cg(KGlobal::config(), "General");
       m_maxLength = cg.readEntry("MaximumTabLength", 30);
       m_minLength = cg.readEntry("MinimumTabLength", 3);
-      m_currentMaxLength = m_minLength;
+      Q_ASSERT(m_maxLength >= m_minLength);
+      m_currentTabLength = m_minLength;
     }
 
     KTabWidget *m_parent;
     bool m_automaticResizeTabs;
+    bool m_tabBarHidden;
     int m_maxLength;
     int m_minLength;
-    int m_currentMaxLength;
+    int m_currentTabLength;
 
     //holds the full names of the tab, otherwise all we
     //know about is the shortened name
     QStringList m_tabNames;
-    
-    // Used when tabCloseActivatePrevious is enabled
-    QList<QWidget*> m_previousTabList;
 
     bool isEmptyTabbarSpace( const QPoint & )  const;
     void resizeTabs( int changedTabIndex = -1 );
@@ -107,54 +116,83 @@ bool KTabWidget::Private::isEmptyTabbarSpace( const QPoint &point ) const
 
 void KTabWidget::Private::removeTab( int index )
 {
-  m_previousTabList.removeOne( m_parent->widget( index ) );
-  
-  // We need to get the widget for "activate previous" before calling to 
-  // QTabWidget::removeTab() because that call changes currentIndex which calls
-  // to currentChanged() and thus modifies m_previousTabList.first(). And we
-  // store the widget and not the index of it because the index might be changed
-  // by the removeTab call.
-  QWidget *widget = 0;
-  if( !m_previousTabList.isEmpty() && m_parent->tabCloseActivatePrevious() )
-    widget = m_previousTabList.first();
-  
+  // prevent cascading resize slowness, not to mention crashes due to tab count()
+  // and m_tabNames.count() being out of sync!
+  m_resizeSuspend = ResizeDisabled;
+
+  // Need to do this here, rather than in tabRemoved().  Calling
+  // QTabWidget::removeTab() below may cause a relayout of the tab bar, which
+  // will call resizeTabs() immediately.  If m_automaticResizeTabs is true,
+  // that will use the m_tabNames[] list before it has been updated to reflect
+  // the new tab arrangement.  See bug 190528.
+  m_tabNames.removeAt( index );
+
   m_parent->QTabWidget::removeTab( index );
-  
-  if( widget )
-    m_parent->setCurrentIndex( m_parent->indexOf( widget ) );
+
+  const bool doResize = (m_resizeSuspend == ResizeLater) || m_automaticResizeTabs;
+  m_resizeSuspend = ResizeEnabled;
+  if (doResize) {
+    resizeTabs();
+  }
+
 }
 
 void KTabWidget::Private::resizeTabs( int changeTabIndex )
 {
-  int newMaxLength;
-  if ( m_automaticResizeTabs ) {
-    // Calculate new max length
-    newMaxLength = m_maxLength;
-    int lcw = 0, rcw = 0;
-
-    int tabBarHeight = m_parent->tabBar()->sizeHint().height();
-    if ( m_parent->cornerWidget( Qt::TopLeftCorner ) && m_parent->cornerWidget( Qt::TopLeftCorner )->isVisible() )
-      lcw = qMax( m_parent->cornerWidget( Qt::TopLeftCorner )->width(), tabBarHeight );
-
-    if ( m_parent->cornerWidget( Qt::TopRightCorner ) && m_parent->cornerWidget( Qt::TopRightCorner )->isVisible() )
-      rcw = qMax( m_parent->cornerWidget( Qt::TopRightCorner )->width(), tabBarHeight );
-
-    int maxTabBarWidth = m_parent->width() - lcw - rcw;
-
-    for ( ; newMaxLength > m_minLength; newMaxLength-- ) {
-      if ( m_parent->tabBarWidthForMaxChars( newMaxLength ) < maxTabBarWidth )
-        break;
+    if (m_resizeSuspend != ResizeEnabled) {
+        m_resizeSuspend = ResizeLater;
+        return;
     }
-  } else
-    newMaxLength = 4711;
 
-  // Update hinted or all tabs
-  if ( m_currentMaxLength != newMaxLength ) {
-    m_currentMaxLength = newMaxLength;
-    for ( int i = 0; i < m_parent->count(); ++i )
-      updateTab( i );
-  } else if ( changeTabIndex != -1 )
-    updateTab( changeTabIndex );
+    int newTabLength = m_maxLength;
+
+    if (m_automaticResizeTabs) {
+        // Calculate new max length
+        int lcw = 0, rcw = 0;
+
+        const int tabBarHeight = m_parent->tabBar()->sizeHint().height();
+        if (m_parent->cornerWidget(Qt::TopLeftCorner) &&
+            m_parent->cornerWidget( Qt::TopLeftCorner )->isVisible()) {
+            lcw = qMax(m_parent->cornerWidget(Qt::TopLeftCorner)->width(), tabBarHeight);
+        }
+        if (m_parent->cornerWidget(Qt::TopRightCorner) &&
+            m_parent->cornerWidget(Qt::TopRightCorner)->isVisible()) {
+            rcw = qMax( m_parent->cornerWidget(Qt::TopRightCorner)->width(), tabBarHeight);
+        }
+
+        const int maxTabBarWidth = m_parent->width() - lcw - rcw;
+
+        // binary search for the best fitting tab title length; some wiggling was
+        // required to make this behave in the face of rounding.
+        int newTabLengthHi = m_maxLength + 1;
+        int newTabLengthLo = m_minLength;
+        int prevTabLengthMid = -1;
+        while (true) {
+            int newTabLengthMid = (newTabLengthHi + newTabLengthLo) / 2;
+            if (prevTabLengthMid == newTabLengthMid) {
+                // no change, we're stuck due to rounding.
+                break;
+            }
+            prevTabLengthMid = newTabLengthMid;
+
+            if (m_parent->tabBarWidthForMaxChars(newTabLengthMid) > maxTabBarWidth) {
+                newTabLengthHi = newTabLengthMid;
+            } else {
+                newTabLengthLo = newTabLengthMid;
+            }
+        }
+        newTabLength = qMin(newTabLengthLo, m_maxLength);
+    }
+
+    // Update hinted or all tabs
+    if (m_currentTabLength != newTabLength) {
+        m_currentTabLength = newTabLength;
+        for (int i = 0; i < m_parent->count(); i++) {
+            updateTab(i);
+        }
+    } else if (changeTabIndex != -1) {
+        updateTab(changeTabIndex);
+    }
 }
 
 void KTabWidget::Private::updateTab( int index )
@@ -162,14 +200,19 @@ void KTabWidget::Private::updateTab( int index )
   QString title = m_automaticResizeTabs ? m_tabNames[ index ] : m_parent->QTabWidget::tabText( index );
   m_parent->setTabToolTip( index, QString() );
 
-  if ( title.length() > m_currentMaxLength ) {
-    if ( Qt::mightBeRichText( title ) )
-      m_parent->setTabToolTip( index, Qt::escape( title ) );
+  if ( title.length() > m_currentTabLength ) {
+    QString toolTipText = title;
+    // Remove '&'s, which are indicators for keyboard shortcuts in tab titles. "&&" is replaced by '&'.
+    for ( int i = toolTipText.indexOf( '&' ); i >= 0 && i < toolTipText.length(); i = toolTipText.indexOf( '&', i + 1 ) )
+      toolTipText.remove( i, 1 );
+
+    if ( Qt::mightBeRichText( toolTipText ) )
+      m_parent->setTabToolTip( index, Qt::escape( toolTipText ) );
     else
-      m_parent->setTabToolTip( index, title );
+      m_parent->setTabToolTip( index, toolTipText );
   }
 
-  title = KStringHandler::rsqueeze( title, m_currentMaxLength ).leftJustified( m_minLength, ' ' );
+  title = KStringHandler::rsqueeze( title, m_currentTabLength ).leftJustified( m_minLength, ' ' );
 
   if ( m_parent->QTabWidget::tabText( index ) != title )
     m_parent->QTabWidget::setTabText( index, title );
@@ -186,13 +229,13 @@ KTabWidget::KTabWidget( QWidget *parent, Qt::WFlags flags )
 
   connect(tabBar(), SIGNAL(contextMenu( int, const QPoint & )), SLOT(contextMenu( int, const QPoint & )));
   connect(tabBar(), SIGNAL(tabDoubleClicked( int )), SLOT(mouseDoubleClick( int )));
+  connect(tabBar(), SIGNAL(newTabRequest()), this, SIGNAL(mouseDoubleClick())); // #185487
   connect(tabBar(), SIGNAL(mouseMiddleClick( int )), SLOT(mouseMiddleClick( int )));
   connect(tabBar(), SIGNAL(initiateDrag( int )), SLOT(initiateDrag( int )));
   connect(tabBar(), SIGNAL(testCanDecode(const QDragMoveEvent *, bool & )), SIGNAL(testCanDecode(const QDragMoveEvent *, bool & )));
   connect(tabBar(), SIGNAL(receivedDropEvent( int, QDropEvent * )), SLOT(receivedDropEvent( int, QDropEvent * )));
   connect(tabBar(), SIGNAL(moveTab( int, int )), SLOT(moveTab( int, int )));
-  connect(tabBar(), SIGNAL(closeRequest( int )), SLOT(closeRequest( int )));
-  connect(tabBar(), SIGNAL(currentChanged( int )), SLOT(currentChanged( int )));
+  connect(tabBar(), SIGNAL(tabCloseRequested( int )), SLOT(closeRequest( int )));
 #ifndef QT_NO_WHEELEVENT
   connect(tabBar(), SIGNAL(wheelDelta( int )), SLOT(wheelDelta( int )));
 #endif
@@ -230,9 +273,13 @@ void KTabWidget::insertTab( QWidget *child, QTab *tab, int index )
 
 void KTabWidget::setTabBarHidden( bool hide )
 {
+  if (hide == isTabBarHidden())
+    return;
+
   QWidget *rightcorner = cornerWidget( Qt::TopRightCorner );
   QWidget *leftcorner = cornerWidget( Qt::TopLeftCorner );
 
+  d->m_tabBarHidden = hide;
   if ( hide ) {
     if ( leftcorner ) leftcorner->hide();
     if ( rightcorner ) rightcorner->hide();
@@ -246,7 +293,7 @@ void KTabWidget::setTabBarHidden( bool hide )
 
 bool KTabWidget::isTabBarHidden() const
 {
-  return !( tabBar()->isVisible() );
+  return d->m_tabBarHidden;
 }
 
 void KTabWidget::setTabTextColor( int index, const QColor& color )
@@ -288,7 +335,7 @@ int KTabWidget::tabBarWidthForMaxChars( int maxLength )
   const QFontMetrics fm = tabBar()->fontMetrics();
   int x = 0;
   for ( int i = 0; i < count(); ++i ) {
-    QString newTitle = d->m_tabNames[ i ];
+    QString newTitle = d->m_tabNames.value( i );
     newTitle = KStringHandler::rsqueeze( newTitle, maxLength ).leftJustified( d->m_minLength, ' ' );
 
     int lw = fm.width( newTitle );
@@ -340,6 +387,12 @@ void KTabWidget::setTabText( int index, const QString &text )
     QTabWidget::setTabText( index, text );
 
     if ( index != -1 ) {
+        if (index >= d->m_tabNames.count()) {
+            kWarning(240) << "setTabText(" << index << ") called but d->m_tabNames has only" << d->m_tabNames.count() << "entries";
+            while (index >= d->m_tabNames.count()) {
+                d->m_tabNames.append(QString());
+            }
+        }
       d->m_tabNames[ index ] = text;
       d->resizeTabs( index );
     }
@@ -441,14 +494,21 @@ void KTabWidget::mousePressEvent( QMouseEvent *event )
       emit( contextMenu( mapToGlobal( event->pos() ) ) );
       return;
     }
-  } else if ( event->button() == Qt::MidButton ) {
+  }
+
+  QTabWidget::mousePressEvent( event );
+}
+
+void KTabWidget::mouseReleaseEvent( QMouseEvent *event )
+{
+  if ( event->button() == Qt::MidButton ) {
     if ( d->isEmptyTabbarSpace( event->pos() ) ) {
       emit( mouseMiddleClick() );
       return;
     }
   }
 
-  QTabWidget::mousePressEvent( event );
+  QTabWidget::mouseReleaseEvent( event );
 }
 
 void KTabWidget::receivedDropEvent( int index, QDropEvent *event )
@@ -476,6 +536,19 @@ void KTabWidget::mouseMiddleClick( int index )
   emit( mouseMiddleClick( widget( index ) ) );
 }
 
+// Helper: return true if w is inside parent (directly or indirectly)
+static bool isWidgetInside(QWidget* w, QWidget* parent)
+{
+  while (w) {
+      if (w == parent)
+          return true;
+      if (w->isWindow())
+          break;
+      w = w->parentWidget();
+  }
+  return false;
+}
+
 void KTabWidget::moveTab( int from, int to )
 {
   setUpdatesEnabled(false);
@@ -489,9 +562,18 @@ void KTabWidget::moveTab( int from, int to )
   const bool enabled = isTabEnabled( from );
 
   const bool blocked = blockSignals( true );
+
+  QWidget *fw = QApplication::focusWidget();
+
   removeTab( from );
   insertTab( to, w, tablabel );
 
+  // Don't lose focus due to moving the tab (#159295)
+  // (removeTab hides the widget, which gives focus to the "next in chain", could be anything)
+  if (isWidgetInside(fw, w)) {
+      fw->setFocus();
+  }
+
   setTabIcon( to, tabiconset );
   setTabText( to, tablabel );
   setTabToolTip( to, tabtooltip );
@@ -511,31 +593,24 @@ void KTabWidget::moveTab( int from, int to )
 
 void KTabWidget::removePage( QWidget *widget )
 {
+  // not just calling removeTab() because that one is also virtual.
+  const int index = indexOf(widget);
   if ( d->m_automaticResizeTabs ) {
-
     setUpdatesEnabled(false);
-
-    d->removeTab( indexOf( widget ) );
-    d->resizeTabs();
-
+    d->removeTab(index);
     setUpdatesEnabled(true);
-
   } else {
-    d->removeTab( indexOf( widget ) );
+    d->removeTab(index);
   }
 }
 
 void KTabWidget::removeTab( int index )
 {
   if ( d->m_automaticResizeTabs ) {
-
+    const bool wasUpdatesEnabled = updatesEnabled();
     setUpdatesEnabled(false);
-
     d->removeTab( index );
-    d->resizeTabs();
-
-    setUpdatesEnabled(true);
-
+    setUpdatesEnabled(wasUpdatesEnabled);
   } else {
     d->removeTab( index );
   }
@@ -544,7 +619,7 @@ void KTabWidget::removeTab( int index )
 void KTabWidget::setHoverCloseButton( bool button )
 {
   // deprecated
-  setCloseButtonEnabled( button );
+  setTabsClosable( button );
 }
 
 bool KTabWidget::hoverCloseButton() const
@@ -562,17 +637,17 @@ void KTabWidget::setHoverCloseButtonDelayed( bool delayed )
 bool KTabWidget::hoverCloseButtonDelayed() const
 {
   // deprecated
-  return isCloseButtonEnabled();
+  return tabsClosable();
 }
 
 void KTabWidget::setCloseButtonEnabled( bool enable )
 {
-  static_cast<KTabBar*>( tabBar() )->setCloseButtonEnabled( enable );
+  static_cast<KTabBar*>( tabBar() )->setTabsClosable( enable );
 }
 
 bool KTabWidget::isCloseButtonEnabled() const
 {
-  return static_cast<KTabBar*>( tabBar() )->isCloseButtonEnabled();
+  return static_cast<KTabBar*>( tabBar() )->tabsClosable();
 }
 
 void KTabWidget::setAutomaticResizeTabs( bool enabled )
@@ -619,15 +694,13 @@ void KTabWidget::tabInserted( int idx )
 
 void KTabWidget::tabRemoved( int idx )
 {
-   d->m_tabNames.removeAt( idx );
+  Q_UNUSED(idx)
+// d->m_tabNames is now updated in KTabWidget::Private::removeTab()
 }
 
-void KTabWidget::currentChanged( int idx )
+/* This function is kept only for BC reasons, it is not useful anymore */
+void KTabWidget::currentChanged( int )
 {
-   // The tab should appear only once
-   d->m_previousTabList.removeOne( widget(idx) );
-   
-   d->m_previousTabList.push_front( widget(idx) );
 }
 
 #include "ktabwidget.moc"
