@@ -19,14 +19,20 @@
 */
 
 #include "ktextedit.h"
+#include <ktoolinvocation.h>
 #include <kdebug.h>
 
 #include <QApplication>
 #include <QClipboard>
 #include <QKeyEvent>
 #include <QMenu>
+#include <QPainter>
 #include <QScrollBar>
 #include <QTextCursor>
+#include <QTextDocumentFragment>
+#include <QDBusInterface>
+#include <QDBusConnection>
+#include <QDBusConnectionInterface>
 
 #include <configdialog.h>
 #include <dialog.h>
@@ -47,6 +53,7 @@
 #include <kmessagebox.h>
 #include <kmenu.h>
 #include <kwindowsystem.h>
+#include <QDebug>
 
 class KTextEdit::Private
 {
@@ -56,8 +63,21 @@ class KTextEdit::Private
         customPalette( false ),
         checkSpellingEnabled( false ),
         findReplaceEnabled(true),
-        highlighter( 0 ), findDlg(0),find(0),repDlg(0),replace(0), findIndex(0), repIndex(0)
+        highlighter( 0 ), findDlg(0),find(0),repDlg(0),replace(0), findIndex(0), repIndex(0),
+        lastReplacedPosition(-1)
     {
+        //Check the default sonnet settings to see if spellchecking should be enabled.
+        sonnetKConfig = new KConfig("sonnetrc");
+        KConfigGroup group(sonnetKConfig, "Spelling");
+        checkSpellingEnabled = group.readEntry("checkerEnabledByDefault", false);
+
+        // i18n: Placeholder text in text edit widgets is the text appearing
+        // before any user input, briefly explaining to the user what to type
+        // (e.g. "Enter message").
+        // By default the text is set in italic, which may not be appropriate
+        // for some languages and scripts (e.g. for CJK ideographs).
+        QString metaMsg = i18nc("Italic placeholder text in line edits: 0 no, 1 yes", "1");
+        italicizePlaceholder = (metaMsg.trimmed() != QString('0'));
     }
 
     ~Private()
@@ -67,6 +87,7 @@ class KTextEdit::Private
       delete find;
       delete replace;
       delete repDlg;
+      delete sonnetKConfig;
     }
 
     /**
@@ -99,6 +120,8 @@ class KTextEdit::Private
     void slotAllowTab();
     void menuActivated( QAction* action );
 
+    void updateClickMessageRect();
+
     void init();
 
     KTextEdit *parent;
@@ -106,13 +129,14 @@ class KTextEdit::Private
     QAction *autoSpellCheckAction;
     QAction *allowTab;
     QAction *spellCheckAction;
+    QString clickMessage;
+    bool italicizePlaceholder : 1;
     bool customPalette : 1;
 
     bool checkSpellingEnabled : 1;
     bool findReplaceEnabled: 1;
-    QString originalBuffer;
-    QString originalHtml;
-    QString spellChechingConfigFileName;
+    QTextDocumentFragment originalDoc;
+    QString spellCheckingConfigFileName;
     QString spellCheckingLanguage;
     Sonnet::Highlighter *highlighter;
     KFindDialog *findDlg;
@@ -120,15 +144,16 @@ class KTextEdit::Private
     KReplaceDialog *repDlg;
     KReplace *replace;
     int findIndex, repIndex;
+    int lastReplacedPosition;
+    KConfig *sonnetKConfig;
 };
 
 void KTextEdit::Private::spellCheckerCanceled()
 {
-    parent->selectAll();
-    if(parent->acceptRichText ())
-      parent->setHtml(originalHtml);
-    else
-      parent->setPlainText(originalBuffer);
+    QTextDocument *doc = parent->document();
+    doc->clear();
+    QTextCursor cursor(doc);
+    cursor.insertFragment(originalDoc);
     spellCheckerFinished();
 }
 
@@ -206,18 +231,26 @@ void KTextEdit::Private::slotFindHighlight(const QString& text, int matchingInde
 }
 
 
-void KTextEdit::Private::slotReplaceText(const QString &text, int replacementIndex, int /*replacedLength*/, int matchedLength) {
-    Q_UNUSED(text)
+void KTextEdit::Private::slotReplaceText(const QString &text, int replacementIndex, int replacedLength, int matchedLength) {
     //kDebug() << "Replace: [" << text << "] ri:" << replacementIndex << " rl:" << replacedLength << " ml:" << matchedLength;
     QTextCursor tc = parent->textCursor();
     tc.setPosition(replacementIndex);
     tc.movePosition(QTextCursor::NextCharacter, QTextCursor::KeepAnchor, matchedLength);
     tc.removeSelectedText();
-    tc.insertText(repDlg->replacement());
-    parent->setTextCursor(tc);
+    tc.insertText(text.mid(replacementIndex, replacedLength));
     if (replace->options() & KReplaceDialog::PromptOnReplace) {
+        parent->setTextCursor(tc);
         parent->ensureCursorVisible();
     }
+    lastReplacedPosition = replacementIndex;
+}
+
+void KTextEdit::Private::updateClickMessageRect()
+{
+    int margin = int(parent->document()->documentMargin());
+    QRect rect = parent->viewport()->rect().adjusted(margin, margin, -margin, -margin);
+    rect = parent->fontMetrics().boundingRect(rect, Qt::AlignTop | Qt::TextWordWrap, clickMessage);
+    parent->viewport()->update(rect);
 }
 
 void KTextEdit::Private::init()
@@ -247,7 +280,7 @@ KTextEdit::~KTextEdit()
 
 void KTextEdit::setSpellCheckingConfigFileName(const QString &_fileName)
 {
-    d->spellChechingConfigFileName = _fileName;
+    d->spellCheckingConfigFileName = _fileName;
 }
 
 const QString& KTextEdit::spellCheckingLanguage() const
@@ -310,11 +343,13 @@ bool KTextEdit::Private::handleShortcut(const QKeyEvent* event)
     parent->cut();
     return true;
   } else if ( KStandardShortcut::undo().contains( key ) ) {
-    parent->document()->undo();
-    return true;
+      if(!parent->isReadOnly())
+          parent->undo();
+      return true;
   } else if ( KStandardShortcut::redo().contains( key ) ) {
-    parent->document()->redo();
-    return true;
+      if(!parent->isReadOnly())
+         parent->redo();
+      return true;
   } else if ( KStandardShortcut::deleteWordBack().contains( key ) ) {
     parent->deleteWordBack();
     return true;
@@ -408,20 +443,21 @@ bool KTextEdit::Private::handleShortcut(const QKeyEvent* event)
   return false;
 }
 
+static void deleteWord(QTextCursor cursor, QTextCursor::MoveOperation op)
+{
+  cursor.clearSelection();
+  cursor.movePosition( op, QTextCursor::KeepAnchor );
+  cursor.removeSelectedText();
+}
+
 void KTextEdit::deleteWordBack()
 {
-  QTextCursor cursor = textCursor();
-  cursor.clearSelection();
-  cursor.movePosition( QTextCursor::PreviousWord, QTextCursor::KeepAnchor );
-  cursor.removeSelectedText();
+  deleteWord(textCursor(), QTextCursor::PreviousWord);
 }
 
 void KTextEdit::deleteWordForward()
 {
-  QTextCursor cursor = textCursor();
-  cursor.clearSelection();
-  cursor.movePosition( QTextCursor::EndOfWord, QTextCursor::KeepAnchor );
-  cursor.removeSelectedText();
+  deleteWord(textCursor(), QTextCursor::WordRight);
 }
 
 QMenu *KTextEdit::mousePopupMenu()
@@ -486,9 +522,35 @@ QMenu *KTextEdit::mousePopupMenu()
           popup->addAction(replaceAction);
       }
   }
+  popup->addSeparator();
+  QAction *speakAction = popup->addAction(i18n("Speak Text"));
+  speakAction->setIcon(KIcon("preferences-desktop-text-to-speech"));
+  speakAction->setEnabled(!emptyDocument );
+  connect( speakAction, SIGNAL(triggered(bool)), this, SLOT(slotSpeakText()) );
   return popup;
 }
 
+void KTextEdit::slotSpeakText()
+{
+    // If KTTSD not running, start it.
+    if (!QDBusConnection::sessionBus().interface()->isServiceRegistered("org.kde.kttsd"))
+    {
+        QString error;
+        if (KToolInvocation::startServiceByDesktopName("kttsd", QStringList(), &error))
+        {
+            KMessageBox::error(this, i18n( "Starting Jovie Text-to-Speech Service Failed"), error );
+            return;
+        }
+    }
+    QDBusInterface ktts("org.kde.kttsd", "/KSpeech", "org.kde.KSpeech");
+    QString text;
+    if(textCursor().hasSelection())
+        text = textCursor().selectedText();
+    else
+        text = toPlainText();
+    ktts.asyncCall("say", text, 0);
+}
+
 void KTextEdit::contextMenuEvent(QContextMenuEvent *event)
 {
     // Obtain the cursor at the mouse position and the current cursor
@@ -508,6 +570,13 @@ void KTextEdit::contextMenuEvent(QContextMenuEvent *event)
     wordSelectCursor.select(QTextCursor::WordUnderCursor);
     QString selectedWord = wordSelectCursor.selectedText();
 
+    bool isMouseCursorInsideWord = true;
+    if ((mousePos < wordSelectCursor.selectionStart() ||
+            mousePos >= wordSelectCursor.selectionEnd())
+                                        && (selectedWord.length() > 1)) {
+         isMouseCursorInsideWord = false;
+    }
+
     // Clear the selection again, we re-select it below (without the apostrophes).
     wordSelectCursor.setPosition(wordSelectCursor.position()-selectedWord.size());
     if (selectedWord.startsWith('\'') || selectedWord.startsWith('\"')) {
@@ -520,7 +589,8 @@ void KTextEdit::contextMenuEvent(QContextMenuEvent *event)
     wordSelectCursor.movePosition(QTextCursor::NextCharacter,
                                   QTextCursor::KeepAnchor, selectedWord.size());
 
-    const bool wordIsMisspelled = checkSpellingEnabled() &&
+    const bool wordIsMisspelled = isMouseCursorInsideWord &&
+                            checkSpellingEnabled() &&
                             !selectedWord.isEmpty() &&
                             highlighter() &&
                             highlighter()->isWordMisspelled(selectedWord);
@@ -546,30 +616,30 @@ void KTextEdit::contextMenuEvent(QContextMenuEvent *event)
     if (!wordIsMisspelled || selectedWordClicked || inQuote) {
         QMenu *popup = mousePopupMenu();
         if ( popup ) {
+            aboutToShowContextMenu(popup);
             popup->exec( event->globalPos() );
             delete popup;
         }
     }
     else {
-        KMenu suggestions;
-        KMenu menu;
-
-        //Add the suggestions to the popup menu
-        QStringList reps = highlighter()->suggestionsForWord(selectedWord);
-        if (reps.count() > 0) {
-            for (QStringList::const_iterator it = reps.constBegin(); it != reps.constEnd(); ++it) {
-                suggestions.addAction(*it);
-            }
+        QMenu menu; //don't use KMenu here we don't want auto management accelerator
 
+        //Add the suggestions to the menu
+        const QStringList reps = highlighter()->suggestionsForWord(selectedWord);
+        if (reps.isEmpty()) {
+            QAction *suggestionsAction = menu.addAction(i18n("No suggestions for %1", selectedWord));
+            suggestionsAction->setEnabled(false);
         }
-        suggestions.setTitle(i18n("Suggestions"));
+        else {
+            for (QStringList::const_iterator it = reps.constBegin(); it != reps.constEnd(); ++it) {
+                menu.addAction(*it);
+            }
+        }
+
+        menu.addSeparator();
 
         QAction *ignoreAction = menu.addAction(i18n("Ignore"));
         QAction *addToDictAction = menu.addAction(i18n("Add to Dictionary"));
-        QAction *suggestionsAction = menu.addMenu(&suggestions);
-        if (reps.count() == 0)
-            suggestionsAction->setEnabled(false);
-
         //Execute the popup inline
         const QAction *selectedAction = menu.exec(event->globalPos());
 
@@ -606,7 +676,7 @@ void KTextEdit::wheelEvent( QWheelEvent *event )
 
 void KTextEdit::createHighlighter()
 {
-    setHighlighter(new Sonnet::Highlighter(this, d->spellChechingConfigFileName));
+    setHighlighter(new Sonnet::Highlighter(this, d->spellCheckingConfigFileName));
 }
 
 Sonnet::Highlighter* KTextEdit::highlighter() const
@@ -659,6 +729,9 @@ void KTextEdit::focusInEvent( QFocusEvent *event )
   if ( d->checkSpellingEnabled && !isReadOnly() && !d->highlighter )
     createHighlighter();
 
+  if (!d->clickMessage.isEmpty()) {
+      d->updateClickMessageRect();
+  }
   QTextEdit::focusInEvent( event );
 }
 
@@ -735,10 +808,8 @@ void KTextEdit::checkSpelling()
           this,SIGNAL(spellCheckStatus(const QString &)));
   connect(spellDialog, SIGNAL(languageChanged(const QString &)),
           this, SIGNAL(languageChanged(const QString &)));
-  d->originalBuffer = toPlainText();
-  if(acceptRichText ())
-    d->originalHtml = toHtml();
-  spellDialog->setBuffer(d->originalBuffer);
+  d->originalDoc = QTextDocumentFragment(document());
+  spellDialog->setBuffer(toPlainText());
   spellDialog->show();
 }
 
@@ -774,8 +845,7 @@ void KTextEdit::slotDoReplace()
     }
 
     if(d->repDlg->pattern().isEmpty()) {
-        d->replace->disconnect(this);
-        d->replace->deleteLater(); // we are in a slot connected to m_replace, don't delete it right away
+	delete d->replace;
         d->replace = 0;
         ensureCursorVisible();
         return;
@@ -806,8 +876,11 @@ void KTextEdit::slotReplaceNext()
     if (!d->replace)
         return;
 
-    if (!(d->replace->options() & KReplaceDialog::PromptOnReplace))
+    d->lastReplacedPosition = -1;
+    if (!(d->replace->options() & KReplaceDialog::PromptOnReplace)) {
+        textCursor().beginEditBlock(); // #48541
         viewport()->setUpdatesEnabled(false);
+    }
 
     KFind::Result res = KFind::NoMatch;
 
@@ -815,6 +888,14 @@ void KTextEdit::slotReplaceNext()
         d->replace->setData(toPlainText(), d->repIndex);
     res = d->replace->replace();
     if (!(d->replace->options() & KReplaceDialog::PromptOnReplace)) {
+        textCursor().endEditBlock(); // #48541
+        if (d->lastReplacedPosition >= 0) {
+            QTextCursor tc = textCursor();
+            tc.setPosition(d->lastReplacedPosition);
+            setTextCursor(tc);
+            ensureCursorVisible();
+        }
+
         viewport()->setUpdatesEnabled(true);
         viewport()->update();
     }
@@ -840,8 +921,7 @@ void KTextEdit::slotDoFind()
     }
     if( d->findDlg->pattern().isEmpty())
     {
-        d->find->disconnect(this);
-        d->find->deleteLater(); // we are in a slot connected to m_find, don't delete right away
+        delete d->find;
         d->find = 0;
         return;
     }
@@ -999,4 +1079,52 @@ void KTextEdit::keyPressEvent( QKeyEvent *event )
     }
 }
 
+void KTextEdit::setClickMessage(const QString &msg)
+{
+    if (msg != d->clickMessage) {
+        if (!d->clickMessage.isEmpty()) {
+            d->updateClickMessageRect();
+        }
+        d->clickMessage = msg;
+        if (!d->clickMessage.isEmpty()) {
+            d->updateClickMessageRect();
+        }
+    }
+}
+
+QString KTextEdit::clickMessage() const
+{
+    return d->clickMessage;
+}
+
+void KTextEdit::paintEvent(QPaintEvent *ev)
+{
+    QTextEdit::paintEvent(ev);
+
+    if (!d->clickMessage.isEmpty() && !hasFocus() && document()->isEmpty()) {
+        QPainter p(viewport());
+
+        QFont f = font();
+        f.setItalic(d->italicizePlaceholder);
+        p.setFont(f);
+
+        QColor color(palette().color(foregroundRole()));
+        color.setAlphaF(0.5);
+        p.setPen(color);
+
+        int margin = int(document()->documentMargin());
+        QRect cr = viewport()->rect().adjusted(margin, margin, -margin, -margin);
+
+        p.drawText(cr, Qt::AlignTop | Qt::TextWordWrap, d->clickMessage);
+    }
+}
+
+void KTextEdit::focusOutEvent(QFocusEvent *ev)
+{
+    if (!d->clickMessage.isEmpty()) {
+        d->updateClickMessageRect();
+    }
+    QTextEdit::focusOutEvent(ev);
+}
+
 #include "ktextedit.moc"
