@@ -1187,38 +1187,41 @@ bool KToolBar::eventFilter(QObject * watched, QEvent * event)
     }
 
     QToolButton* tb;
-    if ((tb = qobject_cast<QToolButton*>(watched)) && !tb->actions().isEmpty()) {
-        // Handle MMB on toolbar buttons
-        QAction* act = tb->actions().first();
-        if (event->type() == QEvent::MouseButtonPress || event->type() == QEvent::MouseButtonRelease) {
-            QMouseEvent* me = static_cast<QMouseEvent*>(event);
-            if (me->button() == Qt::MidButton /*&&
+    if ((tb = qobject_cast<QToolButton*>(watched))) {
+        const QList<QAction*> tbActions = tb->actions();
+        if (!tbActions.isEmpty()) {
+            // Handle MMB on toolbar buttons
+            if (event->type() == QEvent::MouseButtonPress || event->type() == QEvent::MouseButtonRelease) {
+                QMouseEvent* me = static_cast<QMouseEvent*>(event);
+                if (me->button() == Qt::MidButton /*&&
                                                  act->receivers(SIGNAL(triggered(Qt::MouseButtons,Qt::KeyboardModifiers)))*/) {
-                if (me->type() == QEvent::MouseButtonPress)
-                    tb->setDown(act->isEnabled());
-                else {
-                    tb->setDown(false);
-                    if (act->isEnabled()) {
-                        QMetaObject::invokeMethod(act, "triggered", Qt::DirectConnection,
-                                                  Q_ARG(Qt::MouseButtons, me->button()),
-                                                  Q_ARG(Qt::KeyboardModifiers, QApplication::keyboardModifiers()));
+                    QAction* act = tbActions.first();
+                    if (me->type() == QEvent::MouseButtonPress)
+                        tb->setDown(act->isEnabled());
+                    else {
+                        tb->setDown(false);
+                        if (act->isEnabled()) {
+                            QMetaObject::invokeMethod(act, "triggered", Qt::DirectConnection,
+                                                      Q_ARG(Qt::MouseButtons, me->button()),
+                                                      Q_ARG(Qt::KeyboardModifiers, QApplication::keyboardModifiers()));
+                        }
                     }
                 }
             }
-        }
 
-        // CJK languages use more verbose accelerator marker: they add a Latin
-        // letter in parenthesis, and put accelerator on that. Hence, the default
-        // removal of ampersand only may not be enough there, instead the whole
-        // parenthesis construct should be removed. Use KLocale's method to do this.
-        if (event->type() == QEvent::Show || event->type() == QEvent::Paint || event->type() == QEvent::EnabledChange) {
-            act = tb->defaultAction();
-            if (act) {
-                const QString text = KGlobal::locale()->removeAcceleratorMarker(act->iconText().isEmpty() ? act->text() : act->iconText());
-                const QString toolTip = KGlobal::locale()->removeAcceleratorMarker(act->toolTip());
-                // Filtering messages requested by translators (scripting).
-                tb->setText(i18nc("@action:intoolbar Text label of toolbar button", "%1", text));
-                tb->setToolTip(i18nc("@info:tooltip Tooltip of toolbar button", "%1", toolTip));
+            // CJK languages use more verbose accelerator marker: they add a Latin
+            // letter in parenthesis, and put accelerator on that. Hence, the default
+            // removal of ampersand only may not be enough there, instead the whole
+            // parenthesis construct should be removed. Use KLocale's method to do this.
+            if (event->type() == QEvent::Show || event->type() == QEvent::Paint || event->type() == QEvent::EnabledChange) {
+                QAction *act = tb->defaultAction();
+                if (act) {
+                    const QString text = KGlobal::locale()->removeAcceleratorMarker(act->iconText().isEmpty() ? act->text() : act->iconText());
+                    const QString toolTip = KGlobal::locale()->removeAcceleratorMarker(act->toolTip());
+                    // Filtering messages requested by translators (scripting).
+                    tb->setText(i18nc("@action:intoolbar Text label of toolbar button", "%1", text));
+                    tb->setToolTip(i18nc("@info:tooltip Tooltip of toolbar button", "%1", toolTip));
+                }
             }
         }
     }
@@ -1278,6 +1281,13 @@ void KToolBar::actionEvent(QActionEvent * event)
 
         foreach (QWidget* child, widget->findChildren<QWidget*>())
             child->installEventFilter(this);
+        // Center widgets that do not have any use for more space. See bug 165274
+        if (!(widget->sizePolicy().horizontalPolicy() & QSizePolicy::GrowFlag)) {
+            const int index = layout()->indexOf(widget);
+            if (index != -1) {
+                layout()->itemAt(index)->setAlignment(Qt::AlignJustify);
+            }
+        }
     }
   }
 
