@@ -34,6 +34,10 @@
 #include <windows.h>
 #include <windowsx.h>
 
+#ifdef __WIN64
+#define GCL_HICON GCLP_HICON
+#define GCL_HICONSM GCLP_HICONSM
+#endif
 
 //function to register us as taskmanager
 #define RSH_UNREGISTER  0
@@ -134,15 +138,7 @@ static HICON QPixmap2HIcon(const QPixmap &pix)
 
 static QPixmap HIcon2QPixmap( HICON hIcon )
 {
-    ICONINFO ii;
-    if( GetIconInfo( hIcon, &ii ) == 0 )
-        return QPixmap();
-
-    QPixmap pix  = QPixmap::fromWinHBITMAP( ii.hbmColor );
-    pix.setMask( QPixmap::fromWinHBITMAP( ii.hbmMask ) );
-
-    DeleteObject( ii.hbmMask );
-    DeleteObject( ii.hbmColor );
+    QPixmap pix  = QPixmap::fromWinHICON( hIcon );
 
     return pix;
 }
