@@ -168,8 +168,12 @@ bool KWindowSystemPrivate::x11Event( XEvent * ev )
             else if( ev->xproperty.atom == XA_WM_ICON_NAME )
                 dirty[ NETWinInfo::PROTOCOLS ] |= NET::WMIconName; // support for old iconic name
         }
-        if( mapViewport() && ( dirty[ NETWinInfo::PROTOCOLS ] & NET::WMState ))
-            dirty[ NETWinInfo::PROTOCOLS ] |= NET::WMDesktop; // possible NET::Sticky change
+        if( mapViewport() && ( dirty[ NETWinInfo::PROTOCOLS ] & (NET::WMState | NET::WMGeometry) )) {
+	    /* geometry change -> possible viewport change
+	     * state change -> possible NET::Sticky change
+	     */
+            dirty[ NETWinInfo::PROTOCOLS ] |= NET::WMDesktop;
+	}
 	if ( (dirty[ NETWinInfo::PROTOCOLS ] & NET::WMStrut) != 0 ) {
             removeStrutWindow( ev->xany.window );
             if ( !possibleStrutWindows.contains( ev->xany.window ) )
@@ -502,7 +506,7 @@ void KWindowSystem::setOnDesktop( WId win, int desktop )
         x -= w / 2; // from center back to topleft
         y -= h / 2;
         p = constrainViewportRelativePosition( QPoint( x, y ));
-        int flags = ( 0x02 << 12 ) | ( 0x03 << 8 ) | 10; // from tool(?), x/y, static gravity
+        int flags = ( NET::FromTool << 12 ) | ( 0x03 << 8 ) | 10; // from tool(?), x/y, static gravity
         KWindowSystemPrivate* const s_d = s_d_func();
         s_d->moveResizeWindowRequest( win, flags, p.x(), p.y(), w, h );
         return;
@@ -966,6 +970,13 @@ void KWindowSystem::doNotManage( const QString& title )
         .call("doNotManage", title);
 }
 
+void KWindowSystem::allowExternalProcessWindowActivation( int pid )
+{
+    // Normally supported by X11, but may depend on some window managers ?
+    Q_UNUSED(pid)
+}
+
+
 bool KWindowSystem::mapViewport()
 {
     KWindowSystemPrivate* const s_d = s_d_func();
