@@ -19,12 +19,14 @@
 */
 
 #include "kdiroperator.h"
+#include <kprotocolmanager.h>
 #include "kdirmodel.h"
 #include "kdiroperatordetailview_p.h"
 #include "kdirsortfilterproxymodel.h"
 #include "kfileitem.h"
 #include "kfilemetapreview.h"
 #include "kpreviewwidgetbase.h"
+#include "knewfilemenu.h"
 
 #include <config-kfile.h>
 
@@ -69,6 +71,7 @@
 #include <kio/previewjob.h>
 #include <kio/renamedialog.h>
 #include <kfilepreviewgenerator.h>
+#include <krun.h>
 #include <kpropertiesdialog.h>
 #include <kstandardshortcut.h>
 #include <kde_file.h>
@@ -81,7 +84,7 @@
 
 template class QHash<QString, KFileItem>;
 
-// QDir::SortByMask is not only undocumented, it also omits QDir::Type which  is another 
+// QDir::SortByMask is not only undocumented, it also omits QDir::Type which  is another
 // sorting mode.
 static const int QDirSortMask = QDir::SortByMask | QDir::Type;
 
@@ -112,14 +115,13 @@ KDirOperatorIconView::KDirOperatorIconView(KDirOperator *dirOperator, QWidget *p
     setViewMode(QListView::IconMode);
     setFlow(QListView::TopToBottom);
     setResizeMode(QListView::Adjust);
-    setSpacing(KDialog::spacingHint());
+    setSpacing(0);
     setMovement(QListView::Static);
     setDragDropMode(QListView::DragOnly);
     setVerticalScrollMode(QListView::ScrollPerPixel);
     setHorizontalScrollMode(QListView::ScrollPerPixel);
     setEditTriggers(QAbstractItemView::NoEditTriggers);
     setWordWrap(true);
-    setSpacing(0);
     setIconSize(QSize(KIconLoader::SizeSmall, KIconLoader::SizeSmall));
 }
 
@@ -176,6 +178,13 @@ void KDirOperatorIconView::wheelEvent(QWheelEvent *event)
     }
 }
 
+void KDirOperator::keyPressEvent(QKeyEvent *e)
+{
+    if (!(e->key() == Qt::Key_Return || e->key() == Qt::Key_Enter )) {
+        QWidget::keyPressEvent(e);
+    }
+}
+
 class KDirOperator::Private
 {
 public:
@@ -208,6 +217,7 @@ public:
     void _k_slotToggleHidden(bool);
     void _k_togglePreview(bool);
     void _k_toggleInlinePreviews(bool);
+    void _k_slotOpenFileManager();
     void _k_slotSortByName();
     void _k_slotSortBySize();
     void _k_slotSortByDate();
@@ -235,6 +245,7 @@ public:
     void _k_slotChangeDecorationPosition();
     void _k_slotExpandToUrl(const QModelIndex&);
     void _k_slotItemsChanged();
+    void _k_slotDirectoryCreated(const KUrl&);
 
     void updateListViewGrid();
     int iconSizeForViewType(QAbstractItemView *itemView) const;
@@ -284,6 +295,8 @@ public:
     KActionMenu *actionMenu;
     KActionCollection *actionCollection;
 
+    KNewFileMenu *newFileMenu;
+
     KConfigGroup *configGroup;
 
     KFilePreviewGenerator *previewGenerator;
@@ -319,6 +332,7 @@ KDirOperator::Private::Private(KDirOperator *_parent) :
     dropOptions(0),
     actionMenu(0),
     actionCollection(0),
+    newFileMenu(0),
     configGroup(0),
     previewGenerator(0),
     showPreviews(false),
@@ -420,6 +434,7 @@ KDirOperator::KDirOperator(const KUrl& _url, QWidget *parent) :
 KDirOperator::~KDirOperator()
 {
     resetCursor();
+    disconnect(d->dirLister, 0, this, 0);
     delete d;
 }
 
@@ -436,6 +451,14 @@ QDir::SortFlags KDirOperator::sorting() const
 
 bool KDirOperator::isRoot() const
 {
+#ifdef Q_WS_WIN
+    if (url().isLocalFile()) {
+        const QString path = url().toLocalFile();
+        if (path.length() == 3)
+            return (path[0].isLetter() && path[1] == ':' && path[2] == '/');
+        return false;
+    } else
+#endif
     return url().path() == QString(QLatin1Char('/'));
 }
 
@@ -479,7 +502,7 @@ void KDirOperator::sortReversed()
 
 void KDirOperator::toggleDirsFirst()
 {
-    // TODO: not offered yet
+    d->_k_slotToggleDirsFirst();
 }
 
 void KDirOperator::toggleIgnoreCase()
@@ -663,6 +686,11 @@ void KDirOperator::Private::_k_toggleInlinePreviews(bool show)
     }
 }
 
+void KDirOperator::Private::_k_slotOpenFileManager()
+{
+    new KRun(currUrl, parent);
+}
+
 void KDirOperator::Private::_k_slotSortByName()
 {
     parent->sortByName();
@@ -694,16 +722,8 @@ void KDirOperator::Private::_k_slotSortReversed(bool doReverse)
 
 void KDirOperator::Private::_k_slotToggleDirsFirst()
 {
-    // TODO: port to Qt4's QAbstractItemView
-    /*if ( !d->fileView )
-      return;
-
-    QDir::SortFlags sorting = d->fileView->sorting();
-    if ( !KFile::isSortDirsFirst( sorting ) )
-        d->fileView->setSorting( sorting | QDir::DirsFirst );
-    else
-        d->fileView->setSorting( sorting & ~QDir::DirsFirst );
-    d->sorting = d->fileView->sorting();*/
+    QDir::SortFlags s = (sorting ^ QDir::DirsFirst);
+    updateSorting(s);
 }
 
 void KDirOperator::Private::_k_slotToggleIgnoreCase()
@@ -722,21 +742,9 @@ void KDirOperator::Private::_k_slotToggleIgnoreCase()
 
 void KDirOperator::mkdir()
 {
-    bool ok;
-    QString where = url().pathOrUrl();
-    QString name = i18n("New Folder");
-#ifdef Q_WS_WIN
-    if (url().isLocalFile() && QFileInfo(url().toLocalFile() + name).exists())
-#else
-    if (url().isLocalFile() && QFileInfo(url().path(KUrl::AddTrailingSlash) + name).exists())
-#endif
-        name = KIO::RenameDialog::suggestName(url(), name);
-
-    QString folder = KInputDialog::getText(i18n("New Folder"),
-                                           i18n("Create new folder in:\n%1" ,  where),
-                                           name, &ok, this);
-    if (ok)
-        KDirOperator::mkdir(KIO::encodeFileName(folder), true);
+    d->newFileMenu->setPopupFiles(url());
+    d->newFileMenu->setViewShowsHiddenFiles(showHiddenFiles());
+    d->newFileMenu->createDirectory();
 }
 
 bool KDirOperator::mkdir(const QString& directory, bool enterDirectory)
@@ -749,7 +757,7 @@ bool KDirOperator::mkdir(const QString& directory, bool enterDirectory)
     bool exists = false;
     KUrl url(d->currUrl);
 
-    const QStringList dirs = directory.split(QDir::separator(), QString::SkipEmptyParts);
+    const QStringList dirs = directory.split('/', QString::SkipEmptyParts);
     QStringList::ConstIterator it = dirs.begin();
 
     for (; it != dirs.end(); ++it) {
@@ -760,7 +768,6 @@ bool KDirOperator::mkdir(const QString& directory, bool enterDirectory)
 
     if (exists) { // url was already existent
         KMessageBox::sorry(d->itemView, i18n("A file or folder named %1 already exists.", url.pathOrUrl()));
-        enterDirectory = false;
     } else if (!writeOk) {
         KMessageBox::sorry(d->itemView, i18n("You do not have permission to "
                                               "create that folder."));
@@ -1049,7 +1056,7 @@ void KDirOperator::setUrl(const KUrl& _newurl, bool clearforward)
 
     if (!Private::isReadable(newurl)) {
         // maybe newurl is a file? check its parent directory
-        newurl.setPath(newurl.directory());
+        newurl.setPath(newurl.directory(KUrl::ObeyTrailingSlash));
         if (newurl.equals(d->currUrl, KUrl::CompareWithoutTrailingSlash))
             return; // parent is current dir, nothing to do (fixes #173454, too)
         KIO::UDSEntry entry;
@@ -1108,7 +1115,7 @@ void KDirOperator::rereadDir()
 
 bool KDirOperator::Private::openUrl(const KUrl& url, KDirLister::OpenUrlFlags flags)
 {
-    bool result = dirLister->openUrl(url, flags);
+    const bool result = KProtocolManager::supportsListing(url) && dirLister->openUrl(url, flags);
     if (!result)   // in that case, neither completed() nor canceled() will be emitted by KDL
         _k_slotCanceled();
 
@@ -1144,6 +1151,17 @@ void KDirOperator::Private::updateSorting(QDir::SortFlags sort)
         return;
     }
 
+    if ((sorting ^ sort) & QDir::DirsFirst) {
+        // The "Folders First" setting has been changed.
+        // We need to make sure that the files and folders are really re-sorted.
+        // Without the following intermediate "fake resorting",
+        // QSortFilterProxyModel::sort(int column, Qt::SortOrder order)
+        // would do nothing because neither the column nor the sort order have been changed.
+        Qt::SortOrder tmpSortOrder = (sortOrder() == Qt::AscendingOrder ? Qt::DescendingOrder : Qt::AscendingOrder);
+        proxyModel->sort(sortOrder(), tmpSortOrder);
+        proxyModel->setSortFoldersFirst(sort & QDir::DirsFirst);
+    }
+
     sorting = sort;
     parent->updateSortActions();
     proxyModel->sort(sortColumn(), sortOrder());
@@ -1278,6 +1296,16 @@ QStringList KDirOperator::mimeFilter() const
     return d->dirLister->mimeFilters();
 }
 
+void KDirOperator::setNewFileMenuSupportedMimeTypes(const QStringList& mimeTypes)
+{
+    d->newFileMenu->setSupportedMimeTypes(mimeTypes);
+}
+
+QStringList KDirOperator::newFileMenuSupportedMimeTypes() const
+{
+    return d->newFileMenu->supportedMimeTypes();
+}
+
 bool KDirOperator::checkPreviewSupport()
 {
     KToggleAction *previewAction = static_cast<KToggleAction*>(d->actionCollection->action("preview"));
@@ -1294,9 +1322,12 @@ bool KDirOperator::checkPreviewSupport()
 void KDirOperator::activatedMenu(const KFileItem &item, const QPoint &pos)
 {
     Q_UNUSED(item);
-    setupMenu();
     updateSelectionDependentActions();
 
+    d->newFileMenu->setPopupFiles(url());
+    d->newFileMenu->setViewShowsHiddenFiles(showHiddenFiles());
+    d->newFileMenu->checkUpToDate();
+
     emit contextMenuAboutToShow( item, d->actionMenu->menu() );
 
     d->actionMenu->menu()->exec(pos);
@@ -1509,7 +1540,7 @@ void KDirOperator::setMode(KFile::Modes mode)
 
     // reset the view with the different mode
     if (d->itemView != 0)
-        setView(static_cast<KFile::FileView>(d->viewKind));
+    setView(static_cast<KFile::FileView>(d->viewKind));
 }
 
 void KDirOperator::setView(QAbstractItemView *view)
@@ -1606,8 +1637,8 @@ void KDirOperator::setView(QAbstractItemView *view)
         d->itemsToBeSetAsCurrent.clear();
     }
 
-    const bool previewShown = d->inlinePreviewState == Private::NotForced ? d->showPreviews : d->inlinePreviewState;
     const bool previewForcedToTrue = d->inlinePreviewState == Private::ForcedToTrue;
+    const bool previewShown = d->inlinePreviewState == Private::NotForced ? d->showPreviews : previewForcedToTrue;
     d->previewGenerator = new KFilePreviewGenerator(d->itemView);
     const int maxSize = KIconLoader::SizeEnormous - KIconLoader::SizeSmall;
     const int val = (maxSize * d->iconsZoom / 100) + KIconLoader::SizeSmall;
@@ -1884,6 +1915,10 @@ void KDirOperator::setupActions()
     d->actionCollection->addAction("descending", descendingAction);
     connect(descendingAction, SIGNAL(triggered(bool)), this, SLOT(_k_slotSortReversed(bool)));
 
+    KToggleAction *dirsFirstAction = new KToggleAction(i18n("Folders First"), this);
+    d->actionCollection->addAction("dirs first", dirsFirstAction);
+    connect(dirsFirstAction, SIGNAL(triggered(bool)), this, SLOT(_k_slotToggleDirsFirst()));
+
     QActionGroup* sortGroup = new QActionGroup(this);
     byNameAction->setActionGroup(sortGroup);
     bySizeAction->setActionGroup(sortGroup);
@@ -1948,6 +1983,11 @@ void KDirOperator::setupActions()
     d->actionCollection->addAction("inline preview", inlinePreview);
     connect(inlinePreview, SIGNAL(toggled(bool)), SLOT(_k_toggleInlinePreviews(bool)));
 
+    KAction *fileManager = new KAction(i18n("Open File Manager"), this);
+    d->actionCollection->addAction("file manager", fileManager);
+    fileManager->setIcon(KIcon(QLatin1String("system-file-manager")));
+    connect(fileManager, SIGNAL(triggered()), SLOT(_k_slotOpenFileManager()));
+
     action = new KAction(i18n("Properties"), this);
     d->actionCollection->addAction("properties", action);
     action->setIcon(KIcon("document-properties"));
@@ -1965,7 +2005,10 @@ void KDirOperator::setupActions()
     // TODO: QAbstractItemView does not offer an action collection. Provide
     // an interface to add a custom action collection.
 
-    d->actionCollection->addAssociatedWidget(parentWidget());
+    d->newFileMenu = new KNewFileMenu(d->actionCollection, "new", this);
+    connect(d->newFileMenu, SIGNAL(directoryCreated(KUrl)), this, SLOT(_k_slotDirectoryCreated(KUrl)));
+    
+    d->actionCollection->addAssociatedWidget(this);
     foreach (QAction* action, d->actionCollection->actions())
       action->setShortcutContext(Qt::WidgetWithChildrenShortcut);
 }
@@ -1986,6 +2029,7 @@ void KDirOperator::setupMenu(int whichActions)
     sortMenu->addAction(d->actionCollection->action("by type"));
     sortMenu->addSeparator();
     sortMenu->addAction(d->actionCollection->action("descending"));
+    sortMenu->addAction(d->actionCollection->action("dirs first"));
 
     // now plug everything into the popupmenu
     d->actionMenu->menu()->clear();
@@ -1998,7 +2042,7 @@ void KDirOperator::setupMenu(int whichActions)
     }
 
     if (whichActions & FileActions) {
-        d->actionMenu->addAction(d->actionCollection->action("mkdir"));
+        d->actionMenu->addAction(d->actionCollection->action("new"));
         if (d->currUrl.isLocalFile() && !(QApplication::keyboardModifiers() & Qt::ShiftModifier)) {
             d->actionMenu->addAction(d->actionCollection->action("trash"));
         }
@@ -2025,6 +2069,7 @@ void KDirOperator::setupMenu(int whichActions)
     }
 
     if (whichActions & FileActions) {
+        d->actionMenu->addAction(d->actionCollection->action("file manager"));
         d->actionMenu->addAction(d->actionCollection->action("properties"));
     }
 }
@@ -2041,6 +2086,7 @@ void KDirOperator::updateSortActions()
         d->actionCollection->action("by type")->setChecked(true);
     }
     d->actionCollection->action("descending")->setChecked(d->sorting & QDir::Reversed);
+    d->actionCollection->action("dirs first")->setChecked(d->sorting & QDir::DirsFirst);
 }
 
 void KDirOperator::updateViewActions()
@@ -2130,6 +2176,9 @@ void KDirOperator::writeConfig(KConfigGroup& configGroup)
     configGroup.writeEntry(QLatin1String("Sort reversed"),
                            d->actionCollection->action("descending")->isChecked());
 
+    configGroup.writeEntry(QLatin1String("Sort directories first"),
+                           d->actionCollection->action("dirs first")->isChecked());
+
     // don't save the preview when an application specific preview is in use.
     bool appSpecificPreview = false;
     if (d->preview) {
@@ -2183,7 +2232,7 @@ void KDirOperator::resizeEvent(QResizeEvent *)
 
     d->splitter->resize(size());
     sizes = d->splitter->sizes();
-    
+
     const bool restorePreviewWidth = hasPreview && (d->previewWidth != sizes[1]);
     if (restorePreviewWidth) {
         const int availableWidth = sizes[0] + sizes[1];
@@ -2329,7 +2378,7 @@ void KDirOperator::Private::_k_slotActivated(const QModelIndex& index)
 
     if (item.isDir()) {
         parent->selectDir(item);
-    } else if (!isSaving) {
+    } else {
         parent->selectFile(item);
     }
 }
@@ -2607,7 +2656,7 @@ bool KDirOperator::showHiddenFiles() const
     return d->actionCollection->action("show hidden")->isChecked();
 }
 
-QStyleOptionViewItem::Position KDirOperator::decorationPosition() const 
+QStyleOptionViewItem::Position KDirOperator::decorationPosition() const
 {
     return d->decorationPosition;
 }
@@ -2633,7 +2682,7 @@ bool KDirOperator::Private::isReadable(const KUrl& url)
 #else
     QString ts = url.path(KUrl::AddTrailingSlash);
 #endif
-    bool readable = (KDE_stat(QFile::encodeName(ts), &buf) == 0);
+    bool readable = (KDE::stat(ts, &buf) == 0);
     if (readable) { // further checks
         DIR *test;
         test = opendir(QFile::encodeName(ts));    // we do it just to test here
@@ -2644,4 +2693,9 @@ bool KDirOperator::Private::isReadable(const KUrl& url)
     return readable;
 }
 
+void KDirOperator::Private::_k_slotDirectoryCreated(const KUrl& url)
+{
+    parent->setUrl(url, true);
+}
+
 #include "kdiroperator.moc"
