@@ -27,6 +27,7 @@
 #include <QtGui/QAbstractItemDelegate>
 #include <QtGui/QKeyEvent>
 #include <QtGui/QApplication>
+#include <QtGui/QScrollBar>
 
 #include <kdebug.h>
 
@@ -159,7 +160,13 @@ void KFilePlacesViewDelegate::paint(QPainter *painter, const QStyleOptionViewIte
     painter->drawPixmap(point, pm);
 
     if (option.state & QStyle::State_Selected) {
-        painter->setPen(option.palette.highlightedText().color());
+        QPalette::ColorGroup cg = QPalette::Active;
+        if (!(option.state & QStyle::State_Enabled)) {
+            cg = QPalette::Disabled;
+        } else if (!(option.state & QStyle::State_Active)) {
+            cg = QPalette::Inactive;
+        }
+        painter->setPen(option.palette.color(cg, QPalette::HighlightedText));
     }
 
     QRect rectText;
@@ -413,6 +420,13 @@ KFilePlacesView::KFilePlacesView(QWidget *parent)
 
     d->pollDevices.setInterval(5000);
     connect(&d->pollDevices, SIGNAL(timeout()), this, SLOT(_k_triggerDevicePolling()));
+
+    // FIXME: this is necessary to avoid flashes of black with some widget styles.
+    // could be a bug in Qt (e.g. QAbstractScrollArea) or KFilePlacesView, but has not
+    // yet been tracked down yet. until then, this works and is harmlessly enough.
+    // in fact, some QStyle (Oxygen, Skulpture, others?) do this already internally.
+    // See br #242358 for more information
+    verticalScrollBar()->setAttribute(Qt::WA_OpaquePaintEvent, false);
 }
 
 KFilePlacesView::~KFilePlacesView()
@@ -680,7 +694,7 @@ void KFilePlacesView::contextMenuEvent(QContextMenuEvent *event)
             QString appName;
             if (appLocal) appName = KGlobal::mainComponent().componentName();
 
-            placesModel->addPlace(label, url, iconName, appName);
+            placesModel->addPlace(label, url, iconName, appName, index);
         }
     }
 
