@@ -612,7 +612,7 @@ KFileWidget::KFileWidget( const KUrl& _startDir, QWidget *parent )
     }
 
     d->ops->setUrl(startDir, true);
-    d->urlNavigator->setUrl(startDir);
+    d->urlNavigator->setLocationUrl(startDir);
     if (d->placesView) {
         d->placesView->setUrl(startDir);
     }
@@ -1100,7 +1100,9 @@ void KFileWidgetPrivate::_k_fileSelected(const KFileItem &i)
 
     // if we are saving, let another chance to the user before accepting the dialog (or trying to
     // accept). This way the user can choose a file and add a "_2" for instance to the filename
-    if (operationMode != KFileWidget::Saving) {
+    if (operationMode == KFileWidget::Saving) {
+        locationEdit->setFocus();
+    } else {
         q->slotOk();
     }
 }
@@ -1415,7 +1417,7 @@ void KFileWidgetPrivate::_k_urlEntered(const KUrl& url)
 
     locationEdit->blockSignals( blocked );
 
-    urlNavigator->setUrl(url);
+    urlNavigator->setLocationUrl(url);
 
     // is trigged in ctor before completion object is set
     KUrlCompletion *completion = dynamic_cast<KUrlCompletion*>(locationEdit->completionObject());
@@ -2002,9 +2004,11 @@ void KFileWidgetPrivate::_k_activateUrlNavigator()
 {
 //     kDebug(kfile_area);
 
-    urlNavigator->setUrlEditable(true);
-    urlNavigator->setFocus();
-    urlNavigator->editor()->lineEdit()->selectAll();
+    urlNavigator->setUrlEditable(!urlNavigator->isUrlEditable());
+    if(urlNavigator->isUrlEditable()) {
+        urlNavigator->setFocus();
+        urlNavigator->editor()->lineEdit()->selectAll();
+    }
 }
 
 void KFileWidgetPrivate::_k_zoomOutIconsSize()
