@@ -31,9 +31,6 @@
 #include <dom/dom_exception.h>
 #include <dom/dom_string.h>
 
-#include <misc/loader.h>
-#include <misc/htmltags.h>
-#include <misc/htmlattrs.h>
 #include <xml/dom_docimpl.h>
 
 using namespace DOM;
@@ -466,7 +463,7 @@ CSSRuleImpl* CSSRuleListImpl::item(unsigned long index)
         assert(!rule || rule->isRule());
         return static_cast<CSSRuleImpl*>(rule);
     }
-    return m_lstCSSRules.at(index);
+    return index < length() ? m_lstCSSRules.at(index) : 0;
 }
 
 void CSSRuleListImpl::deleteRule ( unsigned long index )
