@@ -25,6 +25,7 @@
 #include "css_valueimpl.h"
 #include "css_ruleimpl.h"
 #include "css_stylesheetimpl.h"
+#include "css/csshelper.h"
 #include "cssparser.h"
 #include "cssproperties.h"
 #include "cssvalues.h"
@@ -1051,11 +1052,7 @@ void CSSPrimitiveValueImpl::cleanup()
 
 int CSSPrimitiveValueImpl::computeLength( khtml::RenderStyle *style, int logicalDpiY)
 {
-    double result = computeLengthFloat( style, logicalDpiY );
-    // This conversion is imprecise, often resulting in values of, e.g., 44.99998.  We
-    // need to go ahead and round if we're really close to the next integer value.
-    int intResult = (int)(result + (result < 0 ? -0.01 : +0.01));
-    return intResult;
+    return snapValue( computeLengthFloat( style, logicalDpiY ) );
 }
 
 double CSSPrimitiveValueImpl::computeLengthFloat( khtml::RenderStyle *style, int logicalDpiY)
@@ -1370,7 +1367,7 @@ void PairImpl::setSecond(CSSPrimitiveValueImpl* second)
 
 // -----------------------------------------------------------------
 
-CSSImageValueImpl::CSSImageValueImpl(const DOMString &url, const StyleBaseImpl* style)
+CSSImageValueImpl::CSSImageValueImpl(const DOMString &url, StyleBaseImpl* style)
     : CSSPrimitiveValueImpl(url, CSSPrimitiveValue::CSS_URI)
 {
     khtml::DocLoader *docLoader = 0;
@@ -1380,7 +1377,8 @@ CSSImageValueImpl::CSSImageValueImpl(const DOMString &url, const StyleBaseImpl*
     if (root->isCSSStyleSheet())
 	docLoader = static_cast<const CSSStyleSheetImpl*>(root)->docLoader();
 
-    m_image = docLoader->requestImage(url);
+    KUrl fullURL( style->baseURL(), khtml::parseURL(url).string() );
+    m_image = docLoader->requestImage( fullURL.url() );
     if(m_image) m_image->ref(this);
 }
 
@@ -1617,7 +1615,7 @@ bool CSSFontFaceSrcValueImpl::isSupportedFormat() const
         return true;
     }
 
-    return !strcasecmp(m_format, "truetype") || !strcasecmp(m_format, "opentype")
+    return !strcasecmp(m_format, "truetype") || !strcasecmp(m_format, "opentype") || !strcasecmp(m_format, "woff")
 #if 0
     //ENABLE(SVG_FONTS)
            || isSVGFontFaceSrc()
