@@ -705,6 +705,9 @@ KJS_IMPLEMENT_PROTOTYPE("CanvasPatternProto", CanvasPatternProto, CanvasPatternF
 
 JSValue *CanvasPatternFunction::callAsFunction(ExecState *exec, JSObject *thisObj, const List &args)
 {
+    Q_UNUSED(exec);
+    Q_UNUSED(thisObj);
+    Q_UNUSED(args);
     assert(0);
     return NULL;
 }
@@ -752,6 +755,7 @@ void CanvasImageDataArray::mark()
 
 JSValue* CanvasImageDataArray::indexGetter(ExecState* exec, unsigned index)
 {
+    Q_UNUSED(exec);
     if (index >= size) // paranoia..
         return jsNull();
 
@@ -875,6 +879,10 @@ DOM::CanvasImageDataImpl* toCanvasImageData(ExecState* exec, JSValue* val)
     return id;
 }
 
+// This is completely fake!
+IMPLEMENT_PSEUDO_CONSTRUCTOR(SVGAnglePseudoCtor, "SVGAngle", Context2DProto)
+
+
 } // namespace
 
 // kate: indent-width 4; replace-tabs on; tab-width 4; space-indent on;
