@@ -3,6 +3,7 @@
  *  This file is part of the KDE libraries
  *  Copyright (C) 2000 Harri Porten (porten@kde.org)
  *  Copyright (C) 2003 Apple Computer, Inc.
+ *  Copyright (C) 2010 Maksim Orlovich (maksim@kde.org)
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -28,8 +29,7 @@
 #include <rendering/render_layer.h>
 #include <xml/dom_nodeimpl.h>
 #include <xml/dom_docimpl.h>
-#include <misc/htmltags.h> // ID_*
-#include <misc/htmlattrs.h> // ATTR_*
+#include <xml/dom3_xpathimpl.h>
 #include <html/html_baseimpl.h>
 #include <html/html_documentimpl.h>
 #include <html/html_miscimpl.h>
@@ -48,6 +48,7 @@
 #include "kjs_events.h"
 #include "kjs_views.h"
 #include "kjs_window.h"
+#include "kjs_xpath.h"
 #include "xmlhttprequest.h"
 #include <kjs/PropertyNameArray.h>
 #include <dom/dom_exception.h>
@@ -60,7 +61,7 @@ using namespace DOM;
 
 // -------------------------------------------------------------------------
 /* Source for DOMNodeConstantsTable.
-@begin DOMNodeConstantsTable 11
+@begin DOMNodeConstantsTable 21
   ELEMENT_NODE      DOM::Node::ELEMENT_NODE     DontDelete|ReadOnly
   ATTRIBUTE_NODE    DOM::Node::ATTRIBUTE_NODE       DontDelete|ReadOnly
   TEXT_NODE     DOM::Node::TEXT_NODE        DontDelete|ReadOnly
@@ -73,6 +74,7 @@ using namespace DOM;
   DOCUMENT_TYPE_NODE    DOM::Node::DOCUMENT_TYPE_NODE   DontDelete|ReadOnly
   DOCUMENT_FRAGMENT_NODE DOM::Node::DOCUMENT_FRAGMENT_NODE  DontDelete|ReadOnly
   NOTATION_NODE     DOM::Node::NOTATION_NODE        DontDelete|ReadOnly
+  XPATH_NAMESPACE_NODE	DOM::Node::XPATH_NAMESPACE_NODE         DontDelete|ReadOnly
   DOCUMENT_POSITION_DISCONNECTED                DOM::Node::DOCUMENT_POSITION_DISCONNECTED  DontDelete|ReadOnly
   DOCUMENT_POSITION_PRECEDING                   DOM::Node::DOCUMENT_POSITION_PRECEDING  DontDelete|ReadOnly
   DOCUMENT_POSITION_FOLLOWING                   DOM::Node::DOCUMENT_POSITION_FOLLOWING   DontDelete|ReadOnly
@@ -681,11 +683,13 @@ JSValue* DOMNodeProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj, co
         node.insertBefore(docFrag.get(), node.firstChild(), exception);
       else if (where == "beforeend")
         return getDOMNode(exec, node.appendChild(docFrag.get(), exception));
-      else if (where == "afterend")
-        if (node.nextSibling())
+      else if (where == "afterend") {
+        if (node.nextSibling()) {
 	  node.parentNode()->insertBefore(docFrag.get(), node.nextSibling(),exception);
-	else
+        } else {
 	  node.parentNode()->appendChild(docFrag.get(),exception);
+        }
+      }
 
       return jsUndefined();
     }
@@ -938,6 +942,11 @@ AttrImpl *toAttr(JSValue *val)
   queryCommandSupported DOMDocument::QueryCommandSupported     DontDelete|Function 1
   queryCommandValue DOMDocument::QueryCommandValue             DontDelete|Function 1
   getElementsByClassName DOMDocument::GetElementsByClassName   DontDelete|Function 1
+  querySelector          DOMDocument::QuerySelector            DontDelete|Function 1
+  querySelectorAll       DOMDocument::QuerySelectorAll         DontDelete|Function 1
+  createExpression       DOMDocument::CreateExpression         DontDelete|Function 2
+  createNSResolver       DOMDocument::CreateNSResolver         DontDelete|Function 1
+  evaluate               DOMDocument::Evaluate                 DontDelete|Function 4
 @end
 */
 
@@ -1185,6 +1194,38 @@ JSValue* DOMDocumentProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj
   }
   case DOMDocument::GetElementsByClassName:
     return getDOMNodeList(exec, doc.getElementsByClassName(s));
+  case DOMDocument::QuerySelector: {
+    RefPtr<DOM::ElementImpl> e = doc.querySelector(s, exception);
+    return getDOMNode(exec, e.get());
+  }
+  case DOMDocument::QuerySelectorAll: {
+    RefPtr<DOM::NodeListImpl> l = doc.querySelectorAll(s, exception);
+    return getDOMNodeList(exec, l.get());
+  }
+  case DOMDocument::CreateExpression: {
+    RefPtr<khtml::XPathNSResolverImpl> res = toResolver(exec, args[1]);
+    RefPtr<khtml::XPathExpressionImpl> e = doc.createExpression(s, res.get(), exception);
+    JSValue* wrapper = getWrapper<KJS::XPathExpression>(exec, e.get());
+
+    // protect the resolver if needed
+    if (!wrapper->isNull() && res && res->type() == khtml::XPathNSResolverImpl::JS) {
+      static_cast<XPathExpression*>(wrapper)->setAssociatedResolver(static_cast<JSXPathNSResolver*>(res.get())->resolverObject());
+    }
+    return wrapper;
+  }
+  case DOMDocument::CreateNSResolver: {
+    DOM::NodeImpl* node = toNode(args[0]);
+    return getWrapper<KJS::XPathNSResolver>(exec, doc.createNSResolver(node));
+  }
+  case DOMDocument::Evaluate: {
+    return getWrapper<KJS::XPathResult>(exec,
+          doc.evaluate(s, // expression
+                      toNode(args[1]), // contextNode,
+                      toResolver(exec, args[2]), // resolver
+                      args[3]->toInt32(exec), // type
+                      0, // result reuse, ignored
+                      exception));
+  }
   default:
     break;
   }
@@ -1215,6 +1256,8 @@ JSValue* DOMDocumentProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj
 # Extensions
   blur          DOMElement::Blur    DontDelete|Function 0
   focus         DOMElement::Focus   DontDelete|Function 0
+  querySelector          DOMElement::QuerySelector            DontDelete|Function 1
+  querySelectorAll       DOMElement::QuerySelectorAll         DontDelete|Function 1
 @end
 */
 KJS_IMPLEMENT_PROTOFUNC(DOMElementProtoFunc)
@@ -1279,40 +1322,9 @@ bool DOMElement::getOwnPropertySlot(ExecState *exec, const Identifier& propertyN
   if (getStaticOwnValueSlot(&DOMElementTable, this, propertyName, slot))
     return true;
 
-  // We have to check in DOMNode before giving access to attributes, otherwise
-  // onload="..." would make onload return the string (attribute value) instead of
-  // the listener object (function).
-  if (DOMNode::getOwnPropertySlot(exec, propertyName, slot))
-    return true;
-
-  //Check the prototype -before- the attribute thing
-  JSValue *proto = prototype();
-  if (proto->isObject() && static_cast<JSObject *>(proto)->getOwnPropertySlot(exec, propertyName, slot))
-    return true;
-
-#if 0 // IE-only feature
-  // Give access to attributes
-  ElementImpl &element = *static_cast<ElementImpl *>(impl());
-  if (element.hasAttribute(propertyName.domString())) {
-    slot.setCustom(this, attributeGetter);
-    return true;
-  }
-#endif
-
-  return false;
+  return DOMNode::getOwnPropertySlot(exec, propertyName, slot);
 }
 
-#if 0
-JSValue *DOMElement::attributeGetter(ExecState*, JSObject*, const Identifier& propertyName, const PropertySlot& slot)
-{
-  DOMElement *thisObj = static_cast<DOMElement *>(slot.slotBase());
-
-  ElementImpl *element = static_cast<ElementImpl *>(thisObj->impl());
-  DOM::DOMString attr = element->getAttribute(propertyName.domString());
-  return getStringOrNull(attr);
-}
-#endif
-
 JSValue* DOMElementProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj, const List &args)
 {
   KJS_CHECK_THIS( KJS::DOMNode, thisObj ); // node should be enough here, given the cast
@@ -1365,6 +1377,15 @@ JSValue* DOMElementProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj,
       return jsBoolean(element.hasAttributeNS(args[0]->toString(exec).domString(),args[1]->toString(exec).domString()));
     case DOMElement::GetElementsByClassName: // HTML 5
       return getDOMNodeList(exec, element.getElementsByClassName(args[0]->toString(exec).domString()));
+    case DOMElement::QuerySelector: { // WA Selectors 1
+      RefPtr<DOM::ElementImpl> e = element.querySelector(args[0]->toString(exec).domString(), exception);
+      return getDOMNode(exec, e.get());
+    }
+    case DOMElement::QuerySelectorAll: {  // WA Selectors 1
+      RefPtr<DOM::NodeListImpl> l = element.querySelectorAll(args[0]->toString(exec).domString(), exception);
+      return getDOMNodeList(exec, l.get());
+    }
+      
     default:
 
       // Make sure our layout is up to date before we call these
@@ -1759,10 +1780,14 @@ JSValue* KJS::getEventTarget(ExecState* exec, DOM::EventTargetImpl* t)
 {
     if (!t)
         return jsNull();
-    if (t->eventTargetType() == EventTargetImpl::DOM_NODE)
+    if (t->eventTargetType() == EventTargetImpl::DOM_NODE) {
         return getDOMNode(exec, static_cast<DOM::NodeImpl*>(t));
-    else
-        return static_cast<XMLHttpRequest*>(t); //static
+    } else if (t->eventTargetType() == EventTargetImpl::WINDOW) {
+        Window* w = static_cast<WindowEventTargetImpl*>(t)->window();
+        return w ? w : jsNull();
+    } else {
+        return static_cast<XMLHttpRequest*>(t);
+    }
 }
 
 JSValue* KJS::getDOMNode(ExecState *exec, DOM::NodeImpl* n)
@@ -1808,7 +1833,7 @@ JSValue* KJS::getDOMNode(ExecState *exec, DOM::NodeImpl* n)
       ret = new DOMProcessingInstruction(exec, static_cast<DOM::ProcessingInstructionImpl*>(n));
       break;
     case DOM::Node::COMMENT_NODE:
-      ret = new DOMCharacterData(exec, static_cast<DOM::CharacterDataImpl*>(n));
+      ret = new DOMComment(exec, static_cast<DOM::CommentImpl*>(n));
       break;
     case DOM::Node::DOCUMENT_NODE:
       if (static_cast<DOM::DocumentImpl*>(n)->isHTMLDocument())
@@ -1820,7 +1845,7 @@ JSValue* KJS::getDOMNode(ExecState *exec, DOM::NodeImpl* n)
       ret = new DOMDocumentType(exec, static_cast<DOM::DocumentTypeImpl*>(n));
       break;
     case DOM::Node::DOCUMENT_FRAGMENT_NODE:
-      ret = new DOMNode(exec, n);
+      ret = new DOMDocumentFragment(exec, static_cast<DOM::DocumentFragmentImpl*>(n));
       break;
     case DOM::Node::NOTATION_NODE:
       ret = new DOMNotation(exec, static_cast<DOM::NotationImpl*>(n));
@@ -1854,7 +1879,7 @@ IMPLEMENT_PSEUDO_CONSTRUCTOR_WITH_PARENT(NodeConstructor, "NodeConstructor", DOM
 // -------------------------------------------------------------------------
 
 /* Source for DOMExceptionProtoTable.
-@begin DOMExceptionProtoTable 15
+@begin DOMExceptionProtoTable 29
   INDEX_SIZE_ERR		DOM::DOMException::INDEX_SIZE_ERR		DontDelete|ReadOnly
   DOMSTRING_SIZE_ERR		DOM::DOMException::DOMSTRING_SIZE_ERR	DontDelete|ReadOnly
   HIERARCHY_REQUEST_ERR		DOM::DOMException::HIERARCHY_REQUEST_ERR	DontDelete|ReadOnly
@@ -1872,6 +1897,15 @@ IMPLEMENT_PSEUDO_CONSTRUCTOR_WITH_PARENT(NodeConstructor, "NodeConstructor", DOM
   INVALID_ACCESS_ERR		DOM::DOMException::INVALID_ACCESS_ERR	DontDelete|ReadOnly
   VALIDATION_ERR               DOM::DOMException::VALIDATION_ERR      DontDelete|ReadOnly
   TYPE_MISMATCH_ERR            DOM::DOMException::TYPE_MISMATCH_ERR   DontDelete|ReadOnly
+  SECURITY_ERR                 DOM::DOMException::SECURITY_ERR        DontDelete|ReadOnly            
+  NETWORK_ERR                  DOM::DOMException::NETWORK_ERR         DontDelete|ReadOnly  
+  ABORT_ERR                    DOM::DOMException::ABORT_ERR           DontDelete|ReadOnly
+  URL_MISMATCH_ERR             DOM::DOMException::URL_MISMATCH_ERR    DontDelete|ReadOnly
+  QUOTA_EXCEEDED_ERR           DOM::DOMException::QUOTA_EXCEEDED_ERR  DontDelete|ReadOnly
+  TIMEOUT_ERR                  DOM::DOMException::TIMEOUT_ERR         DontDelete|ReadOnly
+  NOT_READABLE_ERR             DOM::DOMException::NOT_READABLE_ERR    DontDelete|ReadOnly
+  DATA_CLONE_ERR               DOM::DOMException::DATA_CLONE_ERR      DontDelete|ReadOnly
+  ENCODING_ERR                 DOM::DOMException::ENCODING_ERR        DontDelete|ReadOnly
 @end
 */
 
@@ -2084,3 +2118,75 @@ JSValue* DOMTextProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj, co
   }
   return jsUndefined();
 }
+
+// -------------------------------------------------------------------------
+
+const ClassInfo DOMComment::info = { "Comment",
+                                    &DOMCharacterData::info, 0, 0 };
+/*
+@begin DOMCommentProtoTable 1
+@end
+*/
+
+KJS_DEFINE_PROTOTYPE(DOMCommentProto)
+KJS_IMPLEMENT_PROTOFUNC(DOMCommentProtoFunc)
+KJS_IMPLEMENT_PROTOTYPE("DOMComment",DOMCommentProto,DOMCommentProtoFunc,DOMCharacterDataProto)
+
+DOMComment::DOMComment(ExecState *exec, DOM::CommentImpl* d)
+ : DOMCharacterData(exec, d)
+ {
+    setPrototype(DOMCommentProto::self(exec));
+ }
+
+JSValue* DOMCommentProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj, const List &args)
+{
+  KJS_CHECK_THIS( KJS::DOMComment, thisObj );
+  return jsUndefined();
+}
+
+// -------------------------------------------------------------------------
+
+const ClassInfo DOMDocumentFragment::info = { "DocumentFragment",
+                                    &DOMNode::info, 0, 0 };
+/*
+@begin DOMDocumentFragmentProtoTable 2
+  querySelector          DOMDocumentFragment::QuerySelector            DontDelete|Function 1
+  querySelectorAll       DOMDocumentFragment::QuerySelectorAll         DontDelete|Function 1
+@end
+*/
+
+KJS_DEFINE_PROTOTYPE(DOMDocumentFragmentProto)
+KJS_IMPLEMENT_PROTOFUNC(DOMDocumentFragmentProtoFunc)
+KJS_IMPLEMENT_PROTOTYPE("DocumentFragment",DOMDocumentFragmentProto,
+                        DOMDocumentFragmentProtoFunc,DOMNodeProto)
+IMPLEMENT_PSEUDO_CONSTRUCTOR(DocumentFragmentPseudoCtor, "DocumentFragment", DOMDocumentFragmentProto)
+
+DOMDocumentFragment::DOMDocumentFragment(ExecState* exec, DOM::DocumentFragmentImpl* i)
+ : DOMNode(exec, i)
+{
+    setPrototype(DOMDocumentFragmentProto::self(exec));
+}
+
+JSValue* DOMDocumentFragmentProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj, const List &args)
+{
+  KJS_CHECK_THIS( KJS::DOMDocumentFragment, thisObj );
+  DOMExceptionTranslator exception(exec);  
+
+  DOM::NodeImpl* n = static_cast<DOMDocumentFragment*>(thisObj)->impl();
+  DOM::DOMString s = args[0]->toString(exec).domString();
+  
+  switch (id) {
+  case DOMDocumentFragment::QuerySelector: {
+    RefPtr<DOM::ElementImpl> e = n->querySelector(s, exception);
+    return getDOMNode(exec, e.get());
+  }
+  case DOMDocumentFragment::QuerySelectorAll: {
+    RefPtr<DOM::NodeListImpl> l = n->querySelectorAll(s, exception);
+    return getDOMNodeList(exec, l.get());
+  }
+  }
+  return jsUndefined();
+}
+
+
+// kate: indent-width 2; replace-tabs on; tab-width 4; space-indent on;
