@@ -30,16 +30,20 @@
 
 #include "ecma/kjs_binding.h"
 #include "ecma/kjs_dom.h"
+#include "ecma/kjs_scriptable.h"
 #include "xml/dom_nodeimpl.h"  // for NodeImpl::Id
 
 namespace KJS {
   class HTMLElement;
+  class Window;
 
   class HTMLDocument : public DOMDocument {
   public:
     HTMLDocument(ExecState *exec, DOM::HTMLDocumentImpl* d);
     JSValue* getValueProperty(ExecState *exec, int token);
+    using KJS::JSObject::getOwnPropertySlot;
     virtual bool getOwnPropertySlot(ExecState *exec, const Identifier& propertyName, PropertySlot& slot);
+    using KJS::JSObject::put;
     virtual void put(ExecState *exec, const Identifier &propertyName, JSValue* value, int attr = None);
     void putValueProperty(ExecState *exec, int token, JSValue* value, int /*attr*/);
 
@@ -49,7 +53,7 @@ namespace KJS {
            Images, Applets, Links, Forms, Layers, Anchors, Scripts, All, Clear, Open, Close,
            Write, WriteLn, GetElementsByName, GetSelection, CaptureEvents, ReleaseEvents,
            BgColor, FgColor, AlinkColor, LinkColor, VlinkColor, LastModified,
-           Height, Width, Dir, Frames, CompatMode, DesignMode };
+           Height, Width, Dir, Frames, CompatMode, DesignMode, ActiveElement };
     DOM::HTMLDocumentImpl* impl() const { return static_cast<DOM::HTMLDocumentImpl*>( m_impl.get() ); }
   private:
     static JSValue *nameGetter(ExecState *exec, JSObject*, const Identifier& name, const PropertySlot& slot);
@@ -63,8 +67,10 @@ namespace KJS {
   class HTMLElement : public DOMElement {
   public:
     HTMLElement(ExecState *exec, DOM::HTMLElementImpl* e);
+    using KJS::JSObject::getOwnPropertySlot;
     virtual bool getOwnPropertySlot(ExecState *exec, const Identifier& propertyName, PropertySlot& slot);
     JSValue* getValueProperty(ExecState *exec, int token) const;
+    using KJS::JSObject::put;
     virtual void put(ExecState *exec, const Identifier &propertyName, JSValue* value, int attr = None);
     void putValueProperty(ExecState *exec, int token, JSValue* value, int);
     virtual UString toString(ExecState *exec) const;
@@ -88,7 +94,8 @@ namespace KJS {
            LinkSheet, TitleText, MetaName, MetaHttpEquiv, MetaContent, MetaScheme,
            BaseHref, BaseTarget, IsIndexForm, IsIndexPrompt, StyleDisabled,
            StyleSheet, StyleType, StyleMedia, BodyBackground, BodyVLink, BodyText,
-           BodyLink, BodyALink, BodyBgColor,  BodyOnLoad, BodyFocus, 
+           BodyLink, BodyALink, BodyBgColor, BodyOnLoad, BodyOnBlur, BodyOnFocus,
+           BodyOnError, BodyFocus,
            FormAction, FormEncType, FormElements, FormLength, FormAcceptCharset,
            FormReset, FormTarget, FormName, FormMethod, FormSubmit, SelectAdd,
            SelectTabIndex, SelectValue, SelectSelectedIndex, SelectLength,
@@ -123,7 +130,7 @@ namespace KJS {
            ImageLowSrc, ImageWidth, ImageIsMap, ImageBorder, ImageHeight,
            ImageLongDesc, ImageSrc, ImageX, ImageY, ImageComplete, ObjectHspace, ObjectHeight, ObjectAlign,
            ObjectBorder, ObjectCode, ObjectType, ObjectVspace, ObjectArchive,
-           ObjectDeclare, ObjectForm, ObjectCodeBase, ObjectCodeType, ObjectData,
+           ObjectDeclare, ObjectForm, ObjectCodeBase, ObjectCodeType, ObjectData, ObjectGetSVGDocument,
            ObjectName, ObjectStandby, ObjectTabIndex, ObjectUseMap, ObjectWidth, ObjectContentDocument,
            ParamName, ParamType, ParamValueType, ParamValue, AppletArchive,
            AppletAlt, AppletCode, AppletWidth, AppletAlign, AppletCodeBase,
@@ -154,7 +161,7 @@ namespace KJS {
            FrameNoResize, FrameWidth, FrameHeight, IFrameLongDesc, IFrameAlign,
            IFrameFrameBorder, IFrameSrc, IFrameName, IFrameHeight,
            IFrameMarginHeight, IFrameMarginWidth, IFrameScrolling, IFrameWidth,
-           IFrameContentDocument, IFrameContentWindow,
+           IFrameContentDocument, IFrameContentWindow, IFrameGetSVGDocument,
            MarqueeStart, MarqueeStop,
            CanvasGetContext, CanvasWidth, CanvasHeight, CanvasToDataURL,
            LayerTop, LayerLeft, LayerVisibility, LayerBgColor, LayerClip, LayerDocument, LayerLayers,
@@ -170,7 +177,7 @@ namespace KJS {
 
     QString getURLArg(unsigned id) const;
 
-    static KParts::LiveConnectExtension *getLiveConnectExtension(const DOM::HTMLElementImpl &element);
+    static KParts::ScriptableExtension *getScriptableExtension(const DOM::HTMLElementImpl &element);
 
     /* Many of properties in the DOM bindings can be implemented by merely returning
       an attribute as the right type, and setting it in similar manner; or perhaps
@@ -205,6 +212,11 @@ namespace KJS {
 
     static QHash<int, const BoundPropInfo*>* s_boundPropInfo;
     static QHash<int, const BoundPropInfo*>* boundPropInfo();
+
+    // returns our window object --- may return 0.
+    KJS::Window* ourWindow() const;
+    JSValue*     getWindowListener(ExecState *exec, int eventId) const;
+    void         setWindowListener(ExecState *exec, int eventId, JSValue* val) const;
   };
 
   class HTMLCollection : public DOMObject {
@@ -213,6 +225,7 @@ namespace KJS {
     HTMLCollection(KJS::JSObject *proto, DOM::HTMLCollectionImpl* c);
     ~HTMLCollection();
     JSValue* getValueProperty(ExecState *exec, int token);
+    using KJS::JSObject::getOwnPropertySlot;
     virtual bool getOwnPropertySlot(ExecState *exec, const Identifier& propertyName, PropertySlot& slot);
 
     virtual JSValue* callAsFunction(ExecState *exec, JSObject* thisObj, const List& args);
@@ -240,7 +253,9 @@ namespace KJS {
   public:
     enum { Add };
     HTMLSelectCollection(ExecState *exec, DOM::HTMLCollectionImpl* c, DOM::HTMLSelectElementImpl* e);
+    using KJS::JSObject::getOwnPropertySlot;
     virtual bool getOwnPropertySlot(ExecState *exec, const Identifier& propertyName, PropertySlot& slot);
+    using KJS::JSObject::put;
     virtual void put(ExecState *exec, const Identifier &propertyName, JSValue* value, int attr = None);
 
     virtual const ClassInfo* classInfo() const { return &info; }
@@ -259,6 +274,7 @@ namespace KJS {
   public:
     OptionConstructorImp(ExecState *exec, DOM::DocumentImpl* d);
     virtual bool implementsConstruct() const;
+    using KJS::JSObject::construct;
     virtual JSObject* construct(ExecState *exec, const List &args);
   private:
     SharedPtr<DOM::DocumentImpl> doc;
@@ -270,6 +286,7 @@ namespace KJS {
   public:
     ImageConstructorImp(ExecState *exec, DOM::DocumentImpl* d);
     virtual bool implementsConstruct() const;
+    using KJS::JSObject::construct;
     virtual JSObject* construct(ExecState *exec, const List &args);
   private:
     SharedPtr<DOM::DocumentImpl> doc;
