@@ -29,7 +29,6 @@
 #include <xml/dom2_eventsimpl.h>
 #include <xml/dom_docimpl.h>
 #include <dom/html_document.h>
-#include <misc/htmltags.h>
 #include <html/html_documentimpl.h>
 #include <rendering/render_frames.h>
 
@@ -69,19 +68,14 @@
 #include "kjs_views.h"
 #include "kjs_audio.h"
 #include "kjs_context2d.h"
+#include "kjs_xpath.h"
+#include "kjs_scriptable.h"
 #include "xmlhttprequest.h"
 #include "xmlserializer.h"
 #include "domparser.h"
 
 #include <rendering/render_replaced.h>
 
-#if QT_VERSION < 0x040500
-  #define SCROLLBAR_WIDTH_HACK
-#include <QStyle>
-#include <QLayout>
-#include <QStyleOption>
-#endif
-
 using namespace KJS;
 using namespace DOM;
 
@@ -92,6 +86,7 @@ namespace KJS {
   public:
     History(ExecState *exec, KHTMLPart *p)
       : JSObject(exec->lexicalInterpreter()->builtinObjectPrototype()), part(p) { }
+    using KJS::JSObject::getOwnPropertySlot;
     virtual bool getOwnPropertySlot(ExecState *exec, const Identifier& propertyName, PropertySlot& slot);
     JSValue *getValueProperty(ExecState *exec, int token) const;
     virtual const ClassInfo* classInfo() const { return &info; }
@@ -106,6 +101,7 @@ namespace KJS {
   public:
     External(ExecState *exec, KHTMLPart *p)
       : JSObject(exec->lexicalInterpreter()->builtinObjectPrototype()), part(p) { }
+    using KJS::JSObject::getOwnPropertySlot;
     virtual bool getOwnPropertySlot(ExecState *exec, const Identifier& propertyName, PropertySlot& slot);
     virtual const ClassInfo* classInfo() const { return &info; }
     static const ClassInfo info;
@@ -267,6 +263,7 @@ const ClassInfo Window::info = { "Window", &DOMAbstractView::info, &WindowTable,
   print		Window::Print		DontDelete|Function 0
   addEventListener	Window::AddEventListener	DontDelete|Function 3
   removeEventListener	Window::RemoveEventListener	DontDelete|Function 3
+  getSelection  Window::GetSelection            DontDelete|Function 0
 # Normally found in prototype. Add to window object itself to make them
 # accessible in closed and cross-site windows
   valueOf       Window::ValueOf		DontEnum|DontDelete|Function 0
@@ -330,6 +327,7 @@ const ClassInfo Window::info = { "Window", &DOMAbstractView::info, &WindowTable,
 # Mozilla dom emulation ones.
   Element   Window::ElementCtor DontEnum|DontDelete
   Document  Window::DocumentCtor DontEnum|DontDelete
+  DocumentFragment Window::DocumentFragmentCtor DontEnum|DontDelete
   #this one is an alias since we don't have a separate XMLDocument
   XMLDocument Window::DocumentCtor DontEnum|DontDelete
   HTMLElement  Window::HTMLElementCtor DontEnum|DontDelete
@@ -391,7 +389,12 @@ const ClassInfo Window::info = { "Window", &DOMAbstractView::info, &WindowTable,
   HTMLCollection Window::HTMLCollectionCtor DontEnum|DontDelete
   HTMLCanvasElement Window::HTMLCanvasElementCtor DontEnum|DontDelete
   CSSStyleDeclaration Window::CSSStyleDeclarationCtor DontEnum|DontDelete
+  StyleSheet   Window::StyleSheetCtor DontEnum|DontDelete
   CanvasRenderingContext2D Window::Context2DCtor DontEnum|DontDelete
+  SVGAngle Window::SVGAngleCtor DontEnum|DontDelete
+  XPathResult Window::XPathResultCtor DontEnum|DontDelete
+  XPathExpression Window::XPathExpressionCtor DontEnum|DontDelete
+  XPathNSResolver Window::XPathNSResolverCtor DontEnum|DontDelete
 @end
 */
 KJS_IMPLEMENT_PROTOFUNC(WindowFunc)
@@ -571,14 +574,8 @@ bool Window::getOwnPropertySlot(ExecState *exec, const Identifier& propertyName,
     }
   } else if (!part) {
     // not a  KHTMLPart
-    QString rvalue;
-    KParts::LiveConnectExtension::Type rtype;
-    unsigned long robjid;
-    if (m_frame->m_liveconnect &&
-        isSafeScript(exec) &&
-        m_frame->m_liveconnect->get(0, propertyName.qstring(), rtype, robjid, rvalue))
-      return getImmediateValueSlot(this,
-                       getLiveConnectValue(m_frame->m_liveconnect, propertyName.qstring(), rtype, rvalue, robjid), slot);
+    if (isSafeScript(exec) && pluginRootGet(exec, m_frame->m_scriptable.data(), propertyName, slot))
+        return true;
 
     slot.setUndefined(this);
     return true;
@@ -646,6 +643,7 @@ KParts::ReadOnlyPart* Window::frameByIndex(unsigned i)
 
 JSValue* Window::indexGetter(ExecState *exec, unsigned index)
 {
+  Q_UNUSED(exec);
   KParts::ReadOnlyPart* frame = frameByIndex(index);
   if (frame)
     return Window::retrieve(frame);
@@ -654,6 +652,7 @@ JSValue* Window::indexGetter(ExecState *exec, unsigned index)
 
 JSValue *Window::framePartGetter(ExecState *exec, JSObject*, const Identifier& propertyName, const PropertySlot& slot)
 {
+  Q_UNUSED(exec);
   Window* thisObj = static_cast<Window*>(slot.slotBase());
   KHTMLPart *part = qobject_cast<KHTMLPart*>(thisObj->m_frame->m_part);
   KParts::ReadOnlyPart *rop = part->findFramePart( propertyName.qstring() );
@@ -755,6 +754,8 @@ JSValue* Window::getValueProperty(ExecState *exec, int token)
       return getCSSRuleConstructor(exec);
     case ElementCtor:
       return ElementPseudoCtor::self(exec);
+    case DocumentFragmentCtor:
+      return DocumentFragmentPseudoCtor::self(exec);      
     case HTMLElementCtor:
       return HTMLElementPseudoCtor::self(exec);
     case HTMLHtmlElementCtor:
@@ -871,12 +872,22 @@ JSValue* Window::getValueProperty(ExecState *exec, int token)
       return HTMLCanvasElementPseudoCtor::self(exec);
     case Context2DCtor:
       return Context2DPseudoCtor::self(exec);
+    case SVGAngleCtor:
+      return SVGAnglePseudoCtor::self(exec);
+    case XPathResultCtor:
+      return XPathResultPseudoCtor::self(exec);
+    case XPathExpressionCtor:
+      return XPathExpressionPseudoCtor::self(exec);
+    case XPathNSResolverCtor:
+       return XPathNSResolverPseudoCtor::self(exec);
     case DocumentCtor:
       return DocumentPseudoCtor::self(exec);
     case HTMLDocumentCtor:
       return HTMLDocumentPseudoCtor::self(exec);
     case CSSStyleDeclarationCtor:
         return CSSStyleDeclarationPseudoCtor::self(exec);
+    case StyleSheetCtor:
+        return DOMStyleSheetPseudoCtor::self(exec);
     case EventCtor:
       return EventConstructor::self(exec);
     case MutationEventCtor:
@@ -910,14 +921,6 @@ JSValue* Window::getValueProperty(ExecState *exec, int token)
       // match Gecko which does not subtract the scrollbars
       if (part->view()->horizontalScrollBar()->isVisible()) {
           ret += part->view()->horizontalScrollBar()->sizeHint().height();
-#ifdef SCROLLBAR_WIDTH_HACK
-          // ### hackish turnaround for a bug in QAbstractScrollArea triggered by Oxygen.
-          QStyleOption opt(0);
-          opt.init(part->view());
-          if (part->view()->style()->styleHint(QStyle::SH_ScrollView_FrameOnlyAroundContents, &opt, part->view())) {
-              ret += part->view()->style()->pixelMetric(QStyle::PM_DefaultFrameWidth)*2;
-          }
-#endif
       }
       return jsNumber(ret);
     }
@@ -929,14 +932,6 @@ JSValue* Window::getValueProperty(ExecState *exec, int token)
       // match Gecko which does not subtract the scrollbars
       if (part->view()->verticalScrollBar()->isVisible()) {
           ret += part->view()->verticalScrollBar()->sizeHint().width();
-#ifdef SCROLLBAR_WIDTH_HACK
-          // ### hackish turnaround for a bug in QAbstractScrollArea triggered by Oxygen.
-          QStyleOption opt(0);
-          opt.init(part->view());
-          if (part->view()->style()->styleHint(QStyle::SH_ScrollView_FrameOnlyAroundContents, &opt, part->view())) {
-              ret += part->view()->style()->pixelMetric(QStyle::PM_DefaultFrameWidth)*2;
-          }
-#endif
       }
       return jsNumber(ret);
     }
@@ -1087,7 +1082,7 @@ void Window::put(ExecState* exec, const Identifier &propertyName, JSValue *value
     JSObject::put( exec, propertyName, value, attr );
     return;
   }
-  
+
 
   // If we already have a variable, that's writeable w/o a getter/setter mess, just write to it.
   bool safe = isSafeScript(exec);
@@ -1235,9 +1230,8 @@ void Window::put(ExecState* exec, const Identifier &propertyName, JSValue *value
     }
     }
   }
-  if (m_frame->m_liveconnect &&
-      isSafeScript(exec) &&
-      m_frame->m_liveconnect->put(0, propertyName.qstring(), value->toString(exec).qstring()))
+  if (isSafeScript(exec) &&
+      pluginRootPut(exec, m_frame->m_scriptable.data(), propertyName, value))
     return;
   if (safe) {
     //kDebug(6070) << "Window("<<this<<")::put storing " << propertyName.qstring();
@@ -1425,6 +1419,7 @@ JSLazyEventListener *Window::getJSLazyEventListener(const QString& code, const Q
 
 void Window::clear( ExecState *exec )
 {
+  Q_UNUSED(exec);
   delete winq;
   winq = 0L;
   // Get rid of everything, those user vars could hold references to DOM nodes
@@ -1600,7 +1595,7 @@ JSValue *Window::openWindow(ExecState *exec, const List& args)
   // Always permit opening in an exist frame (including _self, etc.)
   if ( targetIsExistingWindow( part, frameName ) )
     policy = KHTMLSettings::KJSWindowOpenAllow;
-    
+
   if ( policy == KHTMLSettings::KJSWindowOpenAsk ) {
     emit part->browserExtension()->requestFocus(part);
     QString caption;
@@ -1948,6 +1943,9 @@ JSValue *WindowFunc::callAsFunction(ExecState *exec, JSObject *thisObj, const Li
     }
     return jsUndefined();
   }
+  case Window::GetSelection:
+    return new KJS::DOMSelection(exec, part->xmlDocImpl());
+
   case Window::Navigate:
     window->goURL(exec, args[0]->toString(exec).qstring(), false /*don't lock history*/);
     return jsUndefined();
@@ -2123,7 +2121,10 @@ JSValue *WindowFunc::callAsFunction(ExecState *exec, JSObject *thisObj, const Li
         JSEventListener *listener = Window::retrieveActive(exec)->getJSEventListener(args[1]);
         if (listener) {
 	    DOM::DocumentImpl* docimpl = static_cast<DOM::DocumentImpl *>(part->document().handle());
-            docimpl->addWindowEventListener(EventName::fromString(args[0]->toString(exec).domString()),listener,args[2]->toBoolean(exec));
+	    if (docimpl)
+		docimpl->addWindowEventListener(EventName::fromString(args[0]->toString(exec).domString()),listener,args[2]->toBoolean(exec));
+	    else
+		kWarning() << "document missing on Window::AddEventListener. why?";
         }
         return jsUndefined();
     }
@@ -2131,7 +2132,10 @@ JSValue *WindowFunc::callAsFunction(ExecState *exec, JSObject *thisObj, const Li
         JSEventListener *listener = Window::retrieveActive(exec)->getJSEventListener(args[1]);
         if (listener) {
 	    DOM::DocumentImpl* docimpl = static_cast<DOM::DocumentImpl *>(part->document().handle());
-            docimpl->removeWindowEventListener(EventName::fromString(args[0]->toString(exec).domString()),listener,args[2]->toBoolean(exec));
+	    if (docimpl)
+		docimpl->removeWindowEventListener(EventName::fromString(args[0]->toString(exec).domString()),listener,args[2]->toBoolean(exec));
+	    else
+		kWarning() << "document missing on Window::RemoveEventListener. why?";
         }
         return jsUndefined();
     }
@@ -2687,7 +2691,7 @@ UString Location::toString(ExecState *exec) const
       if (url.isEmpty())
         return "about:blank";
       else if (!url.hasPath())
-        return url.prettyUrl()+'/';
+        return QString(url.prettyUrl()+'/');
       else
         return url.prettyUrl();
     }
@@ -2828,7 +2832,7 @@ bool History::getOwnPropertySlot(ExecState *exec, const Identifier &p, PropertyS
 
 JSValue *History::getValueProperty(ExecState *, int token) const
 {
-  // if previous or next is implemented, make sure its not a major
+  // if previous or next is implemented, make sure it is not a major
   // privacy leak (see i.e. http://security.greymagic.com/adv/gm005-op/)
   switch (token) {
   case Length:
@@ -2903,3 +2907,4 @@ JSValue *HistoryFunc::callAsFunction(ExecState *exec, JSObject *thisObj, const L
 
 
 #include "kjs_window.moc"
+// kate: indent-width 4; replace-tabs on; tab-width 4; space-indent on;
