@@ -35,8 +35,6 @@
 #include "html_documentimpl.h"
 #include "htmltokenizer.h"
 
-#include <misc/htmlhashes.h>
-
 #include <khtmlview.h>
 #include <khtml_part.h>
 
@@ -110,8 +108,7 @@ bool HTMLElementImpl::isInline() const
 
 DOMString HTMLElementImpl::namespaceURI() const
 {
-    return (!m_htmlCompat) ?
-        DOMString(XHTML_NAMESPACE) : DOMString();
+    return DOMString(XHTML_NAMESPACE);
 }
 
 void HTMLElementImpl::parseAttribute(AttributeImpl *attr)
@@ -508,6 +505,13 @@ DocumentFragment HTMLElementImpl::createContextualFragment( const DOMString &htm
 
 void HTMLElementImpl::setInnerHTML( const DOMString &html, int &exceptioncode )
 {
+    if (id() == ID_SCRIPT || id() == ID_STYLE) {
+        // Script and CSS source shouldn't be parsed as HTML.
+        removeChildren();
+        appendChild(document()->createTextNode(html), exceptioncode);
+        return;
+    }
+
     DocumentFragment fragment = createContextualFragment( html );
     if ( fragment.isNull() ) {
         exceptioncode = DOMException::NO_MODIFICATION_ALLOWED_ERR;
@@ -654,6 +658,12 @@ HTMLGenericElementImpl::HTMLGenericElementImpl(DocumentImpl *doc, ushort i)
     m_localName = LocalName::fromId(i);
 }
 
+HTMLGenericElementImpl::HTMLGenericElementImpl(DocumentImpl *doc, LocalName l)
+    : HTMLElementImpl(doc),
+      m_localName(l)
+{}
+
+
 HTMLGenericElementImpl::~HTMLGenericElementImpl()
 {
 }
