@@ -468,12 +468,38 @@ public:
     // on the CFI itself determines the lifetime
 
     CachedFontInstance* queryFont(int pixelSize);
+    void invalidateAllInstances();
+    void markAllInstancesAsValid();
 };
 
 static QHash<CachedFontFamilyKey, CachedFontFamily*>* fontCache;
 // This is a hash and not a cache since the top-level info is tiny,
 // and the actual font instances have precise lifetime
 
+void Font::invalidateCachedFontFamily( const QString& familyName ) // static
+{
+    if (!fontCache)
+        return;
+    QHash<CachedFontFamilyKey, CachedFontFamily*>::const_iterator i;
+    QHash<CachedFontFamilyKey, CachedFontFamily*>::const_iterator end = fontCache->constEnd();
+    for (i = fontCache->constBegin(); i != end; ++i) {
+        if (i.key().family.contains( familyName, Qt::CaseInsensitive )) {
+            i.value()->invalidateAllInstances();
+        }
+    }
+}
+
+void Font::markAllCachedFontsAsValid() // static
+{
+    if (!fontCache)
+        return;
+    QHash<CachedFontFamilyKey, CachedFontFamily*>::const_iterator i;
+    QHash<CachedFontFamilyKey, CachedFontFamily*>::const_iterator end = fontCache->constEnd();
+    for (i = fontCache->constBegin(); i != end; ++i) {
+            i.value()->markAllInstancesAsValid();
+    }
+}
+
 CachedFontFamily* Font::queryFamily(const QString& name, int weight, bool italic)
 {
     if (!fontCache)
@@ -516,8 +542,26 @@ CachedFontInstance* CachedFontFamily::queryFont(int pixelSize)
     return cfi;
 }
 
+void CachedFontFamily::invalidateAllInstances()
+{
+    QHash<int, CachedFontInstance*>::const_iterator i;
+    QHash<int, CachedFontInstance*>::const_iterator end = instances.constEnd();
+    for (i = instances.constBegin(); i != end; ++i) {
+        i.value()->invalidate();
+    }
+}
+
+void CachedFontFamily::markAllInstancesAsValid()
+{
+    QHash<int, CachedFontInstance*>::const_iterator i;
+    QHash<int, CachedFontInstance*>::const_iterator end = instances.constEnd();
+    for (i = instances.constBegin(); i != end; ++i) {
+        i.value()->invalidated = false;
+    }
+}
+
 CachedFontInstance::CachedFontInstance(CachedFontFamily* p, int sz):
-    f(p->def.family), fm(f), parent(p), size(sz)
+    f(p->def.family), fm(f), invalidated(false), parent(p), size(sz)
 {
     f.setItalic(p->def.italic);
     f.setWeight(p->def.weight);
@@ -534,6 +578,28 @@ CachedFontInstance::CachedFontInstance(CachedFontFamily* p, int sz):
     lineSpacing = fm.lineSpacing();
 }
 
+void CachedFontInstance::invalidate()
+{
+    QFont nf( f.family() );
+    nf.setWeight( f.weight() );
+    nf.setItalic( f.italic() );
+    nf.setPixelSize( f.pixelSize() );
+    f = nf;
+    invalidated = true;
+    fm = QFontMetrics(f);
+
+    // Cleanup metrics caches
+    for (int c = 0; c < 256; ++c) {
+        delete rows[c];
+        rows[c] = 0;
+    }
+
+    ascent  = fm.ascent();
+    descent = fm.descent();
+    height  = fm.height();
+    lineSpacing = fm.lineSpacing();
+}
+
 CachedFontInstance::~CachedFontInstance()
 {
     for (int c = 0; c < 256; ++c)
@@ -652,7 +718,7 @@ void Font::drawDecoration(QPainter *pt, int _tx, int _ty, int baseline, int widt
 }
 
 // WebCore SVG
-float Font::floatWidth(QChar* str, int pos, int len, int extraCharsAvailable, int& charsConsumed, DOM::DOMString& glyphName) const
+float Font::floatWidth(QChar* str, int pos, int len, int /*extraCharsAvailable*/, int& charsConsumed, DOM::DOMString& glyphName) const
 {
     charsConsumed = len;
     glyphName = "";
@@ -660,6 +726,15 @@ float Font::floatWidth(QChar* str, int pos, int len, int extraCharsAvailable, in
     return width(str, 0, pos, len, false /*fast algorithm*/);
 }
 
+float Font::floatWidth(QChar* str, int pos, int len) const
+{
+    // For now, this is slow but correct...
+    // or rather it /would/ be correct if QFontMetricsF had charWidth(); 
+    // so instead the approximate width is used
+    QFontMetricsF fm(cfi->f);
+    return float(fm.width(QString::fromRawData(str + pos, len)));
+}
+
 }
 
 // kate: indent-width 4; replace-tabs off; tab-width 8; space-indent on; hl c++;
