@@ -60,17 +60,13 @@
 #include <QtGui/QStyle>
 #include <QStyleOptionButton>
 #include <QtGui/QLabel>
-#if QT_VERSION < 0x040500
-  #include <QtGui/QStyleOptionFrameV2>
-#else
-  #include <QtGui/QStyleOptionFrameV3>
-#endif
+#include <QtGui/QStyleOptionFrameV3>
+#include <QtGui/QStandardItemModel>
 
 #include <misc/helper.h>
 #include <xml/dom2_eventsimpl.h>
 #include <html/html_formimpl.h>
 #include <html/html_miscimpl.h>
-#include <misc/htmlhashes.h>
 
 #include <assert.h>
 
@@ -92,12 +88,20 @@ using namespace DOM;
     {
         KHTMLProxyStyle(QWidget *parent)
             : KdeUiProxyStyle(parent)
-        { noBorder = false; left = right = top = bottom = 0; m_style = parent->style(); }
+        {
+            noBorder = false;
+            left = right = top = bottom = 0;
+            m_proxy = qobject_cast<KdeUiProxyStyle*>(parent->style());
+            setParent(parent);
+        }
+
+        QStyle* proxy() const { return m_proxy ? m_proxy : style(); }
+
         QRect subElementRect(
                 SubElement element, const QStyleOption *option, const QWidget *widget
             ) const
         {
-            QRect r = m_style->subElementRect(element, option, widget);
+            QRect r = proxy()->subElementRect(element, option, widget);
             switch (element) {
               case QStyle::SE_PushButtonContents:
               case QStyle::SE_LineEditContents:
@@ -114,7 +118,7 @@ using namespace DOM;
                const QStyleOptionButton *o = qstyleoption_cast<const QStyleOptionButton *>(option);
                if (o) {
                    QStyleOptionButton opt = *o;
-                   opt.rect = m_style->subElementRect(SE_PushButtonFocusRect, &opt, widget);
+                   opt.rect = proxy()->subElementRect(SE_PushButtonFocusRect, &opt, widget);
                    KdeUiProxyStyle::drawControl(CE_PushButtonLabel, &opt, painter, widget);
                }
                return;
@@ -128,13 +132,13 @@ using namespace DOM;
         {
             // Make sure we give combo popup's enough room to display contents;
             // Qt doesn't do this by default
-            
+
             if (cc == QStyle::CC_ComboBox && sc == SC_ComboBoxListBoxPopup) {
                 const QComboBox* cb = qobject_cast<const QComboBox*>(widget);
                 const QStyleOptionComboBox* cbOpt = qstyleoption_cast<const QStyleOptionComboBox*>(opt);
-                
+
                 QFontMetrics fm = cb->fontMetrics();
-                
+
                 if (cb && cbOpt) {
                     // Compute content width; Qt uses the usual +4 magic number for icon/text margin
                     int maxW = 0;
@@ -144,25 +148,25 @@ using namespace DOM;
                             iw += 4 + cb->iconSize().width();
                         maxW = qMax(maxW, iw);
                     }
-                    
+
                     // Now let sizeFromContent add in extra stuff.
-                    maxW = m_style->sizeFromContents(QStyle::CT_ComboBox, opt, QSize(maxW, 1), widget).width();
-                    
+                    maxW = proxy()->sizeFromContents(QStyle::CT_ComboBox, opt, QSize(maxW, 1), widget).width();
+
                     // How much more room do we need for the text?
                     int extraW = maxW > cbOpt->rect.width() ? maxW - cbOpt->rect.width() : 0;
-                    
-                    QRect r = m_style->subControlRect(cc, opt, sc, widget);
+
+                    QRect r = proxy()->subControlRect(cc, opt, sc, widget);
                     r.setWidth(r.width() + extraW);
                     return r;
                 }
             }
-            
-            return m_style->subControlRect(cc, opt, sc, widget);
+
+            return proxy()->subControlRect(cc, opt, sc, widget);
         }
 
         int left, right, top, bottom;
         bool noBorder;
-        QStyle* m_style;
+        KdeUiProxyStyle* m_proxy;
     };
 
 // ---------------------------------------------------------------------
@@ -179,11 +183,7 @@ RenderFormElement::RenderFormElement(HTMLGenericFormElementImpl *element)
 }
 
 RenderFormElement::~RenderFormElement()
-{
-    if (widget())
-        widget()->setStyle(0);
-    delete m_proxyStyle;
-}
+{}
 
 void RenderFormElement::setStyle(RenderStyle *style)
 {
@@ -195,7 +195,7 @@ void RenderFormElement::calcMinMaxWidth()
 {
     // Some form widgets apply the padding internally (i.e. as if they were
     // some kind of inline-block). Thus we only want to expose that padding
-    // while layouting (so that width/height calculations are correct), and 
+    // while layouting (so that width/height calculations are correct), and
     // then pretend it does not exist, as it is beyond the replaced edge and
     // thus should not affect other calculations.
     m_exposeInternalPadding = true;
@@ -254,6 +254,10 @@ short RenderFormElement::baselinePosition( bool f ) const
 
 void RenderFormElement::setQWidget( QWidget *w )
 {
+    // Avoid dangling proxy pointer when we switch widgets.
+    // the widget will cleanup the proxy, as it is its kid.
+    m_proxyStyle = 0;
+
     // sets the Qt Object Name for the purposes
     // of setPadding() -- this is because QStyleSheet
     // will propagate children of 'w' even if they are toplevel, like
@@ -266,10 +270,10 @@ void RenderFormElement::updateFromElement()
 {
     m_widget->setEnabled(!element()->disabled());
 
-    // If we've disabled/made r/o a focused element, clear its focus,
+    // If we've disabled a focused element, clear its focus,
     // so Qt doesn't do funny stuff like let one type into a disabled
     // line edit.
-    if ((element()->disabled() || element()->readOnly()) && element()->focused())
+    if (element()->disabled() && element()->focused())
         document()->quietResetFocus();
 
     RenderWidget::updateFromElement();
@@ -334,7 +338,7 @@ void RenderButton::layout()
 {
     RenderFormElement::layout();
     bool needsTextIndentHack = false;
-    if (!style()->width().isVariable()) {
+    if (!style()->width().isAuto()) {
         // check if we need to simulate the effect of a popular
         // button text hiding 'trick' that makes use of negative text-indent,
         // which we do not support on form widgets.
@@ -352,7 +356,7 @@ void RenderButton::layout()
 void RenderButton::setStyle(RenderStyle *style)
 {
     RenderFormElement::setStyle(style);
-    if (shouldPaintBorder()) {
+    if (shouldDisableNativeBorders()) {
         // we paint the borders ourselves on this button,
         // remove the widget's native ones.
         KHTMLProxyStyle* style = static_cast<KHTMLProxyStyle*>(getProxyStyle());
@@ -486,6 +490,7 @@ bool RenderRadioButton::handleEvent(const DOM::EventImpl& ev)
 
 // -------------------------------------------------------------------------------
 
+static const QString &sBorderNoneSheet = KGlobal::staticQString("QPushButton{border:none}");
 
 RenderSubmitButton::RenderSubmitButton(HTMLInputElementImpl *element)
     : RenderButton(element)
@@ -496,6 +501,59 @@ RenderSubmitButton::RenderSubmitButton(HTMLInputElementImpl *element)
     p->setMouseTracking(true);
 }
 
+static inline void setStyleSheet_helper(const QString& s, QWidget* w)
+{
+    // ### buggy Qt stylesheets mess with the widget palette.
+    // force it again after any stylesheet update.
+    QPalette pal = w->palette();
+    w->setStyleSheet(s);
+    w->setPalette(pal);
+}
+
+void RenderSubmitButton::setPadding()
+{
+    // Proxy styling doesn't work well enough for buttons.
+    // Use stylesheets instead. tests/css/button-padding-top.html
+    assert(!m_proxyStyle);
+
+    if (!includesPadding())
+        return;
+
+    if (!RenderWidget::paddingLeft() && !RenderWidget::paddingRight() &&
+        !RenderWidget::paddingTop() && !RenderWidget::paddingBottom()) {
+        setStyleSheet_helper( (shouldDisableNativeBorders() ? sBorderNoneSheet : QString()), widget() );
+        return;
+    }
+
+    setStyleSheet_helper(
+        QString("QPushButton{padding-left:%1px; padding-right:%2px; padding-top:%3px; padding-bottom:%4px}")
+            .arg( RenderWidget::paddingLeft() )
+            .arg( RenderWidget::paddingRight() )
+            .arg( RenderWidget::paddingTop() )
+            .arg( RenderWidget::paddingBottom()) + (shouldDisableNativeBorders() ? sBorderNoneSheet : QString())
+     , widget());
+}
+
+void RenderSubmitButton::setStyle(RenderStyle *style)
+{
+    // Proxy styling doesn't work well enough for buttons.
+    // Use stylesheets instead. tests/css/button-padding-top.html
+    assert(!m_proxyStyle);
+    RenderFormElement::setStyle(style);
+
+    QString s = widget()->styleSheet();
+    if (shouldDisableNativeBorders()) {
+        // we paint the borders ourselves on this button,
+        // remove the widget's native ones.
+        if (!s.contains(sBorderNoneSheet)) {
+            s.append(sBorderNoneSheet);
+            setStyleSheet_helper( s, widget() );
+        }
+    } else {
+        setStyleSheet_helper( s.remove(sBorderNoneSheet), widget() );
+    }
+}
+
 QString RenderSubmitButton::rawText()
 {
     QString value = element()->valueWithDefault().string();
@@ -511,7 +569,7 @@ QString RenderSubmitButton::rawText()
 
 bool RenderSubmitButton::canHaveBorder() const
 {
-    // ### TODO would be nice to be able to 
+    // ### TODO would be nice to be able to
     // return style()->hasBackgroundImage() here,
     // depending on a config option (e.g. 'favour usability/integration over aspect')
     // so that only buttons with both a custom border
@@ -555,21 +613,23 @@ void RenderSubmitButton::calcMinMaxWidth()
     }
 
     assert(includesPadding());
-    int padding = RenderWidget::paddingLeft() + RenderWidget::paddingRight();
+    int hpadding = RenderWidget::paddingLeft() + RenderWidget::paddingRight();
+    int vpadding = RenderWidget::paddingTop() + RenderWidget::paddingBottom();
 
     // add 30% margins to the width (heuristics to make it look similar to IE)
     // ### FIXME BASELINE: we could drop this emulation and adopt Mozilla style buttons
     // (+/- padding: 0px 8px 0px 8px) - IE is most often in a separate css
     // code path nowadays, so we have wider buttons than other engines.
-    int toAdd = (w*13/10)-w-padding;
+    int toAdd = (w*13/10)-w-hpadding;
     toAdd = qMax(0,toAdd);
     w += toAdd;
 
-    if (shouldPaintBorder()) {
+    if (shouldDisableNativeBorders()) {
         // we paint the borders ourselves, so let's override our height to something saner
         h = ts.height();
+    } else {
+        h -= vpadding;
     }
-
     s = QSize(w,h).expandedTo(QApplication::globalStrut());
 
     setIntrinsicWidth( s.width() );
@@ -735,21 +795,21 @@ bool WebShortcutCreator::createWebShortcut(QString query)
             isOk = true;
             QStringList keyList( keys.split( ',' ) );
             KService::List providers = KServiceTypeTrader::self()->query( "SearchProvider" );
-            foreach ( KService::Ptr provider, providers ) {
+            foreach ( const KService::Ptr &provider, providers ) {
                 if ( !isOk ) {
                     break;
                 }
-                foreach ( QString s, provider->property( "Keys" ).toStringList() ) {
+                foreach ( const QString &s, provider->property( "Keys" ).toStringList() ) {
                     if ( !isOk ) {
                         break;
                     }
-                    foreach ( QString t, keys ) {
+                    foreach ( const QString &t, keys ) {
                         if ( !isOk ) {
                             break;
                         }
                         if ( s == t ) {
                             KMessageBox::sorry( 0, i18n( "%1 is already assigned to %2", s, provider->name() ), i18n( "Error" ) );
-                            isOk = false; 
+                            isOk = false;
                         }
                     }
                 }
@@ -834,7 +894,7 @@ void LineEditWidget::slotCreateWebShortcut()
     for( unsigned long i = 0; ( node = form->elements()->item( i ) ); i++ ) {
         inputNode = dynamic_cast<HTMLInputElementImpl*>( node );
         if ( inputNode ) {
-            if ( ( !inputNode->name().string().size() ) || 
+            if ( ( !inputNode->name().string().size() ) ||
                  (inputNode->name().string() == queryName) ) {
                 continue;
             } else {
@@ -911,7 +971,7 @@ void LineEditWidget::paintEvent( QPaintEvent *pe )
         QPainter p(this);
         p.fillRect(pe->rect(), palette().brush(QPalette::Base));
         p.end();
-    }   
+    }
     KLineEdit::paintEvent( pe );
 }
 
@@ -952,12 +1012,11 @@ void LineEditWidget::mouseMoveEvent(QMouseEvent *e)
 // -----------------------------------------------------------------------------
 
 RenderLineEdit::RenderLineEdit(HTMLInputElementImpl *element)
-    : RenderFormElement(element), m_blockElementUpdates(false)
+    : RenderFormElement(element)
 {
     LineEditWidget *edit = new LineEditWidget(element, view(), view()->widget());
     connect(edit,SIGNAL(returnPressed()), this, SLOT(slotReturnPressed()));
-    connect(edit,SIGNAL(textChanged(const QString &)),this,SLOT(slotTextChanged(const QString &)));
-    connect(edit->completionBox(),SIGNAL(currentTextChanged(const QString &)),this,SLOT(slotCompletionBoxActivated(const QString &)));
+    connect(edit,SIGNAL(textEdited(QString)),this,SLOT(slotTextEdited(QString)));
 
     if(element->inputType() == HTMLInputElementImpl::PASSWORD)
         edit->setEchoMode( QLineEdit::Password );
@@ -990,13 +1049,22 @@ void RenderLineEdit::setStyle(RenderStyle* _style)
     RenderFormElement::setStyle( _style );
 
     widget()->setAlignment(textAlignment());
-    bool showClearButton = (!shouldPaintBorder() && !_style->hasBackgroundImage());
+    bool showClearButton = (!shouldDisableNativeBorders() && !_style->hasBackgroundImage());
     static_cast<LineEditWidget*>(widget())->setClearButtonShown( showClearButton );
     if (showClearButton) {
-        QList<QWidget *> wl = qFindChildren<QWidget *>(m_widget, QString());
-        foreach (QWidget* w, wl)
-            if (!w->isWindow())
+        QObjectList children = widget()->children();
+        foreach (QObject* object, children) {
+            QWidget *w = qobject_cast<QWidget*>(object);
+            if (w && !w->isWindow()) {
+                // this duplicates KHTMLView's handleWidget but this widget
+                // is created on demand, so it might not be here at ChildPolished time
                 w->setObjectName("KHTMLLineEditButton");
+                w->installEventFilter(view());
+                w->setAttribute(Qt::WA_NoSystemBackground);
+                w->setAttribute(Qt::WA_WState_InPaintEvent);
+                w->setAttribute(Qt::WA_OpaquePaintEvent);
+            }
+        }
     }
 }
 
@@ -1074,32 +1142,22 @@ void RenderLineEdit::updateFromElement()
     }
 
     if (element()->value().string() != widget()->text()) {
-        m_blockElementUpdates = true;
         int pos = widget()->cursorPosition();
         widget()->setText(element()->value().string());
         widget()->setCursorPosition(pos);
-        m_blockElementUpdates = false;
     }
     widget()->setReadOnly(element()->readOnly());
 
     RenderFormElement::updateFromElement();
 }
 
-void RenderLineEdit::slotTextChanged(const QString &string)
+void RenderLineEdit::slotTextEdited(const QString &string)
 {
-    if (m_blockElementUpdates) return;
-
     // don't use setValue here!
     element()->m_value = string;
     element()->m_unsubmittedFormChange = true;
 }
 
-void RenderLineEdit::slotCompletionBoxActivated(const QString &/*string*/)
-{
-    // TODO: See todo in KLineEdit::setCompletionBox()
-    widget()->setModified( true );
-}
-
 void RenderLineEdit::select()
 {
     static_cast<LineEditWidget*>(m_widget)->selectAll();
@@ -1592,6 +1650,20 @@ RenderSelect::RenderSelect(HTMLSelectElementImpl *element)
     }
 }
 
+void RenderSelect::clearItemFlags(int index, Qt::ItemFlags flags)
+{
+    if(m_useListBox) {
+        QListWidgetItem* item = static_cast<KListWidget*>(m_widget)->item(index);
+        item->setFlags(item->flags() & ~flags);
+    } else {
+        KComboBox* combo = static_cast<KComboBox*>(m_widget);
+        if (QStandardItemModel* model = qobject_cast<QStandardItemModel*>(combo->model())) {
+            QStandardItem* item = model->item(index);
+            item->setFlags(item->flags() & ~flags);
+        }
+    }
+}
+
 void RenderSelect::updateFromElement()
 {
     m_ignoreSelectEvents = true;
@@ -1630,10 +1702,8 @@ void RenderSelect::updateFromElement()
         const QVector<HTMLGenericFormElementImpl*> listItems = element()->listItems();
         int listIndex;
 
-        if(m_useListBox) {
+        if(m_useListBox)
             static_cast<KListWidget*>(m_widget)->clear();
-        }
-
         else
             static_cast<KComboBox*>(m_widget)->clear();
 
@@ -1643,45 +1713,53 @@ void RenderSelect::updateFromElement()
                 if (text.isNull())
                     text = "";
 
+                text = text.implementation()->collapseWhiteSpace(false, false);
+
                 if(m_useListBox) {
                     QListWidgetItem *item = new QListWidgetItem(QString(text.implementation()->s, text.implementation()->l));
-                    static_cast<KListWidget*>(m_widget)
-                        ->insertItem(listIndex,item);
-                    item->setFlags(item->flags() & ~Qt::ItemIsSelectable);
+                    static_cast<KListWidget*>(m_widget)->insertItem(listIndex,item);
+                } else {
+                    static_cast<KComboBox*>(m_widget)->insertItem(listIndex, QString(text.implementation()->s, text.implementation()->l));
                 }
-                else {
-                    static_cast<KComboBox*>(m_widget)
-                        ->insertItem(listIndex, QString(text.implementation()->s, text.implementation()->l));
-#ifdef __GNUC__
-  #warning "This needs fixing (though did it work in 3?)"
-#endif
-#if 0
-		    static_cast<KComboBox*>(m_widget)->listBox()->item(listIndex)->setSelectable(false);
-#endif
-		}
+
+                bool disabled = !listItems[listIndex]->getAttribute(ATTR_DISABLED).isNull();
+                if (disabled)
+                    clearItemFlags(listIndex, Qt::ItemIsSelectable | Qt::ItemIsEnabled);
+                else
+                    clearItemFlags(listIndex, Qt::ItemIsSelectable);
             }
             else if (listItems[listIndex]->id() == ID_OPTION) {
                 HTMLOptionElementImpl* optElem = static_cast<HTMLOptionElementImpl*>(listItems[listIndex]);
-                QString text = optElem->text().string();
-                if (optElem->parentNode()->id() == ID_OPTGROUP)
-                {
-                    // Prefer label if set
-                    DOMString label = optElem->getAttribute(ATTR_LABEL);
-                    if (!label.isEmpty())
-                        text = label.string();
-                    text = QLatin1String("    ")+text;
+
+                DOMString domText = optElem->text();
+                // Prefer label if set
+                DOMString label = optElem->getAttribute(ATTR_LABEL);
+                if (!label.isEmpty())
+                    domText = label.string();
+                domText = domText.implementation()->collapseWhiteSpace(false, false);
+
+                QString   text;
+
+                ElementImpl* parentOptGroup = optElem->parentNode()->id() == ID_OPTGROUP ?
+                                                 static_cast<ElementImpl*>(optElem->parentNode()) : 0;
+
+                if (parentOptGroup) {
+                    text = QLatin1String("    ") + domText.string();
+                } else {
+                    text = domText.string();
                 }
 
-                if(m_useListBox) {
-                    KListWidget *l = static_cast<KListWidget*>(m_widget);
-                    l->insertItem(listIndex,text);
-                    DOMString disabled = optElem->getAttribute(ATTR_DISABLED);
-                    if (!disabled.isNull() && l->item( listIndex )) {
-                        l->item( listIndex )->setFlags( l->item(listIndex)->flags()
-                                                            & ~Qt::ItemIsSelectable );
-                    }
-                }  else
+                if(m_useListBox)
+                    static_cast<KListWidget*>(m_widget)->insertItem(listIndex,text);
+                else
                     static_cast<KComboBox*>(m_widget)->insertItem(listIndex, text);
+
+                bool disabled = !optElem->getAttribute(ATTR_DISABLED).isNull();
+                if (parentOptGroup)
+                    disabled = disabled || !parentOptGroup->getAttribute(ATTR_DISABLED).isNull();
+
+                if (disabled)
+                    clearItemFlags(listIndex, Qt::ItemIsSelectable | Qt::ItemIsEnabled);
             }
             else
                 KHTMLAssert(false);
@@ -1781,20 +1859,12 @@ void RenderSelect::layout( )
         if(size < 1)
             size = qMin(w->count(), 10);
 
-#if QT_VERSION < 0x040500
-        QStyleOptionFrameV2 opt;
-        opt.initFrom(w);
-        opt.lineWidth = w->lineWidth();
-        opt.midLineWidth = w->midLineWidth();
-        QRect r = w->style()->subElementRect(QStyle::SE_FrameContents, &opt, w);
-#else
         QStyleOptionFrameV3 opt;
         opt.initFrom(w);
         opt.lineWidth = w->lineWidth();
         opt.midLineWidth = w->midLineWidth();
         opt.frameShape = w->frameShape();
         QRect r = w->style()->subElementRect(QStyle::SE_ShapedFrameContents, &opt, w);
-#endif
         QRect o = opt.rect;
         int hfw = (r.left()-o.left()) + (o.right()-r.right());
         int vfw = (r.top()-o.top()) + (o.bottom()-r.bottom());
@@ -1803,7 +1873,7 @@ void RenderSelect::layout( )
         // FIXME BASELINE: the 3 lines below could be removed.
         int lhs = m_widget->style()->pixelMetric(QStyle::PM_LayoutHorizontalSpacing);
         if (lhs>0)
-           width += lhs;        
+           width += lhs;
         height = size*height + vfw;
 
         assert( includesPadding() );
@@ -1986,16 +2056,15 @@ TextAreaWidget::TextAreaWidget(int wrap, QWidget* parent)
     : KTextEdit(parent)
 {
     m_kwp->setIsRedirected( true );
-    if(wrap != DOM::HTMLTextAreaElementImpl::ta_NoWrap) {
+
+    if(wrap != DOM::HTMLTextAreaElementImpl::ta_NoWrap)
         setLineWrapMode(QTextEdit::WidgetWidth);
-        setHorizontalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
-        setVerticalScrollBarPolicy( Qt::ScrollBarAlwaysOn );
-    }
-    else {
+    else
         setLineWrapMode(QTextEdit::NoWrap);
-        setHorizontalScrollBarPolicy( Qt::ScrollBarAsNeeded );
-        setVerticalScrollBarPolicy( Qt::ScrollBarAsNeeded );
-    }
+
+    setHorizontalScrollBarPolicy( Qt::ScrollBarAsNeeded );
+    setVerticalScrollBarPolicy( Qt::ScrollBarAsNeeded );
+
     KCursor::setAutoHideCursor(viewport(), true);
     setAcceptRichText (false);
     setMouseTracking(true);
@@ -2104,35 +2173,27 @@ void RenderTextArea::calcMinMaxWidth()
     int lrm = qMax(0, w->style()->pixelMetric(QStyle::PM_LayoutRightMargin));
     int lbm = qMax(0, w->style()->pixelMetric(QStyle::PM_LayoutBottomMargin));
     int ltm = qMax(0, w->style()->pixelMetric(QStyle::PM_LayoutTopMargin));
-#if QT_VERSION < 0x040500
-    QStyleOptionFrameV2 opt;
-    opt.initFrom(w);
-    opt.lineWidth = w->lineWidth();
-    opt.midLineWidth = w->midLineWidth();
-    QRect r = w->style()->subElementRect(QStyle::SE_FrameContents, &opt, w);
-#else
+
     QStyleOptionFrameV3 opt;
     opt.initFrom(w);
     opt.lineWidth = w->lineWidth();
     opt.midLineWidth = w->midLineWidth();
     opt.frameShape = w->frameShape();
     QRect r = w->style()->subElementRect(QStyle::SE_ShapedFrameContents, &opt, w);
-#endif
     QRect o = opt.rect;
     int hfw = (r.left()-o.left()) + (o.right()-r.right());
     int vfw = (r.top()-o.top()) + (o.bottom()-r.bottom());
 
     QSize size( qMax(element()->cols(), 1L)*m.width('x') + hfw + llm+lrm +
                 w->verticalScrollBar()->sizeHint().width()+lhs,
-                qMax(element()->rows(), 1L)*m.lineSpacing() + 2*vfw + lbm+ltm +
+                qMax(element()->rows(), 1L)*m.lineSpacing() + vfw + lbm+ltm +
                 (w->lineWrapMode() == QTextEdit::NoWrap ?
                  w->horizontalScrollBar()->sizeHint().height()+lvs : 0)
         );
 
     assert( includesPadding() );
     size.rwidth() -= RenderWidget::paddingLeft() + RenderWidget::paddingRight();
-    // ### FIXME BASELINE: would like to remove that 2* (and the 2* in '2*vfw' above).
-    size.rheight() -= 2*(RenderWidget::paddingTop() + RenderWidget::paddingBottom());
+    size.rheight() -= RenderWidget::paddingTop() + RenderWidget::paddingBottom();
 
     setIntrinsicWidth( size.width() );
     setIntrinsicHeight( size.height() );
@@ -2153,6 +2214,19 @@ void RenderTextArea::setStyle(RenderStyle* _style)
     scrollbarsStyled = false;
 
     element()->m_unsubmittedFormChange = unsubmittedFormChange;
+    TextAreaWidget* w = static_cast<TextAreaWidget*>(m_widget);
+    if (style()->overflowX() == OSCROLL)
+        w->setHorizontalScrollBarPolicy( Qt::ScrollBarAlwaysOn );
+    else if (style()->overflowX() == OHIDDEN)
+        w->setHorizontalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
+    else
+        w->setHorizontalScrollBarPolicy( Qt::ScrollBarAsNeeded );
+    if (style()->overflowY() == OSCROLL)
+        w->setVerticalScrollBarPolicy( Qt::ScrollBarAlwaysOn );
+    else if(style()->overflowY() == OHIDDEN)
+        w->setVerticalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
+    else
+        w->setVerticalScrollBarPolicy( Qt::ScrollBarAsNeeded );
 }
 
 void RenderTextArea::layout()
@@ -2170,6 +2244,18 @@ void RenderTextArea::layout()
     }
 }
 
+short RenderTextArea::scrollWidth() const
+{
+    return RenderObject::scrollWidth();
+}
+
+int RenderTextArea::scrollHeight() const
+{
+    TextAreaWidget* w = static_cast<TextAreaWidget*>(m_widget);
+    int contentHeight = qRound(w->document()->size().height());
+    return qMax(contentHeight, RenderObject::clientHeight());
+}
+
 void RenderTextArea::setText(const QString& newText)
 {
     TextAreaWidget* w = static_cast<TextAreaWidget*>(m_widget);
@@ -2193,7 +2279,7 @@ void RenderTextArea::setText(const QString& newText)
             tc.setPosition( ex, QTextCursor::MoveAnchor );
             tc.movePosition(QTextCursor::End, QTextCursor::KeepAnchor);
             tc.insertText(newText.right( newText.length()-ex ));
-        } else {                            
+        } else {
             w->setPlainText( newText );
         }
         w->setTextCursor(tc);
@@ -2229,7 +2315,7 @@ QString RenderTextArea::text()
         while (!tc.atEnd()) {
            tc.movePosition(QTextCursor::EndOfLine, QTextCursor::KeepAnchor);
            txt += tc.selectedText();
-           if (tc.movePosition(QTextCursor::Down)) {
+           if (tc.movePosition(QTextCursor::Right)) {
                txt += QLatin1String("\n");
                tc.movePosition(QTextCursor::StartOfLine);
            } else {
