@@ -43,10 +43,6 @@ class QWidget;
 #include <kcombobox.h>
 #include "dom/dom_misc.h"
 
-class KFindDialog;
-class KReplaceDialog;
-class KFind;
-class KReplace;
 class KAction;
 class KUrlRequester;
 class KdeUiProxyStyle;
@@ -216,6 +212,8 @@ public:
     virtual void updateFromElement();
     virtual short baselinePosition( bool ) const;
 protected:
+    virtual void setPadding();
+    virtual void setStyle(RenderStyle *style);
     virtual bool canHaveBorder() const;
 private:
     QString rawText();
@@ -284,14 +282,12 @@ public:
     void setSelectionRange(long start, long end);
 public Q_SLOTS:
     void slotReturnPressed();
-    void slotTextChanged(const QString &string);
-    void slotCompletionBoxActivated(const QString &string);
+    void slotTextEdited(const QString &string);
 protected:
 
 private:
     virtual bool isEditable() const { return true; }
     virtual bool canHaveBorder() const { return true; }
-    bool m_blockElementUpdates;
 };
 
 // -------------------------------------------------------------------------
@@ -482,6 +478,8 @@ protected:
     bool m_ignoreSelectEvents;
     bool m_optionsChanged;
 
+    void clearItemFlags(int index, Qt::ItemFlags flags);
+
 protected Q_SLOTS:
     void slotSelected(int index);
     void slotSelectionChanged();
