@@ -36,9 +36,7 @@
 
 #include "css/csshelper.h"
 #include "misc/helper.h"
-#include "misc/htmlattrs.h"
 #include "misc/loader.h"
-#include "misc/htmltags.h"
 #include "html/html_formimpl.h"
 #include "html/html_imageimpl.h"
 #include "html/dtd.h"
@@ -188,8 +186,8 @@ void RenderImage::updatePixmap( const QRect& r, CachedImage *o)
     }
     else
     {
-        int cHeight = contentHeight();
         //### FIXME!
+        //int cHeight = contentHeight();
         //int scaledHeight = intrinsicHeight() ? ((o->valid_rect().height()*cHeight)/intrinsicHeight()) : 0;
 
         repaintRectangle(r.x() + borderLeft() + paddingLeft(), r.y() + borderTop() + paddingTop(),
@@ -364,7 +362,7 @@ void RenderImage::layout()
     KHTMLAssert( needsLayout());
     KHTMLAssert( minMaxKnown() );
 
-    short m_width = 0;
+    //short m_width = 0;
     int m_height;
 
     // minimum height
