@@ -6,6 +6,8 @@
  *           (C) 2001 Dirk Mueller (mueller@kde.org)
  *           (C) 2002-2006 Apple Computer, Inc.
  *           (C) 2006 Allan Sandfeld Jensen (kde@carewolf.com)
+ *           (C) 2005 Frerich Raabe <raabe@kde.org>
+ *           (C) 2010 Maksim Orlovich <maksim@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -38,7 +40,6 @@
 #include <css/csshelper.h>
 #include <css/cssstyleselector.h>
 #include <css/css_stylesheetimpl.h>
-#include <misc/htmlhashes.h>
 #include <misc/helper.h>
 #include <misc/seed.h>
 #include <misc/loader.h>
@@ -69,6 +70,7 @@
 #include <khtml_settings.h>
 #include <khtmlpart_p.h>
 
+#include <xml/dom3_xpathimpl.h>
 #include <html/html_baseimpl.h>
 #include <html/html_blockimpl.h>
 #include <html/html_canvasimpl.h>
@@ -85,6 +87,8 @@
 #include <html/HTMLSourceElement.h>
 #include <editing/jsediting.h>
 
+#include <ecma/kjs_window.h>
+
 // SVG (WebCore)
 #include <svg/SVGElement.h>
 #include <svg/SVGSVGElement.h>
@@ -129,8 +133,6 @@ using namespace khtml;
 
 // ------------------------------------------------------------------------
 
-DOMImplementationImpl *DOMImplementationImpl::m_instance = 0;
-
 DOMImplementationImpl::DOMImplementationImpl()
 {
 }
@@ -165,6 +167,10 @@ bool DOMImplementationImpl::hasFeature ( const DOMString &feature, const DOMStri
          lower == "htmlevents" || lower == "textevents" ) &&
         (version.isEmpty() || version == "2.0" || version == "3.0"))
         return true;
+
+    if (lower == "selectors-api" && version == "1.0")
+        return true;
+        
     return false;
 }
 
@@ -190,13 +196,7 @@ DocumentTypeImpl *DOMImplementationImpl::createDocumentType( const DOMString &qu
         return 0;
     }
 
-    return new DocumentTypeImpl(this,0,qualifiedName,publicId,systemId);
-}
-
-DOMImplementationImpl* DOMImplementationImpl::getInterface(const DOMString& /*feature*/) const
-{
-    // ###
-    return 0;
+    return new DocumentTypeImpl(this, 0, qualifiedName, publicId, systemId);
 }
 
 DocumentImpl *DOMImplementationImpl::createDocument( const DOMString &namespaceURI, const DOMString &qualifiedName,
@@ -212,7 +212,10 @@ DocumentImpl *DOMImplementationImpl::createDocument( const DOMString &namespaceU
 
     // WRONG_DOCUMENT_ERR: Raised if doctype has already been used with a different document or was
     // created from a different implementation.
-    if (dtype && (dtype->document() || dtype->implementation() != this)) {
+    // We elide the "different implementation" case here, since we're not doing interop
+    // of different implementations, and different impl objects exist only for
+    // isolation  reasons
+    if (dtype && dtype->document()) {
         exceptioncode = DOMException::WRONG_DOCUMENT_ERR;
         return 0;
     }
@@ -220,9 +223,9 @@ DocumentImpl *DOMImplementationImpl::createDocument( const DOMString &namespaceU
     // ### view can be 0 which can cause problems
     DocumentImpl* doc;
     if (namespaceURI == XHTML_NAMESPACE)
-        doc = new HTMLDocumentImpl(this, v);
+        doc = new HTMLDocumentImpl(v);
     else
-        doc = new DocumentImpl(this, v);
+        doc = new DocumentImpl(v);
 
     if (dtype) {
         dtype->setDocument(doc);
@@ -257,21 +260,21 @@ CSSStyleSheetImpl *DOMImplementationImpl::createCSSStyleSheet(DOMStringImpl* tit
 
 DocumentImpl *DOMImplementationImpl::createDocument( KHTMLView *v )
 {
-    DocumentImpl* doc = new DocumentImpl(this, v);
+    DocumentImpl* doc = new DocumentImpl(v);
 
     return doc;
 }
 
 XMLDocumentImpl *DOMImplementationImpl::createXMLDocument( KHTMLView *v )
 {
-    XMLDocumentImpl* doc = new XMLDocumentImpl(this, v);
+    XMLDocumentImpl* doc = new XMLDocumentImpl(v);
 
     return doc;
 }
 
 HTMLDocumentImpl *DOMImplementationImpl::createHTMLDocument( KHTMLView *v )
 {
-    HTMLDocumentImpl* doc = new HTMLDocumentImpl(this, v);
+    HTMLDocumentImpl* doc = new HTMLDocumentImpl(v);
 
     return doc;
 }
@@ -279,7 +282,7 @@ HTMLDocumentImpl *DOMImplementationImpl::createHTMLDocument( KHTMLView *v )
 // create SVG document
 WebCore::SVGDocument *DOMImplementationImpl::createSVGDocument( KHTMLView *v )
 {
-    WebCore::SVGDocument* doc = new WebCore::SVGDocument(this, v);
+    WebCore::SVGDocument* doc = new WebCore::SVGDocument(v);
 
     return doc;
 }
@@ -296,16 +299,6 @@ HTMLDocumentImpl* DOMImplementationImpl::createHTMLDocument( const DOMString& ti
     return r;
 }
 
-DOMImplementationImpl *DOMImplementationImpl::instance()
-{
-    if (!m_instance) {
-        m_instance = new DOMImplementationImpl();
-        m_instance->ref();
-    }
-
-    return m_instance;
-}
-
 // ------------------------------------------------------------------------
 
 ElementMappingCache::ElementMappingCache():m_dict()
@@ -380,7 +373,7 @@ typedef QList<DocumentImpl*> ChangedDocuments ;
 K_GLOBAL_STATIC(ChangedDocuments, s_changedDocuments)
 
 // KHTMLView might be 0
-DocumentImpl::DocumentImpl(DOMImplementationImpl *_implementation, KHTMLView *v)
+DocumentImpl::DocumentImpl(KHTMLView *v)
     : NodeBaseImpl( 0 ), m_svgExtensions(0), m_counterDict(),
       m_imageLoadEventTimer(0)
 {
@@ -410,8 +403,7 @@ DocumentImpl::DocumentImpl(DOMImplementationImpl *_implementation, KHTMLView *v)
     m_tokenizer = 0;
     m_decoder = 0;
     m_doctype = 0;
-    m_implementation = _implementation;
-    m_implementation->ref();
+    m_implementation = 0;
     pMode = Strict;
     hMode = XHtml;
     m_htmlCompat = false;
@@ -436,6 +428,7 @@ DocumentImpl::DocumentImpl(DOMImplementationImpl *_implementation, KHTMLView *v)
     m_async = true;
     m_hadLoadError = false;
     m_docLoading = false;
+    m_bVariableLength = false;
     m_inSyncLoad = 0;
     m_loadingXMLDoc = 0;
     m_documentElement = 0;
@@ -443,6 +436,8 @@ DocumentImpl::DocumentImpl(DOMImplementationImpl *_implementation, KHTMLView *v)
     m_jsEditor = 0;
     m_dynamicDomRestyler = new khtml::DynamicDomRestyler();
     m_stateRestorePos = 0;
+    m_windowEventTarget = new WindowEventTargetImpl(this);
+    m_windowEventTarget->ref();
     
     for (int c = 0; c < NumTreeVersions; ++c)
         m_domTreeVersions[c] = 0;
@@ -509,7 +504,7 @@ DocumentImpl::~DocumentImpl()
     //you may also have to fix removedLastRef() above - M.O.
     assert( !m_render );
 
-    QHashIterator<long,NodeListImpl::Cache*> it(m_nodeListCache);
+    QHashIterator<long,DynamicNodeListImpl::Cache*> it(m_nodeListCache);
     while (it.hasNext())
         it.next().value()->deref();
 
@@ -524,7 +519,8 @@ DocumentImpl::~DocumentImpl()
     if (m_elemSheet )  m_elemSheet->deref();
     if (m_doctype)
         m_doctype->deref();
-    m_implementation->deref();
+    if (m_implementation)
+        m_implementation->deref();
     delete m_dynamicDomRestyler;
     delete m_jsEditor;
     m_defaultView->deref();
@@ -541,6 +537,7 @@ DocumentImpl::~DocumentImpl()
         m_activeNode->deref();
     if (m_documentElement)
         m_documentElement->deref();
+    m_windowEventTarget->deref();
     qDeleteAll(m_counterDict);
 
     m_renderArena.reset();
@@ -551,6 +548,10 @@ DocumentImpl::~DocumentImpl()
 
 DOMImplementationImpl *DocumentImpl::implementation() const
 {
+    if (!m_implementation) {
+        m_implementation = new DOMImplementationImpl();
+        m_implementation->ref();
+    }
     return m_implementation;
 }
 
@@ -675,7 +676,7 @@ NodeImpl *DocumentImpl::importNode(NodeImpl *importedNode, bool deep, int &excep
 	NamedAttrMapImpl *otherMap = static_cast<ElementImpl *>(importedNode)->attributes(true);
 
 	ElementImpl *tempElementImpl;
-	tempElementImpl = createElementNS(otherElem->namespaceURI(),otherElem->nodeName());
+	tempElementImpl = createElementNS(otherElem->namespaceURI(),otherElem->nonCaseFoldedTagName());
 	tempElementImpl->setHTMLCompat(htmlMode() != XHtml && otherElem->htmlCompat());
 	result = tempElementImpl;
 
@@ -752,19 +753,19 @@ ElementImpl *DocumentImpl::createElementNS( const DOMString &_namespaceURI, cons
         }
     }
 
-    if ((isHTMLDocument() && _namespaceURI.isNull()) ||
-        (strcasecmp(_namespaceURI, XHTML_NAMESPACE) == 0 && localName == localName.lower())) {
-        e = createHTMLElement(localName);
-        if (e) {
-            int _exceptioncode = 0;
-            if (!prefix.isNull())
-                e->setPrefix(prefix, _exceptioncode);
-            if ( _exceptioncode ) {
-                 if ( pExceptioncode ) *pExceptioncode = _exceptioncode;
-                 delete e;
-                 return 0;
-            }
-            e->setHTMLCompat( _namespaceURI.isNull() && htmlMode() != XHtml );
+    // Regardless of document type (even for HTML), this method will only create HTML
+    // elements if given the namespace explicitly. Further, this method is always
+    // case sensitive, again, even in HTML; however .tagName will case-normalize
+    // in HTML regardless
+    if (_namespaceURI == XHTML_NAMESPACE) {
+        e = createHTMLElement(localName, false /* case sensitive */);
+        int _exceptioncode = 0;
+        if (!prefix.isNull())
+            e->setPrefix(prefix, _exceptioncode);
+        if ( _exceptioncode ) {
+                if ( pExceptioncode ) *pExceptioncode = _exceptioncode;
+                delete e;
+                return 0;
         }
     }
     if (!e) {
@@ -868,7 +869,7 @@ void DocumentImpl::setTitle(const DOMString& _title)
 	    titleStr = url.prettyUrl();
 	}
 
-	emit view()->part()->setWindowCaption( KStringHandler::csqueeze( titleStr, 128 ) );
+	emit view()->part()->setWindowCaption( titleStr );
     }
 }
 
@@ -882,9 +883,10 @@ unsigned short DocumentImpl::nodeType() const
     return Node::DOCUMENT_NODE;
 }
 
-ElementImpl *DocumentImpl::createHTMLElement( const DOMString &name )
+ElementImpl *DocumentImpl::createHTMLElement( const DOMString &name, bool caseInsensitive )
 {
-    LocalName localname = LocalName::fromString(name.lower());
+    LocalName localname = LocalName::fromString(name,
+                                  caseInsensitive ? IDS_NormalizeLower : IDS_CaseSensitive);
     uint id = localname.id();
 
     ElementImpl *n = 0;
@@ -1164,8 +1166,10 @@ ElementImpl *DocumentImpl::createHTMLElement( const DOMString &name )
         break;
 
     default:
+        n = new HTMLGenericElementImpl(docPtr(), localname);
         break;
     }
+    assert(n);
     return n;
 }
 
@@ -1619,7 +1623,7 @@ void DocumentImpl::open( bool clearEventListeners )
         attach();
 
     if (clearEventListeners)
-        m_windowEventListeners.clear();
+        windowEventTarget()->listenerList().clear();
 
     m_tokenizer = createTokenizer();
     //m_decoderMibEnum = 0;
@@ -1647,7 +1651,10 @@ HTMLElementImpl* DocumentImpl::body() const
 
 void DocumentImpl::close(  )
 {
-    if (parsing() || !m_tokenizer) return;
+    if (parsing() && hasVariableLength() && m_tokenizer) {
+        m_tokenizer->finish();
+    } else if (parsing() || !m_tokenizer)
+        return;
 
     if ( m_render )
         m_render->close();
@@ -1675,7 +1682,7 @@ void DocumentImpl::write( const QString &text )
         open();
         if (m_view)
             m_view->part()->resetFromScript();
-        m_tokenizer->setAutoClose();
+        setHasVariableLength();
     }
     m_tokenizer->write(text, false);
 }
@@ -2105,7 +2112,7 @@ WTF::PassRefPtr<NodeImpl> DocumentImpl::cloneNode ( bool deep )
 #endif
 
     int exceptioncode;
-    WTF::RefPtr<NodeImpl> clone = m_implementation->createDocument("",
+    WTF::RefPtr<NodeImpl> clone = DOMImplementationImpl::createDocument("",
 							   "",
                                                            0, 0,
                                                            exceptioncode);
@@ -2127,6 +2134,8 @@ void DocumentImpl::styleSheetLoaded()
 
   m_pendingStylesheets--;
   updateStyleSelector();
+  if (!m_pendingStylesheets && m_tokenizer)
+      m_tokenizer->executeScriptsWaitingForStylesheets();
 }
 
 void DocumentImpl::addPendingSheet()
@@ -2563,7 +2572,7 @@ bool DocumentImpl::isURLAllowed(const QString& url) const
         return false;
 
     // Prohibit non-file URLs if we are asked to.
-    if (!thisPart || thisPart->onlyLocalReferences() && newURL.protocol() != "file" && newURL.protocol() != "data")
+    if (!thisPart || ( thisPart->onlyLocalReferences() && newURL.protocol() != "file" && newURL.protocol() != "data" ))
         return false;
 
     // do we allow this suburl ?
@@ -2728,65 +2737,43 @@ void DocumentImpl::error(int err, const QString &text)
 
 void DocumentImpl::defaultEventHandler(EventImpl *evt)
 {
-    // if any html event listeners are registered on the window, then dispatch them here
-    if (m_windowEventListeners.listeners && !evt->propagationStopped()) {
-
-        QList<RegisteredEventListener>::iterator it;
-
-        //Grab a copy in case of clear
-        QList<RegisteredEventListener> listeners = *m_windowEventListeners.listeners;
-        Event ev(evt);
-        for (it = listeners.begin(); it != listeners.end(); ++it) {
-            //Check to make sure it didn't get removed. KDE4: use Java-style iterators
-            if (!m_windowEventListeners.stillContainsListener(*it))
-                continue;
-
-            if ((*it).eventName == evt->name()) {
-                // currentTarget must be 0 in khtml for kjs_events to set "this" correctly.
-                // (this is how we identify events dispatched to the window, like window.onmousedown)
-                // ## currentTarget is unimplemented in IE, and is "window" in Mozilla (how? not a DOM node)
-                evt->setCurrentTarget(0);
-                (*it).listener->handleEvent(ev);
-	    }
-        }
-    }
     if ( evt->id() == EventImpl::KHTML_CONTENTLOADED_EVENT && !evt->propagationStopped() && !evt->defaultPrevented() )
         contentLoaded();
 }
 
 void DocumentImpl::setHTMLWindowEventListener(EventName id, EventListener *listener)
 {
-    m_windowEventListeners.setHTMLEventListener(id, listener);
+    windowEventTarget()->listenerList().setHTMLEventListener(id, listener);
 }
 
 void DocumentImpl::setHTMLWindowEventListener(unsigned id, EventListener *listener)
 {
-    m_windowEventListeners.setHTMLEventListener(EventName::fromId(id), listener);
+    windowEventTarget()->listenerList().setHTMLEventListener(EventName::fromId(id), listener);
 }
 
 EventListener *DocumentImpl::getHTMLWindowEventListener(EventName id)
 {
-    return m_windowEventListeners.getHTMLEventListener(id);
+    return windowEventTarget()->listenerList().getHTMLEventListener(id);
 }
 
 EventListener *DocumentImpl::getHTMLWindowEventListener(unsigned id)
 {
-    return m_windowEventListeners.getHTMLEventListener(EventName::fromId(id));
+    return windowEventTarget()->listenerList().getHTMLEventListener(EventName::fromId(id));
 }
 
 void DocumentImpl::addWindowEventListener(EventName id, EventListener *listener, const bool useCapture)
 {
-    m_windowEventListeners.addEventListener(id, listener, useCapture);
+    windowEventTarget()->listenerList().addEventListener(id, listener, useCapture);
 }
 
 void DocumentImpl::removeWindowEventListener(EventName id, EventListener *listener, bool useCapture)
 {
-    m_windowEventListeners.removeEventListener(id, listener, useCapture);
+    windowEventTarget()->listenerList().removeEventListener(id, listener, useCapture);
 }
 
 bool DocumentImpl::hasWindowEventListener(EventName id)
 {
-    return m_windowEventListeners.hasEventListener(id);
+    return windowEventTarget()->listenerList().hasEventListener(id);
 }
 
 EventListener *DocumentImpl::createHTMLEventListener(const QString& code, const QString& name, NodeImpl* node)
@@ -2834,8 +2821,9 @@ void DocumentImpl::dispatchImageLoadEventsNow()
     m_imageLoadEventDispatchingList.clear();
 }
 
-void DocumentImpl::timerEvent(QTimerEvent *)
+void DocumentImpl::timerEvent(QTimerEvent *e)
 {
+    assert(e->timerId() == m_imageLoadEventTimer);
     dispatchImageLoadEventsNow();
 }
 
@@ -2915,16 +2903,16 @@ KHTMLPart* DOM::DocumentImpl::part() const
     return view() ? view()->part() : 0;
 }
 
-NodeListImpl::Cache* DOM::DocumentImpl::acquireCachedNodeListInfo(
-       NodeListImpl::CacheFactory* factory, NodeImpl* base, int type)
+DynamicNodeListImpl::Cache* DOM::DocumentImpl::acquireCachedNodeListInfo(
+       DynamicNodeListImpl::CacheFactory* factory, NodeImpl* base, int type)
 {
     //### might want to flush the dict when the version number
     //changes
-    NodeListImpl::CacheKey key(base, type);
+    DynamicNodeListImpl::CacheKey key(base, type);
 
     //Check to see if we have this sort of item cached.
-    NodeListImpl::Cache* cached =
-        (type == NodeListImpl::UNCACHEABLE) ? 0 : m_nodeListCache.value(key.hash());
+    DynamicNodeListImpl::Cache* cached =
+        (type == DynamicNodeListImpl::UNCACHEABLE) ? 0 : m_nodeListCache.value(key.hash());
 
     if (cached) {
         if (cached->key == key) {
@@ -2937,12 +2925,12 @@ NodeListImpl::Cache* DOM::DocumentImpl::acquireCachedNodeListInfo(
     }
 
     //Nothing to reuse, make a new item.
-    NodeListImpl::Cache* newInfo = factory();
+    DynamicNodeListImpl::Cache* newInfo = factory();
     newInfo->key = key;
     newInfo->clear(this);
     newInfo->ref(); //Add the nodelist's reference
 
-    if (type != NodeListImpl::UNCACHEABLE) {
+    if (type != DynamicNodeListImpl::UNCACHEABLE) {
         newInfo->ref(); //Add the cache's reference
         m_nodeListCache.insert(key.hash(), newInfo);
     }
@@ -2950,7 +2938,7 @@ NodeListImpl::Cache* DOM::DocumentImpl::acquireCachedNodeListInfo(
     return newInfo;
 }
 
-void DOM::DocumentImpl::releaseCachedNodeListInfo(NodeListImpl::Cache* entry)
+void DOM::DocumentImpl::releaseCachedNodeListInfo(DynamicNodeListImpl::Cache* entry)
 {
     entry->deref();
 }
@@ -3015,6 +3003,71 @@ DOMString DocumentImpl::queryCommandValue(const DOMString &command)
     return jsEditor()->queryCommandValue(jsEditor()->commandImp(command));
 }
 
+// ----------------------------------------------------------------------------
+// DOM3 XPath, from XPathEvaluator interface
+
+khtml::XPathExpressionImpl* DocumentImpl::createExpression(DOMString &expression,
+                                                  khtml::XPathNSResolverImpl *resolver,
+                                                  int &exceptioncode )
+{
+    XPathExpressionImpl* cand = new XPathExpressionImpl( expression, resolver );
+    if ((exceptioncode = cand->parseExceptionCode())) {
+        delete cand;
+        return 0;
+    }
+
+    return cand;
+}
+
+khtml::XPathNSResolverImpl* DocumentImpl::createNSResolver( NodeImpl *nodeResolver )
+{
+    return nodeResolver ? new DefaultXPathNSResolverImpl(nodeResolver) : 0;
+}
+
+khtml::XPathResultImpl * DocumentImpl::evaluate( DOMString &expression,
+                                      NodeImpl *contextNode,
+                                      khtml::XPathNSResolverImpl *resolver,
+                                      unsigned short type,
+                                      khtml::XPathResultImpl * /*result*/,
+                                      int &exceptioncode )
+{
+    XPathExpressionImpl *expr = createExpression( expression, resolver, exceptioncode );
+    if ( exceptioncode )
+        return 0;
+
+    XPathResultImpl *res = expr->evaluate( contextNode, type, 0, exceptioncode );
+    delete expr;  // don't need it anymore.
+    
+    if ( exceptioncode ) {
+        delete res;
+        return 0;
+    }
+
+    return res;
+}
+// ----------------------------------------------------------------------------
+
+WindowEventTargetImpl::WindowEventTargetImpl(DOM::DocumentImpl* owner):
+    m_owner(owner)
+{}
+
+EventTargetImpl::Type WindowEventTargetImpl::eventTargetType() const
+{
+    return WINDOW;
+}
+
+DocumentImpl* WindowEventTargetImpl::eventTargetDocument()
+{
+    return m_owner;
+}
+
+KJS::Window* WindowEventTargetImpl::window()
+{
+    if (m_owner->part())
+        return KJS::Window::retrieveWindow(m_owner->part());
+    else
+        return 0;
+}
 // ----------------------------------------------------------------------------
 
 DocumentFragmentImpl::DocumentFragmentImpl(DocumentImpl *doc) : NodeBaseImpl(doc)
