@@ -50,6 +50,7 @@ class QPaintDevice;
 class QTextCodec;
 class KHTMLView;
 class QEventLoop;
+class KEncodingDetector;
 
 namespace khtml {
     class Tokenizer;
@@ -62,6 +63,14 @@ namespace khtml {
     class CachedObject;
     class CachedCSSStyleSheet;
     class DynamicDomRestyler;
+
+    class XPathResultImpl;
+    class XPathExpressionImpl;
+    class XPathNSResolverImpl;
+}
+
+namespace KJS {
+    class Window;
 }
 
 namespace WebCore
@@ -102,6 +111,7 @@ namespace DOM {
     class StyleSheetListImpl;
     class TextImpl;
     class TreeWalkerImpl;
+    class WindowEventTargetImpl;    
 
 class DOMImplementationImpl : public khtml::Shared<DOMImplementationImpl>
 {
@@ -109,34 +119,25 @@ public:
     DOMImplementationImpl();
     ~DOMImplementationImpl();
 
-    // DOM methods & attributes for DOMImplementation
-    bool hasFeature ( const DOMString &feature, const DOMString &version );
+    // DOM methods & attributes for DOMImplementation.
+    static bool hasFeature ( const DOMString &feature, const DOMString &version );
     DocumentTypeImpl *createDocumentType( const DOMString &qualifiedName, const DOMString &publicId,
                                           const DOMString &systemId, int &exceptioncode );
-    DocumentImpl *createDocument( const DOMString &namespaceURI, const DOMString &qualifiedName,
+    static DocumentImpl *createDocument( const DOMString &namespaceURI, const DOMString &qualifiedName,
                                   DocumentTypeImpl* dtype,
                                   KHTMLView* v, int &exceptioncode );
 
-    DOMImplementationImpl* getInterface(const DOMString& feature) const;
-
     // From the DOMImplementationCSS interface
-    CSSStyleSheetImpl *createCSSStyleSheet(DOMStringImpl *title, DOMStringImpl *media, int &exceptioncode);
+    static CSSStyleSheetImpl *createCSSStyleSheet(DOMStringImpl *title, DOMStringImpl *media, int &exceptioncode);
 
     // From the HTMLDOMImplementation interface
-    HTMLDocumentImpl* createHTMLDocument( const DOMString& title);
+    static HTMLDocumentImpl* createHTMLDocument( const DOMString& title);
 
     // Other methods (not part of DOM)
-    DocumentImpl *createDocument( KHTMLView *v = 0 );
-    XMLDocumentImpl *createXMLDocument( KHTMLView *v = 0 );
-    HTMLDocumentImpl *createHTMLDocument( KHTMLView *v = 0 );
-    WebCore::SVGDocument *createSVGDocument( KHTMLView *v = 0 );
-
-    // Returns the static instance of this class - only one instance of this class should
-    // ever be present, and is used as a factory method for creating DocumentImpl objects
-    static DOMImplementationImpl *instance();
-
-protected:
-    static DOMImplementationImpl *m_instance;
+    static DocumentImpl *createDocument( KHTMLView *v = 0 );
+    static XMLDocumentImpl *createXMLDocument( KHTMLView *v = 0 );
+    static HTMLDocumentImpl *createHTMLDocument( KHTMLView *v = 0 );
+    static WebCore::SVGDocument *createSVGDocument( KHTMLView *v = 0 );
 };
 
 /**
@@ -190,7 +191,6 @@ private:
     QHash<DOMString,ItemInfo*> m_dict;
 };
 
-
 /**
  * @internal
  */
@@ -198,7 +198,7 @@ class DocumentImpl : public QObject, private khtml::CachedObjectClient, public N
 {
     Q_OBJECT
 public:
-    DocumentImpl(DOMImplementationImpl *_implementation, KHTMLView *v);
+    DocumentImpl(KHTMLView *v);
     ~DocumentImpl();
 
     // DOM methods & attributes for Document
@@ -227,6 +227,17 @@ public:
                                            int* pExceptioncode = 0 );
     ElementImpl *getElementById ( const DOMString &elementId ) const;
 
+    // DOM3 XPath, from XPathEvaluator interface
+    khtml::XPathExpressionImpl *createExpression( DOM::DOMString& expression,
+                                                  khtml::XPathNSResolverImpl *resolver,
+                                                  int &exceptioncode );
+    khtml::XPathNSResolverImpl *createNSResolver( NodeImpl *nodeResolver );
+    khtml::XPathResultImpl *evaluate( DOM::DOMString& expression,
+                                      NodeImpl *contextNode,
+                                      khtml::XPathNSResolverImpl *resolver,
+                                      unsigned short type,
+                                      khtml::XPathResultImpl *result,
+                                      int &exceptioncode );
 
     // Actually part of HTMLDocument, but used for giving XML documents a window title as well
     DOMString title() const { return m_title; }
@@ -240,8 +251,9 @@ public:
     // Other methods (not part of DOM)
     virtual bool isDocumentNode() const { return true; }
     virtual bool isHTMLDocument() const { return false; }
+    virtual bool isSVGDocument() const;
 
-    virtual ElementImpl *createHTMLElement ( const DOMString &tagName );
+    virtual ElementImpl *createHTMLElement ( const DOMString &tagName, bool caseInsensitive = true );
     // SVG
     virtual ElementImpl *createSVGElement(const QualifiedName& name);
 
@@ -287,7 +299,9 @@ public:
     void ensureStyleSheetListUpToDate() { if (m_styleSheetListDirty) rebuildStyleSheetList(true); }
 
     bool readyForLayout() const;
-    
+
+    // DOM representation of the JS Window object, for event handling
+    WindowEventTargetImpl* windowEventTarget() const { return m_windowEventTarget; }
 private:    
     void rebuildStyleSheetList(bool force = false);
     void rebuildStyleSelector ();
@@ -358,7 +372,7 @@ public:
     QString baseTarget() const { return m_baseTarget; }
     void setBaseTarget(const QString& baseTarget) { m_baseTarget = baseTarget; }
 
-    QString completeURL(const QString& url) const { return KUrl(baseURL(),url /*,m_decoderMibEnum*/).url(); }
+    QString completeURL(const QString& url) const;
     DOMString canonURL(const DOMString& url) const { return url.isEmpty() ? url : completeURL(url.string()); }
 
     void setUserStyleSheet(const QString& sheet);
@@ -369,6 +383,8 @@ public:
     CSSStyleSheetImpl* elementSheet();
     virtual khtml::Tokenizer *createTokenizer();
     khtml::Tokenizer *tokenizer() { return m_tokenizer; }
+    KEncodingDetector* decoder() { return m_decoder; }
+    void setDecoder(KEncodingDetector* enc) { m_decoder = enc; }
 
     void setPaintDevice(QPaintDevice *dev){m_paintDevice = dev;}
     QPaintDevice *paintDevice() const {return m_paintDevice;}
@@ -400,6 +416,9 @@ public:
     void setParsing(bool b) { m_bParsing = b; }
     bool parsing() const { return m_bParsing; }
 
+    void setHasVariableLength(bool b=true) { m_bVariableLength = b; }
+    bool hasVariableLength() const { return m_bVariableLength; }
+
     void setTextColor( QColor color ) { m_textColor = color; }
     QColor textColor() const { return m_textColor; }
 
@@ -573,9 +592,9 @@ public:
     // Since applications often re-creat nodelists all over the place, we cache 
     // their caches in the documents. For now, we only do it for things that can be 
     // parametrices by type + base node.
-    NodeListImpl::Cache* acquireCachedNodeListInfo(NodeListImpl::CacheFactory* fact,
-                                                   NodeImpl* base, int type);
-    void                 releaseCachedNodeListInfo(NodeListImpl::Cache* cache);
+    DynamicNodeListImpl::Cache* acquireCachedNodeListInfo(DynamicNodeListImpl::CacheFactory* fact,
+                                                          NodeImpl* base, int type);
+    void releaseCachedNodeListInfo(DynamicNodeListImpl::Cache* cache);
 
     JSEditor *jsEditor();
 
@@ -612,12 +631,14 @@ protected:
 
     khtml::DocLoader *m_docLoader;
     khtml::Tokenizer *m_tokenizer;
+    KEncodingDetector *m_decoder;
     KUrl m_url;
     KUrl m_baseURL;
     QString m_baseTarget;
 
     mutable DocumentTypeImpl *m_doctype;
-    DOMImplementationImpl *m_implementation;
+
+    mutable DOMImplementationImpl *m_implementation; // lazily created
 
     QString m_usersheet;
     QString m_printSheet;
@@ -655,6 +676,7 @@ protected:
     StyleSheetListImpl* m_styleSheets;
     StyleSheetListImpl *m_addedStyleSheets; // programmatically added style sheets
     LocalStyleRefs m_localStyleRefs; // references to inlined style elements
+    WindowEventTargetImpl* m_windowEventTarget;
     RegisteredListenerList m_windowEventListeners;
     QList<NodeImpl*> m_maintainsState;
 
@@ -671,6 +693,7 @@ protected:
     bool m_async;
     bool m_hadLoadError;
     bool m_docLoading;
+    bool m_bVariableLength;
 
     QEventLoop* m_inSyncLoad;
 
@@ -686,7 +709,7 @@ protected:
     ElementMappingCache m_underDocNamedCache;
 
     //Cache for nodelists and collections.
-    QHash<long,NodeListImpl::Cache*> m_nodeListCache;
+    QHash<long,DynamicNodeListImpl::Cache*> m_nodeListCache;
 
     QLinkedList<HTMLImageElementImpl*> m_imageLoadEventDispatchSoonList;
     QLinkedList<HTMLImageElementImpl*> m_imageLoadEventDispatchingList;
@@ -718,6 +741,23 @@ public:
     virtual void removedLastRef();
 };
 
+/*
+ * This represent the Window object at the DOM level --- for now it only plays
+ * the role of an event dispatch target, and isn't accessible directly
+ * (it turns into Window in JS land)
+ */
+class WindowEventTargetImpl : public EventTargetImpl
+{
+public:
+    WindowEventTargetImpl(DOM::DocumentImpl* owner);
+    
+    virtual Type eventTargetType() const;
+    virtual DocumentImpl* eventTargetDocument();    
+    KJS::Window* window();
+private:
+    DOM::DocumentImpl* m_owner;
+};
+
 class DocumentFragmentImpl : public NodeBaseImpl
 {
 public:
@@ -782,7 +822,7 @@ protected:
 class XMLDocumentImpl : public DocumentImpl
 {
 public:
-    XMLDocumentImpl(DOMImplementationImpl *_implementation, KHTMLView *v) : DocumentImpl(_implementation, v) { }
+    XMLDocumentImpl(KHTMLView *v) : DocumentImpl(v) { }
 
     virtual void close (  );
 };
