@@ -924,6 +924,7 @@ KLauncher::send_service_startup_info( KLaunchRequest *request, KService::Ptr ser
         data.setWMClass( wmclass );
     if( silent )
         data.setSilent( KStartupInfoData::Yes );
+    data.setApplicationId( service->entryPath());
     // the rest will be sent by kdeinit
     KStartupInfo::sendStartupX( dpy, id, data );
     if( mCached_dpy != dpy && mCached_dpy != NULL )
@@ -993,7 +994,7 @@ KLauncher::kdeinit_exec(const QString &app, const QStringList &args,
        KService::Ptr service = KService::serviceByDesktopName(desktopName);
        if (service)
            send_service_startup_info(request, service,
-                                     request->startup_id, QStringList());
+                                     request->startup_id, envs);
        else // no .desktop file, no startup info
            cancel_service_startup_info(request, request->startup_id, envs);
    }
