@@ -320,7 +320,9 @@ QString KBookmark::fullText() const
     if (isSeparator())
         return i18n("--- separator ---");
 
-    return element.namedItem("title").toElement().text();
+    QString text = element.namedItem("title").toElement().text();
+    text.replace('\n', ' '); // #140673
+    return text;
 }
 
 void KBookmark::setFullText(const QString &fullText)
@@ -360,6 +362,8 @@ QString KBookmark::icon() const
     // migration code
     if (icon.isEmpty())
       icon = element.attribute("icon");
+    if (icon == "www") // common icon for kde3 bookmarks
+        return "internet-web-browser";
     // end migration code
 
     if (icon == "bookmark_folder") {
@@ -402,6 +406,33 @@ void KBookmark::setIcon(const QString &icon)
         element.removeAttribute("icon");
 }
 
+QString KBookmark::description() const
+{
+    if (isSeparator())
+        return QString();
+
+    QString description = element.namedItem("desc").toElement().text();
+    description.replace('\n', ' '); // #140673
+    return description;
+}
+
+void KBookmark::setDescription(const QString &description)
+{
+    QDomNode descNode = element.namedItem("desc");
+    if (descNode.isNull()) {
+        descNode = element.ownerDocument().createElement("desc");
+        element.appendChild(descNode);
+    }
+
+    if (descNode.firstChild().isNull()) {
+        QDomText domtext = descNode.ownerDocument().createTextNode(QString());
+        descNode.appendChild(domtext);
+    }
+
+    QDomText domtext = descNode.firstChild().toText();
+    domtext.setData(description);
+}
+
 QString KBookmark::mimeType() const
 {
     QDomNode metaDataNode = metaData(METADATA_MIME_OWNER, false);
@@ -686,10 +717,10 @@ KBookmark::List KBookmark::List::fromMimeData( const QMimeData *mimeData, QDomDo
     if ( !payload.isEmpty() ) {
         doc.setContent( payload );
         QDomElement elem = doc.documentElement();
-        QDomNodeList children = elem.childNodes();
+        const QDomNodeList children = elem.childNodes();
         for ( int childno = 0; childno < children.count(); childno++)
         {
-            bookmarks.append( KBookmark( children.item(childno).cloneNode(true).toElement() ));
+            bookmarks.append( KBookmark( children.item(childno).toElement() ));
         }
         return bookmarks;
     }
