@@ -6,6 +6,7 @@
                   1999,2000,2001,2002,2003 Carsten Pfeiffer <pfeiffer@kde.org>
                   2003 Clarence Dang <dang@kde.org>
                   2008 Jarosław Staniek <staniek@kde.org>
+                  2009 David Jarvie <djarvie@kde.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -28,6 +29,8 @@
 #include <QtGui/QCheckBox>
 #include <QtGui/QKeyEvent>
 #include <QtGui/QFileDialog>
+#include <QtGui/QApplication>
+#include <QtGui/QDesktopWidget>
 
 #include <kimageio.h>
 #include <klocale.h>
@@ -39,11 +42,10 @@
 #include <kpluginfactory.h>
 #include <kdebug.h>
 #include <kwindowsystem.h>
-#include <kcompletion.h>
-#include <kurlcombobox.h>
 #include "kabstractfilewidget.h"
 #include "kabstractfilemodule.h"
 #include "krecentdirs.h"
+#include "kservice.h"
 
 /** File dialogs are native by default on Windows. */
 #ifdef Q_WS_WIN
@@ -78,8 +80,8 @@ static QStringList mime2KdeFilter( const QStringList &mimeTypes, QString *allExt
 static QString qtFilter(const QStringList& _filters)
 {
     QString converted;
-    QStringList filters = _filters;
-    qSort( filters );
+    const QStringList filters = _filters;
+
     foreach (const QString& current, filters) {
         QString new_f; //filter part
         QString new_name; //filter name part
@@ -92,37 +94,25 @@ static QString qtFilter(const QStringList& _filters)
             new_f = current.left(p);
             new_name = current.mid(p+1);
         }
-        // remove (.....) from name
-        p = new_name.indexOf('(');
-        int p2 = new_name.lastIndexOf(')');
-        QString new_name1, new_name2;
-        if (p!=-1)
-            new_name1 = new_name.left(p);
-        if (p2!=-1)
-            new_name2 = new_name.mid(p2+1);
-        if (!new_name1.isEmpty() || !new_name2.isEmpty())
-            new_name = new_name1.trimmed() + QLatin1Char(' ') + new_name2.trimmed();
-        new_name.remove('(');
-        new_name.remove(')');
-        new_name = new_name.trimmed();
+	//Qt filters assume anything in () is the file extension list
+	new_name = new_name.replace('(', '[').replace(')',']').trimmed();
 
-        // make filters unique: remove uppercase extensions (case doesn't matter on win32, BTW)
+        //convert everything to lower case and remove dupes (doesn't matter on win32)
         QStringList allfiltersUnique;
         const QStringList origList( new_f.split(' ', QString::SkipEmptyParts) );
         foreach (const QString& origFilter, origList) {
-            if (origFilter == origFilter.toLower())
-                allfiltersUnique += origFilter;
+	if (!allfiltersUnique.contains(origFilter, Qt::CaseInsensitive))
+                allfiltersUnique += origFilter.toLower();
         }
 
         if (!converted.isEmpty())
             converted += ";;";
 
         converted += (new_name + " (" + allfiltersUnique.join(" ") + QLatin1Char(')'));
-    } // foreach
+    }
 
     // Strip escape characters from escaped '/' characters.
-    for (int pos = 0; (pos = converted.indexOf("\\/", pos)) != -1; ++pos)
-      converted.remove(pos, 1);
+    converted.replace("\\/","/");
 
     return converted;
 }
@@ -149,18 +139,25 @@ static QString qtFilter(const QString& filter)
 }
 
 static KAbstractFileModule* s_module = 0;
+static KAbstractFileModule* loadFileModule( const QString& moduleName )
+{
+    KService::Ptr fileModuleService = KService::serviceByDesktopName(moduleName);
+    if(fileModuleService)
+        return fileModuleService->createInstance<KAbstractFileModule>();
+    else
+        return 0;
+}
+
+static const char s_defaultFileModuleName[] = "kfilemodule";
 static KAbstractFileModule* fileModule()
 {
-    if (!s_module) {
-        // TODO fix memleak -- qApp post routine for deleting the module ?
-        KPluginLoader loader("libkfilemodule");
-        KPluginFactory *factory = loader.factory();
-        if (!factory) {
-            kWarning() << "KFileDialog wasn't able to find libkfilemodule: " << loader.errorString();
-        } else {
-            s_module = factory->create<KAbstractFileModule>();
-            if (!s_module) {
-                kWarning() << "An error occurred while loading libkfilemodule";
+    if(!s_module) {
+        QString moduleName = KConfigGroup(KGlobal::config(), ConfigGroup).readEntry("file module", s_defaultFileModuleName);
+        if(!(s_module = loadFileModule(moduleName))) {
+            kDebug() << "Failed to load configured file module" << moduleName;
+            if(moduleName != s_defaultFileModuleName) {
+                kDebug() << "Falling back to default file module.";
+                s_module = loadFileModule(s_defaultFileModuleName);
             }
         }
     }
@@ -197,7 +194,9 @@ public:
             return s_startDir;
         }
         static KUrl s_startDir;
+        static bool s_allowNative;  // as fallback when we can't use native dialog
         QString filter;
+        QString selectedFilter;
         QStringList mimeTypes;
         KUrl::List selectedUrls;
         KFile::Modes mode;
@@ -209,15 +208,37 @@ public:
         w(0),
         cfgGroup(KGlobal::config(), ConfigGroup)
     {
-        if (cfgGroup.readEntry("Native", NATIVE_FILEDIALOGS_BY_DEFAULT))
+        if (cfgGroup.readEntry("Native", NATIVE_FILEDIALOGS_BY_DEFAULT) &&
+            KFileDialogPrivate::Native::s_allowNative)
             native = new Native;
     }
 
     static bool isNative()
     {
+        if(!KFileDialogPrivate::Native::s_allowNative)
+            return false;
         KConfigGroup cfgGroup(KGlobal::config(), ConfigGroup);
         return cfgGroup.readEntry("Native", NATIVE_FILEDIALOGS_BY_DEFAULT);
     }
+    
+    static QString getOpenFileName(const KUrl& startDir, const QString& filter,
+                                   QWidget *parent, const QString& caption,
+                                   QString *selectedFilter);
+    static KUrl getOpenUrl(const KUrl& startDir, const QString& filter,
+                           QWidget *parent, const QString& caption, 
+                           QString *selectedFilter);
+    static QStringList getOpenFileNames(const KUrl& startDir, const QString& filter,
+                                        QWidget *parent, const QString& caption,
+                                        QString *selectedFilter);
+    static KUrl::List getOpenUrls(const KUrl& startDir, const QString& filter,
+                                  QWidget *parent, const QString& caption,
+                                  QString *selectedFilter);
+    static QString getSaveFileName(const KUrl& dir, const QString& filter,
+                                   QWidget *parent, const QString& caption, 
+                                   KFileDialog::Options options, QString *selectedFilter);
+    static KUrl getSaveUrl(const KUrl& dir, const QString& filter,
+                           QWidget *parent, const QString& caption, 
+                           KFileDialog::Options options, QString *selectedFilter);
 
     ~KFileDialogPrivate()
     {
@@ -230,6 +251,7 @@ public:
 };
 
 KUrl KFileDialogPrivate::Native::s_startDir;
+bool KFileDialogPrivate::Native::s_allowNative = true;
 
 KFileDialog::KFileDialog( const KUrl& startDir, const QString& filter,
                           QWidget *parent, QWidget* customWidget)
@@ -241,12 +263,8 @@ KFileDialog::KFileDialog( const KUrl& startDir, const QString& filter,
       d( new KFileDialogPrivate )
 
 {
-    if (!d->native) {
-        setButtons( KDialog::None );
-        restoreDialogSize(d->cfgGroup); // call this before the fileQWidget is set as the main widget.
-                                        // otherwise the sizes for the components are not obeyed (ereslibre)
-    }
-
+    // It would be nice to have this behind d->native but it doesn't work
+    // because of derived classes like KEncodingDialog...
     // Dlopen the file widget from libkfilemodule
     QWidget* fileQWidget = fileModule()->createFileWidget(startDir, this);
     d->w = ::qobject_cast<KAbstractFileWidget *>(fileQWidget);
@@ -262,6 +280,10 @@ KFileDialog::KFileDialog( const KUrl& startDir, const QString& filter,
         return;
     }
 
+    setButtons( KDialog::None );
+    restoreDialogSize(d->cfgGroup); // call this before the fileQWidget is set as the main widget.
+                                   // otherwise the sizes for the components are not obeyed (ereslibre)
+
     d->w->setFilter(filter);
     setMainWidget(fileQWidget);
 
@@ -273,17 +295,22 @@ KFileDialog::KFileDialog( const KUrl& startDir, const QString& filter,
     // Publish signals
     // TODO: Move the relevant signal declarations from KFileWidget to the
     //       KAbstractFileWidget interface?
-    //
     //       Else, all of these connects (including "accepted") are not typesafe.
+    // Answer: you cannot define signals in a non-qobject base class (DF).
+    //         I simply documentde them in kabstractfilewidget.h now.
     kDebug (kfile_area) << "KFileDialog connecting signals";
-    connect(fileQWidget, SIGNAL(fileSelected(const QString&)),
-                         SIGNAL(fileSelected(const QString&)));
-    connect(fileQWidget, SIGNAL(fileHighlighted(const QString&)),
-                         SIGNAL(fileHighlighted(const QString&)));
+    connect(fileQWidget, SIGNAL(fileSelected(KUrl)),
+                         SIGNAL(fileSelected(KUrl)));
+    connect(fileQWidget, SIGNAL(fileHighlighted(KUrl)),
+                         SIGNAL(fileHighlighted(KUrl)));
+    connect(fileQWidget, SIGNAL(fileSelected(QString)),
+                         SIGNAL(fileSelected(QString)));
+    connect(fileQWidget, SIGNAL(fileHighlighted(QString)),
+                         SIGNAL(fileHighlighted(QString)));
     connect(fileQWidget, SIGNAL(selectionChanged()),
                          SIGNAL(selectionChanged()));
-    connect(fileQWidget, SIGNAL(filterChanged(const QString&)),
-                         SIGNAL(filterChanged(const QString&)));
+    connect(fileQWidget, SIGNAL(filterChanged(QString)),
+                         SIGNAL(filterChanged(QString)));
 
     connect(fileQWidget, SIGNAL(accepted()), SLOT(accept()));
     //connect(fileQWidget, SIGNAL(canceled()), SLOT(slotCancel()));
@@ -347,6 +374,13 @@ void KFileDialog::clearFilter()
 
 QString KFileDialog::currentMimeFilter() const
 {
+    if (d->native) {
+        // adapted from qt2KdeFilter
+        QString filter = d->native->selectedFilter.split(";;").replaceInStrings("/", "\\/")[0];
+        filter = filter.mid(filter.indexOf('(') + 1, filter.indexOf(')') - filter.indexOf('(') - 1);
+        QString mimetype = KMimeType::findByPath("test" + filter.mid(1).split(' ')[0])->name();
+        return mimetype;
+    }
     return d->w->currentMimeFilter();
 }
 
@@ -370,9 +404,15 @@ void KFileDialog::setInlinePreviewShown(bool show)
     d->w->setInlinePreviewShown(show);
 }
 
+// This is only used for the initial size when no configuration has been saved
 QSize KFileDialog::sizeHint() const
 {
-    return QSize(700, 450);
+    int fontSize = fontMetrics().height();
+    QSize goodSize(48 * fontSize, 30 * fontSize);
+    QSize screenSize = QApplication::desktop()->availableGeometry(this).size();
+    QSize minSize(screenSize / 2);
+    QSize maxSize(screenSize * qreal(0.9));
+    return (goodSize.expandedTo(minSize).boundedTo(maxSize));
 }
 
 // This slot still exists mostly for compat purposes; for subclasses which reimplement slotOk
@@ -427,24 +467,33 @@ void KFileDialog::setSelection(const QString& name)
 QString KFileDialog::getOpenFileName(const KUrl& startDir,
                                      const QString& filter,
                                      QWidget *parent, const QString& caption)
+{
+    return KFileDialogPrivate::getOpenFileName(startDir, filter, parent, caption, 0);
+}
+
+QString KFileDialogPrivate::getOpenFileName(const KUrl& startDir,
+                                            const QString& filter,
+                                            QWidget *parent, 
+                                            const QString& caption,
+                                            QString *selectedFilter)
 {
     if (KFileDialogPrivate::isNative() && (!startDir.isValid() || startDir.isLocalFile())) {
         return QFileDialog::getOpenFileName(
             parent,
             caption.isEmpty() ? i18n("Open") : caption,
-            KFileDialogPrivate::Native::staticStartDir( startDir ).path(),
-            qtFilter(filter) );
+            KFileDialogPrivate::Native::staticStartDir( startDir ).toLocalFile(),
+            qtFilter(filter),
+            selectedFilter );
 // TODO use extra args?     QString * selectedFilter = 0, Options options = 0
     }
     KFileDialog dlg(startDir, filter, parent);
-    dlg.setOperationMode( Opening );
 
+    dlg.setOperationMode( KFileDialog::Opening );
     dlg.setMode( KFile::File | KFile::LocalOnly | KFile::ExistingOnly );
     dlg.setCaption(caption.isEmpty() ? i18n("Open") : caption);
 
-    //dlg.d->ops->clearHistory();
     dlg.exec();
-
+    if(selectedFilter) *selectedFilter = dlg.currentMimeFilter();
     return dlg.selectedFile();
 }
 
@@ -455,16 +504,15 @@ QString KFileDialog::getOpenFileNameWId(const KUrl& startDir,
     if (KFileDialogPrivate::isNative() && (!startDir.isValid() || startDir.isLocalFile()))
         return KFileDialog::getOpenFileName(startDir, filter, 0, caption); // everything we can do...
     QWidget* parent = QWidget::find( parent_id );
+    KFileDialogPrivate::Native::s_allowNative = false;
     KFileDialog dlg(startDir, filter, parent);
     if( parent == NULL && parent_id != 0 )
         KWindowSystem::setMainWindow( &dlg, parent_id );
 
     dlg.setOperationMode( KFileDialog::Opening );
-
     dlg.setMode( KFile::File | KFile::LocalOnly | KFile::ExistingOnly );
     dlg.setCaption(caption.isEmpty() ? i18n("Open") : caption);
 
-    //dlg.d->ops->clearHistory();
     dlg.exec();
 
     return dlg.selectedFile();
@@ -474,63 +522,91 @@ QStringList KFileDialog::getOpenFileNames(const KUrl& startDir,
                                           const QString& filter,
                                           QWidget *parent,
                                           const QString& caption)
+{
+    return KFileDialogPrivate::getOpenFileNames(startDir, filter, parent, caption, 0);
+}
+
+QStringList KFileDialogPrivate::getOpenFileNames(const KUrl& startDir,
+                                                 const QString& filter,
+                                                 QWidget *parent,
+                                                 const QString& caption,
+                                                 QString *selectedFilter)
 {
     if (KFileDialogPrivate::isNative() && (!startDir.isValid() || startDir.isLocalFile())) {
         return QFileDialog::getOpenFileNames(
             parent,
             caption.isEmpty() ? i18n("Open") : caption,
-            KFileDialogPrivate::Native::staticStartDir( startDir ).path(),
-            qtFilter( filter ) );
+            KFileDialogPrivate::Native::staticStartDir( startDir ).toLocalFile(),
+            qtFilter( filter ), selectedFilter );
 // TODO use extra args?  QString * selectedFilter = 0, Options options = 0
     }
+    KFileDialogPrivate::Native::s_allowNative = false;
     KFileDialog dlg(startDir, filter, parent);
-    dlg.setOperationMode( Opening );
 
-    dlg.setCaption(caption.isEmpty() ? i18n("Open") : caption);
+    dlg.setOperationMode( KFileDialog::Opening );
     dlg.setMode(KFile::Files | KFile::LocalOnly | KFile::ExistingOnly);
-    //dlg.d->ops->clearHistory();
-    dlg.exec();
+    dlg.setCaption(caption.isEmpty() ? i18n("Open") : caption);
 
+    dlg.exec();
+    if(selectedFilter) *selectedFilter = dlg.currentMimeFilter();
     return dlg.selectedFiles();
 }
 
 KUrl KFileDialog::getOpenUrl(const KUrl& startDir, const QString& filter,
-                                QWidget *parent, const QString& caption)
+                             QWidget *parent, const QString& caption)
+{
+    return KFileDialogPrivate::getOpenUrl(startDir, filter, parent, caption, 0);
+}
+KUrl KFileDialogPrivate::getOpenUrl(const KUrl& startDir, const QString& filter,
+                                    QWidget *parent, const QString& caption, 
+                                    QString *selectedFilter)
 {
     if (KFileDialogPrivate::isNative() && (!startDir.isValid() || startDir.isLocalFile())) {
-        const QString fileName( KFileDialog::getOpenFileName(
-            startDir, filter, parent, caption) );
+        const QString fileName( KFileDialogPrivate::getOpenFileName(
+            startDir, filter, parent, caption, selectedFilter) );
         return fileName.isEmpty() ? KUrl() : KUrl::fromPath(fileName);
     }
+    KFileDialogPrivate::Native::s_allowNative = false;
     KFileDialog dlg(startDir, filter, parent);
-    dlg.setOperationMode( Opening );
 
-    dlg.setCaption(caption.isEmpty() ? i18n("Open") : caption);
+    dlg.setOperationMode( KFileDialog::Opening );
     dlg.setMode( KFile::File | KFile::ExistingOnly );
-    //dlg.d->ops->clearHistory();
-    dlg.exec();
+    dlg.setCaption(caption.isEmpty() ? i18n("Open") : caption);
 
+    dlg.exec();
+    if(selectedFilter) *selectedFilter = dlg.currentMimeFilter();
     return dlg.selectedUrl();
 }
 
 KUrl::List KFileDialog::getOpenUrls(const KUrl& startDir,
-                                          const QString& filter,
-                                          QWidget *parent,
-                                          const QString& caption)
+                                    const QString& filter,
+                                    QWidget *parent,
+                                    const QString& caption)
+{
+    return KFileDialogPrivate::getOpenUrls(startDir, filter, parent, caption, 0);
+}
+
+KUrl::List KFileDialogPrivate::getOpenUrls(const KUrl& startDir,
+                                           const QString& filter,
+                                           QWidget *parent,
+                                           const QString& caption,
+                                           QString *selectedFilter)
 {
     if (KFileDialogPrivate::isNative() && (!startDir.isValid() || startDir.isLocalFile())) {
-        const QStringList fileNames( KFileDialog::getOpenFileNames(
-            startDir, filter, parent, caption) );
+        const QStringList fileNames( KFileDialogPrivate::getOpenFileNames(
+            startDir, filter, parent, caption, selectedFilter) );
         return KUrl::List(fileNames);
     }
+    KFileDialogPrivate::Native::s_allowNative = false;
+
     KFileDialog dlg(startDir, filter, parent);
-    dlg.setOperationMode( Opening );
 
+    dlg.setOperationMode( KFileDialog::Opening );
+    dlg.setMode( KFile::Files | KFile::ExistingOnly );
     dlg.setCaption(caption.isEmpty() ? i18n("Open") : caption);
-    dlg.setMode(KFile::Files | KFile::ExistingOnly);
-    //dlg.d->ops->clearHistory();
-    dlg.exec();
 
+    dlg.exec();
+    if(selectedFilter) *selectedFilter = dlg.currentMimeFilter();
     return dlg.selectedUrls();
 }
 
@@ -547,7 +623,7 @@ KUrl KFileDialog::getExistingDirectoryUrl(const KUrl& startDir,
 {
     if (KFileDialogPrivate::isNative() && (!startDir.isValid() || startDir.isLocalFile())) {
         QString result( QFileDialog::getExistingDirectory(parent, caption,
-            KFileDialogPrivate::Native::staticStartDir( startDir ).path(),
+            KFileDialogPrivate::Native::staticStartDir( startDir ).toLocalFile(),
             QFileDialog::ShowDirsOnly) );
         return result.isEmpty() ? KUrl() : KUrl::fromPath(result);
     }
@@ -560,7 +636,7 @@ QString KFileDialog::getExistingDirectory(const KUrl& startDir,
 {
     if (KFileDialogPrivate::isNative() && (!startDir.isValid() || startDir.isLocalFile())) {
         return QFileDialog::getExistingDirectory(parent, caption,
-            KFileDialogPrivate::Native::staticStartDir( startDir ).path(),
+            KFileDialogPrivate::Native::staticStartDir( startDir ).toLocalFile(),
             QFileDialog::ShowDirsOnly);
     }
     KUrl url = fileModule()->selectDirectory(startDir, true, parent, caption);
@@ -576,13 +652,13 @@ KUrl KFileDialog::getImageOpenUrl( const KUrl& startDir, QWidget *parent,
         const QStringList mimetypes( KImageIO::mimeTypes( KImageIO::Reading ) );
         return KFileDialog::getOpenUrl(startDir, mimetypes.join(" "), parent, caption);
     }
-    QStringList mimetypes = KImageIO::mimeTypes( KImageIO::Reading );
-    KFileDialog dlg(startDir,
-                    mimetypes.join(" "),
-                    parent);
-    dlg.setOperationMode( Opening );
+    const QStringList mimetypes = KImageIO::mimeTypes( KImageIO::Reading );
+    KFileDialogPrivate::Native::s_allowNative = false;
+    KFileDialog dlg(startDir, mimetypes.join(" "), parent);
+
+    dlg.setOperationMode( KFileDialog::Opening );
+    dlg.setMode( KFile::File | KFile::ExistingOnly );
     dlg.setCaption( caption.isEmpty() ? i18n("Open") : caption );
-    dlg.setMode( KFile::File );
     dlg.setInlinePreviewShown( true );
 
     dlg.exec();
@@ -607,7 +683,7 @@ KUrl::List KFileDialog::selectedUrls() const
 QString KFileDialog::selectedFile() const
 {
     if (d->native)
-        return selectedUrl().path();
+        return selectedUrl().toLocalFile();
     return d->w->selectedFile();
 }
 
@@ -628,6 +704,22 @@ KUrl KFileDialog::baseUrl() const
 QString KFileDialog::getSaveFileName(const KUrl& dir, const QString& filter,
                                      QWidget *parent,
                                      const QString& caption)
+{
+    //TODO KDE5: replace this method by the method below (with default parameter values in declaration)
+    // Set no confirm-overwrite mode for backwards compatibility
+    return KFileDialogPrivate::getSaveFileName(dir, filter, parent, caption, Options(0), 0);
+}
+
+QString KFileDialog::getSaveFileName(const KUrl& dir, const QString& filter,
+                                     QWidget *parent,
+                                     const QString& caption, Options options)
+{
+    return KFileDialogPrivate::getSaveFileName(dir, filter, parent, caption, options, 0);
+}
+
+QString KFileDialogPrivate::getSaveFileName(const KUrl& dir, const QString& filter,
+                                            QWidget *parent, const QString& caption, 
+                                            KFileDialog::Options options, QString *selectedFilter)
 {
     if (KFileDialogPrivate::isNative()) {
         bool defaultDir = dir.isEmpty();
@@ -643,12 +735,14 @@ QString KFileDialog::getSaveFileName(const KUrl& dir, const QString& filter,
           startDir = dir;
         }
 
+        QFileDialog::Options opts = (options & KFileDialog::ConfirmOverwrite) ? QFileDialog::Options(0) : QFileDialog::DontConfirmOverwrite;
         const QString result = QFileDialog::getSaveFileName(
             parent,
             caption.isEmpty() ? i18n("Save As") : caption,
-            KFileDialogPrivate::Native::staticStartDir( startDir ).path(),
-            qtFilter(filter) );
-// TODO use extra args?     QString * selectedFilter = 0, Options options = 0
+            KFileDialogPrivate::Native::staticStartDir( startDir ).toLocalFile(),
+            qtFilter(filter),
+// TODO use extra args?     QString * selectedFilter = 0, Options opts = 0
+            selectedFilter, opts );
         if (!result.isEmpty()) {
             if (!recentDirClass.isEmpty())
                 KRecentDirs::add(recentDirClass, KUrl::fromPath(result).url());
@@ -656,17 +750,13 @@ QString KFileDialog::getSaveFileName(const KUrl& dir, const QString& filter,
         }
         return result;
     }
-    bool defaultDir = dir.isEmpty();
-    bool specialDir = !defaultDir && dir.protocol() == "kfiledialog";
-    KFileDialog dlg( specialDir ? dir : KUrl(), filter, parent);
-    if ( !specialDir && !defaultDir ) {
-        if (!dir.isLocalFile())
-            kWarning() << "KFileDialog::getSaveFileName called with non-local start dir " << dir;
-        dlg.setSelection( dir.path() ); // may also be a filename
-    }
 
-    dlg.setOperationMode( Saving );
-    dlg.setMode( KFile::File );
+    KFileDialog dlg(dir, filter, parent);
+
+    dlg.setOperationMode( KFileDialog::Saving );
+    dlg.setMode( KFile::File | KFile::LocalOnly );
+    dlg.setConfirmOverwrite(options & KFileDialog::ConfirmOverwrite);
+    dlg.setInlinePreviewShown(options & KFileDialog::ShowInlinePreview);
     dlg.setCaption(caption.isEmpty() ? i18n("Save As") : caption);
 
     dlg.exec();
@@ -681,25 +771,29 @@ QString KFileDialog::getSaveFileName(const KUrl& dir, const QString& filter,
 QString KFileDialog::getSaveFileNameWId(const KUrl& dir, const QString& filter,
                                      WId parent_id,
                                      const QString& caption)
+{
+    //TODO KDE5: replace this method by the method below (with default parameter values in declaration)
+    // Set no confirm-overwrite mode for backwards compatibility
+    return getSaveFileNameWId(dir, filter, parent_id, caption, Options(0));
+}
+
+QString KFileDialog::getSaveFileNameWId(const KUrl& dir, const QString& filter,
+                                     WId parent_id,
+                                     const QString& caption, Options options)
 {
     if (KFileDialogPrivate::isNative()) {
-        return KFileDialog::getSaveFileName(dir, filter, 0, caption); // everything we can do...
+        return KFileDialog::getSaveFileName(dir, filter, 0, caption, options); // everything we can do...
     }
-    bool defaultDir = dir.isEmpty();
-    bool specialDir = !defaultDir && dir.protocol() == "kfiledialog";
+
     QWidget* parent = QWidget::find( parent_id );
-    KFileDialog dlg( specialDir ? dir : KUrl(), filter, parent);
+    KFileDialog dlg(dir, filter, parent);
     if( parent == NULL && parent_id != 0 )
         KWindowSystem::setMainWindow( &dlg, parent_id);
 
-    if ( !specialDir && !defaultDir ) {
-        if (!dir.isLocalFile())
-            kWarning() << "KFileDialog::getSaveFileNameWId called with non-local start dir " << dir;
-        dlg.setSelection( dir.path() ); // may also be a filename
-    }
-
-    dlg.setOperationMode( Saving );
-    dlg.setMode( KFile::File );
+    dlg.setOperationMode( KFileDialog::Saving );
+    dlg.setMode( KFile::File | KFile::LocalOnly );
+    dlg.setConfirmOverwrite(options & ConfirmOverwrite);
+    dlg.setInlinePreviewShown(options & ShowInlinePreview);
     dlg.setCaption(caption.isEmpty() ? i18n("Save As") : caption);
 
     dlg.exec();
@@ -713,24 +807,39 @@ QString KFileDialog::getSaveFileNameWId(const KUrl& dir, const QString& filter,
 
 KUrl KFileDialog::getSaveUrl(const KUrl& dir, const QString& filter,
                              QWidget *parent, const QString& caption)
+{
+    //TODO KDE5: replace this method by the method below (with default parameter values in declaration)
+    // Set no confirm-overwrite mode for backwards compatibility
+    return KFileDialogPrivate::getSaveUrl(dir, filter, parent, caption, Options(0), 0);
+}
+
+KUrl KFileDialog::getSaveUrl(const KUrl& dir, const QString& filter,
+                             QWidget *parent, const QString& caption, Options options)
+{
+    return KFileDialogPrivate::getSaveUrl(dir, filter, parent, caption, options, 0);
+}
+KUrl KFileDialogPrivate::getSaveUrl(const KUrl& dir, const QString& filter,
+                                    QWidget *parent, const QString& caption, 
+                                    KFileDialog::Options options, QString *selectedFilter)
 {
     if (KFileDialogPrivate::isNative() && (!dir.isValid() || dir.isLocalFile())) {
-        const QString fileName( KFileDialog::getSaveFileName(
-            dir.path(), filter, parent, caption) );
+        const QString fileName( KFileDialogPrivate::getSaveFileName(
+            dir, filter, parent, caption, options, selectedFilter) );
         return fileName.isEmpty() ? KUrl() : KUrl::fromPath(fileName);
     }
-    bool defaultDir = dir.isEmpty();
-    bool specialDir = !defaultDir && dir.protocol() == "kfiledialog";
-    KFileDialog dlg(specialDir ? dir : KUrl(), filter, parent);
-    if ( !specialDir )
-        dlg.setSelection( dir.url() ); // may also be a filename
 
-    dlg.setCaption(caption.isEmpty() ? i18n("Save As") : caption);
-    dlg.setOperationMode( Saving );
+    KFileDialogPrivate::Native::s_allowNative = false;
+
+    KFileDialog dlg(dir, filter, parent);
+
+    dlg.setOperationMode( KFileDialog::Saving );
     dlg.setMode( KFile::File );
+    dlg.setConfirmOverwrite(options & KFileDialog::ConfirmOverwrite);
+    dlg.setInlinePreviewShown(options & KFileDialog::ShowInlinePreview);
+    dlg.setCaption(caption.isEmpty() ? i18n("Save As") : caption);
 
     dlg.exec();
-
+    if(selectedFilter) *selectedFilter = dlg.currentMimeFilter();
     KUrl url = dlg.selectedUrl();
     if (url.isValid())
         KRecentDocument::add( url );
@@ -858,64 +967,91 @@ KAbstractFileWidget* KFileDialog::fileWidget()
 #ifdef Q_WS_WIN
 int KFileDialog::exec()
 {
-    if (!d->native)
-      return KDialog::exec();
+    if (!d->native || !KFileDialogPrivate::Native::s_allowNative) {
+        KFileDialogPrivate::Native::s_allowNative = true;
+        return KDialog::exec();
+    }
 
 // not clear here to let KFileDialogPrivate::Native::startDir() return a useful value
 // d->native->selectedUrls.clear();
+    int res = QDialog::Rejected;
     switch (d->native->operationMode) {
     case KAbstractFileWidget::Opening:
     case KAbstractFileWidget::Other:
         if (d->native->mode & KFile::File) {
-            KUrl url( KFileDialog::getOpenUrl(
-               d->native->startDir(), d->native->filter, parentWidget(), windowTitle()) );
-            if (url.isEmpty() || !url.isValid())
-                return QDialog::Rejected;
+            KUrl url( KFileDialogPrivate::getOpenUrl(
+               d->native->startDir(), d->native->filter, parentWidget(), windowTitle(), &d->native->selectedFilter ) );
+            if (url.isEmpty() || !url.isValid()) {
+                res = QDialog::Rejected;
+                break;
+            }
             d->native->selectedUrls.clear();
             d->native->selectedUrls.append(url);
-            return QDialog::Accepted;
+            res = QDialog::Accepted;
+            break;
         }
         else if (d->native->mode & KFile::Files) {
-            KUrl::List urls( KFileDialog::getOpenUrls(
-                d->native->startDir(), d->native->filter, parentWidget(), windowTitle()) );
-            if (urls.isEmpty())
-                return QDialog::Rejected;
+            KUrl::List urls( KFileDialogPrivate::getOpenUrls(
+                d->native->startDir(), d->native->filter, parentWidget(), windowTitle(), &d->native->selectedFilter ) );
+            if (urls.isEmpty()) {
+                res = QDialog::Rejected;
+                break;
+            }
             d->native->selectedUrls = urls;
-            return QDialog::Accepted;
+            res = QDialog::Accepted;
+            break;
         }
         else if (d->native->mode & KFile::Directory) {
             KUrl url( KFileDialog::getExistingDirectoryUrl(
                 d->native->startDir(), parentWidget(), windowTitle()) );
-            if (url.isEmpty() || !url.isValid())
-                return QDialog::Rejected;
+            if (url.isEmpty() || !url.isValid()) {
+                res = QDialog::Rejected;
+            }
             d->native->selectedUrls.clear();
             d->native->selectedUrls.append(url);
-            return QDialog::Accepted;
+            res = QDialog::Accepted;
+            break;
         }
         break;
     case KAbstractFileWidget::Saving:
         if (d->native->mode & KFile::File) {
-            KUrl url( KFileDialog::getSaveUrl(
-                d->native->startDir(), d->native->filter, parentWidget(), windowTitle()) );
-            if (url.isEmpty() || !url.isValid())
-                return QDialog::Rejected;
+            KUrl url( KFileDialogPrivate::getSaveUrl(
+                d->native->startDir(), d->native->filter, parentWidget(), windowTitle(), Options(0), &d->native->selectedFilter ) );
+            if (url.isEmpty() || !url.isValid())  {
+                res = QDialog::Rejected;
+                break;
+            }
             d->native->selectedUrls.clear();
             d->native->selectedUrls.append(url);
-            return QDialog::Accepted;
+            res = QDialog::Accepted;
+            break;
         }
         else if (d->native->mode & KFile::Directory) {
             KUrl url( KFileDialog::getExistingDirectoryUrl(
                 d->native->startDir(), parentWidget(), windowTitle()) );
-            if (url.isEmpty() || !url.isValid())
-                return QDialog::Rejected;
+            if (url.isEmpty() || !url.isValid()) {
+                res = QDialog::Rejected;
+                break;
+            }
             d->native->selectedUrls.clear();
             d->native->selectedUrls.append(url);
-            return QDialog::Accepted;
+            res = QDialog::Accepted;
+            break;
         }
         break;
     default:;
     }
-    return QDialog::Rejected;
+
+    setResult(res);
+    emit finished();
+
+    if (res == QDialog::Accepted) {
+        emit accepted();
+    } else {
+        emit rejected();
+    }
+
+    return res;
 }
 #endif // Q_WS_WIN
 
@@ -986,7 +1122,7 @@ public:
     {
         QString               filter;
         QTextStream           str(&filter, QIODevice::WriteOnly);
-        QStringList           list(f.split(";;"));
+        QStringList           list(f.split(";;").replaceInStrings("/", "\\/"));
         QStringList::const_iterator it(list.begin()),
                               end(list.end());
         bool                  first=true;
@@ -1115,6 +1251,7 @@ public:
         dlg.setOperationMode(KFileDialog::Saving);
         dlg.setMode(KFile::File|KFile::LocalOnly);
         dlg.setCaption(caption);
+        dlg.setConfirmOverwrite(!(options & QFileDialog::DontConfirmOverwrite));
         dlg.exec();
 
         QString rv(dlg.selectedFile());
