@@ -29,6 +29,8 @@
 #include <QtGui/QLabel>
 #include <QtGui/QLayout>
 #include <QtGui/QCheckBox>
+#include <QtGui/QStyle>
+#include <QtGui/QStyleOptionButton>
 
 #include <kauthorized.h>
 #include <khistorycombobox.h>
@@ -402,7 +404,7 @@ void KApplicationView::slotSelectionChanged(const QItemSelection &selected, cons
 {
     Q_UNUSED(deselected)
 
-    QModelIndexList indexes = selected.indexes();
+    const QModelIndexList indexes = selected.indexes();
     if (indexes.count() == 1 && !d->appModel->isDirectory(indexes.at(0))) {
         QString exec = d->appModel->execFor(indexes.at(0));
         if (!exec.isEmpty()) {
@@ -565,7 +567,6 @@ void KOpenWithDialogPrivate::init(const QString &_text, const QString &_value)
 
   QBoxLayout *topLayout = new QVBoxLayout( mainWidget );
   topLayout->setMargin(0);
-  topLayout->setSpacing( KDialog::spacingHint() );
     label = new QLabel(_text, q);
   label->setWordWrap(true);
   topLayout->addWidget(label);
@@ -583,7 +584,7 @@ void KOpenWithDialogPrivate::init(const QString &_text, const QString &_value)
     combo->setMaxCount( max );
     int mode = cg.readEntry( "CompletionMode", int(KGlobalSettings::completionMode()));
     combo->setCompletionMode((KGlobalSettings::Completion)mode);
-    QStringList list = cg.readEntry( "History", QStringList() );
+    const QStringList list = cg.readEntry( "History", QStringList() );
     combo->setHistoryItems( list, true );
     edit = new KUrlRequester( combo, mainWidget );
   }
@@ -594,7 +595,7 @@ void KOpenWithDialogPrivate::init(const QString &_text, const QString &_value)
     edit->button()->hide();
   }
 
-  edit->setPath( _value );
+  edit->setText( _value );
   edit->setWhatsThis(i18n(
     "Following the command, you can have several place holders which will be replaced "
     "with the actual values when the actual program is run:\n"
@@ -621,6 +622,7 @@ void KOpenWithDialogPrivate::init(const QString &_text, const QString &_value)
     view = new KApplicationView(mainWidget);
     view->setModel(new KApplicationModel(view));
     topLayout->addWidget(view);
+    topLayout->setStretchFactor(view, 1);
 
     QObject::connect(view, SIGNAL(selected(QString, QString)),
                      q, SLOT(slotSelected(QString, QString)));
@@ -636,10 +638,14 @@ void KOpenWithDialogPrivate::init(const QString &_text, const QString &_value)
 
   topLayout->addWidget(terminal);
 
+  QStyleOptionButton checkBoxOption;
+  checkBoxOption.initFrom(terminal);
+  int checkBoxIndentation = terminal->style()->pixelMetric( QStyle::PM_IndicatorWidth, &checkBoxOption, terminal );
+  checkBoxIndentation += terminal->style()->pixelMetric( QStyle::PM_CheckBoxLabelSpacing, &checkBoxOption, terminal );
+
   QBoxLayout* nocloseonexitLayout = new QHBoxLayout();
   nocloseonexitLayout->setMargin( 0 );
-  nocloseonexitLayout->setSpacing( KDialog::spacingHint() );
-  QSpacerItem* spacer = new QSpacerItem( 20, 0, QSizePolicy::Fixed, QSizePolicy::Minimum );
+  QSpacerItem* spacer = new QSpacerItem( checkBoxIndentation, 0, QSizePolicy::Fixed, QSizePolicy::Minimum );
   nocloseonexitLayout->addItem( spacer );
 
   nocloseonexit = new QCheckBox( i18n("&Do not close when command exits"), mainWidget );
@@ -666,6 +672,7 @@ void KOpenWithDialogPrivate::init(const QString &_text, const QString &_value)
   else
     remember = 0L;
 
+  q->setMinimumSize(q->minimumSizeHint());
   //edit->setText( _value );
   // This is what caused "can't click on items before clicking on Name header".
   // Probably due to the resizeEvent handler using width().
@@ -688,7 +695,7 @@ KOpenWithDialog::~KOpenWithDialog()
 void KOpenWithDialog::slotSelected( const QString& /*_name*/, const QString& _exec )
 {
     KService::Ptr pService = d->curService;
-    d->edit->setPath(_exec); // calls slotTextChanged :(
+    d->edit->setText(_exec); // calls slotTextChanged :(
     d->curService = pService;
 }
 
