@@ -216,14 +216,14 @@ void KFileItemPrivate::init()
              * This is the reason for the -1
              */
             KDE_struct_stat buf;
-            const QByteArray path = QFile::encodeName(m_url.path( KUrl::RemoveTrailingSlash ));
-            if ( KDE_lstat( path.data(), &buf ) == 0 )
+            const QString path = m_url.toLocalFile( KUrl::RemoveTrailingSlash );
+            if ( KDE::lstat( path, &buf ) == 0 )
             {
                 mode = buf.st_mode;
                 if ( S_ISLNK( mode ) )
                 {
                     m_bLink = true;
-                    if ( KDE_stat( path.data(), &buf ) == 0 )
+                    if ( KDE::stat( path, &buf ) == 0 )
                         mode = buf.st_mode;
                     else // link pointing to nowhere (see kio/file/file.cc)
                         mode = (S_IFMT-1) | S_IRWXU | S_IRWXG | S_IRWXO;
@@ -235,6 +235,8 @@ void KFileItemPrivate::init()
                     m_fileMode = mode & S_IFMT; // extract file type
                 if ( m_permissions == KFileItem::Unknown )
                     m_permissions = mode & 07777; // extract permissions
+            } else {
+                kDebug() << path << "does not exist anymore";
             }
         }
     }
@@ -292,7 +294,7 @@ KIO::filesize_t KFileItemPrivate::size() const
     // If not in the KIO::UDSEntry, or if UDSEntry empty, use stat() [if local URL]
     if ( m_bIsLocalUrl ) {
         KDE_struct_stat buf;
-        if ( KDE_stat( QFile::encodeName(m_url.path(KUrl::RemoveTrailingSlash)), &buf ) == 0 )
+        if ( KDE::stat( m_url.toLocalFile(KUrl::RemoveTrailingSlash), &buf ) == 0 )
             return buf.st_size;
     }
     return 0;
@@ -331,7 +333,7 @@ KDateTime KFileItemPrivate::time( KFileItem::FileTimes mappedWhich ) const
     if ( m_bIsLocalUrl )
     {
         KDE_struct_stat buf;
-        if ( KDE_stat( QFile::encodeName(m_url.path(KUrl::RemoveTrailingSlash)), &buf ) == 0 )
+        if ( KDE::stat( m_url.toLocalFile(KUrl::RemoveTrailingSlash), &buf ) == 0 )
         {
             setTime(KFileItem::ModificationTime, buf.st_mtime);
             setTime(KFileItem::AccessTime, buf.st_atime);
@@ -374,7 +376,7 @@ bool KFileItemPrivate::cmp( const KFileItemPrivate & item ) const
              && m_bLink == item.m_bLink
              && m_hidden == item.m_hidden
              && size() == item.size()
-             && time(KFileItem::ModificationTime) == item.time(KFileItem::ModificationTime)
+             && time(KFileItem::ModificationTime) == item.time(KFileItem::ModificationTime) // TODO only if already known!
              && m_entry.stringValue( KIO::UDSEntry::UDS_ICON_NAME ) == item.m_entry.stringValue( KIO::UDSEntry::UDS_ICON_NAME )
         );
 
@@ -530,6 +532,9 @@ void KFileItem::setName( const QString& name )
 {
     d->m_strName = name;
     d->m_strText = KIO::decodeFileName( d->m_strName );
+    if (d->m_entry.contains(KIO::UDSEntry::UDS_NAME))
+        d->m_entry.insert(KIO::UDSEntry::UDS_NAME, d->m_strName); // #195385
+
 }
 
 QString KFileItem::linkDest() const
@@ -543,7 +548,7 @@ QString KFileItem::linkDest() const
     if ( d->m_bIsLocalUrl )
     {
         char buf[1000];
-        int n = readlink( QFile::encodeName(d->m_url.path( KUrl::RemoveTrailingSlash )), buf, sizeof(buf)-1 );
+        int n = readlink( QFile::encodeName(d->m_url.toLocalFile( KUrl::RemoveTrailingSlash )), buf, sizeof(buf)-1 );
         if ( n != -1 )
         {
             buf[ n ] = 0;
@@ -556,7 +561,7 @@ QString KFileItem::linkDest() const
 QString KFileItem::localPath() const
 {
   if ( d->m_bIsLocalUrl ) {
-    return d->m_url.path();
+    return d->m_url.toLocalFile();
   }
 
   // Extract the local path from the KIO::UDSEntry
@@ -624,12 +629,12 @@ QString KFileItemPrivate::user() const
     QString userName = m_entry.stringValue(KIO::UDSEntry::UDS_USER);
     if (userName.isEmpty() && m_bIsLocalUrl) {
 #ifdef Q_WS_WIN
-        QFileInfo a(m_url.path( KUrl::RemoveTrailingSlash ));
+        QFileInfo a(m_url.toLocalFile( KUrl::RemoveTrailingSlash ));
         userName = a.owner();
         m_entry.insert( KIO::UDSEntry::UDS_USER, userName );
 #else
         KDE_struct_stat buff;
-        if ( KDE_lstat( QFile::encodeName(m_url.path( KUrl::RemoveTrailingSlash )), &buff ) == 0) // get uid/gid of the link, if it's a link
+        if ( KDE::lstat( m_url.toLocalFile( KUrl::RemoveTrailingSlash ), &buff ) == 0) // get uid/gid of the link, if it's a link
         {
             struct passwd *pwuser = getpwuid( buff.st_uid );
             if ( pwuser != 0 ) {
@@ -653,12 +658,12 @@ QString KFileItemPrivate::group() const
     if (groupName.isEmpty() && m_bIsLocalUrl )
     {
 #ifdef Q_WS_WIN
-        QFileInfo a(m_url.path( KUrl::RemoveTrailingSlash ));
+        QFileInfo a(m_url.toLocalFile( KUrl::RemoveTrailingSlash ));
         groupName = a.group();
         m_entry.insert( KIO::UDSEntry::UDS_GROUP, groupName );
 #else
         KDE_struct_stat buff;
-        if ( KDE_lstat( QFile::encodeName(m_url.path( KUrl::RemoveTrailingSlash )), &buff ) == 0) // get uid/gid of the link, if it's a link
+        if ( KDE::lstat( m_url.toLocalFile( KUrl::RemoveTrailingSlash ), &buff ) == 0) // get uid/gid of the link, if it's a link
         {
             struct group *ge = getgrgid( buff.st_gid );
             if ( ge != 0 ) {
@@ -690,7 +695,7 @@ KMimeType::Ptr KFileItem::determineMimeType() const
 
         d->m_pMimeType = KMimeType::findByUrl( url, d->m_fileMode, isLocalUrl );
         Q_ASSERT(d->m_pMimeType);
-        //kDebug() << "finding mimetype for " << url.url() << " : " << d->m_pMimeType->name();
+        //kDebug() << d << "finding final mimetype for" << url << ":" << d->m_pMimeType->name();
         d->m_bMimeTypeKnown = true;
     }
 
@@ -707,14 +712,20 @@ bool KFileItem::isMimeTypeKnown() const
 
 QString KFileItem::mimeComment() const
 {
+    const QString displayType = d->m_entry.stringValue( KIO::UDSEntry::UDS_DISPLAY_TYPE );
+    if (!displayType.isEmpty())
+        return displayType;
+
     KMimeType::Ptr mType = determineMimeType();
 
     bool isLocalUrl;
     KUrl url = mostLocalUrl(isLocalUrl);
 
     KMimeType::Ptr mime = mimeTypePtr();
+    // This cannot move to kio_file (with UDS_DISPLAY_TYPE) because it needs
+    // the mimetype to be determined, which is done here, and possibly delayed...
     if (isLocalUrl && mime->is("application/x-desktop")) {
-        KDesktopFile cfg( url.path() );
+        KDesktopFile cfg( url.toLocalFile() );
         QString comment = cfg.desktopGroup().readEntry( "Comment" );
         if (!comment.isEmpty())
             return comment;
@@ -731,27 +742,16 @@ QString KFileItem::mimeComment() const
 static QString iconFromDesktopFile(const QString& path)
 {
     KDesktopFile cfg( path );
-    const KConfigGroup group = cfg.desktopGroup();
     const QString icon = cfg.readIcon();
-    const QString type = cfg.readPath();
-
-    if ( cfg.hasDeviceType() )
-    {
-        const QString unmount_icon = group.readEntry( "UnmountIcon" );
-        const QString dev = cfg.readDevice();
-        if ( !icon.isEmpty() && !unmount_icon.isEmpty() && !dev.isEmpty() )
-        {
-            KMountPoint::Ptr mountPoint = KMountPoint::currentMountPoints().findByDevice(dev);
-            if (!mountPoint) // not mounted?
-                return unmount_icon;
-        }
-    } else if ( cfg.hasLinkType() ) {
+    if ( cfg.hasLinkType() ) {
+        const KConfigGroup group = cfg.desktopGroup();
+        const QString type = cfg.readPath();
         const QString emptyIcon = group.readEntry( "EmptyIcon" );
         if ( !emptyIcon.isEmpty() ) {
             const QString u = cfg.readUrl();
             const KUrl url( u );
             if ( url.protocol() == "trash" ) {
-                // We need to find if the trash is empty, preferrably without using a KIO job.
+                // We need to find if the trash is empty, preferably  without using a KIO job.
                 // So instead kio_trash leaves an entry in its config file for us.
                 KConfig trashConfig( "trashrc", KConfig::SimpleConfig );
                 if ( trashConfig.group("Status").readEntry( "Empty", true ) ) {
@@ -780,22 +780,50 @@ QString KFileItem::iconName() const
 
     KMimeType::Ptr mime = mimeTypePtr();
     if (isLocalUrl && mime->is("application/x-desktop")) {
-        d->m_iconName = iconFromDesktopFile(url.path());
+        d->m_iconName = iconFromDesktopFile(url.toLocalFile());
         if (!d->m_iconName.isEmpty()) {
             d->m_useIconNameCache = d->m_bMimeTypeKnown;
             return d->m_iconName;
         }
     }
 
-    //kDebug() << "finding icon for " << url.url() << " : " << mimeTypePtr()->name();
+    // KDE5: handle .directory files here too, and get rid of
+    // KFolderMimeType and the url argument in KMimeType::iconName().
+
     d->m_iconName = mime->iconName(url);
     d->m_useIconNameCache = d->m_bMimeTypeKnown;
+    //kDebug() << "finding icon for" << url << ":" << d->m_iconName;
     return d->m_iconName;
 }
 
+/**
+ * Returns true if this is a desktop file.
+ * Mimetype determination is optional.
+ */
+static bool checkDesktopFile(const KFileItem& item, bool _determineMimeType)
+{
+    // only local files
+    bool isLocal;
+    const KUrl url = item.mostLocalUrl(isLocal);
+    if (!isLocal)
+        return false;
+
+    // only regular files
+    if (!item.isRegularFile())
+        return false;
+
+    // only if readable
+    if (!item.isReadable())
+        return false;
+
+    // return true if desktop file
+    KMimeType::Ptr mime = _determineMimeType ? item.determineMimeType() : item.mimeTypePtr();
+    return mime->is("application/x-desktop");
+}
+
 QStringList KFileItem::overlays() const
 {
-    QStringList names;
+    QStringList names = d->m_entry.stringValue( KIO::UDSEntry::UDS_ICON_OVERLAY_NAMES ).split(',');
     if ( d->m_bLink ) {
         names.append("emblem-symbolic-link");
     }
@@ -805,6 +833,26 @@ QStringList KFileItem::overlays() const
         names.append("object-locked");
     }
 
+    if ( checkDesktopFile(*this, false) ) {
+        KDesktopFile cfg( localPath() );
+        const KConfigGroup group = cfg.desktopGroup();
+
+        // Add a warning emblem if this is an executable desktop file
+        // which is untrusted.
+        if ( group.hasKey( "Exec" ) && !KDesktopFile::isAuthorizedDesktopFile( localPath() ) ) {
+            names.append( "emblem-important" );
+        }
+
+        if (cfg.hasDeviceType()) {
+            const QString dev = cfg.readDevice();
+            if (!dev.isEmpty()) {
+                KMountPoint::Ptr mountPoint = KMountPoint::currentMountPoints().findByDevice(dev);
+                if (mountPoint) // mounted?
+                    names.append("emblem-mounted");
+            }
+        }
+    }
+
     if ( isHidden() ) {
         names.append("hidden");
     }
@@ -812,8 +860,8 @@ QStringList KFileItem::overlays() const
 #ifndef Q_OS_WIN
     if( S_ISDIR( d->m_fileMode ) && d->m_bIsLocalUrl)
     {
-        if (KSambaShare::instance()->isDirectoryShared( d->m_url.path() ) ||
-            KNFSShare::instance()->isDirectoryShared( d->m_url.path() ))
+        if (KSambaShare::instance()->isDirectoryShared( d->m_url.toLocalFile() ) ||
+            KNFSShare::instance()->isDirectoryShared( d->m_url.toLocalFile() ))
         {
             //kDebug() << d->m_url.path();
             names.append("network-workgroup");
@@ -829,15 +877,16 @@ QStringList KFileItem::overlays() const
     return names;
 }
 
+// ## where is this used?
 QPixmap KFileItem::pixmap( int _size, int _state ) const
 {
     const QString iconName = d->m_entry.stringValue( KIO::UDSEntry::UDS_ICON_NAME );
     if ( !iconName.isEmpty() )
         return DesktopIcon(iconName, _size, _state);
 
-    if ( !d->m_pMimeType )
-    {
-        if ( S_ISDIR( d->m_fileMode ) ) {
+    if (!d->m_pMimeType) {
+        // No mimetype determined yet, go for a fast default icon
+        if (S_ISDIR(d->m_fileMode)) {
             static const QString * defaultFolderIcon = 0;
             if ( !defaultFolderIcon ) {
                 const KMimeType::Ptr mimeType = KMimeType::mimeType( "inode/directory" );
@@ -902,7 +951,7 @@ bool KFileItem::isReadable() const
     }
 
     // Or if we can't read it [using ::access()] - not network transparent
-    if ( d->m_bIsLocalUrl && ::access( QFile::encodeName(d->m_url.path()), R_OK ) == -1 )
+    if ( d->m_bIsLocalUrl && KDE::access( d->m_url.toLocalFile(), R_OK ) == -1 )
         return false;
 
     return true;
@@ -925,7 +974,7 @@ bool KFileItem::isWritable() const
     }
 
     // Or if we can't read it [using ::access()] - not network transparent
-    if ( d->m_bIsLocalUrl && ::access( QFile::encodeName(d->m_url.path()), W_OK ) == -1 )
+    if ( d->m_bIsLocalUrl && KDE::access( d->m_url.toLocalFile(), W_OK ) == -1 )
         return false;
 
     return true;
@@ -946,20 +995,12 @@ bool KFileItem::isHidden() const
 
 bool KFileItem::isDir() const
 {
-    if ( d->m_fileMode == KFileItem::Unknown )
-    {
-        kDebug(7101) << "can't say -> false";
+    if (d->m_fileMode == KFileItem::Unknown) {
+        // Probably the file was deleted already, and KDirLister hasn't told the world yet.
+        //kDebug() << d << url() << "can't say -> false";
         return false; // can't say for sure, so no
     }
     return (S_ISDIR(d->m_fileMode));
-/*
-  if  (!S_ISDIR(d->m_fileMode)) {
-  if (d->m_url.isLocalFile()) {
-  KMimeType::Ptr ptr=KMimeType::findByUrl(d->m_url,0,true,true);
-  if ((ptr!=0) && (ptr->is("directory/inode"))) return true;
-  }
-  return false
-  } else return true;*/
 }
 
 bool KFileItem::isFile() const
@@ -982,7 +1023,7 @@ bool KFileItem::acceptsDrops() const
         return true;
 
     // Executable, shell script ... ?
-    if ( QFileInfo(d->m_url.path()).isExecutable() )
+    if ( QFileInfo(d->m_url.toLocalFile()).isExecutable() )
         return true;
 
     return false;
@@ -1199,13 +1240,13 @@ void KFileItem::setMetaInfo( const KFileMetaInfo & info ) const
     d->m_metaInfo = info;
 }
 
-KFileMetaInfo KFileItem::metaInfo(bool autoget, int) const
+KFileMetaInfo KFileItem::metaInfo(bool autoget, int what) const
 {
-    if (autoget && !d->m_metaInfo.isValid())
+    if ((isRegularFile() || isDir()) && autoget && !d->m_metaInfo.isValid())
     {
         bool isLocalUrl;
         KUrl url(mostLocalUrl(isLocalUrl));
-        d->m_metaInfo = KFileMetaInfo(url);//, mimetype() );
+        d->m_metaInfo = KFileMetaInfo(url.toLocalFile(), mimetype(), (KFileMetaInfo::What)what);
     }
     return d->m_metaInfo;
 }
@@ -1303,6 +1344,20 @@ KUrl KFileItem::targetUrl() const
       return url();
 }
 
+KUrl KFileItem::nepomukUri() const
+{
+    const QString nepomukUriStr = d->m_entry.stringValue( KIO::UDSEntry::UDS_NEPOMUK_URI );
+    if(!nepomukUriStr.isEmpty()) {
+        return KUrl(nepomukUriStr);
+    }
+    else if(targetUrl().isLocalFile()) {
+        return targetUrl();
+    }
+    else {
+        return KUrl();
+    }
+}
+
 /*
  * Mimetype handling.
  *
@@ -1329,7 +1384,7 @@ KMimeType::Ptr KFileItem::mimeTypePtr() const
         // If we used the "fast mode" (no sniffing), and we didn't get a perfect (extension-based) match,
         // then determineMimeType will be able to do better.
         const bool canDoBetter = d->m_delayedMimeTypes && accuracy < 100;
-        //kDebug() << "finding mimetype for" << url() << ":" << m_pMimeType->name();
+        //kDebug() << "finding mimetype for" << url << ":" << d->m_pMimeType->name() << "canDoBetter=" << canDoBetter;
         d->m_bMimeTypeKnown = !canDoBetter;
     }
     return d->m_pMimeType;
@@ -1418,23 +1473,23 @@ KUrl::List KFileItemList::targetUrlList() const {
     return lst;
 }
 
+
 bool KFileItem::isDesktopFile() const
 {
-    // only local files
-    bool isLocal;
-    const KUrl url = mostLocalUrl(isLocal);
-    if (!isLocal)
-        return false;
-
-    // only regular files
-    if (!S_ISREG(d->m_fileMode))
-        return false;
-
-    // only if readable
-    if (!isReadable())
-        return false;
-
-    // return true if desktop file
-    return determineMimeType()->is("application/x-desktop");
+    return checkDesktopFile(*this, true);
 }
 
+bool KFileItem::isRegularFile() const
+{
+    return S_ISREG(d->m_fileMode);
+}
+
+QDebug operator<<(QDebug stream, const KFileItem& item)
+{
+    if (item.isNull()) {
+        stream << "[null KFileItem]";
+    } else {
+        stream << "[KFileItem for" << item.url() << "]";
+    }
+    return stream;
+}
