@@ -22,6 +22,7 @@
 */
 
 #include "previewjob.h"
+#include <kdebug.h>
 
 #include <sys/stat.h>
 #ifdef __FreeBSD__
@@ -100,11 +101,12 @@ public:
     // Whether we should save the thumbnail
     bool bSave;
     bool ignoreMaximumSize;
+    int sequenceIndex;
     bool succeeded;
     // If the file to create a thumb for was a temp file, this is its name
     QString tempName;
-    // Over that, it's too much
-    KIO::filesize_t maximumSize;
+    KIO::filesize_t maximumLocalSize;
+    KIO::filesize_t maximumRemoteSize;
     // the size for the icon overlay
     int iconSize;
     // the transparency of the blended mimetype icon
@@ -151,6 +153,9 @@ PreviewJob::PreviewJob( const KFileItemList &items, int width, int height,
     d->succeeded = false;
     d->thumbRoot = QDir::homePath() + "/.thumbnails/";
     d->ignoreMaximumSize = false;
+    d->sequenceIndex = 0;
+    d->maximumLocalSize = 0;
+    d->maximumRemoteSize = 0;
 
     // Return to event loop first, determineNextFile() might delete this;
     QTimer::singleShot(0, this, SLOT(startPreview()));
@@ -250,8 +255,9 @@ void PreviewJobPrivate::startPreview()
         }
     }
 
-  // Read configuration value for the maximum allowed size
-    maximumSize = PreviewJob::maximumFileSize();
+    KConfigGroup cg( KGlobal::config(), "PreviewSettings" );
+    maximumLocalSize = cg.readEntry( "MaximumSize", 5*1024*1024LL /* 5MB */ );
+    maximumRemoteSize = cg.readEntry( "MaximumRemoteSize", 0 );
 
     if (bNeedCache)
     {
@@ -286,6 +292,14 @@ void PreviewJob::removeItem( const KUrl& url )
     }
 }
 
+void KIO::PreviewJob::setSequenceIndex(int index) {
+    d_func()->sequenceIndex = index;
+}
+
+int KIO::PreviewJob::sequenceIndex() const {
+    return d_func()->sequenceIndex;
+}
+
 void PreviewJob::setIgnoreMaximumSize(bool ignoreSize)
 {
     d_func()->ignoreMaximumSize = ignoreSize;
@@ -336,15 +350,38 @@ void PreviewJob::slotResult( KJob *job )
             }
             const KIO::UDSEntry entry = static_cast<KIO::StatJob*>(job)->statResult();
             d->tOrig = entry.numberValue( KIO::UDSEntry::UDS_MODIFICATION_TIME, 0 );
-            if ( !d->ignoreMaximumSize &&
-                 (KIO::filesize_t)entry.numberValue( KIO::UDSEntry::UDS_SIZE, 0 ) > d->maximumSize &&
-                 !d->currentItem.plugin->property("IgnoreMaximumSize").toBool()
-                ) {
+
+            bool skipCurrentItem = false;
+            const KIO::filesize_t size = (KIO::filesize_t)entry.numberValue( KIO::UDSEntry::UDS_SIZE, 0 );
+            bool local = false;
+            if (d->currentItem.item.mostLocalUrl(local).isLocalFile())
+            {
+                skipCurrentItem = !d->ignoreMaximumSize && size > d->maximumLocalSize
+                                  && !d->currentItem.plugin->property("IgnoreMaximumSize").toBool();
+            }
+            else
+            {
+                // For remote items the "IgnoreMaximumSize" plugin property is not respected
+                skipCurrentItem = !d->ignoreMaximumSize && size > d->maximumRemoteSize;
+
+                // Remote directories are not supported, don't try to do a file_copy on them
+                if (!skipCurrentItem) {
+                    // TODO update item.mimeType from the UDS entry, in case it wasn't set initially
+                    KMimeType::Ptr mime = d->currentItem.item.mimeTypePtr();
+                    if (mime && mime->is("inode/directory")) {
+                        skipCurrentItem = true;
+                    }
+                }
+            }
+            if (skipCurrentItem)
+            {
                 d->determineNextFile();
                 return;
             }
 
-            if ( !d->currentItem.plugin->property( "CacheThumbnail" ).toBool() )
+            bool pluginHandlesSequences = d->currentItem.plugin->property("HandleSequences", QVariant::Bool).toBool();
+
+            if ( !d->currentItem.plugin->property( "CacheThumbnail" ).toBool()  || (d->sequenceIndex && pluginHandlesSequences) )
             {
                 // This preview will not be cached, no need to look for a saved thumbnail
                 // Just create it, and be done
@@ -366,7 +403,7 @@ void PreviewJob::slotResult( KJob *job )
                 return;
             }
 
-            d->createThumbnail( static_cast<KIO::FileCopyJob*>(job)->destUrl().path() );
+            d->createThumbnail( static_cast<KIO::FileCopyJob*>(job)->destUrl().toLocalFile() );
             return;
         }
         case PreviewJobPrivate::STATE_CREATETHUMB:
@@ -387,7 +424,8 @@ bool PreviewJobPrivate::statResultThumbnail()
     if ( thumbPath.isEmpty() )
         return false;
 
-    KUrl url = currentItem.item.url();
+    bool local = false;
+    KUrl url = currentItem.item.mostLocalUrl(local);
     // Don't include the password if any
     url.setPass(QString());
     origName = url.url();
@@ -401,6 +439,30 @@ bool PreviewJobPrivate::statResultThumbnail()
     if ( thumb.text( "Thumb::URI", 0 ) != origName ||
          thumb.text( "Thumb::MTime", 0 ).toInt() != tOrig ) return false;
 
+    QString thumbnailerVersion = currentItem.plugin->property("ThumbnailerVersion", QVariant::String).toString();
+
+    if (!thumbnailerVersion.isEmpty() && thumb.text("Software", 0).startsWith("KDE Thumbnail Generator")) {
+        //Check if the version matches
+        //The software string should read "KDE Thumbnail Generator pluginName (vX)"
+        QString softwareString = thumb.text("Software", 0).remove("KDE Thumbnail Generator").trimmed();
+        if (softwareString.isEmpty()) {
+            // The thumbnail has been created with an older version, recreating
+            return false;
+        }
+        int versionIndex = softwareString.lastIndexOf("(v");
+        if (versionIndex < 0) {
+            return false;
+        }
+
+        QString cachedVersion = softwareString.remove(0, versionIndex+2);
+        cachedVersion.chop(1);
+        uint thumbnailerMajor = thumbnailerVersion.toInt();
+        uint cachedMajor = cachedVersion.toInt();
+        if (thumbnailerMajor > cachedMajor) {
+            return false;
+        }
+    }
+
     // Found it, use it
     emitPreview( thumb );
     succeeded = true;
@@ -425,7 +487,8 @@ void PreviewJobPrivate::getOrCreateThumbnail()
         localFile.open();
         KUrl localURL;
         localURL.setPath( tempName = localFile.fileName() );
-        const KUrl currentURL = item.url();
+        bool local = false;
+        const KUrl currentURL = item.mostLocalUrl(local);
         KIO::Job * job = KIO::file_copy( currentURL, localURL, -1, KIO::Overwrite | KIO::HideProgressInfo /* No GUI */ );
         job->addMetaData("thumbnail","1");
         q->addSubjob(job);
@@ -442,13 +505,16 @@ void PreviewJobPrivate::createThumbnail( const QString &pixPath )
     KIO::TransferJob *job = KIO::get(thumbURL, NoReload, HideProgressInfo);
     q->addSubjob(job);
     q->connect(job, SIGNAL(data(KIO::Job *, const QByteArray &)), SLOT(slotThumbData(KIO::Job *, const QByteArray &)));
-    bool save = bSave && currentItem.plugin->property("CacheThumbnail").toBool();
+    bool save = bSave && currentItem.plugin->property("CacheThumbnail").toBool() && !sequenceIndex;
     job->addMetaData("mimeType", currentItem.item.mimetype());
     job->addMetaData("width", QString().setNum(save ? cacheWidth : width));
     job->addMetaData("height", QString().setNum(save ? cacheHeight : height));
     job->addMetaData("iconSize", QString().setNum(save ? 64 : iconSize));
     job->addMetaData("iconAlpha", QString().setNum(iconAlpha));
     job->addMetaData("plugin", currentItem.plugin->library());
+    if(sequenceIndex)
+        job->addMetaData("sequence-index", QString().setNum(sequenceIndex));
+
 #ifdef Q_OS_UNIX
     if (shmid == -1)
     {
@@ -480,7 +546,7 @@ void PreviewJobPrivate::slotThumbData(KIO::Job *, const QByteArray &data)
     bool save = bSave &&
                 currentItem.plugin->property("CacheThumbnail").toBool() &&
                 (currentItem.item.url().protocol() != "file" ||
-                 !currentItem.item.url().directory( KUrl::AppendTrailingSlash ).startsWith(thumbRoot));
+                 !currentItem.item.url().directory( KUrl::AppendTrailingSlash ).startsWith(thumbRoot)) && !sequenceIndex;
     QImage thumb;
 #ifdef Q_OS_UNIX
     if (shmaddr)
@@ -491,7 +557,7 @@ void PreviewJobPrivate::slotThumbData(KIO::Job *, const QByteArray &data)
         quint8 iFormat;
         str >> width >> height >> iFormat;
         QImage::Format format = static_cast<QImage::Format>( iFormat );
-        thumb = QImage(shmaddr, width, height, format );
+        thumb = QImage(shmaddr, width, height, format ).copy();
     }
     else
 #endif
@@ -502,23 +568,35 @@ void PreviewJobPrivate::slotThumbData(KIO::Job *, const QByteArray &data)
         s >> thumb;
     }
 
+    QString tempFileName;
+    bool savedCorrectly = false;
     if (save)
     {
         thumb.setText("Thumb::URI", origName);
         thumb.setText("Thumb::MTime", QString::number(tOrig));
         thumb.setText("Thumb::Size", number(currentItem.item.size()));
         thumb.setText("Thumb::Mimetype", currentItem.item.mimetype());
-        thumb.setText("Software", "KDE Thumbnail Generator");
+        QString thumbnailerVersion = currentItem.plugin->property("ThumbnailerVersion", QVariant::String).toString();
+        QString signature = QString("KDE Thumbnail Generator "+currentItem.plugin->name());
+        if (!thumbnailerVersion.isEmpty()) {
+            signature.append(" (v"+thumbnailerVersion+')');
+        }
+        thumb.setText("Software", signature);
         KTemporaryFile temp;
         temp.setPrefix(thumbPath + "kde-tmp-");
         temp.setSuffix(".png");
         temp.setAutoRemove(false);
         if (temp.open()) //Only try to write out the thumbnail if we
         {                //actually created the temp file.
-            thumb.save(temp.fileName(), "PNG");
-            KDE_rename(QFile::encodeName(temp.fileName()), QFile::encodeName(thumbPath + thumbName));
+            tempFileName = temp.fileName();
+            savedCorrectly = thumb.save(tempFileName, "PNG");
         }
     }
+    if(savedCorrectly)
+    {
+        Q_ASSERT(!tempFileName.isEmpty());
+        KDE::rename(tempFileName, thumbPath + thumbName);
+    }
     emitPreview( thumb );
     succeeded = true;
 }
