@@ -20,32 +20,37 @@
 */
 
 #include "kio/renamedialog.h"
-#include "kio/renamedialogplugin.h"
 #include <stdio.h>
 #include <assert.h>
 
 #include <QtCore/QDate>
 #include <QtCore/QFileInfo>
+#include <QtGui/QCheckBox>
 #include <QtGui/QLabel>
 #include <QtGui/QLayout>
+#include <QtGui/QPixmap>
+#include <QtGui/QScrollArea>
 #include <QtCore/QDir>
 
 #include <klineedit.h>
 #include <kmessagebox.h>
 #include <kpushbutton.h>
 #include <kio/global.h>
-#include <kmimetypetrader.h>
+#include <kio/udsentry.h>
 #include <kdialog.h>
 #include <klocale.h>
 #include <kglobal.h>
 #include <kdebug.h>
 #include <kurl.h>
+#include <kfileitem.h>
 #include <kmimetype.h>
 #include <kseparator.h>
 #include <kstringhandler.h>
 #include <kstandardguiitem.h>
 #include <kguiitem.h>
 #include <ksqueezedtextlabel.h>
+#include <kfilemetadatawidget.h>
+#include <previewjob.h>
 
 using namespace KIO;
 
@@ -55,25 +60,40 @@ class RenameDialog::RenameDialogPrivate
  public:
   RenameDialogPrivate(){
     bCancel = 0;
-    bRename = bSkip = bAutoSkip = bOverwrite = bOverwriteAll = 0;
-    bResume = bResumeAll = bSuggestNewName = 0;
+    bRename = bSkip = bOverwrite = 0;
+    bResume = bSuggestNewName = 0;
+    bApplyAll = 0;
     m_pLineEdit = 0;
+    m_srcPendingPreview = false;
+    m_destPendingPreview = false;
+    m_srcPreview = 0;
+    m_destPreview = 0;
+  }
+  void setRenameBoxText(const QString& fileName) {
+    // sets the text in file name line edit box, selecting the filename (but not the extension if there is one).
+    const QString extension = KMimeType::extractKnownExtension(fileName);
+    m_pLineEdit->setText(fileName);
+    if (!extension.isEmpty()) {
+        const int selectionLength = fileName.length() - extension.length() - 1;
+        m_pLineEdit->setSelection(0, selectionLength);
+    } else {
+        m_pLineEdit->selectAll();
+    }
   }
   KPushButton *bCancel;
   QPushButton *bRename;
   QPushButton *bSkip;
-  QPushButton *bAutoSkip;
   QPushButton *bOverwrite;
-  QPushButton *bOverwriteAll;
   QPushButton *bResume;
-  QPushButton *bResumeAll;
   QPushButton *bSuggestNewName;
+  QCheckBox *bApplyAll;
   KLineEdit* m_pLineEdit;
   KUrl src;
   KUrl dest;
-  QString mimeSrc;
-  QString mimeDest;
-  bool plugin;
+  bool m_srcPendingPreview;
+  bool m_destPendingPreview;
+  QLabel* m_srcPreview;
+  QLabel* m_destPreview;
 };
 
 RenameDialog::RenameDialog(QWidget *parent, const QString & _caption,
@@ -91,13 +111,20 @@ RenameDialog::RenameDialog(QWidget *parent, const QString & _caption,
 
     d->src = _src;
     d->dest = _dest;
-    d->plugin = false;
 
     setWindowTitle( _caption );
 
     d->bCancel = new KPushButton( KStandardGuiItem::cancel(), this );
     connect(d->bCancel, SIGNAL(clicked()), this, SLOT(cancelPressed()));
 
+
+    if (_mode & M_MULTI) {
+        d->bApplyAll = new QCheckBox(i18n("Appl&y to All"), this);
+        d->bApplyAll->setToolTip((_mode & M_ISDIR) ? i18n("When this is checked the button pressed will be applied to all subsequent folder conflicts for the remainder of the current job.\nUnless you press Skip you will still be prompted in case of a conflict with an existing file in the directory.")
+                                                      : i18n("When this is checked the button pressed will be applied to all subsequent conflicts for the remainder of the current job."));
+        connect(d->bApplyAll, SIGNAL(clicked()), this, SLOT(applyAllPressed()));
+    }
+
     if ( ! (_mode & M_NORENAME ) ) {
         d->bRename = new QPushButton( i18n( "&Rename" ), this );
         d->bRename->setEnabled(false);
@@ -111,11 +138,6 @@ RenameDialog::RenameDialog(QWidget *parent, const QString & _caption,
         d->bSkip->setToolTip((_mode & M_ISDIR) ? i18n("Do not copy or move this folder, skip to the next item instead")
                              : i18n("Do not copy or move this file, skip to the next item instead"));
         connect(d->bSkip, SIGNAL(clicked()), this, SLOT(skipPressed()));
-
-        d->bAutoSkip = new QPushButton( i18n( "&Auto Skip" ), this );
-        d->bAutoSkip->setToolTip((_mode & M_ISDIR) ? i18n("Do not copy or move any folder that already exists in the destination folder.\nYou will be prompted again in case of a conflict with an existing file though.")
-                                 : i18n("Do not copy or move any file that already exists in the destination folder.\nYou will be prompted again in case of a conflict with an existing directory though."));
-        connect(d->bAutoSkip, SIGNAL(clicked()), this, SLOT(autoSkipPressed()));
     }
 
     if ( _mode & M_OVERWRITE ) {
@@ -123,29 +145,14 @@ RenameDialog::RenameDialog(QWidget *parent, const QString & _caption,
         d->bOverwrite = new QPushButton(text, this);
         d->bOverwrite->setToolTip(i18n("Files and folders will be copied into the existing directory, alongside its existing contents.\nYou will be prompted again in case of a conflict with an existing file in the directory."));
         connect(d->bOverwrite, SIGNAL(clicked()), this, SLOT(overwritePressed()));
-
-        if ( _mode & M_MULTI ) {
-            const QString textAll = (_mode & M_ISDIR) ? i18nc("Write files into any existing directory", "&Write Into All") : i18n("&Overwrite All");
-            d->bOverwriteAll = new QPushButton( textAll, this );
-            d->bOverwriteAll->setToolTip(i18n("Files and folders will be copied into any existing directory, alongside its existing contents.\nYou will be prompted again in case of a conflict with an existing file in a directory, but not in case of another existing directory."));
-            connect(d->bOverwriteAll, SIGNAL(clicked()), this, SLOT(overwriteAllPressed()));
-        }
     }
 
     if ( _mode & M_RESUME ) {
         d->bResume = new QPushButton( i18n( "&Resume" ), this );
         connect(d->bResume, SIGNAL(clicked()), this, SLOT(resumePressed()));
-
-        if ( _mode & M_MULTI )
-        {
-            d->bResumeAll = new QPushButton( i18n( "R&esume All" ), this );
-            connect(d->bResumeAll, SIGNAL(clicked()), this, SLOT(resumeAllPressed()));
-        }
     }
 
     QVBoxLayout* pLayout = new QVBoxLayout( this );
-    pLayout->setMargin( KDialog::marginHint() );
-    pLayout->setSpacing( KDialog::spacingHint() );
     pLayout->addStrut( 360 );	// makes dlg at least that wide
 
     // User tries to overwrite a file with itself ?
@@ -156,120 +163,94 @@ RenameDialog::RenameDialog(QWidget *parent, const QString & _caption,
         pLayout->addWidget( lb );
     }
     else if ( _mode & M_OVERWRITE ) {
+        KFileItem srcItem;
+        KFileItem destItem;
 
-        // Figure out the mimetype and load one plugin
-        // (This is the only mode that is handled by plugins)
-        pluginHandling();
-        KService::List plugin_offers;
-        if( d->mimeSrc != KMimeType::defaultMimeType()   ){
-            plugin_offers = KMimeTypeTrader::self()->query(d->mimeSrc, "RenameDialog/Plugin");
-
-        }else if(d->mimeDest != KMimeType::defaultMimeType() ) {
-            plugin_offers = KMimeTypeTrader::self()->query(d->mimeDest, "RenameDialog/Plugin");
-        }
-        if(!plugin_offers.isEmpty() ){
-            RenameDialogPlugin::FileItem src( _src, d->mimeSrc, sizeSrc, ctimeSrc, mtimeSrc );
-            RenameDialogPlugin::FileItem dst( _dest,d->mimeDest, sizeDest, ctimeDest, mtimeDest );
-            foreach (const KService::Ptr &ptr, plugin_offers) {
-                RenameDialogPlugin *plugin = ptr->createInstance<RenameDialogPlugin>(this);
-                if( !plugin )
-                    continue;
-
-                plugin->setObjectName( ptr->name() );
-                if( plugin->wantToHandle( _mode, src, dst ) ) {
-                    d->plugin = true;
-                    plugin->handle( _mode, src, dst );
-                    pLayout->addWidget(plugin );
-                    break;
-                } else {
-                    delete plugin;
-                }
-            }
+        if (d->src.isLocalFile()) {
+            srcItem = KFileItem(KFileItem::Unknown, KFileItem::Unknown, d->src);
+        } else {
+            UDSEntry srcUds;
+            srcUds.insert(UDSEntry::UDS_NAME, d->src.fileName());
+            srcUds.insert(UDSEntry::UDS_MODIFICATION_TIME, mtimeSrc);
+            srcUds.insert(UDSEntry::UDS_CREATION_TIME, ctimeSrc);
+            srcUds.insert(UDSEntry::UDS_SIZE, sizeSrc);
 
+            srcItem = KFileItem(srcUds, d->src);
         }
 
-        if( !d->plugin ){
-            // No plugin found, build default dialog
-            QGridLayout * gridLayout = new QGridLayout();
-            gridLayout->setMargin( KDialog::marginHint() );
-            gridLayout->setSpacing( KDialog::spacingHint() );
-            pLayout->addLayout(gridLayout);
-            gridLayout->setColumnStretch(0,0);
-            gridLayout->setColumnStretch(1,10);
+        if (d->dest.isLocalFile()) {
+            destItem = KFileItem(KFileItem::Unknown, KFileItem::Unknown, d->dest);
+        } else {
+            UDSEntry destUds;
+            destUds.insert(UDSEntry::UDS_NAME, d->dest.fileName());
+            destUds.insert(UDSEntry::UDS_MODIFICATION_TIME, mtimeDest);
+            destUds.insert(UDSEntry::UDS_CREATION_TIME, ctimeDest);
+            destUds.insert(UDSEntry::UDS_SIZE, sizeDest);
 
-            QString sentence1;
-            if (mtimeDest < mtimeSrc)
-                sentence1 = i18n("An older item named '%1' already exists.", d->dest.pathOrUrl());
-            else if (mtimeDest == mtimeSrc)
-                sentence1 = i18n("A similar file named '%1' already exists.", d->dest.pathOrUrl());
-            else
-                sentence1 = i18n("A newer item named '%1' already exists.", d->dest.pathOrUrl());
-
-            QLabel * lb1 = new KSqueezedTextLabel( sentence1, this );
-            gridLayout->addWidget( lb1, 0, 0, 1, 2 ); // takes the complete first line
-
-            lb1 = new QLabel( this );
-            lb1->setPixmap( KIO::pixmapForUrl( d->dest ) );
-            gridLayout->addWidget( lb1, 1, 0, 3, 1 ); // takes the first column on rows 1-3
-
-            int row = 1;
-            if ( sizeDest != (KIO::filesize_t)-1 )
-            {
-                QLabel * lb = new QLabel( i18n("size %1",  KIO::convertSize(sizeDest) ), this );
-                gridLayout->addWidget( lb, row, 1 );
-                row++;
-
-            }
-            if ( ctimeDest != (time_t)-1 )
-            {
-                QDateTime dctime; dctime.setTime_t( ctimeDest );
-                QLabel * lb = new QLabel( i18n("created on %1",  KGlobal::locale()->formatDateTime(dctime) ), this );
-                gridLayout->addWidget( lb, row, 1 );
-                row++;
-            }
-            if ( mtimeDest != (time_t)-1 )
-            {
-                QDateTime dmtime; dmtime.setTime_t( mtimeDest );
-                QLabel * lb = new QLabel( i18n("modified on %1",  KGlobal::locale()->formatDateTime(dmtime) ), this );
-                gridLayout->addWidget( lb, row, 1 );
-                row++;
-            }
-
-            if ( !d->src.isEmpty() )
-            {
-                // rows 1 to 3 are the details (size/ctime/mtime), row 4 is empty
-
-                QLabel * lb2 = new KSqueezedTextLabel( i18n("The source file is '%1'", d->src.pathOrUrl()), this );
-                gridLayout->addWidget( lb2, 5, 0, 1, 2 ); // takes the complete first line
-
-                lb2 = new QLabel( this );
-                lb2->setPixmap( KIO::pixmapForUrl( d->src ) );
-                gridLayout->addWidget( lb2, 6, 0, 3, 1 ); // takes the first column on rows 6-8
-
-                row = 6;
-
-                if ( sizeSrc != (KIO::filesize_t)-1 )
-                {
-                    QLabel * lb = new QLabel( i18n("size %1",  KIO::convertSize(sizeSrc) ), this );
-                    gridLayout->addWidget( lb, row, 1 );
-                    row++;
-                }
-                if ( ctimeSrc != (time_t)-1 )
-                {
-                    QDateTime dctime; dctime.setTime_t( ctimeSrc );
-                    QLabel * lb = new QLabel( i18n("created on %1",  KGlobal::locale()->formatDateTime(dctime) ), this );
-                    gridLayout->addWidget( lb, row, 1 );
-                    row++;
-                }
-                if ( mtimeSrc != (time_t)-1 )
-                {
-                    QDateTime dmtime; dmtime.setTime_t( mtimeSrc );
-                    QLabel * lb = new QLabel( i18n("modified on %1",  KGlobal::locale()->formatDateTime(dmtime) ), this );
-                    gridLayout->addWidget( lb, row, 1 );
-                    row++;
-                }
-            }
+            destItem = KFileItem(destUds, d->dest);
         }
+
+        d->m_srcPreview = createLabel(parent, QString());
+        d->m_destPreview = createLabel(parent, QString());
+
+        d->m_srcPreview->setMinimumHeight(KIconLoader::SizeEnormous);
+        d->m_destPreview->setMinimumHeight(KIconLoader::SizeEnormous);
+
+        d->m_srcPendingPreview = true;
+        d->m_destPendingPreview = true;
+        KIO::PreviewJob* srcJob = KIO::filePreview(KFileItemList() << srcItem,
+                                                   d->m_srcPreview->width(),
+                                                   d->m_srcPreview->height(),
+                                                   0,
+                                                   0,
+                                                   false /*don't scale*/);
+        KIO::PreviewJob* destJob = KIO::filePreview(KFileItemList() << destItem,
+                                                    d->m_destPreview->width(),
+                                                    d->m_destPreview->height(),
+                                                    0,
+                                                    0,
+                                                    false /*don't scale*/);
+
+        connect(srcJob, SIGNAL(gotPreview(const KFileItem&, const QPixmap&)),
+                this, SLOT(showSrcPreview(const KFileItem&, const QPixmap&)));
+        connect(destJob, SIGNAL(gotPreview(const KFileItem&, const QPixmap&)),
+                this, SLOT(showDestPreview(const KFileItem&, const QPixmap&)));
+        connect(srcJob, SIGNAL(failed(const KFileItem&)),
+                this, SLOT(showSrcIcon(const KFileItem&)));
+        connect(destJob, SIGNAL(failed(const KFileItem&)),
+                this, SLOT(showDestIcon(const KFileItem&)));
+
+
+        // widget
+        QScrollArea* srcWidget = createContainerLayout(parent, srcItem, d->m_srcPreview);
+        QScrollArea* destWidget = createContainerLayout(parent, destItem, d->m_destPreview);
+
+        // create layout
+        QGridLayout* gridLayout = new QGridLayout();
+        pLayout->addLayout(gridLayout);
+
+        QString sentence1;
+        if (mtimeDest < mtimeSrc)
+            sentence1 = i18n("This action will overwrite '%1' with a newer file '%2'.", d->dest.pathOrUrl(), d->src.pathOrUrl());
+        else if (mtimeDest == mtimeSrc)
+            sentence1 = i18n("This action will overwrite '%1' with a file of the same age '%2'.", d->dest.pathOrUrl(), d->src.pathOrUrl());
+        else
+            sentence1 = i18n("This action will overwrite '%1' with an older file '%2'.", d->dest.pathOrUrl(), d->src.pathOrUrl());
+
+        QLabel* titleLabel = new KSqueezedTextLabel( sentence1, this );
+
+        QLabel* srcTitle = createLabel(parent, i18n("Source"));
+        QLabel* destTitle = createLabel(parent, i18n("Destination"));
+
+        gridLayout->addWidget( titleLabel, 0, 0, 1, 2 ); // takes the complete first line
+
+        gridLayout->addWidget( srcTitle, 1, 0 );
+        gridLayout->addWidget( srcWidget, 2, 0 );
+
+        gridLayout->addWidget( destTitle, 1, 1 );
+        gridLayout->addWidget( destWidget, 2, 1 );
+
+        gridLayout->setRowStretch(1, 2);
     }
     else
     {
@@ -293,7 +274,7 @@ RenameDialog::RenameDialog(QWidget *parent, const QString & _caption,
     layout2->addWidget( d->m_pLineEdit );
     if ( d->bRename ) {
         const QString fileName = d->dest.fileName();
-        d->m_pLineEdit->setText( KIO::decodeFileName( fileName ) );
+        d->setRenameBoxText( KIO::decodeFileName( fileName ) );
         connect(d->m_pLineEdit, SIGNAL(textChanged(const QString &)),
                 SLOT(enableRenameButton(const QString &)));
         d->m_pLineEdit->setFocus();
@@ -314,6 +295,10 @@ RenameDialog::RenameDialog(QWidget *parent, const QString & _caption,
 
     layout->addStretch(1);
 
+    if (d->bApplyAll) {
+        layout->addWidget(d->bApplyAll);
+        setTabOrder(d->bApplyAll, d->bCancel);
+    }
     if ( d->bRename )
     {
         layout->addWidget( d->bRename );
@@ -324,31 +309,16 @@ RenameDialog::RenameDialog(QWidget *parent, const QString & _caption,
         layout->addWidget( d->bSkip );
         setTabOrder( d->bSkip, d->bCancel );
     }
-    if ( d->bAutoSkip )
-    {
-        layout->addWidget( d->bAutoSkip );
-        setTabOrder( d->bAutoSkip, d->bCancel );
-    }
     if ( d->bOverwrite )
     {
         layout->addWidget( d->bOverwrite );
         setTabOrder( d->bOverwrite, d->bCancel );
     }
-    if ( d->bOverwriteAll )
-    {
-        layout->addWidget( d->bOverwriteAll );
-        setTabOrder( d->bOverwriteAll, d->bCancel );
-    }
     if ( d->bResume )
     {
         layout->addWidget( d->bResume );
         setTabOrder( d->bResume, d->bCancel );
     }
-    if ( d->bResumeAll )
-    {
-        layout->addWidget( d->bResumeAll );
-        setTabOrder( d->bResumeAll, d->bCancel );
-    }
 
     d->bCancel->setDefault( true );
     layout->addWidget( d->bCancel );
@@ -387,6 +357,15 @@ KUrl RenameDialog::newDestUrl()
   return newDest;
 }
 
+KUrl RenameDialog::autoDestUrl() const
+{
+    KUrl newDest(d->dest);
+    KUrl destDirectory(d->dest);
+    destDirectory.setPath(destDirectory.directory());
+    newDest.setFileName(suggestName(destDirectory, d->dest.fileName()));
+    return newDest;
+}
+
 void RenameDialog::cancelPressed()
 {
     done( R_CANCEL );
@@ -395,17 +374,21 @@ void RenameDialog::cancelPressed()
 // Rename
 void RenameDialog::renamePressed()
 {
-  if ( d->m_pLineEdit->text().isEmpty() )
-    return;
+    if (d->m_pLineEdit->text().isEmpty()) {
+        return;
+    }
 
-  KUrl u = newDestUrl();
-  if ( !u.isValid() )
-  {
-    KMessageBox::error( this, i18n( "Malformed URL\n%1" ,  u.url() ) );
-    return;
-  }
-
-  done( R_RENAME );
+    if (d->bApplyAll  && d->bApplyAll->isChecked()) {
+        done(R_AUTO_RENAME);
+    }
+    else {
+        KUrl u = newDestUrl();
+        if (!u.isValid()) {
+            KMessageBox::error(this, i18n("Malformed URL\n%1" ,  u.url()));
+            return;
+        }
+        done(R_RENAME);
+    }
 }
 
 QString RenameDialog::suggestName(const KUrl& baseURL, const QString& oldName)
@@ -441,7 +424,7 @@ QString RenameDialog::suggestName(const KUrl& baseURL, const QString& oldName)
   // TODO: network transparency. However, using NetAccess from a modal dialog
   // could be a problem, no? (given that it uses a modal widget itself....)
   if ( baseURL.isLocalFile() )
-    exists = QFileInfo( baseURL.path(KUrl::AddTrailingSlash) + suggestedName ).exists();
+    exists = QFileInfo( baseURL.toLocalFile(KUrl::AddTrailingSlash) + suggestedName ).exists();
 
   if ( !exists )
     return suggestedName;
@@ -458,13 +441,18 @@ void RenameDialog::suggestNewNamePressed()
 
   KUrl destDirectory( d->dest );
   destDirectory.setPath( destDirectory.directory() );
-  d->m_pLineEdit->setText( suggestName( destDirectory, d->m_pLineEdit->text() ) );
+  d->setRenameBoxText( suggestName( destDirectory, d->m_pLineEdit->text() ) );
   return;
 }
 
 void RenameDialog::skipPressed()
 {
-  done( R_SKIP );
+    if (d->bApplyAll  && d->bApplyAll->isChecked()) {
+        done(R_AUTO_SKIP);
+    }
+    else {
+        done(R_SKIP);
+    }
 }
 
 void RenameDialog::autoSkipPressed()
@@ -474,7 +462,12 @@ void RenameDialog::autoSkipPressed()
 
 void RenameDialog::overwritePressed()
 {
-  done( R_OVERWRITE );
+    if (d->bApplyAll  && d->bApplyAll->isChecked()) {
+        done(R_OVERWRITE_ALL);
+    }
+    else {
+        done(R_OVERWRITE);
+    }
 }
 
 void RenameDialog::overwriteAllPressed()
@@ -484,7 +477,12 @@ void RenameDialog::overwriteAllPressed()
 
 void RenameDialog::resumePressed()
 {
-  done( R_RESUME );
+    if (d->bApplyAll  && d->bApplyAll->isChecked()) {
+        done(R_RESUME_ALL);
+    }
+    else {
+        done(R_RESUME);
+    }
 }
 
 void RenameDialog::resumeAllPressed()
@@ -492,28 +490,110 @@ void RenameDialog::resumeAllPressed()
   done( R_RESUME_ALL );
 }
 
-static QString mime( const KUrl& src )
+void RenameDialog::applyAllPressed()
 {
-  KMimeType::Ptr type = KMimeType::findByUrl( src );
-  //if( type->name() == KMimeType::defaultMimeType() ){ // ok no mimetype
-    //    QString ty = KIO::NetAccess::mimetype(d->src );
-    // return ty;
-    return type->name();
+    if (d->bApplyAll  && d->bApplyAll->isChecked()) {
+        d->m_pLineEdit->setText(KIO::decodeFileName(d->dest.fileName()));
+        d->m_pLineEdit->setEnabled(false);
+
+        if (d->bRename) {
+            d->bRename->setEnabled(true);
+        }
+        if (d->bSuggestNewName) {
+            d->bSuggestNewName->setEnabled(false);
+        }
+    }
+    else {
+        d->m_pLineEdit->setEnabled(true);
+
+        if (d->bRename) {
+            d->bRename->setEnabled(false);
+        }
+        if (d->bSuggestNewName) {
+            d->bSuggestNewName->setEnabled(true);
+        }
+    }
 }
 
-/** This will figure out the mimetypes and query for a plugin
- *  Loads it then and asks the plugin if it wants to do the job
- *  We'll take the first available mimetype
- *  The scanning for a mimetype will be done in 2 ways
- *
- */
-void RenameDialog::pluginHandling()
+void RenameDialog::showSrcIcon(const KFileItem& fileitem)
 {
-  d->mimeSrc = mime( d->src );
-  d->mimeDest = mime(d->dest );
+    // The preview job failed, show a standard file icon.
+    d->m_destPendingPreview = false;
+    d->m_destPreview->setPixmap(fileitem.pixmap(d->m_srcPreview->height()));
+}
 
-  kDebug(7024) << "Source Mimetype: "<< d->mimeSrc;
-  kDebug(7024) << "Dest Mimetype: "<< d->mimeDest;
+void RenameDialog::showDestIcon(const KFileItem& fileitem)
+{
+    // The preview job failed, show a standard file icon.
+    d->m_srcPendingPreview = false;
+    d->m_srcPreview->setPixmap(fileitem.pixmap(d->m_srcPreview->height()));
+}
+
+void RenameDialog::showSrcPreview(const KFileItem& fileitem, const QPixmap& pixmap)
+{
+    Q_UNUSED(fileitem);
+    if (d->m_srcPendingPreview) {
+        d->m_srcPreview->setPixmap(pixmap);
+        d->m_srcPendingPreview = false;
+    }
+}
+
+void RenameDialog::showDestPreview(const KFileItem& fileitem, const QPixmap& pixmap)
+{
+    Q_UNUSED(fileitem);
+    if (d->m_destPendingPreview) {
+        d->m_destPreview->setPixmap(pixmap);
+        d->m_destPendingPreview = false;
+    }
+}
+
+QScrollArea* RenameDialog::createContainerLayout(QWidget* parent, const KFileItem& item, QLabel* preview)
+{
+    KFileItemList itemList;
+    itemList << item;
+
+    // widget
+    KFileMetaDataWidget* metaWidget =  new KFileMetaDataWidget(this);
+
+    metaWidget->setReadOnly(true);
+
+    metaWidget->setItems(itemList);
+
+    // Encapsulate the MetaDataWidgets inside a container with stretch at the bottom.
+    // This prevents that the meta data widgets get vertically stretched
+    // in the case where the height of m_metaDataArea > m_metaDataWidget.
+
+    QWidget* widgetContainer = new QWidget(parent);
+    QVBoxLayout* containerLayout = new QVBoxLayout(widgetContainer);
+    containerLayout->setContentsMargins(0, 0, 0, 0);
+    containerLayout->setSpacing(0);
+    containerLayout->addWidget(preview);
+    containerLayout->addWidget(metaWidget);
+    containerLayout->addStretch(1);
+
+    QScrollArea* metaDataArea = new QScrollArea(parent);
+    metaDataArea->setWidget(widgetContainer);
+    metaDataArea->setWidgetResizable(true);
+    metaDataArea->setFrameShape(QFrame::NoFrame);
+
+    return metaDataArea;
+}
+
+QLabel* RenameDialog::createLabel(QWidget* parent, const QString& text)
+{
+    QLabel* label = new QLabel(parent);
+
+    QFont font = label->font();
+    font.setBold(true);
+    label->setFont(font);
+
+    label->setAlignment(Qt::AlignHCenter);
+
+    label->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
+
+    label->setText(text);
+
+    return label;
 }
 
 #include "renamedialog.moc"
