@@ -37,7 +37,6 @@
 #include <QtCore/QFile>
 #include <QtCore/QList>
 #include <QtCore/QDateTime>
-#include <QtDBus/QtDBus>
 
 #include <kapplication.h>
 #include <kcrash.h>
@@ -52,6 +51,7 @@
 #include "connection.h"
 #include "ioslave_defaults.h"
 #include "slaveinterface.h"
+#include "kpasswdserver_p.h"
 
 #ifndef NDEBUG
 #ifdef HAVE_BACKTRACE
@@ -580,14 +580,14 @@ static bool isSubCommand(int cmd)
 
 void SlaveBase::mimeType( const QString &_type)
 {
-  // kDebug(7019) << "(" << getpid() << ")" << _type;
+  kDebug(7019) << _type;
   int cmd;
   do
   {
     // Send the meta-data each time we send the mime-type.
     if (!mOutgoingMetaData.isEmpty())
     {
-      // kDebug(7019) << "(" << getpid() << ") mimeType: emitting meta data";
+      // kDebug(7019) << "emitting meta data";
       KIO_DATA << mOutgoingMetaData;
       send( INF_META_DATA, data );
     }
@@ -601,11 +601,11 @@ void SlaveBase::mimeType( const QString &_type)
            ret = d->appConnection.read( &cmd, data );
        }
        if (ret == -1) {
-           kDebug(7019) << "SlaveBase: mimetype: read error";
+           kDebug(7019) << "read error";
            exit();
            return;
        }
-       // kDebug(7019) << "(" << getpid() << ") Slavebase: mimetype got " << cmd;
+       // kDebug(7019) << "got" << cmd;
        if ( cmd == CMD_HOST) // Ignore.
           continue;
        if (!isSubCommand(cmd))
@@ -820,49 +820,26 @@ void SlaveBase::reparseConfiguration()
 
 bool SlaveBase::openPasswordDialog( AuthInfo& info, const QString &errorMsg )
 {
-    AuthInfo authResult;
     const long windowId = metaData("window-id").toLong();
     const unsigned long userTimestamp = metaData("user-timestamp").toULong();
-
-    kDebug(7019) << "window-id=" << windowId;
-
-    QDBusInterface kps( "org.kde.kded", "/modules/kpasswdserver", "org.kde.KPasswdServer" );
-
-    // #### TODO rewrite this with QDBusArgument streaming operators
-    // and QDBusReply.
-    QByteArray data;
-    QDataStream stream(&data, QIODevice::WriteOnly);
-    stream << info;
-    QDBusMessage reply;
-
-    if (metaData("no-auth-prompt").toLower() == "true")
-       reply = kps.call("queryAuthInfo", data, QString(QLatin1String("<NoAuthPrompt>")),
-                         qlonglong(windowId), SlaveBasePrivate::s_seqNr, qlonglong(userTimestamp));
-    else
-       reply = kps.call("queryAuthInfo", data, errorMsg, qlonglong(windowId),
-                        SlaveBasePrivate::s_seqNr, qlonglong(userTimestamp));
-
-    bool callOK = reply.type() == QDBusMessage::ReplyMessage;
-
-    if (!callOK)
-    {
-       kWarning(7019) << "Can't communicate with kded_kpasswdserver (for queryAuthInfo)!";
-       kDebug(7019) << reply.arguments().at(0).toString();
-       return false;
+    QString errorMessage;
+    if (metaData("no-auth-prompt").toLower() == "true") {
+        errorMessage = QLatin1String("<NoAuthPrompt>");
+    } else {
+        errorMessage = errorMsg;
     }
 
-    QDataStream stream2( reply.arguments().at(0).toByteArray() );
-    stream2 >> authResult;
-    SlaveBasePrivate::s_seqNr = reply.arguments().at(1).toLongLong();
+    KPasswdServer srv;
+    qlonglong seqNr = srv.queryAuthInfo(info, errorMessage, windowId,
+                                        SlaveBasePrivate::s_seqNr, userTimestamp);
+    if (seqNr > 0) {
+        SlaveBasePrivate::s_seqNr = seqNr;
+        if (info.isModified()) {
+            return true;
+        }
+    }
 
-    if (!authResult.isModified())
-       return false;
-
-    info = authResult;
-
-    kDebug(7019) << "username=" << info.username << "password=[hidden]";
-
-    return true;
+    return false;
 }
 
 int SlaveBase::messageBox( MessageBoxType type, const QString &text, const QString &caption,
@@ -935,7 +912,7 @@ int SlaveBase::waitForAnswer( int expected1, int expected2, QByteArray & data, i
         }
         else
         {
-            kWarning(7019) << "Got cmd " << cmd << " while waiting for an answer!";
+            kFatal(7019) << "Got cmd " << cmd << " while waiting for an answer!";
         }
     }
 }
@@ -1002,6 +979,7 @@ void SlaveBase::dispatch( int command, const QByteArray &data )
         disconnectSlave();
         d->isConnectedToApp = true;
         connectSlave(app_socket);
+        virtual_hook(AppConnectionMade, 0);
     } break;
     case CMD_SLAVE_HOLD: {
         KUrl url;
@@ -1190,40 +1168,8 @@ void SlaveBase::dispatch( int command, const QByteArray &data )
 
 bool SlaveBase::checkCachedAuthentication( AuthInfo& info )
 {
-    AuthInfo authResult;
-    long windowId = metaData("window-id").toLong();
-    unsigned long userTimestamp = metaData("user-timestamp").toULong();
-
-    kDebug(7019) << "window =" << windowId << "url =" << info.url;
-
-    QDBusInterface kps( "org.kde.kded", "/modules/kpasswdserver", "org.kde.KPasswdServer" );
-
-    QByteArray data;
-    {
-       QDataStream stream(&data, QIODevice::WriteOnly);
-       stream << info;
-    }
-    QDBusReply<QByteArray> reply = kps.call("checkAuthInfo", data, qlonglong(windowId), qlonglong(userTimestamp));
-
-    if ( !reply.isValid() )
-    {
-       kWarning(7019) << "Can't communicate with kded_kpasswdserver (for checkAuthInfo)!";
-       kDebug(7019) << reply.error().message();
-       return false;
-    }
-
-    data = reply.value();
-    {
-       QDataStream stream2(&data, QIODevice::ReadOnly);
-       stream2 >> authResult;
-    }
-    if (!authResult.isModified())
-    {
-       return false;
-    }
-
-    info = authResult;
-    return true;
+    return KPasswdServer().checkAuthInfo(info, metaData("window-id").toLong(),
+                                         metaData("user-timestamp").toULong());
 }
 
 void SlaveBase::dispatchOpenCommand( int command, const QByteArray &data )
@@ -1260,15 +1206,7 @@ void SlaveBase::dispatchOpenCommand( int command, const QByteArray &data )
 
 bool SlaveBase::cacheAuthentication( const AuthInfo& info )
 {
-    QByteArray params;
-    qlonglong windowId = metaData("window-id").toLongLong();
-
-    QDataStream stream(&params, QIODevice::WriteOnly);
-    stream << info;
-
-    QDBusInterface( "org.kde.kded", "/modules/kpasswdserver", "org.kde.KPasswdServer" ).
-       call("addAuthInfo", params, windowId);
-
+    KPasswdServer().addAuthInfo(info, metaData("window-id").toLongLong());
     return true;
 }
 
@@ -1349,7 +1287,7 @@ int SlaveBase::waitForHostInfo(QHostInfo& info)
 
     if (result  == -1) {
         info.setError(QHostInfo::UnknownError);
-        info.setErrorString(i18n("Unknown Error."));
+        info.setErrorString(i18n("Unknown Error"));
         return result;
     }
 
