@@ -1,9 +1,10 @@
 /*
- * Copyright (C) 2000 Alex Zepeda <zipzippy@sonic.net
+ * Copyright (C) 2000 Alex Zepeda <zipzippy@sonic.net>
  * Copyright (C) 2001-2003 George Staikos <staikos@kde.org>
  * Copyright (C) 2001 Dawit Alemayehu <adawit@kde.org>
  * Copyright (C) 2007,2008 Andreas Hartmetz <ahartmetz@gmail.com>
  * Copyright (C) 2008 Roland Harnau <tau@gmx.eu>
+ * Copyright (C) 2010 Richard Moore <rich@kde.org>
  *
  * This file is part of the KDE project
  *
@@ -105,25 +106,20 @@ class TCPSlaveBase::TcpSlaveBasePrivate
 public:
     TcpSlaveBasePrivate(TCPSlaveBase* qq) : q(qq) {}
 
-    QList<KSslError> nonIgnorableErrors(const QList<KSslError> &/*e*/) const
-    {
-        QList<KSslError> ret;
-        //TODO :)
-        return ret;
-    }
-
-    void prepareSslRelatedMetaData()
+    void setSslMetaData()
     {
+        sslMetaData.insert("ssl_in_use", "TRUE");
         KSslCipher cipher = socket.sessionCipher();
-        q->setMetaData("ssl_protocol_version", socket.negotiatedSslVersionName());
+        sslMetaData.insert("ssl_protocol_version", socket.negotiatedSslVersionName());
         QString sslCipher = cipher.encryptionMethod() + '\n';
         sslCipher += cipher.authenticationMethod() + '\n';
         sslCipher += cipher.keyExchangeMethod() + '\n';
         sslCipher += cipher.digestMethod();
-        q->setMetaData("ssl_cipher", sslCipher);
-        q->setMetaData("ssl_cipher_used_bits", QString::number(cipher.usedBits()));
-        q->setMetaData("ssl_cipher_bits", QString::number(cipher.supportedBits()));
-        q->setMetaData("ssl_peer_ip", ip);
+        sslMetaData.insert("ssl_cipher", sslCipher);
+        sslMetaData.insert("ssl_cipher_name", cipher.name());
+        sslMetaData.insert("ssl_cipher_used_bits", QString::number(cipher.usedBits()));
+        sslMetaData.insert("ssl_cipher_bits", QString::number(cipher.supportedBits()));
+        sslMetaData.insert("ssl_peer_ip", ip);
 
         // try to fill in the blanks, i.e. missing certificates, and just assume that
         // those belong to the peer (==website or similar) certificate.
@@ -136,8 +132,8 @@ public:
 
         QString errorStr;
         // encode the two-dimensional numeric error list using '\n' and '\t' as outer and inner separators
-        foreach (const QSslCertificate &cert, socket.peerCertificateChain()) {
-            foreach (const KSslError &error, sslErrors) {
+        Q_FOREACH (const QSslCertificate &cert, socket.peerCertificateChain()) {
+            Q_FOREACH (const KSslError &error, sslErrors) {
                 if (error.certificate() == cert) {
                     errorStr += QString::number(static_cast<int>(error.error())) + '\t';
                 }
@@ -148,20 +144,35 @@ public:
             errorStr += '\n';
         }
         errorStr.chop(1);
-        q->setMetaData("ssl_cert_errors", errorStr);
+        sslMetaData.insert("ssl_cert_errors", errorStr);
 
         QString peerCertChain;
-        foreach (const QSslCertificate &cert, socket.peerCertificateChain()) {
+        Q_FOREACH (const QSslCertificate &cert, socket.peerCertificateChain()) {
             peerCertChain.append(cert.toPem());
             peerCertChain.append('\x01');
         }
         peerCertChain.chop(1);
-        q->setMetaData("ssl_peer_chain", peerCertChain);
+        sslMetaData.insert("ssl_peer_chain", peerCertChain);
+        sendSslMetaData();
+    }
+    
+    void clearSslMetaData()
+    {
+        sslMetaData.clear();
+        sslMetaData.insert("ssl_in_use", "FALSE");
+        sendSslMetaData();
+    }
+    
+    void sendSslMetaData()
+    {
+        MetaData::ConstIterator it = sslMetaData.constBegin();
+        for (; it != sslMetaData.constEnd(); ++it) {
+            q->setMetaData(it.key(), it.value());
+        }
     }
 
     TCPSlaveBase* q;
 
-    int timeout;
     bool isBlocking;
 
     KTcpSocket socket;
@@ -177,6 +188,8 @@ public:
     bool sslNoUi; // If true, we just drop the connection silently
                   // if SSL certificate check fails in some way.
     QList<KSslError> sslErrors;
+    
+    MetaData sslMetaData;
 };
 
 
@@ -194,13 +207,16 @@ TCPSlaveBase::TCPSlaveBase(const QByteArray &protocol,
  : SlaveBase(protocol, poolSocket, appSocket),
    d(new TcpSlaveBasePrivate(this))
 {
-    d->timeout = KProtocolManager::connectTimeout();
     d->isBlocking = true;
     d->port = 0;
     d->serviceName = protocol;
     d->usingSSL = false;
     d->autoSSL = autoSSL;
     d->sslNoUi = false;
+    // Limit the read buffer size to 14 MB (14*1024*1024) (based on the upload limit
+    // in TransferJob::slotDataReq). See the docs for QAbstractSocket::setReadBufferSize
+    // and the BR# 187876 to understand why setting this limit is necessary.
+    d->socket.setReadBufferSize(14680064);
 }
 
 
@@ -245,7 +261,7 @@ ssize_t TCPSlaveBase::write(const char *data, ssize_t len)
 ssize_t TCPSlaveBase::read(char* data, ssize_t len)
 {
     if (d->usingSSL && (d->socket.encryptionMode() != KTcpSocket::SslClientMode)) {
-        setMetaData("ssl_in_use", "FALSE");
+        d->clearSslMetaData();
         kDebug(7029) << "lost SSL connection.";
         return -1;
     }
@@ -254,7 +270,7 @@ ssize_t TCPSlaveBase::read(char* data, ssize_t len)
         if (d->isBlocking) {
             d->socket.waitForReadyRead(-1);
         } else {
-            d->socket.waitForReadyRead(0);
+            d->socket.waitForReadyRead(readTimeout());
         }
     } else if (d->socket.encryptionMode() != KTcpSocket::SslClientMode ||
                QNetworkProxy::applicationProxy().type() == QNetworkProxy::NoProxy) {
@@ -270,26 +286,18 @@ ssize_t TCPSlaveBase::read(char* data, ssize_t len)
 ssize_t TCPSlaveBase::readLine(char *data, ssize_t len)
 {
     if (d->usingSSL && (d->socket.encryptionMode() != KTcpSocket::SslClientMode)) {
-        setMetaData("ssl_in_use", "FALSE");
+        d->clearSslMetaData();
         kDebug(7029) << "lost SSL connection.";
         return -1;
     }
 
-    //FIXME! Old client code expects waitForResponse(long time); readLine();
-    //to return error *only* on real errors even in nonblocking mode and
-    //never an incomplete line. That doesn't make sense to me.
-#ifdef PIGS_CAN_FLY
-    if (!d->isBlocking) {
-        d->socket.waitForReadyRead(0);
-        return d->socket.readLine(data, len);
-    }
-#endif
+    const int timeout = (d->isBlocking ? -1: readTimeout());
     ssize_t readTotal = 0;
     do {
         if (!d->socket.bytesAvailable())
-            d->socket.waitForReadyRead(-1);
+            d->socket.waitForReadyRead(timeout);
         ssize_t readStep = d->socket.readLine(&data[readTotal], len-readTotal);
-        if (readStep == -1) {
+        if (readStep == -1 || (readStep == 0 && d->socket.state() != KTcpSocket::ConnectedState)) {
             return -1;
         }
         readTotal += readStep;
@@ -303,7 +311,7 @@ bool TCPSlaveBase::connectToHost(const QString &/*protocol*/,
                                  const QString &host,
                                  quint16 port)
 {
-    setMetaData("ssl_in_use", "FALSE"); //We have separate connection and SSL setup phases
+    d->clearSslMetaData(); //We have separate connection and SSL setup phases
 
     //  - leaving SSL - warn before we even connect
     //### see if it makes sense to move this into the HTTP ioslave which is the only
@@ -325,6 +333,7 @@ bool TCPSlaveBase::connectToHost(const QString &/*protocol*/,
                                     "WarnOnLeaveSSLMode");
 
             if (result == KMessageBox::Cancel) {
+                error(ERR_USER_CANCELED, host);
                 return false;
             }
         }
@@ -354,7 +363,7 @@ bool TCPSlaveBase::connectToHost(const QString &/*protocol*/,
         }
 
         QListIterator<QHostAddress> it(addresses);
-        int timeout = d->timeout * 1000;
+        int timeout = connectTimeout() * 1000;
         QTime time;
         forever {
             time.start();
@@ -365,7 +374,7 @@ bool TCPSlaveBase::connectToHost(const QString &/*protocol*/,
             timeout -= time.elapsed();
             if (!it.hasNext() || (timeout < 0)) {
                 error(ERR_COULD_NOT_CONNECT,
-                    host + QLatin1String(": ") + d->socket.errorString());
+                      host + QLatin1String(": ") + d->socket.errorString());
                 return false;
             }
         }
@@ -444,6 +453,49 @@ bool TCPSlaveBase::startSsl()
     return startTLSInternal(KTcpSocket::TlsV1) & ResultOk;
 }
 
+// Find out if a hostname matches an SSL certificate's Common Name (including wildcards)
+static bool isMatchingHostname(const QString &cnIn, const QString &hostnameIn)
+{
+    const QString cn = cnIn.toLower();
+    const QString hostname = hostnameIn.toLower();
+
+    const int wildcard = cn.indexOf(QLatin1Char('*'));
+
+    // Check this is a wildcard cert, if not then just compare the strings
+    if (wildcard < 0)
+        return cn == hostname;
+
+    const int firstCnDot = cn.indexOf(QLatin1Char('.'));
+    const int secondCnDot = cn.indexOf(QLatin1Char('.'), firstCnDot+1);
+
+    // Check at least 3 components
+    if ((-1 == secondCnDot) || (secondCnDot+1 >= cn.length()))
+        return false;
+
+    // Check * is last character of 1st component (ie. there's a following .)
+    if (wildcard+1 != firstCnDot)
+        return false;
+
+    // Check only one star
+    if (cn.lastIndexOf(QLatin1Char('*')) != wildcard)
+        return false;
+
+    // Check characters preceding * (if any) match
+    if (wildcard && (hostname.leftRef(wildcard) != cn.leftRef(wildcard)))
+        return false;
+
+    // Check characters following first . match
+    if (hostname.midRef(hostname.indexOf(QLatin1Char('.'))) != cn.midRef(firstCnDot))
+        return false;
+
+    // Check if the hostname is an IP address, if so then wildcards are not allowed
+    QHostAddress addr(hostname);
+    if (!addr.isNull())
+        return false;
+
+    // Ok, I guess this was a wildcard CN and the hostname matches.
+    return true;
+}
 
 TCPSlaveBase::SslResult TCPSlaveBase::startTLSInternal(uint v_)
 {
@@ -454,7 +506,6 @@ TCPSlaveBase::SslResult TCPSlaveBase::startTLSInternal(uint v_)
     //### we don't support session reuse for now...
 
     d->usingSSL = true;
-    setMetaData("ssl_in_use", "TRUE");
 
     d->socket.setAdvertisedSslVersion(sslVersion);
 
@@ -470,13 +521,13 @@ TCPSlaveBase::SslResult TCPSlaveBase::startTLSInternal(uint v_)
     KSslCipher cipher = d->socket.sessionCipher();
 
     if (!encryptionStarted || d->socket.encryptionMode() != KTcpSocket::SslClientMode
-        || cipher.isNull() || cipher.usedBits() == 0) {
-         //TODO error(foo, bar);
+        || cipher.isNull() || cipher.usedBits() == 0 || d->socket.peerCertificateChain().isEmpty()) {
         d->usingSSL = false;
-        setMetaData("ssl_in_use", "FALSE");
+        d->clearSslMetaData();
         kDebug(7029) << "Initial SSL handshake failed. encryptionStarted is"
                      << encryptionStarted << ", cipher.isNull() is" << cipher.isNull()
                      << ", cipher.usedBits() is" << cipher.usedBits()
+                     << ", length of certificate chain is" << d->socket.peerCertificateChain().count()
                      << ", the socket says:" << d->socket.errorString()
                      << "and the list of SSL errors contains"
                      << d->socket.sslErrors().count() << "items.";
@@ -500,7 +551,6 @@ TCPSlaveBase::SslResult TCPSlaveBase::startTLSInternal(uint v_)
     QSslCertificate peerCert = d->socket.peerCertificateChain().first();
     QStringList domainPatterns(peerCert.subjectInfo(QSslCertificate::CommonName));
     domainPatterns += peerCert.alternateSubjectNames().values(QSsl::DnsEntry);
-    QRegExp domainMatcher(QString(), Qt::CaseInsensitive, QRegExp::Wildcard);
     QMutableListIterator<KSslError> it(d->sslErrors);
     while (it.hasNext()) {
         // As of 4.4.0 Qt does not assign a certificate to the QSslError it emits
@@ -510,14 +560,14 @@ TCPSlaveBase::SslResult TCPSlaveBase::startTLSInternal(uint v_)
         if (it.next().error() != KSslError::HostNameMismatch) {
             continue;
         }
-        foreach (const QString &dp, domainPatterns) {
-            domainMatcher.setPattern(dp);
-            if (domainMatcher.exactMatch(d->host)) {
+        Q_FOREACH (const QString &dp, domainPatterns) {
+            if (isMatchingHostname(dp, d->host)) {
                 it.remove();
             }
         }
     }
 
+    // TODO: review / rewrite / remove the comment
     // The app side needs the metadata now for the SSL error dialog (if any) but
     // the same metadata will be needed later, too. When "later" arrives the slave
     // may actually be connected to a different application that doesn't know
@@ -526,13 +576,13 @@ TCPSlaveBase::SslResult TCPSlaveBase::startTLSInternal(uint v_)
     // from here, for example. And Konqi will be the second application to connect
     // to the slave.
     // Therefore we choose to have our metadata and send it, too :)
-    d->prepareSslRelatedMetaData();
+    d->setSslMetaData();
     sendAndKeepMetaData();
 
     SslResult rc = verifyServerCertificate();
     if (rc & ResultFailed) {
         d->usingSSL = false;
-        setMetaData("ssl_in_use", "FALSE");
+        d->clearSslMetaData();
         kDebug(7029) << "server certificate verification failed.";
         d->socket.disconnectFromHost();     //Make the connection fail (cf. ignoreSslErrors())
         return ResultFailed;
@@ -738,61 +788,45 @@ TCPSlaveBase::SslResult TCPSlaveBase::verifyServerCertificate()
 {
     d->sslNoUi = hasMetaData("ssl_no_ui") && (metaData("ssl_no_ui") != "FALSE");
 
-    if (d->sslErrors.isEmpty())
+    if (d->sslErrors.isEmpty()) {
         return ResultOk;
-    if (d->sslNoUi)
+    } else if (d->sslNoUi) {
         return ResultFailed;
-
-    QString message = i18n("The server failed the authenticity check (%1).\n\n",
-                           d->host);
-
-    foreach (const KSslError &err, d->sslErrors) {
-        //### use our own wording that is "closer to the user"
-        message.append(err.errorString());
-        message.append('\n');
     }
 
-    //### Consider that hostname mismatch and e.g. an expired certificate are very different.
-    //    Maybe there should be no option to acceptForever a cert with bad hostname.
-
-    /* We need a list of ignorable errors. I don't think it makes sense to ignore
-       malformed certificates, for example, as other environments probably don't do
-       that either. It would be similar to a compiler trying to correct syntax errors.
-       There are also hard errors that are just impossible to override.
-      Ignorable so far, determined by watching real connection errors:
-        -UnableToGetLocalIssuerCertificate
-        -CertificateUntrusted
-        -UnableToVerifyFirstCertificate
-    */
-
-    KSslCertificateManager *const cm = KSslCertificateManager::self();
-    KSslCertificateRule rule = cm->rule(d->socket.peerCertificateChain().first(), d->host);
-
-    //TODO put nonIgnorableErrors into the cert manager
-    QList<KSslError> fatalErrors = d->nonIgnorableErrors(d->sslErrors);
+    QList<KSslError> fatalErrors = KSslCertificateManager::nonIgnorableErrors(d->sslErrors);
     if (!fatalErrors.isEmpty()) {
         //TODO message "sorry, fatal error, you can't override it"
         return ResultFailed;
     }
 
-    //throw out previously seen errors that are supposed to be ignored.
+    KSslCertificateManager *const cm = KSslCertificateManager::self();
+    KSslCertificateRule rule = cm->rule(d->socket.peerCertificateChain().first(), d->host);
+
+    // remove previously seen and acknowledged errors
     QList<KSslError> remainingErrors = rule.filterErrors(d->sslErrors);
     if (remainingErrors.isEmpty()) {
         kDebug(7029) << "Error list empty after removing errors to be ignored. Continuing.";
         return ResultOk | ResultOverridden;
     }
 
-    //### We don't present the option to permanently reject the certificate even though
-    //    it would be technically possible.
+    //### We don't ask to permanently reject the certificate
+
+    QString message = i18n("The server failed the authenticity check (%1).\n\n", d->host);
+    Q_FOREACH (const KSslError &err, d->sslErrors) {
+        message.append(err.errorString());
+        message.append('\n');
+    }
+    message = message.trimmed();
 
     int msgResult;
     do {
-        msgResult = messageBox(WarningYesNoCancel, message.trimmed(),
+        msgResult = messageBox(WarningYesNoCancel, message,
                                i18n("Server Authentication"),
                                i18n("&Details"), i18n("Co&ntinue"));
         if (msgResult == KMessageBox::Yes) {
-            //Details was chosen - we pop up the details dialog and present the choices again
-            messageBox(SSLMessageBox /*==the SSL info dialog*/, d->host);
+            //Details was chosen- show the certificate and error details
+            messageBox(SSLMessageBox /*the SSL info dialog*/, d->host);
         } else if (msgResult == KMessageBox::Cancel) {
             return ResultFailed;
         }
@@ -807,13 +841,13 @@ TCPSlaveBase::SslResult TCPSlaveBase::verifyServerCertificate()
                                  "being prompted?"),
                             i18n("Server Authentication"),
                             i18n("&Forever"),
-                            i18n("&Current Sessions Only"));
+                            i18n("&Current Session only"));
     QDateTime ruleExpiry = QDateTime::currentDateTime();
     if (msgResult == KMessageBox::Yes) {
-        //accept forever (we interpret this as "for a very long time")
+        //accept forever ("for a very long time")
         ruleExpiry = ruleExpiry.addYears(1000);
     } else {
-        //accept for a short time, say half an hour.
+        //accept "for a short time", half an hour.
         ruleExpiry = ruleExpiry.addSecs(30*60);
     }
 
@@ -918,7 +952,7 @@ TCPSlaveBase::SslResult TCPSlaveBase::verifyServerCertificate()
                     }
                 }
             } else if (cp == KSSLCertificateCache::Reject) {      // fail
-                messageBox(Information, i18n("SSL certificate is being rejected as requested. You can disable this in the KDE Control Center."),
+                messageBox(Information, i18n("SSL certificate is being rejected as requested. You can disable this in the KDE System Settings."),
                            i18n("Server Authentication"));
                 rc = -1;
                 setMetaData("ssl_action", "reject");
@@ -958,7 +992,9 @@ void TCPSlaveBase::setBlocking(bool b)
 
 void TCPSlaveBase::virtual_hook(int id, void* data)
 {
-    SlaveBase::virtual_hook(id, data);
+    if (id == SlaveBase::AppConnectionMade) {
+        d->sendSslMetaData();
+    } else {
+        SlaveBase::virtual_hook(id, data);
+    }
 }
-
-
