@@ -25,6 +25,7 @@
 #include "kdirmodeltest.moc"
 #include <kdirmodel.h>
 #include <kdirlister.h>
+//TODO #include "../../kdeui/tests/proxymodeltestsuite/modelspy.h"
 
 #include <qtest_kde.h>
 
@@ -40,6 +41,12 @@
 
 QTEST_KDEMAIN( KDirModelTest, NoGUI )
 
+#ifndef USE_QTESTEVENTLOOP
+#define exitLoop quit
+#endif
+
+#define connect(a,b,c,d) QVERIFY(QObject::connect(a,b,c,d))
+
 #ifndef Q_WS_WIN
 #define SPECIALCHARS "specialchars%:.pdf"
 #else
@@ -70,8 +77,15 @@ void KDirModelTest::initTestCase()
 
 void KDirModelTest::recreateTestData()
 {
-    delete m_tempDir;
+    if (m_tempDir) {
+        kDebug() << "Deleting old tempdir" << m_tempDir->name();
+        delete m_tempDir;
+        qApp->processEvents(); // process inotify events so they don't pollute us later on
+    }
+
+
     m_tempDir = new KTempDir;
+    kDebug() << "new tmp dir:" << m_tempDir->name();
     // Create test data:
     /*
      * PATH/toplevelfile_1
@@ -114,8 +128,10 @@ void KDirModelTest::fillModel(bool reload, bool expectAllIndexes)
     m_dirModel->dirLister()->setAutoErrorHandlingEnabled(false, 0);
     const QString path = m_tempDir->name();
     KDirLister* dirLister = m_dirModel->dirLister();
+    kDebug() << "Calling openUrl";
     dirLister->openUrl(KUrl(path), reload ? KDirLister::Reload : KDirLister::NoFlags);
     connect(dirLister, SIGNAL(completed()), this, SLOT(slotListingCompleted()));
+    kDebug() << "enterLoop, waiting for completed()";
     enterLoop();
 
     if (expectAllIndexes)
@@ -164,6 +180,7 @@ void KDirModelTest::collectKnownIndexes()
     // Now list subdir/
     QVERIFY(m_dirModel->canFetchMore(m_dirIndex));
     m_dirModel->fetchMore(m_dirIndex);
+    kDebug() << "Listing subdir/";
     enterLoop();
 
     // Index of a file inside a directory (subdir/testfile)
@@ -179,6 +196,7 @@ void KDirModelTest::collectKnownIndexes()
 
     // List subdir/subsubdir
     QVERIFY(m_dirModel->canFetchMore(subdirIndex));
+    kDebug() << "Listing subdir/subsubdir";
     m_dirModel->fetchMore(subdirIndex);
     enterLoop();
 
@@ -188,13 +206,22 @@ void KDirModelTest::collectKnownIndexes()
 
 void KDirModelTest::enterLoop()
 {
-    m_eventLoop.exec(QEventLoop::ExcludeUserInputEvents);
+#ifdef USE_QTESTEVENTLOOP
+    m_eventLoop.enterLoop(10 /*seconds max*/);
+    QVERIFY(!m_eventLoop.timeout());
+#else
+    m_eventLoop.exec();
+#endif
 }
 
 void KDirModelTest::slotListingCompleted()
 {
     kDebug();
+#ifdef USE_QTESTEVENTLOOP
+    m_eventLoop.exitLoop();
+#else
     m_eventLoop.quit();
+#endif
 }
 
 void KDirModelTest::testRowCount()
@@ -215,7 +242,7 @@ void KDirModelTest::testIndex()
 
     // Index of the first file
     QVERIFY(m_fileIndex.isValid());
-    QCOMPARE(m_fileIndex.model(), m_dirModel);
+    QCOMPARE(m_fileIndex.model(), static_cast<const QAbstractItemModel*>(m_dirModel));
     //QCOMPARE(m_fileIndex.row(), 0);
     QCOMPARE(m_fileIndex.column(), 0);
     QVERIFY(!m_fileIndex.parent().isValid());
@@ -223,7 +250,7 @@ void KDirModelTest::testIndex()
 
     // Index of a directory
     QVERIFY(m_dirIndex.isValid());
-    QCOMPARE(m_dirIndex.model(), m_dirModel);
+    QCOMPARE(m_dirIndex.model(), static_cast<const QAbstractItemModel*>(m_dirModel));
     //QCOMPARE(m_dirIndex.row(), 3);
     QCOMPARE(m_dirIndex.column(), 0);
     QVERIFY(!m_dirIndex.parent().isValid());
@@ -231,7 +258,7 @@ void KDirModelTest::testIndex()
 
     // Index of a file inside a directory (subdir/testfile)
     QVERIFY(m_fileInDirIndex.isValid());
-    QCOMPARE(m_fileInDirIndex.model(), m_dirModel);
+    QCOMPARE(m_fileInDirIndex.model(), static_cast<const QAbstractItemModel*>(m_dirModel));
     //QCOMPARE(m_fileInDirIndex.row(), 0);
     QCOMPARE(m_fileInDirIndex.column(), 0);
     QVERIFY(m_fileInDirIndex.parent() == m_dirIndex);
@@ -239,7 +266,7 @@ void KDirModelTest::testIndex()
 
     // Index of subdir/subsubdir/testfile
     QVERIFY(m_fileInSubdirIndex.isValid());
-    QCOMPARE(m_fileInSubdirIndex.model(), m_dirModel);
+    QCOMPARE(m_fileInSubdirIndex.model(), static_cast<const QAbstractItemModel*>(m_dirModel));
     //QCOMPARE(m_fileInSubdirIndex.row(), 0);
     QCOMPARE(m_fileInSubdirIndex.column(), 0);
     QVERIFY(m_fileInSubdirIndex.parent().parent() == m_dirIndex);
@@ -367,9 +394,13 @@ void KDirModelTest::testModifyFile()
     const QString file = m_tempDir->name() + "toplevelfile_2";
     const KUrl url(file);
 
+#if 1
     QSignalSpy spyDataChanged(m_dirModel, SIGNAL(dataChanged(QModelIndex, QModelIndex)));
+#else
+    ModelSpy modelSpy(m_dirModel);
+#endif
     connect( m_dirModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
-             &m_eventLoop, SLOT(quit()) );
+             &m_eventLoop, SLOT(exitLoop()) );
 
     // "Touch" the file
     setTimeStamp(file, s_referenceTimeStamp.addSecs(20) );
@@ -383,14 +414,19 @@ void KDirModelTest::testModifyFile()
     enterLoop();
 
     // If we come here, then dataChanged() was emitted - all good.
-    QCOMPARE(spyDataChanged.count(), 1);
-    QModelIndex receivedIndex = spyDataChanged[0][0].value<QModelIndex>();
+#if 0
+    QCOMPARE(modelSpy.count(), 1);
+    const QVariantList dataChanged = modelSpy.first();
+#else
+    const QVariantList dataChanged = spyDataChanged[0];
+#endif
+    QModelIndex receivedIndex = dataChanged[0].value<QModelIndex>();
     COMPARE_INDEXES(receivedIndex, m_secondFileIndex);
-    receivedIndex = spyDataChanged[0][1].value<QModelIndex>();
+    receivedIndex = dataChanged[1].value<QModelIndex>();
     QCOMPARE(receivedIndex.row(), m_secondFileIndex.row()); // only compare row; column is count-1
 
     disconnect( m_dirModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
-                &m_eventLoop, SLOT(quit()) );
+                &m_eventLoop, SLOT(exitLoop()) );
 }
 
 void KDirModelTest::testRenameFile()
@@ -400,11 +436,10 @@ void KDirModelTest::testRenameFile()
 
     QSignalSpy spyDataChanged(m_dirModel, SIGNAL(dataChanged(QModelIndex, QModelIndex)));
     connect( m_dirModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
-             &m_eventLoop, SLOT(quit()) );
+             &m_eventLoop, SLOT(exitLoop()) );
 
     KIO::SimpleJob* job = KIO::rename(url, newUrl, KIO::HideProgressInfo);
-    bool ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY(job->exec());
 
     // Wait for the DBUS signal from KDirNotify, it's the one the triggers dataChanged
     enterLoop();
@@ -420,20 +455,19 @@ void KDirModelTest::testRenameFile()
 
     // check that KDirLister::cachedItemForUrl won't give a bad name if copying that item (#195385)
     KFileItem cachedItem = KDirLister::cachedItemForUrl(newUrl);
-    Q_ASSERT(!cachedItem.isNull());
+    QVERIFY(!cachedItem.isNull());
     QCOMPARE(cachedItem.name(), QString("toplevelfile_2_renamed"));
     QCOMPARE(cachedItem.entry().stringValue(KIO::UDSEntry::UDS_NAME), QString("toplevelfile_2_renamed"));
 
     // Put things back to normal
     job = KIO::rename(newUrl, url, KIO::HideProgressInfo);
-    ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY(job->exec());
     // Wait for the DBUS signal from KDirNotify, it's the one the triggers dataChanged
     enterLoop();
     QCOMPARE( m_dirModel->itemForIndex( m_secondFileIndex ).url().url(), url.url() );
 
     disconnect( m_dirModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
-                &m_eventLoop, SLOT(quit()) );
+                &m_eventLoop, SLOT(exitLoop()) );
 }
 
 void KDirModelTest::testMoveDirectory()
@@ -451,7 +485,7 @@ void KDirModelTest::testMoveDirectory(const QString& dir /*just a dir name, no s
     QVERIFY(QDir(dest).exists());
 
     connect(m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
-            &m_eventLoop, SLOT(quit()));
+            &m_eventLoop, SLOT(exitLoop()));
 
     // Move
     kDebug() << "Moving" << srcdir << "to" << dest;
@@ -463,13 +497,13 @@ void KDirModelTest::testMoveDirectory(const QString& dir /*just a dir name, no s
     enterLoop();
 
     disconnect(m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
-               &m_eventLoop, SLOT(quit()));
+               &m_eventLoop, SLOT(exitLoop()));
 
     QVERIFY(!m_dirModel->indexForUrl(path + "subdir").isValid());
     QVERIFY(!m_dirModel->indexForUrl(path + "subdir_renamed").isValid());
 
     connect(m_dirModel, SIGNAL(rowsInserted(QModelIndex,int,int)),
-            &m_eventLoop, SLOT(quit()));
+            &m_eventLoop, SLOT(exitLoop()));
 
     // Move back
     kDebug() << "Moving" << dest+dir << "back to" << srcdir;
@@ -481,7 +515,7 @@ void KDirModelTest::testMoveDirectory(const QString& dir /*just a dir name, no s
 
     QVERIFY(QDir(srcdir).exists());
     disconnect(m_dirModel, SIGNAL(rowsInserted(QModelIndex,int,int)),
-            &m_eventLoop, SLOT(quit()));
+            &m_eventLoop, SLOT(exitLoop()));
 
     // m_dirIndex is invalid after the above...
     fillModel(true);
@@ -505,10 +539,9 @@ void KDirModelTest::testRenameDirectory() // #172945, #174703, (and #180156)
     // Now do the renaming
     QSignalSpy spyDataChanged(m_dirModel, SIGNAL(dataChanged(QModelIndex, QModelIndex)));
     connect( m_dirModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
-             &m_eventLoop, SLOT(quit()) );
+             &m_eventLoop, SLOT(exitLoop()) );
     KIO::SimpleJob* job = KIO::rename(url, newUrl, KIO::HideProgressInfo);
-    bool ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY(job->exec());
 
     // Wait for the DBUS signal from KDirNotify, it's the one the triggers dataChanged
     enterLoop();
@@ -538,14 +571,13 @@ void KDirModelTest::testRenameDirectory() // #172945, #174703, (and #180156)
 
     // Put things back to normal
     job = KIO::rename(newUrl, url, KIO::HideProgressInfo);
-    ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY(job->exec());
     // Wait for the DBUS signal from KDirNotify, it's the one the triggers dataChanged
     enterLoop();
     QCOMPARE(m_dirModel->itemForIndex(m_dirIndex).url().url(), url.url());
 
     disconnect( m_dirModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
-                &m_eventLoop, SLOT(quit()) );
+                &m_eventLoop, SLOT(exitLoop()) );
 
     QCOMPARE(m_dirModel->itemForIndex(m_dirIndex).url().url(), url.url());
     QCOMPARE(m_dirModel->indexForUrl(url), m_dirIndex);
@@ -583,13 +615,11 @@ void KDirModelTest::testRenameDirectoryInCache() // #188807
     newUrl.setPath(newUrl.path() + "_renamed");
     kDebug() << newUrl;
     KIO::SimpleJob* job = KIO::rename(url, newUrl, KIO::HideProgressInfo);
-    bool ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY(job->exec());
 
     // Put things back to normal
     job = KIO::rename(newUrl, url, KIO::HideProgressInfo);
-    ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY(job->exec());
 
     // KDirNotify emits FileRenamed for each rename() above, which in turn
     // re-lists the directory. We need to wait for both signals to be emitted
@@ -607,7 +637,7 @@ void KDirModelTest::testChmodDirectory() // #53397
 {
     QSignalSpy spyDataChanged(m_dirModel, SIGNAL(dataChanged(QModelIndex, QModelIndex)));
     connect( m_dirModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
-             &m_eventLoop, SLOT(quit()) );
+             &m_eventLoop, SLOT(exitLoop()) );
     const QString path = m_tempDir->name();
     KFileItem rootItem = m_dirModel->itemForIndex(QModelIndex());
     const mode_t origPerm = rootItem.permissions();
@@ -616,8 +646,7 @@ void KDirModelTest::testChmodDirectory() // #53397
     KFileItemList items; items << rootItem;
     KIO::Job* job = KIO::chmod(items, newPerm, S_IWGRP, QString(), QString(), false, KIO::HideProgressInfo);
     job->setUiDelegate(0);
-    bool ok = KIO::NetAccess::synchronousRun(job, 0);
-    QVERIFY(ok);
+    QVERIFY(KIO::NetAccess::synchronousRun(job, 0));
     // ChmodJob doesn't talk to KDirNotify, kpropertiesdialog does.
     // [this allows to group notifications after all the changes one can make in the dialog]
     org::kde::KDirNotify::emitFilesChanged( QStringList() << path );
@@ -633,62 +662,81 @@ void KDirModelTest::testChmodDirectory() // #53397
     QCOMPARE(m_dirModel->itemForIndex(QModelIndex()).permissions(), newPerm);
 
     disconnect( m_dirModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
-                &m_eventLoop, SLOT(quit()) );
+                &m_eventLoop, SLOT(exitLoop()) );
 }
 
+enum {
+    NoFlag = 0,
+    NewDir = 1, // whether to re-create a new KTempDir completely, to avoid cached fileitems
+    ListFinalDir = 2, // whether to list the target dir at the same time, like k3b, for #193364
+    Recreate = 4
+    // flags, next item is 8!
+};
+
 void KDirModelTest::testExpandToUrl_data()
 {
-    QTest::addColumn<bool>("newdir"); // whether to re-create a new KTempDir completely, to avoid cached fileitems
+    QTest::addColumn<int>("flags"); // see enum above
     QTest::addColumn<QString>("expandToPath"); // relative path
     QTest::addColumn<QStringList>("expectedExpandSignals");
 
     QTest::newRow("the root, nothing to do")
-        << false << QString() << QStringList();
+        << int(NoFlag) << QString() << QStringList();
     QTest::newRow(".")
-        << false << "." << (QStringList());
+        << int(NoFlag) << "." << (QStringList());
     QTest::newRow("subdir")
-        << false << "subdir" << (QStringList()<<"subdir");
+        << int(NoFlag) << "subdir" << (QStringList()<<"subdir");
     QTest::newRow("subdir/.")
-        << false << "subdir/." << (QStringList()<<"subdir");
+        << int(NoFlag) << "subdir/." << (QStringList()<<"subdir");
 
     const QString subsubdir = "subdir/subsubdir";
     // Must list root, emit expand for subdir, list subdir, emit expand for subsubdir.
     QTest::newRow("subdir/subsubdir")
-        << false << subsubdir << (QStringList()<<"subdir"<<subsubdir);
+        << int(NoFlag) << subsubdir << (QStringList()<<"subdir"<<subsubdir);
 
     // Must list root, emit expand for subdir, list subdir, emit expand for subsubdir, list subsubdir.
     const QString subsubdirfile = subsubdir + "/testfile";
     QTest::newRow("subdir/subsubdir/testfile sync")
-        << false << subsubdirfile << (QStringList()<<"subdir"<<subsubdir<<subsubdirfile);
+        << int(NoFlag) << subsubdirfile << (QStringList()<<"subdir"<<subsubdir<<subsubdirfile);
 
 #ifndef Q_WS_WIN
     // Expand a symlink to a directory (#219547)
     const QString dirlink = m_tempDir->name() + "dirlink";
     createTestSymlink(dirlink, "/");
     QTest::newRow("dirlink")
-        << false << "dirlink/tmp" << (QStringList()<<"dirlink"<<"dirlink/tmp");
+        << int(NoFlag) << "dirlink/tmp" << (QStringList()<<"dirlink"<<"dirlink/tmp");
 #endif
 
     // Do a cold-cache test too, but nowadays it doesn't change anything anymore,
     // apart from testing different code paths inside KDirLister.
     QTest::newRow("subdir/subsubdir/testfile with reload")
-        << true << subsubdirfile << (QStringList()<<"subdir"<<subsubdir<<subsubdirfile);
+        << int(NewDir) << subsubdirfile << (QStringList()<<"subdir"<<subsubdir<<subsubdirfile);
+
+    QTest::newRow("hold dest dir") // #193364
+        << int(NewDir|ListFinalDir|Recreate) << subsubdirfile << (QStringList()<<"subdir"<<subsubdir<<subsubdirfile);
+
+    // Make sure the last test has the Recreate option set, for the subsequent test methods.
 }
 
 void KDirModelTest::testExpandToUrl()
 {
-    QFETCH(bool, newdir);
+    QFETCH(int, flags);
     QFETCH(QString, expandToPath); // relative
     QFETCH(QStringList, expectedExpandSignals);
 
-    if (newdir) {
+    if (flags & NewDir) {
         recreateTestData();
         // WARNING! m_dirIndex, m_fileIndex, m_secondFileIndex etc. are not valid anymore after this point!
 
     }
 
     const QString path = m_tempDir->name();
-    if (!m_dirModelForExpand || newdir) {
+    if (flags & ListFinalDir) {
+        // This way, the last listDir will find items in cache, and will schedule a CachedItemsJob
+        m_dirModel->dirLister()->openUrl(path + "subdir/subsubdir");
+        QTest::kWaitForSignal(m_dirModel->dirLister(), SIGNAL(completed()), 2000);
+    }
+
+    if (!m_dirModelForExpand || (flags & NewDir)) {
         delete m_dirModelForExpand;
         m_dirModelForExpand = new KDirModel;
         connect(m_dirModelForExpand, SIGNAL(expand(QModelIndex)),
@@ -725,9 +773,15 @@ void KDirModelTest::testExpandToUrl()
         QVERIFY(m_dirModelForExpand->indexForUrl(m_urlToExpandTo).isValid());
     }
 
-    // recreateTestData was called -> fill again, for the next tests
-    if (newdir)
+    if (flags & ListFinalDir) {
+        testUpdateParentAfterExpand();
+    }
+
+    if (flags & Recreate) {
+        // Clean up, for the next tests
+        recreateTestData();
         fillModel(false);
+    }
 }
 
 void KDirModelTest::slotExpand(const QModelIndex& index)
@@ -747,7 +801,7 @@ void KDirModelTest::slotExpand(const QModelIndex& index)
     }
 
     if (m_nextExpectedExpandSignals == m_expectedExpandSignals.count())
-        m_eventLoop.quit(); // done
+        m_eventLoop.exitLoop(); // done
 }
 
 void KDirModelTest::slotRowsInserted(const QModelIndex&, int, int)
@@ -755,6 +809,18 @@ void KDirModelTest::slotRowsInserted(const QModelIndex&, int, int)
     m_rowsInsertedEmitted = true;
 }
 
+// This code is called by testExpandToUrl
+void KDirModelTest::testUpdateParentAfterExpand() // #193364
+{
+    const QString path = m_tempDir->name();
+    const QString file = path + "subdir/aNewFile";
+    kDebug() << "Creating" << file;
+    QVERIFY(!QFile::exists(file));
+    createTestFile(file);
+    //QSignalSpy spyRowsInserted(m_dirModelForExpand, SIGNAL(rowsInserted(QModelIndex,int,int)));
+    QTest::kWaitForSignal(m_dirModelForExpand, SIGNAL(rowsInserted(QModelIndex,int,int)));
+}
+
 void KDirModelTest::testFilter()
 {
     QVERIFY(m_dirIndex.isValid());
@@ -979,11 +1045,10 @@ void KDirModelTest::testDeleteFile()
 
     QSignalSpy spyRowsRemoved(m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)));
     connect( m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
-             &m_eventLoop, SLOT(quit()) );
+             &m_eventLoop, SLOT(exitLoop()) );
 
     KIO::DeleteJob* job = KIO::del(url, KIO::HideProgressInfo);
-    bool ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY(job->exec());
 
     // Wait for the DBUS signal from KDirNotify, it's the one the triggers rowsRemoved
     enterLoop();
@@ -995,7 +1060,7 @@ void KDirModelTest::testDeleteFile()
     QCOMPARE(spyRowsRemoved[0][1].toInt(), m_fileIndex.row());
     QCOMPARE(spyRowsRemoved[0][2].toInt(), m_fileIndex.row());
     disconnect( m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
-                &m_eventLoop, SLOT(quit()) );
+                &m_eventLoop, SLOT(exitLoop()) );
 
     QModelIndex fileIndex = m_dirModel->indexForUrl(path + "toplevelfile_1");
     QVERIFY(!fileIndex.isValid());
@@ -1012,6 +1077,45 @@ void KDirModelTest::testDeleteFile()
     fillModel(false);
 }
 
+void KDirModelTest::testDeleteFileWhileListing() // doesn't really test that yet, the kdirwatch deleted signal comes too late
+{
+    const int oldTopLevelRowCount = m_dirModel->rowCount();
+    const QString path = m_tempDir->name();
+    const QString file = path + "toplevelfile_1";
+    const KUrl url(file);
+
+    KDirLister* dirLister = m_dirModel->dirLister();
+    QSignalSpy spyCompleted(dirLister, SIGNAL(completed()));
+    connect(dirLister, SIGNAL(completed()), this, SLOT(slotListingCompleted()));
+    dirLister->openUrl(KUrl(path), KDirLister::NoFlags);
+    if (!spyCompleted.isEmpty())
+        QSKIP("listing completed too early", SkipAll);
+    QSignalSpy spyRowsRemoved(m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)));
+    KIO::DeleteJob* job = KIO::del(url, KIO::HideProgressInfo);
+    QVERIFY(job->exec());
+
+    if (spyCompleted.isEmpty())
+        enterLoop();
+    // TODO QTest::kWaitForSignalSpy(spyRowsRemoved)?
+    QTest::kWaitForSignal(m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)));
+
+    const int topLevelRowCount = m_dirModel->rowCount();
+    QCOMPARE(topLevelRowCount, oldTopLevelRowCount - 1); // one less than before
+    QCOMPARE(spyRowsRemoved.count(), 1);
+    QCOMPARE(spyRowsRemoved[0][1].toInt(), m_fileIndex.row());
+    QCOMPARE(spyRowsRemoved[0][2].toInt(), m_fileIndex.row());
+
+    QModelIndex fileIndex = m_dirModel->indexForUrl(path + "toplevelfile_1");
+    QVERIFY(!fileIndex.isValid());
+
+    kDebug() << "Test done, recreating file";
+
+    // Recreate the file, for consistency in the next tests
+    // So the second part of this test is a "testCreateFile"
+    createTestFile(file);
+    fillModel(true, false); // see testDeleteFile
+    fillModel(false);
+}
 
 void KDirModelTest::testOverwriteFileWithDir() // #151851 c4
 {
@@ -1023,9 +1127,9 @@ void KDirModelTest::testOverwriteFileWithDir() // #151851 c4
 
     QSignalSpy spyRowsRemoved(m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)));
     connect( m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
-             &m_eventLoop, SLOT(quit()) );
+             &m_eventLoop, SLOT(exitLoop()) );
 
-    KIO::Job* job = KIO::move(dir, file, KIO::Overwrite | KIO::HideProgressInfo);
+    KIO::Job* job = KIO::move(dir, file, KIO::HideProgressInfo);
     PredefinedAnswerJobUiDelegate* delegate = new PredefinedAnswerJobUiDelegate;
     delegate->m_renameResult = KIO::R_OVERWRITE;
     job->setUiDelegate(delegate);
@@ -1033,8 +1137,11 @@ void KDirModelTest::testOverwriteFileWithDir() // #151851 c4
 
     QCOMPARE(delegate->m_askFileRenameCalled, 1);
 
-    // Wait for the DBUS signal from KDirNotify, it's the one the triggers rowsRemoved
-    enterLoop();
+    if (spyRowsRemoved.isEmpty()) {
+        // Wait for the DBUS signal from KDirNotify, it's the one the triggers rowsRemoved
+        enterLoop();
+        QVERIFY(!spyRowsRemoved.isEmpty());
+    }
 
     // If we come here, then rowsRemoved() was emitted - all good.
     const int topLevelRowCount = m_dirModel->rowCount();
@@ -1046,6 +1153,8 @@ void KDirModelTest::testOverwriteFileWithDir() // #151851 c4
     KFileItem newItem = m_dirModel->itemForIndex(newIndex);
     QVERIFY(newItem.isDir()); // yes, the file is a dir now ;-)
 
+    kDebug() << "========= Test done, recreating test data =========";
+
     recreateTestData();
     fillModel(false);
 }
@@ -1060,8 +1169,7 @@ void KDirModelTest::testDeleteFiles()
     QSignalSpy spyRowsRemoved(m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)));
 
     KIO::DeleteJob* job = KIO::del(urls, KIO::HideProgressInfo);
-    bool ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY(job->exec());
 
     int numRowsRemoved = 0;
     while (numRowsRemoved < 3) {
@@ -1077,7 +1185,9 @@ void KDirModelTest::testDeleteFiles()
     const int topLevelRowCount = m_dirModel->rowCount();
     QCOMPARE(topLevelRowCount, oldTopLevelRowCount - 3); // three less than before
 
+    kDebug() << "Recreating test data";
     recreateTestData();
+    kDebug() << "Re-filling model";
     fillModel(false);
 }
 
@@ -1091,11 +1201,10 @@ void KDirModelTest::testRenameFileToHidden() // #174721
     QSignalSpy spyRowsRemoved(m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)));
     QSignalSpy spyRowsInserted(m_dirModel, SIGNAL(rowsInserted(QModelIndex,int,int)));
     connect( m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
-             &m_eventLoop, SLOT(quit()) );
+             &m_eventLoop, SLOT(exitLoop()) );
 
     KIO::SimpleJob* job = KIO::rename(url, newUrl, KIO::HideProgressInfo);
-    bool ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY(job->exec());
 
     // Wait for the DBUS signal from KDirNotify, it's the one the triggers KDirLister
     enterLoop();
@@ -1109,15 +1218,14 @@ void KDirModelTest::testRenameFileToHidden() // #174721
     QCOMPARE(row, m_secondFileIndex.row()); // only compare row
 
     disconnect(m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
-               &m_eventLoop, SLOT(quit()));
+               &m_eventLoop, SLOT(exitLoop()));
     spyRowsRemoved.clear();
 
     // Put things back to normal, should make the file reappear
     connect(m_dirModel, SIGNAL(rowsInserted(QModelIndex,int,int)),
-            &m_eventLoop, SLOT(quit()));
+            &m_eventLoop, SLOT(exitLoop()));
     job = KIO::rename(newUrl, url, KIO::HideProgressInfo);
-    ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY(job->exec());
     // Wait for the DBUS signal from KDirNotify, it's the one the triggers KDirLister
     enterLoop();
     QCOMPARE(spyDataChanged.count(), 0);
@@ -1136,13 +1244,12 @@ void KDirModelTest::testDeleteDirectory()
 
     QSignalSpy spyRowsRemoved(m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)));
     connect( m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
-             &m_eventLoop, SLOT(quit()) );
+             &m_eventLoop, SLOT(exitLoop()) );
 
     QSignalSpy spyDirWatchDeleted(KDirWatch::self(), SIGNAL(deleted(QString)));
 
     KIO::DeleteJob* job = KIO::del(url, KIO::HideProgressInfo);
-    bool ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY(job->exec());
 
     // Wait for the DBUS signal from KDirNotify, it's the one the triggers rowsRemoved
     enterLoop();
@@ -1150,14 +1257,14 @@ void KDirModelTest::testDeleteDirectory()
     // If we come here, then rowsRemoved() was emitted - all good.
     QCOMPARE(spyRowsRemoved.count(), 1);
     disconnect( m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
-                &m_eventLoop, SLOT(quit()) );
+                &m_eventLoop, SLOT(exitLoop()) );
 
     QModelIndex deletedDirIndex = m_dirModel->indexForUrl(path + "subdir/subsubdir");
     QVERIFY(!deletedDirIndex.isValid());
     QModelIndex dirIndex = m_dirModel->indexForUrl(path + "subdir");
     QVERIFY(dirIndex.isValid());
 
-    // TODO!!! Bug in KDirWatch?
+    // TODO!!! Bug in KDirWatch? ###
     // QCOMPARE(spyDirWatchDeleted.count(), 1);
 }
 
@@ -1169,11 +1276,13 @@ void KDirModelTest::testDeleteCurrentDirectory()
 
     QSignalSpy spyRowsRemoved(m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)));
     connect( m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
-             &m_eventLoop, SLOT(quit()) );
+             &m_eventLoop, SLOT(exitLoop()) );
+
+
+    KDirWatch::self()->statistics();
 
     KIO::DeleteJob* job = KIO::del(url, KIO::HideProgressInfo);
-    bool ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY(job->exec());
 
     // Wait for the DBUS signal from KDirNotify, it's the one the triggers rowsRemoved
     enterLoop();
@@ -1181,21 +1290,37 @@ void KDirModelTest::testDeleteCurrentDirectory()
     // If we come here, then rowsRemoved() was emitted - all good.
     const int topLevelRowCount = m_dirModel->rowCount();
     QCOMPARE(topLevelRowCount, 0); // empty
-    QCOMPARE(spyRowsRemoved.count(), 1);
-    QCOMPARE(spyRowsRemoved[0][1].toInt(), 0);
-    QCOMPARE(spyRowsRemoved[0][2].toInt(), oldTopLevelRowCount - 1);
+
+    // We can get rowsRemoved for subdirs first, since kdirwatch notices that.
+    QVERIFY(spyRowsRemoved.count() >= 1);
+
+    // Look for the signal(s) that had QModelIndex() as parent.
+    int i;
+    int numDeleted = 0;
+    for (i = 0; i < spyRowsRemoved.count(); ++i) {
+        const int from = spyRowsRemoved[i][1].toInt();
+        const int to = spyRowsRemoved[i][2].toInt();
+        kDebug() << spyRowsRemoved[i][0].value<QModelIndex>() << from << to;
+        if (!spyRowsRemoved[i][0].value<QModelIndex>().isValid()) {
+            numDeleted += (to - from) + 1;
+        }
+    }
+
+    QCOMPARE(numDeleted, oldTopLevelRowCount);
     disconnect( m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
-                &m_eventLoop, SLOT(quit()) );
+                &m_eventLoop, SLOT(exitLoop()) );
 
     QModelIndex fileIndex = m_dirModel->indexForUrl(path + "toplevelfile_1");
     QVERIFY(!fileIndex.isValid());
 }
 
+#if QT_VERSION < 0x040700
 // The old slow way. (this isn't QUrl's fault, I'm just using QUrl in order
 // to be able to test a different hashing function than the KUrl one).
 inline uint qHash(const QUrl& qurl) {
     return qHash(qurl.toEncoded());
 }
+#endif
 
 void KDirModelTest::testKUrlHash()
 {
