@@ -22,7 +22,10 @@
    Boston, MA 02110-1301, USA.
 */
 
+#define QT_NO_CAST_FROM_ASCII
+
 #include "file.h"
+#include <QDirIterator>
 
 #include <config.h>
 #include <config-acl.h>
@@ -35,15 +38,6 @@
 #include <sys/time.h>
 #endif
 
-//sendfile has different semantics in different platforms
-#if defined HAVE_SENDFILE && defined Q_OS_LINUX
-#define USE_SENDFILE 1
-#endif
-
-#ifdef USE_SENDFILE
-#include <sys/sendfile.h>
-#endif
-
 #ifdef HAVE_POSIX_ACL
 #include <sys/acl.h>
 #include <acl/libacl.h>
@@ -93,6 +87,7 @@
 #include <sys/mnttab.h>
 #endif
 
+#include <kdirnotify.h>
 #include <kio/ioslave_defaults.h>
 #include <kde_file.h>
 #include <kglobal.h>
@@ -115,7 +110,7 @@ extern "C" int KDE_EXPORT kdemain( int argc, char **argv )
   KComponentData componentData( "kio_file", "kdelibs4" );
   ( void ) KGlobal::locale();
 
-  kDebug(7101) << "Starting " << getpid();
+  kDebug(7101) << "Starting" << getpid();
 
   if (argc != 4)
   {
@@ -154,12 +149,12 @@ int FileProtocol::setACL( const char *path, mode_t perm, bool directoryDefault )
     int ret = 0;
 #ifdef HAVE_POSIX_ACL
 
-    const QString ACLString = metaData( "ACL_STRING" );
-    const QString defaultACLString = metaData( "DEFAULT_ACL_STRING" );
+    const QString ACLString = metaData(QLatin1String("ACL_STRING"));
+    const QString defaultACLString = metaData(QLatin1String("DEFAULT_ACL_STRING"));
     // Empty strings mean leave as is
     if ( !ACLString.isEmpty() ) {
         acl_t acl = 0;
-        if ( ACLString == "ACL_DELETE" ) {
+        if (ACLString == QLatin1String("ACL_DELETE")) {
             // user told us to delete the extended ACL, so let's write only
             // the minimal (UNIX permission bits) part
             acl = acl_from_mode( perm );
@@ -167,21 +162,21 @@ int FileProtocol::setACL( const char *path, mode_t perm, bool directoryDefault )
         acl = acl_from_text( ACLString.toLatin1() );
         if ( acl_valid( acl ) == 0 ) { // let's be safe
             ret = acl_set_file( path, ACL_TYPE_ACCESS, acl );
-            kDebug(7101) << "Set ACL on: " << path << " to: " << aclToText(acl);
+            kDebug(7101) << "Set ACL on:" << path << "to:" << aclToText(acl);
         }
         acl_free( acl );
         if ( ret != 0 ) return ret; // better stop trying right away
     }
 
     if ( directoryDefault && !defaultACLString.isEmpty() ) {
-        if ( defaultACLString == "ACL_DELETE" ) {
+        if ( defaultACLString == QLatin1String("ACL_DELETE") ) {
             // user told us to delete the default ACL, do so
             ret += acl_delete_def_file( path );
         } else {
             acl_t acl = acl_from_text( defaultACLString.toLatin1() );
             if ( acl_valid( acl ) == 0 ) { // let's be safe
                 ret += acl_set_file( path, ACL_TYPE_DEFAULT, acl );
-                kDebug(7101) << "Set Default ACL on: " << path << " to: " << aclToText(acl);
+                kDebug(7101) << "Set Default ACL on:" << path << "to:" << aclToText(acl);
             }
             acl_free( acl );
         }
@@ -196,9 +191,10 @@ int FileProtocol::setACL( const char *path, mode_t perm, bool directoryDefault )
 
 void FileProtocol::chmod( const KUrl& url, int permissions )
 {
-    QByteArray _path( QFile::encodeName(url.toLocalFile()) );
+    const QString path(url.toLocalFile());
+    const QByteArray _path( QFile::encodeName(path) );
     /* FIXME: Should be atomic */
-    if ( ::chmod( _path.data(), permissions ) == -1 ||
+    if ( KDE::chmod( path, permissions ) == -1 ||
         ( setACL( _path.data(), permissions, false ) == -1 ) ||
         /* if not a directory, cannot set default ACLs */
         ( setACL( _path.data(), permissions, true ) == -1 && errno != ENOTDIR ) ) {
@@ -206,18 +202,18 @@ void FileProtocol::chmod( const KUrl& url, int permissions )
         switch (errno) {
             case EPERM:
             case EACCES:
-                error( KIO::ERR_ACCESS_DENIED, _path );
+                error(KIO::ERR_ACCESS_DENIED, path);
                 break;
 #if defined(ENOTSUP)
             case ENOTSUP: // from setACL since chmod can't return ENOTSUP
-                error( KIO::ERR_UNSUPPORTED_ACTION, i18n( "Setting ACL for %1" ,  url.path() ) );
+                error(KIO::ERR_UNSUPPORTED_ACTION, i18n("Setting ACL for %1", path));
                 break;
 #endif
             case ENOSPC:
-                error( KIO::ERR_DISK_FULL, _path );
+                error(KIO::ERR_DISK_FULL, path);
                 break;
             default:
-        error( KIO::ERR_CANNOT_CHMOD, _path );
+                error(KIO::ERR_CANNOT_CHMOD, path);
         }
     } else
         finished();
@@ -225,41 +221,45 @@ void FileProtocol::chmod( const KUrl& url, int permissions )
 
 void FileProtocol::setModificationTime( const KUrl& url, const QDateTime& mtime )
 {
-    const QByteArray path = QFile::encodeName(url.toLocalFile());
+    const QString path(url.toLocalFile());
     KDE_struct_stat statbuf;
-    if (KDE_lstat(path, &statbuf) == 0) {
+    if (KDE::lstat(path, &statbuf) == 0) {
         struct utimbuf utbuf;
         utbuf.actime = statbuf.st_atime; // access time, unchanged
         utbuf.modtime = mtime.toTime_t(); // modification time
-        if (utime(path, &utbuf) != 0) {
+        if (KDE::utime(path, &utbuf) != 0) {
             // TODO: errno could be EACCES, EPERM, EROFS
             error(KIO::ERR_CANNOT_SETTIME, path);
         } else {
             finished();
         }
     } else {
-        error( KIO::ERR_DOES_NOT_EXIST, path );
+        error(KIO::ERR_DOES_NOT_EXIST, path);
     }
 }
 
 void FileProtocol::mkdir( const KUrl& url, int permissions )
 {
-    QByteArray _path( QFile::encodeName(url.toLocalFile()));
+    const QString path(url.toLocalFile());
 
-    kDebug(7101) << "mkdir(): " << _path << ", permission = " << permissions;
+    kDebug(7101) << path << "permission=" << permissions;
+
+    // Remove existing file or symlink, if requested (#151851)
+    if (metaData(QLatin1String("overwrite")) == QLatin1String("true"))
+        QFile::remove(path);
 
     KDE_struct_stat buff;
-    if ( KDE_lstat( _path.data(), &buff ) == -1 ) {
-        if ( KDE_mkdir( _path.data(), 0777 /*umask will be applied*/ ) != 0 ) {
+    if ( KDE::lstat( path, &buff ) == -1 ) {
+        if ( KDE::mkdir( path, 0777 /*umask will be applied*/ ) != 0 ) {
             if ( errno == EACCES ) {
-          error( KIO::ERR_ACCESS_DENIED, _path );
-          return;
+                error(KIO::ERR_ACCESS_DENIED, path);
+                return;
             } else if ( errno == ENOSPC ) {
-          error( KIO::ERR_DISK_FULL, _path );
-          return;
+                error(KIO::ERR_DISK_FULL, path);
+                return;
             } else {
-          error( KIO::ERR_COULD_NOT_MKDIR, _path );
-          return;
+                error(KIO::ERR_COULD_NOT_MKDIR, path);
+                return;
             }
         } else {
             if ( permissions != -1 )
@@ -272,10 +272,10 @@ void FileProtocol::mkdir( const KUrl& url, int permissions )
 
     if ( S_ISDIR( buff.st_mode ) ) {
         kDebug(7101) << "ERR_DIR_ALREADY_EXIST";
-        error( KIO::ERR_DIR_ALREADY_EXIST, _path );
+        error(KIO::ERR_DIR_ALREADY_EXIST, path);
         return;
     }
-    error( KIO::ERR_FILE_ALREADY_EXIST, _path );
+    error(KIO::ERR_FILE_ALREADY_EXIST, path);
     return;
 }
 
@@ -289,28 +289,28 @@ void FileProtocol::get( const KUrl& url )
 	return;
     }
 
-    QByteArray _path( QFile::encodeName(url.toLocalFile()));
+    const QString path(url.toLocalFile());
     KDE_struct_stat buff;
-    if ( KDE_stat( _path.data(), &buff ) == -1 ) {
+    if ( KDE::stat( path, &buff ) == -1 ) {
         if ( errno == EACCES )
-           error( KIO::ERR_ACCESS_DENIED, _path );
+           error(KIO::ERR_ACCESS_DENIED, path);
         else
-           error( KIO::ERR_DOES_NOT_EXIST, _path );
+           error(KIO::ERR_DOES_NOT_EXIST, path);
         return;
     }
 
     if ( S_ISDIR( buff.st_mode ) ) {
-        error( KIO::ERR_IS_DIRECTORY, _path );
+        error(KIO::ERR_IS_DIRECTORY, path);
         return;
     }
     if ( !S_ISREG( buff.st_mode ) ) {
-        error( KIO::ERR_CANNOT_OPEN_FOR_READING, _path );
+        error(KIO::ERR_CANNOT_OPEN_FOR_READING, path);
         return;
     }
 
-    int fd = KDE_open( _path.data(), O_RDONLY);
+    int fd = KDE::open( path, O_RDONLY);
     if ( fd < 0 ) {
-        error( KIO::ERR_CANNOT_OPEN_FOR_READING, _path );
+        error(KIO::ERR_CANNOT_OPEN_FOR_READING, path);
         return;
     }
 
@@ -330,7 +330,7 @@ void FileProtocol::get( const KUrl& url )
 
     KIO::filesize_t processed_size = 0;
 
-    QString resumeOffset = metaData("resume");
+    const QString resumeOffset = metaData(QLatin1String("resume"));
     if ( !resumeOffset.isEmpty() )
     {
         bool ok;
@@ -341,7 +341,7 @@ void FileProtocol::get( const KUrl& url )
             {
                 canResume ();
                 processed_size = offset;
-                kDebug( 7101 ) << "Resume offset: " << KIO::number(offset);
+                kDebug(7101) << "Resume offset:" << KIO::number(offset);
             }
         }
     }
@@ -356,7 +356,7 @@ void FileProtocol::get( const KUrl& url )
        {
           if (errno == EINTR)
               continue;
-          error( KIO::ERR_COULD_NOT_READ, _path );
+          error(KIO::ERR_COULD_NOT_READ, path);
           ::close(fd);
           return;
        }
@@ -370,7 +370,7 @@ void FileProtocol::get( const KUrl& url )
        processed_size += n;
        processedSize( processed_size );
 
-       //kDebug( 7101 ) << "Processed: " << KIO::number (processed_size);
+       //kDebug(7101) << "Processed: " << KIO::number (processed_size);
     }
 
     data( QByteArray() );
@@ -400,24 +400,24 @@ int write_all(int fd, const char *buf, size_t len)
 
 void FileProtocol::open(const KUrl &url, QIODevice::OpenMode mode)
 {
-    kDebug(7101) << "FileProtocol::open " << url.url();
+    kDebug(7101) << url;
 
-    openPath = QFile::encodeName(url.toLocalFile());
+    openPath = url.toLocalFile();
     KDE_struct_stat buff;
-    if ( KDE_stat( openPath.data(), &buff ) == -1 ) {
+    if (KDE::stat(openPath, &buff) == -1) {
         if ( errno == EACCES )
-           error( KIO::ERR_ACCESS_DENIED, openPath );
+           error(KIO::ERR_ACCESS_DENIED, openPath);
         else
-           error( KIO::ERR_DOES_NOT_EXIST, openPath );
+           error(KIO::ERR_DOES_NOT_EXIST, openPath);
         return;
     }
 
     if ( S_ISDIR( buff.st_mode ) ) {
-        error( KIO::ERR_IS_DIRECTORY, openPath );
+        error(KIO::ERR_IS_DIRECTORY, openPath);
         return;
     }
     if ( !S_ISREG( buff.st_mode ) ) {
-        error( KIO::ERR_CANNOT_OPEN_FOR_READING, openPath );
+        error(KIO::ERR_CANNOT_OPEN_FOR_READING, openPath);
         return;
     }
 
@@ -440,11 +440,11 @@ void FileProtocol::open(const KUrl &url, QIODevice::OpenMode mode)
 
     int fd = -1;
     if ( flags & O_CREAT)
-       fd = KDE_open( openPath.data(), flags, 0666);
+        fd = KDE::open( openPath, flags, 0666);
     else
-       fd = KDE_open( openPath.data(), flags);
+        fd = KDE::open( openPath, flags);
     if ( fd < 0 ) {
-        error( KIO::ERR_CANNOT_OPEN_FOR_READING, openPath );
+        error(KIO::ERR_CANNOT_OPEN_FOR_READING, openPath);
         return;
     }
     // Determine the mimetype of the file to be retrieved, and emit it.
@@ -465,7 +465,7 @@ void FileProtocol::open(const KUrl &url, QIODevice::OpenMode mode)
 
 void FileProtocol::read(KIO::filesize_t bytes)
 {
-    kDebug( 7101 ) << "File::open -- read";
+    kDebug(7101) << "File::open -- read";
     Q_ASSERT(openFd != -1);
 
     QVarLengthArray<char> buffer(bytes);
@@ -476,7 +476,7 @@ void FileProtocol::read(KIO::filesize_t bytes)
         } while (res == -1 && errno == EINTR);
 
         if (res > 0) {
-            QByteArray array = array.fromRawData(buffer.data(), res);
+            QByteArray array = QByteArray::fromRawData(buffer.data(), res);
             data( array );
             bytes -= res;
         } else {
@@ -494,16 +494,16 @@ void FileProtocol::read(KIO::filesize_t bytes)
 
 void FileProtocol::write(const QByteArray &data)
 {
-    kDebug( 7101 ) << "File::open -- write";
+    kDebug(7101) << "File::open -- write";
     Q_ASSERT(openFd != -1);
 
     if (write_all(openFd, data.constData(), data.size())) {
         if (errno == ENOSPC) { // disk full
-            error( KIO::ERR_DISK_FULL, openPath );
+            error(KIO::ERR_DISK_FULL, openPath);
             close();
         } else {
             kWarning(7101) << "Couldn't write. Error:" << strerror(errno);
-            error( KIO::ERR_COULD_NOT_WRITE, openPath );
+            error(KIO::ERR_COULD_NOT_WRITE, openPath);
             close();
         }
     } else {
@@ -513,21 +513,21 @@ void FileProtocol::write(const QByteArray &data)
 
 void FileProtocol::seek(KIO::filesize_t offset)
 {
-    kDebug( 7101 ) << "File::open -- seek";
+    kDebug(7101) << "File::open -- seek";
     Q_ASSERT(openFd != -1);
 
     int res = KDE_lseek(openFd, offset, SEEK_SET);
     if (res != -1) {
         position( offset );
     } else {
-        error(KIO::ERR_COULD_NOT_SEEK, openPath );
+        error(KIO::ERR_COULD_NOT_SEEK, openPath);
         close();
     }
 }
 
 void FileProtocol::close()
 {
-    kDebug( 7101 ) << "File::open -- close ";
+    kDebug(7101) << "File::open -- close ";
     Q_ASSERT(openFd != -1);
 
     ::close( openFd );
@@ -539,36 +539,32 @@ void FileProtocol::close()
 
 void FileProtocol::put( const KUrl& url, int _mode, KIO::JobFlags _flags )
 {
-    QString dest_orig = url.toLocalFile();
-    QByteArray _dest_orig( QFile::encodeName(dest_orig));
+    const QString dest_orig = url.toLocalFile();
 
-    kDebug(7101) << "put(): " << dest_orig << ", mode=" << _mode;
+    kDebug(7101) << dest_orig << "mode=" << _mode;
 
-    QString dest_part( dest_orig );
-    dest_part += QLatin1String(".part");
-    QByteArray _dest_part( QFile::encodeName(dest_part));
+    QString dest_part(dest_orig + QLatin1String(".part"));
 
     KDE_struct_stat buff_orig;
-    bool bOrigExists = (KDE_lstat( _dest_orig.data(), &buff_orig ) != -1);
+    const bool bOrigExists = (KDE::lstat(dest_orig, &buff_orig) != -1);
     bool bPartExists = false;
-    bool bMarkPartial = config()->readEntry("MarkPartial", true);
+    const bool bMarkPartial = config()->readEntry("MarkPartial", true);
 
     if (bMarkPartial)
     {
         KDE_struct_stat buff_part;
-        bPartExists = (KDE_stat( _dest_part.data(), &buff_part ) != -1);
+        bPartExists = (KDE::stat( dest_part, &buff_part ) != -1);
 
         if (bPartExists && !(_flags & KIO::Resume) && !(_flags & KIO::Overwrite) && buff_part.st_size > 0 && S_ISREG(buff_part.st_mode))
         {
-            kDebug(7101) << "FileProtocol::put : calling canResume with "
-                          << KIO::number(buff_part.st_size);
+            kDebug(7101) << "calling canResume with" << KIO::number(buff_part.st_size);
 
             // Maybe we can use this partial file for resuming
             // Tell about the size we have, and the app will tell us
             // if it's ok to resume or not.
             _flags |= canResume( buff_part.st_size ) ? KIO::Resume : KIO::DefaultFlags;
 
-            kDebug(7101) << "FileProtocol::put got answer " << (_flags & KIO::Resume);
+            kDebug(7101) << "got answer" << (_flags & KIO::Resume);
         }
     }
 
@@ -600,12 +596,12 @@ void FileProtocol::put( const KUrl& url, int _mode, KIO::JobFlags _flags )
             {
                 if (bMarkPartial)
                 {
-                    kDebug(7101) << "Appending .part extension to " << dest_orig;
+                    kDebug(7101) << "Appending .part extension to" << dest_orig;
                     dest = dest_part;
                     if ( bPartExists && !(_flags & KIO::Resume) )
                     {
-                        kDebug(7101) << "Deleting partial file " << dest_part;
-                        remove( _dest_part.data() );
+                        kDebug(7101) << "Deleting partial file" << dest_part;
+                        QFile::remove( dest_part );
                         // Catch errors when we try to open the file.
                     }
                 }
@@ -614,17 +610,15 @@ void FileProtocol::put( const KUrl& url, int _mode, KIO::JobFlags _flags )
                     dest = dest_orig;
                     if ( bOrigExists && !(_flags & KIO::Resume) )
                     {
-                        kDebug(7101) << "Deleting destination file " << dest_orig;
-                        remove( _dest_orig.data() );
+                        kDebug(7101) << "Deleting destination file" << dest_orig;
+                        QFile::remove( dest_orig );
                         // Catch errors when we try to open the file.
                     }
                 }
 
-                _dest = QFile::encodeName(dest);
-
                 if ( (_flags & KIO::Resume) )
                 {
-                    fd = KDE_open( _dest.data(), O_RDWR );  // append if resuming
+                    fd = KDE::open( dest, O_RDWR );  // append if resuming
                     KDE_lseek(fd, 0, SEEK_END); // Seek to end
                 }
                 else
@@ -637,17 +631,17 @@ void FileProtocol::put( const KUrl& url, int _mode, KIO::JobFlags _flags )
                     else
                         initialMode = 0666;
 
-                    fd = KDE_open(_dest.data(), O_CREAT | O_TRUNC | O_WRONLY, initialMode);
+                    fd = KDE::open(dest, O_CREAT | O_TRUNC | O_WRONLY, initialMode);
                 }
 
                 if ( fd < 0 )
                 {
-                    kDebug(7101) << "####################### COULD NOT WRITE " << dest << " _mode=" << _mode;
+                    kDebug(7101) << "####################### COULD NOT WRITE" << dest << "_mode=" << _mode;
                     kDebug(7101) << "errno==" << errno << "(" << strerror(errno) << ")";
                     if ( errno == EACCES )
-                        error( KIO::ERR_WRITE_ACCESS_DENIED, dest );
+                        error(KIO::ERR_WRITE_ACCESS_DENIED, dest);
                     else
-                        error( KIO::ERR_CANNOT_OPEN_FOR_WRITING, dest );
+                        error(KIO::ERR_CANNOT_OPEN_FOR_WRITING, dest);
                     return;
                 }
             }
@@ -656,13 +650,13 @@ void FileProtocol::put( const KUrl& url, int _mode, KIO::JobFlags _flags )
             {
                 if ( errno == ENOSPC ) // disk full
                 {
-                  error( KIO::ERR_DISK_FULL, dest_orig);
+                  error(KIO::ERR_DISK_FULL, dest_orig);
                   result = -2; // means: remove dest file
                 }
                 else
                 {
                   kWarning(7101) << "Couldn't write. Error:" << strerror(errno);
-                  error( KIO::ERR_COULD_NOT_WRITE, dest_orig);
+                  error(KIO::ERR_COULD_NOT_WRITE, dest_orig);
                   result = -1;
                 }
             }
@@ -680,7 +674,7 @@ void FileProtocol::put( const KUrl& url, int _mode, KIO::JobFlags _flags )
           ::close(fd);
 
           KDE_struct_stat buff;
-          if (bMarkPartial && KDE_stat( _dest.data(), &buff ) == 0)
+          if (bMarkPartial && KDE::stat( dest, &buff ) == 0)
           {
             int size = config()->readEntry("MinimumKeepSize", DEFAULT_MINIMUM_KEEP_SIZE);
             if (buff.st_size <  size)
@@ -700,7 +694,7 @@ void FileProtocol::put( const KUrl& url, int _mode, KIO::JobFlags _flags )
     if ( ::close(fd) )
     {
         kWarning(7101) << "Error when closing file descriptor:" << strerror(errno);
-        error( KIO::ERR_COULD_NOT_WRITE, dest_orig);
+        error(KIO::ERR_COULD_NOT_WRITE, dest_orig);
         return;
     }
 
@@ -711,45 +705,39 @@ void FileProtocol::put( const KUrl& url, int _mode, KIO::JobFlags _flags )
         // remove the symlink first. This ensures that we do not overwrite the
         // current source if the symlink points to it.
         if( (_flags & KIO::Overwrite) && S_ISLNK( buff_orig.st_mode ) )
-          remove( _dest_orig.data() );
-
-#ifdef Q_OS_WIN
-        if ( MoveFileExA( _dest.data(),
-                          _dest_orig.data(),
-                          MOVEFILE_REPLACE_EXISTING|MOVEFILE_COPY_ALLOWED ) == 0 )
-#else
-        if ( KDE_rename( _dest.data(), _dest_orig.data() ) )
-#endif
+          QFile::remove( dest_orig );
+        if ( KDE::rename( dest, dest_orig ) )
         {
-            kWarning(7101) << " Couldn't rename " << _dest << " to " << _dest_orig;
-            error( KIO::ERR_CANNOT_RENAME_PARTIAL, dest_orig );
+            kWarning(7101) << " Couldn't rename " << _dest << " to " << dest_orig;
+            error(KIO::ERR_CANNOT_RENAME_PARTIAL, dest_orig);
             return;
         }
+        org::kde::KDirNotify::emitFileRenamed(dest, dest_orig);
     }
 
     // set final permissions
     if ( _mode != -1 && !(_flags & KIO::Resume) )
     {
-        if (::chmod(_dest_orig.data(), _mode) != 0)
+        if (KDE::chmod(dest_orig, _mode) != 0)
         {
             // couldn't chmod. Eat the error if the filesystem apparently doesn't support it.
-            KMountPoint::Ptr mp = KMountPoint::currentMountPoints().findByPath(_dest_orig);
+            KMountPoint::Ptr mp = KMountPoint::currentMountPoints().findByPath(dest_orig);
             if (mp && mp->testFileSystemFlag(KMountPoint::SupportsChmod))
                  warning( i18n( "Could not change permissions for\n%1" ,  dest_orig ) );
         }
     }
 
     // set modification time
-    const QString mtimeStr = metaData( "modified" );
+    const QString mtimeStr = metaData(QLatin1String("modified"));
     if ( !mtimeStr.isEmpty() ) {
         QDateTime dt = QDateTime::fromString( mtimeStr, Qt::ISODate );
         if ( dt.isValid() ) {
             KDE_struct_stat dest_statbuf;
-            if (KDE_stat( _dest_orig.data(), &dest_statbuf ) == 0) {
+            if (KDE::stat( dest_orig, &dest_statbuf ) == 0) {
                 struct utimbuf utbuf;
                 utbuf.actime = dest_statbuf.st_atime; // access time, unchanged
                 utbuf.modtime = dt.toTime_t(); // modification time
-                utime( _dest_orig.data(), &utbuf );
+                KDE::utime( dest_orig, &utbuf );
             }
         }
 
@@ -794,10 +782,6 @@ bool FileProtocol::createUDSEntry( const QString & filename, const QByteArray &
     assert(entry.count() == 0); // by contract :-)
     // entry.reserve( 8 ); // speed up QHash insertion
 
-    // Note: details = 0 (only "file or directory or symlink or doesn't exist") isn't implemented
-    // because there's no real performance penalty in kio_file for returning the complete
-    // details. Please consider doing it in your kioslave if you're using this one as a model :)
-
     entry.insert( KIO::UDSEntry::UDS_NAME, filename );
 
     mode_t type;
@@ -843,17 +827,21 @@ bool FileProtocol::createUDSEntry( const QString & filename, const QByteArray &
     entry.insert( KIO::UDSEntry::UDS_SIZE, buff.st_size );
 
 #ifdef HAVE_POSIX_ACL
-    /* Append an atom indicating whether the file has extended acl information
-     * and if withACL is specified also one with the acl itself. If it's a directory
-     * and it has a default ACL, also append that. */
-    appendACLAtoms( path, entry, type, withACL );
+    if (details > 0) {
+        /* Append an atom indicating whether the file has extended acl information
+         * and if withACL is specified also one with the acl itself. If it's a directory
+         * and it has a default ACL, also append that. */
+        appendACLAtoms( path, entry, type, withACL );
+    }
 #endif
 
  notype:
-    entry.insert( KIO::UDSEntry::UDS_MODIFICATION_TIME, buff.st_mtime );
-    entry.insert( KIO::UDSEntry::UDS_USER, getUserName( buff.st_uid ) );
-    entry.insert( KIO::UDSEntry::UDS_GROUP, getGroupName( buff.st_gid ) );
-    entry.insert( KIO::UDSEntry::UDS_ACCESS_TIME, buff.st_atime );
+    if (details > 0) {
+        entry.insert( KIO::UDSEntry::UDS_MODIFICATION_TIME, buff.st_mtime );
+        entry.insert( KIO::UDSEntry::UDS_USER, getUserName( buff.st_uid ) );
+        entry.insert( KIO::UDSEntry::UDS_GROUP, getGroupName( buff.st_gid ) );
+        entry.insert( KIO::UDSEntry::UDS_ACCESS_TIME, buff.st_atime );
+    }
 
     // Note: buff.st_ctime isn't the creation time !
     // We made that mistake for KDE 2.0, but it's in fact the
@@ -862,71 +850,6 @@ bool FileProtocol::createUDSEntry( const QString & filename, const QByteArray &
     return true;
 }
 
-void FileProtocol::stat( const KUrl & url )
-{
-    if (!url.isLocalFile()) {
-        KUrl redir(url);
-	redir.setProtocol(config()->readEntry("DefaultRemoteProtocol", "smb"));
-	redirection(redir);
-	kDebug(7101) << "redirecting to " << redir.url();
-	finished();
-	return;
-    }
-
-    /* directories may not have a slash at the end if
-     * we want to stat() them; it requires that we
-     * change into it .. which may not be allowed
-     * stat("/is/unaccessible")  -> rwx------
-     * stat("/is/unaccessible/") -> EPERM            H.Z.
-     * This is the reason for the -1
-     */
-#ifdef Q_WS_WIN
-    QByteArray _path( QFile::encodeName(url.toLocalFile()) );
-#else
-    QByteArray _path( QFile::encodeName(url.path(KUrl::RemoveTrailingSlash)));
-#endif
-    QString sDetails = metaData(QLatin1String("details"));
-    int details = sDetails.isEmpty() ? 2 : sDetails.toInt();
-    kDebug(7101) << "FileProtocol::stat details=" << details;
-
-    UDSEntry entry;
-    if ( !createUDSEntry( url.fileName(), _path, entry, details, true /*with acls*/ ) )
-    {
-        error( KIO::ERR_DOES_NOT_EXIST, _path );
-        return;
-    }
-#if 0
-///////// debug code
-    MetaData::iterator it1 = mOutgoingMetaData.begin();
-    for ( ; it1 != mOutgoingMetaData.end(); it1++ ) {
-        kDebug(7101) << it1.key() << " = " << it1.data();
-    }
-/////////
-#endif
-    statEntry( entry );
-
-    finished();
-}
-
-/*
-void FileProtocol::testDir( const QString& path )
-{
-    QByteArray _path( QFile::encodeName(path));
-    KDE_struct_stat buff;
-    if ( KDE_stat( _path.data(), &buff ) == -1 ) {
-	error( KIO::ERR_DOES_NOT_EXIST, path );
-	return;
-    }
-
-    if ( S_ISDIR( buff.st_mode ) )
-	isDirectory();
-    else
-	isFile();
-
-    finished();
-}
-*/
-
 void FileProtocol::special( const QByteArray &data)
 {
     int tmp;
@@ -971,7 +894,7 @@ void FileProtocol::special( const QByteArray &data)
 
 void FileProtocol::mount( bool _ro, const char *_fstype, const QString& _dev, const QString& _point )
 {
-    kDebug(7101) << "FileProtocol::mount _fstype=" << _fstype;
+    kDebug(7101) << "fstype=" << _fstype;
 
 #ifdef HAVE_VOLMGT
 	/*
@@ -1008,11 +931,11 @@ void FileProtocol::mount( bool _ro, const char *_fstype, const QString& _dev, co
     tmpFile.open();
     QByteArray tmpFileName = QFile::encodeName(tmpFile.fileName());
     QByteArray dev;
-    if ( _dev.startsWith( "LABEL=" ) ) { // turn LABEL=foo into -L foo (#71430)
+    if (_dev.startsWith(QLatin1String("LABEL="))) { // turn LABEL=foo into -L foo (#71430)
         QString labelName = _dev.mid( 6 );
         dev = "-L ";
         dev += QFile::encodeName( KShell::quoteArg( labelName ) ); // is it correct to assume same encoding as filesystem?
-    } else if ( _dev.startsWith( "UUID=" ) ) { // and UUID=bar into -U bar
+    } else if (_dev.startsWith(QLatin1String("UUID="))) { // and UUID=bar into -U bar
         QString uuidName = _dev.mid( 5 );
         dev = "-U ";
         dev += QFile::encodeName( KShell::quoteArg( uuidName ) );
@@ -1022,32 +945,32 @@ void FileProtocol::mount( bool _ro, const char *_fstype, const QString& _dev, co
 
     QByteArray point = QFile::encodeName( KShell::quoteArg(_point) );
     bool fstype_empty = !_fstype || !*_fstype;
-    QByteArray fstype = KShell::quoteArg(_fstype).toLatin1(); // good guess
+    QByteArray fstype = KShell::quoteArg(QString::fromLatin1(_fstype)).toLatin1(); // good guess
     QByteArray readonly = _ro ? "-r" : "";
     QString epath = QString::fromLocal8Bit(qgetenv("PATH"));
     QString path = QLatin1String("/sbin:/bin");
     if(!epath.isEmpty())
         path += QLatin1String(":") + epath;
-    QByteArray mountProg = KGlobal::dirs()->findExe("mount", path).toLocal8Bit();
+    QByteArray mountProg = KGlobal::dirs()->findExe(QLatin1String("mount"), path).toLocal8Bit();
     if (mountProg.isEmpty()){
       error( KIO::ERR_COULD_NOT_MOUNT, i18n("Could not find program \"mount\""));
       return;
     }
-    QByteArray buffer = mountProg + ' ';
 
     // Two steps, in case mount doesn't like it when we pass all options
     for ( int step = 0 ; step <= 1 ; step++ )
     {
+        QByteArray buffer = mountProg + ' ';
         // Mount using device only if no fstype nor mountpoint (KDE-1.x like)
-        if ( !_dev.isEmpty() && _point.isEmpty() && fstype_empty )
+        if ( !dev.isEmpty() && _point.isEmpty() && fstype_empty )
             buffer += dev;
         else
           // Mount using the mountpoint, if no fstype nor device (impossible in first step)
-          if ( !_point.isEmpty() && _dev.isEmpty() && fstype_empty )
+          if ( !_point.isEmpty() && dev.isEmpty() && fstype_empty )
               buffer += point;
           else
             // mount giving device + mountpoint but no fstype
-            if ( !_point.isEmpty() && !_dev.isEmpty() && fstype_empty )
+            if ( !_point.isEmpty() && !dev.isEmpty() && fstype_empty )
                 buffer += readonly + ' ' + dev + ' ' + point;
             else
               // mount giving device + mountpoint + fstype
@@ -1075,7 +998,7 @@ void FileProtocol::mount( bool _ro, const char *_fstype, const QString& _dev, co
             // Is the device mounted ?
             if ( mp && mount_ret == 0)
             {
-                kDebug(7101) << "mount got a warning: " << err;
+                kDebug(7101) << "mount got a warning:" << err;
                 warning( err );
                 finished();
                 return;
@@ -1134,7 +1057,7 @@ void FileProtocol::unmount( const QString& _point )
 			<< _point.toLocal8Bit();
 
 		if( (mnttab = KDE_fopen( MNTTAB, "r" )) == NULL ) {
-			err = "could not open mnttab";
+			err = QLatin1String("could not open mnttab");
 			kDebug(7101) << "VOLMGT: " << err;
 			error( KIO::ERR_COULD_NOT_UNMOUNT, err );
 			return;
@@ -1157,7 +1080,7 @@ void FileProtocol::unmount( const QString& _point )
 		fclose( mnttab );
 
 		if( devname == NULL ) {
-			err = "not in mnttab";
+			err = QLatin1String("not in mnttab");
 			kDebug(7101) << "VOLMGT: "
 				<< QFile::encodeName(_point).data()
 				<< ": " << err;
@@ -1183,9 +1106,10 @@ void FileProtocol::unmount( const QString& _point )
 		if( WEXITSTATUS( system( buffer.constData() )) == 4 ) {
 			/*
 			 *  this is not an error, so skip "testLogFile()"
-			 *  to avoid wrong/confusing error popup
+			 *  to avoid wrong/confusing error popup. The
+			 *  temporary file is removed by KTemporaryFile's
+			 *  destructor, so don't do that manually.
 			 */
-			QFile::remove( tmpFileName );
 			finished();
 			return;
 		}
@@ -1205,14 +1129,14 @@ void FileProtocol::unmount( const QString& _point )
     QString epath = QString::fromLocal8Bit(qgetenv("PATH"));
     QString path = QLatin1String("/sbin:/bin");
     if (!epath.isEmpty())
-       path += ':' + epath;
-    QByteArray umountProg = KGlobal::dirs()->findExe("umount", path).toLocal8Bit();
+       path += QLatin1Char(':') + epath;
+    QByteArray umountProg = KGlobal::dirs()->findExe(QLatin1String("umount"), path).toLocal8Bit();
 
     if (umountProg.isEmpty()) {
         error( KIO::ERR_COULD_NOT_UNMOUNT, i18n("Could not find program \"umount\""));
         return;
     }
-    buffer = umountProg + QFile::encodeName(KShell::quoteArg(_point)) + " 2>" + tmpFileName;
+    buffer = umountProg + ' ' + QFile::encodeName(KShell::quoteArg(_point)) + " 2>" + tmpFileName;
     system( buffer.constData() );
 #endif /* HAVE_VOLMGT */
 
@@ -1234,8 +1158,8 @@ bool FileProtocol::pmount(const QString &dev)
     QString epath = QString::fromLocal8Bit(qgetenv("PATH"));
     QString path = QLatin1String("/sbin:/bin");
     if (!epath.isEmpty())
-        path += ':' + epath;
-    QString pmountProg = KGlobal::dirs()->findExe("pmount", path);
+        path += QLatin1Char(':') + epath;
+    QString pmountProg = KGlobal::dirs()->findExe(QLatin1String("pmount"), path);
 
     if (pmountProg.isEmpty())
         return false;
@@ -1259,8 +1183,8 @@ bool FileProtocol::pumount(const QString &point)
     QString epath = QString::fromLocal8Bit(qgetenv("PATH"));
     QString path = QLatin1String("/sbin:/bin");
     if (!epath.isEmpty())
-        path += ':' + epath;
-    QString pumountProg = KGlobal::dirs()->findExe("pumount", path);
+        path += QLatin1Char(':') + epath;
+    QString pumountProg = KGlobal::dirs()->findExe(QLatin1String("pumount"), path);
 
     if (pumountProg.isEmpty())
         return false;
@@ -1301,7 +1225,7 @@ static QString testLogFile( const QByteArray& _filename )
 	return result;
     }
 
-    result = "";
+    result.clear();
     const char *p = "";
     while ( p != 0L ) {
 	p = fgets( buffer, sizeof(buffer)-1, f );
@@ -1350,19 +1274,19 @@ static void appendACLAtoms( const QByteArray & path, UDSEntry& entry, mode_t typ
         defaultAcl = acl_get_file( path.data(), ACL_TYPE_DEFAULT );
     }
     if ( acl || defaultAcl ) {
-      kDebug(7101) << path.data() << " has extended ACL entries ";
+      kDebug(7101) << path.constData() << "has extended ACL entries";
       entry.insert( KIO::UDSEntry::UDS_EXTENDED_ACL, 1 );
     }
     if ( withACL ) {
         if ( acl ) {
             const QString str = aclToText(acl);
             entry.insert( KIO::UDSEntry::UDS_ACL_STRING, str );
-            kDebug(7101) << path.data() << "ACL: " << str;
+            kDebug(7101) << path.constData() << "ACL:" << str;
         }
         if ( defaultAcl ) {
             const QString str = aclToText(defaultAcl);
             entry.insert( KIO::UDSEntry::UDS_DEFAULT_ACL_STRING, str );
-            kDebug(7101) << path.data() << "DEFAULT ACL: " << str;
+            kDebug(7101) << path.constData() << "DEFAULT ACL:" << str;
         }
     }
     if ( acl ) acl_free( acl );
@@ -1370,4 +1294,35 @@ static void appendACLAtoms( const QByteArray & path, UDSEntry& entry, mode_t typ
 }
 #endif
 
+bool FileProtocol::deleteRecursive(const QString& path)
+{
+    //kDebug() << path;
+    QDirIterator it(path, QDir::AllEntries | QDir::NoDotAndDotDot | QDir::System | QDir::Hidden,
+                    QDirIterator::Subdirectories);
+    QStringList dirsToDelete;
+    while ( it.hasNext() ) {
+        const QString itemPath = it.next();
+        //kDebug() << "itemPath=" << itemPath;
+        const QFileInfo info = it.fileInfo();
+        if (info.isDir() && !info.isSymLink())
+            dirsToDelete.prepend(itemPath);
+        else {
+            //kDebug() << "QFile::remove" << itemPath;
+            if (!QFile::remove(itemPath)) {
+                error(KIO::ERR_CANNOT_DELETE, itemPath);
+                return false;
+            }
+        }
+    }
+    QDir dir;
+    Q_FOREACH(const QString& itemPath, dirsToDelete) {
+        //kDebug() << "QDir::rmdir" << itemPath;
+        if (!dir.rmdir(itemPath)) {
+            error(KIO::ERR_CANNOT_DELETE, itemPath);
+            return false;
+        }
+    }
+    return true;
+}
+
 #include "file.moc"
