@@ -287,7 +287,6 @@ KPageWidgetItem *DialogPrivate::createPageItem(KPageWidgetItem *parentItem,
 
 void DialogPrivate::parseGroupFile( const QString & filename )
 {
-    Q_Q(Dialog);
 	KConfig file( filename, KConfig::SimpleConfig );
     const QStringList groups = file.groupList();
     foreach (const QString &group, groups) {
@@ -339,7 +338,7 @@ void DialogPrivate::createDialogFromServices()
         if (!parent) {
             // dummy kcm
             bool foundPlugin = false;
-            foreach (KPluginInfo pinfo, plugininfos) {
+            foreach (const KPluginInfo &pinfo, plugininfos) {
                 if (pinfo.service() == info.service()) {
                     if (!pinfo.kcmServices().count()) {
                         const KService::Ptr service = info.service();
