@@ -1,6 +1,6 @@
 /*
  * This file is part of the Nepomuk KDE project.
- * Copyright (C) 2006-2008 Sebastian Trueg <trueg@kde.org>
+ * Copyright (C) 2006-2009 Sebastian Trueg <trueg@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,21 +23,27 @@
 
 #include <QtCore/QHash>
 #include <QtCore/QStringList>
-#include <QtCore/QUrl>
+
+#include <kurl.h>
+
+#include "class.h"
 
 #include "nepomuk_export.h"
 
-
 namespace Nepomuk {
 
+    class ResourceManager;
     class ResourceData;
     class Variant;
     class Tag;
     class Thing;
+    namespace Types {
+        class Property;
+    }
 
     enum ErrorCode {
         NoError = 0,
-        CommunicationError, /**< A commication error, i.e. connection failure */
+        CommunicationError, /**< A communication error, i.e. connection failure */
         InvalidType,
         UnknownError
     };
@@ -65,6 +71,23 @@ namespace Nepomuk {
      *
      * \see ResourceManager
      *
+     * \section nepomuk_resource_file_uris Special case: file URLs
+     *
+     * \p file:/ URLs are handled as a special case in Nepomuk. Starting with KDE 4.4
+     * they are no longer used to identify the Nepomuk resource but only stored as
+     * nie:url property. All resources have \p nepomuk:/res/<UUID> URIs. The Resource
+     * constructors handle this automatically. Thus, one can still use file URLs to
+     * construct the objects. But be aware of the following example:
+     *
+     * \code
+     * KUrl fileUrl("file:///home/foobar/example.txt");
+     * Nepomuk::Resource fileRes(fileUrl);
+     * QUrl fileResUri = fileRes.resourceUri();
+     * \endcode
+     *
+     * Here \p fileUrl and \p fileResUri are NOT equal. The latter is the resource URI
+     * of the form \p nepomuk:/res/<UUID>.
+     *
      * \author Sebastian Trueg <trueg@kde.org>
      */
     class NEPOMUK_EXPORT Resource
@@ -77,6 +100,21 @@ namespace Nepomuk {
          */
         Resource();
 
+        /**
+         * Creates an empty invalid Resource.
+         * An invalid resource will become valid (i.e. get a new random URI) once setProperty
+         * is called.
+         *
+         * \param manager The resource manager to use. This allows to mix resources from different
+         * managers and, thus, different models.
+         *
+         * \since 4.3
+         */
+        Resource( ResourceManager* manager );
+
+        /**
+         * Copy constructor
+         */
         Resource( const Resource& );
 
         /**
@@ -85,57 +123,59 @@ namespace Nepomuk {
          * The actual resource data is loaded on demand. Thus, it is possible to work
          * with Resources as if they were in memory all the time.
          *
-         * \param uriOrIdentifier The unique URI or an arbitrary identifier of the resource.
-         *                        If it exists as a resource URI in the RDF store it is used to load the
-         *                        related properties. If not the passed string is treated
-         *                        as an identifier.
-         *                        If a resource exists in the store which has this identifier
-         *                        set this resource's properties are loaded. Otherwise the
-         *                        resource is created in the store
-         *                        with a new random URI which can be accessed through \a uri
-         *                        after the resource has been synced. The resource can later
-         *                        again be found through the same identifier.
-         *                        In Nepomuk there are two ways of identifying a resource
-         *                        uniquely:
-         *                        \li The URI of the resource which is generated randomly
-         *                        by the framework and can only be accessed in a read-only
-         *                        fashion.
-         *                        \li An identifier in combination with the resource type (Be
-         *                        aware that this identification can only be guranteed if identifiers
-         *                        are never set manually via addIdentifier but only through the
-         *                        constructor.
+         * \param pathOrIdentifier The path to a file or an arbitrary identifier of the resource.
+         * The following cases are handled:
+         * \li A local file path is converted to a local file URL
+         * \li A URI which already exist in Nepomuk results in loading of that particular resource.
+         * \li A string which already exists as the nao:identifier of a resource results in loading
+         * of that particular resource.
+         * \li Any other string is used as nao:identifier for a new resource. This resource can
+         * later be loaded again by using the same identifier with this constructor.
          *
          * \param type The URI identifying the type of the resource. If it is empty
          *             Resource falls back to http://www.w3.org/2000/01/rdf-schema\#Resource or
          *             in case the resource already exists the type will be read from the
-         *             store. (This is a QString instead of a QUrl for historical reasons)
+         *             store.
          *
          * Example:
          *
-         * The best way to understand the URI and identifier system is through file resources.
-         * When a Resource object is created with the local path of the file as an identifier:
+         * The best way to understand the path or identifier system is through tags.
+         * There are two ways to create a resource that represents an existing tag. The first is the
+         * low level one: use the unique URI of the tag with the Resource(QUrl,QUrl) constructor.
+         * The second one is to use this constructor with the name of the tag as its identifier:
          *
          * \code
-         * Resource myfile( "/tmp/testfile.txt" );
+         * Resource myTag( "Nepomuk" );
          * \endcode
          *
-         * Now the URI of the resource in the store representing metadata for the file /tmp/testfile.txt
-         * is referred to by myfile.uri() which differs from the path of the file. However, the path of
-         * the file is saved as a \a hasIdentifier relation which means that it can be used to easily find
-         * the related resource.
+         * This will result in Resource loading the tag with nao:identifier "Nepomuk".
          */
-        Resource( const QString& uriOrIdentifier, const QUrl& type = QUrl() );
+        Resource( const QString& pathOrIdentifier, const QUrl& type = QUrl() );
+
+        /**
+         * \overload
+         *
+         * \param manager The resource manager to use. This allows to mix resources from different
+         * managers and, thus, different models.
+         *
+         * \since 4.3
+         */
+        Resource( const QString& pathOrIdentifier, const QUrl& type, ResourceManager* manager );
 
         /**
          * \deprecated use Resource( const QString&, const QUrl& )
          */
-        KDE_DEPRECATED Resource( const QString& uriOrIdentifier, const QString& type );
+        KDE_DEPRECATED Resource( const QString& pathOrIdentifier, const QString& type );
 
         /**
          * Creates a new Resource object.
          *
          * \param uri The URI of the resource. If no resource with this URI exists, a new one is
-         * created.
+         * created. Using an empty QUrl will result in a new resource with a random URI being created
+         * on the first call to setProperty.
+         *
+         * See the \ref nepomuk_resource_file_uris Special file URL handling.
+         *
          * \param type The URI identifying the type of the resource. If it is empty
          *             Resource falls back to http://www.w3.org/2000/01/rdf-schema\#Resource or
          *             in case the resource already exists the type will be read from the
@@ -143,23 +183,49 @@ namespace Nepomuk {
          */
         Resource( const QUrl& uri, const QUrl& type = QUrl() );
 
+        /**
+         * \overload
+         *
+         * \param manager The resource manager to use. This allows to mix resources from different
+         * managers and, thus, different models.
+         *
+         * \since 4.3
+         */
+        Resource( const QUrl& uri, const QUrl& type, ResourceManager* manager );
+
         /**
          * Constructor used internally.
          */
         Resource( ResourceData* );
 
+        /**
+         * Destructor
+         */
         virtual ~Resource();
 
-        Resource& operator=( const Resource& );
+        /**
+         * Makes this instance of Resource a copy of other.
+         */
+        Resource& operator=( const Resource& other );
 
-        Resource& operator=( const QUrl& );
+        /**
+         * Same as operator=( Resource( uri ) )
+         */
+        Resource& operator=( const QUrl& uri );
+
+        /**
+         * The Resource manager that manages this resource.
+         *
+         * \since 4.3
+         */
+        ResourceManager* manager() const;
 
         /**
          * The URI of the resource, uniquely identifying it. This URI in most
          * cases is a virtual one which has been created from a generic base
          * namespace and some identifier.
          *
-         * the most important thing to remember is that the URI of for example
+         * The most important thing to remember is that the URI of for example
          * a file does not necessarily have a relation to its local path.
          * (Although Nepomuk tries to keep the URI of file resources in sync
          * with the file URL for convenience.)
@@ -167,9 +233,9 @@ namespace Nepomuk {
          * For historical reasons the method does return a URI as QString instead
          * of QUrl. The value equals resourceUri().toString().
          *
-         * \sa resourceUri, getIdentifiers
+         * \sa resourceUri(), getIdentifiers()
          *
-         * \deprecated use resourceUri instead
+         * \deprecated use resourceUri() instead
          */
         KDE_DEPRECATED QString uri() const;
 
@@ -178,12 +244,13 @@ namespace Nepomuk {
          * cases is a virtual one which has been created from a generic base
          * namespace and some identifier.
          *
-         * the most important thing to remember is that the URI of for example
+         * The most important thing to remember is that the URI of for example
          * a file does not necessarily have a relation to its local path.
-         * (Although Nepomuk tries to keep the URI of file resources in sync
-         * with the file URL for convenience.)
          *
-         * \sa uri, getIdentifiers
+         * \return The resource URI of the resource or an empty url if the
+         * resource does not exist() yet.
+         *
+         * \sa getIdentifiers()
          */
         QUrl resourceUri() const;
 
@@ -272,6 +339,13 @@ namespace Nepomuk {
          */
         bool hasProperty( const QUrl& uri ) const;
 
+        /**
+         * Check if the resource has a property \p p with value \p v.
+         *
+         * \since 4.5
+         */
+        bool hasProperty( const Types::Property& p, const Variant& v ) const;
+
         /**
          * \deprecated use hasProperty( const QUrl& ) const
          */
@@ -298,6 +372,16 @@ namespace Nepomuk {
          */
         void setProperty( const QUrl& uri, const Variant& value );
 
+        /**
+         * Add a property value to the existing values.
+         *
+         * \param uri The URI identifying the property.
+         * \param value The value of the property (i.e. the object of the RDF triple(s))
+         *
+         * \since 4.3
+         */
+        void addProperty( const QUrl& uri, const Variant& value );
+
         /**
          * \deprecated use setProperty( const QUrl& )
          */
@@ -310,6 +394,16 @@ namespace Nepomuk {
          */
         void removeProperty( const QUrl& uri );
 
+        /**
+         * Remove \a value from property \a uri of this resource object.
+         *
+         * \param uri The URI identifying the property.
+         * \param value The value to remove
+         *
+         * \since 4.3
+         */
+        void removeProperty( const QUrl& uri, const Variant& value );
+
         /**
          * \deprecated use removeProperty( const QUrl& )
          */
@@ -374,254 +468,296 @@ namespace Nepomuk {
         Thing pimoThing();
 
         /**
-         * Operator to compare two Resource objects. Normally one does not need this. It is
-         * mainly intended for testing and debugging purposes.
+         * Operator to compare two Resource objects.
          */
         bool operator==( const Resource& ) const;
 
         /**
-         * Get property 'description'. Everything can be annotated with 
-         * a simple string comment. 
+         * Operator to compare two Resource objects.
+         *
+         * \since 4.4
+         */
+        bool operator!=( const Resource& ) const;
+
+        /**
+         * Get property 'description'. Everything can be annotated with
+         * a simple string comment.
          */
         QString description() const;
 
         /**
-         * Set property 'description'. Everything can be annotated with 
-         * a simple string comment. 
+         * Set property 'description'. Everything can be annotated with
+         * a simple string comment.
          */
         void setDescription( const QString& value );
 
         /**
-         * \return The URI of the property 'description'. 
+         * \return The URI of the property 'description'.
          */
         static QString descriptionUri();
 
         /**
-         * Get property 'identifier'. 
+         * Get property 'identifier'.
          */
         QStringList identifiers() const;
 
         /**
-         * Set property 'identifier'. 
+         * Set property 'identifier'.
          */
         void setIdentifiers( const QStringList& value );
 
         /**
-         * Add a value to property 'identifier'. 
+         * Add a value to property 'identifier'.
          */
         void addIdentifier( const QString& value );
 
         /**
-         * \return The URI of the property 'identifier'. 
+         * \return The URI of the property 'identifier'.
          */
         static QString identifierUri();
 
         /**
-         * Get property 'altLabel'. 
+         * Get property 'altLabel'.
          */
         QStringList altLabels() const;
 
         /**
-         * Set property 'altLabel'. 
+         * Set property 'altLabel'.
          */
         void setAltLabels( const QStringList& value );
 
         /**
-         * Add a value to property 'altLabel'. 
+         * Add a value to property 'altLabel'.
          */
         void addAltLabel( const QString& value );
 
         /**
-         * \return The URI of the property 'altLabel'. 
+         * \return The URI of the property 'altLabel'.
          */
         static QString altLabelUri();
 
         /**
-         * Get property 'annotation'. 
+         * Get property 'annotation'.
          */
         QList<Resource> annotations() const;
 
         /**
-         * Set property 'annotation'. 
+         * Set property 'annotation'.
          */
         void setAnnotations( const QList<Resource>& value );
 
         /**
-         * Add a value to property 'annotation'. 
+         * Add a value to property 'annotation'.
          */
         void addAnnotation( const Resource& value );
 
         /**
-         * \return The URI of the property 'annotation'. 
+         * \return The URI of the property 'annotation'.
          */
         static QString annotationUri();
 
         /**
-         * Get property 'Tag'. Each Resource can be tagged with an arbitrary 
-         * number of Tags. This allows a simple grouping of resources. 
+         * Get property 'Tag'. Each Resource can be tagged with an arbitrary
+         * number of Tags. This allows a simple grouping of resources.
          */
         QList<Tag> tags() const;
 
         /**
-         * Set property 'Tag'. Each Resource can be tagged with an arbitrary 
-         * number of Tags. This allows a simple grouping of resources. 
+         * Set property 'Tag'. Each Resource can be tagged with an arbitrary
+         * number of Tags. This allows a simple grouping of resources.
          */
         void setTags( const QList<Tag>& value );
 
         /**
-         * Add a value to property 'Tag'. Each Resource can be tagged with 
-         * an arbitrary number of Tags. This allows a simple grouping of 
-         * resources. 
+         * Add a value to property 'Tag'. Each Resource can be tagged with
+         * an arbitrary number of Tags. This allows a simple grouping of
+         * resources.
          */
         void addTag( const Tag& value );
 
         /**
-         * \return The URI of the property 'Tag'. 
+         * \return The URI of the property 'Tag'.
          */
         static QString tagUri();
 
         /**
-         * Get property 'Topic'. 
+         * Get property 'Topic'.
          */
         QList<Resource> topics() const;
 
         /**
-         * Set property 'Topic'. 
+         * Set property 'Topic'.
          */
         void setTopics( const QList<Resource>& value );
 
         /**
-         * Add a value to property 'Topic'. 
+         * Add a value to property 'Topic'.
          */
         void addTopic( const Resource& value );
 
         /**
-         * \return The URI of the property 'Topic'. 
+         * \return The URI of the property 'Topic'.
          */
         static QString topicUri();
 
         /**
-         * Get property 'isTopicOf'. 
+         * Get property 'isTopicOf'.
          */
         QList<Resource> isTopicOfs() const;
 
         /**
-         * Set property 'isTopicOf'. 
+         * Set property 'isTopicOf'.
          */
         void setIsTopicOfs( const QList<Resource>& value );
 
         /**
-         * Add a value to property 'isTopicOf'. 
+         * Add a value to property 'isTopicOf'.
          */
         void addIsTopicOf( const Resource& value );
 
         /**
-         * \return The URI of the property 'isTopicOf'. 
+         * \return The URI of the property 'isTopicOf'.
          */
         static QString isTopicOfUri();
 
         /**
-         * Get property 'isRelated'. 
+         * Get property 'isRelated'.
          */
         QList<Resource> isRelateds() const;
 
         /**
-         * Set property 'isRelated'. 
+         * Set property 'isRelated'.
          */
         void setIsRelateds( const QList<Resource>& value );
 
         /**
-         * Add a value to property 'isRelated'. 
+         * Add a value to property 'isRelated'.
          */
         void addIsRelated( const Resource& value );
 
         /**
-         * \return The URI of the property 'isRelated'. 
+         * \return The URI of the property 'isRelated'.
          */
         static QString isRelatedUri();
 
         /**
-         * Get property 'label'. 
+         * Get property 'label'.
          */
         QString label() const;
 
         /**
-         * Set property 'label'. 
+         * Set property 'label'.
          */
         void setLabel( const QString& value );
 
         /**
-         * \return The URI of the property 'label'. 
+         * \return The URI of the property 'label'.
          */
         static QString labelUri();
 
         /**
-         * Get property 'Rating'. 
+         * Get property 'Rating'.
          */
         quint32 rating() const;
 
         /**
-         * Set property 'Rating'. 
+         * Set property 'Rating'.
          */
         void setRating( const quint32& value );
 
         /**
-         * \return The URI of the property 'Rating'. 
+         * \return The URI of the property 'Rating'.
          */
         static QString ratingUri();
 
         /**
-         * Get property 'Symbol'. Each resource can have a symbol assigned. 
-         * For now this is a simple string which can either be the patch to 
-         * an actual pixmap file or just the name of an icon as defined by 
-         * the freedesktop.org standard. 
+         * Get property 'Symbol'. Each resource can have a symbol assigned.
+         * For now this is a simple string which can either be the path to
+         * an actual pixmap file or just the name of an icon as defined by
+         * the freedesktop.org standard.
          */
         QStringList symbols() const;
 
         /**
-         * Set property 'Symbol'. Each resource can have a symbol assigned. 
-         * For now this is a simple string which can either be the patch to 
-         * an actual pixmap file or just the name of an icon as defined by 
-         * the freedesktop.org standard. 
+         * Set property 'Symbol'. Each resource can have a symbol assigned.
+         * For now this is a simple string which can either be the path to
+         * an actual pixmap file or just the name of an icon as defined by
+         * the freedesktop.org standard.
          */
         void setSymbols( const QStringList& value );
 
         /**
-         * Add a value to property 'Symbol'. Each resource can have a symbol 
-         * assigned. For now this is a simple string which can either be 
-         * the patch to an actual pixmap file or just the name of an icon as 
-         * defined by the freedesktop.org standard. 
+         * Add a value to property 'Symbol'. Each resource can have a symbol
+         * assigned. For now this is a simple string which can either be
+         * the path to an actual pixmap file or just the name of an icon as
+         * defined by the freedesktop.org standard.
          */
         void addSymbol( const QString& value );
 
         /**
-         * \return The URI of the property 'Symbol'. 
+         * \return The URI of the property 'Symbol'.
          */
         static QString symbolUri();
 
         /**
-         * Get all resources that have this resource set as property 'annotation'. 
-         * \sa ResourceManager::allResourcesWithProperty 
+         * Get all resources that have this resource set as property 'annotation'.
+         * \sa ResourceManager::allResourcesWithProperty
          */
         QList<Resource> annotationOf() const;
 
         /**
-         * Get all resources that have this resource set as property 'isRelated'. 
-         * \sa ResourceManager::allResourcesWithProperty 
+         * Get all resources that have this resource set as property 'isRelated'.
+         * \sa ResourceManager::allResourcesWithProperty
          */
         QList<Resource> isRelatedOf() const;
 
         /**
-         * Retrieve a list of all available Resource resources. This list 
-         * consists of all resource of type Resource that are stored in 
-         * the local Nepomuk meta data storage and any changes made locally. 
-         * Be aware that in some cases this list can get very big. Then it 
-         * might be better to use libKNep directly. 
+         * Retrieve a list of all available Resource resources. This list
+         * consists of all resource of type Resource that are stored in
+         * the local Nepomuk meta data storage and any changes made locally.
+         * Be aware that in some cases this list can get very big. Then it
+         * might be better to use libKNep directly.
+         *
+         * \sa ResourceManager::allResources
          */
         static QList<Resource> allResources();
 
+        /**
+         * \return The usage count stored for this resource.
+         *
+         * \sa increaseUsageCount()
+         *
+         * \since 4.5
+         */
+        int usageCount() const;
+
+        /**
+         * Increase the usage count of this resource and also
+         * update the last used date to the current date and time.
+         *
+         * \since 4.5
+         */
+        void increaseUsageCount();
+
+        /**
+         * Allows to quickly load a resource from its resource URI without any
+         * additional checks. This is mostly used for optimized code within Nepomuk.
+         *
+         * In most situations the construtor Resource( QUrl, QUrl ) is better suited.
+         *
+         * \since 4.5
+         */
+        static Resource fromResourceUri( const KUrl& uri, const Nepomuk::Types::Class& type = Nepomuk::Types::Class(), ResourceManager* manager = 0 );
 
     private:
+        /**
+         * Determines the final ResourceData and updates m_data if
+         * necessary. This will call ResourceData::determineUri()
+         * and optionally merge with already loaded ResourceData
+         * instances representing the same resource.
+         */
+        void determineFinalResourceData() const;
+
         ResourceData* m_data;
 
         class Private;
@@ -629,6 +765,8 @@ namespace Nepomuk {
 
         friend class ResourceData;
     };
+
+    NEPOMUK_EXPORT uint qHash( const Resource& res );
 }
 
 #endif
