@@ -34,6 +34,7 @@
 #include <plasma/version.h>
 
 class QAction;
+class QMimeData;
 
 class KCompletion;
 
@@ -280,12 +281,9 @@ class PLASMA_EXPORT AbstractRunner : public QObject
         friend class RunnerManager;
         friend class RunnerManagerPrivate;
 
-        /**
-         * Constructs a Runner object. Since AbstractRunner has pure virtuals,
-         * this constructor can not be called directly. Rather a subclass must
-         * be created
-         */
-        explicit AbstractRunner(QObject *parent = 0, const QString &serviceId = QString());
+        explicit AbstractRunner(QObject *parent = 0, const QString &path = QString());
+        explicit AbstractRunner(const KService::Ptr service, QObject *parent = 0);
+
         AbstractRunner(QObject *parent, const QVariantList &args);
 
         /**
@@ -443,6 +441,13 @@ class PLASMA_EXPORT AbstractRunner : public QObject
     protected Q_SLOTS:
         void init();
 
+        /**
+         * Reimplement this slot if you want your runner
+         * to support serialization and drag and drop
+         * @since 4.5
+         */
+        QMimeData * mimeDataForMatch(const Plasma::QueryMatch *match);
+
     private:
         friend class RunnerScript;
 
