@@ -22,6 +22,9 @@
 
 #include "applet.h"
 #include "private/applet_p.h"
+
+#include "config-plasma.h"
+
 #include <plasma/animations/animation.h>
 
 #include <cmath>
@@ -50,7 +53,6 @@
 #include <kactioncollection.h>
 #include <kauthorized.h>
 #include <kcolorscheme.h>
-#include <kconfigdialog.h>
 #include <kdialog.h>
 #include <kicon.h>
 #include <kiconloader.h>
@@ -63,12 +65,12 @@
 #include <kwindowsystem.h>
 #include <kpushbutton.h>
 #include <krun.h>
-#include "kio/jobclasses.h" // for KIO::JobFlags
-#include "kio/job.h"
-
 
+#ifndef PLASMA_NO_SOLID
 #include <solid/powermanagement.h>
+#endif
 
+#include "abstracttoolbox.h"
 #include "authorizationmanager.h"
 #include "authorizationrule.h"
 #include "configloader.h"
@@ -84,8 +86,9 @@
 #include "svg.h"
 #include "framesvg.h"
 #include "popupapplet.h"
-#include "private/framesvg_p.h"
 #include "private/applethandle_p.h"
+#include "private/extenderitem_p.h"
+#include "private/framesvg_p.h"
 #include "theme.h"
 #include "view.h"
 #include "widgets/iconwidget.h"
@@ -95,6 +98,7 @@
 #include "tooltipmanager.h"
 #include "wallpaper.h"
 #include "paintutils.h"
+#include "abstractdialogmanager.h"
 
 #include "private/associatedapplicationmanager_p.h"
 #include "private/authorizationmanager_p.h"
@@ -106,10 +110,8 @@
 #include "private/popupapplet_p.h"
 #include "private/remotedataengine_p.h"
 #include "private/service_p.h"
-#include "private/internaltoolbox_p.h"
 #include "ui_publish.h"
 
-#include "config-plasma.h"
 
 namespace Plasma
 {
@@ -177,6 +179,14 @@ Applet::Applet(QObject *parentObject, const QVariantList &args)
     // inflexibility of KService::createInstance
 }
 
+Applet::Applet(const QString &packagePath, uint appletId, const QVariantList &args)
+    : QGraphicsWidget(0),
+      d(new AppletPrivate(KService::Ptr(new KService(packagePath + "/metadata.desktop")), appletId, this))
+{
+    Q_UNUSED(args) // FIXME?
+    d->init(packagePath);
+}
+
 Applet::~Applet()
 {
     //let people know that i will die
@@ -215,6 +225,7 @@ PackageStructure::Ptr Applet::packageStructure()
 
 void Applet::init()
 {
+    setFlag(ItemIsMovable, true);
     if (d->script) {
         d->setupScriptSupport();
 
@@ -352,7 +363,6 @@ void Applet::setFailedToLaunch(bool failed, const QString &reason)
     d->failed = failed;
     prepareGeometryChange();
 
-
     foreach (QGraphicsItem *item, childItems()) {
         if (!dynamic_cast<AppletHandle *>(item)) {
             delete item;
@@ -401,6 +411,7 @@ void Applet::saveState(KConfigGroup &group) const
     if (d->script) {
         emit d->script->saveState(group);
     }
+
     if (group.config()->name() != config().config()->name()) {
         // we're being saved to a different file!
         // let's just copy the current values in our configuration over
@@ -459,8 +470,7 @@ void Applet::destroy()
     if (isContainment()) {
         d->cleanUpAndDelete();
     } else {
-        Animation *zoomAnim =
-        Plasma::Animator::create(Plasma::Animator::ZoomAnimation);
+        Animation *zoomAnim = Plasma::Animator::create(Plasma::Animator::ZoomAnimation);
         connect(zoomAnim, SIGNAL(finished()), this, SLOT(appletAnimationComplete()));
         zoomAnim->setTargetWidget(this);
         zoomAnim->start();
@@ -780,17 +790,12 @@ QString Applet::name() const
 {
     if (d->isContainment) {
         const Containment *c = qobject_cast<const Containment*>(this);
-        if (c && !c->activity().isNull()) {
-            return i18n("%1 Activity", c->activity());
-        }
-
-        if (!d->appletDescription.isValid()) {
-            return i18n("Unknown Activity");
-        } else if (c && (c->d->type == Containment::PanelContainment ||
-                   c->d->type == Containment::CustomPanelContainment)) {
-            return d->appletDescription.name();
+        if (c && c->d->isPanelContainment()) {
+            return i18n("Panel");
+        } else if (!d->appletDescription.isValid()) {
+            return i18n("Unknown");
         } else {
-            return i18n("%1 Activity", d->appletDescription.name());
+            return d->appletDescription.name();
         }
     } else if (!d->appletDescription.isValid()) {
         return i18n("Unknown Widget");
@@ -824,7 +829,11 @@ QString Applet::pluginName() const
 
 bool Applet::shouldConserveResources() const
 {
+#ifndef PLASMA_NO_SOLID
     return Solid::PowerManagement::appShouldConserveResources();
+#else
+    return true;
+#endif
 }
 
 QString Applet::category() const
@@ -850,10 +859,6 @@ QString Applet::category(const QString &appletName)
     const QString constraint = QString("[X-KDE-PluginInfo-Name] == '%1'").arg(appletName);
     KService::List offers = KServiceTypeTrader::self()->query("Plasma/Applet", constraint);
 
-    if (offers.isEmpty()) {
-        offers = KServiceTypeTrader::self()->query("Plasma/PopupApplet", constraint);
-    }
-
     if (offers.isEmpty()) {
         return QString();
     }
@@ -881,7 +886,7 @@ ImmutabilityType Applet::immutability() const
 
 void Applet::setImmutability(const ImmutabilityType immutable)
 {
-    if (d->immutability == immutable) {
+    if (d->immutability == immutable || d->immutability == Plasma::SystemImmutable) {
         return;
     }
 
@@ -907,6 +912,7 @@ void Applet::setBackgroundHints(const BackgroundHints hints)
     if ((hints & StandardBackground) || (hints & TranslucentBackground)) {
         if (!d->background) {
             d->background = new Plasma::FrameSvg(this);
+            QObject::connect(d->background, SIGNAL(repaintNeeded()), this, SLOT(themeChanged()));
         }
 
         if ((hints & TranslucentBackground) &&
@@ -1003,7 +1009,13 @@ void Applet::setConfigurationRequired(bool needsConfig, const QString &reason)
     }
 
     PushButton *configWidget = new PushButton(d->messageOverlay);
-    configWidget->setText(i18n("Configure..."));
+    if (!qobject_cast<Plasma::PopupApplet *>(this) && (formFactor() == Plasma::Horizontal || formFactor() == Plasma::Vertical)) {
+        configWidget->setImage("widgets/configuration-icons", "configure");
+        configWidget->setMaximumSize(24,24);
+        configWidget->setMinimumSize(24,24);
+    } else {
+        configWidget->setText(i18n("Configure..."));
+    }
     connect(configWidget, SIGNAL(clicked()), this, SLOT(showConfigurationInterface()));
     configLayout->addItem(configWidget, row, 1);
 
@@ -1103,6 +1115,15 @@ void Applet::showMessage(const QIcon &icon, const QString &message, const Messag
         KWindowSystem::setState(d->messageDialog.data()->winId(), NET::SkipTaskbar | NET::SkipPager);
         d->messageDialog.data()->setGraphicsWidget(mainWidget);
         connect(d->messageDialog.data(), SIGNAL(destroyed(QObject*)), mainWidget, SLOT(deleteLater()));
+
+        // if we are going to show it in a popup, then at least make sure it can be dismissed
+        if (buttonLayout->count() < 1) {
+            PushButton *ok = new PushButton(mainWidget);
+            ok->setText(i18n("OK"));
+            ok->setIcon(KIcon("dialog-ok"));
+            buttonLayout->addItem(ok);
+            connect(ok, SIGNAL(clicked()), this, SLOT(destroyMessageOverlay()));
+        }
     } else {
         delete d->messageDialog.data();
         d->createMessageOverlay();
@@ -1176,9 +1197,11 @@ void Applet::flushPendingConstraintsEvents()
                 connect(configAction, SIGNAL(triggered(bool)), this, SLOT(showConfigurationInterface()), Qt::UniqueConnection);
             }
 
-            bool canConfig = unlocked || KAuthorized::authorize("plasma/allow_configure_when_locked");
-            configAction->setVisible(canConfig);
-            configAction->setEnabled(canConfig);
+            if (d->hasConfigurationInterface) {
+                bool canConfig = unlocked || KAuthorized::authorize("plasma/allow_configure_when_locked");
+                configAction->setVisible(canConfig);
+                configAction->setEnabled(canConfig);
+            }
         }
 
         QAction *runAssociatedApplication = d->actions->action("run associated application");
@@ -1202,11 +1225,18 @@ void Applet::flushPendingConstraintsEvents()
         }
 
         action = d->actions->action("configure");
-        if (action) {
+        if (action && d->hasConfigurationInterface) {
             bool canConfig = unlocked || KAuthorized::authorize("plasma/allow_configure_when_locked");
             action->setVisible(canConfig);
             action->setEnabled(canConfig);
         }
+
+        if (d->extender) {
+            foreach (ExtenderItem *item, d->extender.data()->attachedItems()) {
+                item->d->setMovable(unlocked);
+            }
+        }
+
         emit immutabilityChanged(immutability());
     }
 
@@ -1235,14 +1265,38 @@ void Applet::flushPendingConstraintsEvents()
                 delete item;
             }
         }
+
+        // avoid putting rotated applets in panels
+        if (f == Vertical || f == Horizontal) {
+            QTransform at;
+            at.rotateRadians(0);
+            setTransform(at);
+        }
+
+        //was a size saved for a particular form factor?
+        if (d->sizeForFormFactor.contains(f)) {
+            resize(d->sizeForFormFactor.value(f));
+        }
     }
+
+    if (!size().isEmpty() &&
+        ((c & Plasma::StartupCompletedConstraint) || (c & Plasma::SizeConstraint && !(c & Plasma::FormFactorConstraint)))) {
+        d->sizeForFormFactor[formFactor()] = size();
+    }
+
     if (c & Plasma::SizeConstraint || c & Plasma::FormFactorConstraint) {
         if (aspectRatioMode() == Plasma::Square || aspectRatioMode() == Plasma::ConstrainedSquare) {
             // enforce square size in panels
+            //save the old size policy. since ignored doesn't (yet) have a valid use case in containments, use it as special unset value
+            if (d->preferredSizePolicy == QSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored)) {
+                d->preferredSizePolicy = sizePolicy();
+            }
             if (formFactor() == Horizontal) {
                 setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Expanding));
             } else if (formFactor() == Vertical) {
                 setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed));
+            } else if (d->preferredSizePolicy != QSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored)) {
+                setSizePolicy(d->preferredSizePolicy);
             }
         }
         updateGeometry();
@@ -1329,17 +1383,6 @@ void Applet::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QW
         View* v = qobject_cast<Plasma::View*>(widget->parent());
         Containment* c = qobject_cast<Plasma::Containment*>(this);
 
-        //update the view transform of the toolbox, since it ignores transforms
-        if (v && c && c->d->toolBox) {
-            InternalToolBox *toolBox = qobject_cast<InternalToolBox *>(c->d->toolBox.data());
-            if (toolBox && toolBox->viewTransform().isScaling() && !v->transform().isScaling()) {
-                c->d->positionToolBox();
-            }
-            if (toolBox && v) {
-                toolBox->setViewTransform(v->transform());
-            }
-        }
-
         if (!v || v->isWallpaperEnabled()) {
 
             // paint the wallpaper
@@ -1362,8 +1405,11 @@ void Applet::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QW
 
             // .. and now paint the actual containment interface, but with
             //  a Containment style option based on the one we get
+            //  the view must be assigned only if its containment is actually our own
             Containment::StyleOption coption(*option);
-            coption.view = v;
+            if (v && v->containment() == containment()) {
+                coption.view = v;
+            }
             paintInterface(painter, &coption, contentsRect);
         }
     } else {
@@ -1385,18 +1431,37 @@ void Applet::paintInterface(QPainter *painter, const QStyleOptionGraphicsItem *o
 FormFactor Applet::formFactor() const
 {
     Containment *c = containment();
-    QGraphicsWidget *p = dynamic_cast<QGraphicsWidget *>(parentItem());
+    QGraphicsWidget *pw = qobject_cast<QGraphicsWidget *>(parent());
+    if (!pw) {
+        pw = dynamic_cast<QGraphicsWidget *>(parentItem());
+    }
+    Plasma::Applet *parentApplet = qobject_cast<Plasma::Applet *>(pw);
+    //assumption: this loop is usually is -really- short or doesn't run at all
+    while (!parentApplet && pw && pw->parentWidget()) {
+        QGraphicsWidget *parentWidget = qobject_cast<QGraphicsWidget *>(pw->parent());
+        if (!parentWidget) {
+            parentWidget = dynamic_cast<QGraphicsWidget *>(pw->parentItem());
+        }
+        pw = parentWidget;
+        parentApplet = qobject_cast<Plasma::Applet *>(pw);
+    }
 
 
+    const PopupApplet *pa = dynamic_cast<const PopupApplet *>(this);
+
     //if the applet is in a widget that isn't a containment
     //try to retrieve the formFactor from the parent size
-    //we can't use our own sizeHint here because it needs formFactor, so endless recursion
-    if (p && p != c && c != this && layout()) {
-        if (p->size().width() < layout()->effectiveSizeHint(Qt::MinimumSize).width()) {
-            return Plasma::Vertical;
-        } else if (p->size().height() < layout()->effectiveSizeHint(Qt::MinimumSize).height()) {
+    //we can't use our own sizeHint here because it needs formFactor, so endless recursion.
+    // a popupapplet can always be constrained.
+    // a normal applet should to but
+    //FIXME: not always constrained to not break systemmonitor
+    if (parentApplet && parentApplet != c && c != this && (pa || layout())) {
+        if (pa || (parentApplet->size().height() < layout()->effectiveSizeHint(Qt::MinimumSize).height())) {
             return Plasma::Horizontal;
+        } else if (pa || (parentApplet->size().width() < layout()->effectiveSizeHint(Qt::MinimumSize).width())) {
+            return Plasma::Vertical;
         }
+        return parentApplet->formFactor();
     }
 
     return c ? c->d->formFactor : Plasma::Planar;
@@ -1433,6 +1498,8 @@ void Applet::setGlobalShortcut(const KShortcut &shortcut)
         d->activationAction->setText(i18n("Activate %1 Widget", name()));
         d->activationAction->setObjectName(QString("activate widget %1").arg(id())); // NO I18N
         connect(d->activationAction, SIGNAL(triggered()), this, SIGNAL(activate()));
+        connect(d->activationAction, SIGNAL(globalShortcutChanged(QKeySequence)),
+                this, SLOT(globalShortcutChanged()));
 
         QList<QWidget *> widgets = d->actions->associatedWidgets();
         foreach (QWidget *w, widgets) {
@@ -1445,10 +1512,18 @@ void Applet::setGlobalShortcut(const KShortcut &shortcut)
         shortcut,
         KAction::ShortcutTypes(KAction::ActiveShortcut | KAction::DefaultShortcut),
         KAction::NoAutoloading);
+    d->globalShortcutChanged();
+}
 
-    KConfigGroup shortcutConfig(d->mainConfigGroup(), "Shortcuts");
-    shortcutConfig.writeEntry("global", d->activationAction->globalShortcut().toString());
-    d->scheduleModificationNotification();
+void AppletPrivate::globalShortcutChanged()
+{
+    if (!activationAction) {
+        return;
+    }
+
+    KConfigGroup shortcutConfig(mainConfigGroup(), "Shortcuts");
+    shortcutConfig.writeEntry("global", activationAction->globalShortcut().toString());
+    scheduleModificationNotification();
     //kDebug() << "after" << shortcut.primary() << d->activationAction->globalShortcut().primary();
 }
 
@@ -1574,14 +1649,22 @@ bool Applet::isPublished() const
     }
 }
 
-//it bugs me that this can get turned on and off at will. I don't see it being useful and it just
-//makes more work for me and more code duplication.
 void Applet::setHasConfigurationInterface(bool hasInterface)
 {
-    if (d->hasConfigurationInterface == hasInterface) {
+    if (hasInterface == d->hasConfigurationInterface) {
         return;
     }
 
+    QAction *configAction = d->actions->action("configure");
+    if (configAction) {
+        bool enable = hasInterface;
+        if (enable) {
+            const bool unlocked = immutability() == Mutable;
+            enable = unlocked || KAuthorized::authorize("plasma/allow_configure_when_locked");
+        }
+        configAction->setEnabled(enable);
+    }
+
     d->hasConfigurationInterface = hasInterface;
 }
 
@@ -1606,7 +1689,7 @@ KActionCollection* AppletPrivate::defaultActions(QObject *parent)
 
     KAction *runAssociatedApplication = actions->addAction("run associated application");
     runAssociatedApplication->setAutoRepeat(false);
-    runAssociatedApplication->setText(i18n("Run the associated application"));
+    runAssociatedApplication->setText(i18n("Run the Associated Application"));
     runAssociatedApplication->setIcon(KIcon("system-run"));
     runAssociatedApplication->setShortcut(KShortcut("alt+d, t"));
     runAssociatedApplication->setVisible(false);
@@ -1628,7 +1711,7 @@ bool Applet::sceneEventFilter(QGraphicsItem *watched, QEvent *event)
     {
         // don't move when the containment is not mutable,
         // in the rare case the containment doesn't exists consider it as mutable
-        if (d->registeredAsDragHandle.contains(watched)) {
+        if ((flags() & ItemIsMovable) && d->registeredAsDragHandle.contains(watched)) {
             Containment *c = containment();
             if (!c || c->immutability() == Mutable) {
                 mouseMoveEvent(static_cast<QGraphicsSceneMouseEvent*>(event));
@@ -1647,7 +1730,7 @@ bool Applet::sceneEventFilter(QGraphicsItem *watched, QEvent *event)
 
 void Applet::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
 {
-    if (immutability() == Mutable && formFactor() == Plasma::Planar) {
+    if (immutability() == Mutable && formFactor() == Plasma::Planar && (flags() & ItemIsMovable)) {
         QGraphicsItem *parent = parentItem();
         Plasma::Applet *applet = qgraphicsitem_cast<Plasma::Applet*>(parent);
 
@@ -1699,6 +1782,11 @@ void Applet::resizeEvent(QGraphicsSceneResizeEvent *event)
     emit geometryChanged();
 }
 
+bool Applet::isUserConfiguring() const
+{
+    return KConfigDialog::exists(d->configDialogId());
+}
+
 void Applet::showConfigurationInterface()
 {
     if (!hasConfigurationInterface()) {
@@ -1729,7 +1817,7 @@ void Applet::showConfigurationInterface()
             QUiLoader loader;
             QWidget *w = loader.load(&f);
             if (w) {
-                dialog = new KConfigDialog(0, d->configDialogId(), d->configLoader);
+                dialog = new AppletConfigDialog(0, d->configDialogId(), d->configLoader);
                 dialog->setWindowTitle(d->configWindowTitle());
                 dialog->setAttribute(Qt::WA_DeleteOnClose, true);
                 dialog->addPage(w, i18n("Settings"), icon(), i18n("%1 Settings", name()));
@@ -1748,12 +1836,23 @@ void Applet::showConfigurationInterface()
         KConfigDialog *dialog = d->generateGenericConfigDialog();
         //createConfigurationInterface(dialog);
         d->addStandardConfigurationPages(dialog);
-        dialog->show();
+        showConfigurationInterface(dialog);
     }
 
     emit releaseVisualFocus();
 }
 
+void Applet::showConfigurationInterface(QWidget *widget)
+{
+    if (!containment() || !containment()->corona() ||
+        !containment()->corona()->dialogManager()) {
+        widget->show();
+        return;
+    }
+
+    QMetaObject::invokeMethod(containment()->corona()->dialogManager(), "showDialog", Q_ARG(QWidget *, widget), Q_ARG(Plasma::Applet *, this));
+}
+
 QString AppletPrivate::configDialogId() const
 {
     return QString("%1settings%2").arg(appletId).arg(q->name());
@@ -1794,13 +1893,14 @@ QSet<QString> AppletPrivate::knownCategories()
 KConfigDialog *AppletPrivate::generateGenericConfigDialog()
 {
     KConfigSkeleton *nullManager = new KConfigSkeleton(0);
-    KConfigDialog *dialog = new KConfigDialog(0, configDialogId(), nullManager);
+    KConfigDialog *dialog = new AppletConfigDialog(0, configDialogId(), nullManager);
     dialog->setFaceType(KPageDialog::Auto);
     dialog->setWindowTitle(configWindowTitle());
     dialog->setAttribute(Qt::WA_DeleteOnClose, true);
     q->createConfigurationInterface(dialog);
     //TODO: Apply button does not correctly work for now, so do not show it
     dialog->showButton(KDialog::Apply, false);
+    dialog->showButton(KDialog::Default, false);
     QObject::connect(dialog, SIGNAL(applyClicked()), q, SLOT(configDialogFinished()));
     QObject::connect(dialog, SIGNAL(okClicked()), q, SLOT(configDialogFinished()));
     QObject::connect(dialog, SIGNAL(finished()), nullManager, SLOT(deleteLater()));
@@ -1927,7 +2027,7 @@ void AppletPrivate::updateShortcuts()
         //we pull them out, then read, then put them back
         QList<QString> names;
         QList<QAction*> qactions;
-        names << "zoom out" << "add sibling containment" << "configure shortcuts" << "lock widgets";
+        names << "add sibling containment" << "configure shortcuts" << "lock widgets";
         foreach (const QString &name, names) {
             QAction *a = actions->action(name);
             actions->takeAction(a); //FIXME this is stupid, KActionCollection needs a takeAction(QString) method
@@ -2048,18 +2148,21 @@ void AppletPrivate::filterOffers(QList<KService::Ptr> &offers)
     }
 }
 
-KPluginInfo::List Applet::listAppletInfo(const QString &category,
-                                         const QString &parentApp)
+QString AppletPrivate::parentAppConstraint(const QString &parentApp)
 {
-    QString constraint;
-
     if (parentApp.isEmpty()) {
-        constraint.append("(not exist [X-KDE-ParentApp] or [X-KDE-ParentApp] == '')");
-    } else {
-        constraint.append("[X-KDE-ParentApp] == '").append(parentApp).append("'");
+        return QString("((not exist [X-KDE-ParentApp] or [X-KDE-ParentApp] == '') or [X-KDE-ParentApp] == '%1')")
+                      .arg(KGlobal::mainComponent().aboutData()->appName());
     }
-    //note: constraint guaranteed non-empty from here down
 
+    return QString("[X-KDE-ParentApp] == '%1'").arg(parentApp);
+}
+
+KPluginInfo::List Applet::listAppletInfo(const QString &category, const QString &parentApp)
+{
+    QString constraint = AppletPrivate::parentAppConstraint(parentApp);
+
+    //note: constraint guaranteed non-empty from here down
     if (category.isEmpty()) { //use all but the excluded categories
         KConfigGroup group(KGlobal::config(), "General");
         QStringList excluded = group.readEntry("ExcludeCategories", QStringList());
@@ -2067,16 +2170,13 @@ KPluginInfo::List Applet::listAppletInfo(const QString &category,
             constraint.append(" and [X-KDE-PluginInfo-Category] != '").append(category).append("'");
         }
     } else { //specific category (this could be an excluded one - is that bad?)
-        constraint.append(" and ");
-
-        constraint.append("[X-KDE-PluginInfo-Category] == '").append(category).append("'");
+        constraint.append(" and ").append("[X-KDE-PluginInfo-Category] == '").append(category).append("'");
         if (category == "Miscellaneous") {
             constraint.append(" or (not exist [X-KDE-PluginInfo-Category] or [X-KDE-PluginInfo-Category] == '')");
         }
     }
 
     KService::List offers = KServiceTypeTrader::self()->query("Plasma/Applet", constraint);
-    offers << KServiceTypeTrader::self()->query("Plasma/PopupApplet", constraint);
 
     //now we have to do some manual filtering because the constraint can't handle everything
     AppletPrivate::filterOffers(offers);
@@ -2088,19 +2188,19 @@ KPluginInfo::List Applet::listAppletInfo(const QString &category,
 
 KPluginInfo::List Applet::listAppletInfoForMimetype(const QString &mimetype)
 {
-    QString constraint = QString("'%1' in [X-Plasma-DropMimeTypes]").arg(mimetype);
+    QString constraint = AppletPrivate::parentAppConstraint();
+    constraint.append(QString(" and '%1' in [X-Plasma-DropMimeTypes]").arg(mimetype));
     //kDebug() << "listAppletInfoForMimetype with" << mimetype << constraint;
     KService::List offers = KServiceTypeTrader::self()->query("Plasma/Applet", constraint);
-    offers << KServiceTypeTrader::self()->query("Plasma/PopupApplet", constraint);
     AppletPrivate::filterOffers(offers);
     return KPluginInfo::fromServices(offers);
 }
 
 KPluginInfo::List Applet::listAppletInfoForUrl(const QUrl &url)
 {
-    QString constraint = "exist [X-Plasma-DropUrlPatterns]";
+    QString constraint = AppletPrivate::parentAppConstraint();
+    constraint.append(" and exist [X-Plasma-DropUrlPatterns]");
     KService::List offers = KServiceTypeTrader::self()->query("Plasma/Applet", constraint);
-    offers << KServiceTypeTrader::self()->query("Plasma/PopupApplet", constraint);
     AppletPrivate::filterOffers(offers);
 
     KPluginInfo::List allApplets = KPluginInfo::fromServices(offers);
@@ -2122,13 +2222,8 @@ KPluginInfo::List Applet::listAppletInfoForUrl(const QUrl &url)
 
 QStringList Applet::listCategories(const QString &parentApp, bool visibleOnly)
 {
-    QString constraint = "exist [X-KDE-PluginInfo-Category]";
-
-    if (parentApp.isEmpty()) {
-        constraint.append("and (not exist [X-KDE-ParentApp] or [X-KDE-ParentApp] == '')");
-    } else {
-        constraint.append(" and [X-KDE-ParentApp] == '").append(parentApp).append("'");
-    }
+    QString constraint = AppletPrivate::parentAppConstraint(parentApp);
+    constraint.append(" and exist [X-KDE-PluginInfo-Category]");
 
     KConfigGroup group(KGlobal::config(), "General");
     const QStringList excluded = group.readEntry("ExcludeCategories", QStringList());
@@ -2137,7 +2232,6 @@ QStringList Applet::listCategories(const QString &parentApp, bool visibleOnly)
     }
 
     KService::List offers = KServiceTypeTrader::self()->query("Plasma/Applet", constraint);
-    offers << KServiceTypeTrader::self()->query("Plasma/PopupApplet", constraint);
     AppletPrivate::filterOffers(offers);
 
     QStringList categories;
@@ -2179,14 +2273,6 @@ QStringList Applet::customCategories()
     return AppletPrivate::s_customCategories.toList();
 }
 
-Applet::Applet(const QString &packagePath, uint appletId, const QVariantList &args)
-    : QGraphicsWidget(0),
-      d(new AppletPrivate(KService::Ptr(new KService(packagePath + "/metadata.desktop")), appletId, this))
-{
-    Q_UNUSED(args) // FIXME?
-    d->init(packagePath);
-}
-
 Applet *Applet::loadPlasmoid(const QString &path, uint appletId, const QVariantList &args)
 {
     if (QFile::exists(path + "/metadata.desktop")) {
@@ -2224,14 +2310,6 @@ Applet *Applet::load(const QString &appletName, uint appletId, const QVariantLis
         }
     }
 
-    bool isPopupApplet = false;
-    if (offers.isEmpty()) {
-        offers = KServiceTypeTrader::self()->query("Plasma/PopupApplet", constraint);
-        if (offers.count() > 0) {
-            isPopupApplet = true;
-        }
-    }
-
     /* if (offers.count() > 1) {
         kDebug() << "hey! we got more than one! let's blindly take the first one";
     } */
@@ -2256,10 +2334,12 @@ Applet *Applet::load(const QString &appletName, uint appletId, const QVariantLis
                  << offer->property("X-Plasma-API").toString() << "API";
         if (isContainment) {
             return new Containment(0, allArgs);
-        } else if (isPopupApplet) {
-            return new PopupApplet(0, allArgs);
         } else {
-            return new Applet(0, allArgs);
+            if (offer->serviceTypes().contains("Plasma/PopupApplet")) {
+                return new PopupApplet(0, allArgs);
+            } else {
+                return new Applet(0, allArgs);
+            }
         }
     }
 
@@ -2335,7 +2415,7 @@ QVariant Applet::itemChange(GraphicsItemChange change, const QVariant &value)
         }
         break;
     case ItemPositionChange:
-        return (immutability() == Mutable || isContainment() || formFactor() == Horizontal || formFactor() == Vertical) ? value : pos();
+        return (immutability() == Mutable || isContainment() || formFactor() == Horizontal || formFactor() == Vertical || (parentLayoutItem() && parentLayoutItem()->isLayout()) || !d->started) ? value : pos();
         break;
     case ItemTransformChange:
         return immutability() == Mutable ? value : transform();
@@ -2491,6 +2571,11 @@ void AppletPrivate::setIsContainment(bool nowIsContainment, bool forceUpdate)
 
     delete mainConfig;
     mainConfig = 0;
+
+    Containment *c = q->containment();
+    if (c) {
+        c->d->checkContainmentFurniture();
+    }
 }
 
 bool Applet::isContainment() const
@@ -2522,6 +2607,7 @@ AppletPrivate::AppletPrivate(KService::Ptr service, int uniqueID, Applet *applet
           activationAction(0),
           shortcutEditor(0),
           itemStatus(UnknownStatus),
+          preferredSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored),
           modificationsTimer(0),
           hasConfigurationInterface(false),
           failed(false),
@@ -2544,9 +2630,7 @@ AppletPrivate::~AppletPrivate()
         activationAction->forgetGlobalShortcut();
     }
 
-    if (extender) {
-        delete extender.data();
-    }
+    delete extender.data();
 
     delete script;
     script = 0;
@@ -2571,6 +2655,12 @@ void AppletPrivate::init(const QString &packagePath)
     // but it doesn't actually work anyways =/
     q->setLayoutDirection(qApp->layoutDirection());
 
+    //set a default size before any saved settings are read
+    QSize size(200, 200);
+    q->setBackgroundHints(Applet::DefaultBackground);
+    q->setHasConfigurationInterface(true); //FIXME why not default it to true in the constructor?
+
+
     if (!appletDescription.isValid()) {
         kDebug() << "Check your constructor! "
                  << "You probably want to be passing in a Service::Ptr "
@@ -2578,6 +2668,13 @@ void AppletPrivate::init(const QString &packagePath)
         return;
     }
 
+    QVariant s = appletDescription.property("X-Plasma-DefaultSize");
+    if (s.isValid()) {
+        size = s.toSize();
+    }
+    //kDebug() << "size" << size;
+    q->resize(size);
+
     QString api = appletDescription.property("X-Plasma-API").toString();
 
     // we have a scripted plasmoid
@@ -2633,17 +2730,6 @@ void AppletPrivate::init(const QString &packagePath)
         }
     }
 
-    //set a default size before any saved settings are read
-    QSize size = appletDescription.property("X-Plasma-DefaultSize").toSize();
-    if (size.isEmpty()) {
-        size = QSize(200, 200);
-    }
-    //kDebug() << "size" << size;
-    q->resize(size);
-
-    q->setBackgroundHints(Applet::DefaultBackground);
-    q->setHasConfigurationInterface(true); //FIXME why not default it to true in the constructor?
-
     QAction *closeApplet = actions->action("remove");
     if (closeApplet) {
         closeApplet->setText(i18nc("%1 is the name of the applet", "Remove this %1", q->name()));
@@ -2654,7 +2740,6 @@ void AppletPrivate::init(const QString &packagePath)
         configAction->setText(i18nc("%1 is the name of the applet", "%1 Settings", q->name()));
     }
 
-    QObject::connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), q, SLOT(themeChanged()));
     QObject::connect(q, SIGNAL(activate()), q, SLOT(setFocus()));
 }
 
@@ -2755,7 +2840,7 @@ KConfigGroup *AppletPrivate::mainConfigGroup()
             containmentConfig =  KConfigGroup(KGlobal::config(), "Containments");
         }
 
-        if (!containmentConfig.hasGroup(QString::number(appletId))) {
+        if (package && !containmentConfig.hasGroup(QString::number(appletId))) {
             newGroup = true;
         }
 
@@ -2779,19 +2864,21 @@ KConfigGroup *AppletPrivate::mainConfigGroup()
             appletConfig = KConfigGroup(KGlobal::config(), "Applets");
         }
 
-        if (!appletConfig.hasGroup(QString::number(appletId))) {
+        if (package && !appletConfig.hasGroup(QString::number(appletId))) {
             newGroup = true;
         }
 
         mainConfig = new KConfigGroup(&appletConfig, QString::number(appletId));
     }
 
-    if (newGroup && q->package()) {
+    if (newGroup) {
         //see if we have a default configuration in our package
-        kDebug() << "copying default config: " << q->package()->filePath("defaultconfig");
-        KConfigGroup defaultConfig(KSharedConfig::openConfig(q->package()->filePath("config",
-                                    "default-configrc"))->group("Configuration"));
-        defaultConfig.copyTo(mainConfig);
+        const QString defaultConfigFile = q->package()->filePath("defaultconfig");
+        if (!defaultConfigFile.isEmpty()) {
+            kDebug() << "copying default config: " << q->package()->filePath("defaultconfig");
+            KConfigGroup defaultConfig(KSharedConfig::openConfig(defaultConfigFile)->group("Configuration"));
+            defaultConfig.copyTo(mainConfig);
+        }
     }
 
     return mainConfig;
@@ -2868,11 +2955,19 @@ AppletOverlayWidget::AppletOverlayWidget(QGraphicsWidget *parent)
 
 void AppletOverlayWidget::destroy()
 {
-    Animation *zoomAnim =
-        Plasma::Animator::create(Plasma::Animator::ZoomAnimation);
-    connect(zoomAnim, SIGNAL(finished()), this, SLOT(overlayAnimationComplete()));
-    zoomAnim->setTargetWidget(this);
-    zoomAnim->start();
+    Animation *anim = Plasma::Animator::create(Plasma::Animator::DisappearAnimation);
+    if (anim) {
+        connect(anim, SIGNAL(finished()), this, SLOT(overlayAnimationComplete()));
+        anim->setTargetWidget(this);
+        anim->start();
+    } else {
+        overlayAnimationComplete();
+    }
+}
+
+void AppletOverlayWidget::mousePressEvent(QGraphicsSceneMouseEvent *event)
+{
+    event->accept();
 }
 
 void AppletOverlayWidget::overlayAnimationComplete()
@@ -2911,10 +3006,8 @@ void AppletOverlayWidget::paint(QPainter *painter,
         backgroundShape = shape();
     }
 
-    painter->save();
     painter->setRenderHints(QPainter::Antialiasing);
     painter->fillPath(backgroundShape, wash);
-    painter->restore();
 }
 
 } // Plasma namespace
