@@ -1,6 +1,7 @@
 /*
  *   Copyright 2007 by Aaron Seigo <aseigo@kde.org>
  *   Copyright 2008 by Ménard Alexis <darktears31@gmail.com>
+ *   Copyright (c) 2009 Chani Armitage <chani@kde.org>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -32,16 +33,24 @@
 #include <plasma/applet.h>
 #include <plasma/animator.h>
 
+namespace KIO
+{
+    class Job;
+}
+
 namespace Plasma
 {
 
+class AccessAppletJob;
 class AppletHandle;
 class DataEngine;
 class Package;
 class Corona;
 class View;
 class Wallpaper;
+class ContainmentActions;
 class ContainmentPrivate;
+class AbstractToolBox;
 
 /**
  * @class Containment plasma/containment.h <Plasma/Containment>
@@ -143,9 +152,9 @@ class PLASMA_EXPORT Containment : public Applet
         /**
          * Returns a list of all known containments.
          *
-         * @param category Only applets matchin this category will be returned.
+         * @param category Only containments matching this category will be returned.
          *                 Useful in conjunction with knownCategories.
-         *                 If "Misc" is passed in, then applets without a
+         *                 If "Miscellaneous" is passed in, then applets without a
          *                 Categories= entry are also returned.
          *                 If an empty string is passed in, all applets are
          *                 returned.
@@ -159,6 +168,34 @@ class PLASMA_EXPORT Containment : public Applet
         static KPluginInfo::List listContainments(const QString &category = QString(),
                                                   const QString &parentApp = QString());
 
+        /**
+         * Returns a list of all known Containments that match the parameters.
+         *
+         * @param type Only Containments with this string in X-Plasma-ContainmentCategories
+         *             in their .desktop files will be returned. Common values are panel and
+         *             desktop
+         * @param category Only applets matchin this category will be returned.
+         *                 Useful in conjunction with knownCategories.
+         *                 If "Miscellaneous" is passed in, then applets without a
+         *                 Categories= entry are also returned.
+         *                 If an empty string is passed in, all applets are
+         *                 returned.
+         * @param parentApp the application to filter applets on. Uses the
+         *                  X-KDE-ParentApp entry (if any) in the plugin info.
+         *                  The default value of QString() will result in a
+         *                  list containing only applets not specifically
+         *                  registered to an application.
+         * @return list of applets
+         **/
+        static KPluginInfo::List listContainmentsOfType(const QString &type,
+                                                        const QString &category = QString(),
+                                                        const QString &parentApp = QString());
+
+        /**
+         * @return a list of all known types of Containments on this system
+         */
+        static QStringList listContainmentTypes();
+
         /**
          * Returns a list of all known applets associated with a certain mimetype
          *
@@ -218,11 +255,25 @@ class PLASMA_EXPORT Containment : public Applet
          */
         int screen() const;
 
+        /**
+         * @return the last screen number this containment had
+         *         only returns -1 if it's never ever been on a screen
+         * @since 4.5
+         */
+        int lastScreen() const;
+
         /**
          * @return the viewport (e.g. virtual desktop) this Containment is associated with.
          */
         int desktop() const;
 
+        /**
+         * @return the viewport (e.g. virtual desktop) this Containment was associated with
+         * last time it had a screen
+         * @since 4.5
+         */
+        int lastDesktop() const;
+
         /**
          * @reimp
          * @sa Applet::save(KConfigGroup &)
@@ -260,6 +311,12 @@ class PLASMA_EXPORT Containment : public Applet
          */
         void setToolBoxOpen(bool open);
 
+        /**
+         * @return true if the toolbox is open
+         * @since 4.5
+         */
+        bool isToolBoxOpen() const;
+
         /**
          * Open the Containment's toolbox
          */
@@ -305,16 +362,21 @@ class PLASMA_EXPORT Containment : public Applet
         /**
          * Sets the current activity by name
          *
-         * @param activity the name of the activity; if it doesn't exist in the
-         *        semantic store, it will be created.
+         * @param activity the name of the activity
          */
         void setActivity(const QString &activity);
 
         /**
-         * @return the current activity associated with this activity
+         * @return the current activity name associated with this containment
          */
         QString activity() const;
 
+        /**
+         * @return the context for this containment
+         * @since 4.5
+         */
+        Context* context() const;
+
         /**
          * Shows the context menu for the containment directly, bypassing Applets
          * altogether.
@@ -331,6 +393,27 @@ class PLASMA_EXPORT Containment : public Applet
          */
         virtual void showDropZone(const QPoint pos);
 
+        /**
+         * Sets a containmentactions plugin.
+         *
+         * @param trigger the mouse button (and optional modifier) to associate the plugin with
+         * @param pluginName the name of the plugin to attempt to load. blank = set no plugin.
+         * @since 4.4
+         */
+        void setContainmentActions(const QString &trigger, const QString &pluginName);
+
+        /**
+         * @return a list of all triggers that have a containmentactions plugin associated
+         * @since 4.4
+         */
+        QStringList containmentActionsTriggers();
+
+        /**
+         * @return the plugin name for the given trigger
+         * @since 4.4
+         */
+        QString containmentActions(const QString &trigger);
+
     Q_SIGNALS:
         /**
          * This signal is emitted when a new applet is created by the containment
@@ -344,6 +427,7 @@ class PLASMA_EXPORT Containment : public Applet
 
         /**
          * Emitted when the containment requests zooming in or out one step.
+         * @deprecated
          */
         void zoomRequested(Plasma::Containment *containment, Plasma::ZoomDirection direction);
 
@@ -352,6 +436,12 @@ class PLASMA_EXPORT Containment : public Applet
          */
         void toolBoxToggled();
 
+        /**
+         * Emitted when the toolbox is hidden or shown
+         * @since 4.3
+         */
+        void toolBoxVisibilityChanged(bool);
+
         /**
          * Emitted when the containment wants a new containment to be created.
          * Usually only used for desktop containments.
@@ -383,14 +473,14 @@ class PLASMA_EXPORT Containment : public Applet
         void configureRequested(Plasma::Containment *containment);
 
         /**
-         * The activity associated to this containemnt has changed
+         * The context associated to this containment has changed
          */
         void contextChanged(Plasma::Context *context);
 
     public Q_SLOTS:
         /**
          * Informs the Corona as to what position it is in. This is informational
-         * only, as the Corona doesn't change it's actual location. This is,
+         * only, as the Corona doesn't change its actual location. This is,
          * however, passed on to Applets that may be managed by this Corona.
          *
          * @param location the new location of this Corona
@@ -483,6 +573,12 @@ class PLASMA_EXPORT Containment : public Applet
          */
         void dragEnterEvent(QGraphicsSceneDragDropEvent *event);
 
+        /**
+         * @reimp
+         * @sa QGraphicsItem::dragLeaveEvent()
+         */
+        void dragLeaveEvent(QGraphicsSceneDragDropEvent *event);
+
         /**
          * @reimp
          * @sa QGraphicsItem::dragMoveEvent()
@@ -504,24 +600,58 @@ class PLASMA_EXPORT Containment : public Applet
         /**
          * @returns the toolbox associated with this containment, or a null pointer if none
          */
-        const QGraphicsItem *toolBoxItem() const;
+        KDE_DEPRECATED const QGraphicsItem *toolBoxItem() const;
+
+        /**
+         * Sets a custom ToolBox
+         * if there was an old one it will be deleted
+         * and the new one won't have any actions in it
+         *
+         * @param item the new toolbox item
+         * @since 4.4
+         */
+        void setToolBox(AbstractToolBox *toolBox);
+
+        /**
+         * @return the ToolBox
+         * @since 4.4
+         */
+        AbstractToolBox *toolBox() const;
 
     private:
+        /**
+         * @internal This constructor is to be used with the Package loading system.
+         *
+         * @param parent a QObject parent; you probably want to pass in 0
+         * @param args a list of strings containing two entries: the service id
+         *      and the applet id
+         * @since 4.3
+         */
+        Containment(const QString &packagePath, uint appletId, const QVariantList &args);
+
         Q_PRIVATE_SLOT(d, void appletDestroyed(Plasma::Applet*))
-        Q_PRIVATE_SLOT(d, void containmentAppletAnimationComplete(QGraphicsItem *item,
-                                                                  Plasma::Animator::Animation anim))
+        Q_PRIVATE_SLOT(d, void appletAppearAnimationComplete())
         Q_PRIVATE_SLOT(d, void triggerShowAddWidgets())
         Q_PRIVATE_SLOT(d, void handleDisappeared(AppletHandle *handle))
         Q_PRIVATE_SLOT(d, void positionToolBox())
-        Q_PRIVATE_SLOT(d, void zoomIn())
-        Q_PRIVATE_SLOT(d, void zoomOut())
-        Q_PRIVATE_SLOT(d, void toggleDesktopImmutability())
         Q_PRIVATE_SLOT(d, void requestConfiguration())
+        Q_PRIVATE_SLOT(d, void updateToolBoxVisibility())
+        Q_PRIVATE_SLOT(d, void showDropZoneDelayed())
+        Q_PRIVATE_SLOT(d, void checkStatus(Plasma::ItemStatus))
+        Q_PRIVATE_SLOT(d, void remoteAppletReady(Plasma::AccessAppletJob *))
+        Q_PRIVATE_SLOT(d, void onContextChanged(Plasma::Context *con))
+        /**
+        * This slot is called when the 'stat' after a job event has finished.
+        */
+        Q_PRIVATE_SLOT(d, void mimeTypeRetrieved(KIO::Job *, const QString &))
+        Q_PRIVATE_SLOT(d, void dropJobResult(KJob *))
 
         friend class Applet;
         friend class AppletPrivate;
         friend class CoronaPrivate;
         friend class ContainmentPrivate;
+        friend class ContainmentActions;
+        friend class PopupApplet;
         ContainmentPrivate *const d;
 };
 
