@@ -38,6 +38,7 @@
 #include <QApplication>
 #include <QDesktopWidget>
 #include <QVarLengthArray>
+#include <QGraphicsLayout>
 
 #include <kdebug.h>
 #include <kwindowsystem.h>
@@ -45,6 +46,8 @@
 
 #include "plasma/applet.h"
 #include "plasma/animator.h"
+#include "plasma/containment.h"
+#include "plasma/corona.h"
 #include "plasma/extenders/extender.h"
 #include "plasma/private/extender_p.h"
 #include "plasma/framesvg.h"
@@ -68,7 +71,8 @@ public:
               resizeCorners(Dialog::NoCorner),
               resizeStartCorner(Dialog::NoCorner),
               moveTimer(0),
-              aspectRatioMode(Plasma::IgnoreAspectRatio)
+              aspectRatioMode(Plasma::IgnoreAspectRatio),
+              resizeChecksWithBorderCheck(false)
     {
     }
 
@@ -76,10 +80,15 @@ public:
     {
     }
 
+    void scheduleBorderCheck(bool triggeredByResize = false);
     void themeChanged();
-    void adjustView();
+    void updateMask();
+    void checkBorders();
+    void checkBorders(bool updateMaskIfNeeded);
     void updateResizeCorners();
     int calculateWidthForHeightAndRatio(int height, qreal ratio);
+    Plasma::Applet *applet();
+    void delayedAdjustSize();
 
     Plasma::Dialog *q;
 
@@ -95,33 +104,80 @@ public:
     int resizeStartCorner;
     QTimer *moveTimer;
     QTimer *adjustViewTimer;
+    QTimer *adjustSizeTimer;
+    QSize oldGraphicsWidgetMinimumSize;
+    QSize oldGraphicsWidgetMaximumSize;
     Plasma::AspectRatioMode aspectRatioMode;
+    bool resizeChecksWithBorderCheck;
 };
 
+void DialogPrivate::scheduleBorderCheck(bool triggeredByResize)
+{
+    //kDebug();
+    if (triggeredByResize) {
+        resizeChecksWithBorderCheck = true;
+
+        // to keep the UI as fluid as possible, we call checkBorders
+        // immediately when there is a resize, and therefore stop any
+        // move-triggered scheduled calls to it. this keeps things
+        // looking reasonable during resize while avoiding as many
+        // calls to checkBorders as possible
+        if (moveTimer) {
+            moveTimer->stop();
+        }
+
+        checkBorders();
+        return;
+    }
+
+    if (!moveTimer) {
+        moveTimer = new QTimer(q);
+        moveTimer->setSingleShot(true);
+        QObject::connect(moveTimer, SIGNAL(timeout()), q, SLOT(checkBorders()));
+    }
+
+    moveTimer->start(triggeredByResize ? 0 : 200);
+}
+
 void DialogPrivate::themeChanged()
 {
-    QGraphicsWidget *graphicsWidget = graphicsWidgetPtr.data();
+    checkBorders(false);
 
-    qreal topHeight;
-    qreal leftWidth;
-    qreal rightWidth;
-    qreal bottomHeight;
+    const bool translucency = Plasma::Theme::defaultTheme()->windowTranslucencyEnabled();
+    // WA_NoSystemBackground is going to fail combined with sliding popups, but is needed
+    // when we aren't compositing
+    q->setAttribute(Qt::WA_NoSystemBackground, !translucency);
+    updateMask();
+}
 
-    //kDebug() << leftWidth << topHeight << rightWidth << bottomHeight;
-    if (Plasma::Theme::defaultTheme()->windowTranslucencyEnabled()) {
+void DialogPrivate::updateMask()
+{
+    const bool translucency = Plasma::Theme::defaultTheme()->windowTranslucencyEnabled();
+    WindowEffects::enableBlurBehind(q->winId(), translucency,
+                                    translucency ? background->mask() : QRegion());
+    if (translucency) {
         q->clearMask();
     } else {
         q->setMask(background->mask());
     }
+}
 
-    FrameSvg::EnabledBorders borders = FrameSvg::AllBorders;
+void DialogPrivate::checkBorders()
+{
+    checkBorders(true);
+}
 
-    Extender *extender = qobject_cast<Extender*>(graphicsWidget);
+Plasma::Applet *DialogPrivate::applet()
+{
+    Extender *extender = qobject_cast<Extender*>(graphicsWidgetPtr.data());
     Plasma::Applet *applet = 0;
     if (extender) {
-        applet = extender->d->applet;
-    } else if (graphicsWidget) {
-        QObject *pw = graphicsWidget;
+        if (!extender->d->applet) {
+            return 0;
+        }
+        applet = extender->d->applet.data();
+    } else if (graphicsWidgetPtr) {
+        QObject *pw = graphicsWidgetPtr.data();
 
         while ((pw = pw->parent())) {
             applet = dynamic_cast<Plasma::Applet *>(pw);
@@ -130,34 +186,83 @@ void DialogPrivate::themeChanged()
             }
         }
     }
+    return applet;
+}
+
+void DialogPrivate::delayedAdjustSize()
+{
+    q->syncToGraphicsWidget();
+}
+
+void DialogPrivate::checkBorders(bool updateMaskIfNeeded)
+{
+    if (resizeChecksWithBorderCheck) {
+        background->resizeFrame(q->size());
+    }
+
+    QGraphicsWidget *graphicsWidget = graphicsWidgetPtr.data();
+    const FrameSvg::EnabledBorders currentBorders = background->enabledBorders();
+    FrameSvg::EnabledBorders borders = FrameSvg::AllBorders;
+
+    Extender *extender = qobject_cast<Extender*>(graphicsWidget);
+    Plasma::Applet *applet = this->applet();
+
+    //used to remove borders at the edge of the desktop
+    QDesktopWidget *desktop = QApplication::desktop();
+    QRect avail = desktop->availableGeometry(desktop->screenNumber(q));
+    QRect screenGeom = desktop->screenGeometry(desktop->screenNumber(q));
+    QRect dialogGeom = q->geometry();
+
+    qreal topHeight(0);
+    qreal leftWidth(0);
+    qreal rightWidth(0);
+    qreal bottomHeight(0);
+
+    //decide about disabling the border attached to the panel
     if (applet) {
         background->getMargins(leftWidth, topHeight, rightWidth, bottomHeight);
 
         switch (applet->location()) {
         case BottomEdge:
-            borders &= ~FrameSvg::BottomBorder;
-            leftWidth = 0;
-            rightWidth = 0;
-            bottomHeight = 0;
+            if (applet->containment() &&
+                dialogGeom.bottom() + 2 >= screenGeom.bottom() - applet->containment()->size().height() &&
+                dialogGeom.width() <= applet->containment()->size().width()) {
+                borders &= ~FrameSvg::BottomBorder;
+                leftWidth = 0;
+                rightWidth = 0;
+                bottomHeight = 0;
+            }
         break;
 
         case TopEdge:
-            borders &= ~FrameSvg::TopBorder;
-            topHeight = 0;
-            leftWidth = 0;
-            rightWidth = 0;
+            if (applet->containment() &&
+                dialogGeom.top() <= screenGeom.top() + applet->containment()->size().height() &&
+                dialogGeom.width() <= applet->containment()->size().width()) {
+                borders &= ~FrameSvg::TopBorder;
+                topHeight = 0;
+                leftWidth = 0;
+                rightWidth = 0;
+            }
         break;
 
         case LeftEdge:
-            borders &= ~FrameSvg::LeftBorder;
-            leftWidth = 0;
-            rightWidth = 0;
+            if (applet->containment() &&
+                dialogGeom.left() <= screenGeom.left() + applet->containment()->size().width() &&
+                dialogGeom.height() <= applet->containment()->size().height()) {
+                borders &= ~FrameSvg::LeftBorder;
+                leftWidth = 0;
+                rightWidth = 0;
+            }
         break;
 
         case RightEdge:
-            borders &= ~FrameSvg::RightBorder;
-            leftWidth = 0;
-            rightWidth = 0;
+            if (applet->containment() &&
+                dialogGeom.right() + 2 >= screenGeom.right() - applet->containment()->size().width() &&
+                dialogGeom.height() <= applet->containment()->size().height()) {
+                borders &= ~FrameSvg::RightBorder;
+                leftWidth = 0;
+                rightWidth = 0;
+            }
         break;
 
         default:
@@ -165,23 +270,26 @@ void DialogPrivate::themeChanged()
         }
     }
 
+    //decide if to disable the other borders
     if (!extender && q->isVisible()) {
-        //remove borders at the edge of the desktop
-        QDesktopWidget *desktop = QApplication::desktop();
-        QRect avail = desktop->availableGeometry(desktop->screenNumber(q));
-        QRect dialogGeom = q->geometry();
+        QRect geom;
+        if (applet) {
+            geom = screenGeom;
+        } else {
+            geom = avail;
+        }
 
-        if (dialogGeom.left() <= avail.left()) {
+        if (dialogGeom.left() <= geom.left()) {
             borders &= ~FrameSvg::LeftBorder;
         }
-        if (dialogGeom.top() <= avail.top()) {
+        if (dialogGeom.top() <= geom.top()) {
             borders &= ~FrameSvg::TopBorder;
         }
         //FIXME: that 2 pixels offset has probably something to do with kwin
-        if (dialogGeom.right() + 2 > avail.right()) {
+        if (dialogGeom.right() + 2 > geom.right()) {
             borders &= ~FrameSvg::RightBorder;
         }
-        if (dialogGeom.bottom() + 2 > avail.bottom()) {
+        if (dialogGeom.bottom() + 2 > geom.bottom()) {
             borders &= ~FrameSvg::BottomBorder;
         }
     }
@@ -192,17 +300,32 @@ void DialogPrivate::themeChanged()
         background->getMargins(leftWidth, topHeight, rightWidth, bottomHeight);
     }
 
+    //kDebug() << leftWidth << topHeight << rightWidth << bottomHeight;
     q->setContentsMargins(leftWidth, topHeight, rightWidth, bottomHeight);
-    q->update();
+
+    if (resizeChecksWithBorderCheck) {
+        updateResizeCorners();
+        updateMask();
+        q->update();
+    } else if (currentBorders != borders) {
+        if (updateMaskIfNeeded) {
+            updateMask();
+        }
+
+        q->update();
+    } 
+
+    resizeChecksWithBorderCheck = false;
 }
 
-void DialogPrivate::adjustView()
+void Dialog::syncToGraphicsWidget()
 {
-    QGraphicsWidget *graphicsWidget = graphicsWidgetPtr.data();
-    if (view && graphicsWidget) {
-        const int prevStartCorner = resizeStartCorner;
-        resizeStartCorner = -1;
-        QSize prevSize = q->size();
+    d->adjustViewTimer->stop();
+    QGraphicsWidget *graphicsWidget = d->graphicsWidgetPtr.data();
+    if (d->view && graphicsWidget && d->resizeStartCorner != -1) {
+        const int prevStartCorner = d->resizeStartCorner;
+        d->resizeStartCorner = -1;
+        QSize prevSize = size();
         /*
         kDebug() << "Widget size:" << graphicsWidget->size()
                  << "| Widget size hint:" << graphicsWidget->effectiveSizeHint(Qt::PreferredSize)
@@ -212,19 +335,50 @@ void DialogPrivate::adjustView()
         */
         //set the sizehints correctly:
         int left, top, right, bottom;
-        q->getContentsMargins(&left, &top, &right, &bottom);
+        getContentsMargins(&left, &top, &right, &bottom);
 
         QDesktopWidget *desktop = QApplication::desktop();
-        QSize maxSize = desktop->availableGeometry(desktop->screenNumber(q)).size();
+        QSize maxSize = desktop->availableGeometry(desktop->screenNumber(this)).size();
+
+        graphicsWidget->setMaximumSize(maxSize - QSize(left + right, top + bottom).boundedTo(graphicsWidget->effectiveSizeHint(Qt::MaximumSize).toSize()));
+
+        setMinimumSize(-1, -1);
+        setMaximumSize(QWIDGETSIZE_MAX, QWIDGETSIZE_MAX);
+
+        QSize newSize(qMin(int(graphicsWidget->size().width()) + left + right, maxSize.width()),
+               qMin(int(graphicsWidget->size().height()) + top + bottom, maxSize.height()));
+
+        QSize newMinimumSize(qMin(int(graphicsWidget->minimumSize().width()) + left + right, maxSize.width()),
+                       qMin(int(graphicsWidget->minimumSize().height()) + top + bottom, maxSize.height()));
 
 
-        q->setMinimumSize(qMin(int(graphicsWidget->minimumSize().width()) + left + right, maxSize.width()),
-                          qMin(int(graphicsWidget->minimumSize().height()) + top + bottom, maxSize.height()));
-        q->setMaximumSize(qMin(int(graphicsWidget->maximumSize().width()) + left + right, maxSize.width()),
-                          qMin(int(graphicsWidget->maximumSize().height()) + top + bottom, maxSize.height()));
-        q->resize(qMin(int(graphicsWidget->size().width()) + left + right, maxSize.width()),
-                  qMin(int(graphicsWidget->size().height()) + top + bottom, maxSize.height()));
-        q->updateGeometry();
+        QSize newMaximumSize(qMin(int(graphicsWidget->maximumSize().width()) + left + right, maxSize.width()),
+                       qMin(int(graphicsWidget->maximumSize().height()) + top + bottom, maxSize.height()));
+
+
+        Plasma::Applet *applet = d->applet();
+        if (applet) {
+            QRect currentGeometry(geometry());
+            currentGeometry.setSize(newSize);
+            if (applet->location() == Plasma::TopEdge ||
+                applet->location() == Plasma::LeftEdge) {
+                currentGeometry.setSize(newSize);
+            } else if (applet->location() == Plasma::RightEdge) {
+                currentGeometry.moveTopRight(geometry().topRight());
+            //BottomEdge and floating
+            } else {
+                currentGeometry.moveBottomLeft(geometry().bottomLeft());
+            }
+            setGeometry(currentGeometry);
+        } else {
+            resize(newSize);
+        }
+
+        setMinimumSize(newMinimumSize);
+        setMaximumSize(newMaximumSize);
+
+
+        updateGeometry();
 
         //reposition and resize the view.
         //force a valid rect, otherwise it will take up the whole scene
@@ -232,10 +386,10 @@ void DialogPrivate::adjustView()
 
         sceneRect.setWidth(qMax(qreal(1), sceneRect.width()));
         sceneRect.setHeight(qMax(qreal(1), sceneRect.height()));
-        view->setSceneRect(sceneRect);
+        d->view->setSceneRect(sceneRect);
 
-        view->resize(graphicsWidget->size().toSize());
-        view->centerOn(graphicsWidget);
+        //d->view->resize(graphicsWidget->size().toSize());
+        d->view->centerOn(graphicsWidget);
 
         //if the view resized and a border is disabled move the dialog to make sure it will still look attached to panel/screen edge
         qreal topHeight;
@@ -243,20 +397,14 @@ void DialogPrivate::adjustView()
         qreal rightWidth;
         qreal bottomHeight;
 
-        background->getMargins(leftWidth, topHeight, rightWidth, bottomHeight);
+        d->background->getMargins(leftWidth, topHeight, rightWidth, bottomHeight);
 
-        if (rightWidth == 0) {
-            q->move(q->pos().x() + (prevSize.width() - q->size().width()), q->pos().y());
-        }
-        if (bottomHeight == 0) {
-            q->move(q->pos().x(), q->pos().y() + (prevSize.height() - q->size().height()));
-        }
-
-        if (q->size() != prevSize) {
+        if (size() != prevSize) {
             //the size of the dialog has changed, emit the signal:
-            emit q->dialogResized();
+            emit dialogResized();
         }
-        resizeStartCorner = prevStartCorner;
+
+        d->resizeStartCorner = prevStartCorner;
     }
 }
 
@@ -278,11 +426,10 @@ int DialogPrivate::calculateWidthForHeightAndRatio(int height, qreal ratio)
 }
 
 Dialog::Dialog(QWidget *parent, Qt::WindowFlags f)
-    : QWidget(parent, f),
+    : QWidget(parent, f | Qt::FramelessWindowHint),
       d(new DialogPrivate(this))
 {
     setAttribute(Qt::WA_TranslucentBackground);
-    setWindowFlags(Qt::FramelessWindowHint);
     d->background = new FrameSvg(this);
     d->background->setImagePath("dialogs/background");
     d->background->setEnabledBorders(FrameSvg::AllBorders);
@@ -295,7 +442,11 @@ Dialog::Dialog(QWidget *parent, Qt::WindowFlags f)
 
     d->adjustViewTimer = new QTimer(this);
     d->adjustViewTimer->setSingleShot(true);
-    connect(d->adjustViewTimer, SIGNAL(timeout()), this, SLOT(adjustView()));
+    connect(d->adjustViewTimer, SIGNAL(timeout()), this, SLOT(syncToGraphicsWidget()));
+
+    d->adjustSizeTimer = new QTimer(this);
+    d->adjustSizeTimer->setSingleShot(true);
+    connect(d->adjustSizeTimer, SIGNAL(timeout()), this, SLOT(delayedAdjustSize()));
 
     connect(d->background, SIGNAL(repaintNeeded()), this, SLOT(update()));
 
@@ -319,7 +470,9 @@ void Dialog::paintEvent(QPaintEvent *e)
 
 void Dialog::mouseMoveEvent(QMouseEvent *event)
 {
-    if (d->resizeAreas[Dialog::NorthEast].contains(event->pos())) {
+    if (event->modifiers() == Qt::AltModifier) {
+        unsetCursor();
+    } else if (d->resizeAreas[Dialog::NorthEast].contains(event->pos())) {
         setCursor(Qt::SizeBDiagCursor);
     } else if (d->resizeAreas[Dialog::NorthWest].contains(event->pos())) {
         setCursor(Qt::SizeFDiagCursor);
@@ -436,15 +589,11 @@ bool Dialog::event(QEvent *event)
     return QWidget::event(event);
 }
 
-void Dialog::resizeEvent(QResizeEvent *e)
+void Dialog::resizeEvent(QResizeEvent *event)
 {
-    d->background->resizeFrame(e->size());
-
-    if (Plasma::Theme::defaultTheme()->windowTranslucencyEnabled()) {
-        clearMask();
-    } else {
-        setMask(d->background->mask());
-    }
+    Q_UNUSED(event)
+    //kDebug();
+    d->scheduleBorderCheck(true);
 
     if (d->resizeStartCorner != -1 && d->view && d->graphicsWidgetPtr) {
         QGraphicsWidget *graphicsWidget = d->graphicsWidgetPtr.data();
@@ -456,8 +605,6 @@ void Dialog::resizeEvent(QResizeEvent *e)
         d->view->setSceneRect(sceneRect);
         d->view->centerOn(graphicsWidget);
     }
-
-    d->updateResizeCorners();
 }
 
 void DialogPrivate::updateResizeCorners()
@@ -498,7 +645,6 @@ void DialogPrivate::updateResizeCorners()
         resizeAreas[Dialog::SouthWest] = QRect(0, r.bottom() - resizeAreaMargin,
                                                resizeAreaMargin, resizeAreaMargin);
     }
-
 }
 
 void Dialog::setGraphicsWidget(QGraphicsWidget *widget)
@@ -516,7 +662,7 @@ void Dialog::setGraphicsWidget(QGraphicsWidget *widget)
             lay->setSpacing(0);
         }
 
-        d->themeChanged();
+        d->checkBorders();
 
         if (!d->view) {
             d->view = new QGraphicsView(this);
@@ -528,18 +674,28 @@ void Dialog::setGraphicsWidget(QGraphicsWidget *widget)
         }
 
         d->view->setScene(widget->scene());
-        d->adjustView();
 
-        adjustSize();
+        //try to have the proper size -before- showing the dialog
+        d->view->centerOn(widget);
+        if (widget->layout()) {
+            widget->layout()->activate();
+        }
+        static_cast<QGraphicsLayoutItem *>(widget)->updateGeometry();
+        widget->resize(widget->size().expandedTo(widget->effectiveSizeHint(Qt::MinimumSize)));
+
+        syncToGraphicsWidget();
+
+        //d->adjustSizeTimer->start(150);
 
         widget->installEventFilter(this);
+        d->view->installEventFilter(this);
     } else {
         delete d->view;
         d->view = 0;
     }
 }
 
-//BIC FIXME: should be const
+//KDE5 FIXME: should be const
 QGraphicsWidget *Dialog::graphicsWidget()
 {
     return d->graphicsWidgetPtr.data();
@@ -552,6 +708,15 @@ bool Dialog::eventFilter(QObject *watched, QEvent *event)
         d->adjustViewTimer->start(150);
     }
 
+    // when moving the cursor with a 45 angle from the outside
+    // to the inside passing over a resize angle the cursor changes its
+    // shape to a resize cursor. As a side effect this is the only case
+    // when the cursor immediately enters the view without giving
+    // the dialog the chance to restore the original cursor shape.
+    if (event->type() == QEvent::Enter && watched == d->view) {
+        unsetCursor();
+    }
+
     return QWidget::eventFilter(watched, event);
 }
 
@@ -566,12 +731,18 @@ void Dialog::showEvent(QShowEvent * event)
     Q_UNUSED(event);
 
     //check if the widget size is still synced with the view
-    d->themeChanged();
+    d->checkBorders();
     d->updateResizeCorners();
 
     QGraphicsWidget *graphicsWidget = d->graphicsWidgetPtr.data();
-    if (graphicsWidget && d->view && graphicsWidget->size().toSize() != d->view->size()) {
-        d->adjustView();
+    if (graphicsWidget &&
+        ((d->view && graphicsWidget->size().toSize() != d->view->size()) ||
+         d->oldGraphicsWidgetMinimumSize != graphicsWidget->minimumSize() ||
+         d->oldGraphicsWidgetMaximumSize != graphicsWidget->maximumSize())) {
+        //here have to be done immediately, ideally should have to be done -before- shwing, but is not possible to catch show() so early
+        syncToGraphicsWidget();
+        d->oldGraphicsWidgetMinimumSize = graphicsWidget->minimumSize().toSize();
+        d->oldGraphicsWidgetMaximumSize = graphicsWidget->maximumSize().toSize();
     }
 
     if (d->view) {
@@ -603,14 +774,8 @@ void Dialog::focusInEvent(QFocusEvent *event)
 void Dialog::moveEvent(QMoveEvent *event)
 {
     Q_UNUSED(event)
-
-    if (!d->moveTimer) {
-        d->moveTimer = new QTimer(this);
-        d->moveTimer->setSingleShot(true);
-        connect(d->moveTimer, SIGNAL(timeout()), this, SLOT(themeChanged()));
-    }
-
-    d->moveTimer->start(200);
+    //kDebug();
+    d->scheduleBorderCheck();
 }
 
 void Dialog::setResizeHandleCorners(ResizeCorners corners)
