@@ -54,15 +54,67 @@
 #include "private/extendergroup_p.h"
 #include "private/extenderitem_p.h"
 #include "private/extenderitemmimedata_p.h"
+#include "widgets/label.h"
 
 namespace Plasma
 {
 
+class ExtenderItemToolbox : public QGraphicsWidget
+{
+public:
+    ExtenderItemToolbox(QGraphicsWidget *parent)
+        : QGraphicsWidget(parent),
+          m_background(new FrameSvg(this))
+    {
+        m_background->setImagePath("widgets/extender-dragger");
+        setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
+        updateTheme();
+    }
+
+    qreal iconSize()
+    {
+        return m_iconSize;
+    }
+
+    void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *)
+    {
+        m_background->paintFrame(painter, option->exposedRect, option->exposedRect);
+    }
+
+    void updateTheme()
+    {
+        //Read the preferred icon size hint, look at the font size, and calculate the desired title bar
+        //icon height.
+        m_background->resize();
+        QSizeF size = m_background->elementSize("hint-preferred-icon-size");
+        size = size.expandedTo(QSizeF(KIconLoader::SizeSmall,KIconLoader::SizeSmall));
+
+        Plasma::Theme *theme = Plasma::Theme::defaultTheme();
+        QFont font = theme->font(Plasma::Theme::DefaultFont);
+        QFontMetrics fm(font);
+        m_iconSize = qMax(size.height(), (qreal) fm.height());
+    }
+
+protected:
+    void resizeEvent(QGraphicsSceneResizeEvent *)
+    {
+        m_background->resizeFrame(size());
+        qreal left, top, right, bottom;
+        m_background->getMargins(left, top, right, bottom);
+        setContentsMargins(0, top, 0, bottom);
+    }
+
+private:
+    FrameSvg *m_background;
+    qreal m_iconSize;
+};
+
 ExtenderItem::ExtenderItem(Extender *hostExtender, uint extenderItemId)
         : QGraphicsWidget(hostExtender),
           d(new ExtenderItemPrivate(this, hostExtender))
 {
     Q_ASSERT(hostExtender);
+    setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
 
     //set the extenderId
     if (extenderItemId) {
@@ -74,42 +126,53 @@ ExtenderItem::ExtenderItem(Extender *hostExtender, uint extenderItemId)
     }
 
     //create the toolbox.
-    d->toolbox = new QGraphicsWidget(this);
+    d->toolbox = new ExtenderItemToolbox(this);
     d->toolboxLayout = new QGraphicsLinearLayout(d->toolbox);
-    d->toolbox->setLayout(d->toolboxLayout);
 
     //create items's configgroup
-    KConfigGroup cg = hostExtender->d->applet->config("ExtenderItems");
+    KConfigGroup cg = hostExtender->d->applet.data()->config("ExtenderItems");
     KConfigGroup dg = KConfigGroup(&cg, QString::number(d->extenderItemId));
 
+    //create own layout
+    d->layout = new QGraphicsLinearLayout(Qt::Vertical, this);
+    d->layout->addItem(d->toolbox);
+
     uint sourceAppletId = dg.readEntry("sourceAppletId", 0);
 
     //check if we're creating a new item or reinstantiating an existing one.
+    d->collapseIcon = new IconWidget(d->toolbox);
+    d->collapseIcon->setCursor(Qt::ArrowCursor);
+    d->titleLabel = new Label(d->toolbox);
+
+    d->toolboxLayout->addItem(d->collapseIcon);
+    d->toolboxLayout->addItem(d->titleLabel);
+    d->toolboxLayout->setStretchFactor(d->titleLabel, 10);
+
     if (!sourceAppletId) {
         //The item is new
-        dg.writeEntry("sourceAppletPluginName", hostExtender->d->applet->pluginName());
-        dg.writeEntry("sourceAppletId", hostExtender->d->applet->id());
-        dg.writeEntry("extenderIconName", hostExtender->d->applet->icon());
-        d->sourceApplet = hostExtender->d->applet;
-        d->collapseIcon = new IconWidget(KIcon(hostExtender->d->applet->icon()), "", this);
+        dg.writeEntry("sourceAppletPluginName", hostExtender->d->applet.data()->pluginName());
+        dg.writeEntry("sourceAppletId", hostExtender->d->applet.data()->id());
+        dg.writeEntry("extenderIconName", hostExtender->d->applet.data()->icon());
+        d->sourceApplet = hostExtender->d->applet.data();
+        d->collapseIcon->setIcon(KIcon(hostExtender->d->applet.data()->icon()));
     } else {
         //The item already exists.
         d->name = dg.readEntry("extenderItemName", "");
-        d->title = dg.readEntry("extenderTitle", "");
+        d->titleLabel->setText(dg.readEntry("extenderTitle", ""));
         setCollapsed(dg.readEntry("isCollapsed", false));
 
         QString iconName = dg.readEntry("extenderIconName", "utilities-desktop-extra");
         if (iconName.isEmpty()) {
             iconName = "utilities-desktop-extra";
         }
-        d->collapseIcon = new IconWidget(KIcon(iconName), "", this);
+        d->collapseIcon->setIcon(iconName);
 
         //Find the group if it's already there.
         QString groupName = dg.readEntry("group", "");
         d->group = hostExtender->d->findGroup(groupName);
 
         //Find the sourceapplet.
-        Corona *corona = hostExtender->d->applet->containment()->corona();
+        Corona *corona = hostExtender->d->applet.data()->containment()->corona();
         foreach (Containment *containment, corona->containments()) {
             foreach (Applet *applet, containment->applets()) {
                 if (applet->id() == sourceAppletId &&
@@ -122,43 +185,46 @@ ExtenderItem::ExtenderItem(Extender *hostExtender, uint extenderItemId)
 
     //make sure we keep monitoring if the source applet still exists, so the return to source icon
     //can be hidden if it is removed.
-    connect(d->sourceApplet, SIGNAL(destroyed()), this, SLOT(sourceAppletRemoved()));
+    if (d->sourceApplet) {
+        connect(d->sourceApplet, SIGNAL(destroyed()), this, SLOT(sourceAppletRemoved()));
+    }
+
     connect(d->collapseIcon, SIGNAL(clicked()), this, SLOT(toggleCollapse()));
 
     //set the extender we want to move to.
     setExtender(hostExtender);
 
-    //show or hide the return to source icon.
-    d->updateToolBox();
-
-    //set the image paths, image sizes and collapseIcon position.
+    //set the image paths, image sizes
     d->themeChanged();
 
+    //show or hide the toolbox interface itmems
+    d->updateToolBox();
+
     setAcceptsHoverEvents(true);
 
     connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), this, SLOT(themeChanged()));
+    d->setMovable(d->extender->d->applet.data()->immutability() == Plasma::Mutable);
 }
 
 ExtenderItem::~ExtenderItem()
 {
     emit destroyed(this);
-    //make sure the original mousepointer always get's restored.
-    if (d->mouseOver) {
-        QApplication::restoreOverrideCursor();
-    }
     delete d;
 }
 
 KConfigGroup ExtenderItem::config() const
 {
-    KConfigGroup cg = d->extender->d->applet->config("ExtenderItems");
+    if (!d->extender->d->applet) {
+        return KConfigGroup();
+    }
+    KConfigGroup cg = d->extender->d->applet.data()->config("ExtenderItems");
     return KConfigGroup(&cg, QString::number(d->extenderItemId));
 }
 
 void ExtenderItem::setTitle(const QString &title)
 {
-    if (d->title != title) {
-        d->title = title;
+    if (d->titleLabel->text() != title) {
+        d->titleLabel->setText(title);
         config().writeEntry("extenderTitle", title);
         update();
     }
@@ -166,7 +232,7 @@ void ExtenderItem::setTitle(const QString &title)
 
 QString ExtenderItem::title() const
 {
-    return d->title;
+    return d->titleLabel->text();
 }
 
 void ExtenderItem::setName(const QString &name)
@@ -184,21 +250,17 @@ void ExtenderItem::setWidget(QGraphicsItem *widget)
 {
     if (d->widget) {
         d->widget->removeSceneEventFilter(this);
+        d->layout->removeItem(d->widget);
     }
 
-    if (!widget) {
+    if (!widget || !widget->isWidget()) {
         d->widget = 0;
         return;
     }
 
     widget->setParentItem(this);
-    widget->installSceneEventFilter(this);
-
-    QSizeF panelSize(QSizeF(size().width() - d->bgLeft - d->bgRight,
-                     d->iconSize + d->dragTop + d->dragBottom));
-    widget->setPos(QPointF(d->bgLeft + d->dragLeft, panelSize.height() + d->bgTop));
-    d->widget = widget;
-    d->updateSizeHints();
+    d->widget = static_cast<QGraphicsWidget *>(widget);
+    d->layout->insertItem(1, d->widget);
     d->widget->setVisible(!d->collapsed);
 }
 
@@ -256,7 +318,7 @@ void ExtenderItem::setExtender(Extender *extender, const QPointF &pos)
 
     //move the configuration.
     if (d->hostApplet() && (extender != d->extender)) {
-        KConfigGroup c = extender->d->applet->config("ExtenderItems");
+        KConfigGroup c = extender->d->applet.data()->config("ExtenderItems");
         config().reparent(&c);
     }
 
@@ -265,7 +327,11 @@ void ExtenderItem::setExtender(Extender *extender, const QPointF &pos)
 
     setParentItem(extender);
     setParent(extender);
+    if (d->extender) {
+        disconnect(d->extender->applet(), SIGNAL(immutabilityChanged(Plasma::ImmutabilityType)), this, SLOT(updateToolBox()));
+    }
     d->extender = extender;
+    connect(d->extender->applet(), SIGNAL(immutabilityChanged(Plasma::ImmutabilityType)), this, SLOT(updateToolBox()));
 
     //change parent.
     extender->d->addExtenderItem(this, pos);
@@ -311,7 +377,13 @@ Extender *ExtenderItem::extender() const
     return d->extender;
 }
 
+//TODO KDE5: only one setGroup()
 void ExtenderItem::setGroup(ExtenderGroup *group)
+{
+    setGroup(group, QPointF(-1, -1));
+}
+
+void ExtenderItem::setGroup(ExtenderGroup *group, const QPointF &pos)
 {
     if (isGroup()) {
         //nesting extender groups is just insane. I don't think we'd even want to support that.
@@ -319,6 +391,7 @@ void ExtenderItem::setGroup(ExtenderGroup *group)
         return;
     }
 
+    ExtenderGroup *oldGroup = d->group;
     d->group = group;
 
     if (group) {
@@ -328,8 +401,11 @@ void ExtenderItem::setGroup(ExtenderGroup *group)
             kDebug() << "moving to another extender because we're joining a detached group.";
             setExtender(group->extender());
         }
-        group->d->addItemToGroup(this);
+        group->d->addItemToGroup(this, pos);
     } else {
+        if (oldGroup) {
+            oldGroup->d->removeItemFromGroup(this);
+        }
         config().deleteEntry("group");
     }
 }
@@ -341,7 +417,7 @@ ExtenderGroup *ExtenderItem::group() const
 
 bool ExtenderItem::isGroup() const
 {
-    return (config().readEntry("isGroup", false));
+    return (config().readEntry("isGroup", false) && qobject_cast<const Plasma::ExtenderGroup *>(this));
 }
 
 bool ExtenderItem::isCollapsed() const
@@ -439,9 +515,11 @@ void ExtenderItem::destroy()
 
     //remove global entry if needed.
     Corona *corona = qobject_cast<Corona*>(scene());
-    KConfigGroup extenderItemGroup(corona->config(), "DetachedExtenderItems");
-    if (extenderItemGroup.hasGroup(QString::number(d->extenderItemId))) {
-        extenderItemGroup.deleteGroup(QString::number(d->extenderItemId));
+    if (corona) {
+        KConfigGroup extenderItemGroup(corona->config(), "DetachedExtenderItems");
+        if (extenderItemGroup.hasGroup(QString::number(d->extenderItemId))) {
+            extenderItemGroup.deleteGroup(QString::number(d->extenderItemId));
+        }
     }
 
     d->hostApplet()->config("ExtenderItems").deleteGroup(QString::number(d->extenderItemId));
@@ -455,9 +533,18 @@ void ExtenderItem::setCollapsed(bool collapsed)
 {
     config().writeEntry("isCollapsed", collapsed);
     d->collapsed = collapsed;
+    d->collapseIcon->setToolTip(collapsed ? i18n("Expand this widget") : i18n("Collapse this widget"));
     if (d->widget) {
         d->widget->setVisible(!collapsed);
-        d->updateSizeHints();
+        if (collapsed) {
+            d->layout->removeItem(d->widget);
+        } else {
+            d->layout->insertItem(1, d->widget);
+        }
+        updateGeometry();
+        if (extender()) {
+            extender()->resize(extender()->effectiveSizeHint(Qt::PreferredSize));
+        }
     }
 }
 
@@ -466,75 +553,20 @@ void ExtenderItem::returnToSource()
     if (!d || !d->sourceApplet) {
         return;
     }
+
     if (d->sourceApplet->d) {
         setExtender(d->sourceApplet->extender());
     }
 }
 
-void ExtenderItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *option,
-                         QWidget *widget)
+void ExtenderItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *)
 {
-    Q_UNUSED(option);
-    Q_UNUSED(widget);
-
-    painter->setRenderHint(QPainter::TextAntialiasing, true);
-    painter->setRenderHint(QPainter::Antialiasing, true);
-
     if (d->background->enabledBorders() != (FrameSvg::LeftBorder | FrameSvg::RightBorder) &&
         d->background->enabledBorders()) {
         //Don't paint if only the left and right borders are enabled, we only use the left and right
         //border in this situation to set the correct margins on this item.
-        d->background->paintFrame(painter, QPointF(0, 0));
+        d->background->paintFrame(painter, option->exposedRect, option->exposedRect);
     }
-
-    d->dragger->paintFrame(painter, QPointF(d->bgLeft, d->bgTop));
-
-    //We don't need to paint a title if there is no title.
-    if (d->title.isEmpty()) {
-        return;
-    }
-
-    //set the font for the title.
-    Plasma::Theme *theme = Plasma::Theme::defaultTheme();
-    QFont font = theme->font(Plasma::Theme::DefaultFont);
-    font.setWeight(QFont::Bold);
-
-    //create a pixmap with the title that is faded out at the right side of the titleRect.
-    QRectF rect;
-    if (option->direction == Qt::LeftToRight) {
-        rect = QRectF(d->titleRect().width() - 30, 0, 30, d->titleRect().height());
-    } else {
-        rect = QRectF(0, 0, 30, d->titleRect().height());
-    }
-
-    QPixmap pixmap(d->titleRect().size().toSize());
-    pixmap.fill(Qt::transparent);
-
-    QPainter p(&pixmap);
-    p.setPen(theme->color(Plasma::Theme::TextColor));
-    p.setFont(font);
-    p.drawText(QRectF(QPointF(0, 0), d->titleRect().size()),
-               Qt::TextSingleLine | Qt::AlignVCenter | Qt::AlignLeft,
-               d->title);
-
-    // Create the alpha gradient for the fade out effect
-    QLinearGradient alphaGradient(0, 0, 1, 0);
-    alphaGradient.setCoordinateMode(QGradient::ObjectBoundingMode);
-    //TODO: correct handling of right to left text.
-    if (option->direction == Qt::LeftToRight) {
-        alphaGradient.setColorAt(0, QColor(0, 0, 0, 255));
-        alphaGradient.setColorAt(1, QColor(0, 0, 0, 0));
-    } else {
-        alphaGradient.setColorAt(1, QColor(0, 0, 0, 255));
-        alphaGradient.setColorAt(0, QColor(0, 0, 0, 0));
-    }
-
-    p.setCompositionMode(QPainter::CompositionMode_DestinationIn);
-    p.fillRect(rect, alphaGradient);
-
-    p.end();
-
-    painter->drawPixmap(d->titleRect().topLeft(), pixmap);
 }
 
 void ExtenderItem::moveEvent(QGraphicsSceneMoveEvent *event)
@@ -546,27 +578,33 @@ void ExtenderItem::moveEvent(QGraphicsSceneMoveEvent *event)
 void ExtenderItem::resizeEvent(QGraphicsSceneResizeEvent *event)
 {
     Q_UNUSED(event)
-    d->themeChanged();
+    //resize the applet background
+    d->background->resizeFrame(size());
+    //d->resizeContent(size());
 }
 
 void ExtenderItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
 {
     if (!(d->dragHandleRect().contains(event->pos())) ||
-        d->extender->d->applet->immutability() != Plasma::Mutable) {
+        d->extender->d->applet.data()->immutability() != Plasma::Mutable) {
         event->ignore();
         return;
     }
-
-    d->mousePos = event->pos().toPoint();
 }
 
 void ExtenderItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
 {
-    if ((event->pos().toPoint() - d->mousePos).manhattanLength()
+    QPoint mousePressPos = event->buttonDownPos(Qt::LeftButton).toPoint();
+    if (!(event->buttons() & Qt::LeftButton) ||
+        (event->pos().toPoint() - mousePressPos).manhattanLength()
         < QApplication::startDragDistance()) {
         return;
     }
 
+    if (!d->extender->d->applet) {
+        return;
+    }
+
     //Start the drag:
     d->dragStarted = true;
     QPointF curPos = pos();
@@ -588,7 +626,7 @@ void ExtenderItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
     pixmap.fill(Qt::transparent);
     QPainter p(&pixmap);
 
-    //the following is necessary to avoid having an offset when rendering the widget into the
+    //the following is necesarry to avoid having an offset when rendering the widget into the
     //pixmap.
     view.setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
     view.setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
@@ -599,14 +637,14 @@ void ExtenderItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
     view.setSceneRect(sceneBoundingRect());
     view.render(&p, QRectF(QPointF(0, 0), pixmap.size()), QRect(QPoint(0, 0), screenSize));
 
-    //create the necessary mimedata.
+    //create the necesarry mimedata.
     ExtenderItemMimeData *mimeData = new ExtenderItemMimeData();
     mimeData->setExtenderItem(this);
-    mimeData->setPointerOffset(d->mousePos);
+    mimeData->setPointerOffset(mousePressPos);
 
     //Hide empty internal extender containers when we drag the last item away. Avoids having
     //an ugly empty applet on the desktop temporarily.
-    ExtenderApplet *extenderApplet = qobject_cast<ExtenderApplet*>(d->extender->d->applet);
+    ExtenderApplet *extenderApplet = qobject_cast<ExtenderApplet*>(d->extender->d->applet.data());
     if (extenderApplet && d->extender->attachedItems().count() < 2 &&
         extenderApplet->formFactor() != Plasma::Horizontal &&
         extenderApplet->formFactor() != Plasma::Vertical) {
@@ -621,12 +659,17 @@ void ExtenderItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
         group->collapseGroup();
     }
 
+    if (!isGroup() && this->group()) {
+        setGroup(0);
+    }
+
     //and execute the drag.
-    QWidget *dragParent = extender()->d->applet->view();
+    QWidget *dragParent = extender()->d->applet.data()->view();
     QDrag *drag = new QDrag(dragParent);
     drag->setPixmap(pixmap);
     drag->setMimeData(mimeData);
-    drag->setHotSpot(d->mousePos);
+    drag->setHotSpot(mousePressPos);
+
     Qt::DropAction action = drag->exec();
 
     corona->removeOffscreenWidget(this);
@@ -640,28 +683,22 @@ void ExtenderItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
         if (extenderApplet) {
             extenderApplet->show();
         }
-
-        d->extender->itemAddedEvent(this, curPos);
     }
 
     if (isGroup() && !collapsedGroup) {
         group->expandGroup();
     }
-
 }
 
 void ExtenderItem::mouseDoubleClickEvent(QGraphicsSceneMouseEvent *event)
 {
-    if (d->titleRect().contains(event->pos())) {
+    if (d->dragHandleRect().contains(event->pos())) {
         d->toggleCollapse();
     }
 }
 
-bool ExtenderItem::sceneEventFilter(QGraphicsItem *watched, QEvent *event)
+bool ExtenderItem::sceneEventFilter(QGraphicsItem *, QEvent *)
 {
-    if (watched == d->widget && event->type() == QEvent::GraphicsSceneResize) {
-        d->updateSizeHints();
-    }
     return false;
 }
 
@@ -671,44 +708,17 @@ void ExtenderItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
     //not needed anymore, but here for binary compatibility
 }
 
-void ExtenderItem::hoverMoveEvent(QGraphicsSceneHoverEvent *event)
+void ExtenderItem::hoverMoveEvent(QGraphicsSceneHoverEvent *)
 {
-    if (d->titleRect().contains(event->pos()) &&
-        d->extender->d->applet->immutability() == Plasma::Mutable) {
-        if (!d->mouseOver) {
-            QApplication::setOverrideCursor(Qt::OpenHandCursor);
-            d->mouseOver = true;
-        }
-    } else {
-        if (d->mouseOver) {
-            QApplication::restoreOverrideCursor();
-            d->mouseOver = false;
-        }
-    }
 }
 
-void ExtenderItem::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
+void ExtenderItem::hoverLeaveEvent(QGraphicsSceneHoverEvent *)
 {
-    Q_UNUSED(event);
-
-    if (d->mouseOver) {
-        QApplication::restoreOverrideCursor();
-        d->mouseOver = false;
-    }
 }
 
 QSizeF ExtenderItem::sizeHint(Qt::SizeHint which, const QSizeF &constraint) const
 {
-    Q_UNUSED(constraint)
-    switch (which) {
-    case Qt::MinimumSize:
-        return d->minimumSize;
-    case Qt::MaximumSize:
-        return d->maximumSize;
-    case Qt::PreferredSize:
-    default:
-        return d->preferredSize;
-    }
+    return QGraphicsWidget::sizeHint(which, constraint);
 }
 
 ExtenderItemPrivate::ExtenderItemPrivate(ExtenderItem *extenderItem, Extender *hostExtender)
@@ -718,20 +728,13 @@ ExtenderItemPrivate::ExtenderItemPrivate(ExtenderItem *extenderItem, Extender *h
       extender(hostExtender),
       sourceApplet(0),
       group(0),
-      dragger(new FrameSvg(extenderItem)),
       background(new FrameSvg(extenderItem)),
       collapseIcon(0),
-      destroyButton(0),
-      title(QString()),
-      mouseOver(false),
       dragStarted(false),
       destroyActionVisibility(false),
       collapsed(false),
-      expirationTimer(0),
-      iconSize(qreal(0))
+      expirationTimer(0)
 {
-    dragLeft = dragTop = dragRight = dragBottom = 0;
-    bgLeft = bgTop = bgRight = bgBottom = 0;
 }
 
 ExtenderItemPrivate::~ExtenderItemPrivate()
@@ -743,15 +746,7 @@ ExtenderItemPrivate::~ExtenderItemPrivate()
 //returns a Rect containing the area of the detachable where the draghandle will be drawn.
 QRectF ExtenderItemPrivate::dragHandleRect()
 {
-    QSizeF panelSize(QSizeF(q->size().width() - bgLeft - bgRight,
-                     iconSize + dragTop + dragBottom));
-    return QRectF(QPointF(bgLeft, bgTop), panelSize);
-}
-
-QRectF ExtenderItemPrivate::titleRect()
-{
-    return dragHandleRect().adjusted(dragLeft + collapseIcon->size().width() + 1, dragTop,
-                                     -toolbox->size().width() - dragRight, -dragBottom);
+    return toolbox->boundingRect();
 }
 
 void ExtenderItemPrivate::toggleCollapse()
@@ -762,85 +757,114 @@ void ExtenderItemPrivate::toggleCollapse()
 void ExtenderItemPrivate::updateToolBox()
 {
     Q_ASSERT(toolbox);
-    Q_ASSERT(dragger);
     Q_ASSERT(toolboxLayout);
 
-    //TODO: only delete items that actually have to be deleted, current performance is horrible.
-    while (toolboxLayout->count()) {
-        QGraphicsLayoutItem *icon = toolboxLayout->itemAt(0);
-        QGraphicsWidget *widget = dynamic_cast<QGraphicsWidget*>(icon);
+    QAction *closeAction = actions.value("close");
+    QAction *returnToSourceAction = actions.value("extenderItemReturnToSource");
+    bool returnToSourceVisibility = q->isDetached() && sourceApplet && (hostApplet()->immutability() == Plasma::Mutable);
+    int closeIndex = -1;
+    int returnToSourceIndex = -1;
+    const int startingIndex = 2; // collapse item is index 0, title label is 1
+    const QSizeF widgetSize = collapseIcon->sizeFromIconSize(toolbox->iconSize());
+
+    QSet<QAction*> shownActions = actionsInOrder.toSet();
+    QHash<QAction *, IconWidget *> actionIcons;
+    for (int index = startingIndex; index < toolboxLayout->count(); ++index) {
+        IconWidget *widget = dynamic_cast<IconWidget*>(toolboxLayout->itemAt(index));
+
+        if (!widget) {
+            continue;
+        }
+
+        if (closeIndex == -1 && destroyActionVisibility &&
+            closeAction && widget->action() == closeAction) {
+            closeIndex = index;
+            continue;
+        }
+
+        if (returnToSourceIndex == -1 && returnToSourceVisibility &&
+            returnToSourceAction && widget->action() == returnToSourceAction) {
+            returnToSourceIndex = index;
+            continue;
+        }
+
+        if (shownActions.contains(widget->action())) {
+            actionIcons.insert(widget->action(), widget);
+            continue;
+        }
+
+        toolboxLayout->removeAt(index);
         widget->deleteLater();
-        toolboxLayout->removeAt(0);
     }
 
+    // ensure the collapseIcon is the correct size.
+    collapseIcon->setMinimumSize(widgetSize);
+    collapseIcon->setMaximumSize(widgetSize);
+
     //add the actions that are actually set to visible.
     foreach (QAction *action, actionsInOrder) {
-        if (action->isVisible()) {
-            IconWidget *icon = new IconWidget(q);
-            icon->setAction(action);
-            QSizeF size = icon->sizeFromIconSize(iconSize);
-            icon->setMinimumSize(size);
-            icon->setMaximumSize(size);
-            if (action != actions.value("close")) {
-                toolboxLayout->addItem(icon);
+        if (action->isVisible() && action != closeAction) {
+            IconWidget *icon = actionIcons.value(action);
+            if (!icon) {
+                icon = new IconWidget(q);
+                icon->setAction(action);
             }
+
+            icon->setMinimumSize(widgetSize);
+            icon->setMaximumSize(widgetSize);
+            icon->setCursor(Qt::ArrowCursor);
+            toolboxLayout->insertItem(startingIndex, icon);
         }
     }
 
     //add the returntosource icon if we are detached, and have a source applet.
-    if (q->isDetached() && sourceApplet) {
-        IconWidget *returnToSource = new IconWidget(q);
-        returnToSource->setSvg("widgets/configuration-icons", "return-to-source");
-        QSizeF size = returnToSource->sizeFromIconSize(iconSize);
-        returnToSource->setMinimumSize(size);
-        returnToSource->setMaximumSize(size);
+    if (returnToSourceVisibility && returnToSourceIndex == -1) {
+        IconWidget *returnToSourceIcon = new IconWidget(q);
+        if (!returnToSourceAction) {
+            returnToSourceAction = new QAction(q);
+            returnToSourceAction->setToolTip(i18n("Reattach"));
+            actions.insert("extenderItemReturnToSource", returnToSourceAction);
+            QObject::connect(returnToSourceAction, SIGNAL(triggered()), q, SLOT(returnToSource()));
+        }
 
-        toolboxLayout->addItem(returnToSource);
-        QObject::connect(returnToSource, SIGNAL(clicked()), q, SLOT(returnToSource()));
+        returnToSourceIcon->setAction(returnToSourceAction);
+        returnToSourceIcon->setSvg("widgets/configuration-icons", "return-to-source");
+        returnToSourceIcon->setMinimumSize(widgetSize);
+        returnToSourceIcon->setMaximumSize(widgetSize);
+        returnToSourceIcon->setCursor(Qt::ArrowCursor);
+
+        if (closeIndex == -1) {
+            toolboxLayout->addItem(returnToSourceIcon);
+        } else {
+            toolboxLayout->insertItem(closeIndex - 1, returnToSourceIcon);
+        }
     }
 
     //add the close icon if desired.
-    if (destroyActionVisibility) {
-        destroyButton = new IconWidget(q);
-        if (!actions.contains("close")) {
-            QAction *destroyAction = new QAction(q);
-            actions["close"] = destroyAction;
+    if (destroyActionVisibility && closeIndex == -1) {
+        IconWidget *destroyButton = new IconWidget(q);
+        if (!closeAction) {
+            closeAction = new QAction(q);
+            actions.insert("close", closeAction);
+            if (returnToSourceAction) {
+                returnToSourceAction->setToolTip(i18n("Close"));
+            }
+            QObject::connect(closeAction, SIGNAL(triggered()), q, SLOT(destroy()));
         }
-        destroyButton->setAction(actions["close"]);
+
+        destroyButton->setAction(closeAction);
         destroyButton->setSvg("widgets/configuration-icons", "close");
-        QSizeF size = destroyButton->sizeFromIconSize(iconSize);
-        destroyButton->setMinimumSize(size);
-        destroyButton->setMaximumSize(size);
-
+        destroyButton->setMinimumSize(widgetSize);
+        destroyButton->setMaximumSize(widgetSize);
+        destroyButton->setCursor(Qt::ArrowCursor);
         toolboxLayout->addItem(destroyButton);
-
-        QObject::connect(actions["close"], SIGNAL(triggered()), q, SLOT(destroy()));
-    } else if (actions.contains("close")) {
-        QAction *destroyAction = actions["close"];
-        actions.remove("close");
-        destroyAction->deleteLater();
     }
-
-    toolboxLayout->updateGeometry();
-
-    //position the toolbox correctly.
-    QSizeF minimum = toolboxLayout->minimumSize();
-    toolbox->resize(minimum);
-    repositionToolbox();
-}
-
-void ExtenderItemPrivate::repositionToolbox()
-{
-    QSizeF minimum = toolboxLayout->minimumSize();
-    toolbox->setPos(q->size().width() - minimum.width() - bgRight,
-                    (dragHandleRect().height()/2) -
-                    (minimum.height()/2) + bgTop);
 }
 
 Applet *ExtenderItemPrivate::hostApplet() const
 {
     if (extender) {
-        return extender->d->applet;
+        return extender->d->applet.data();
     } else {
         return 0;
     }
@@ -848,6 +872,7 @@ Applet *ExtenderItemPrivate::hostApplet() const
 
 void ExtenderItemPrivate::themeChanged()
 {
+    kDebug();
     if (dragStarted) {
         background->setImagePath("opaque/widgets/extender-background");
         background->setEnabledBorders(FrameSvg::AllBorders);
@@ -855,49 +880,12 @@ void ExtenderItemPrivate::themeChanged()
         background->setImagePath("widgets/extender-background");
         background->setEnabledBorders(extender->enabledBordersForItem(q));
     }
-    background->getMargins(bgLeft, bgTop, bgRight, bgBottom);
 
-    dragger->setImagePath("widgets/extender-dragger");
+    qreal left, top, right, bottom;
+    background->getMargins(left, top, right, bottom);
+    layout->setContentsMargins(left, top, right, bottom);
 
-    //Read the preferred icon size hint, look at the font size, and calculate the desired title bar
-    //icon height.
-    dragger->resize();
-    QSizeF size = dragger->elementSize("hint-preferred-icon-size");
-    size = size.expandedTo(QSizeF(KIconLoader::SizeSmall,KIconLoader::SizeSmall));
-
-    Plasma::Theme *theme = Plasma::Theme::defaultTheme();
-    QFont font = theme->font(Plasma::Theme::DefaultFont);
-    QFontMetrics fm(font);
-
-    iconSize = qMax(size.height(), (qreal) fm.height());
-
-
-    dragger->getMargins(dragLeft, dragTop, dragRight, dragBottom);
-
-    QSizeF panelSize(QSizeF(q->size().width() - bgLeft - bgRight,
-                     iconSize + dragTop + dragBottom));
-
-    //resize the collapse icon.
-    collapseIcon->resize(collapseIcon->sizeFromIconSize(iconSize));
-
-    //reposition the collapse icon based on the new margins and size.
-    collapseIcon->setPos(bgLeft + dragLeft,
-                         panelSize.height()/2 -
-                         collapseIcon->size().height()/2 + bgTop);
-
-    //reposition the widget based on the new margins.
-    if (widget) {
-        widget->setPos(QPointF(bgLeft + dragLeft, panelSize.height() +
-                                                  bgTop));
-    }
-
-    //reposition the toolbox.
-    repositionToolbox();
-
-    updateSizeHints();
-
-    if (!q->size().isEmpty())
-        resizeContent(q->size());
+    toolbox->updateTheme();
 }
 
 void ExtenderItemPrivate::sourceAppletRemoved()
@@ -908,34 +896,6 @@ void ExtenderItemPrivate::sourceAppletRemoved()
     updateToolBox();
 }
 
-void ExtenderItemPrivate::resizeContent(const QSizeF &newSize)
-{
-    qreal width = newSize.width();
-    qreal height = newSize.height();
-
-    //resize the dragger
-    dragger->resizeFrame(QSizeF(width - bgLeft - bgRight,
-                         iconSize + dragTop + dragBottom));
-
-    //resize the applet background
-    background->resizeFrame(newSize);
-
-    //resize the widget
-    if (widget && widget->isWidget()) {
-        QSizeF newWidgetSize(width - bgLeft - bgRight - dragLeft - dragRight,
-                             height - dragHandleRect().height() - bgTop - bgBottom -
-                             dragTop - dragBottom);
-
-        QGraphicsWidget *graphicsWidget = static_cast<QGraphicsWidget*>(widget);
-        graphicsWidget->resize(newWidgetSize);
-    }
-
-    //reposition the toolbox.
-    repositionToolbox();
-
-    q->update();
-}
-
 void ExtenderItemPrivate::actionDestroyed(QObject *o)
 {
     QAction *action = static_cast<QAction *>(o);
@@ -956,69 +916,14 @@ void ExtenderItemPrivate::actionDestroyed(QObject *o)
     }
 }
 
-void ExtenderItemPrivate::updateSizeHints()
+void ExtenderItemPrivate::setMovable(bool movable)
 {
-    if (!widget) {
-        return;
-    }
-
-    qreal marginWidth = bgLeft + bgRight + dragLeft + dragRight;
-    qreal marginHeight = bgTop + bgBottom + dragTop + dragBottom;
-
-    QSizeF min;
-    QSizeF pref;
-    QSizeF max;
-
-    if (widget->isWidget()) {
-        QGraphicsWidget *graphicsWidget = static_cast<QGraphicsWidget*>(widget);
-        min = graphicsWidget->effectiveSizeHint(Qt::MinimumSize);
-        pref = graphicsWidget->effectiveSizeHint(Qt::PreferredSize);
-        max = graphicsWidget->effectiveSizeHint(Qt::MaximumSize);
+    if (movable) {
+        titleLabel->setCursor(Qt::OpenHandCursor);
+        toolbox->setCursor(Qt::OpenHandCursor);
     } else {
-        min = widget->boundingRect().size();
-        pref = widget->boundingRect().size();
-        max = widget->boundingRect().size();
-    }
-
-    if (collapsed) {
-        preferredSize = QSizeF(pref.width() + marginWidth,
-                               dragHandleRect().height() + marginHeight);
-        minimumSize = QSizeF(min.width() + marginWidth,
-                             dragHandleRect().height() + marginHeight);
-        maximumSize = QSizeF(max.width() + marginWidth,
-                             dragHandleRect().height() + marginHeight);
-        q->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
-
-        if (collapseIcon) {
-            collapseIcon->setToolTip(i18n("Expand this widget"));
-        }
-    } else {
-        preferredSize = QSizeF(pref.width() + marginWidth,
-                               pref.height() + dragHandleRect().height() + marginHeight);
-        minimumSize = QSizeF(min.width() + marginWidth,
-                             min.height() + dragHandleRect().height() + marginHeight);
-        maximumSize = QSizeF(max.width() + marginWidth,
-                             max.height() + dragHandleRect().height() + marginHeight);
-
-        //set sane size policies depending on the appearance.
-        if (extender->d->appearance == Extender::TopDownStacked ||
-            extender->d->appearance == Extender::BottomUpStacked) {
-            //used in popups, so fixed make sense.
-            q->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
-        } else if (extender->d->appearance == Extender::NoBorders) {
-            //on the desktop or panel so take all the space we want.
-            q->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
-        }
-
-        if (collapseIcon) {
-            collapseIcon->setToolTip(i18n("Collapse this widget"));
-        }
-    }
-
-    q->updateGeometry();
-    //FIXME: it should be done from updateGeometry() ??!!??
-    if (extender && extender->layout()) {
-        extender->layout()->activate();
+        titleLabel->unsetCursor();
+        toolbox->unsetCursor();
     }
 }
 
