@@ -19,96 +19,26 @@
  */
 
 #include "framesvg.h"
+#include "private/framesvg_p.h"
 
 #include <QPainter>
 #include <QSize>
 #include <QBitmap>
 #include <QRegion>
 #include <QTimer>
+#include <QCryptographicHash>
 
 #include <kdebug.h>
 
 #include <plasma/theme.h>
+#include <plasma/applet.h>
 
 namespace Plasma
 {
 
-class FrameData
-{
-public:
-    FrameData()
-      : enabledBorders(FrameSvg::AllBorders),
-        frameSize(-1,-1)
-    {
-    }
-
-    FrameData(const FrameData &other)
-      : enabledBorders(other.enabledBorders),
-        frameSize(other.frameSize)
-    {
-    }
-
-    ~FrameData()
-    {
-    }
-
-    FrameSvg::EnabledBorders enabledBorders;
-    QPixmap cachedBackground;
-    QRegion cachedMask;
-    QSizeF frameSize;
-
-    //measures
-    int topHeight;
-    int leftWidth;
-    int rightWidth;
-    int bottomHeight;
-
-    //margins, are equal to the measures by default
-    int topMargin;
-    int leftMargin;
-    int rightMargin;
-    int bottomMargin;
-
-    //size of the svg where the size of the "center"
-    //element is contentWidth x contentHeight
-    bool noBorderPadding : 1;
-    bool stretchBorders : 1;
-    bool tileCenter : 1;
-};
-
-class FrameSvgPrivate
-{
-public:
-    FrameSvgPrivate(FrameSvg *psvg)
-      : q(psvg),
-        cacheAll(false),
-        saveTimer(0)
-    {
-    }
-
-    ~FrameSvgPrivate()
-    {
-        qDeleteAll(frames);
-        frames.clear();
-    }
-
-    void generateBackground(FrameData *frame);
-    void scheduledCacheUpdate();
-    void updateSizes();
-    void updateNeeded();
-    void updateAndSignalSizes();
-
-    Location location;
-    QString prefix;
-
-    FrameSvg *q;
-
-    bool cacheAll : 1;
-    QStringList framesToSave;
-    QTimer *saveTimer;
-
-    QHash<QString, FrameData*> frames;
-};
+// Any attempt to generate a frame whose width or height is larger than this
+// will be rejected
+static const int MAX_FRAME_SIZE = 100000;
 
 FrameSvg::FrameSvg(QObject *parent)
     : Svg(parent),
@@ -116,10 +46,6 @@ FrameSvg::FrameSvg(QObject *parent)
 {
     connect(this, SIGNAL(repaintNeeded()), this, SLOT(updateNeeded()));
     d->frames.insert(QString(), new FrameData());
-
-    d->saveTimer = new QTimer(this);
-    d->saveTimer->setSingleShot(true);
-    connect(d->saveTimer, SIGNAL(timeout()), this, SLOT(scheduledCacheUpdate()));
 }
 
 FrameSvg::~FrameSvg()
@@ -152,6 +78,10 @@ void FrameSvg::setEnabledBorders(const EnabledBorders borders)
 
 FrameSvg::EnabledBorders FrameSvg::enabledBorders() const
 {
+    if (d->frames.isEmpty()) {
+        return NoBorder;
+    }
+
     QHash<QString, FrameData*>::const_iterator it = d->frames.constFind(d->prefix);
 
     if (it != d->frames.constEnd()) {
@@ -180,10 +110,11 @@ void FrameSvg::setElementPrefix(Plasma::Location location)
             setElementPrefix(QString());
             break;
     }
+
     d->location = location;
 }
 
-void FrameSvg::setElementPrefix(const QString & prefix)
+void FrameSvg::setElementPrefix(const QString &prefix)
 {
     const QString oldPrefix(d->prefix);
 
@@ -194,21 +125,25 @@ void FrameSvg::setElementPrefix(const QString & prefix)
         if (!d->prefix.isEmpty()) {
             d->prefix += '-';
         }
-
     }
 
-    if (oldPrefix == d->prefix && d->frames[oldPrefix]) {
+    FrameData *oldFrameData = d->frames.value(oldPrefix);
+    if (oldPrefix == d->prefix && oldFrameData) {
         return;
     }
 
     if (!d->frames.contains(d->prefix)) {
-        d->frames.insert(d->prefix, new FrameData(*(d->frames[oldPrefix])));
+        if (oldFrameData) {
+            d->frames.insert(d->prefix, new FrameData(*oldFrameData));
+        } else {
+            d->frames.insert(d->prefix, new FrameData());
+        }
+
         d->updateSizes();
     }
 
     if (!d->cacheAll) {
         delete d->frames[oldPrefix];
-        d->framesToSave.removeAll(oldPrefix);
         d->frames.remove(oldPrefix);
     }
 
@@ -268,17 +203,17 @@ void FrameSvg::resizeFrame(const QSizeF &size)
     }
 
     d->updateSizes();
-    d->frames[d->prefix]->frameSize = size;
+    d->frames[d->prefix]->frameSize = size.toSize();
 }
 
 QSizeF FrameSvg::frameSize() const
 {
     QHash<QString, FrameData*>::const_iterator it = d->frames.constFind(d->prefix);
 
-    if (it != d->frames.constEnd()) {
-        return it.value()->frameSize;
-    } else {
+    if (it == d->frames.constEnd()) {
         return QSize(-1, -1);
+    } else {
+        return d->frameSize(it.value());
     }
 }
 
@@ -312,7 +247,7 @@ void FrameSvg::getMargins(qreal &left, qreal &top, qreal &right, qreal &bottom)
 {
     FrameData *frame = d->frames[d->prefix];
 
-    if (!frame || frame->noBorderPadding) {
+    if (frame->noBorderPadding) {
         left = top = right = bottom = 0;
         return;
     }
@@ -338,44 +273,17 @@ QRectF FrameSvg::contentsRect() const
     }
 }
 
+QPixmap FrameSvg::alphaMask() const
+{
+    //FIXME: the distinction between overlay and 
+    return d->alphaMask();
+}
+
 QRegion FrameSvg::mask() const
 {
     FrameData *frame = d->frames[d->prefix];
-
     if (frame->cachedMask.isEmpty()) {
-        // ivan: we are testing whether we have the mask prefixed
-        // elements to use for creating the mask.
-        if (hasElement("mask-" + d->prefix + "center")) {
-            QString oldPrefix = d->prefix;
-
-            // We are setting the prefix only temporary to generate
-            // the needed mask image
-            d->prefix = "mask-" + oldPrefix;
-
-            if (!d->frames.contains(d->prefix)) {
-                d->frames.insert(d->prefix, new FrameData(*(d->frames[oldPrefix])));
-                d->updateSizes();
-            }
-
-            FrameData *maskFrame = d->frames[d->prefix];
-            if (maskFrame->cachedBackground.isNull()) {
-                d->generateBackground(maskFrame);
-                if (maskFrame->cachedBackground.isNull()) {
-                    return QRegion();
-                }
-            }
-
-            frame->cachedMask = QBitmap(maskFrame->cachedBackground.alphaChannel().createMaskFromColor(Qt::black));
-            d->prefix = oldPrefix;
-        } else {
-            if (frame->cachedBackground.isNull()) {
-                d->generateBackground(frame);
-                if (frame->cachedBackground.isNull()) {
-                    return QRegion();
-                }
-            }
-            frame->cachedMask = QRegion(QBitmap(frame->cachedBackground.alphaChannel().createMaskFromColor(Qt::black)));
-        }
+        frame->cachedMask = QRegion(QBitmap(d->alphaMask().alphaChannel().createMaskFromColor(Qt::black)));
     }
     return frame->cachedMask;
 }
@@ -398,15 +306,13 @@ void FrameSvg::clearCache()
 {
     FrameData *frame = d->frames[d->prefix];
 
-    d->saveTimer->stop();
-    d->framesToSave.clear();
-
     // delete all the frames that aren't this one
     QMutableHashIterator<QString, FrameData*> it(d->frames);
     while (it.hasNext()) {
         FrameData *p = it.next().value();
 
         if (frame != p) {
+            //TODO: should we clear from the pixmap cache as well?
             delete p;
             it.remove();
         }
@@ -423,6 +329,7 @@ QPixmap FrameSvg::framePixmap()
         }
     }
 
+
     return frame->cachedBackground;
 }
 
@@ -452,34 +359,153 @@ void FrameSvg::paintFrame(QPainter *painter, const QPointF &pos)
     painter->drawPixmap(pos, frame->cachedBackground);
 }
 
+QPixmap FrameSvgPrivate::alphaMask()
+{
+    FrameData *frame = frames[prefix];
+    QString maskPrefix;
+
+    if (q->hasElement("mask-" + prefix + "center")) {
+        maskPrefix = "mask-";
+    }
+
+    if (!maskPrefix.isNull()) {
+        QString oldPrefix = prefix;
+
+        // We are setting the prefix only temporary to generate
+        // the needed mask image
+        prefix = maskPrefix + oldPrefix;
+
+        if (!frames.contains(prefix)) {
+            frames.insert(prefix, new FrameData(*(frames[oldPrefix])));
+            updateSizes();
+        }
+
+        FrameData *maskFrame = frames[prefix];
+        if (maskFrame->cachedBackground.isNull() || maskFrame->frameSize != frameSize(frame)) {
+            maskFrame->frameSize = frameSize(frame).toSize();
+            maskFrame->cachedBackground = QPixmap();
+
+            generateBackground(maskFrame);
+            if (maskFrame->cachedBackground.isNull()) {
+                return QPixmap();
+            }
+        }
+
+        prefix = oldPrefix;
+        return maskFrame->cachedBackground;
+    } else {
+        if (frame->cachedBackground.isNull()) {
+            generateBackground(frame);
+            if (frame->cachedBackground.isNull()) {
+                return QPixmap();
+            }
+        }
+        return frame->cachedBackground;
+    }
+}
+
 void FrameSvgPrivate::generateBackground(FrameData *frame)
 {
     if (!frame->cachedBackground.isNull()) {
         return;
     }
 
-    QString id = QString::fromLatin1("%5_%4_%3_%2_%1_").
-                         arg(frame->enabledBorders).arg(frame->frameSize.width()).arg(frame->frameSize.height()).arg(prefix).arg(q->imagePath());
+    const QString id = cacheId(frame, prefix);
 
     Theme *theme = Theme::defaultTheme();
-    if (theme->findInCache(id, frame->cachedBackground) && !frame->cachedBackground.isNull()) {
-        return;
+    bool frameCached = !frame->cachedBackground.isNull();
+    bool overlayCached = false;
+    const bool overlayAvailable = !prefix.startsWith("mask-") && q->hasElement(prefix + "overlay");
+    QPixmap overlay;
+    if (q->isUsingRenderingCache()) {
+        frameCached = theme->findInCache(id, frame->cachedBackground) && !frame->cachedBackground.isNull();
+
+        if (overlayAvailable) {
+            overlayCached = theme->findInCache("overlay_" + id, overlay) && !overlay.isNull();
+        }
     }
 
+    if (!frameCached) {
+        generateFrameBackground(frame);
+    }
+
+    //Overlays
+    QSize overlaySize;
+    QPoint actualOverlayPos = QPoint(0, 0);
+    if (overlayAvailable && !overlayCached) {
+        QPoint pos = QPoint(0, 0);
+        overlaySize = q->elementSize(prefix+"overlay");
+
+        //Random pos, stretched and tiled are mutually exclusive
+        if (q->hasElement(prefix + "hint-overlay-random-pos")) {
+            actualOverlayPos = overlayPos;
+        } else if (q->hasElement(prefix + "hint-overlay-pos-right")) {
+            actualOverlayPos.setX(frame->frameSize.width() - overlaySize.width());
+        } else if (q->hasElement(prefix + "hint-overlay-pos-bottom")) {
+            actualOverlayPos.setY(frame->frameSize.height() - overlaySize.height());
+        //Stretched or Tiled?
+        } else if (q->hasElement(prefix + "hint-overlay-stretch")) {
+            overlaySize = frameSize(frame).toSize();
+        } else {
+            if (q->hasElement(prefix + "hint-overlay-tile-horizontal")) {
+                overlaySize.setWidth(frameSize(frame).width());
+            }
+            if (q->hasElement(prefix + "hint-overlay-tile-vertical")) {
+                overlaySize.setHeight(frameSize(frame).height());
+            }
+        }
+
+        overlay = alphaMask();
+        QPainter overlayPainter(&overlay);
+        overlayPainter.setCompositionMode(QPainter::CompositionMode_SourceIn);
+        //Tiling?
+        if (q->hasElement(prefix+"hint-overlay-tile-horizontal") ||
+            q->hasElement(prefix+"hint-overlay-tile-vertical")) {
+
+            QSize s = q->size();
+            q->resize(q->elementSize(prefix+"overlay"));
+
+            overlayPainter.drawTiledPixmap(QRect(QPoint(0,0), overlaySize), q->pixmap(prefix+"overlay"));
+            q->resize(s);
+        } else {
+            q->paint(&overlayPainter, QRect(actualOverlayPos, overlaySize), prefix+"overlay");
+        }
+
+        overlayPainter.end();
+    }
+
+    if (!frameCached) {
+        cacheFrame(prefix, frame->cachedBackground, overlayCached ? overlay : QPixmap());
+    }
+
+    if (!overlay.isNull()) {
+        QPainter p(&frame->cachedBackground);
+        p.setCompositionMode(QPainter::CompositionMode_SourceOver);
+        p.drawPixmap(actualOverlayPos, overlay, QRect(actualOverlayPos, overlaySize));
+    }
+}
+
+void FrameSvgPrivate::generateFrameBackground(FrameData *frame)
+{
     //kDebug() << "generating background";
+    const QSizeF size = frameSize(frame);
     const int topWidth = q->elementSize(prefix + "top").width();
     const int leftHeight = q->elementSize(prefix + "left").height();
     const int topOffset = 0;
     const int leftOffset = 0;
 
 
-    if (!frame->frameSize.isValid()) {
-        kWarning() << "Invalid frame size" << frame->frameSize;
+    if (!size.isValid()) {
+        kWarning() << "Invalid frame size" << size;
+        return;
+    }
+    if (size.width() >= MAX_FRAME_SIZE || size.height() >= MAX_FRAME_SIZE) {
+        kWarning() << "Not generating frame background for a size whose width or height is more than" << MAX_FRAME_SIZE << size;
         return;
     }
 
-    const int contentWidth = frame->frameSize.width() - frame->leftWidth  - frame->rightWidth;
-    const int contentHeight = frame->frameSize.height() - frame->topHeight  - frame->bottomHeight;
+    const int contentWidth = size.width() - frame->leftWidth  - frame->rightWidth;
+    const int contentHeight = size.height() - frame->topHeight  - frame->bottomHeight;
     int contentTop = 0;
     int contentLeft = 0;
     int rightOffset = contentWidth;
@@ -492,23 +518,11 @@ void FrameSvgPrivate::generateBackground(FrameData *frame)
     p.setCompositionMode(QPainter::CompositionMode_Source);
     p.setRenderHint(QPainter::SmoothPixmapTransform);
 
-    //if we must stretch the center or the borders we compute how much we will have to stretch
-    //the svg to get the desired element sizes
-    QSizeF  scaledContentSize(0, 0);
-    if (q->elementSize(prefix + "center").width() > 0 &&
-        q->elementSize(prefix + "center").height() > 0 &&
-        (!frame->tileCenter || frame->stretchBorders)) {
-        scaledContentSize = QSizeF(contentWidth * ((qreal)q->size().width() / (qreal)q->elementSize(prefix + "center").width()),
-                                   contentHeight * ((qreal)q->size().height() / (qreal)q->elementSize(prefix + "center").height()));
-    }
-
     //CENTER
     if (frame->tileCenter) {
         if (contentHeight > 0 && contentWidth > 0) {
-            int centerTileHeight;
-            int centerTileWidth;
-            centerTileHeight = q->elementSize(prefix + "center").height();
-            centerTileWidth = q->elementSize(prefix + "center").width();
+            const int centerTileHeight = q->elementSize(prefix + "center").height();
+            const int centerTileWidth = q->elementSize(prefix + "center").width();
             QPixmap center(centerTileWidth, centerTileHeight);
             center.fill(Qt::transparent);
 
@@ -518,17 +532,36 @@ void FrameSvgPrivate::generateBackground(FrameData *frame)
                 q->paint(&centerPainter, QRect(QPoint(0, 0), q->elementSize(prefix + "center")), prefix + "center");
             }
 
-            p.drawTiledPixmap(QRect(frame->leftWidth, frame->topHeight,
-                                    contentWidth, contentHeight), center);
+            if (frame->composeOverBorder) {
+                p.drawTiledPixmap(QRect(QPoint(0, 0), size.toSize()), center);
+            } else {
+                p.drawTiledPixmap(QRect(frame->leftWidth, frame->topHeight,
+                                        contentWidth, contentHeight), center);
+            }
         }
     } else {
         if (contentHeight > 0 && contentWidth > 0) {
-            q->paint(&p, QRect(frame->leftWidth, frame->topHeight,
-                               contentWidth, contentHeight),
+            if (frame->composeOverBorder) {
+                q->paint(&p, QRect(QPoint(0, 0), size.toSize()),
                                prefix + "center");
+            } else {
+                q->paint(&p, QRect(frame->leftWidth, frame->topHeight,
+                                contentWidth, contentHeight),
+                                prefix + "center");
+            }
         }
     }
 
+    if (frame->composeOverBorder) {
+        p.setCompositionMode(QPainter::CompositionMode_DestinationIn);
+        p.drawPixmap(QRect(QPoint(0, 0), size.toSize()), alphaMask());
+        p.setCompositionMode(QPainter::CompositionMode_SourceOver);
+    }
+
+    if (frame->enabledBorders & FrameSvg::LeftBorder && q->hasElement(prefix + "left")) {
+        rightOffset += frame->leftWidth;
+    }
+
     // Corners
     if (frame->enabledBorders & FrameSvg::TopBorder && q->hasElement(prefix + "top")) {
         contentTop = frame->topHeight;
@@ -538,7 +571,6 @@ void FrameSvgPrivate::generateBackground(FrameData *frame)
             q->paint(&p, QRect(leftOffset, topOffset, frame->leftWidth, frame->topHeight), prefix + "topleft");
 
             contentLeft = frame->leftWidth;
-            rightOffset = contentWidth + frame->leftWidth;
         }
 
         if (q->hasElement(prefix + "topright") && frame->enabledBorders & FrameSvg::RightBorder) {
@@ -551,7 +583,6 @@ void FrameSvgPrivate::generateBackground(FrameData *frame)
             q->paint(&p, QRect(leftOffset, bottomOffset, frame->leftWidth, frame->bottomHeight), prefix + "bottomleft");
 
             contentLeft = frame->leftWidth;
-            rightOffset = contentWidth + frame->leftWidth;
         }
 
         if (frame->enabledBorders & FrameSvg::RightBorder && q->hasElement(prefix + "bottomright")) {
@@ -632,29 +663,44 @@ void FrameSvgPrivate::generateBackground(FrameData *frame)
         }
     }
 
-    if (!framesToSave.contains(prefix)) {
-        framesToSave.append(prefix);
-    }
-
-    saveTimer->start(300);
 }
 
-void FrameSvgPrivate::scheduledCacheUpdate()
+QString FrameSvgPrivate::cacheId(FrameData *frame, const QString &prefixToSave) const
 {
-    foreach ( QString prefixToSave, framesToSave) {
-        FrameData *frame = frames[prefix];
-        framesToSave.removeAll(prefixToSave);
+    const QSizeF size = frameSize(frame);
+    return QString::fromLatin1("%5_%4_%3_%2_%1_").arg(frame->enabledBorders)
+                                                 .arg(size.width())
+                                                 .arg(size.height())
+                                                 .arg(prefixToSave)
+                                                 .arg(q->imagePath());
+}
 
-        QString id = QString::fromLatin1("%5_%4_%3_%2_%1_").
-                            arg(frame->enabledBorders).arg(frame->frameSize.width()).arg(frame->frameSize.height()).arg(prefix).arg(q->imagePath());
+void FrameSvgPrivate::cacheFrame(const QString &prefixToSave, const QPixmap &background, const QPixmap &overlay)
+{
+    if (!q->isUsingRenderingCache()) {
+        return;
+    }
 
-        //kDebug()<<"Saving to cache frame"<<id;
+    //insert background
+    FrameData *frame = frames.value(prefixToSave);
 
-        Theme::defaultTheme()->insertIntoCache(id, frame->cachedBackground);
+    if (!frame) {
+        return;
+    }
+
+    const QString id = cacheId(frame, prefixToSave);
+
+    //kDebug()<<"Saving to cache frame"<<id;
+
+    Theme::defaultTheme()->insertIntoCache(id, background, QString::number((qint64)q, 16)+prefixToSave);
+
+    if (!overlay.isNull()) {
+        //insert overlay
+        Theme::defaultTheme()->insertIntoCache("overlay_" + id, overlay, QString::number((qint64)q, 16) + prefixToSave + "overlay");
     }
 }
 
-void FrameSvgPrivate::updateSizes()
+void FrameSvgPrivate::updateSizes() const
 {
     //kDebug() << "!!!!!!!!!!!!!!!!!!!!!! updating sizes" << prefix;
     FrameData *frame = frames[prefix];
@@ -713,7 +759,11 @@ void FrameSvgPrivate::updateSizes()
         frame->bottomMargin = frame->bottomHeight = 0;
     }
 
+    frame->composeOverBorder = (q->hasElement(prefix + "hint-compose-over-border") &&
+                                q->hasElement(prefix + "mask-center"));
+
     //since it's rectangular, topWidth and bottomWidth must be the same
+    //the ones that don't have a prefix is for retrocompatibility
     frame->tileCenter = q->hasElement("hint-tile-center");
     frame->noBorderPadding = q->hasElement("hint-no-border-padding");
     frame->stretchBorders = q->hasElement("hint-stretch-borders");
@@ -732,6 +782,17 @@ void FrameSvgPrivate::updateAndSignalSizes()
     emit q->repaintNeeded();
 }
 
+QSizeF FrameSvgPrivate::frameSize(FrameData *frame) const
+{
+    if (!frame->frameSize.isValid()) {
+        updateSizes();
+        frame->frameSize = q->size();
+    }
+
+    return frame->frameSize;
+}
+
+
 } // Plasma namespace
 
 #include "framesvg.moc"
