@@ -46,6 +46,8 @@
 #include "plasma/extenders/extender.h"
 #include "plasma/extenders/extenderitem.h"
 #include "plasma/package.h"
+#include "plasma/theme.h"
+#include "plasma/scripting/appletscript.h"
 #include "plasma/tooltipmanager.h"
 #include "plasma/widgets/iconwidget.h"
 
@@ -56,22 +58,12 @@ PopupApplet::PopupApplet(QObject *parent, const QVariantList &args)
     : Plasma::Applet(parent, args),
       d(new PopupAppletPrivate(this))
 {
-    int iconSize = IconSize(KIconLoader::Desktop);
-    resize(iconSize, iconSize);
-    disconnect(this, SIGNAL(activate()), (Applet*)this, SLOT(setFocus()));
-    connect(this, SIGNAL(activate()), this, SLOT(togglePopup()));
-    setAcceptDrops(true);
 }
 
 PopupApplet::PopupApplet(const QString &packagePath, uint appletId, const QVariantList &args)
     : Plasma::Applet(packagePath, appletId, args),
       d(new PopupAppletPrivate(this))
 {
-    int iconSize = IconSize(KIconLoader::Desktop);
-    resize(iconSize, iconSize);
-    disconnect(this, SIGNAL(activate()), (Applet*)this, SLOT(setFocus()));
-    connect(this, SIGNAL(activate()), this, SLOT(togglePopup()));
-    setAcceptDrops(true);
 }
 
 PopupApplet::~PopupApplet()
@@ -111,14 +103,40 @@ void PopupApplet::setPopupIcon(const QIcon &icon)
 void PopupApplet::setPopupIcon(const QString &iconName)
 {
     if (package()) {
+        //Attempt1: is it in the plasmoid package?
         const QString file = package()->filePath("images", iconName);
         if (!file.isEmpty()) {
             setPopupIcon(KIcon(file));
             return;
         }
+    //Attempt2: is it a svg in the icons directory?
     }
+    QString name = QString("icons/") + iconName.split("-").first();
+    if (!Plasma::Theme::defaultTheme()->imagePath(name).isEmpty()) {
+        if (!d->icon) {
+            d->icon = new Plasma::IconWidget(this);
+            d->icon->setSvg(name, iconName);
+            if (d->icon->svg().isEmpty()) {
+                setPopupIcon(KIcon(iconName));
+            }
+            connect(d->icon, SIGNAL(clicked()), this, SLOT(internalTogglePopup()));
 
-    setPopupIcon(KIcon(iconName));
+            QGraphicsLinearLayout *layout = new QGraphicsLinearLayout();
+            layout->setContentsMargins(0, 0, 0, 0);
+            layout->setSpacing(0);
+            layout->setOrientation(Qt::Horizontal);
+
+            setLayout(layout);
+        } else {
+            d->icon->setSvg(name, iconName);
+            if (d->icon->svg().isEmpty()) {
+                setPopupIcon(KIcon(iconName));
+            }
+        }
+    // Final Attempt: use KIcon
+    } else {
+        setPopupIcon(KIcon(iconName));
+    }
 }
 
 QIcon PopupApplet::popupIcon() const
@@ -218,7 +236,7 @@ void PopupAppletPrivate::popupConstraintsEvent(Plasma::Constraints constraints)
          (f == Plasma::Vertical || f == Plasma::Horizontal))) {
         QGraphicsLinearLayout *lay = dynamic_cast<QGraphicsLinearLayout *>(q->layout());
 
-        if (icon && !icon->icon().isNull() && lay) {
+        if (icon && !icon->icon().isNull() && lay && lay->count() > 0) {
             lay->removeAt(0);
         }
 
@@ -242,18 +260,32 @@ void PopupAppletPrivate::popupConstraintsEvent(Plasma::Constraints constraints)
         }
 
         //99% of the times q->parentWidget() is the containment, but using it  we can also manage the applet-in-applet case (i.e. systray)
+        //there are also cases where the parentlayoutitem is bigger than the containment (e.g. newspaper)
         if (q->parentLayoutItem()) {
             parentSize = q->parentLayoutItem()->geometry().size();
         } else if (q->parentWidget()) {
             parentSize = q->parentWidget()->size();
         }
 
+        //check if someone did the nasty trick of applets in applets, in this case we always want to be collapsed
+        QGraphicsWidget *candidateParentApplet = q;
+        Plasma::Applet *parentApplet = 0;
+        //this loop should be executed normally a single time, at most 2-3 times for quite complex containments
+        while (candidateParentApplet) {
+            candidateParentApplet = candidateParentApplet->parentWidget();
+            parentApplet = qobject_cast<Plasma::Applet *>(candidateParentApplet);
+            if (parentApplet) {
+                break;
+            }
+        }
+
         //Applet on desktop
-        if (icon && !icon->icon().isNull() && ((f != Plasma::Vertical && f != Plasma::Horizontal) ||
+        if ((!parentApplet || parentApplet->isContainment() ) && icon && (!icon->svg().isEmpty() || !icon->icon().isNull()) && ((f != Plasma::Vertical && f != Plasma::Horizontal) ||
             ((f == Plasma::Vertical && parentSize.width() >= minimum.width()) ||
              (f == Plasma::Horizontal && parentSize.height() >= minimum.height())))) {
             //kDebug() << "we are expanding the popupapplet";
 
+
             // we only switch to expanded if we aren't horiz/vert constrained and
             // this applet has an icon.
             // otherwise, we leave it up to the applet itself to figure it out
@@ -325,7 +357,7 @@ void PopupAppletPrivate::popupConstraintsEvent(Plasma::Constraints constraints)
             qreal left, top, right, bottom;
             q->getContentsMargins(&left, &top, &right, &bottom);
             QSizeF oldSize(q->size());
-            q->setMinimumSize(minimum + QSizeF(left+right, top+bottom));
+
             //size not saved/invalid size saved
             if (oldSize.width() < q->minimumSize().width() || oldSize.height() < q->minimumSize().height()) {
                 q->resize(prefSize);
@@ -348,7 +380,7 @@ void PopupAppletPrivate::popupConstraintsEvent(Plasma::Constraints constraints)
                     q->setAspectRatioMode(Plasma::ConstrainedSquare);
                 }
 
-                Dialog *dialog = new Plasma::Dialog();
+                Dialog *dialog = new Dialog();
                 dialogPtr = dialog;
 
                 dialog->setAspectRatioMode(savedAspectRatio);
@@ -358,20 +390,15 @@ void PopupAppletPrivate::popupConstraintsEvent(Plasma::Constraints constraints)
                 //emulate the same kind of behavior as Qt::Popup (close when you click somewhere
                 //else.
 
-                if (icon) {
-                    q->setMinimumSize(QSize(0, 0));
-                }
-
                 if (gWidget) {
                     Corona *corona = qobject_cast<Corona *>(gWidget->scene());
 
-                    //could that cast ever fail??
                     if (corona) {
                         corona->addOffscreenWidget(gWidget);
-                        dialog->setGraphicsWidget(gWidget);
-                        gWidget->resize(gWidget->preferredSize());
                     }
 
+                    dialog->setGraphicsWidget(gWidget);
+                    gWidget->resize(gWidget->preferredSize());
                     dialog->setWindowFlags(Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint | (gWidget->windowFlags() & Qt::X11BypassWindowManagerHint));
                 } else if (qWidget) {
                     QVBoxLayout *l_layout = new QVBoxLayout(dialog);
@@ -384,6 +411,7 @@ void PopupAppletPrivate::popupConstraintsEvent(Plasma::Constraints constraints)
                     dialog->setWindowFlags(Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint);
                 }
 
+                restoreDialogSize();
                 KWindowSystem::setState(dialog->winId(), NET::SkipTaskbar | NET::SkipPager);
                 dialog->installEventFilter(q);
 
@@ -394,10 +422,6 @@ void PopupAppletPrivate::popupConstraintsEvent(Plasma::Constraints constraints)
             if (icon && lay) {
                 lay->addItem(icon);
             }
-
-            if (icon) {
-                q->setMinimumSize(0,0);
-            }
         }
     }
 
@@ -408,6 +432,33 @@ void PopupAppletPrivate::popupConstraintsEvent(Plasma::Constraints constraints)
     emit q->sizeHintChanged(Qt::PreferredSize);
 }
 
+void PopupAppletPrivate::appletActivated()
+{
+    q->setStatus(Plasma::NeedsAttentionStatus);
+    QTimer::singleShot(0, q, SLOT(showPopup()));
+}
+
+QSizeF PopupApplet::sizeHint(Qt::SizeHint which, const QSizeF & constraint) const
+{
+    if (!d->dialogPtr || which != Qt::PreferredSize) {
+        return Applet::sizeHint(which, constraint);
+    }
+
+    switch (formFactor()) {
+        case Vertical:
+        case Horizontal: {
+            const int size = IconSize(KIconLoader::Panel);
+            return QSizeF(size, size);
+            break;
+        }
+        default:
+            break;
+    }
+
+    const int size = IconSize(KIconLoader::Desktop);
+    return QSizeF(size, size);
+}
+
 void PopupApplet::mousePressEvent(QGraphicsSceneMouseEvent *event)
 {
     if (!d->icon && !d->popupLostFocus && event->buttons() == Qt::LeftButton) {
@@ -434,29 +485,29 @@ bool PopupApplet::eventFilter(QObject *watched, QEvent *event)
 {
     if (!d->passive && watched == d->dialogPtr.data() && (event->type() == QEvent::WindowDeactivate)) {
         d->popupLostFocus = true;
-        hidePopup();
         QTimer::singleShot(100, this, SLOT(clearPopupLostFocus()));
     }
 
-    /**
-    if (layout() && watched == graphicsWidget() && (event->type() == QEvent::GraphicsSceneResize)) {
-        //sizes are recalculated in the constraintsevent so let's just call that.
-        d->popupConstraintsEvent(Plasma::FormFactorConstraint);
-
-        //resize vertically if necesarry.
-        if (formFactor() == Plasma::MediaCenter || formFactor() == Plasma::Planar) {
-            resize(QSizeF(size().width(), minimumHeight()));
-        }
-    }
-    */
-
     if (watched == d->dialogPtr.data() && event->type() == QEvent::ContextMenu) {
         //pass it up to the applet
         //well, actually we have to pass it to the *containment*
         //because all the code for showing an applet's contextmenu is actually in Containment.
         Containment *c = containment();
         if (c) {
-            return c->d->showAppletContextMenu(this, static_cast<QContextMenuEvent*>(event)->globalPos());
+            Applet *applet = this;
+            Dialog *dialog = d->dialogPtr.data();
+            if (dialog && dialog->graphicsWidget()) {
+                int left, top, right, bottom;
+                dialog->getContentsMargins(&left, &top, &right, &bottom);
+                const QPoint eventPos = static_cast<QContextMenuEvent*>(event)->pos() - QPoint(left, top);
+                QPointF pos = dialog->graphicsWidget()->mapToScene(eventPos);
+
+                if (Applet *actual = c->d->appletAt(pos)) {
+                    applet = actual;
+                }
+            }
+
+            return c->d->showAppletContextMenu(applet, static_cast<QContextMenuEvent*>(event)->globalPos());
         }
     }
 
@@ -524,6 +575,10 @@ void PopupApplet::showPopup(uint popupDuration)
             }
 
             d->timer->start(popupDuration);
+        } else {
+            if (d->timer) {
+                d->timer->stop();
+            }
         }
     }
 }
@@ -550,8 +605,11 @@ Plasma::PopupPlacement PopupApplet::popupPlacement() const
     return d->popupPlacement;
 }
 
-void PopupApplet::popupEvent(bool)
+void PopupApplet::popupEvent(bool popped)
 {
+    if (Applet::d->script) {
+        emit Applet::d->script->popupEvent(popped);
+    }
 }
 
 void PopupApplet::setPassivePopup(bool passive)
@@ -580,6 +638,12 @@ PopupAppletPrivate::PopupAppletPrivate(PopupApplet *applet)
           popupLostFocus(false),
           passive(false)
 {
+    int iconSize = IconSize(KIconLoader::Desktop);
+    q->resize(iconSize, iconSize);
+    q->setAcceptDrops(true);
+    QObject::disconnect(q, SIGNAL(activate()), static_cast<Applet*>(q), SLOT(setFocus()));
+    QObject::connect(q, SIGNAL(activate()), q, SLOT(appletActivated()));
+    QObject::connect(KGlobalSettings::self(), SIGNAL(iconChanged(int)), q, SLOT(iconSizeChanged(int)));
 }
 
 PopupAppletPrivate::~PopupAppletPrivate()
@@ -592,6 +656,13 @@ PopupAppletPrivate::~PopupAppletPrivate()
     delete icon;
 }
 
+void PopupAppletPrivate::iconSizeChanged(int group)
+{
+    if (icon && (group == KIconLoader::Desktop || group == KIconLoader::Panel)) {
+        q->updateGeometry();
+    }
+}
+
 void PopupAppletPrivate::internalTogglePopup()
 {
     if (timer) {
@@ -656,6 +727,9 @@ void PopupAppletPrivate::hideTimedPopup()
 
 void PopupAppletPrivate::clearPopupLostFocus()
 {
+    if (!icon || !icon->isDown()) {
+        q->hidePopup();
+    }
     popupLostFocus = false;
 }
 
@@ -676,9 +750,7 @@ void PopupAppletPrivate::dialogSizeChanged()
 
         updateDialogPosition();
 
-        // saving the dialog height/width on a popup isn't critical enough to warrant a hit to disk
-        // emit q->configNeedsSaving();
-
+        emit q->configNeedsSaving();
         emit q->appletTransformedByUser();
     }
 }
@@ -688,14 +760,13 @@ void PopupAppletPrivate::dialogStatusChanged(bool status)
     q->popupEvent(status);
 }
 
-void PopupAppletPrivate::updateDialogPosition()
+void PopupAppletPrivate::restoreDialogSize()
 {
     Plasma::Dialog *dialog = dialogPtr.data();
     if (!dialog) {
         return;
     }
 
-
     Corona *corona = qobject_cast<Corona *>(q->scene());
     if (!corona) {
         return;
@@ -726,6 +797,19 @@ void PopupAppletPrivate::updateDialogPosition()
     if (saved.width() != dialog->width() || saved.height() != dialog->height()) {
         dialog->resize(saved);
     }
+}
+
+void PopupAppletPrivate::updateDialogPosition()
+{
+    Plasma::Dialog *dialog = dialogPtr.data();
+    if (!dialog) {
+        return;
+    }
+
+    Corona *corona = qobject_cast<Corona *>(q->scene());
+    if (!corona) {
+        return;
+    }
 
     QGraphicsView *view = q->view();
     if (!view) {
@@ -735,7 +819,11 @@ void PopupAppletPrivate::updateDialogPosition()
     QSize s = dialog->size();
     QPoint pos = view->mapFromScene(q->scenePos());
 
-    pos = corona->popupPosition(q, s);
+    if (!q->containment() || view == q->containment()->view()) {
+        pos = corona->popupPosition(q, s);
+    } else {
+        pos = corona->popupPosition(q->parentItem(), s);
+    }
 
     bool reverse = false;
     if (q->formFactor() == Plasma::Vertical) {
