@@ -48,7 +48,9 @@ public:
           focusedApplet(0),
           wallpaper(0),
           screen(-1), // no screen
+          lastScreen(-1),
           desktop(-1), // all desktops
+          lastDesktop(-1),
           con(0),
           type(Containment::NoContainmentType),
           showDropZoneDelayTimer(0),
@@ -65,11 +67,12 @@ public:
         qDeleteAll(dropMenus);
     }
 
-    AbstractToolBox *createToolBox();
+    void createToolBox();
     void positionToolBox();
     void updateToolBoxVisibility();
     void triggerShowAddWidgets();
     void requestConfiguration();
+    void checkStatus(Plasma::ItemStatus status);
 
     /**
      * Called when constraints have been updated on this containment to provide
@@ -78,15 +81,18 @@ public:
      */
     void containmentConstraintsEvent(Plasma::Constraints constraints);
 
+    void initApplets();
+    void checkContainmentFurniture();
     bool regionIsEmpty(const QRectF &region, Applet *ignoredApplet=0) const;
     void positionPanel(bool force = false);
-    void positionContainments();
+    bool isPanelContainment() const;
+    QPointF preferredPos(Corona *corona) const;
+    QPointF preferredPanelPos(Corona *corona) const;
     void setLockToolText();
     void handleDisappeared(AppletHandle *handle);
-    void appletDestroyed(Plasma::Applet*);
+    void appletDestroyed(Applet*);
     void appletAppearAnimationComplete();
-    void zoomIn();
-    void zoomOut();
+    void appletAppeared(Applet*);
     void clearDataForMimeJob(KIO::Job *job);
     void remoteAppletReady(Plasma::AccessAppletJob *job);
     void mimeTypeRetrieved(KIO::Job *job, const QString &mimetype);
@@ -117,6 +123,11 @@ public:
      */
     Context *context();
 
+    /**
+     * propogates context changes
+     */
+    void onContextChanged(Plasma::Context *con);
+
     /**
      * Handles dropped/pasted mimetype data
      * @param scenePos scene-relative position
@@ -145,7 +156,7 @@ public:
      */
     void showDropZoneDelayed();
 
-    static bool s_positioning;
+    static bool s_positioningPanels;
 
     Containment *q;
     FormFactor formFactor;
@@ -156,7 +167,9 @@ public:
     QMap<Applet*, AppletHandle*> handles;
     QHash<QString, ContainmentActions*> actionPlugins;
     int screen;
+    int lastScreen;
     int desktop;
+    int lastDesktop;
     QWeakPointer<AbstractToolBox> toolBox;
     Context *con;
     Containment::Type type;
