@@ -41,6 +41,7 @@ class DesktopToolBoxPrivate;
 class DesktopToolBox : public InternalToolBox
 {
     Q_OBJECT
+    Q_PROPERTY(qreal highlight READ highlight WRITE setHighlight)
 
 public:
     explicit DesktopToolBox(Containment *parent = 0);
@@ -48,6 +49,8 @@ public:
     QRectF boundingRect() const;
     QPainterPath shape() const;
 
+    void addTool(QAction *action);
+    void removeTool(QAction *action);
     void showToolBox();
     void hideToolBox();
 
@@ -68,7 +71,8 @@ protected:
     void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);
 
 protected Q_SLOTS:
-    void animateHighlight(qreal progress);
+    void setHighlight(qreal progress);
+    qreal highlight();
     void updateTheming();
     void toolTriggered(bool);
     void hideToolBacker();
@@ -77,6 +81,7 @@ protected Q_SLOTS:
      */
     void toggle();
 private:
+    void highlight(bool highlighting);
     void adjustToolBackerGeometry();
     DesktopToolBoxPrivate *d;
 };
