@@ -33,15 +33,18 @@ public:
     PopupAppletPrivate(PopupApplet *applet);
     ~PopupAppletPrivate();
 
+    void iconSizeChanged(int group);
     void internalTogglePopup();
     void hideTimedPopup();
     void clearPopupLostFocus();
     void dialogSizeChanged();
     void dialogStatusChanged(bool status);
+    void restoreDialogSize();
     void updateDialogPosition();
     void popupConstraintsEvent(Plasma::Constraints constraints);
     void checkExtenderAppearance(Plasma::FormFactor f);
     KConfigGroup popupConfigGroup();
+    void appletActivated();
 
     PopupApplet *q;
     Plasma::IconWidget *icon;
