@@ -28,10 +28,12 @@
 #include <QIcon>
 #include <QStyleOption>
 #include <QPainter>
+#include <QParallelAnimationGroup>
 
 #include <kdebug.h>
 
 #include <plasma/animator.h>
+#include <plasma/animations/animation.h>
 #include <plasma/theme.h>
 
 #include "private/nativetabbar_p.h"
@@ -73,8 +75,6 @@ public:
           tabProxy(0),
           currentIndex(0),
           tabWidgetMode(true),
-          oldPage(0),
-          newPage(0),
           oldPageAnimId(-1),
           newPageAnimId(-1),
           customFont(false)
@@ -87,6 +87,8 @@ public:
 
     void updateTabWidgetMode();
     void slidingCompleted(QGraphicsItem *item);
+    void slidingNewPageCompleted();
+    void slidingOldPageCompleted();
     void shapeChanged(const KTabBar::Shape shape);
     void setPalette();
 
@@ -99,10 +101,13 @@ public:
     int currentIndex;
     bool tabWidgetMode;
 
-    QGraphicsWidget *oldPage;
-    QGraphicsWidget *newPage;
+    QWeakPointer<QGraphicsWidget> oldPage;
+    QWeakPointer<QGraphicsWidget> newPage;
     int oldPageAnimId;
     int newPageAnimId;
+    Animation *oldPageAnim;
+    Animation *newPageAnim;
+    QParallelAnimationGroup *animGroup;
     bool customFont;
 };
 
@@ -137,22 +142,36 @@ void TabBarPrivate::updateTabWidgetMode()
     }
 
     tabWidgetMode = tabWidget;
+    if (!tabWidgetMode) {
+        q->setMinimumSize(QSize(0, 0));
+        q->setMaximumSize(QWIDGETSIZE_MAX, QWIDGETSIZE_MAX);
+    } else {
+        tabProxy->native->setMinimumSize(QSize(0,0));
+        tabProxy->setMinimumSize(QSize(0,0));
+    }
 }
 
-void TabBarPrivate::slidingCompleted(QGraphicsItem *item)
+void TabBarPrivate::slidingNewPageCompleted()
 {
-    if (item == oldPage || item == newPage) {
-        if (item == newPage) {
-            tabWidgetLayout->addItem(newPage);
-            newPageAnimId = -1;
-            mainLayout->invalidate();
-            emit q->currentChanged(currentIndex);
-        } else {
-            oldPageAnimId = -1;
-            item->hide();
-        }
-        q->setFlags(0);
+    if (newPage) {
+        tabWidgetLayout->addItem(newPage.data());
     }
+    newPageAnimId = -1;
+    mainLayout->invalidate();
+    emit q->currentChanged(currentIndex);
+
+   q->setFlags(0);
+}
+
+void TabBarPrivate::slidingOldPageCompleted()
+{
+    QGraphicsWidget *item = oldPageAnim->targetWidget();
+
+    oldPageAnimId = -1;
+    if (item) {
+        item->hide();
+    }
+    q->setFlags(0);
 }
 
 void TabBarPrivate::shapeChanged(const QTabBar::Shape shape)
@@ -165,11 +184,12 @@ void TabBarPrivate::shapeChanged(const QTabBar::Shape shape)
 
     case QTabBar::RoundedEast:
     case QTabBar::TriangularEast:
+        q->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Expanding);
         tabBarLayout->setOrientation(Qt::Vertical);
         tabWidgetLayout->setOrientation(Qt::Horizontal);
         tabWidgetLayout->itemAt(0)->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Preferred);
         if (tabWidgetLayout->count() > 1) {
-            tabWidgetLayout->itemAt(1)->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
+            tabWidgetLayout->itemAt(1)->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
         }
         tabProxy->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Preferred);
         break;
@@ -180,11 +200,12 @@ void TabBarPrivate::shapeChanged(const QTabBar::Shape shape)
     case QTabBar::RoundedNorth:
     case QTabBar::TriangularNorth:
     default:
+        q->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
         tabBarLayout->setOrientation(Qt::Horizontal);
         tabWidgetLayout->setOrientation(Qt::Vertical);
-        tabWidgetLayout->itemAt(0)->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Expanding);
+        tabWidgetLayout->itemAt(0)->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
         if (tabWidgetLayout->count() > 1) {
-            tabWidgetLayout->itemAt(1)->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
+            tabWidgetLayout->itemAt(1)->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
         }
         tabProxy->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
     }
@@ -215,6 +236,7 @@ TabBar::TabBar(QGraphicsWidget *parent)
     : QGraphicsWidget(parent),
       d(new TabBarPrivate(this))
 {
+    setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
     setContentsMargins(0,0,0,0);
     d->tabProxy = new TabBarProxy(this);
     d->tabWidgetLayout = new QGraphicsLinearLayout(Qt::Vertical);
@@ -236,12 +258,20 @@ TabBar::TabBar(QGraphicsWidget *parent)
     d->tabBarLayout->setContentsMargins(0,0,0,0);
     //d->tabBarLayout->setStretchFactor(d->tabProxy, 2);
 
+
+    d->newPageAnim = Animator::create(Animator::SlideAnimation);
+    d->oldPageAnim = Animator::create(Animator::SlideAnimation);
+    d->animGroup = new QParallelAnimationGroup(this);
+
+    d->animGroup->addAnimation(d->newPageAnim);
+    d->animGroup->addAnimation(d->oldPageAnim);
+
     connect(d->tabProxy->native, SIGNAL(currentChanged(int)),
             this, SLOT(setCurrentIndex(int)));
     connect(d->tabProxy->native, SIGNAL(shapeChanged(QTabBar::Shape)),
             this, SLOT(shapeChanged(QTabBar::Shape)));
-    connect(Plasma::Animator::self(), SIGNAL(movementFinished(QGraphicsItem*)),
-            this, SLOT(slidingCompleted(QGraphicsItem*)));
+    connect(d->newPageAnim, SIGNAL(finished()), this, SLOT(slidingNewPageCompleted()));
+    connect(d->oldPageAnim, SIGNAL(finished()), this, SLOT(slidingOldPageCompleted()));
     connect(Theme::defaultTheme(), SIGNAL(themeChanged()),
             this, SLOT(setPalette()));
 }
@@ -257,7 +287,7 @@ int TabBar::insertTab(int index, const QIcon &icon, const QString &label,
 {
     QGraphicsWidget *page = new QGraphicsWidget(this);
     page->setContentsMargins(0,0,0,0);
-    page->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
+    page->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
     if (content) {
         if (content->isLayout()) {
             page->setLayout(static_cast<QGraphicsLayout *>(content));
@@ -318,90 +348,90 @@ void TabBar::resizeEvent(QGraphicsSceneResizeEvent * event)
     if (!d->tabWidgetMode) {
         d->tabProxy->setMinimumSize(event->newSize().toSize());
         setMinimumSize(QSize(0, 0));
+        setMinimumHeight(d->tabProxy->widget()->minimumSizeHint().height());
         setMaximumSize(QWIDGETSIZE_MAX, QWIDGETSIZE_MAX);
     } else {
+        setMinimumSize(QSize(-1, -1));
         d->tabProxy->native->setMinimumSize(QSize(0,0));
     }
 }
 
 void TabBar::setCurrentIndex(int index)
 {
-    if (index >= d->tabProxy->native->count() ||
-        d->tabProxy->native->count() <= 1 ||
+    if (index >= d->pages.count() ||
+        d->pages.count() < 2 ||
         d->currentIndex == index) {
         return;
     }
 
-    if (d->currentIndex >= 0) {
-        d->oldPage = d->pages[d->currentIndex];
-    } else {
-        d->oldPage = 0;
+    d->oldPage = d->pages.value(d->currentIndex);
+
+    if (d->oldPage) {
+        d->tabWidgetLayout->removeItem(d->oldPage.data());
     }
 
-    d->tabWidgetLayout->removeItem(d->oldPage);
-
     if (index >= 0) {
-        d->newPage = d->pages[index];
-    } else {
-        d->newPage = 0;
+        d->newPage = d->pages.value(index);
     }
 
     setFlags(QGraphicsItem::ItemClipsChildrenToShape);
 
     //if an animation was in rogress hide everything to avoid an inconsistent state
-    if (d->newPageAnimId != -1 || d->oldPageAnimId != -1) {
+
+    if (d->animGroup->state() != QAbstractAnimation::Stopped) {
         foreach (QGraphicsWidget *page, d->pages) {
             page->hide();
         }
-        if (d->newPageAnimId != -1) {
-            Animator::self()->stopItemMovement(d->newPageAnimId);
-        }
-        if (d->oldPageAnimId != -1) {
-            Animator::self()->stopItemMovement(d->oldPageAnimId);
-        }
+        d->animGroup->stop();
     }
 
     if (d->newPage) {
-        d->newPage->show();
-        d->newPage->setEnabled(true);
+        d->newPage.data()->show();
+        d->newPage.data()->setEnabled(true);
     }
 
     if (d->oldPage) {
-        d->oldPage->show();
-        d->oldPage->setEnabled(false);
+        d->oldPage.data()->show();
+        d->oldPage.data()->setEnabled(false);
     }
 
     if (d->newPage && d->oldPage) {
         //FIXME: it seems necessary to resiz the thing 2 times to have effect
-        d->newPage->resize(1,1);
-        d->newPage->resize(d->oldPage->size());
+        d->newPage.data()->resize(1,1);
+        d->newPage.data()->resize(d->oldPage.data()->size());
 
-        QRect beforeCurrentGeom(d->oldPage->geometry().toRect());
+        QRect beforeCurrentGeom(d->oldPage.data()->geometry().toRect());
         beforeCurrentGeom.moveTopRight(beforeCurrentGeom.topLeft());
 
         if (index > d->currentIndex) {
-            d->newPage->setPos(d->oldPage->geometry().topRight());
-            d->newPageAnimId = Animator::self()->moveItem(
-            d->newPage, Plasma::Animator::SlideOutMovement,
-            d->oldPage->pos().toPoint());
-            d->oldPageAnimId = Animator::self()->moveItem(
-                d->oldPage, Plasma::Animator::SlideOutMovement,
-                beforeCurrentGeom.topLeft());
+            d->newPage.data()->setPos(d->oldPage.data()->geometry().topRight());
+            d->newPageAnim->setProperty("movementDirection", Animation::MoveLeft);
+            d->newPageAnim->setProperty("distancePointF", QPointF(d->oldPage.data()->size().width(), 0));
+            d->newPageAnim->setTargetWidget(d->newPage.data());
+
+            d->oldPageAnim->setProperty("movementDirection", Animation::MoveLeft);
+            d->oldPageAnim->setProperty("distancePointF", QPointF(beforeCurrentGeom.width(), 0));
+            d->oldPageAnim->setTargetWidget(d->oldPage.data());
+
+            d->animGroup->start();
         } else {
-            d->newPage->setPos(beforeCurrentGeom.topLeft());
-            d->newPageAnimId = Animator::self()->moveItem(
-            d->newPage, Plasma::Animator::SlideOutMovement,
-            d->oldPage->pos().toPoint());
-            d->oldPageAnimId = Animator::self()->moveItem(
-                d->oldPage, Plasma::Animator::SlideOutMovement,
-                d->oldPage->geometry().topRight().toPoint());
+            d->newPage.data()->setPos(beforeCurrentGeom.topLeft());
+            d->newPageAnim->setProperty("movementDirection", Animation::MoveRight);
+            d->newPageAnim->setProperty("distancePointF", QPointF(d->oldPage.data()->size().width(), 0));
+            d->newPageAnim->setTargetWidget(d->newPage.data());
+
+            d->oldPageAnim->setProperty("movementDirection", Animation::MoveRight);
+            d->oldPageAnim->setProperty("distancePointF",
+                                        QPointF(d->oldPage.data()->size().width(), 0));
+            d->oldPageAnim->setTargetWidget(d->oldPage.data());
+
+            d->animGroup->start();
         }
-    } else {
-        d->tabWidgetLayout->addItem(d->newPage);
+    } else if (d->newPage) {
+        d->tabWidgetLayout->addItem(d->newPage.data());
     }
 
     d->currentIndex = index;
-
     d->tabProxy->native->setCurrentIndex(index);
 }
 
@@ -412,28 +442,33 @@ int TabBar::count() const
 
 void TabBar::removeTab(int index)
 {
-    if (index >= d->pages.count()) {
+    if (index >= d->pages.count() || index < 0) {
         return;
     }
 
+    d->newPageAnim->stop();
+    d->oldPageAnim->stop();
+
     int oldCurrentIndex = d->tabProxy->native->currentIndex();
     d->tabProxy->native->removeTab(index);
-    QGraphicsWidget *page = d->pages.takeAt(index);
 
+    d->currentIndex = oldCurrentIndex;
     int currentIndex = d->tabProxy->native->currentIndex();
 
     if (oldCurrentIndex == index) {
         d->tabWidgetLayout->removeAt(1);
+        if (d->tabProxy->native->count() > 0) {
+            setCurrentIndex(currentIndex >= oldCurrentIndex ? currentIndex + 1 : currentIndex);
+        }
     }
 
+    QGraphicsWidget *page = d->pages.takeAt(index);
     scene()->removeItem(page);
     page->deleteLater();
 
-    if (oldCurrentIndex != currentIndex) {
-        setCurrentIndex(currentIndex);
+    if (d->pages.count() > 0) {
+        d->updateTabWidgetMode();
     }
-
-    d->updateTabWidgetMode();
     d->tabProxy->setPreferredSize(d->tabProxy->native->sizeHint());
 }
 
@@ -445,14 +480,17 @@ QGraphicsLayoutItem *TabBar::takeTab(int index)
 
     int oldCurrentIndex = d->tabProxy->native->currentIndex();
     d->tabProxy->native->removeTab(index);
-    QGraphicsWidget *page = d->pages.takeAt(index);
 
     int currentIndex = d->tabProxy->native->currentIndex();
 
     if (oldCurrentIndex == index) {
         d->tabWidgetLayout->removeAt(1);
+        if (d->tabProxy->native->count() > 0) {
+            setCurrentIndex(currentIndex >= oldCurrentIndex ? currentIndex + 1 : currentIndex);
+        }
     }
 
+    QGraphicsWidget *page = d->pages.takeAt(index);
     QGraphicsLayoutItem *returnItem = 0;
     QGraphicsLayout *lay = page->layout();
     if (lay && lay->count() == 1) {
@@ -464,6 +502,9 @@ QGraphicsLayoutItem *TabBar::takeTab(int index)
 
     if (returnItem) {
         returnItem->setParentLayoutItem(0);
+        if (QGraphicsItem *item = returnItem->graphicsItem()) {
+            item->setParentItem(0);
+        }
     }
 
     page->setLayout(0);
