@@ -47,6 +47,9 @@ class PLASMA_EXPORT ToolButton : public QGraphicsProxyWidget
     Q_PROPERTY(QString styleSheet READ styleSheet WRITE setStyleSheet)
     Q_PROPERTY(QToolButton *nativeWidget READ nativeWidget)
     Q_PROPERTY(QAction *action READ action WRITE setAction)
+    Q_PROPERTY(bool down READ isDown WRITE setDown)
+
+    Q_PROPERTY(qreal animationUpdate READ animationUpdate WRITE setAnimationUpdate)
 
 public:
     explicit ToolButton(QGraphicsWidget *parent = 0);
@@ -98,6 +101,14 @@ public:
      */
     QString image() const;
 
+    /**
+     * Sets the status of the button to pressed
+     *
+     * @arg down
+     * @since 4.5
+     */
+    void setDown(bool down);
+
     /**
      * @return true if the button is pressed down
      * @since 4.4
@@ -172,11 +183,15 @@ protected:
     void hoverEnterEvent(QGraphicsSceneHoverEvent *event);
     void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);
     void changeEvent(QEvent *event);
+    QVariant itemChange(GraphicsItemChange change, const QVariant &value); 
     QSizeF sizeHint(Qt::SizeHint which, const QSizeF & constraint) const;
 
+private slots:
+    void setAnimationUpdate(qreal progress);
+    qreal animationUpdate() const;
+
 private:
     Q_PRIVATE_SLOT(d, void syncBorders())
-    Q_PRIVATE_SLOT(d, void animationUpdate(qreal progress))
     Q_PRIVATE_SLOT(d, void syncToAction())
     Q_PRIVATE_SLOT(d, void clearAction())
     Q_PRIVATE_SLOT(d, void setPixmap())
